unit T_IniF;

interface
uses
  IniFiles, Classes,
  Elements, Tools,
  E_Type;

type

T_IniFile = class(TTool)
private
   FFileName: string;
   FSection: string;
   FIdent: string;
   FDefaultValue: string;
   FSLE: TStringListElement;
   FIni: TIniFile;
   procedure SetFileName(aValue: string);
   function GetValue: string;
   procedure SetValue(aValue: string);
   procedure SetIni(const Value: TIniFile);
    function GetSLE: TStringListElement;
    function GetSectionElement: TEntity;
    procedure SetSectionElement(const Value: TEntity);
public
   destructor Destroy; override;
   property  Ini: TIniFile read FIni write SetIni;
   property  SLE: TStringListElement read GetSLE;
published
  property FileName: string read FFileName write SetFileName;
  property Section: string read FSection write FSection;
  property Ident: string read FIdent write FIdent;
  property DefaultValue: string read FDefaultValue write FDefaultValue;
  property Value: string read GetValue write SetValue;
  property SectionElement: TEntity read GetSectionElement write SetSectionElement;
end;

T_IniWrite = class(TTool)
  private
    FValue: string;
    FSection: string;
    FIdent: string;
    procedure SetSection(const Value: string);
    procedure SetValue(const Value: string);
    procedure SetIdent(const Value: string);
public
   function NumberToPropertyName(aNumber: longInt): string; override;
   procedure DoIt; override;
published
   property Section: string read FSection write SetSection;
   property Ident: string read FIdent write SetIdent;
   property Value: string read FValue write SetValue;
end;

T_IniRead = class(TResultTool)
  private
    FValue: string;
    FSection: string;
    FIdent: string;
    FDefaultValue: string;
    procedure SetDefaultValue(const Value: string);
    procedure SetIdent(const Value: string);
    procedure SetSection(const Value: string);
    procedure SetValue(const Value: string);
public
   function NumberToPropertyName(aNumber: longInt): string; override;
   procedure DoIt; override;
published
   property Section: string read FSection write SetSection;
   property Ident: string read FIdent write SetIdent;
   property Value: string read FValue write SetValue;
   property DefaultValue: string read FDefaultValue write SetDefaultValue;
end;


implementation

uses
  Dialogs,
  SysUtils,
  E_Class,
  {!F_Start,}
  U_String;



procedure T_IniFile.SetFileName(aValue: string);
begin
  FFileName:= GetAbsolutePath(aValue);
  if Ini <> nil then
  begin
     Ini.Free;
     Ini := nil;
  end;
  Ini := TIniFile.Create(FFileName);
end;



function T_IniFile.GetValue: string;
begin
  result:= DefaultValue;
  if Ini = nil then exit;
  result:= Ini.ReadString(Section, Ident, DefaultValue);
end;

procedure T_IniFile.SetValue(aValue: string);
begin
  if Ini = nil then exit;
  Ini.WriteString(Section, Ident, aValue);
end;





destructor T_IniFile.Destroy;
begin
   if Ini <> nil then Ini.Free;
   Ini := nil;
   inherited;
end;

procedure T_IniFile.SetIni(const Value: TIniFile);
begin
   FIni := Value;
end;


function T_IniFile.GetSLE: TStringListElement;
begin
   if FSLE = nil then
   begin
      FSLE := TStringListElement.Create;
      FSLE.Name := 'SLE';
      AddElement(FSLE);
   end;
   result := FSLE;
end;


function T_IniFile.GetSectionElement: TEntity;
var
   SL: TStringList;
   i: integer;
begin
   SLE.FreeAll;
   result:= SLE;
   if Ini = nil then exit;
   SL:= TStringList.Create;
   Ini.ReadSection(Section, SL);
   if SL.Count > 0 then
   begin
      SLE.AddStringList(SL);
      { naplnn obsahu poloky }
      for I:= 0 to SLE.LastIndex do
      begin
        SLE.Strings[I]:= Ini.ReadString(Section, SLE.Names[I], '');
      end;
   end;
   SL.Free;
end;

procedure T_IniFile.SetSectionElement(const Value: TEntity);
var
   aSLE: TStringListElement;
   SE: TStringElement;
   i: integer;
begin
   SLE.FreeAll;
   try
   Ini.EraseSection(Section);
   except
      ShowMessage('Ini section '+Section+' error!');
   end;
   if Value = nil then exit;
   if Value is TStringListElement then
   begin
      aSLE := TStringListElement(Value);
      for i := 0 to aSLE.LastIndex do
      begin
         SE := aSLE.StringElements[i];
         SLE.AddNameString(SE.Name,SE.Value);
      end;
   end
   else
   begin
      for i := 0 to Value.LastIndex do
      begin
         SLE.NextName := Value.Elements[i].Name;
      end;
   end;
   for i := 0 to SLE.LastIndex do
   begin
      SE := SLE.StringElements[i];
      Ini.WriteString(Section,SE.Name,SE.Value);
   end;
end;

{ T_IniWrite }

procedure T_IniWrite.DoIt;
var
   _ini: T_IniFile;
begin
   if Input = nil then exit;
   if not (Input is T_IniFile) then exit;
   _Ini := T_IniFile(Input);
   _Ini.Section := Section;
   _Ini.Ident := Ident;
   _Ini.Value := Value;
end;

function T_IniWrite.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'Section';
      1: result := 'Ident';
   else
      result := 'Value';
   end;
end;

procedure T_IniWrite.SetIdent(const Value: string);
begin
  FIdent := Value;
end;

procedure T_IniWrite.SetSection(const Value: string);
begin
  FSection := Value;
end;

procedure T_IniWrite.SetValue(const Value: string);
begin
  FValue := Value;
end;

{ T_IniRead }

procedure T_IniRead.DoIt;
var
   _ini: T_IniFile;
begin
   if Input = nil then exit;
   if not (Input is T_IniFile) then exit;
   _Ini := T_IniFile(Input);
   _Ini.Section := Section;
   _Ini.Ident := Ident;
   _ini.DefaultValue := DefaultValue;
   result := _Ini.Value;
end;

function T_IniRead.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'Section';
      1: result := 'Ident';
   else
      result := 'DefaultValue';
   end;
end;

procedure T_IniRead.SetDefaultValue(const Value: string);
begin
  FDefaultValue := Value;
end;

procedure T_IniRead.SetIdent(const Value: string);
begin
  FIdent := Value;
end;

procedure T_IniRead.SetSection(const Value: string);
begin
  FSection := Value;
end;

procedure T_IniRead.SetValue(const Value: string);
begin
  FValue := Value;
end;

INITIALIZATION
   RegisterOrbisClass(T_IniFile);
   RegisterOrbisClass(T_IniWrite);
   RegisterOrbisClass(T_IniRead);
end.
