unit T_Orbis_ExcelWriter;

interface
uses
   Excel97,
   Variants,
   Tools;

type

TOrbis_ExcelWriter = class(TTool)
private
    ExcelApplication: TExcelApplication;
    ExcelWorksheet: TExcelWorksheet;
    ExcelWorkbook: TExcelWorkbook;
    A,B,Format: OleVariant;
    FRowB: integer;
    FRowA: integer;
    FColB: string;
    FColA: string;
    function GetRangeFontBold: boolean;
    function GetRangeHorizontalAlignment: string;
    function GetRangeNumberFormat: string;
    function GetRangeValue: string;
    function GetWorkSheetName: string;
    procedure SetColA(const Value: string);
    procedure SetColB(const Value: string);
    procedure SetRangeFontBold(const Value: boolean);
    procedure SetRangeHorizontalAlignment(const Value: string);
    procedure SetRangeNumberFormat(const Value: string);
    procedure SetRangeValue(const Value: string);
    procedure SetRowA(const Value: integer);
    procedure SetRowB(const Value: integer);
    procedure SetWorkSheetName(const Value: string);
    procedure SetRangeColumnsAutoFit(const Value: boolean);
    function GetRangeFloat: double;
    procedure SetRangeFloat(const Value: double);
    procedure SetNewWorkSheetName(const Value: string);
    function GetRangeFontSize: integer;
    function GetRangeFontUnderLine: boolean;
    procedure SetRangeFontSize(const Value: integer);
    procedure SetRangeFontUnderLine(const Value: boolean);
public
   Constructor Create; override;
   destructor Destroy; override;
   procedure DoIt; override;
   function  ActualRange: Range;
   function IsError: boolean;
published
   property RowA: integer read FRowA write SetRowA;
   property ColA: string read FColA write SetColA;
   property RowB: integer read FRowB write SetRowB;
   property ColB: string read FColB write SetColB;
   property WorkSheetName: string read GetWorkSheetName write SetWorkSheetName;
   property NewWorkSheetName: string write SetNewWorkSheetName;
   property RangeValue: string read GetRangeValue write SetRangeValue;
   property RangeFloat: double read GetRangeFloat write SetRangeFloat;
   property RangeNumberFormat: string read GetRangeNumberFormat write SetRangeNumberFormat;
   property RangeFontBold: boolean read GetRangeFontBold write SetRangeFontBold;
   property RangeFontUnderline: boolean read GetRangeFontUnderLine write SetRangeFontUnderLine;
   property RangeFontSize: integer read GetRangeFontSize write SetRangeFontSize;
   property RangeHorizontalAlignment: string read GetRangeHorizontalAlignment write SetRangeHorizontalAlignment;
   property RangeColumnsAutofit: boolean write SetRangeColumnsAutoFit;
end;

implementation
uses
   Dialogs,
   Forms,
   SysUtils,
   e_Class;

{ TOrbis_ExcelWriter }

function TOrbis_ExcelWriter.ActualRange: Range;
begin
   result := nil;
   if Error > 0 then exit;
   A := ColA + IntToStr(RowA);
   B := ColB + IntToStr(RowB);
   result := ExcelWorkSheet.Range[A,B];
end;

constructor TOrbis_ExcelWriter.Create;
var
   OV: OleVariant;
   EWB: _WorkBook;
begin
  inherited;
  Error := 0;
  ProgressText := 'Inicializace Excelu ...';
  Application.ProcessMessages;
  ExcelApplication := TExcelApplication.Create(Application);
  ExcelWorksheet   := TExcelWorksheet.Create(Application);
  ExcelWorkbook    := TExcelWorkbook.Create(Application);
  try
     ExcelApplication.Connect;
     ExcelApplication.Visible[0] := true;
  except
     ShowMessage('Nepodailo se navzat spojen s EXCELem !');
     Error := 1;
     ExcelWorksheet.Free;
     ExcelWorkbook.Free;
     ExcelApplication.Free;
     exit;
  end;
  try
     OV := True;
     EWB := ExcelApplication.Workbooks.Add(OV,0);
     ExcelWorkBook.ConnectTo(EWB);
     ExcelWorkSheet.ConnectTo(ExcelApplication.ActiveSheet as _WorkSheet);
  except
     ShowMessage('Nepodailo se navzat spojen s EXCELem !(1)');
     Error := 1;
     ExcelWorksheet.Free;
     ExcelWorkbook.Free;
     ExcelApplication.Free;
  end;
end;

destructor TOrbis_ExcelWriter.Destroy;
begin
  if not IsError then
  begin
     ExcelWorksheet.Free;
     ExcelWorkbook.Free;
     ExcelApplication.Free;
  end;
  inherited;
end;

procedure TOrbis_ExcelWriter.DoIt;
begin

end;

function TOrbis_ExcelWriter.GetRangeFloat: double;
begin
   result := 0;
end;

function TOrbis_ExcelWriter.GetRangeFontBold: boolean;
begin
   result := false;
   if IsError then exit;
   result := ActualRange.Font.Bold;
end;

function TOrbis_ExcelWriter.GetRangeFontSize: integer;
begin

end;

function TOrbis_ExcelWriter.GetRangeFontUnderLine: boolean;
begin

end;

function TOrbis_ExcelWriter.GetRangeHorizontalAlignment: string;
begin
   result := '';
end;

function TOrbis_ExcelWriter.GetRangeNumberFormat: string;
begin
   result := '';
end;

function TOrbis_ExcelWriter.GetRangeValue: string;
begin
   result := '';
end;

function TOrbis_ExcelWriter.GetWorkSheetName: string;
begin
   result := ''
end;

function TOrbis_ExcelWriter.IsError: boolean;
begin
   result := Error > 0;
end;

procedure TOrbis_ExcelWriter.SetColA(const Value: string);
begin
  FColA := Value;
  ColB := Value;
end;

procedure TOrbis_ExcelWriter.SetColB(const Value: string);
begin
  FColB := Value;
end;

procedure TOrbis_ExcelWriter.SetNewWorkSheetName(const Value: string);
var
   EWS: _WorkSheet;
begin
   EWS := ExcelWorkBook.Sheets.Add(EmptyParam,EmptyParam,1,xlWorksheet,0) as _WorkSheet;
   ExcelWorkSheet.ConnectTo(EWS);
   ExcelWorkSheet.Name := value;
end;

procedure TOrbis_ExcelWriter.SetRangeColumnsAutoFit(const Value: boolean);
begin
   if IsError then exit;
   if not Value then exit;
   ActualRange.Columns.AutoFit;
end;

procedure TOrbis_ExcelWriter.SetRangeFloat(const Value: double);
begin
   if IsError then exit;
   ActualRange.Value := Value;
end;

procedure TOrbis_ExcelWriter.SetRangeFontBold(const Value: boolean);
begin
   if IsError then exit;
   ActualRange.Font.Bold := Value;
end;

procedure TOrbis_ExcelWriter.SetRangeFontSize(const Value: integer);
begin
   if IsError then exit;
   ActualRange.Font.Size := Value;
end;

procedure TOrbis_ExcelWriter.SetRangeFontUnderLine(const Value: boolean);
begin
   if IsError then exit;
   ActualRange.Font.Underline := Value;
end;

procedure TOrbis_ExcelWriter.SetRangeHorizontalAlignment(const Value: string);
var
   S: string;
begin
   if IsError then exit;
   S := UpperCase(Value);
   if S = 'CENTER' then
      ActualRange.HorizontalAlignment := xlHAlignCenter;
end;

procedure TOrbis_ExcelWriter.SetRangeNumberFormat(const Value: string);
begin
   if IsError then exit;
   Format := FormatFloat(Value,0);
   ActualRange.NumberFormat := Format;
end;

procedure TOrbis_ExcelWriter.SetRangeValue(const Value: string);
begin
   if IsError then exit;
   ActualRange.Value := Value;
end;

procedure TOrbis_ExcelWriter.SetRowA(const Value: integer);
begin
  FRowA := Value;
  RowB := Value;
end;

procedure TOrbis_ExcelWriter.SetRowB(const Value: integer);
begin
  FRowB := Value;
end;

procedure TOrbis_ExcelWriter.SetWorkSheetName(const Value: string);
begin
   if IsError then exit;
   ExcelWorkSheet.Name := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TOrbis_ExcelWriter);
END.
