unit d_Geometry;

interface
uses
   e_Device,
   e_Geometry,
   Elements;

type

TPolyLine_Device = class(TDevice_Device)
  private
    FLines: TEntity;
    FActiveLine: TPolyLineElement;
    procedure SetLines(const Value: TEntity);
    procedure SetActiveLine(const Value: TPolyLineElement);
public
    procedure LineTo(X, Y: double); override;
    procedure MoveTo(X, Y: double); override;
    procedure Reset; override;
    procedure Close; override;
    procedure FinishLine;
    property  ActiveLine: TPolyLineElement read FActiveLine write SetActiveLine;
published
   property Lines: TEntity read FLines write SetLines;
end;

implementation
uses
   G_Elems;

{ TPolyLine_Device }

procedure TPolyLine_Device.Close;
begin
   //inherited;
   if Lines = nil then exit;
   FinishLine;
   if Lines is TRectEntity then
      TRectEntity(Lines).Update;
end;

procedure TPolyLine_Device.FinishLine;
begin
   if ActiveLine = nil then exit;
   if ActiveLine.Count < 2 then
   begin
      ActiveLine.Free;
      ActiveLine := nil;
      exit;
   end;
   ActiveLine.RecalculateRect;
   ActiveLine.CalculateLabelProps;
end;

procedure TPolyLine_Device.LineTo(X, Y: double);
begin
   //inherited;
   if ActiveLine = nil then exit;
   ActiveLine.AddXY(X, Y);
end;

procedure TPolyLine_Device.MoveTo(X, Y: double);
begin
   //inherited;
   if Lines = nil then exit;
   FinishLine;
   ActiveLine := TPolyLineElement.Create;
   Lines.AddElement(ActiveLine);
   ActiveLine.AddXY(X, Y);
end;

procedure TPolyLine_Device.Reset;
begin
  //inherited;
  ActiveLine := nil;
end;

procedure TPolyLine_Device.SetActiveLine(const Value: TPolyLineElement);
begin
  FActiveLine := Value;
end;

procedure TPolyLine_Device.SetLines(const Value: TEntity);
begin
  ExchangeFieldElement(TElement(FLines), Value);
end;

INITIALIZATION
END.

