unit t_LoadRasters;

interface
uses
   Tools;

type

TLoadRasters = class(TTool)
  private
    FColor: integer;
    FDirectory: string;
    FFilter: string;
    FCollectionName: string;
    procedure SetCollectionName(const Value: string);
    procedure SetColor(const Value: integer);
    procedure SetDirectory(const Value: string);
    procedure SetFilter(const Value: string);
public
   procedure  DoIt; override;
   function   NumberToPropertyName(aNumber: longInt): string; override;
   procedure  ExtractFilterFromDirectory;
published
   property   CollectionName: string read FCollectionName write SetCollectionName;
   property   Directory: string read FDirectory write SetDirectory;
   property   Filter: string read FFilter write SetFilter;
   property   Color: integer read FColor write SetColor;
end;

implementation
uses
   Forms,
   SysUtils,
   e_Files,
   e_Raster,
   Elements,
   e_Class,
   u_String;

{ TLoadRasters }

procedure TLoadRasters.DoIt;
var
   RC: TRasterCollection;
   RR: TRasterRepository;
   R: e_Raster.TRaster;
   e: TElement;
   D: TDirectoryElement;
   F: TFileElement;
   i: integer;
   S: string;
begin
   RC := nil;
   RR := nil;
   e := input;
   output := nil;
   Error := 0;
   if e is TRasterRepository then
      RR := TRasterRepository(e)
   else
   if e is TRasterCollection then
      RC := TRasterCollection(e);
   if RC <> nil then
   begin
      CollectionName := RC.Name;
      Directory := RC.Directory;
      Filter := RC.Filter;
      Color := RC.Color;
   end;
   if (Filter = '') or (Filter = '-') then
      ExtractFilterFromDirectory;
   if not DirectoryExists(Directory) then
   begin
      Error := 1;
      exit;
   end;
   if RC = nil then
   begin
      RC := TRasterCollection.Create;
      RC.Name := CollectionName;
      RC.Directory := Directory;
      RC.Filter := Filter;
      RC.Color := Color;
      if RR <> nil then
         RR.NextElement := RC;
   end;
   Output := RC;
   StartProgress('Rastry '+Filter+' z '+Directory,0,0);
   ProgressText := 'ten adrese ...';
   Application.ProcessMessages;
   D := TDirectoryElement.Create;
   AddElement(D);
   D.FileName := Directory;
   MaxProgress := D.LastIndex;
   for i := 0 to D.LastIndex do
   begin
      e := D.Elements[i];
      if e is TDirectoryElement then continue;
      if not (e is TFileElement) then continue;
      F := TFileElement(e);
      Progress := i;
      ProgressText := F.Name;
      Application.ProcessMessages;
      S := F.UpperExt;
      if (Filter = '*') then
      begin
         if not ((S = 'RAS') or (S = 'RAK') or (S = 'CIT') or (S = 'TIF') or (S = 'BMP') or (S = 'JPG')) then continue;
      end
      else
         if Filter <> S then continue;
      try
         R := e_Raster.TRaster.Create;
         R.Color := Color;
         R.FileName := F.FileName;
         R.Name := F.FileName;
         R.Active := true;
      except
         R.Free;
         Continue;
      end;
      if R.Error > 0 then
      begin
         R.Free;
         Continue;
      end;
      if not R.GeoRaster then
      begin
         R.Free;
         continue;
      end;
      R.Active := false;
      RC.AddElement(R);
   end;
   RC.Update;
end;

procedure TLoadRasters.ExtractFilterFromDirectory;
var
   S, S1: string;
begin
   S := Directory;
   S1 := ExtractFileName(S);
   S := ExtractFilePath(S);
   CheckBackSlash(S);
   Directory := S;
   if StringAsTwoParams(S1,'.',S,S1) then
      Filter := UpperCase(S1)
   else
      Filter := '*';
end;

function TLoadRasters.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'CollectionName';
      1: result := 'Directory';
      2: result := 'Filter';
      3: result := 'Color';
   end;
end;

procedure TLoadRasters.SetCollectionName(const Value: string);
begin
  FCollectionName := Value;
end;

procedure TLoadRasters.SetColor(const Value: integer);
begin
  FColor := Value;
end;

procedure TLoadRasters.SetDirectory(const Value: string);
begin
  FDirectory := Value;
end;

procedure TLoadRasters.SetFilter(const Value: string);
begin
  FFilter := UpperCase(Value);
end;

INITIALIZATION
   RegisterOrbisClass(TLoadRasters);
END.
