unit t_ThematicMapsEditor;

interface
uses
   f_ThematicMapsEditor,
   f_TM_Editor,
   t_FormTool,
   t_OrbisDialog,
   Elements,
   Tools;
type

TThematicMapsEditor = class(TOrbisDialog)
private
    FPostTool: TTool;
    FDirectPost: boolean;
    FAddCaption: string;
    function GetForm: TThematicMapsEditor_Form;
    procedure SetPostTool(const Value: TTool);
    procedure SetReView(const Value: boolean);
    procedure SetDirectPost(const Value: boolean);
    procedure SetAddCaption(const Value: string);
public
   procedure DoIt; override;
   procedure CreateForm; override;
   property  Form: TThematicMapsEditor_Form read GetForm;
published
   property  PostTool: TTool read FPostTool write SetPostTool stored false;
   property  ReView: boolean write SetReView;
   property  DirectPost: boolean read FDirectPost write SetDirectPost;
   property  AddCaption: string read FAddCaption write SetAddCaption;
end;


TThematicMaps_Editor = class(TFormTool)
  private
    FOnReviewScript: string;
    function GetForm: TTematicMap_Editor_Form;
    procedure SetOnReviewScript(const Value: string);
    procedure SetReview(const Value: boolean);
public
   procedure CreateForm; override;
   procedure SetInput(aInput: TElement); override;
   procedure SetVisible(const Value: boolean); override;
   property  Form: TTematicMap_Editor_Form read GetForm;
published
   property OnReviewScript: string read FOnReviewScript write SetOnReviewScript;
   property Review: boolean write SetReview;
end;


implementation
uses
   ComCtrls,
   Forms,
   SysUtils,
   m_Orbis,
   m_Proj,
   e_Class,
   e_ThematicMap;

{ TThematicMapsEditor }

procedure TThematicMapsEditor.CreateForm;
begin
   aForm := TThematicMapsEditor_Form.Create(Application);
   aForm.Tool := Self;
   aForm.IsPossibleToClose := false;
end;

procedure TThematicMapsEditor.DoIt;
var
   Maps: TThematicMaps;
   i: integer;
   L,SL: TListItem;
   E: TElement;
begin
   if not (Input is TThematicMaps) then exit;
   Maps := TThematicMaps(input);
   Form.MapList.Items.BeginUpdate;
   Form.MapList.Items.Clear;
   SL := nil;
   for i := 0 to Maps.LastIndex do
   begin
      L := Form.MapList.Items.Add;
      E := Maps.Elements[i];
      L.Data := E;
      L.Caption := E.Name;
      L.ImageIndex := 0;
      L.StateIndex := 0;
      if AnsiCompareText(Maps.MapName,E.Name) = 0 then
      begin
         Caption := E.Name;
         L.Selected := true;
         SL := L;
      end;
   end;
   Form.MapList.Items.EndUpdate;
   //Form.AddCaption := AddCaption;
   if SL <> nil then
      Form.MapListSelectItem(nil,SL,true);

   inherited;
end;

function TThematicMapsEditor.GetForm: TThematicMapsEditor_Form;
begin
   result := TThematicMapsEditor_Form(aForm);
end;


procedure TThematicMapsEditor.SetAddCaption(const Value: string);
begin
  FAddCaption := Value;
end;

procedure TThematicMapsEditor.SetDirectPost(const Value: boolean);
begin
  FDirectPost := Value;
end;

procedure TThematicMapsEditor.SetPostTool(const Value: TTool);
begin
   FPostTool := Value;
end;

procedure TThematicMapsEditor.SetReView(const Value: boolean);
begin
   if PostTool = nil then exit;
   if not DirectPost then
      PostTool.PostMsgToOwners(tm_Review,nil,false)
   else
      PostTool.PostMsg(tm_Review,PostTool,nil,false);
end;

{ TThematicMaps_Editor }

procedure TThematicMaps_Editor.CreateForm;
begin
   aForm := TTematicMap_Editor_Form.Create(Application);
   aForm.KeyPreview := true;
   aForm.OnCloseQuery := OnFormCloseQuery;
   aForm.OnCanResize := OnFormCanResize;
   aForm.OnConstrainedResize := OnFormConstrainedResize;
   aForm.OnResize := OnFormResize;
   aForm.OnShow := OnFormShow;
   aForm.Tool := Self;
   Form.IgnoreChanges := true;
end;

function TThematicMaps_Editor.GetForm: TTematicMap_Editor_Form;
begin
   result := TTematicMap_Editor_Form(aForm);
end;

procedure TThematicMaps_Editor.SetInput(aInput: TElement);
var
   Maps: TThematicMaps;
   i: integer;
   L,SL: TListItem;
   E: TElement;
begin
   inherited;
   if not (Input is TThematicMaps) then exit;
   Maps := TThematicMaps(input);
   Form.MapList.Items.BeginUpdate;
   Form.MapList.Items.Clear;
   SL := nil;
   for i := 0 to Maps.LastIndex do
   begin
      L := Form.MapList.Items.Add;
      E := Maps.Elements[i];
      L.Data := E;
      L.Caption := E.Name;
      L.ImageIndex := 0;
      L.StateIndex := 0;
      if AnsiCompareText(Maps.MapName,E.Name) = 0 then
      begin
         Caption := E.Name;
         L.Selected := true;
         SL := L;
      end;
   end;
   Form.MapList.Items.EndUpdate;
   //Form.AddCaption := AddCaption;
   if SL <> nil then
      Form.MapListSelectItem(nil,SL,true);
end;

procedure TThematicMaps_Editor.SetOnReviewScript(const Value: string);
begin
  FOnReviewScript := Value;
end;

procedure TThematicMaps_Editor.SetReview(const Value: boolean);
begin
   if OnReviewScript <> '' then
   begin
      PostTextMsg(tm_Run,OnReviewScript,nil);
      Application.ProcessMessages;
   end;
end;

procedure TThematicMaps_Editor.SetVisible(const Value: boolean);
begin
   inherited;
   Form.IgnoreChanges := false;
end;

INITIALIZATION
   RegisterOrbisClass(TThematicMapsEditor);
   RegisterOrbisClass(TThematicMaps_Editor);
END.

