unit t_TransferLineAttributes;

interface
uses
   Elements,
   e_QTree,
   e_Type,
   Tools,
   t_QTreeTools;

type

TTransferLineAttributes = class(TTool)
private
   FQTFinder: TFindRectEntityInQTreeByXY;
   FTargetLevel: TEntity;
   FSourceLevel: TEntity;
    FSourceQT: TRectEntityQTree;
    FTolerance: double;
    FTransferModel: TStringListElement;
    FLCount: integer;
    FSilentMode: boolean;
   function GetQTFinder: TFindRectEntityInQTreeByXY;
   procedure SetSourceLevel(const Value: TEntity);
   procedure SetTargetLevel(const Value: TEntity);
    procedure SetSourceQT(const Value: TRectEntityQTree);
    procedure SetTolerance(const Value: double);
    function GetTransferModel: TStringListElement;
    function GetNoTransferModel: boolean;
    procedure SetLCount(const Value: integer);
    procedure SetSilentMode(const Value: boolean);
public
   procedure DoIt; override;
   procedure PrepareQTree;
   property  SourceQT: TRectEntityQTree read FSourceQT write SetSourceQT;
   property  QTFinder: TFindRectEntityInQTreeByXY read GetQTFinder;
   property  NoTransferModel: boolean read GetNoTransferModel;
   property  LCount: integer read FLCount write SetLCount;
published
   property  SourceLevel: TEntity read FSourceLevel write SetSourceLevel;
   property  TargetLevel: TEntity read FTargetLevel write SetTargetLevel;
   property  Tolerance: double read FTolerance write SetTolerance;
   property  TransferModel: TStringListElement read GetTransferModel;
   property  SilentMode: boolean read FSilentMode write SetSilentMode;
end;

implementation

uses
   Forms,
   Dialogs,
   SysUtils,
   e_Class,
   e_GeometryElement,
   m_Orbis;

{ TTransferLineAttributes }

procedure TTransferLineAttributes.DoIt;
var
   i,j: integer;
   LI: integer;
   SE,TE: TElement;
   SG,TG: TGeometryElement;
   S: TStringElement;
begin
   Error := 0;
   if (SourceLevel = nil) or (TargetLevel = nil) then
   begin
      Error := 1;
      ShowMessage('Nebyla nastavena zdrojov nebo clov vrstva - The end');
      exit;
   end;
   if SourceLevel.NoElements or TargetLevel.NoElements then
   begin
      Error := 1;
      ShowMessage('Zdrojov nebo clov vrstva neobsahuje dn linie - The end');
      exit;
   end;
   if NoTransferModel then
   begin
      Error := 1;
      ShowMessage('Nejsou definovny atributy pro penos - The end');
      exit;
   end;
   PrepareQTree;
   StartProgress('Penos atribut z vrstvy '+SourceLevel.Name+' do '+TargetLevel.Name,0,TargetLevel.LastIndex);
   QTFinder.Input := SourceQT;
   QTFinder.Tolerance := Tolerance;
   LCount := 0;
   LI := TargetLevel.LastIndex;
   for i := 0 to TargetLevel.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      TE := TargetLevel.Elements[i];
      if not (TE is TGeometryElement) then continue;
      TG := TGeometryElement(TE);
      QTFinder.X := TG.X;
      QTFinder.Y := TG.Y;
      QTFinder.DoIt;
      SE := TGeometryElement(QTFinder.Output);
      if SE = nil then continue;
      for j := 0 to TransferModel.LastIndex do
      begin
         S := TransferModel.StringElements[j];
         TE.SetPropertyStringByName(S.Value,SE.GetPropertyStringByName(S.Name));
      end;
      LCount := LCount + 1;
      ProgressText := IntToStr(i)+' zpracovno, '+IntToStr(LCount)+' penos';
      Application.ProcessMessages;
   end;
   if not SilentMode then
      ShowMessage('Asi hotovo (teda jako penos Atribut)');
   PostMsg(tm_ModelChanged,nil,nil,false);
end;

function TTransferLineAttributes.GetNoTransferModel: boolean;
begin
   result := (FTransferModel = nil) or (FTransferModel.NoElements);
end;

function TTransferLineAttributes.GetQTFinder: TFindRectEntityInQTreeByXY;
begin
   if FQTFinder = nil then
   begin
      FQTFinder := TFindRectEntityInQTreeByXY.Create;
      AddElement(FQTFinder);
   end;
   result := FQTFinder;
end;

function TTransferLineAttributes.GetTransferModel: TStringListElement;
begin
   if FTransferModel = nil then
   begin
      FTransferModel := TStringListElement.Create;
      AddElement(FTransferModel);
   end;
   result := FTransferModel;
end;

procedure TTransferLineAttributes.PrepareQTree;
var
   TQ: TAddEntityToQTree;
begin
   if FSourceLevel = nil then exit;
   StartProgress('Pprava zdrojov vrstvy...',0,0);
   TQ := TAddEntityToQTree.Create;
   TQ.Input := SourceLevel;
   TQ.DoIt;
   SourceQT :=TRectEntityQTree(TQ.Output);
   SourceQT.Name := 'Source QT';
   TQ.Free;
   EndProgress;
end;

procedure TTransferLineAttributes.SetLCount(const Value: integer);
begin
  FLCount := Value;
end;

procedure TTransferLineAttributes.SetSilentMode(const Value: boolean);
begin
  FSilentMode := Value;
end;

procedure TTransferLineAttributes.SetSourceLevel(const Value: TEntity);
begin
  FSourceLevel := Value;
end;


procedure TTransferLineAttributes.SetSourceQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FSourceQT), Value);
end;

procedure TTransferLineAttributes.SetTargetLevel(const Value: TEntity);
begin
  FTargetLevel := Value;
end;


procedure TTransferLineAttributes.SetTolerance(const Value: double);
begin
  FTolerance := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TTransferLineAttributes);
END.
