unit t_TranslateFile;

interface
uses
   e_Type,
   Tools;

type
TTranslateFile = class(TTool)
private
   FTranslateTable: TStringListELement;
    FResultFileExt: string;
    FResultFileName: string;
    FFileName: string;
    FNoTranslateText: string;
    function GetTtranslateTable: TStringListElement;
    procedure SetFileName(const Value: string);
    procedure SetResultFileExt(const Value: string);
    procedure SetResultFileName(const Value: string);
    procedure SetTranslateTable(const Value: TStringListElement);
    procedure SetNoTranslateText(const Value: string);
public
   procedure DoIt; override;
   function NumberToPropertyName(aNumber: longInt): string; override;
published
   property FileName: string read FFileName write SetFileName;
   property ResultFileName: string read FResultFileName write SetResultFileName;
   property ResultFileExt: string read FResultFileExt write SetResultFileExt;
   property TranslateTable: TStringListElement read GetTtranslateTable write SetTranslateTable;
   property NoTranslateText: string read FNoTranslateText write SetNoTranslateText;
end;

implementation
uses
   Classes,
   SysUtils,
   Elements,
   e_Class;

{ TTranslateFile }

procedure TTranslateFile.DoIt;
var
   SL: TStringList;
   SE: TStringElement;
   S: string;
   NoTranslate: boolean;
   ChkTranslate: boolean;
   TT: TStringListElement;
   i: integer;
   ii: integer;
begin
   Error := 1;
   ChkTranslate := NoTranslateText <> '';
   if not FileExists(FileName) then exit;
   SL := TStringList.Create;
   SL.LoadFromFile(FileName);
   TT := TranslateTable;
   NoTranslate := (TT = nil) or TT.NoElements;
   for i := 0 to SL.Count - 1 do
   begin
      if not NoTranslate then
      begin
         S := SL.Strings[i];
         if CHkTranslate then
            if pos(NoTranslateText,S) > 0 then
            begin
               NoTranslate := true;
               Continue;
            end;
         for ii := 0 to TT.LastIndex do
         begin
            SE := TT.StringElements[ii];
            S := StringReplace(S,SE.Name,SE.Value,[rfReplaceAll, rfIgnoreCase]);
         end;
         SL.Strings[i] := S;
      end;
   end;
   if ResultFileName='' then
      ResultFileName := CHangeFileExt(FileName,'.trn');
   SL.SaveToFile(ResultFileName);
   SL.Free;
end;

function TTranslateFile.GetTtranslateTable: TStringListElement;
begin
   result := nil;
   if FTranslateTable = nil then
   begin
      FTranslateTable := TStringListElement(FindElementClass(TStringListElement));
      if FTranslateTable = nil then
      begin
         ExChangeFieldElement(TElement(FTranslateTable), TStringListElement.Create);
      end;
   end;
   result := FTranslateTable;
end;

function TTranslateFile.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: Result := 'FileName';
      1: Result := 'ResultFileName';
      2: Result := 'TranslateTable';
      3: Result := 'NoTranslateText';
   end;
end;

procedure TTranslateFile.SetFileName(const Value: string);
begin
  FFileName := Value;
end;

procedure TTranslateFile.SetNoTranslateText(const Value: string);
begin
  FNoTranslateText := Value;
end;

procedure TTranslateFile.SetResultFileExt(const Value: string);
begin
   FResultFileExt := Value;
   FResultFileName := ChangeFileExt(FileName, Value);
end;

procedure TTranslateFile.SetResultFileName(const Value: string);
begin
   FResultFileName := Value;
   if Value <> '' then
      if Value[1] = '.' then
         ResultFileExt := Value;
end;

procedure TTranslateFile.SetTranslateTable(const Value: TStringListElement);
begin
   ExchangeFieldElement(TElement(FTranslateTable),Value);
end;

INITIALIZATION
   RegisterOrbisClass(TTranslateFile);
END.
