unit t_GraphicForm;

interface
uses
   Controls,
   Graphics,
   // KERNEL
   Elements,
   Tools,
   // DEVICE
   e_Proje,
   // INTERPRET
   e_IScrpt,
   // GUI
   m_Intera,
   // APP
   f_GraphicForm,
   t_ExtendedFormTool;

type

{ TGraphicForm }

TGraphicForm = class(TExtendedFormTool)
 private
   FImageW: integer;
   FImageH: integer;
    FMinScale: double;
   function    GetActiveToolName: string;
   function    GetActiveToolClassName: string;
   procedure   SetActiveToolName(aValue: string);
   procedure   SetActiveToolClassName(aValue: string);
   function    GetPageColor: TColor;
   procedure   SetPageColor(aValue: TColor);
   function    GetProjection: TProjection;
   function    GetInterpretScript: TInterpretScript;
   procedure   SetInterpretScript(aValue: TInterpretScript);
    function GetForm: TGraphic_Form;
    procedure SetViewAll(const Value: boolean);
    procedure SetViewByElement(const Value: TElement);
    procedure SetShiftView(const Value: string);
    procedure SetZoom(const Value: string);
    procedure SetElementForProjection(const Value: TElement);
    procedure SetElementForPattern(const Value: TElement);
    procedure SetMinScale(const Value: double);
    function GetText2: string;
    procedure SetText2(const Value: string);
 protected
   procedure   ShowScale(aVal: string); virtual;

   procedure   SetInput(aInput: TElement); override;
   procedure   CreateDevice; virtual;
   procedure   LinkDeviceToForm; virtual;
   procedure   CreateInterpret; virtual;
   procedure   LinkInterpretToDevice; virtual;
   procedure   CreateTools; virtual;
   procedure   SetActiveToolIndex(aIndex: longInt); override;
   procedure   SetNextTool(aTool: TTool); virtual;
   function    GetCaption: string; virtual;
   procedure   SetCaption(const aValue: string); virtual;
   procedure   SetBreakReView(aValue: boolean);
   procedure   SetElementForReView(aValue: TElement);
   procedure   SetElementForKillReView(aValue: TElement);
   procedure   SetFreeForm(aValue: boolean); virtual;
 public
   constructor Create; override;
   procedure   DoIt; override;
   procedure   CreateForm; override;

   procedure   SetVisible(const Value: boolean); override;
   function    GetText: string; override;
   procedure   SetText(const Value: string); override;
   function    GetText1: string; override;
   procedure   SetText1(const Value: string); override;
   function    GetCursor: TCursor; override;
   procedure   SetCursor(const Value: TCursor); override;
   function    AddTool(aTool: TTool): TTool; override;
   function    InsertTool(aIndex: longInt; aTool: TTool): TTool; override;
   procedure   RemoveTool(aTool: TTool); override;
   procedure   FreeTool(aTool: TTool); override;
   procedure   FreeTools;

   procedure   Show;
   procedure   ShowAll;
   procedure   on_tm_Open(var Handled: boolean); virtual;
   procedure   on_tm_SaveAs(var Handled: boolean); virtual;
   procedure   on_tm_QuitTask; override;
   procedure   on_tm_Done(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   on_tm_ToolSelected(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   on_tm_Print(var Handled: boolean); virtual;
   procedure   On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   On_tm_ReviewByElement(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   On_tm_Resize(var Handled: boolean); override;
   procedure   On_tm_RePaint(var Handled: boolean); virtual;
   procedure   On_tm_NewScale(var Handled: boolean); virtual;
   procedure   on_tm_ShowAll(var Handled: boolean); virtual;
   procedure   On_tm_EndOfPrint(var Handled: boolean); virtual;
   procedure   On_tm_Cursor(var aMessage: TOrbisCursorEvent; var Handled: boolean); override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseWheelUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseWheelDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   procedure   On_tm_ChangeThematic(var aMessage: TOrbisMessage; var Handled: boolean);
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   BeginUpdate; override;
   procedure   EndUpdate; override;

   property    Form: TGraphic_Form read GetForm;
published
   property    NextTool: TTool write SetNextTool;
   property    ActiveToolName: string read GetActiveToolName write SetActiveToolName;
   property    ActiveToolClassName: string read GetActiveToolClassName write SetActiveToolClassName;
   property    PageColor: TColor read GetPageColor write SetPageColor;
   property    Projection: TProjection read GetProjection;
   property    InterpretScript: TInterpretScript read GetInterpretScript write SetInterpretScript;
   property    BreakReView: boolean write SetBreakReView stored false;
   property    ElementForReView: TElement write SetElementForReView;
   property    ElementForKillReView: TElement write SetElementForKillReView;
   property    ElementForProjection: TElement write SetElementForProjection;
   property    ElementForPattern: TElement write SetElementForPattern;
   property    ReViewByElement: TElement write SetViewByElement;
   property    ShiftView: string write SetShiftView;
   property    Zoom: string write SetZoom;
   property    ViewAll: boolean write SetViewAll;
   property    FreeForm: boolean write SetFreeForm;
   property    MinScale: double read FMinScale write SetMinScale;
   property    Text2: string Read GetText2 Write SetText2;
end;


implementation
uses
   Classes,
   Dialogs,
   Forms,
   SysUtils,
   Windows,
   // KERNEL
   e_Class,
   e_Type,
   m_ORBIS,
   Worlds,
   // GLIB
   G_Elems,
   // DEVICE
   d_StdImg,
   // INTERPRET
   i_SelectedGeometry,
   i_Uni;

{ TGraphicForm }

constructor TGraphicForm.Create;
begin
   inherited Create;
   CreateForm;
   CreateDevice;
   LinkDeviceToForm;
   CreateInterpret;
   LinkInterpretToDevice;
   CreateTools;
end;


procedure   TGraphicForm.SetBreakReView(aValue: boolean);
begin
   if Interpret.Busy then Interpret.BreakWork := true;
end;


procedure   TGraphicForm.SetElementForReView(aValue: TElement);
begin
   if Idle then exit;
   PostMsg(tm_ReView, Self, aValue, DontKill);
end;


procedure   TGraphicForm.SetElementForKillReView(aValue: TElement);
begin
   if Idle then exit;
   PostMsg(tm_ReView, Self, aValue, KillIt);
end;


procedure   TGraphicForm.SetFreeForm(aValue: boolean);
begin
  if Form = nil then exit;
   if aValue then Form.Free;
end;






function    TGraphicForm.GetInterpretScript: TInterpretScript;
begin
   result := nil;
   if Interpret is TUniversalInterpret then result := TUniversalInterpret(Interpret).Script;
end;


procedure   TGraphicForm.SetInterpretScript(aValue: TInterpretScript);
//var
   //aTool: TUniversalTopolScriptCreator;
   //aScript: TInterpretScript;
begin
   (*
   if aValue = nil then
   begin
   {$IFNDEF NOTIS}
      aTool := TUniversalTopolScriptCreator.Create;
      aTool.DoIt;
      aScript:= TInterpretScript(aTool.Output);
      TUniversalInterpret(FInterpret).Script := aScript;
      aTool.Free;
   {$ENDIF}
   end
   else
   *)
   begin
     if ((TElement(aValue) is TInterpretScript) and (Interpret is TUniversalInterpret))
        then TUniversalInterpret(Interpret).Script := aValue;
   end;
end;


function    TGraphicForm.GetPageColor: TColor;
begin
   try
      result := Device.Page.Color;
   except
      result := clBlack;
   end;
end;

procedure   TGraphicForm.SetPageColor(aValue: TColor);
begin
   try
      Device.Page.Color := aValue;
   except
   end;
end;

function    TGraphicForm.GetProjection: TProjection;
begin
   if Device <> nil then result := Device.Projection
                    else result := nil;
end;


function    TGraphicForm.GetActiveToolName: string;
var
   T: TTool;
begin
   result := '';
   T := ActiveTool;
   if T = nil then exit;
   result := T.Name;
end;

function    TGraphicForm.GetActiveToolClassName: string;
var
   T: TTool;
begin
   result := '';
   T := ActiveTool;
   if T = nil then exit;
   result := T.ClassName;
end;

procedure   TGraphicForm.SetActiveToolName(aValue: string);
var
   T: TTool;
begin
   T := TTool(Tools.FindElementName(aValue));
   if T = nil then exit;
   T.PostMsgToOwners(tm_ToolSelected, T, DontKill);
end;

procedure   TGraphicForm.SetActiveToolClassName(aValue: string);
var
   T: TTool;
begin
   T := nil;
   try
      T := TTool(Tools.FindElementClass(FindOrbisClass(aValue)));
   except
   end;
   if T = nil then exit;
   T.PostMsgToOwners(tm_ToolSelected, T, DontKill);
end;


procedure   TGraphicForm.DoIt;
begin
   Show;
   if Idle then exit;
   PostMsg(tm_ReView, Self, nil, DontKill);
end;

procedure   TGraphicForm.SetInput(aInput: TElement);
var
   Envelope: TRectEntity;
begin
   if FInput <> nil then
   begin
      FreeElement(FInput);
      FInput := nil;
   end;
   if aInput = nil then exit;
   Envelope := TRectEntity.Create;
   Envelope.Name := 'Oblka';      // p.k.17.1.99
   Envelope.AddAndUpdateRect(aInput{.CreateEnvelope});
   AddElement(Envelope);
   try
      if aInput is TRectEntity then
      begin
         Device.Projection.ChangeProjectionByRectangle(TRectEntity(aInput).Rect, Device.Page);
      end
      else
      if aInput is TCoordinate then
      begin
         Device.Projection.ChangeProjectionByRect(TCoordinate(aInput).GetRegion, Device.Page);
      end;
   except
   end;
   inherited SetInput(Envelope);
end;


procedure   TGraphicForm.CreateForm;
begin
   aForm := TGraphic_Form.Create(Application);
   Form.Tool := Self;
   Form.IsPossibleToClose := false;
end;

procedure   TGraphicForm.CreateDevice;
begin
   Device := TImageCanvasDevice.Create;
end;

procedure   TGraphicForm.LinkDeviceToForm;
begin
  if Form = nil then exit;
   with TImageCanvasDevice(Device) do
   begin
      //Canvas := Form.Image.Canvas;
      Bitmap := Form.Image.Picture.Bitmap;
   end;
end;

procedure   TGraphicForm.CreateInterpret;
begin
   Interpret := TUniversalInterpret.Create;
end;



procedure   TGraphicForm.LinkInterpretToDevice;
begin
   Interpret.Device := Device;
end;

procedure   TGraphicForm.CreateTools;
begin
end;

procedure   TGraphicForm.SetActiveToolIndex(aIndex: longInt);
begin
   if aIndex = FActiveToolIndex then exit;
   inherited SetActiveToolIndex(aIndex);
end;

procedure   TGraphicForm.SetNextTool(aTool: TTool);
begin
   {
   aTool.Device := Device;
   aTool.Manager:= Self;
   AddTool(aTool);
   ActiveTool := aTool;
   }
   //CheckMenuItem(aTool.ClassName);
end;


procedure   TGraphicForm.SetVisible(const Value: boolean);
begin
  if Form = nil then exit;
   try
      Form.Visible := Value;
      Device.SetupPage(nil);
   except
   end;
end;


function    TGraphicForm.GetCaption: string;
begin
  result:= '';
  if Form = nil then exit;

   result := Form.Caption;
end;

procedure   TGraphicForm.SetCaption(const aValue: string);
begin
  if Form = nil then exit;
   Form.Caption := aValue;
end;




function    TGraphicForm.AddTool(aTool: TTool): TTool;
begin
   result := inherited AddTool(aTool);
end;


function    TGraphicForm.InsertTool(aIndex: longInt; aTool: TTool): TTool;
begin
   result :=  inherited InsertTool(aIndex, aTool);
end;

procedure   TGraphicForm.RemoveTool(aTool: TTool);
begin
   inherited RemoveTool(aTool);
end;

procedure   TGraphicForm.FreeTool(aTool: TTool);
begin
   inherited FreeTool(aTool);
end;

procedure   TGraphicForm.FreeTools;
var
   i: longInt;
begin
   for i := 0 to Tools.Count do
   begin
      FreeTool(TTool(Tools.Elements[i]));
   end;
end;

procedure   TGraphicForm.Show;
begin
   Visible := true;
end;


procedure   TGraphicForm.ShowAll;
begin
  if FInput = nil then exit;
  try
     if FInput is TRectEntity then
     begin
        Device.Projection.ChangeProjectionByRectangle(TRectEntity(FInput).Rect, Device.Page);
     end
     else
     if FInput is TCoordinate then
     begin
        Device.Projection.ChangeProjectionByRect(TCoordinate(FInput).GetRegion, Device.Page);
     end;
  except
  end;
end;



procedure   TGraphicForm.on_tm_Open(var Handled: boolean);
begin
end;
(*
var
   Loader: TLoader;
   SelEl: TElement;
   SwapG: boolean;
begin
   if Form.OpenDialog.Execute then
   begin
      SwapG := true;
      Loader := TLoader.CreateAll(Form.OpenDialog.FileName);
      try
         Form.OpenDialog.InitialDir := ExtractFilePath(Form.OpenDialog.FileName);
         Loader.DoIt;
         Input := Loader.Output;
         Loader.Output.Free;
         PostMsg(tm_Review, Self, nil, DontKill);
      except
      end;
      Loader.Free;
   end
end;
*)

procedure   TGraphicForm.on_tm_SaveAs(var Handled: boolean);
begin
end;
(*
var
   Saver: TSaver;
begin
   Form.SaveDialog.InitialDir := Form.OpenDialog.InitialDir;
   Form.SaveDialog.FileName   := Form.OpenDialog.FileName;
   if Form.SaveDialog.Execute then
   begin
      Form.OpenDialog.InitialDir := ExtractFilePath(Form.SaveDialog.FileName);
      Form.OpenDialog.FileName := Form.SaveDialog.FileName;
      if Input = nil then exit;
      Input.ResetID;
      Saver := TSaver.CreateAll(Form.SaveDialog.FileName);
      Saver.Input := Input;
      Saver.DoIt;
      Saver.Free
   end;
end;
*)



procedure   TGraphicForm.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);

begin
  {
  if Form = nil then exit;
    Handled := true;
    begin
       Coord := Device.Projection.GetModelPoint(aMessage.x, aMessage.y);
       Form.l_x.Caption := FloatToStrF(Coord.x, ffFixed, 12, 2);
       Form.l_y.Caption := FloatToStrF(Coord.y, ffFixed, 12, 2);
    end;
   }
end;

procedure TGraphicForm.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
{
var
   P: Wintypes.TPoint;
}
begin
  (*
  if Form = nil then exit;
   Handled := true;
   if aMessage.Button = mbRight then
   begin
      {
      if Interpret.Busy then
      begin
         PostMouseEvent(tm_MouseDown, Self, aMessage.Button,
                        aMessage.Shift, aMessage.X, aMessage.Y, aMessage.ActiveDevice);
         exit;
      end;
      }
      P := Point(aMessage.x, aMessage.y);
//      P := Form.PaintBox.ClientToScreen(P);
      P := Form.Image.ClientToScreen(P);
      Form.PopupMenu1.PopUp(P.x,P.y);
   end;
   *)
end;



procedure   TGraphicForm.on_tm_QuitTask;
begin
   if Interpret.Busy then
   begin
      Status := Status + [st_DIE];
      Interpret.BreakWork := true;
   end
   else
   begin
     if Form <> nil then
        Form.Free;
      inherited on_tm_QuitTask;
   end;
end;

procedure   TGraphicForm.on_tm_Done(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := true;
   if aMessage.Sender is TUniversalInterpret then
   begin
      if st_DIE in Status then
      begin
        if Form <> nil then
           Form.Free;
         inherited on_tm_QuitTask;
      end
      else
      if st_BREAK_AND_DOIT in Status then
      begin
         Status := Status - [st_BREAK_AND_DOIT];
         DoIt;
      end;
   end;
end;


procedure   TGraphicForm.on_tm_ToolSelected(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   inherited;
   (*
   if not (aMessage.Data is TInterActiveTool) then exit;

   if ActiveTool is TInterActiveTool then
      TInterActiveTool(ActiveTool).On_DeActivate;

   ActiveTool := TInterActiveTool(aMessage.Data);
   TInterActiveTool(ActiveTool).Device := Device;
   ActiveTool.Input  := Self.Input;
   TInterActiveTool(ActiveTool).On_Activate;
   Handled := true;
   CheckMenuItem(ActiveTool.ClassName);
   FormCursor := TInterActiveTool(ActiveTool).Cursor;
   *)
end;

procedure   TGraphicForm.on_tm_Print(var Handled: boolean);
{
var
   P: TPrint;
}
begin
{
   P := TPrint.Create;
   P.Input := Interpret;
   P.DoIt;
   P.Free;
}
end;


procedure TGraphicForm.ShowScale(aVal: string);
begin
{
   try
       if (Form <> nil) and (Form.l_m <> nil) then
          Form.l_M.Caption := aVal;
   except
   end;
}
end;



procedure   TGraphicForm.On_tm_ReView(var aMessage: TOrbisMessage; var Handled: boolean);
//var
   // s: string;
   //aX,aY,aScale: double;
begin
   if Idle then exit;
   if Interpret.Busy then
   begin
      Status := Status + [st_BREAK_AND_DOIT];
      Interpret.BreakWork := true;
      exit;
   end;
   Device.Pen.Mode:= pmCopy; { novinka 10.6.97 }

   inherited On_tm_ReView(aMessage, Handled);
{
   if Scale < 10 then
   begin
      Str(Device.Projection.Scale:0:3,S);
   end
   else S := IntToStr(round(Device.Projection.Scale));

   ShowScale(S);
   if CLoseAfterPrinting then PostMsg(tm_Print, Self, nil, DontKill);
}
   if ActiveTool <> nil then PostMsg(tm_Review,ActiveTool,nil,dontkill);
   Application.ProcessMessages;
   PostMsg(tm_Done, nil, self, dontKill);

   //aX := Device.Projection.Center_X;
   //aY := Device.Projection.Center_Y;
   //aScale := Device.Projection.Scale;
   {
   if (aX <> 0) and (aY <> 0) and (aScale <> 0) then
   begin
      TStringElement(TEntity(World.SystemRegister.FindElementName('Projection')).FindElementName('X')).AsFloat := aX;
      TStringElement(TEntity(World.SystemRegister.FindElementName('Projection')).FindElementName('Y')).AsFloat := aY;
      TStringElement(TEntity(World.SystemRegister.FindElementName('Projection')).FindElementName('Scale')).AsFloat := aScale;
      TStringElement(TEntity(World.SystemRegister.FindElementName('Projection')).FindElementName('VirtualScale')).AsFloat := Device.Projection.VirtualScale;;
      Form.Maps.ItemIndex := Form.Maps.Items.IndexOf(TStringElement(TEntity(World.SystemRegister.FindElementName('Projection')).FindElementName('MAP')).Value);

      //Form.Maps.Text :=  TStringElement(TEntity(World.SystemRegister.FindElementName('Projection')).FindElementName('MAP')).Value;
   end;
   }
   //TDeviceTool(Tool).Device.Projection.Scale := aScale;
   //Tool.PostMsg(tm_ModelChanged, nil, nil, false);
end;

procedure TGraphicForm.On_tm_ReviewByElement(var aMessage: TOrbisMessage;
  var Handled: boolean);
//var
   //s: string;
begin
   if Idle then exit;
   if Interpret.Busy then
   begin
      Status := Status + [st_BREAK_AND_DOIT];
      Interpret.BreakWork := true;
      exit;
   end;
   Device.Pen.Mode:= pmCopy; { novinka 10.6.97 }

   inherited On_tm_ReViewByElement(aMessage, Handled);
{
   if Scale < 10 then
   begin
      Str(Device.Projection.Scale:0:3,S);
   end
   else S := IntToStr(round(Device.Projection.Scale));

   ShowScale(S);
   if CLoseAfterPrinting then PostMsg(tm_Print, Self, nil, DontKill);
}
   //if ActiveTool <> nil then PostMsg(tm_ReviewByElement,ActiveTool,nil,dontkill);
   Application.ProcessMessages;
   PostMsg(tm_Done, nil, self, dontKill);
end;


procedure TGraphicForm.On_tm_Resize(var Handled: boolean);
begin
   Handled := true;
   {
   PostMsg(tm_BreakInterpret, Self, nil, DontKill);
   }
   Device.SetupPage(nil);
   if Idle then exit;
   PostMsg(tm_BreakAndReView, Self, nil, DontKill);
end;

procedure TGraphicForm.On_tm_RePaint(var Handled: boolean);
begin
   //PostTextMsg(tm_AddToProtocol, 'Repaint', nil);
   Handled := true;
   {
   PostMsg(tm_BreakInterpret, Self, nil, DontKill);
   }
   if assigned(Form) then
      Form.Image.Refresh;
end;



procedure TGraphicForm.On_tm_EndOfPrint(var Handled: boolean);
begin
   Handled := true;
   {
   if CloseAfterPrinting then
   begin
      PostMsgToOwners(tm_EndOfPrint, nil, DontKill);
      on_tm_QuitTask;
   end;
   }
end;



procedure TGraphicForm.On_tm_NewScale(var Handled: boolean);
(*
var
   aForm: TF_SetScale;
   D: double;
*)
begin
(*
  try
    Application.CreateForm(Tf_SetScale, aForm);
    try
      aForm.Combo.Text := IntToStr(round(Scale));
      case aForm.ShowModal of
         mrOK:
         begin
            D:= StrToExtended(aForm.Combo.Text);
            if D > 0 then
            begin
              Scale := D;
              PostMsg(tm_BreakAndReView, self, nil, DontKill);
              Handled:= true
            end
            else
              ShowAll
         end;
      end;
    finally
      aForm.Free;
    end
  except
  end
*)
end;


procedure   TGraphicForm.On_tm_ShowAll(var Handled: boolean);
begin
  ShowAll;
  Handled:= true;
  if Idle then exit;
  PostMsg(tm_BreakAndReView, self, nil, DontKill);
end;



procedure TGraphicForm.On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean);
var
   P: TPoint;
begin
   if Idle then exit;
   case aMessage.Key of
      VK_F5:
      begin
         if ssCtrl in aMessage.Shift then
            ViewAll := true
         else
         begin
            Scale:= Scale / 2;
            PostMsg(tm_BreakAndReView, Self, nil, DontKill);
            PostMsg(tm_ProjectionChanged, Self, Device.Projection, false);
         end;
      end;
      VK_F6:
      begin
         Scale:= Scale * 2;
         PostMsg(tm_BreakAndReView, Self, nil, DontKill);
         PostMsg(tm_ProjectionChanged, Self, Device.Projection, false);
      end;
      VK_F7:
      begin
         P := Form.Image.ScreenToClient(Mouse.CursorPos);
         Device.Projection.Center_X := Device.Projection.GetModelX(P.X);
         Device.Projection.Center_Y := Device.Projection.GetModelY(P.Y);
         PostMsg(tm_BreakAndReView, Self, nil, DontKill);
         PostMsg(tm_ProjectionChanged, Self, Device.Projection, false);
      end;
      VK_ESCAPE:
      begin
         PostMsg(tm_BreakInterpret, Self, nil, DontKill);
      end;
   end;
end;



procedure TGraphicForm.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
  // if aMessage.Message = tm_Resize then
  //    PostTextMsg(tm_AddToProtocol,Name + '.on_Resize GF',nil);
  (*
   try
     if (ActiveTool <> nil) and (aMessage.Message <> tm_Review) then
        ActiveTool.HandleMessage(aMessage, Handled);
   except
   end;
   if Handled then exit;
   *)
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      //tm_ToolSelected: on_tm_ToolSelected(aMessage, Handled);
      tm_Print:        on_tm_Print(Handled);
      tm_EndOfPrint:   on_tm_EndOfPrint(Handled);
      tm_Open:         on_tm_Open(Handled);
      tm_SaveAs:       on_tm_SaveAs(Handled);
      tm_NewScale:     on_tm_NewScale(Handled);
      tm_ShowAll:      on_tm_ShowAll(Handled);
      tm_RePaint:      on_tm_RePaint(Handled);
      tm_ChangeThematic: on_tm_CHangeThematic(aMessage, Handled);
   end;
end;




function TGraphicForm.GetForm: TGraphic_Form;
begin
   result := TGraphic_Form(aForm);
end;

procedure TGraphicForm.SetViewAll(const Value: boolean);
begin
   if not Value then exit;
   ShowAll;
   if Idle then exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, self, Projection, DontKill);
end;

procedure TGraphicForm.On_tm_Cursor(var aMessage: TOrbisCursorEvent;
  var Handled: boolean);
begin
   Cursor := aMessage.Cursor;
end;

function TGraphicForm.GetCursor: TCursor;
begin
  result:= 0;
  if Form = nil then exit;
  result := Form.Image.Cursor;
end;

procedure TGraphicForm.SetCursor(const Value: TCursor);
begin
   if Form = nil then exit;
   if Form.Image.Dragging then Form.Image.DragCursor := Value
                          else Form.Image.Cursor := Value;
end;

procedure TGraphicForm.SetViewByElement(const Value: TElement);
var
   R: TRectEntity;
begin
   if not (Value is TRectEntity) then exit;
   R := TRectEntity(Value);
   Projection.ModelView := R.Rect.AsDoubleRect;
   if Idle then exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
end;

procedure TGraphicForm.SetShiftView(const Value: string);
var
   DX,DY: double;
begin
   if Idle then exit;
   DX := Projection.ModelView.MaxX - Projection.ModelView.MinX;
   DY := Projection.ModelView.MaxY - Projection.ModelView.MinY;
   if CompareText(Value,'UP')=0 then
      Projection.Center_Y := Projection.Center_Y + DY * 0.75
   else
   if CompareText(Value,'DOWN')=0 then
      Projection.Center_Y := Projection.Center_Y - DY * 0.75
   else
   if CompareText(Value,'LEFT')=0 then
      Projection.Center_X := Projection.Center_X - DX * 0.75
   else
   if CompareText(Value,'RIGHT')=0 then
      Projection.Center_X := Projection.Center_X + DX * 0.75
   else
      exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, self, Projection, DontKill);
end;

function TGraphicForm.GetText: string;
begin
   result := '';
   if Form = nil then exit;
   result := Form.StatusBar.Panels.Items[0].Text;
end;

procedure TGraphicForm.SetText(const Value: string);
begin
   if Form = nil then exit;
   Form.StatusBar.Panels.Items[0].Text := value;
end;

procedure TGraphicForm.SetZoom(const Value: string);
begin
   if Idle then exit;
   if CompareText(Value,'IN')=0 then
      Scale:= Scale / 1.5
   else
   if CompareText(Value,'OUT')=0 then
      Scale:= Scale * 1.5
   else
      exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, self, Projection, DontKill);
end;

procedure TGraphicForm.On_tm_ChangeThematic(var aMessage: TOrbisMessage; var Handled: boolean);
var
   aX,aY,aScale: double;
begin
   Handled := true;
   aX := Device.Projection.Center_X;
   aY := Device.Projection.Center_Y;
   aScale := Device.Projection.Scale;
   Input := aMessage.Data;
   if (aX <> 0) and (aY <> 0) then
   begin
      Device.Projection.Center_X := aX;
      Device.Projection.Center_Y := aY;
      Device.Projection.Scale := aScale;
   end
   else
      PostMsg(tm_ProjectionCHanged,Self,Device.Projection,false);
end;

procedure TGraphicForm.BeginUpdate;
begin
   Idle := true;
   FImageW := Form.Image.Width;
   FImageH := Form.Image.Height;
end;

procedure TGraphicForm.EndUpdate;
begin
   Idle := false;
   if (FImageW <> Form.Image.Width) or
      (FImageH <> Form.Image.Height) then
      PostMsg(tm_ReSize, Self, nil, false);
end;

procedure TGraphicForm.SetElementForProjection(const Value: TElement);
begin
   if FInput = nil then exit;
   try
      if Value is TRectEntity then
      begin
         Device.Projection.ChangeProjectionByRectangle(TRectEntity(Value).Rect, Device.Page);
      end
      else
      if Value is TCoordinate then
      begin
         Device.Projection.ChangeProjectionByRect(TCoordinate(Value).GetRegion, Device.Page);
      end;
   except
   end;
   if (MinScale > 0) and (Scale < MinScale) then
      Scale := MinScale;
   if Idle then exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, self, Projection, DontKill);
end;

procedure TGraphicForm.SetElementForPattern(const Value: TElement);
var
   GI: TSelectedGeometryInterpret;
   h: boolean;
begin
   GI := TSelectedGeometryInterpret.Create;
   GI.Input := Value;
   GI.Reset(Device);
   GI.DoIt;
   on_tm_Repaint(h);
   Application.ProcessMessages;
   GI.Free;
end;

procedure TGraphicForm.SetMinScale(const Value: double);
begin
  FMinScale := Value;
end;

function TGraphicForm.GetText1: string;
begin
   result := '';
   if Form = nil then exit;
   result := Form.StatusBar.Panels.Items[1].Text;
end;

procedure TGraphicForm.SetText1(const Value: string);
begin
   if Form = nil then exit;
   Form.StatusBar.Panels.Items[1].Text := value;
end;

procedure TGraphicForm.On_tm_MouseWheelDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   Handled := true;
   Scale:= Scale * 0.75;
   PostMsg(tm_BreakAndReView, Self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, Self, Device.Projection, false);
end;

procedure TGraphicForm.On_tm_MouseWheelUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   Handled := true;
   Scale:= Scale * 1.5;
   PostMsg(tm_BreakAndReView, Self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, Self, Device.Projection, false);
end;

function TGraphicForm.GetText2: string;
begin
   result := '';
   if Form = nil then exit;
   result := Form.StatusBar.Panels.Items[2].Text;
end;

procedure TGraphicForm.SetText2(const Value: string);
begin
   if Form = nil then exit;
   Form.StatusBar.Panels.Items[2].Text := value;
end;

INITIALIZATION
   RegisterOrbisClass(TGraphicForm);
END.
