unit t_OrbisGraphicFrame;

interface
uses
   Controls,
   e_IScrpt,
   e_Proje,
   Elements,
   m_Intera,
   r_OrbisGraphicFrame,
   t_OrbisFrame;

type

TOrbisGraphicFrame = class(TOrbisFrame)
private
    FImageW: integer;
    FImageH: integer;
    FMinScale: double;
    function GetFrame: TOrbisGraphic_Frame;
    function GetActiveToolClassName: string;
    function GetActiveToolName: string;
    function GetInterpretScript: TInterpretScript;
    function GetPageColor: integer;
    function GetProjection: TProjection;
    procedure SetActiveToolClassName(const Value: string);
    procedure SetActiveToolName(const Value: string);
    procedure SetBreakReView(const Value: boolean);
    procedure SetElementForKillReView(const Value: TElement);
    procedure SetElementForPattern(const Value: TElement);
    procedure SetElementForProjection(const Value: TElement);
    procedure SetElementForReView(const Value: TElement);
    procedure SetInterpretScript(const Value: TInterpretScript);
    procedure SetNewThematic(const Value: TElement);
    procedure SetPageColor(const Value: integer);
    procedure SetShiftView(const Value: string);
    procedure SetViewAll(const Value: boolean);
    procedure SetViewByElement(const Value: TElement);
    procedure SetZoom(const Value: string);
    procedure SetMinScale(const Value: double);
protected
    procedure   SetInput(aInput: TElement); override;
    procedure   CreateDevice; virtual;
    procedure   LinkDeviceToGFrame; virtual;
    procedure   CreateInterpret; virtual;
    procedure   LinkInterpretToDevice; virtual;
    procedure   CreateTools; virtual;
public
   constructor Create; override;
   procedure   DoIt; override;

   procedure   CreateFrame; override;
   function    GetCursor: TCursor; override;
   procedure   SetCursor(const Value: TCursor); override;

   procedure   ShowAll;
   procedure   on_tm_QuitTask; override;
   procedure   on_tm_Done(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   On_tm_ReviewByElement(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   On_tm_Resize(var Handled: boolean); override;
   procedure   On_tm_RePaint(var Handled: boolean); virtual;
   procedure   on_tm_ShowAll(var Handled: boolean); virtual;
   procedure   On_tm_Cursor(var aMessage: TOrbisCursorEvent; var Handled: boolean); override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   procedure   On_tm_ChangeThematic(var aMessage: TOrbisMessage; var Handled: boolean);
   procedure   On_tm_ProjectionChanged(var aMessage: TOrbisMessage; var Handled: boolean);
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   //procedure   BeginUpdate; override;
   //procedure   EndUpdate; override;
   property    GFrame: TOrbisGraphic_Frame read GetFrame;
published
   property    ActiveToolName: string read GetActiveToolName write SetActiveToolName;
   property    ActiveToolClassName: string read GetActiveToolClassName write SetActiveToolClassName;
   property    NewThematic: TElement write SetNewThematic;
   property    PageColor: integer read GetPageColor write SetPageColor;
   property    Projection: TProjection read GetProjection;
   property    InterpretScript: TInterpretScript read GetInterpretScript write SetInterpretScript;
   property    BreakReView: boolean write SetBreakReView stored false;
   property    ElementForReView: TElement write SetElementForReView;
   property    ElementForKillReView: TElement write SetElementForKillReView;
   property    ElementForProjection: TElement write SetElementForProjection;
   property    ElementForPattern: TElement write SetElementForPattern;
   property    ReViewByElement: TElement write SetViewByElement;
   property    ShiftView: string write SetShiftView;
   property    Zoom: string write SetZoom;
   property    ViewAll: boolean write SetViewAll;
   property    MinScale: double read FMinScale write SetMinScale;
end;

implementation
uses
   Forms,
   Graphics,
   Windows,
   SysUtils,
   d_StdImg,
   e_Class,
   g_Elems,
   i_Uni,
   i_SelectedGeometry,
   m_Orbis,
   t_InterfaceTool,
   Tools;

{ TOrbisGraphicFrame }


constructor TOrbisGraphicFrame.Create;
begin
   inherited; // tady se vytv GFrame
   CreateDevice;
   //LinkDeviceToImage;
   CreateInterpret;
   LinkInterpretToDevice;
   LinkDeviceToGFrame;
   CreateTools;
end;

procedure TOrbisGraphicFrame.CreateFrame;
begin
   aFrame := TOrbisGraphic_Frame.Create(nil);
   GFrame.Tool := Self;
end;

procedure TOrbisGraphicFrame.DoIt;
begin
   if Idle then exit;
   PostMsg(tm_ReView, Self, nil, DontKill);
end;


function TOrbisGraphicFrame.GetActiveToolClassName: string;
var
   T: TTool;
begin
   result := '';
   T := ActiveTool;
   if T = nil then exit;
   result := T.ClassName;
end;

function TOrbisGraphicFrame.GetActiveToolName: string;
var
   T: TTool;
begin
   result := '';
   T := ActiveTool;
   if T = nil then exit;
   result := T.Name;
end;

function TOrbisGraphicFrame.GetCursor: TCursor;
begin
  result:= 0;
  if GFrame = nil then exit;
  result := GFrame.Image.Cursor;
end;

function TOrbisGraphicFrame.GetFrame: TOrbisGraphic_Frame;
begin
   result := TOrbisGraphic_Frame(aFrame);
end;

function TOrbisGraphicFrame.GetInterpretScript: TInterpretScript;
begin
   result := nil;
   if Interpret is TUniversalInterpret then result := TUniversalInterpret(Interpret).Script;
end;

function TOrbisGraphicFrame.GetPageColor: integer;
begin
   try
      result := Device.Page.Color;
   except
      result := 0;
   end;
end;

function TOrbisGraphicFrame.GetProjection: TProjection;
begin
   if Device <> nil then result := Device.Projection
                    else result := nil;
end;

procedure TOrbisGraphicFrame.HandleMessage(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   Handled := false;
  // if aMessage.Message = tm_Resize then
  //    PostTextMsg(tm_AddToProtocol,Name + '.on_Resize GF',nil);
  (*
   try
     if (ActiveTool <> nil) and (aMessage.Message <> tm_Review) then
        ActiveTool.HandleMessage(aMessage, Handled);
   except
   end;
   if Handled then exit;
   *)
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      //tm_ToolSelected: on_tm_ToolSelected(aMessage, Handled);
      //tm_Print:        on_tm_Print(Handled);
      //tm_EndOfPrint:   on_tm_EndOfPrint(Handled);
      //tm_Open:         on_tm_Open(Handled);
      //tm_SaveAs:       on_tm_SaveAs(Handled);
      //tm_NewScale:     on_tm_NewScale(Handled);
      tm_ShowAll:      on_tm_ShowAll(Handled);
      tm_RePaint:      on_tm_RePaint(Handled);
      tm_ChangeThematic:  on_tm_CHangeThematic(aMessage, Handled);
      //tm_ElementSelected: on_tm_ElementSelected(aMessage, Handled);
      tm_ProjectionCHanged: on_tm_ProjectionChanged(aMessage, Handled);
   end;
end;

procedure TOrbisGraphicFrame.On_tm_Cursor(var aMessage: TOrbisCursorEvent;var Handled: boolean);
begin
   Cursor := aMessage.Cursor;
end;

procedure TOrbisGraphicFrame.on_tm_Done(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   Handled := true;
   if aMessage.Sender is TUniversalInterpret then
   begin
      if st_DIE in Status then
      begin
        {
        if Form <> nil then
           Form.Free;
         }
         inherited on_tm_QuitTask;
      end
      else
      if st_BREAK_AND_DOIT in Status then
      begin
         Status := Status - [st_BREAK_AND_DOIT];
         DoIt;
      end;
   end;
end;

procedure TOrbisGraphicFrame.On_tm_ChangeThematic(
  var aMessage: TOrbisMessage; var Handled: boolean);
var
   aX,aY,aScale: double;
   T: TTool;
   IT: TInterfaceTool;
begin
   Handled := true;
   aX := Device.Projection.Center_X;
   aY := Device.Projection.Center_Y;
   aScale := Device.Projection.Scale;
   Input := aMessage.Data;
   // pas kvli ShiftImage apod. Ztrc se input a vbec
   try
      T := ActiveTool;
      if T <> nil then
      begin
         if (T is TInterfaceTool) then
         begin
            IT := TInterfaceTool(T);
            if IT.ShareInput then
               IT.Input := aMessage.Data;
         end;
      end;
   except
   end;
   // pas END
   if (aX <> 0) and (aY <> 0) then
   begin
      Device.Projection.Center_X := aX;
      Device.Projection.Center_Y := aY;
      Device.Projection.Scale := aScale;
   end
   else
      PostMsg(tm_ProjectionCHanged,Self,Device.Projection,false);
end;

procedure TOrbisGraphicFrame.On_tm_Keydown(var aMessage: TOrbisKeyEvent;
  var Handled: boolean);
begin
   if Idle then exit;
   case aMessage.Key of
      VK_F5:
      begin
         Scale:= Scale / 2;
         PostMsg(tm_BreakAndReView, Self, nil, DontKill);
         PostMsg(tm_ProjectionChanged, Self, Device.Projection, false);
      end;
      VK_F6:
      begin
         Scale:= Scale * 2;
         PostMsg(tm_BreakAndReView, Self, nil, DontKill);
         PostMsg(tm_ProjectionChanged, Self, Device.Projection, false);
      end;
      VK_ESCAPE:
      begin
         PostMsg(tm_BreakInterpret, Self, nil, DontKill);
      end;
   end;
end;

procedure TOrbisGraphicFrame.On_tm_MouseMove(
  var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
  inherited;

end;

procedure TOrbisGraphicFrame.On_tm_MouseUp(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
begin
  inherited;

end;

procedure TOrbisGraphicFrame.On_tm_ProjectionChanged(
  var aMessage: TOrbisMessage; var Handled: boolean);
begin

end;

procedure TOrbisGraphicFrame.on_tm_QuitTask;
begin
   if Interpret.Busy then
   begin
      Status := Status + [st_DIE];
      Interpret.BreakWork := true;
   end
   else
   begin
     {
     if Form <> nil then
        Form.Free;
      }  
      inherited on_tm_QuitTask;
   end;
end;

procedure TOrbisGraphicFrame.On_tm_RePaint(var Handled: boolean);
begin
   Handled := true;
   {
   PostMsg(tm_BreakInterpret, Self, nil, DontKill);
   }
   if assigned(GFrame) then
      GFrame.Image.Refresh;
end;

procedure TOrbisGraphicFrame.On_tm_Resize(var Handled: boolean);
begin
   Handled := true;
   {
   PostMsg(tm_BreakInterpret, Self, nil, DontKill);
   }
   Device.SetupPage(nil);
   if Idle then exit;
   PostMsg(tm_BreakAndReView, Self, nil, DontKill);
end;

procedure TOrbisGraphicFrame.On_tm_Review(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   if Idle then exit;
   if Interpret.Busy then
   begin
      Status := Status + [st_BREAK_AND_DOIT];
      Interpret.BreakWork := true;
      exit;
   end;
   Device.Pen.Mode:= pmCopy; { novinka 10.6.97 }

   inherited On_tm_ReView(aMessage, Handled);
   if ActiveTool <> nil then PostMsg(tm_Review,ActiveTool,nil,dontkill);
   Application.ProcessMessages;
   PostMsg(tm_Done, nil, self, dontKill);

end;

procedure TOrbisGraphicFrame.On_tm_ReviewByElement(
  var aMessage: TOrbisMessage; var Handled: boolean);
begin
   if Idle then exit;
   if Interpret.Busy then
   begin
      Status := Status + [st_BREAK_AND_DOIT];
      Interpret.BreakWork := true;
      exit;
   end;
   Device.Pen.Mode:= pmCopy; { novinka 10.6.97 }

   inherited On_tm_ReViewByElement(aMessage, Handled);
   Application.ProcessMessages;
   PostMsg(tm_Done, nil, self, dontKill);
end;

procedure TOrbisGraphicFrame.on_tm_ShowAll(var Handled: boolean);
begin
  ShowAll;
  Handled:= true;
  if Idle then exit;
  PostMsg(tm_BreakAndReView, self, nil, DontKill);
end;

procedure TOrbisGraphicFrame.SetActiveToolClassName(const Value: string);
var
   T: TTool;
begin
   T := nil;
   try
      T := TTool(Tools.FindElementClass(FindOrbisClass(Value)));
   except
   end;
   if T = nil then exit;
   T.PostMsgToOwners(tm_ToolSelected, T, DontKill);
end;

procedure TOrbisGraphicFrame.SetActiveToolName(const Value: string);
var
   T: TTool;
begin
   T := TTool(Tools.FindElementName(Value));
   if T = nil then exit;
   T.PostMsgToOwners(tm_ToolSelected, T, DontKill);
end;

procedure TOrbisGraphicFrame.SetBreakReView(const Value: boolean);
begin
   if Interpret.Busy then Interpret.BreakWork := true;
end;

procedure TOrbisGraphicFrame.SetCursor(const Value: TCursor);
begin
   if GFrame = nil then exit;
   if GFrame.Image.Dragging then GFrame.Image.DragCursor := Value
                            else GFrame.Image.Cursor := Value;
end;

procedure TOrbisGraphicFrame.SetElementForKillReView(const Value: TElement);
begin
   if Idle then exit;
   PostMsg(tm_ReView, Self, Value, KillIt);
end;

procedure TOrbisGraphicFrame.SetElementForPattern(const Value: TElement);
var
   GI: TSelectedGeometryInterpret;
   h: boolean;
begin
   GI := TSelectedGeometryInterpret.Create;
   GI.Input := Value;
   GI.Reset(Device);
   GI.DoIt;
   on_tm_Repaint(h);
   Application.ProcessMessages;
   GI.Free;
end;

procedure TOrbisGraphicFrame.SetElementForProjection(
  const Value: TElement);
begin
   if FInput = nil then exit;
   try
      if Value is TRectEntity then
      begin
         Device.Projection.ChangeProjectionByRectangle(TRectEntity(Value).Rect, Device.Page);
      end
      else
      if Value is TCoordinate then
      begin
         Device.Projection.ChangeProjectionByRect(TCoordinate(Value).GetRegion, Device.Page);
      end;
   except
   end;
   if (MinScale > 0) and (Scale < MinScale) then
      Scale := MinScale;
   if Idle then exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, self, Projection, DontKill);
end;

procedure TOrbisGraphicFrame.SetElementForReView(const Value: TElement);
begin
   if Idle then exit;
   PostMsg(tm_ReView, Self, Value, DontKill);
end;

procedure TOrbisGraphicFrame.SetInterpretScript(const Value: TInterpretScript);
begin
     if ((TElement(Value) is TInterpretScript) and (Interpret is TUniversalInterpret))
        then TUniversalInterpret(Interpret).Script := Value;
end;

procedure TOrbisGraphicFrame.SetNewThematic(const Value: TElement);
begin
   PostMsg(tm_ChangeThematic,Self,Value,false);
   Application.ProcessMessages;
end;

procedure TOrbisGraphicFrame.SetPageColor(const Value: integer);
begin
   try
      Device.Page.Color := Value;
   except
   end;
end;

procedure TOrbisGraphicFrame.SetShiftView(const Value: string);
var
   DX,DY: double;
begin
   if Idle then exit;
   DX := Projection.ModelView.MaxX - Projection.ModelView.MinX;
   DY := Projection.ModelView.MaxY - Projection.ModelView.MinY;
   if CompareText(Value,'UP')=0 then
      Projection.Center_Y := Projection.Center_Y + DY * 0.75
   else
   if CompareText(Value,'DOWN')=0 then
      Projection.Center_Y := Projection.Center_Y - DY * 0.75
   else
   if CompareText(Value,'LEFT')=0 then
      Projection.Center_X := Projection.Center_X - DX * 0.75
   else
   if CompareText(Value,'RIGHT')=0 then
      Projection.Center_X := Projection.Center_X + DX * 0.75
   else
      exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, self, Projection, DontKill);
end;

procedure TOrbisGraphicFrame.SetViewAll(const Value: boolean);
begin
   if not Value then exit;
   ShowAll;
   if Idle then exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, self, Projection, DontKill);
end;

procedure TOrbisGraphicFrame.SetViewByElement(const Value: TElement);
var
   R: TRectEntity;
begin
   if not (Value is TRectEntity) then exit;
   R := TRectEntity(Value);
   Projection.ModelView := R.Rect.AsDoubleRect;
   if Idle then exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
end;

procedure TOrbisGraphicFrame.SetZoom(const Value: string);
begin
   if Idle then exit;
   if CompareText(Value,'IN')=0 then
      Scale:= Scale / 1.5
   else
   if CompareText(Value,'OUT')=0 then
      Scale:= Scale * 1.5
   else
      exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, self, Projection, DontKill);
end;

procedure TOrbisGraphicFrame.ShowAll;
begin
  if FInput = nil then exit;
  try
     if FInput is TRectEntity then
     begin
        Device.Projection.ChangeProjectionByRectangle(TRectEntity(FInput).Rect, Device.Page);
     end
     else
     if FInput is TCoordinate then
     begin
        Device.Projection.ChangeProjectionByRect(TCoordinate(FInput).GetRegion, Device.Page);
     end;
  except
  end;
end;

procedure TOrbisGraphicFrame.CreateDevice;
begin
   Device := TImageCanvasDevice.Create;
end;

procedure TOrbisGraphicFrame.CreateInterpret;
begin
   Interpret := TUniversalInterpret.Create;
end;

procedure TOrbisGraphicFrame.LinkDeviceToGFrame;
begin
  if GFrame = nil then exit;
   with TImageCanvasDevice(Device) do
   begin
      //Canvas := Form.Image.Canvas;
      Bitmap := GFrame.Image.Picture.Bitmap;
   end;
end;

procedure TOrbisGraphicFrame.LinkInterpretToDevice;
begin
   Interpret.Device := Device;
end;

procedure TOrbisGraphicFrame.SetInput(aInput: TElement);
var
   Envelope: TRectEntity;
begin
   if FInput <> nil then
   begin
      FreeElement(FInput);
      FInput := nil;
   end;
   if aInput = nil then exit;
   Envelope := TRectEntity.Create;
   Envelope.Name := 'Oblka';      // p.k.17.1.99
   Envelope.AddAndUpdateRect(aInput{.CreateEnvelope});
   AddElement(Envelope);
   try
      if aInput is TRectEntity then
      begin
         Device.Projection.ChangeProjectionByRectangle(TRectEntity(aInput).Rect, Device.Page);
      end
      else
      if aInput is TCoordinate then
      begin
         Device.Projection.ChangeProjectionByRect(TCoordinate(aInput).GetRegion, Device.Page);
      end;
   except
   end;
   inherited SetInput(Envelope);
end;

procedure TOrbisGraphicFrame.CreateTools;
begin

end;

procedure TOrbisGraphicFrame.SetMinScale(const Value: double);
begin
  FMinScale := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TOrbisGraphicFrame);
END.
