unit t_ViewTools;

interface
uses
   Controls,
   Windows,
   m_Intera,
   t_InterfaceTool;

type

TStandardViewer = class(TInterfaceTool)
 protected
   ShiftRect: TRect;
 public
   function  GetCursor: TCursor; override;
   procedure RegisterCursors; override;
   procedure On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   procedure On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_StartDrag(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_DragEnd(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
end;


implementation
uses
   Classes,
   Dialogs,
   Forms,
   Graphics,
   e_Class,
   e_Device,
   g_Elems,
   m_Orbis;
   //u_Cursor;
{ TStandardViewer }

function TStandardViewer.GetCursor: TCursor;
begin
   result := inherited GetCursor;
end;


procedure TStandardViewer.On_tm_DragEnd(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
var
   R: TDoubleRect;
begin
    Handled := true;
    //if not aMessage.ActiveDevice then exit;
    Device.Pen.Mode  := pmXOR;
    Device.Pen.Color := clGreen;
    Device.Pen.Style := psSolid;
    Device.Brush.Style := bsClear;
    TCanvasDevice(Device).Canvas.Pen.Width := 3;
    with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
    Application.ProcessMessages;
    TCanvasDevice(Device).Canvas.Pen.Mode  := pmCopy;
    TCanvasDevice(Device).Canvas.Pen.Color := clBlack;
    TCanvasDevice(Device).Canvas.Pen.Width := 1;
    with ShiftRect do
    begin
       R := Device.Projection.GetModelRect(Left, Top, Right, Bottom);
       TCanvasDevice(Device).Projection.ModelView := R;
       PostMsgToOwners(tm_BreakAndReView, nil, false);
       PostMsgToOwners(tm_ProjectionChanged, Device.Projection, false);
    end;
    if CanBeFlash and (FlashTool <> nil) then
    begin
       Application.ProcessMessages;
       PostMsgToOwners(tm_ToolSelected, FlashTool, false);
    end;
end;

procedure TStandardViewer.On_tm_DragOver(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
begin
   Device.Pen.Mode  := pmXOR;
   Device.Pen.Color := clGreen;
   Device.Pen.Style := psSolid;
   Device.Brush.Style := bsClear;
   TCanvasDevice(Device).Canvas.Pen.Width := 3;
   with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
   ShiftRect.Right  := aMessage.x;
   ShiftRect.Bottom := aMessage.y;
   with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
   //Application.ProcessMessages;
   Handled := true;
end;

procedure TStandardViewer.On_tm_Keydown(var aMessage: TOrbisKeyEvent;
  var Handled: boolean);
begin
   Handled := false;
   {
   if aMessage.Key = VK_ESCAPE then PostMsgToOwners(tm_BreakInterpret, nil, DontKill);
   }
   case aMessage.Key of
      VK_F5:
      begin
         Scale:= Scale / 2;
         PostMsgToOwners(tm_BreakAndReView, nil, false);
         PostMsgToOwners(tm_ProjectionChanged, Device.Projection, false);
         Handled := True;
      end;
      VK_F6:
      begin
         Scale:= Scale * 2;
         PostMsgToOwners(tm_BreakAndReView, nil, false);
         PostMsgToOwners(tm_ProjectionChanged, Device.Projection, false);
         Handled := True;
    end;
      VK_ESCAPE:
      begin
         PostMsgToOwners(tm_BreakInterpret, nil, false);
         Handled := True;
      end;
   end;

end;

procedure TStandardViewer.On_tm_MouseDown(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
begin
   Application.ProcessMessages;
   PostMsgToOwners(tm_BreakInterpret, nil, false);
   if aMessage.Button = mbLeft then
   begin
      ShiftRect := Rect(aMessage.x,aMessage.y,aMessage.x,aMessage.y);
      Handled := true;
   end;
end;


procedure TStandardViewer.On_tm_MouseUp(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
begin
    Handled := true;
    if aMessage.Button = mbLeft then
    begin
       //if not aMessage.ActiveDevice then exit;
       with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
       TCanvasDevice(Device).Canvas.Pen.Mode  := pmCopy;
       TCanvasDevice(Device).Canvas.Pen.Color := clBlack;
       TCanvasDevice(Device).Canvas.Pen.Width := 1;
       Device.Projection.CenterPoint := Device.Projection.GetModelPoint(aMessage.x,aMessage.y);
       PostMsgToOwners(tm_BreakAndReView, nil, false);
       PostMsgToOwners(tm_ProjectionChanged, Device.Projection, false);
       if CanBeFlash and (FlashTool <> nil) then
       begin
          Application.ProcessMessages;
          PostMsgToOwners(tm_ToolSelected, FlashTool, false);
       end;
     end
     else handled := false;
end;

procedure TStandardViewer.On_tm_StartDrag(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
begin
   Application.ProcessMessages;
   {
   Device.Pen.Mode  := pmXOR;
   Device.Pen.Color := clGreen;
   Device.Pen.Style := psSolid;
   Device.Brush.Style := bsClear;
   //ShiftRect.Right  := aMessage.x;
   //ShiftRect.Bottom := aMessage.y;
   TCanvasDevice(Device).Canvas.Pen.Width := 3;
   with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
   }
   Handled := true;
end;

procedure TStandardViewer.RegisterCursors;
begin
   NewCursorName('WAIT');
end;

initialization
   RegisterOrbisClass(TStandardViewer);
end.
