unit e_DBTable;

interface
uses
   DbTables,
   DB,
   e_Files,
   Elements;

type

TDBFieldDefsCollection = class(TOwnerEntity)
protected
   function  GetName: string; override;
   procedure SetName(aName: string); override;
end;

TDBFieldDefElement = class(TOwnerElement)
  private
    FSize: integer;
    FPrecision: integer;
    FDataType: TFieldType;
    procedure SetDataType(const Value: TFieldType);
    procedure SetPrecision(const Value: integer);
    procedure SetSize(const Value: integer);
public
   procedure SetByFieldDef(aFieldDef: TFieldDef);
published
   property DataType: TFieldType read FDataType write SetDataType;
   property Size: integer read FSize write SetSize;
   property Precision: integer read FPrecision write SetPrecision;
end;


TDBFieldsCollection = class(TOwnerEntity)
protected
   function  GetName: string; override;
   procedure SetName(aName: string); override;
end;

TDBTableElement = class(TFileBasedEntity)
private
   FTab: TDataset;
   FFields: TDBFieldsCollection;
   FFieldDefs: TDBFieldDefsCollection;
    FNo_BDE: boolean;
    function GetTab: TDataset;
    function GetFields: TDBFieldsCollection;
    function GetActive: boolean;
    procedure SetActive(const Value: boolean);
    function GetEOF: boolean;
    procedure SetNext(const Value: boolean);
    function GetRecordCount: integer;
    function GetRecNo: integer;
    procedure SetRecNo(const Value: integer);
    function GetEdit: boolean;
    function GetEmptyTable: boolean;
    procedure SetAppend(const Value: boolean);
    procedure SetEdit(const Value: boolean);
    procedure SetEmptytable(const Value: boolean);
    procedure SetPost(const Value: boolean);
    procedure SetStoreFields(const Value: boolean);
    procedure SetNo_BDE(const Value: boolean);
    function GetFieldDefs: TDBFieldDefsCollection;
public
   property  Tab: TDataset read GetTab;
   Constructor Create; override;
   destructor Destroy; override;
   procedure LoadFromFile(aFileName: string); override;
   procedure RestoreFields;
   procedure ClearFields;
   property  No_BDE: boolean read FNo_BDE write SetNo_BDE;
published
   property Fields: TDBFieldsCollection read GetFields stored false;
   property FieldDefs: TDBFieldDefsCollection read GetFieldDefs stored false;
   property Active: boolean read GetActive write SetActive stored false;
   property EOF: boolean read GetEOF stored false;
   property Next: boolean write SetNext stored false;
   property RecordCount: integer read GetRecordCount stored false;
   property RecNo: integer read GetRecNo write SetRecNo stored false;
   property Edit: boolean read GetEdit write SetEdit stored false;
   property Append: boolean write SetAppend stored false;
   property EmptyTable: boolean read GetEmptyTable write SetEmptytable stored false;
   property StoreFields: boolean write SetStoreFields stored false;
   property Post: boolean write SetPost stored false;
end;

implementation
uses
   Dialogs,
   SysUtils,
   Forms,
   Dbf,
   u_Global,
   e_Class,
   e_Type;

{ TDBTableElement }

procedure TDBTableElement.ClearFields;
var
   i: integer;
begin
   for i := 0 to Fields.LastIndex do
   begin
      if Fields.Elements[i] is TIntegerElement then
         TIntegerElement(Fields.Elements[i]).Value := 0
      else
      if Fields.Elements[i] is TBooleanElement then
         TBooleanElement(Fields.Elements[i]).Value := false
      else
      if Fields.Elements[i] is TFloatElement then
         TFloatElement(Fields.Elements[i]).Value := 0
      else
      if Fields.Elements[i] is TStringElement then
         TStringElement(Fields.Elements[i]).Value := '';
   end;
end;

constructor TDBTableElement.Create;
begin
   No_BDE := NoBDE;
   inherited;
end;

destructor TDBTableElement.Destroy;
begin
   if FTab <> nil then
   begin
      FTab.Active := false;
      FTab.Free;
   end;
   inherited;
end;

function TDBTableElement.GetActive: boolean;
begin
   result := Tab.Active;
end;

function TDBTableElement.GetEdit: boolean;
begin
   result := Tab.State = dsEdit;
end;

function TDBTableElement.GetEmptyTable: boolean;
begin
   result := Tab.IsEmpty;
end;

function TDBTableElement.GetEOF: boolean;
begin
   result := Tab.Eof;
end;

function TDBTableElement.GetFieldDefs: TDBFieldDefsCollection;
begin
   if FFieldDefs = nil then
   begin
      FFieldDefs := TDBFieldDefsCollection(FindElementClass(TDBFieldDefsCollection));
      if FFieldDefs = nil then
      begin
         FFieldDefs := TDBFieldDefsCollection.Create;
         AddElement(FFieldDefs);
      end;
   end;
   result := FFieldDefs;
end;

function TDBTableElement.GetFields: TDBFieldsCollection;
begin
   if FFields = nil then
   begin
      FFields := TDBFieldsCollection(FindElementClass(TDBFieldsCollection));
      if FFields = nil then
      begin
         FFields := TDBFieldsCollection.Create;
         AddElement(FFields);
      end;
   end;
   result := FFields;
end;

function TDBTableElement.GetRecNo: integer;
begin
   result := Tab.RecNo;
end;

function TDBTableElement.GetRecordCount: integer;
begin
   result := Tab.RecordCount;
end;

function TDBTableElement.GetTab: TDataset;
begin
   if FTab = nil then
      if No_BDE then
         FTab := TDBF.Create(Application)
      else
         FTab := TTable.Create(Application);
   result := FTab;
end;

procedure TDBTableElement.LoadFromFile(aFileName: string);
begin
   if No_BDE  then
      TDBF(Tab).TableName := aFileName
   else
      TTable(Tab).TableName := aFileName; //@@@!
   Name := aFileName;
   //Tab.Active := true;
end;

procedure TDBTableElement.RestoreFields;
var
   i: integer;
   F: TField;
begin
   for i := 0 to Tab.FieldCount - 1 do
   begin
      F := Tab.Fields.Fields[i];
      case F.DataType of
         ftSmallint, ftInteger, ftWord:
         begin
            TIntegerElement(Fields.Elements[i]).Value := F.AsInteger;
         end;
         ftBoolean:
         begin
            TBooleanElement(Fields.Elements[i]).Value := F.AsBoolean;
         end;
         ftFloat:
         begin
            TFloatElement(Fields.Elements[i]).Value := F.AsFloat;
         end;
      else
         begin
            TStringElement(Fields.Elements[i]).Value := F.AsString;
         end;
      end;
   end;
end;

procedure TDBTableElement.SetActive(const Value: boolean);
var
   i: integer;
   F: TField;
   FD: TFieldDef;
   FDE: TDBFieldDefElement;
   IE: TIntegerElement;
   FE: TFloatElement;
   BE: TBooleanElement;
   SE: TStringElement;
begin
   if Value then
   begin
      //No_BDE := NoBDE;
      Error := 0;
      Fields.FreeAll;
      if No_BDE  then
         TDBF(Tab).TableName := FileName
      else
         TTable(Tab).TableName := FileName;  //@@@!
      try
      Tab.Active := true;
      except
        on E: Exception do
         ShowMessage('DB: ' + E.Message);
      end;
      if not Tab.Active then
      begin
         Error := 1;
         exit;
      end;
      // FieldDefs
      for i := 0 to Tab.FieldDefs.Count - 1 do
      begin
         FD := Tab.FieldDefs.Items[i];
         FDE := TDBFieldDefElement.Create;
         FieldDefs.AddElement(FDE);
         FDE.SetByFieldDef(FD);
      end;
      // Fields
      for i := 0 to Tab.FieldCount - 1 do
      begin
         F := Tab.Fields.Fields[i];
         case F.DataType of
            ftSmallint, ftInteger, ftWord:
            begin
               IE := TIntegerElement.CreateAll(F.FieldName,F.AsInteger);
               Fields.AddElement(IE);
            end;
            ftBoolean:
            begin
               BE := TBooleanElement.CreateAll(F.FieldName,F.AsBoolean);
               Fields.AddElement(BE);
            end;
            ftFloat:
            begin
               FE := TFloatElement.CreateAll(F.FieldName,F.AsFloat);
               Fields.AddElement(FE);
            end;
         else
            begin
               SE := TStringElement.CreateAll(F.FieldName,F.AsString);
               Fields.AddElement(SE);
            end;
         end;
      end;
   end
   else
   begin
      Tab.Active := false;
   end;
end;

procedure TDBTableElement.SetAppend(const Value: boolean);
begin
   if not Value then exit;
   Tab.Append;
   RestoreFields;
end;

procedure TDBTableElement.SetEdit(const Value: boolean);
begin
   if not Value then exit;
   Tab.Edit;
end;

procedure TDBTableElement.SetEmptytable(const Value: boolean);
begin
   if No_BDE  then
      TDbf(Tab).EmptyTable
   else
      TTable(Tab).EmptyTable;   //@@@!
end;

procedure TDBTableElement.SetNext(const Value: boolean);
begin
   if Value then Tab.Next;
   RestoreFields;
end;

procedure TDBTableElement.SetNo_BDE(const Value: boolean);
begin
  FNo_BDE := Value;
end;

procedure TDBTableElement.SetPost(const Value: boolean);
begin
   Tab.Post;
end;

procedure TDBTableElement.SetRecNo(const Value: integer);
begin
   Tab.RecNo := Value;
end;

procedure TDBTableElement.SetStoreFields(const Value: boolean);
var
   i: integer;
   F: TField;
begin
   for i := 0 to Tab.FieldCount - 1 do
   begin
      F := Tab.Fields.Fields[i];
      case F.DataType of
         ftSmallint, ftInteger, ftWord:
         begin
            F.AsInteger := TIntegerElement(Fields.Elements[i]).Value;
         end;
         ftBoolean:
         begin
            F.AsBoolean := TBooleanElement(Fields.Elements[i]).Value;
         end;
         ftFloat:
         begin
            F.AsFloat := TFloatElement(Fields.Elements[i]).Value;
         end;
      else
         begin
            F.AsString := TStringElement(Fields.Elements[i]).Value;
         end;
      end;
   end;
end;

{ TDBFieldsCollection }

function TDBFieldsCollection.GetName: string;
begin
   result := 'FIELDS';
end;

procedure TDBFieldsCollection.SetName(aName: string);
begin

end;

{ TDBFieldDefsCollection }

function TDBFieldDefsCollection.GetName: string;
begin
   result := 'FIELDDEFS';
end;

procedure TDBFieldDefsCollection.SetName(aName: string);
begin
  inherited;

end;

{ TDBFieldDefElement }

procedure TDBFieldDefElement.SetByFieldDef(aFieldDef: TFieldDef);
begin
   DataType := aFieldDef.DataType;
   Precision := aFieldDef.Precision;
   Size := aFieldDef.Size;
   Name := aFieldDef.Name;
end;

procedure TDBFieldDefElement.SetDataType(const Value: TFieldType);
begin
  FDataType := Value;
end;

procedure TDBFieldDefElement.SetPrecision(const Value: integer);
begin
  FPrecision := Value;
end;

procedure TDBFieldDefElement.SetSize(const Value: integer);
begin
  FSize := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TDBTableElement);
   RegisterOrbisClass(TDBFieldsCollection);
   RegisterOrbisClass(TDBFieldDefsCollection);
   RegisterOrbisClass(TDBFieldDefElement);
END.
