unit U_SqlList;

interface

uses
  Classes,
//  Controls,
//  DB,
//  DbTables,
  Elements,
//  E_Type,
//  Registry,
  SysUtils,
  Windows;

const
   cNastaveni = 'Nastaveni';
   cGroupe = 'Groupe';
   cGroups = 'Groups';
   cGroupeName = 'GroupeName';
   cGroupePath = 'GroupePath';

type

   TorbSQLQuery = class(TOwnerEntity)
   private
      FFileName: String;
      FFileSize: Integer;
      FIsValid: Boolean;
      FSQL: TStringList;

//      function GetAlias: TorbAlias;
      function GetSQL: TStringList;
//      procedure SetAlias(const Value: TorbAlias);
      procedure SetFileName(const Value: String);
      procedure SetFileSize(const Value: Integer);
      procedure SetIsValid(const Value: Boolean);
      procedure SetSQL(const Value: TStringList);
   public
      constructor Create; override;
      destructor  Destroy; override;
   protected
   published
//      property Alias: TorbAlias read GetAlias write SetAlias;
      property FileName: String read FFileName write SetFileName;
      property FileSize: Integer read FFileSize write SetFileSize;
      property IsValid: Boolean read FIsValid write SetIsValid;
      property SQL: TStringList read GetSQL write SetSQL;
   end;


   TorbSQLGroupe = class(TOwnerEntity)
   private
      FSqlPath: String;
      FSqlPathExists: Boolean;

      procedure AddSQLFileQuery(aDir: String; aSr: TSearchRec);
      procedure FindSQLFiles(aDir: String);
      procedure SetSQLPath(const Value: String);
      procedure SetSQLPathExists(const Value: Boolean);
   public
      constructor CreateAll(aName: String); virtual;
      destructor  Destroy; override;

      function  AddSQLFileQuery2(aDir, aName: String): TorbSQLQuery;
      function  GetSQLQueries: Boolean;
   protected
   published
      property SQLPath: String read FSQLPath write SetSQLPath;
      property SQLPathExists: Boolean read FSQLPathExists write SetSQLPathExists;
   end;



   TorbSqlList = class(TOwnerEntity)
   private
      FGroups: Integer;

      function  RemoveGroupe(aGrp: TorbSQLGroupe): Boolean;

      procedure GetGroupsFromIni(aName: String);
      procedure DecGroup;
      procedure IncGroup;
      procedure ResetGroup;
      procedure RemoveGroupeFromIni(Alias, aName, aPath: String);
      procedure SetGroups(const Value: Integer);
      procedure WriteGroupeToIni(Alias, aName, aPath: String);
   public
      constructor CreateAll(alName: String); virtual;
      destructor  Destroy; override;

      function  CreateNewSqlGroupe(aName, aPath: String): Boolean;

   protected
   published
      property Groups: Integer read FGroups write SetGroups;
   end;


implementation
uses
   e_Class,
   FileCtrl,
   u_BdeObj,
   u_ini,
   u_string;


{ TorbSqlList }

constructor TorbSqlList.CreateAll(alName: String);
begin
  inherited;
  if alName = '' then exit;
  GetGroupsFromIni(alName);
end;



function TorbSqlList.CreateNewSqlGroupe(aName, aPath: String): Boolean;
var
   Grp: TorbSQLGroupe;
   Al: TorbAlias;
begin
   Result := False;

   Grp := TorbSQLGroupe(FindElementName(aName));
   Al := TorbAlias(FindClassInOwners(TorbAlias));

   if ((Grp = nil) and (Al <> nil)) then
   begin
      Grp := TorbSQLGroupe.Create;

      if (Grp <> nil) then
      begin
         Grp.Name := aName;
         Grp.SQLPath := aPath;
         AddElement(Grp);
         IncGroup;
         WriteGroupeToIni(Al.AliasName, aName, aPath);

         Grp.GetSQLQueries;
         Result := True;
      end;
   end;
end;



procedure TorbSqlList.DecGroup;
begin
   if FGroups > 0 then Dec(FGroups);
end;



destructor TorbSqlList.Destroy;
begin
  inherited;

end;


procedure TorbSqlList.GetGroupsFromIni(aName: String);
var
   i: Integer;
   Grp: TorbSQLGroupe;
   GroupeI, GroupeName, GroupePath, IniFile, IniPath: String;
begin
   exit;
   if aName = '' then exit;

   IniPath :=  ExtractFilePath(ParamStr(0));
   CheckBackSlash(IniPath);

   IniFile := aName;
//   IniFile := ExtractFileName(ParamStr(0));
//   IniFile := ChangeFileExt(IniFile, '.INI');
   IniFile := Concat(IniFile, '.INI');
   IniFile := Concat(IniPath,IniFile);

   Groups := ReadIntegerFromIni(IniFile, cNastaveni, cGroups, 0);

   if Groups = 0  then Exit;

   for i:= 1 to Groups do
   begin
      GroupeI := cGroupe + IntToStr(i);
      GroupeName := ReadStringFromIni(IniFile, GroupeI, cGroupeName, '');
      GroupePath := ReadStringFromIni(IniFile, GroupeI, cGroupePath, '');

      if ((GroupeName = '') or (GroupePath = '')) then continue;

      Grp := TorbSQLGroupe(FindElementName(GroupeName));

      if (Grp = nil) then
      begin
            Grp := TorbSQLGroupe.Create;

            if (Grp <> nil) then
            begin
               Grp.Name := GroupeName;
               Grp.SQLPath := GroupePath;
               AddElement(Grp);

               Grp.GetSQLQueries;
            end;
      end;

   end;
end;



procedure TorbSqlList.IncGroup;
begin
   Inc(FGroups);
end;


procedure TorbSqlList.ResetGroup;
begin
  FGroups := 0;
end;

procedure TorbSqlList.SetGroups(const Value: Integer);
begin
  FGroups := Value;
end;



procedure TorbSqlList.RemoveGroupeFromIni(Alias, aName, aPath: String);
var
   GroupeI, IniFile, IniPath: String;
begin
   if ((aName = '') or (aPath = '')) then exit;

   IniPath :=  ExtractFilePath(ParamStr(0));
   CheckBackSlash(IniPath);

   IniFile := Alias;
   IniFile := Concat(IniFile, '.INI');
   IniFile := Concat(IniPath,IniFile);

   WriteIntegerToIni(IniFile, cNastaveni, cGroups, Groups);
   GroupeI := cGroupe + IntToStr(Groups);

   WriteStringToIni(IniFile, GroupeI, cGroupeName, aName);
   WriteStringToIni(IniFile, GroupeI, cGroupePath, aPath);
end;



function TorbSqlList.RemoveGroupe(aGrp: TorbSQLGroupe): Boolean;
var
   Grp: TorbSQLGroupe;
   Al: TorbAlias;
   aNAme, aPath: String;
begin
   Result := False;

   if (aGrp = nil) then exit;

   Al := TorbAlias(aGrp.FindClassInOwners(TorbAlias));

   if (Al <> nil) then
   begin
      aName := aGrp.Name;
      aPath := aGrp.SQLPath;
      RemoveGroupeFromIni(Al.AliasName, aName, aPath);
      DecGroup;
      aGrp.Free;
      Result := True;
   end;
end;



procedure TorbSqlList.WriteGroupeToIni(Alias, aName, aPath: String);
var
   GroupeI, IniFile, IniPath: String;
begin
   if ((aName = '') or (aPath = '') or (Groups = 0)) then exit;

   IniPath :=  ExtractFilePath(ParamStr(0));
   CheckBackSlash(IniPath);

   IniFile := Alias;
   IniFile := Concat(IniFile, '.INI');
   IniFile := Concat(IniPath,IniFile);

   WriteIntegerToIni(IniFile, cNastaveni, cGroups, Groups);
   GroupeI := cGroupe + IntToStr(Groups);

   WriteStringToIni(IniFile, GroupeI, cGroupeName, aName);
   WriteStringToIni(IniFile, GroupeI, cGroupePath, aPath);
end;

{ TorbSQLGroup }

constructor TorbSQLGroupe.CreateAll(aName: String);
begin

end;

destructor TorbSQLGroupe.Destroy;
begin
  inherited;

end;


function TorbSQLGroupe.GetSQLQueries: Boolean;
begin
   result := False;

   if ((SQLPath = '') or (not DirectoryExists(SQLPath))) then
   begin
      SQLPathExists := False;
      Exit;
   end;

   SQLPathExists := True;
   FindSQLFiles(SQLPath);
end;


procedure TorbSQLGroupe.FindSQLFiles(aDir: String);
var
  sr: TSearchRec;
  FileAttrs: Integer;
  aFile: String;
  SQLQuery: TorbSQLQuery;
begin
   FileAttrs := faAnyFile;

   aFile := ConCat(CheckBackSlash(aDir),'*.SQL');

   if FindFirst(aFile, FileAttrs, sr) = 0 then
   begin
      AddSQLFileQuery(aDir, sr);

      while FindNext(sr) = 0 do AddSQLFileQuery(aDir, sr);

      SysUtils.FindClose(sr);
   end;
end;



procedure TorbSQLGroupe.AddSQLFileQuery(aDir: String; aSr: TSearchRec);
var
  SQLQry: TorbSQLQuery;
  aFile: String;
begin
   SQLQry :=TorbSQLQuery(FindElementName(aSr.Name));

   if SQLQry = nil then
   begin
       aFile := CheckBackSlash(aDir);
       aFile := Concat(aFile, aSr.Name);
       SQLQry := TorbSQLQuery.Create;
       SQLQry.Name := aSr.Name;
       SQLQry.FileSize := aSr.Size;
       SQLQry.SQL.LoadFromFile(aFile);
       AddElement(SQLQry);
   end
   else if SQLQry.FileSize <> asr.Size then
   begin
      // jina velikost souboru znovu nacist
       SQLQry.SQL.LoadFromFile(aSr.Name);
   end;

end;



function TorbSQLGroupe.AddSQLFileQuery2(aDir, aName: String): TorbSQLQuery;
var
  sr: TSearchRec;
  FileAttrs: Integer;
  SQLQry: TorbSQLQuery;
  aFile: String;
begin
   FileAttrs := faAnyFile;
   SQLQry :=TorbSQLQuery(FindElementName(aName));

   aFile := CheckBackSlash(aDir);
   aFile := Concat(aFile, aName);

   if FindFirst(aFile, FileAttrs, sr) = 0 then
   begin
      if SQLQry = nil then
      begin
          SQLQry := TorbSQLQuery.Create;
          SQLQry.Name := aName;
          SQLQry.SQL.LoadFromFile(aFile);

          AddElement(SQLQry);
      end
      else if SQLQry.FileSize <> sr.Size then
      begin
         // jina velikost souboru znovu nacist
          SQLQry.SQL.Clear;
          SQLQry.SQL.LoadFromFile(aFile);
      end;

      SysUtils.FindClose(sr);
   end;

   Result := SQLQry;
end;


procedure TorbSQLGroupe.SetSQLPath(const Value: String);
begin
   FSQLPath := Value;

   SQLPathExists := True;

   if not DirectoryExists(FSQLPath) then
   begin
      if not ForceDirectories(FSQLPath)then  SQLPathExists := False;
   end;
end;



procedure TorbSQLGroupe.SetSQLPathExists(const Value: Boolean);
begin
  FSQLPathExists := Value;
end;





{ TorbSQLQuery }

constructor TorbSQLQuery.Create;
begin
  inherited;
end;


destructor TorbSQLQuery.Destroy;
begin
   inherited;

   if FSQL <> nil then
   begin
      FSQL.Free;
      FSQL := nil;
   end;
end;


{
function TorbSQLQuery.GetAlias: TorbAlias;
begin

end;
}

function TorbSQLQuery.GetSQL: TStringList;
begin
   if FSQL = nil then FSQL := TStringList.Create;

   result := FSQL;
end;

{
procedure TorbSQLQuery.SetAlias(const Value: TorbAlias);
begin

end;
}


procedure TorbSQLQuery.SetFileName(const Value: String);
begin
  FFileName := Value;
end;



procedure TorbSQLQuery.SetFileSize(const Value: Integer);
begin
  FFileSize := Value;
end;



procedure TorbSQLQuery.SetIsValid(const Value: Boolean);
begin
  FIsValid := Value;
end;



procedure TorbSQLQuery.SetSQL(const Value: TStringList);
begin
   if FSQL = nil then FSQL := TStringList.Create;

   if Value = nil then exit;

   FSQL.Assign(Value);
end;



INITIALIZATION
   RegisterOrbisClass(TorbSqlList);
   RegisterOrbisClass(TorbSqlGroupe);
   RegisterOrbisClass(TorbSqlQuery);
END.
