(*----------------------------------------------------------------------+
|									|
|   mselems.h -- This Unit contains the structures that define		|
|		  MicroStation elements.				|
|									|
+----------------------------------------------------------------------*)
Unit MSElems;

INTERFACE

uses
   Typky;


(* Definitions specific to element conversion processing *)
const
   FIXEDDEPTH  = 0;	    (* Fixed Z depth *)
   ELEMHIGH    = 1;	    (* High range value of Z range *)
   ELEMLOW     = 2;	    (* Low range value of Z range *)

(*----------------------------------------------------------------------+
|							                |
|   Type	Description						|
|    1		Cell Library Header					|
|    2		Cell (complex)						|
|    3 		Line						        |
|    4		Line String						|
|    5 		Group Data					        |
|    6		Shape							|
|    7		Text Node (complex)					|
|    8 		Digitizer Setup Data				        |
|    9		Design File Header					|
|    10		Level Symbology					        |
|    11		Curve						        |
|    12		Complex String (complex)			        |
|    13	        Conic						        |
|    14		Complex Shape (complex)				        |
|    15		Ellipse						        |
|    16		Arc						        |
|    17		Text						        |
|    18		Surface (complex)				        |
|    19		Solid (complex)					        |
|    20		not used					        |
|    21		B-Spline Pole					        |
|    22		Point String					        |
|    23		Circular Truncated Cone				        |
|    24		B-Spline Surface (complex)			        |
|    25		B-Spline Surface boundary			        |
|    26		B-Spline Knot Element				        |
|    27		B-Spline Curve (complex)			        |
|    28		B-Spline Weight Factor				        |
|    33         Dimension Element		    			|
|    34		Shared Cell Definition Element			        |
|    35		Shared Cell Element				        |
|    36         Multiline Element	    		    	    	|
|    58         MicroStation Text Array Element			        |
|    59         MicroStation Text Stack Element			        |
|    66		MicroStation Application			        |
|    87		Raster Header					        |
|    88		Raster Component				        |
|									|
+----------------------------------------------------------------------*)

  CELL_LIB_ELM	      =  1 shl 8;
  CELL_HEADER_ELM     =  2 shl 8;
  LINE_ELM            =  3 shl 8;
  LINE_STRING_ELM     =  4 shl 8;
  GROUP_DATA_ELM      =  5 shl 8;
  SHAPE_ELM	      =  6 shl 8;
  TEXT_NODE_ELM	      =  7 shl 8;
  DIG_SETDATA_ELM     =  8 shl 8;
  DGNFIL_HEADER_ELM   =  9 shl 8;
  LEV_SYM_ELM	      = 10 shl 8;
  CURVE_ELM	      = 11 shl 8;
  CMPLX_STRING_ELM    = 12 shl 8;
  CONIC_ELM	      = 13 shl 8;
  CMPLX_SHAPE_ELM     = 14 shl 8;
  ELLIPSE_ELM	      = 15 shl 8;
  ARC_ELM	      = 16 shl 8;
  TEXT_ELM	      = 17 shl 8;
  SURFACE_ELM	      = 18 shl 8;
  SOLID_ELM	      = 19 shl 8;
  BSPLINE_POLE_ELM    = 21 shl 8;
  POINT_STRING_ELM    = 22 shl 8;
  CONE_ELM	      = 23 shl 8;
  BSPLINE_SURFACE_ELM = 24 shl 8;
  BSURF_BOUNDARY_ELM  = 25 shl 8;
  BSPLINE_KNOT_ELM    = 26 shl 8;
  BSPLINE_CURVE_ELM   = 27 shl 8;
  BSPLINE_WEIGHT_ELM  = 28 shl 8;
  DIMENSION_ELM	      = 33 shl 8;
  SHAREDCELL_DEF_ELM  = 34 shl 8;
  SHARED_CELL_ELM     = 35 shl 8;
   MULTILINE_ELM      = 36 shl 8;
  TEXT_ARRAY_ELM      = 58 shl 8;
  TEXT_STACK_ELM      = 59 shl 8;
  MICROSTATION_ELM    = 66 shl 8;
  RASTER_HDR	      = 87 shl 8;
  RASTER_COMP	      = 88 shl 8;


(*----------------------------------------------------------------------+
|									|
|	Level definitions						|
|									|
+----------------------------------------------------------------------*)
   MSDIM_LEVEL		   =  1;
   MSREF_LEVEL		   =  5;
   MSLEVELNAME_LEVEL	   =  6;
   MSDIG_LEVEL		   =  8;
   MSTCB_LEVEL		   =  9;
   MSVIEW_LEVEL	           =  7;
   MSSTARTAPP_LEVEL	   = 10;

   (* NOTE: database control elements have levels 11-19 reserved *)
   MSDB_LEVEL		   = 10;	(* dBASE	    *)
   MSIX_LEVEL		   = 11;	(* Informix	    *)
   MSRIS_LEVEL		   = 12;	(* RIS		    *)
   MSORACLE_LEVEL	   = 13;	(* Oracle	    *)

   MSAPPINFO_LEVEL	   = 20;
   MSSTARTMDL_LEVEL	   = 21;
   MSMLINE_LEVEL	   = 22;

(*----------------------------------------------------------------------+
|                                                                       |
|	Class definitions						                                    |
|									                                             |
+----------------------------------------------------------------------*)
   PRIMARY_CLASS		= 0;
   PATTERN_COMPONENT_CLASS	= 1;
   CONSTRUCTION_CLASS		= 2;
   DIMENSION_CLASS		= 3;
   PRIMARY_RULE_CLASS		= 4;
   LINEAR_PATTERNED_CLASS	= 5;
   CONSTRUCTION_RULE_CLASS	= 6;

(*---------------------------------------------------------------------+
|								       |
|   Element Header structure - common to all MicroStation elements     |
|								       |
+----------------------------------------------------------------------*)

{ masks of Header }

  mskDeleted = $8000;
  mskType    = $7F00;
  mskComplex = $0080;
  mskLevel   = $003F;
  mskExtLevel= $007F;   { for extended levels ... 7 bits }

     (*
  ^  |unsigned short    level:6; /* level element is on */
  |  |unsigned short  igresvd:1; /* reserved by Intergraph */
  |  |unsigned short  complex:1; /* part of complex element (cell) if set */
  +--|unsigned short     type:7; /* type of element */
     |unsigned short  deleted:1; /* set if element is deleted */
     *)
type

{ typedef struct elm_hdr }
  Elm_hdr = packed record

     Header : WORD;
     Words  : WORD;	  { number of words to follow in element }
     xlow   : LONGINT;    { range of element - low (reversed longs) }
     ylow   : LONGINT;
     zlow   : LONGINT;
     xhigh  : LONGINT;	  { range of element - high (reversed longs) }
     yhigh  : LONGINT;
     zhigh  : LONGINT;
  end;

(*----------------------------------------------------------------------+
|    									                                          |
|   Display header - common to all displayed MicroStation elements	   |
|									                                             |
+----------------------------------------------------------------------*)

const

  { Props masks }

  mskPropsClass    = $000F;
  mskPropsRes      = $00F0;
  mskPropsL        = $0100;
  mskPropsN        = $0200;
  mskPropsM        = $0400;
  mskPropsA        = $0800;
  mskPropsR        = $1000;
  mskPropsP        = $2000;
  mskPropsS        = $4000;
  mskPropsH        = $8000;

  { Symb Masks }

  mskSymbStyle     = $0007;
  mskSymbWeight    = $00F8;
  mskSymbColor     = $FF00;

type


  Disp_hdr = packed record
    grphgrp: Word;	(* graphics group number *)
    attindx: Word;	(* words between this and attribute linkage *)
    Props  : record
                aClass: byte; (* only low four bites *)
                bits : set of(L,N,M,A,R,P,S,H);
             end;
    Symb   : Word;
  end;


  Header = packed record
    ehdr: Elm_hdr;
    dhdr: Disp_hdr;
  end;

(*----------------------------------------------------------------------+
|									|
|   Cell Library Header Element (type 1, only in cell libraries)	|
|									|
+----------------------------------------------------------------------*)
  TCell_lib_hdr = packed record
    celltype   : Word;	              (* cell type *)
    attindx    : Word;	              (* attribute linkage *)
    name       : LongInt;             (* radix-50 cell name *)
    numwords   : Word;	              (* number of words in descriptions *)
    properties : Word;	              (* properties *)
    dispsymb   : Word;	              (* display symbology *)
    aClass      : Word;	              (* cell class (always 0) *)
    levels     : Array[1..4] of Word; (* levels used in cell *)
    descrip    : Array[1..9] of Word; (* cell description *)
  end;


(*----------------------------------------------------------------------+
|    									|
|   2-D Cell Element (complex element, type 2)				|
|									|
+----------------------------------------------------------------------*)
  TCellName = array [0..1] of word;

  Tcell_2d = packed record
    dhdr      : Disp_hdr;	    (* display header *)
    totlength : word;               (* total length of cell *)
    name      : TCellName;          (* radix 50 name *)
    aClass     : word;	            (* class bit map *)
    levels    : array[1..4]of word; (* levels used in cell *)
    rnglow    : Point2d;	    (* range block lo *)
    rnghigh   : Point2d;	    (* range block hi *)
    trans     : LTrans2d;	    (* transformation matrix *)
    origin    : Point2D;	    (* cell origin *)
  end; {TCell_2d}

(*----------------------------------------------------------------------+
|    									|
|   3-D Cell Element (complex element, type 2)				|
|									|
+----------------------------------------------------------------------*)
  Tcell_3d = packed record
    dhdr      : Disp_hdr;	    (* display header *)
    totlength : word;               (* total length of cell *)
    name      : TCellName;	    (* radix 50 name *)
    aClass     : word;	            (* class bit map *)
    levels    : array[1..4]of word; (* levels used in cell *)
    rnglow    : Point3d;	    (* range block lo *)
    rnghigh   : Point3d;	    (* range block hi *)
    trans     : LTrans3d;	    (* transformation matrix *)
    origin    : Point3D;	    (* cell origin *)
  end; {TCell_3d}


(*----------------------------------------------------------------------+
|									|
|   2-D Line Element (type 3)						|
|									|
+----------------------------------------------------------------------*)
  TLine_2d = packed record
    dhdr     : Disp_hdr ;   (* display header *)
    StartP   : Point2D;	    (* starting point *)
    EndP     : Point2d;	    (* ending point *)
  end;

(*----------------------------------------------------------------------+
|									|
|   3-D Line Element (type 3)						|
|									|
+----------------------------------------------------------------------*)
  TLine_3d = packed record
    dhdr     : Disp_hdr;    (* display header *)
    StartP   : Point3D;	    (* starting point *)
    EndP     : Point3d;	    (* ending point *)
  end;


(*----------------------------------------------------------------------+
|									|
|   2-D Line String Element (type 4)					|
|									|
|   Same structure represents 2-D Shape Elements (type 6), 2-D Curve	|
|   Elements (type 11), and 2-D Conic Elements (type 13).  Shapes differ|
|   from line strings in that shape elements always start and end at	|
|   the same point.  Curve and conic elements use the first and last    |
|   vertices to establish the slope at the beginning and end.		|
|									|
+----------------------------------------------------------------------*)
  Tline_string_2d = packed record
    dhdr     : Disp_hdr;        (* display header *)
    numverts : word;	        (* number of vertices *)
    vertice  : Point2d;	        (* points *)
  end;

(*----------------------------------------------------------------------+
|									|
|   3-D Line String Element (type 4)					|
|									|
|   See comments for 2-D Line String Element.				|
|									|
+----------------------------------------------------------------------*)
  Tline_string_3d = packed record
    dhdr     : Disp_hdr;        (* display header *)
    numverts : word;	        (* number of vertices *)
    vertice  : Point3d;	        (* points *)
  end;

(*----------------------------------------------------------------------+
|									|
|   Reference File Attachment Element (type 5, level 9)			|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct ref_file_type5}
    {
    Elm_hdr	ehdr;		         (* element header *)
    Disp_hdr	dhdr;		      (* display header *)
    short	file_chars;	      (* number of characters in file spec *)
    char	file_spec[65];	      (* file specification *)
    byte	file_num;	         (* file number *)
    Fb_opts	fb_opts;	         (* file builder options mask *)
    Fd_opts	fd_opts;	         (* file displayer options mask *)
    byte	disp_flags[16];	   (* display flags *)
    short	lev_flags[8][4];  (* level on/of flags *)
    long	ref_org[3];	         (* origin in reference file uors *)
    double	trns_mtrx[9];	   (* transformation matrix *)
    double	cnvrs_fact;	      (* conversion factor *)
    long	mast_org[3];	      (* origin in master file uors *)
    short	log_chars;	      (* characters in logical name *)
    char	log_name[22];	      (* logical name (padded) *)
    short	desc_chars;	      (* characters in description *)
    char	description[42];     (* description (padded) *)
    short	lev_sym_mask;	   (* level symbology enable mask *)
    short	lev_sym[63];	   (* level symbology descriptor *)
    long	z_delta;	            (* Z-direction delta *)
    short	clip_vertices;	   (* clipping vertices *)
    Point2d	clip_poly[1];	   (* clipping polygon *)
    } {Ref_file_type5;}

(*----------------------------------------------------------------------+
|									|
|   Named view element (type 5, level 3)				|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct named_view_type5}
    {
    Elm_hdr	ehdr;		    (* element header *)
    short	grphgrp;	    (* graphics group number *)
    short	attindx;	    (* words between this and attrb linkage *)
    short	properties;	 (* property bits (always the same) *)
    unsigned	short num_views:3;  (* number of views *)
    unsigned	short reserved:13;  (* reserved for Intergraph *)
    char	viewdef_descr[18];        (* view definition description *)
    byte	full_scr1;
    byte	full_scr2;
    Viewinfo	view[1];
    char	rest_of_elem[1];          (* record has variable length *)
    } {Named_view_type5;}

(*----------------------------------------------------------------------+
|									|
|   Color table element (type 5, level 1)				|
|									|
+----------------------------------------------------------------------*)
  TColordef = packed record
     R,G,B: byte;
  end;

  TColorTable = packed array[0..255]of TColorDef;

  Tcolor_table_type5 = packed record
    dhdr        : Disp_hdr;    (* display header *)
    screen_flag : word;	       (* screen flag *)
    color_info  : TColorTable; (* color table information *)
  end; {TColor_table_type5}

(*----------------------------------------------------------------------+
|									|
|   short pattern (type 5, level 0)					|
|									|
+----------------------------------------------------------------------*)
  Tshort_pattern_type5 = record
    dhdr           : Disp_hdr;    (* display header *)
    indicator      : word;
    name           : longint;
    scale          : double;
    angle          : double;
    row_spacing    : longint;
    column_spacing : longint;
  end; {TShort_pattern_type5}

(*----------------------------------------------------------------------+
|									|
|   long pattern (type 5, level 0)					|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct long_pattern_type5
    {
    Elm_hdr	    ehdr;		      (* element header *)
    Disp_hdr	    dhdr;		   (* display header *)
    short	    indicator;
    long	    name;
    double	    scale;
    double	    angle;
    long	    row_spacing;
    long	    column_spacing;
    Seven_wd_lknm   library;		(* file spec *)
    Symbology	    symbology;
    short	    fbfdcn;
    byte	    master_type;
    byte	    request_type;
    Point2d	    idpoint;
    Point2d	    accpoint;
    }{ Long_pattern_type5;}

(*----------------------------------------------------------------------+
|									|
|   Auxilliary Coordinate System (type 5, level 3)			|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct aux_coordinate
    {
    Elm_hdr	ehdr;		    (* element hdr (origin stored in range) *)
    short	grphgrp;	    (* graphics group number *)
    short	attindx;	    (* words between this and attrb linkage *)
    short	properties;	    (* property bits (always the same) *)
#if !defined (mc68000)
    unsigned	short type:3;	    (* number of views *)
    unsigned	short reserved:13;  (* reserved for Intergraph *)
#else
    unsigned	short reserved:13;  (* reserved for Intergraph *)
    unsigned	short type:3;	    (* number of views *)
#endif
    char	description[18];    (* definition description *)
    double	trans[3][3];
    short	attribute;	    (* 0001000000000011 *)
    short	user_id;	    (* always 22 *)
    long	name;		    (* 6 characters stored RAD50 *)
    } {Aux_coordinate;}

(*----------------------------------------------------------------------+
|									|
|   2-D Text Node Element (complex element, type 7)			|
|									|
+----------------------------------------------------------------------*)
Ttext_node_2d = packed record
    dhdr: Disp_hdr;	  (* display header *)
    totwords: word;	  (* total words following (cmplx ele) *)
    numstrngs: word;	  (* number of text strings in node *)
    nodenumber: word;     (* text node number *)
    maxlngth: byte;	  (* maximum length allowed *)
    maxused: byte;	  (* maximum length used *)
    font: byte;	          (* text font used *)
    just: byte;	          (* justification type *)
    linespc: longInt;	  (* line spacing *)
    lngthmult: longInt;	  (* length multiplier *)
    hghtmult: longInt;	  (* height multiplier *)
    rotation: longInt;	  (* rotation angle *)
    origin: Point2d;	  (* origin *)
end;
(*----------------------------------------------------------------------+
|									|
|   3-D Text Node Element (complex element, type 7)			|
|									|
+----------------------------------------------------------------------*)
(* Text Node Element (note: this is a complex element) *)
{ typedef  struct text_node_3d
    {
    Elm_hdr	    ehdr;	        (* element header *)
    Disp_hdr	    dhdr;	     (* display header *)
    unsigned short  totwords;	  (* total words following (cmplx ele) *)
    short	    numstrngs;	     (* number of text strings in node *)
    unsigned short  nodenumber; (* text node number *)
    byte	    maxlngth;	        (* maximum length allowed *)
    byte	    maxused;	        (* maximum length used *)
    byte	    font;	           (* text font used *)
    byte	    just;	           (* justification type *)
    long	    linespc;	        (* line spacing *)
    long	    lngthmult;	        (* length multiplier *)
    long	    hghtmult;	        (* height multiplier *)
    long	    quat[4];	        (* quaternion rotations *)
    Point3d	    origin;	        (* origin *)
    } {Text_node_3d;}

(*----------------------------------------------------------------------+
|    									|
|   Design file header (type 9) 					|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct dgn_header
    {
    Elm_hdr	    ehdr;	    (* element header *)
    short	    tcbinfo[1];     (* tcb information *)
    } {Dgn_header;}

(*----------------------------------------------------------------------+
|									|
|    Complex string (type 12, complex)					|
|									|
+----------------------------------------------------------------------*)
  Tcomplex_string = packed record
    dhdr      : Disp_hdr;            (* display header *)
    totlength : word;                (* total length of surface *)
    numelems  : word;	             (* number of elements in surface *)
    attributes: array[1..4]of word;  (* to reach minimum elem size *)
  end; {TComplex_string}


(*----------------------------------------------------------------------+
|    									|
|   2-D Ellipse Element (type 15)					|
|									|
+----------------------------------------------------------------------*)
  Tellipse_2d = packed record
    dhdr      : Disp_hdr;   (* display header *)
    primary   : double;	    (* primary axis *)
    secondary : double;	    (* secondary axis *)
    rotation  : longint;    (* rotation angle *)
    origin    : Dpoint2d;   (* origin *)
  end; {TEllipse_2d}

(*----------------------------------------------------------------------+
|    									|
|   3-D Ellipse Element (type 15)					|
|									|
+----------------------------------------------------------------------*)
  Tellipse_3d = packed record
    dhdr      : Disp_hdr;               (* display header *)
    primary   : double;	                (* primary axis *)
    secondary : double;	                (* secondary axis *)
    quat      : array[1..4]of longint;  (* quaternion rotations *)
    origin    : Dpoint3d;               (* origin *)
  end;   {Ellipse_3d}


(*----------------------------------------------------------------------+
|									|
|   2-D Arc Element (type 16)						|
|									|
+----------------------------------------------------------------------*)
  Tarc_2d = packed record
    dhdr      : Disp_hdr;   (* display header *)
    startang  : longint;    (* start angle *)
    sweepang  : longint;    (* sweep angle *)
    primary   : double;	    (* primary axis *)
    secondary : double;	    (* secondary axis *)
    rotation  : longint;    (* rotation angle *)
    origin    : Dpoint2d;   (* origin *)
  end; {TArc_2d}

(*----------------------------------------------------------------------+
|									|
|   3-D Arc Element (type 16)						|
|									|
+----------------------------------------------------------------------*)
  Tarc_3d = packed record
    dhdr      : Disp_hdr;   (* display header *)
    startang  : longint;    (* start angle *)
    sweepang  : longint;    (* sweep angle *)
    primary   : double;	    (* primary axis *)
    secondary : double;	    (* secondary axis *)
    quat      : packed array[1..4]of longint;  (* quaternion rotations *)
    origin    : Dpoint3d;               (* origin *)
  end;

(*----------------------------------------------------------------------+
|									|
|   2-D Text Element (type 17)						|
|									|
+----------------------------------------------------------------------*)
  Ttext_2d = packed record
    dhdr     :  Disp_hdr; (* display header *)
    font     :  Byte;	  (* text font used *)
    just     :  Byte;	  (* justification type *)
    lngthmult:  LongInt;  (* length multiplier *)
    hghtmult :  LongInt;  (* height multiplier *)
    rotation :  LongInt;  (* rotation angle *)
    origin   :  Point2d;  (* origin *)
    numchars :  Byte;	  (* # of characters *)
    edflds   :  Byte;	  (* # of enter data fields *)
    Pis      :  Char;	  (* characters *)
  end;

(*----------------------------------------------------------------------+
|									|
|   3-D Text Element (type 17)						|
|									|
+----------------------------------------------------------------------*)
  Ttext_3d = packed record
    dhdr     :  Disp_hdr;               (* display header *)
    font     :  Byte;	                (* text font used *)
    just     :  Byte;	                (* justification type *)
    lngthmult:  LongInt;                (* length multiplier *)
    hghtmult :  LongInt;                (* height multiplier *)
    rotation :  array[1..4]of LongInt;  (* rotation angle *)
    origin   :  Point3d;                (* origin *)
    numchars :  Byte;	                (* # of characters *)
    edflds   :  Byte;	                (* # of enter data fields *)
    Pis      :  Char;	                (* characters *)
  end;


(*----------------------------------------------------------------------+
|									|
|    Surface (type 18, complex)	    3d only				|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct surface
    {
    Elm_hdr	    ehdr;	       (* element header *)
    Disp_hdr	    dhdr;	    (* display header *)
    unsigned short  totlength; (* total length of surface *)
    unsigned short  numelems;	 (* number of elements in surface *)
    byte	    surftype;	       (* surface type *)
    byte	    boundelms;	       (* number of boundary elements *)
#ifdef unix
    short	    filler;	       (* needed for proper alignment *)
#endif
    short	    attributes[4]; (* attribute data *)
    } {Surface;}

(*----------------------------------------------------------------------+
|									|
|   2-D  B-Spline pole element (type 21)				|
|									|
+----------------------------------------------------------------------*)
  Tbspline_poles_2d = packed record
    dhdr     :  Disp_hdr;(* display header *)
    numpoles :  word;	 (* number of poles *)
    poles    :  Point2D; (* points *)
  end; {TBspline_pole_2d}


(*----------------------------------------------------------------------+
|									|
|   3-D  B-Spline pole element (type 21)				|
|									|
+----------------------------------------------------------------------*)
  Tbspline_poles_3d = packed record
    dhdr     :  Disp_hdr;(* display header *)
    numpoles :  word;	 (* number of poles *)
    poles    :  Point3D; (* points *)
  end; {TBspline_pole_3d}


(*----------------------------------------------------------------------+
|									|
|   Application startup element (type 21) level 10                      |
|									|
+----------------------------------------------------------------------*)
  TStartApp = packed record
    dhdr     :  Disp_hdr;                (* display header *)
    Command  :  Array[1..255] of char;	 (* App command line *)
  end; {TStartApp}


(*----------------------------------------------------------------------+
|									|
|   Circular truncated cone  (type 23)    3-D files only		|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct cone_3d
    {
    Elm_hdr	       ehdr;	    (* element header *)
    Disp_hdr	    dhdr;	    (* display header *)
    struct
	{
#if !defined (mc68000)
	unsigned short	type:3;	    (* cone type *)
	unsigned short	rsrv:12;     (* reserved *)
	unsigned short	surf:1;	    (* 0 = solid, 1 = surface *)
#else
	unsigned short	surf:1;	    (* 0 = solid, 1 = surface *)
	unsigned short	rsrv:12;     (* reserved *)
	unsigned short	type:3;	    (* cone type *)
#endif
	}{ b;
    long	          quat[4];	    (* orientation quaternion *)
    Dpoint3d	    center_1;	    (* center of first circle *)
    double	       radius_1;	    (* radius of first circle *)
    Dpoint3d	    center_2;	    (* center of second circle *)
    double	       radius_2;	    (* radius of second cirele *)
    } {Cone_3d;}

(*----------------------------------------------------------------------+
|									|
|   Bspline flags							|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct	bspline_flags
    {
#if !defined (mc68000)
    unsigned short  order:4;		     (* order - 2*)
    unsigned short  curve_display:1;  (* curve display flag *)
    unsigned short  poly_display:1;	  (* polygon display flag *)
    unsigned short  rational:1;		  (* rationalization flag *)
    unsigned short  closed:1;		     (* closed curve flag *)
    unsigned short  curve_type:8;	  (* curve type *)
#else
    unsigned short  curve_type:8;	  (* curve type *)
    unsigned short  closed:1;		     (* closed curve flag *)
    unsigned short  rational:1;		  (* rationalization flag *)
    unsigned short  poly_display:1;	  (* polygon display flag *)
    unsigned short  curve_display:1;  (* curve display flag *)
    unsigned short  order:4;		     (* order - 2 *)
#endif
    } {Bspline_flags;}

(*----------------------------------------------------------------------+
|									|
|   Bspline surface flags						|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct bsurf_flags
    {
#ifndef mc68000
    unsigned short v_order:4;		   (* b-spline order - 2 (v Direction)*)
    unsigned short reserved1:2;		(* reserved *)
    unsigned short arcSpacing:1;	   (* rule lines spaced by arc length *)
    unsigned short v_closed:1;		(* closed curve flag *)
    unsigned short reserved2:8;		(* reserved *)
#else
    unsigned short reserved2:8;		(* reserved *)
    unsigned short v_closed:1;		(* closed curve flag *)
    unsigned short arcSpacing:1;	   (* rule lines spaced by arc length *)
    unsigned short reserved1:2;		(* reserved *)
    unsigned short v_order:4;		   (* b-spline order (v Direction)*)
#endif
    } {Bsurf_flags;}

(*----------------------------------------------------------------------+
|									|
|   Bspline surface header	(type 24)				|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct bspline_surface
    {
    Elm_hdr	      ehdr;	          (* element header *)
    Disp_hdr	   dhdr;	          (* display header *)
    long	         desc_words;	    (* # words in description *)
    Bspline_flags flags;	       (* miscellaneous flags *)
    short	      num_poles_u;    (* number of poles *)
    short	      num_knots_u;    (* number of knots *)
    short	      rule_lines_u;   (* number of rule lines *)
    Bsurf_flags	bsurf_flags;    (* bspline surface flags *)
    short	      num_poles_v;    (* number of poles *)
    short	      num_knots_v;    (* number of knots *)
    short	      rule_lines_v;   (* number of rule lines *)
    short	      num_bounds;	    (* number of boundaries *)
    } {Bspline_surface;}

(*----------------------------------------------------------------------+
|									|
|   B-spline Surface boundary (type 25)					|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct bsurf_boundary
    {
    Elm_hdr	    ehdr;	     (* element header *)
    Disp_hdr	 dhdr;	     (* display header *)
    short	    number;	     (* boundary number *)
    short	    numverts;	  (* number of boundary vertices *)
    Point2d	    vertices[1]; (* boundary points (in UV space) *)
    } {Bsurf_boundary;}

(*----------------------------------------------------------------------+
|									|
|   B-spline weight	factor (type 26)				|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct bspline_weight
    {
    Elm_hdr	    ehdr;	    (* element header *)
    Disp_hdr	 dhdr;	    (* display header *)
    long	       weights[1]; (* bspline weights (variable length) *)
    } {Bspline_weight;}

(*----------------------------------------------------------------------+
|									|
|   B-spline curve header  (type 27)					|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct bpsline_curve
    {
    Elm_hdr	    ehdr;	       (* element header *)
    Disp_hdr	    dhdr;	    (* display header *)
    long	    desc_words;	    (* # words in description *)
    Bspline_flags   flags;	    (* miscellaneous flags *)
    short	    num_poles;	    (* number of poles *)
    short	    num_knots;	    (* number of knots *)
    } {Bspline_curve;}

(*----------------------------------------------------------------------+
|									|
|   B-spline knot vector (type 28)					|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct bspline_knot
    {
    Elm_hdr	    ehdr;	    (* element header *)
    Disp_hdr	 dhdr;	    (* display header *)
    long	       knots[1];	 (* knots (variable length) *)
    } {Bspline_knot;}

(*----------------------------------------------------------------------+
|									|
|   Shared Cell Instance (type 35) - complete definition unpublished    |
|									|
+----------------------------------------------------------------------*)
{ typedef  struct scOverride
    {
    unsigned short  level:1;	    (* level override (true for pnt cells)  *)
    unsigned short  relative:1;	    (* relative or absolute		    *)
    unsigned short  class:1;	    (* class value *)
    unsigned short  color:1;	    (* color of all components *)
    unsigned short  weight:1;	    (* weight of all components *)
    unsigned short  style:1;	    (* style of all components *)
    unsigned short  assocPnt:1;	    (* origin of cell is associative point *)
    unsigned short  unused:9;
    } {SCOverride;}

(*----------------------------------------------------------------------+
|									|
| name		element_union - union of all element types		|
|									|
+----------------------------------------------------------------------*)
{ typedef  union msElementUnion
    {
    Cell_Lib_Hdr    cell_lib_hdr;
    Cell_2d         cell_2d;
    Cell_3d	        cell_3d;
    Line_2d	        line_2d;
    Line_3d	        line_3d;
    Line_String_2d  line_string_2d;
    Line_String_3d  line_string_3d;
    Text_node_2d    text_node_2d;
    Text_node_3d    text_node_3d;
    Complex_string  complex_string;
    Ellipse_2d	     ellipse_2d;
    Ellipse_3d	     ellipse_3d;
    Arc_2d	        arc_2d;
    Arc_3d	        arc_3d;
    Text_2d	        text_2d;
    Text_3d	        text_3d;
    Cone_3d	        cone_3d;
    Surface	        surf;
    Bspline_pole_2d bspline_pole_2d;
    Bspline_pole_3d bspline_pole_3d;
    Bspline_curve   bspline_curve;
    Bspline_surface bspline_surface;
    Bspline_weight  bspline_weight;
    Bspline_knot    bspline_knot;
    Bsurf_boundary  bsurf_boundary;
    short	        tmp[768];
#if defined (ip32)
    short	        buf[780];
#else
    short	        buf[780];
#endif
    Extlev_hdr	     extlevhdr;
    Elm_hdr	        ehdr;
    Header	        hdr;
    } {MSElementUnion, MSElement;}

(*----------------------------------------------------------------------+
|									|
|   Element Descriptor structure					|
|									|
+----------------------------------------------------------------------*)
{ typedef  struct msElementDescr MSElementDescr;
struct msElementDescr
    {
    struct
	{
	MSElementDescr	*next;		(* ptr to first entry in list *)
	MSElementDescr	*previous;	(* ptr to last entry in list *)
	MSElementDescr	*myHeader;	(* ptr to my hdr (NULL = not cmplx) *)
	MSElementDescr	*firstElem;	(* ptr to first element if header *)
	int		isHeader;	(* is this a complex header *)
	int		isValid;	(* INTERNAL USE ONLY *)
	long		userData1;	(* available for user *)
	long		userData2;	(* available for user *)
	} {h;
    MSElement     	el;		(* elem data (hdr only if complex) *)
    ;}

{#endif	(* __mselemsH__ *)}

IMPLEMENTATION

end.