unit t_DxfImp;

interface
uses
   Elements,
   Tools;
type

TDxfImport = class(TTool)
private
   F: Text;
   FSize: integer;
   FPosition: integer;
   dSize: integer;
   FLastPos: integer;
   FFileName: string;
   FEOF: boolean;
   FGroup: integer;
   FValue: string;
    FNextEntityReady: boolean;
    FLines: TEntity;
    FBlocks: TEntity;
    FEntities: TEntity;
    FActualEntity: TEntity;
   procedure SetFileName(const Value: string);
    procedure SetNextEntityReady(const Value: boolean);
    procedure SetLines(const Value: TEntity);
    function GetBlocks: TEntity;
    function GetEntities: TEntity;
    procedure SetActualEntity(const Value: TEntity);
public
   procedure DoIt; override;
   function  ReadGroupValue: boolean;
   function  EndOfSection: boolean;
   function  EndOfTable: boolean;
   procedure ReadSection;
   procedure ReadTABLES;
   procedure ReadTable;
   procedure ReadLTYPE;
   procedure ReadLAYER;
   procedure ReadBlocks;
   procedure ReadBlock;
   procedure ReadENTITIES;
   procedure ReadEntity;
   procedure ReadArc;
   procedure ReadCircle;
   procedure ReadLine;
   procedure ReadPolyLine;
   procedure ReadVertex(var aX,aY: double);
   procedure ReadInsert;
   property  NextEntityReady: boolean read FNextEntityReady write SetNextEntityReady;
   property  Blocks: TEntity read GetBlocks;
   property  Entities: TEntity read GetEntities;
   property  ActualEntity: TEntity read FActualEntity write SetActualEntity;
published
   property FileName: string read FFileName write SetFileName;
   property Lines: TEntity read FLines write SetLines stored false;
end;

implementation
uses
   Forms,
   u_Math,
   SysUtils,
   Dialogs,
   MathLib,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   e_TopolElements,
   e_Files,
   g_Elems,
   m_Orbis,
   u_String;

{ TDxfImport }

procedure TDxfImport.DoIt;
var
   FE: TFileElement;
begin
   if not FileExists(FileName) then
   begin
      ShowMessage('Soubor "'+FileName+'" neexistuje');
      exit;
   end;
   if Lines = nil then
   begin
      ShowMessage('LINES nen definovno!');
      exit;
   end;
   FSize := 0;
   FPosition := 0;
   FLastPos := 0;
   FE := TFileElement.Create;
   FE.FileName := FileName;
   FE.ReadInfo := true;
   FSize := FE.FileSize;
   FE.Free;
   StartProgress('DXF Import: '+FileName,0,FSize);
   System.Assign(F, FileName);
   System.Reset(F);
   DSize := FSize div 50;
   FEOF := EOF(F);

   while not FEOF do
   begin
      if not ReadGroupValue then break;
      if (FGroup = 0) and (FValue = 'SECTION') then
         ReadSection;
   end;
   System.Close(F);
end;

function TDxfImport.EndOfSection: boolean;
begin
   result := FEOF or ((FGroup = 0) and (FValue='ENDSEC'))
end;

function TDxfImport.EndOfTable: boolean;
begin
   result := FEOF or ((FGroup = 0) and (FValue='ENDTAB'))
end;

function TDxfImport.GetBlocks: TEntity;
begin
   if FBlocks = nil then
   begin
      FBlocks := TOwnerEntity.Create;
      AddElement(FBlocks);
      FBlocks.Name := 'BLOCKS';
   end;
   result := FBlocks;
end;

function TDxfImport.GetEntities: TEntity;
begin
   if FEntities = nil then
   begin
      FEntities := TOwnerEntity.Create;
      AddElement(FEntities);
      FEntities.Name := 'ENTITIES';
   end;
   result := FEntities;
end;

procedure TDxfImport.ReadArc;
var
   H: string;
   aX: double;
   aY: double;
   R: double;
   U1: double;
   U2: double;
   D: double;
   TL: TTopolLineElement;
   L: TPolyLineElement;
   Ro,dU,Alfa: double;
   X1,Y1: double;
  procedure NacpiUhelDoDvaPi (var Uhel: double);
  //var
     //DvaPI: double;
  begin
     //DvaPI := 2*PI;
     if Uhel < 0 then Uhel := DvaPi + Uhel else
     if Uhel > DvaPi then Uhel := Uhel-(Int(Uhel/DvaPi)*DvaPi);
  end;
begin
   aX := 0;
   aY := 0;
   R  := 0;
   U1 := 0;
   U2 := 0;
   H := 'nil';
   repeat
      ReadGroupValue;
      case FGroup of
         10,20,40,50,51:
         begin
            D := StrToExtended(FValue);
            case FGroup of
               10: aX := D;
               20: aY := D;
               40: R := D;
               50: U1 := DejRadZeStupnu(D);
               51: U2 := DejRadZeStupnu(D);
            end;
         end;
         5: H := FValue;
      end;
   until FGRoup = 0;
   NextEntityReady := true;
   if R = 0 then
   begin
      PostTextMsg(tm_AddToProtocol,'     ARC R=0 H:'+H,nil);
      Application.ProcessMessages;
      exit;
   end;
   TL := TTopolLineElement.Create;
   TL.Druh := 2;
   TL.Barva := 9;
   TL.Znacka := 0;
   ActualEntity.AddElement(TL);
   L := TPolyLineElement.Create;
   TL.Geometry := L;
   NacpiUhelDoDvaPi(U1);
   NacpiUhelDoDvaPi(U2);
   // -----------------linie
   if U2 = 0 then U2 := DvaPi;
   Ro := R - 0.005{PresnostKrivek};
     //Ro := SQRT(SQR(Oblouk_a)-SQR(0.5{PresnostKrivek/2}));
   dU := 2*ACos(Ro/R);
   if dU > PI/20 then dU := PI/20;
   x1 := R * (cos(U1));
   y1 := R * (sin(U1));
   X1 := aX + X1;
   Y1 := aY + Y1;
   //X1 := Simple_RoundTo(X1,-1);
   //Y1 := Simple_RoundTo(Y1,-1);
   L.AddXY(X1,Y1);

   Alfa := U1+dU;
   if U2 < U1 then
      U2 := U2 + PI*2;
   while Alfa < U2 do
   begin
      x1 := (R*(cos(Alfa)));
      y1 := (R*(sin(Alfa)));
      X1 := aX + X1;
      Y1 := aY + Y1;
      //X1 := Simple_RoundTo(X1,-1);
      //Y1 := Simple_RoundTo(Y1,-1);
      L.AddXY(X1,Y1);
      Alfa := Alfa+dU;
   end;
   x1 := R * (cos(U2));
   y1 := R * (sin(U2));
   X1 := aX + X1;
   Y1 := aY + Y1;
   //X1 := Simple_RoundTo(X1,-1);
   //Y1 := Simple_RoundTo(Y1,-1);
   L.AddXY(X1,Y1);
   L.Update;
   L.CalculateLabelProps;
   // -----------------linie
end;

procedure TDxfImport.ReadBlock;
var
   WasName: boolean;
   Blk: TRectEntity;
   aX: double;
   aY: double;
begin
   NextEntityReady := false;
   WasName := false;
   BLK := TRectEntity.Create;
   Blocks.NextElement := BLK;
   ActualEntity := BLK;
   aX := 0;
   aY := 0;
   repeat
      if not NextEntityReady then
         ReadGroupValue;
      NextEntityReady := false;
      case FGroup of
         0: begin
               ReadEntity;
               WasName := true;
            end;
         else
            if not WasName then
            case FGroup of
               2: BLK.Name := FValue;
               10: aX := StrToExtended(FValue);
               20: aY := StrToExtended(FValue);
            end;
      end;
   until (FGroup = 0) and (FValue = 'ENDBLK');
   BLK.Update;
   BLK.AbsRefX := aX;
   BLK.AbsRefY := aY;
end;

procedure TDxfImport.ReadBlocks;
begin
   repeat
      ReadGroupValue;
      if (FGroup = 0) and (FValue='BLOCK') then
         ReadBlock;
   until EndOfSection;
end;

procedure TDxfImport.ReadCircle;
var
   H: string;
   aX: double;
   aY: double;
   R: double;
   U1: double;
   U2: double;
   D: double;
   TL: TTopolLineElement;
   L: TPolyLineElement;
   Ro,dU,Alfa: double;
   X1,Y1: double;
begin
   aX := 0;
   aY := 0;
   R  := 0;
   H := 'nil';
   repeat
      ReadGroupValue;
      case FGroup of
         10,20,40{,50,51}:
         begin
            D := StrToExtended(FValue);
            case FGroup of
               10: aX := D;
               20: aY := D;
               40: R := D;
               //50: U1 := DejRadZeStupnu(D);
               //51: U2 := DejRadZeStupnu(D);
            end;
         end;
         5: H := FValue;
      end;
   until FGRoup = 0;
   NextEntityReady := true;
   if R = 0 then
   begin
      PostTextMsg(tm_AddToProtocol,'     CIR R=0 H:'+H,nil);
      Application.ProcessMessages;
      exit;
   end;
   TL := TTopolLineElement.Create;
   TL.Druh := 3;
   TL.Barva := 8;
   TL.Znacka := 0;
   ActualEntity.AddElement(TL);
   L := TPolyLineElement.Create;
   TL.Geometry := L;
   //NacpiUhelDoDvaPi(U1);
   //NacpiUhelDoDvaPi(U2);
   // -----------------linie
   //if U2 = 0 then U2 := DvaPi;
   U1 := 0;
   U2 := DvaPI;
   Ro := R - 0.005{PresnostKrivek};
     //Ro := SQRT(SQR(Oblouk_a)-SQR(0.5{PresnostKrivek/2}));
   dU := 2*ACos(Ro/R);
   if dU > PI/20 then dU := PI/20;

   x1 := R * (cos(U1));
   y1 := R * (sin(U1));
   X1 := aX + X1;
   Y1 := aY + Y1;
   //X1 := Simple_RoundTo(X1,-1);
   //Y1 := Simple_RoundTo(Y1,-1);
   L.AddXY(X1,Y1);

   Alfa := U1+dU;
   if U2 < U1 then
      U2 := U2 + PI*2;
   while Alfa < U2 do
   begin
      x1 := (R*(cos(Alfa)));
      y1 := (R*(sin(Alfa)));
      X1 := aX + X1;
      Y1 := aY + Y1;
      //X1 := Simple_RoundTo(X1,-1);
      //Y1 := Simple_RoundTo(Y1,-1);
      L.AddXY(X1,Y1);
      Alfa := Alfa+dU;
   end;
   x1 := R * (cos(U2));
   y1 := R * (sin(U2));
   X1 := aX + X1;
   Y1 := aY + Y1;
   //X1 := Simple_RoundTo(X1,-1);
   //Y1 := Simple_RoundTo(Y1,-1);
   L.AddXY(X1,Y1);
   L.Update;
   L.CalculateLabelProps;
   // -----------------linie
end;

procedure TDxfImport.ReadENTITIES;
begin
   NextEntityReady := false;
   ActualEntity := Lines;
   repeat
      if not NextEntityReady then
         ReadGroupValue;
      NextEntityReady := false;
      ReadEntity;
   until EndOfSection;
end;

procedure TDxfImport.ReadEntity;
var
   l: integer;
   E: TElement;
begin
   case FGroup of
      0:
      begin
         Entities.FindOrCreateElementName(TOwnerEntity,FValue);
         l := length(FValue);
         if l > 2 then
         case FValue[1] of
            'A':
            if FValue = 'ARC' then
            begin
               ReadArc;
            end;
            'C':
            if FValue = 'CIRCLE' then
            begin
               ReadCircle;
            end;
            'E':
            if FValue = 'ELLIPSE' then
            begin
            end;
            'I':
            if FValue = 'INSERT' then
            begin
               ReadInsert;
            end;
            'L':
            if FValue = 'LINE' then
            begin
               ReadLine;
            end;
            'P':
            case FValue[3] of
               'I':
               if FValue = 'POINT' then
               begin
               end;
               'L':
               if FValue = 'POLYLINE' then
               begin
                  ReadPolyLine;
               end;
            end;
            'S':
            if FValue = 'SPLINE' then
            begin
            end;
            'T':
            if FValue = 'TEXT' then
            begin
            end;
         end;
      end;
   end;
end;

function TDxfImport.ReadGroupValue: boolean;
var
   S: string;
begin
   result := false;
   ReadLn(F,S);
   FPosition := FPosition + Length(S)+2;
   S := BezVnejsichMezer(S);
   FEOF := EOF(F);
   FGroup := StrToIntDef(S,-1);
   if FEOF then exit;
   result := true;
   ReadLn(F,S);
   FEOF := EOF(F);
   FPosition := FPosition + Length(S)+2;
   FValue := BezVnejsichMezer(S);
   if FGroup = 0 then
      FValue := UpperCase(FValue);
   if FPosition - FLastPos >= DSize then
   begin
      FLastPos := FPosition;
      Progress := FPosition;
      Application.ProcessMessages;
   end;
end;

procedure TDxfImport.ReadInsert;
var
   aX,aY,sX,sY,Angle: double;
   X1,Y1,X2,Y2: double;
   BLK: TEntity;
   i,ii: integer;
   TL,TL1: TTopolLineElement;
   L,L1: TPolyLineElement;
   e: TElement;
begin
   aX := 0;
   aY := 0;
   sX := 1;
   sY := 1;
   Angle := 0;
   BLK := nil;
   repeat
      ReadGroupValue;
      case FGroup of
          2: BLK := TEntity(Blocks.FindElementName(FValue));
         10: aX := StrToExtended(FValue);
         20: aY := StrToExtended(FValue);
         41: sX := StrToExtended(FValue);
         42: sY := StrToExtended(FValue);
         50: Angle := StrToExtended(FValue);
      end;
   until FGroup = 0;
   NextEntityReady := true;
   if (BLK = nil) or BLK.NoElements then exit;
   if Angle <> 0 then
      Angle := DejRadZeStupnu(Angle);
   for i := 0 to BLK.LastIndex do
   begin
      e := BLK.Elements[i];
      if not (e is TTopolLineElement) then continue;
      TL1 := TTopolLineElement(e);
      L1 := TPolyLineElement(TL1.Geometry);
      if L1 = nil then continue;
      TL := TTopolLineElement.Create;
      ActualEntity.AddElement(TL);
      TL.Druh := 5;
      TL.Barva := 11;
      TL.Znacka := 0;
      L := TPolyLineElement.Create;
      TL.Geometry := L;
      // transformace souradnic
      for ii := 0 to L1.LastIndex do
      begin
         X1 := L1.Xi[ii];
         Y1 := L1.Yi[ii];
         // scale;
         X1 := X1 * sX;
         Y1 := Y1 * sY;
         // rotation
         if Angle <> 0 then
         begin
            X2 := (x1*cos(Angle)-y1*sin(Angle));
            Y2 := (x1*sin(Angle)+y1*cos(Angle));
         end
         else
         begin
            X2 := X1;
            Y2 := Y1;
         end;
         X2 := aX + X2;
         Y2 := aY + Y2;
         L.AddXY(X2,Y2);
      end;
      L.Update;
      L.CalculateLabelProps;
   end;
end;

procedure TDxfImport.ReadLAYER;
var
   NewEntry: boolean;
   LayerName: string;
   LineType: string;
   aColor: string;
begin
   NewEntry := false;
   repeat
      ReadGroupValue;
      case FGroup of
         0: if FValue='LAYER' then
            begin
               NewEntry := true;
               LayerName := '';
               LineType := '';
            end;
         2: LayerName := Fvalue;
        62: aColor := FValue;
         6: begin
               LineType := FValue;
               PostTextMsg(tm_AddToProtocol,'      LAYER: '+LayerName+' ['+LineType+']'+aColor,nil);
               Application.ProcessMessages;
            end;
      end;
   until EndOfTable;
end;

procedure TDxfImport.ReadLine;
var
   L: TPolyLineElement;
   TL: TTopolLineElement;
   D: double;
begin
   TL := TTopolLineElement.Create;
   TL.Druh := 1;
   TL.Barva := 13;
   TL.Znacka := 0;
   L := TPolyLineElement.Create;
   TL.Geometry := L;
   ActualEntity.AddElement(TL);
   L.AddXY(0,0);
   L.AddXY(0,0);
   repeat
      ReadGroupValue;
      if FGroup in [10,20,11,21] then
      begin
         D := StrToExtended(FValue);
         case FGroup of
            10: L.Xi[0] := D;
            20: L.Yi[0] := D;
            11: L.Xi[1] := D;
            21: L.Yi[1] := D;
         end;
      end;
   until FGroup = 0;
   L.Update;
   L.CalculateLabelProps;
   NextEntityReady := true;
end;

procedure TDxfImport.ReadLTYPE;
var
   NewEntry: boolean;
   LTypeName: string;
   Desc: string;
begin
   NewEntry := false;
   repeat
      ReadGroupValue;
      case FGroup of
         0: if FValue = 'LTYPE' then
            begin
               NewEntry := true;
               LTYPEName := '';
               Desc := '';
            end;
         2: LTypeName := Fvalue;
         3: begin
               Desc := FValue;
               PostTextMsg(tm_AddToProtocol,'      LTYPE: '+LTypeName+' ['+Desc+']',nil);
               Application.ProcessMessages;
            end;
      end;
   until EndOfTable;
end;

procedure TDxfImport.ReadPolyLine;
var
   aX: double;
   aY: double;
   L: TPolyLineElement;
   TL: TTopolLineElement;
   aFlag: integer;
begin
   TL := TTopolLineElement.Create;
   TL.Druh := 1;
   TL.Barva := 11;
   TL.Znacka := 0;
   aX := 0;
   aY := 0;
   aFlag := 0;
   L := TPolyLineElement.Create;
   TL.Geometry := L;
   ActualEntity.AddElement(TL);
   NextEntityReady := false;
   repeat
      if not NextEntityReady then
         ReadGroupValue;
      NextEntityReady := false;
      case FGroup of
        70: aFlag := StrToIntDef(FValue,0);
         0: if FValue = 'VERTEX' then
            begin
               ReadVertex(aX,aY);
               L.AddXY(aX,aY);
               aX := 0;
               aY := 0;
            end;
      end;
   until (FGroup = 0) and (FValue = 'SEQEND');
   if (aFlag and 1) = 1 then // uzavrena polyline
   if L.Count > 1 then
      L.AddXY(L.Xi[0],L.Yi[0]);
   L.Update;
   L.CalculateLabelProps;
end;

procedure TDxfImport.ReadSection;
var
   SectionName: string;
   WasName: boolean;
begin
   SectionName := '';
   WasName := false;
   repeat
      ReadGroupValue;
      case FGroup of
         2:
         if not WasName then
         begin
            WasName := true;
            SectionName := UpperCase(FValue);
            PostTextMsg(tm_AddToProtocol,'SECTION: '+SectionName,nil);
            Application.ProcessMessages;
            if SectionName = 'TABLES' then
               ReadTABLES
            else
            if SectionName = 'ENTITIES' then
               ReadENTITIES;
            if SectionName = 'BLOCKS' then
               ReadBlocks;
         end;
      end;
   until EndOfSection;
end;

procedure TDxfImport.ReadTable;
var
   TableName: string;
   WasName: boolean;
begin
   TableName := '';
   WasName := false;
   repeat
      ReadGroupValue;
      if not WasName and (FGroup = 2) then
      begin
         WasName := true;
         TableName := UpperCase(FValue);
         PostTextMsg(tm_AddToProtocol,'   Table: '+TableName,nil);
         Application.ProcessMessages;
         if TableName = 'LTYPE' then
            ReadLTYPE
         else
         if TableName = 'LAYER' then
            ReadLAYER;
      end;
   until EndOfTable;
end;

procedure TDxfImport.ReadTABLES;
begin
   repeat
      ReadGroupValue;
      if (FGroup = 0) and (FValue = 'TABLE') then
         ReadTable;
   until EndOfSection;
end;

procedure TDxfImport.ReadVertex(var aX, aY: double);
begin
   repeat
      ReadGroupValue;
      case FGroup of
         10: aX := StrToExtended(FValue);
         20: aY := StrToExtended(FValue);
      end;
   until FGroup = 0;
   NextEntityReady := true;
end;

procedure TDxfImport.SetActualEntity(const Value: TEntity);
begin
  FActualEntity := Value;
end;

procedure TDxfImport.SetFileName(const Value: string);
begin
  FFileName := Value;
end;

procedure TDxfImport.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;

procedure TDxfImport.SetNextEntityReady(const Value: boolean);
begin
  FNextEntityReady := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TDxfImport);
END.
