unit T_ClearAreas;
{
***********************************************************************
**                                                                   **
**  ORBIS ... Cisteni ploch od neplosnych linii                      **
**                                                                   **
**                       Topol Pro 1999                             **
**                                                                   **
***********************************************************************


CLASS: TSubAreaMaker
INPUT:  TSpaceEntityQTree
OUTPUT: TTopolPlochy ... Strom Ploch s Enklvami
}

interface
uses
// KERNEL
   Elements,
   Tools,
// SPACE
   e_Space,
   e_QTree,
// SPACETOOL
   T_ToTool;

type




TClearAreas = class(TProchazecPlochGauge)
private
   FDontRemoveConectionAreaArea: boolean;
   FQTree: TSpaceEntityQTree;
   procedure InitLinesIDs(anArea: TPolyLineAreaElement; anID: integer);
   procedure RemoveLines(anArea: TPolyLineAreaElement);
   procedure ClasifyRing(anArea: TPolylineAreaElement; aLineIndex: integer; aX, aY: double);
   procedure AddHole(anArea: TPolylineAreaElement; aLineIndex: integer);
   procedure AddHoles(anArea: TPolyLineAreaElement);
public
   procedure Process(Element: TElement); override;
   procedure DoIt; override;
published
  property DontRemoveConectionAreaArea: Boolean read FDontRemoveConectionAreaArea write FDontRemoveConectionAreaArea;
end;


implementation

uses
   SysUtils, Dialogs, Forms, Controls,
// KERNEL
   e_Class,
   U_String,
// GLIB
   AnalInt,
   g_Elems,
// DIALOG
   F_Gauge,
// SPACETOOL
   T_Topol;


                   { TClearAreas }

procedure TClearAreas.InitLinesIDs(anArea: TPolyLineAreaElement; anID: integer);
var
  I: integer;
  aLine: TPolyLineElement;
begin
  for I:= 0 to anArea.Count-1 do
  begin
    aLine:= TPolyLineElement(anArea.Elements[I]);
    if not (TObject(aLine) is TPolyLineElement) then
       continue;
    aLine.Id:= anID;
  end;
end;


procedure TClearAreas.RemoveLines(anArea: TPolyLineAreaElement);
var
  I: integer;
  aLine: TPolyLineElement;
begin
  for I:= anArea.Count-1 downto 0 do
  begin
    aLine:= TPolyLineElement(anArea.Elements[I]);
    if not (TObject(aLine) is TPolyLineElement) then
       continue;
    if aLine.ID > 0 then
       anArea.RemoveElement(aLine);
  end;
end;



procedure TClearAreas.ClasifyRing(anArea: TPolylineAreaElement; aLineIndex: integer; aX, aY: double);
var
  I: integer;
  aLine: TPolyLineElement;
  aXN, aYN: Double;
  aCounter: integer;
  aSum: Double;

  procedure NextLine;
  begin
    Inc(I);
    dec(aCounter);
    if I >= anArea.Count then
       I:= 0;
  end;

begin
  I:= aLineIndex;
  aCounter:= anArea.Count;
  aSum:= 0;
  repeat
    aLine:= TPolyLineElement(anArea.Elements[I]);
    if not (TObject(aLine) is TPolyLineElement) then
    begin
       NextLine;
       continue;
    end;

    aSum:= aSum + aLine.GetAreaFromBegin(anArea.LinesOrigines[I]);
    aLine.GetOrientedCoord(0, not anArea.LinesOrigines[I], aXN, aYN);
    NextLine;
  until (aX = aXN) and (aY = aYN) or (aCounter = 0);

  if aCounter= 0 then
  begin
     ShowMessage('Bad ring');
     Exit;
  end;

  if aSum > 0 then
     exit;

  I:= aLineIndex;
  repeat
    aLine:= TPolyLineElement(anArea.Elements[I]);
    if not (TObject(aLine) is TPolyLineElement) then
    begin
       NextLine;
       continue;
    end;
// test zda nejde o pseudohranici
    if anArea.HasLine(aLine, True) and anArea.HasLine(aLine, False) then
      aLine.ID:= 1
    else
      aLine.ID:= 2; // skutecna hranice vnitrni plochy

    aLine.GetOrientedCoord(0, not anArea.LinesOrigines[I], aXN, aYN);
    NextLine;
  until (aX = aXN) and (aY = aYN);
end;




procedure TClearAreas.AddHole(anArea: TPolylineAreaElement; aLineIndex: integer);
var
  I: integer;
  aLine: TPolyLineElement;
  aHole: TPolyLineAreaElement;
  aXP, aYP, aXN, aYN, aXO, aYO: Double;
  aCounter: integer;
  aSum: Double;

begin
  I:= aLineIndex;
  aCounter:= anArea.Count;
  aLine:= TPolyLineElement(anArea.Elements[I]);
  aLine.GetOrientedCoord(0, anArea.LinesOrigines[I], aXP, aYP); // vychozi souradnice

  aSum:= 0;
  aHole:= TPolyLineAreaElement.Create;

  repeat
    aLine:= TPolyLineElement(anArea.Elements[I]);
    if not (TObject(aLine) is TPolyLineElement) then
       aLine:= nil
    else
    if (aLine.ID = 1) then // obejiti pseudohranice
    begin
      aLine.GetOrientedCoord(0, anArea.LinesOrigines[I], aXO, aYO); // pocatecni bod odbocky

      repeat
        Inc(I);
        if I >= anArea.Count then
           I:= 0;
        dec(aCounter);

        // bude dalsi linie mit svuj pocatek v aXO, aYO ?
        aLine:= TPolyLineElement(anArea.Elements[I]);
        if not (TObject(aLine) is TPolyLineElement) then
           aLine:= nil;
        if aLine <> nil then
        begin
          aLine.GetOrientedCoord(0, anArea.LinesOrigines[I], aXN, aYN); // koncova souradnice
        end;

      until (aXN=aXO) and (aYN=aYO) and (aLine.ID <> 2) or (aCounter < 0);

      if aCounter < 0 then
         break;

      Dec(I);

      continue;
    end;

    if aLine <> nil then
    begin
      aLine.GetOrientedCoord(0, not anArea.LinesOrigines[I], aXN, aYN); // koncova souradnice

      aHole.AddPolyLine(aLine, anArea.LinesOrigines[I]);
      aLine.ID:= 3;
    end;

    Inc(I);
    if I >= anArea.Count then
       I:= 0;
    dec(aCounter)
  until (aXP = aXN) and (aYP = aYN) or (aCounter < 0);


  if aCounter < 0 then
  begin
     ShowMessage('Area '+IntToStr(anArea.SpaceID)+':Bad ring near ' + FloatToStr(aXP) + '; ' + FloatToStr(aYP));
     Exit;
  end;

  if aHole.Area1 < 0 then
     aHole.IsHole:= True;

  if FQTree <> nil then
  begin
    FQTree.NextSpaceEntity:= aHole;
  end;
end;



procedure TClearAreas.AddHoles(anArea: TPolyLineAreaElement);
var
  I: integer;
  aLine: TPolyLineElement;
begin
  I:= 0;
  while I < anArea.Count do
  begin
    aLine:= TPolyLineElement(anArea.Elements[I]);
    if not (TObject(aLine) is TPolyLineElement) then
       continue;
    if aLine.ID = 2 then
       AddHole(anArea, I);
    inc(I);
  end;
end;


procedure TClearAreas.Process(Element: TElement);
var
  anArea: TPolyLineAreaElement;
  aLine, aLineN: TPolyLineElement;
  I: integer;
  QTW: TQTreeWalker;
  aX, aY, aXN, aYN: Double;
  WasPseudoBorder: Boolean;
begin
  if not (Element is TPolyLineAreaElement) then
     exit;
  anArea:= TPolyLineAreaElement(Element);
  if anArea.Area1 <= 0 then
     Exit;

  inc(FPocetPloch);
  GaugeSet(FPocetPloch);
{
  GaugeCaption:= 'SAMIQ:' + DoubleToStr(aHole.Xi[0], 11, 2) + ' ' + DoubleToStr(aHole.Yi[0],12, 2) +
                 ' ' + IntToStr(FPocetPloch);
}

  if anArea.IsDone then exit;
  InitLinesIDs(anArea, 0);
  WasPseudoBorder:= False;
// vyhazeni linii, na ktere se plocha odkazuje dvakrat
  I:= 0;
  while I < anArea.Count  do
  begin
    aLine:= TPolyLineElement(anArea.Elements[I]);
    if not (TObject(aLine) is TPolyLineElement) then
    begin
      Inc(I);
      continue;
    end;

    if aLine.ID > 0 then
    begin
      Inc(I);
      continue;
    end;

    if anArea.HasLine(aLine, True) and anArea.HasLine(aLine, False) then
    begin
      aLine.ID:= 1;
      WasPseudoBorder:= True;
    end
    else
    begin
      if WasPseudoBorder then
      begin
        aLine.GetOrientedCoord(0, anArea.LinesOrigines[I], aX, aY);
        ClasifyRing(anArea, I, aX, aY);
      end;
      WasPseudoBorder:= false
    end;
    Inc(I);
  end;
  { !!!!!!!! Test}
  AddHoles(anArea);
  RemoveLines(anArea);
  { !!!!!!!! Test}
  anArea.IsDone:= true
end;


procedure TClearAreas.DoIt;
begin
  if (input is TSpaceEntityQTree) then
     FQTree:= TSpaceEntityQTree(Input)
  else
  begin
    if input is TPolyLineAreaElement then
    begin
      Process(Input)
    end;
    exit;
  end;
  inherited DoIt
end;



initialization
   RegisterOrbisClass(TClearAreas);
end.



