unit T_meas;

interface

uses
   Controls, Graphics,
   Elements,
   e_Space,
   i_Choice,
   M_InterA,
   t_SpPick,
   t_InterA,
   Tools;


const
  PixelR: integer = 3;

type


TMeasureTool = class(TInteractiveTool)
private
  FPoly: TPolyElement;

  FDragIsStarted: boolean;
  FActualCrdIndex: longInt;  // aktuln zpracovvan souadnice
  FMovedCrdIndex: integer;   // souadnice nad kterou se pohybuje my
  FMovedBetween: boolean;
  FHeaderAtTale: boolean;

  FActX, FActY: double;
  FCloseEnds: boolean;
  FColor: TColor;
  procedure XORDraw; virtual;
  procedure SendInfo; virtual;
  procedure CreateMeasuredObject; virtual; abstract;
  procedure ChangeCursor; virtual;
  procedure DoOnMeasureClick(Sender: TObject); virtual;

  procedure DrawRect(X, Y, HalfSize: double);
public
  constructor Create; override;
  procedure DoStartDrag; virtual;
  function  NearTo(anIndex: integer): boolean;
  procedure AddCrdToActualEnd; virtual;
  procedure CreatePopUp(anIndex: integer); virtual;
  procedure On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
  procedure On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
  procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;

  procedure On_tm_StartDrag(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
  procedure On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;

  procedure On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
  procedure On_Activate  ; override;
  procedure On_DeActivate; override;
  procedure On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
published
  property Color: TColor read FColor write FColor;
end;


TMeasureLengthTool = class(TMeasureTool)
private
  procedure XORDraw; override;
  procedure CreateMeasuredObject; override;
  function GetLength: double;
protected
//  procedure ChangeCursor; override;
  function   GetCursor: TCursor; override;
public
   procedure On_tm_DragDrop(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
published
  property Length: double read GetLength;
end;


TMeasureAreaTool = class(TMeasureTool)
private
  procedure XORDraw; override;
  procedure SendInfo; override;
  procedure CreateMeasuredObject; override;
  function GetLength: double;
  function GetArea: double;
protected
   function   GetCursor: TCursor; override;
published
  property Length: double read GetLength;
  property Area: double read GetArea;
end;



implementation
uses
  WinTypes, Forms, Classes, Menus,
  SysUtils, Dialogs,
// KERNEL
  m_Orbis,
  U_String,
  F_Std, D_Std,
  e_Class,
  u_Cursor,
  Worlds,
// GLIB
  G_Elems,
  MathLib
  ;





constructor TMeasureTool.Create;
begin
  CreateMeasuredObject;
  FColor:= clRed;
end;


procedure TMeasureTool.SendInfo;
begin
end;

procedure TMeasureTool.DrawRect(X, Y, HalfSize: double);
begin
  Device.MoveTo(X+HalfSize, Y+HalfSize);
  Device.LineTo(X-HalfSize, Y+HalfSize);
  Device.LineTo(X-HalfSize, Y-HalfSize);
  Device.LineTo(X+HalfSize, Y-HalfSize);
  Device.LineTo(X+HalfSize, Y+HalfSize);
end;




function  TMeasureTool.NearTo(anIndex: integer): boolean;
var
  aR: double;
begin
  aR:= Device.Projection.GetModelLength(PixelR);
  result:= InTolerance(FPoly.Xi[anIndex], FPoly.Yi[anIndex], FActX, FActY, 2*aR)
end;



procedure TMeasureTool.DoOnMeasureClick(Sender: TObject);
begin
  if FPoly = nil then
     Exit;
  if not (input is TRectEntity) then
     Exit;
  FPoly.RecalculateRect;
  TRectEntity(Input).AddAndUpdateRect(FPoly);
  FPoly:= nil;
  CreateMeasuredObject;
end;


procedure TMeasureTool.CreatePopUp(anIndex: integer);
var
  aPopUp: TPopUpMenu;
  aMenu1, aMenu2, aMenu3: TMenuItem;
begin
  aPopUp:= TPopUpMenu.Create(Application);
  aPopUp.AutoPopUp:= false;

  aMenu1:= TMenuItem.Create(aPopUp);
  aMenu2:= TMenuItem.Create(aPopUp);
  aMenu1.Caption:= 'X='+FloatToStr(FActX);
  aMenu2.Caption:= 'Y='+FloatToStr(FActY);
  aPopUp.Items.Add(aMenu1);
  aPopUp.Items.Add(aMenu2);

  aMenu3:= TMenuItem.Create(aPopUp);
  aMenu3.Caption:= 'Potvrzen';
  aMenu3.OnClick:= DoOnMeasureClick;
  aPopUp.Items.Add(aMenu3);

  aPopUp.PopUp(100, 100);
end;




procedure TMeasureTool.XORDraw;
begin
  if FPoly = nil then
     Exit;
  Device.Pen.Color:= FColor;
  Device.Pen.Style:= psSolid;
  Device.Pen.Mode:= pmXOR;
  Device.Pen.Width:= 3;
  Device.Brush.Style:= bsClear;
  Device.Polyline(FPoly)
end;



procedure TMeasureTool.On_tm_StartDrag(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
  FDragIsStarted:= true;
end;



procedure TMeasureTool.AddCrdToActualEnd;
begin
  if FPoly = nil then
     Exit;
  try
    if FPoly.CoordsCount = 0 then
    begin
      FPoly.AddXY(FActX, FActY);
      FActualCrdIndex:= 0;
      XORDraw;
    end
    else
    begin
      XORDraw;
      if FHeaderAtTale then
      begin
        FPoly.InsertXY(0, FActX, FActY);
        FActualCrdIndex:= 0;
      end
      else
      begin
        FPoly.AddXY(FActX, FActY);
        FActualCrdIndex:= FPoly.CoordsCount-1;
      end;
      XORDraw;
    end
  except
    on E: Exception do
     ShowMessage('Zahjen snmn: ' + E.Message);
  end
end;




procedure TMeasureTool.DoStartDrag;
begin
  if FMovedCrdIndex = -1 then exit;

  if not FMovedBetween then
     FActualCrdIndex:= FMovedCrdIndex
  else
  begin
    XORDraw;
    FPoly.InsertXY(FMovedCrdIndex+1, FActX, FActY);
    FActualCrdIndex:= FMovedCrdIndex + 1;
    XORDraw;
  end;
end;



procedure TMeasureTool.On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
  aEndIndex: integer;
begin
  if FDragIsStarted then
     DoStartDrag;
  FDragIsStarted:= false;

  try
   FActX:= Device.Projection.GetModelX(aMessage.x);
   FActY:= Device.Projection.GetModelY(aMessage.y);

   XORDraw;

   FCloseEnds:= false;
   if FActualCrdIndex in [0, FPoly.CoordsCount-1] then
   begin
     aEndIndex:= 0;
     if FHeaderAtTale then
        aEndIndex:= FPoly.CoordsCount-1;

     if NearTo(aEndIndex) then
     begin
       FCloseEnds:= true;
     end
   end;

   FPoly.Xi[FActualCrdIndex]:= FActX;
   FPoly.Yi[FActualCrdIndex]:= FActY;
   XORDraw;

   PostMsg(tm_ToolChange, nil, nil, true);

   SendInfo;
  except
    on E: Exception do
     ShowMessage('Men: ' + E.Message);
  end;
  Handled:= true
end;




procedure TMeasureLengthTool.On_tm_DragDrop(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
  aFixIndex, aMoveIndex: integer;
begin
  try
    if FCloseEnds then
    begin
      if FHeaderAtTale then
      begin
        aFixIndex:= FPoly.CoordsCount-1;
        aMoveIndex:= 0;
      end
      else
      begin
        aFixIndex:= 0;
        aMoveIndex:= FPoly.CoordsCount-1;
      end;
      XOrDraw;
      FPoly.Xi[aMoveIndex]:= FPoly.Xi[aFixIndex];
      FPoly.Yi[aMoveIndex]:= FPoly.Yi[aFixIndex];
      XOrDraw;
    end;
//    SendInfo;
  except
    on E: Exception do
     ShowMessage('Men: ' + E.Message);
  end;
end;



procedure TMeasureTool.On_Activate;
begin
  Application.ProcessMessages;
  XORDraw;
end;


procedure TMeasureTool.On_DeActivate;
var
  aForm: TStandardForm;
begin
  Application.ProcessMessages;
  XORDraw;

{
  if Device is TCanvasToBmpDevice then
  begin
    aForm:= TStandardForm(TCanvasToBMPDevice(Device).CanvasForm);
    if aForm is TStandardForm then
    begin
      aForm.Label3.Caption:= 'M 1 :';
    end;
  end;
}
end;


procedure TMeasureTool.On_tm_ReView(var aMessage: TOrbisMessage; var Handled: boolean);
begin
  Application.ProcessMessages;
  XORDraw;
end;


procedure TMeasureTool.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
  if (aMessage.Button = mbRight) then
  begin
    if (FMovedCrdIndex > -1) then
    begin
      CreatePopUp(0);
      exit;
    end
    else
    begin
    end
  end;

  if aMessage.Button <> mbLeft then
     exit;

  Handled:= false;

  if FMovedCrdIndex = -1 then
  begin
    AddCrdToActualEnd;
    PostMsg(tm_ToolChange, nil, nil, true);
    exit;
  end;

  if not FMovedBetween and (FActualCrdIndex <> FMovedCrdIndex) then
  begin
    XOrDraw;
    FActualCrdIndex:= FMovedCrdIndex;
    if FActualCrdIndex = 0 then
       FHeaderAtTale:= true
    else
    if FPoly.CoordsCount-1 = FActualCrdIndex then
       FHeaderAtTale:= false;
    XOrDraw;
    PostMsg(tm_ToolChange, nil, nil, true);
 end
end;



procedure TMeasureTool.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
  aX, aY: integer;
begin
  aX:= aMessage.x;
  aY:= aMessage.y;
  FActX:= Device.Projection.GetModelX(aX);
  FActY:= Device.Projection.GetModelY(aY);


  if FPoly = nil then exit;
  ChangeCursor;
end;


procedure TMeasureTool.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
end;




procedure TMeasureTool.On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean);
begin
  if (FPoly.CoordsCount = 0) then exit;

   case aMessage.Key of
      VK_Back:
      begin
        if (FPoly.CoordsCount > 1) then
        begin
          XORDraw;
          if FHeaderAtTale then
          begin
            FPoly.FreeElement(FPoly.Elements[0]);
            FActualCrdIndex:= 0;
          end
          else
          begin
            FPoly.FreeElement(FPoly.Elements[FPoly.CoordsCount-1]);
            FActualCrdIndex:= FPoly.CoordsCount-1;
          end;
          XORDraw;
          SendInfo;
        end;
      end;
      VK_DELETE:
      begin
        if (ssCtrl in aMessage.Shift) then
        // vyprzdnn cel linie
        begin
          CreateMeasuredObject;
          SendInfo;
        end
        else
        if (FPoly.CoordsCount > 1) and (FActualCrdIndex > -1) then
        begin
          XORDraw;
          FPoly.FreeElement(FPoly.Elements[FActualCrdIndex]);
          if (FActualCrdIndex >= FPoly.CoordsCount) then
             FActualCrdIndex:= FPoly.CoordsCount-1;
          XORDraw;
          SendInfo;
        end;
      end;
      VK_Insert:
      begin

      end;
      ord('R'), ord('r'):
      begin
        XORDraw;
      end;
   end;
end;






    { TMeasureLengthTool }


function  TMeasureLengthTool.GetCursor: TCursor;
begin
//   result := cr_MLength;
   result := cr_SelectLine;
end;


procedure TMeasureLengthTool.CreateMeasuredObject;
begin
  if FPoly <> nil then
  begin
     XORDraw;
     FreeElement(FPoly);
  end;
  FPoly:= TPolyLineElement.Create;
  FActualCrdIndex:= -1;
  FMovedCrdIndex:= -1;
  if FPoly = nil then exit;
  AddElement(FPoly);
//  FPoly.AddXY(0, 0);
end;




procedure TMeasureLengthTool.XORDraw;
var
  aR: double;
  I: integer;
  K: integer;
begin
  Device.Pen.Color:= FColor;
  Device.Pen.Style:= psSolid;
  Device.Pen.Mode:= pmXOR;
  Device.Pen.Width:= 1;
  Device.Brush.Style:= bsClear;
  Device.Polyline(FPoly);

// vykreslovn znaek
  aR:= Device.Projection.GetModelLength(PixelR);
  K:= FPoly.CoordsCount-1;
  for I:= 0 to K do
  begin
    DrawRect(FPoly.Xi[i], FPoly.Yi[i], aR);
  end;

// Vykreslen vybranho bodu
  I:= FActualCrdIndex;
  aR:= Device.Projection.GetModelLength(PixelR - 1);
  if I >= 0 then
  begin
    DrawRect(FPoly.Xi[i], FPoly.Yi[i], aR);
  end;

// Vykreslen hlaviky
  aR:= Device.Projection.GetModelLength(PixelR + 2);

  if FHeaderAtTale then
     K:= 0;

  if K >= 0 then
  begin
    DrawRect(FPoly.Xi[i], FPoly.Yi[i], aR);

    if FCloseEnds then
    begin
      Device.MoveTo(FPoly.Xi[K]+2*aR, FPoly.Yi[K]+2*aR);
      Device.LineTo(FPoly.Xi[K]-2*aR, FPoly.Yi[K]-2*aR);
      Device.MoveTo(FPoly.Xi[K]-2*aR, FPoly.Yi[K]+2*aR);
      Device.LineTo(FPoly.Xi[K]+2*aR, FPoly.Yi[K]-2*aR);
    end
  end;

  PostMsgToOwners(tm_Repaint,nil,DontKill);
end;



procedure TMeasureTool.ChangeCursor;
var
  aCurs: TCursor;
  I, K: integer;
  aR: double;
  PataUvnitr: boolean;
  PataX, PataY, Vzdalenost: double;

begin
  aR:= Device.Projection.GetModelLength(PixelR);
  K:= FPoly.CoordsCount-1;

  aCurs:= GetCursor;
  FMovedCrdIndex:= -1;
  FMovedBetween:= false;

  for I:= 0 to K do
  begin
    if InTolerance(FPoly.Xi[i], FPoly.Yi[i], FActX, FActY, 2*aR) then
    begin
      aCurs:= cr_Move_Point;
      FMovedCrdIndex:= I;
      break;
    end;
  end;

  if FMovedCrdIndex = -1 then
  for I:= 0 to K-1 do
  begin
    DejVztahyBod_Usecka(FActX, FActY,
                        FPoly.Xi[i], FPoly.Yi[i],
                        FPoly.Xi[i+1], FPoly.Yi[i+1],
                        PataUvnitr, PataX, PataY, Vzdalenost);

    if PataUvnitr and (Vzdalenost < aR) then
    begin
      aCurs:= cr_Insert_Point;
      FMovedCrdIndex:= I;
      FMovedBetween:= true;
      break;
    end;
  end;

  if Manager.FormCursor <> aCurs then
     Manager.FormCursor:= aCurs;
end;



function TMeasureLengthTool.GetLength: double;
begin
  result:= FPoly.Length;
end;



    { TMeasureAreaTool }


function  TMeasureAreaTool.GetCursor: TCursor;
begin
   result := cr_SelectArea;
end;



procedure TMeasureAreaTool.CreateMeasuredObject;
begin
  if FPoly <> nil then
  begin
     XORDraw;
     FreeElement(FPoly);
  end;

  FPoly:= TPolyLineElement.Create;
  FActualCrdIndex:= -1;
  FMovedCrdIndex:= -1;
  if FPoly = nil then exit;
  AddElement(FPoly);
//  FPoly.AddXY(0, 0);

end;



procedure TMeasureAreaTool.XORDraw;
var
  aR: double;
  I: integer;
  K: integer;
begin
  Device.Pen.Color:= FColor;
  Device.Pen.Style:= psSolid;
  Device.Pen.Mode:= pmXOR;
  Device.Pen.Width:= 1;
  Device.Brush.Style:= bsClear;
  if FPoly.Count < 3 then
     Device.Polyline(FPoly)
  else
     Device.Polygon(FPoly);

// vykreslovn znaek
  aR:= Device.Projection.GetModelLength(PixelR);
  K:= FPoly.CoordsCount-1;
  for I:= 0 to K do
  begin
    DrawRect(FPoly.Xi[i], FPoly.Yi[i], aR);
  end;

// Vykreslen vybranho bodu
  I:= FActualCrdIndex;
  aR:= Device.Projection.GetModelLength(PixelR - 1);
  if I >= 0 then
  begin
    DrawRect(FPoly.Xi[i], FPoly.Yi[i], aR);
  end;

// Vykreslen hlaviky
  aR:= Device.Projection.GetModelLength(PixelR + 2);

  if FHeaderAtTale then
     K:= 0;

  if K >= 0 then
  begin
    DrawRect(FPoly.Xi[i], FPoly.Yi[i], aR);

    if FCloseEnds then
    begin
      Device.MoveTo(FPoly.Xi[K]+2*aR, FPoly.Yi[K]+2*aR);
      Device.LineTo(FPoly.Xi[K]-2*aR, FPoly.Yi[K]-2*aR);
      Device.MoveTo(FPoly.Xi[K]-2*aR, FPoly.Yi[K]+2*aR);
      Device.LineTo(FPoly.Xi[K]+2*aR, FPoly.Yi[K]-2*aR);
    end
  end;

  PostMsgToOwners(tm_Repaint,nil,DontKill);
end;



function TMeasureAreaTool.GetLength: double;
begin
  result:= FPoly.Length;
end;


function TMeasureAreaTool.GetArea: double;
begin
  result:= abs(FPoly.Area1);
  if result < 1E-7 then
     result:= 0;
end;



procedure TMeasureAreaTool.SendInfo;
var
  aForm: TStandardForm;
begin
{
  if Device is TCanvasToBmpDevice then
  begin
    aForm:= TStandardForm(TCanvasToBMPDevice(Device).CanvasForm);
    if aForm is TStandardForm then
    begin
      aForm.Label3.Caption:= 'P: ';
      aForm.l_M.Caption:= DoubleToStr(Area, 10, 2);
      Application.ProcessMessages;
    end;
  end;
}
end;




BEGIN
   RegisterOrbisClass(TMeasureLengthTool);
   RegisterOrbisClass(TMeasureAreaTool);
end.
