unit t_FindBreakLineConnections;

interface
uses
   Elements,
   e_Proje,
   t_Topol,
   Tools;

type

TFindBreakLineConnections = class(TTool)
  private
    FByProjection: boolean;
    FProjection: TProjection;
    FConnectionsNumber: integer;
    FAllNodes: boolean;
    procedure SetByProjection(const Value: boolean);
    procedure SetProjection(const Value: TProjection);
    procedure SetConnectionsNumber(const Value: integer);
    procedure SetAllNodes(const Value: boolean);
public
   procedure DoIt; override;
published
   property ByProjection: boolean read FByProjection write SetByProjection;
   property Projection: TProjection read FProjection write SetProjection;
   property ConnectionsNumber: integer read FConnectionsNumber write SetConnectionsNumber;
   property AllNodes: boolean read FAllNodes write SetAllNodes;
end;

TFindSingleNodesInQTree = class(TProchazecListuQTreeGauge)
  private
    FConnectionsNumber: integer;
    procedure SetConnectionsNumber(const Value: integer);
public
  procedure Process(Element: TElement); override;
  property  ConnectionsNumber: integer read FConnectionsNumber write SetConnectionsNumber;
end;

TFindLinesInNodesForEntity = class(TTool)
  private
    FLinesEntity: TEntity;
    procedure SetLinesEntity(const Value: TEntity);
public
   procedure DoIt; override;
published
   property LinesEntity: TEntity read FLinesEntity write SetLinesEntity;
end;

implementation
uses
   Forms,
   Dialogs,
   SysUtils,
   e_Class,
   e_GeometryElement,
   e_Geometry,
   e_Node,
   e_QTree,
   g_Elems,
   t_ToTool;

{ TFindBreakLineConnections }

procedure TFindBreakLineConnections.DoIt;
var
  Tool: TReadLinesIntoQTree;
  DoNodes: TPutNodesIntoQTree;
//  TvorPlochy: TCreateAreasForQTree;
  NodeCon: TNodesConector;
  FindSN: TFindSingleNodesInQTree;
  RusLin: TDeleteDirectRelationsToLinesInQTree;
  S: string;
  QTree: TSpaceLineBegEndCell;
  R: TDoubleRect;
  RE: TRectEntity;
  i: integer;
  G: TGeometryElement;
  E: TElement;
  N: TNode;
begin
  if not (Input is TRectEntity) then
     Exit;
  if AllNodes then
  begin
     ConnectionsNumber := 0;
     S := 'Uzly';
  end
  else
  begin
     if ConnectionsNumber < 1 then
        ConnectionsNumber := 1;
     if ConnectionsNumber = 1 then
        S := 'Voln konce'
     else
        S := 'Uzly s potem '+IntToStr(ConnectionsNumber);
  end;
  if TEntity(Input).Count > 100 then
     StartProgress(S,0,5);
  RE := TRectEntity.Create;
  if ByProjection and (Projection <> nil) then
  begin
     R := Projection.ModelView;
     for i := 0 to TEntity(Input).LastIndex do
     begin
        G := TGeometryElement(TEntity(Input).Elements[i]);
        if not (G is TGeometryElement) then continue;
        if G.IsOverRegion(R.Minx,R.MinY,R.MaxX,R.MaxY) then
           RE.AddAndUpdateRect(G);
     end;
  end
  else
  begin
     RE.AddAllFrom := TEntity(Input);
     RE.ResetAndUpdate;
  end;
  if Output = nil then
  begin
     Output := TOwnerEntity.Create;
  end;

  S := 'Natn lini do QTREE';
  ProgressText := S;
  Tool:= TReadLinesIntoQTree.Create;
  Tool.Input:= RE;
  Tool.Name:= S;
  Tool.DoIt;
  Progress := 1;
  QTree:= TSpaceLineBegEndCell(Tool.Output);
  Tool.Free;

  S := 'Vytven uzl v QTREE';
  ProgressText := S;
  DoNodes:= TPutNodesIntoQTree.Create;
  DoNodes.Name:= S;
  DoNodes.Input:= QTree;
  DoNodes.DoIt;
  DoNodes.Free;
  Progress := 2;

  S := 'Zapojovn uzl v QTree';
  ProgressText := S;
  NodeCon:= TNodesConector.Create;
  NodeCon.Name:= S;
  NodeCon.Input:= QTree;
  NodeCon.DeletePseudoNodes:= false; //ReadIniBoolean('EXPORT', 'RusitPseudouzly', false);
  NodeCon.JoinDifferentLines:= false; //ReadIniBoolean('EXPORT', 'RusitPseudouzlyIProRuzneDruhy', true);
  NodeCon.MaxCoordsCount:= 200; //ReadIniInteger('EXPORT', 'MaxPocetSouradnic', 100);
  NodeCon.DoIt;
  NodeCon.Free;
  Progress := 3;

  // !!!Test nodes
  //World.ObjectModel.NextElement := QTree;
  //exit;
  // !!!Test end;

  S := 'Hledn osamlch smutnch uzl v QTree';
  ProgressText := S;
  FindSN:= TFindSingleNodesInQTree.Create;
  FindSN.Input:= QTree;
  FIndSN.Output := Output;
  FIndSN.ConnectionsNumber := ConnectionsNumber;
  FindSN.DoIt;
  FindSN.Free;
  Progress := 4;

  S := 'Ruen QTree vazeb na linie';
  ProgressText := S;
  RusLin:= TDeleteDirectRelationsToLinesInQTree.Create;
  RusLin.Name:= S;
  RusLin.Input:= QTree;
  RusLin.DoIt;
  RusLin.Free;
  Progress := 5;

  S := 'OK.';
  ProgressText := S;
  RE.Free;
  QTree.Free;
  if ByProjection and (Projection <> nil) then
  for i := TEntity(Output).LastIndex downto 0 do
  begin
     E := TEntity(Output).Elements[i];
     if not (E is TNode) then continue;
     N := TNode(E);
     if not ((N.X > R.Minx) and (N.X < R.MaxX)  and (N.Y > R.MinY) and (N.Y < R.MaxY)) then
        N.Free;
  end

end;



procedure TFindBreakLineConnections.SetAllNodes(const Value: boolean);
begin
  FAllNodes := Value;
end;

procedure TFindBreakLineConnections.SetByProjection(const Value: boolean);
begin
  FByProjection := Value;
end;

procedure TFindBreakLineConnections.SetConnectionsNumber(
  const Value: integer);
begin
  FConnectionsNumber := Value;
end;

procedure TFindBreakLineConnections.SetProjection(
  const Value: TProjection);
begin
  FProjection := Value;
end;

{ TFindSingleNodesInQTree }

procedure TFindSingleNodesInQTree.Process(Element: TElement);
var
  Nodes: TOwnerEntity;
  I, J: longInt;
  Node: TNode;
  Branch: TNodeBranch;

begin
   inc(FPocetListu);
   GaugeSet(FPocetListu);
 try
   Nodes:= TOwnerEntity(TEntity(Element).FindElementName('NODES'));
   if Nodes <> nil then
   begin
      for I:= Nodes.Count-1 downto 0 do
      begin
         if Nodes.Elements[I] is TNode then
         begin
            Node:= TNode(Nodes.Elements[I]);
            if (ConnectionsNumber = 0) or (Node.BranchCount = ConnectionsNumber) then
            begin
               if Output <> nil then
                  TEntity(Output).AddElement(Node);
            end;
         end
      end;
   end;
 except
   ShowMessage('Chyba v FindSingleNodesInQTree');
 end
end;

procedure TFindSingleNodesInQTree.SetConnectionsNumber(
  const Value: integer);
begin
  FConnectionsNumber := Value;
end;

{ TFindLinesInNodesForEntity }

procedure TFindLinesInNodesForEntity.DoIt;
var
   En,OE: TEntity;
   E: TElement;
   i: integer;
   L: TPolyLineElement;
   G: TGeometryElement;
   N: TNode;
begin
   if Input = nil then exit;
   if LinesEntity = nil then exit;
   if not (Input is TEntity) then exit;
   En := TEntity(Input);
   OE := TOwnerEntity.Create;
   Output := OE;
   if En.NoElements then exit;
   StartProgress('Extrakce lini z uzl ...',0,En.LastIndex);
   for i := 0 to En.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      E := En.Elements[i];
      if not (E is TNode) then continue;
      N := TNode(E);
      L := TPolyLineElement(N.FindElementClass(TPolyLineElement));
      if L = nil then continue;
      G := TGeometryElement(L.FindClassWithOwnerInOwners(TGeometryElement,LinesEntity));
      if G = nil then continue;
      if OE.IndexOf(G) < 0 then
         OE.AddElement(G);
   end;
end;

procedure TFindLinesInNodesForEntity.SetLinesEntity(const Value: TEntity);
begin
  FLinesEntity := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TFindBreakLineConnections);
   RegisterOrbisClass(TFindLinesInNodesForEntity);
END.
