unit t_GraphicEditors;

interface

uses
   Controls, Graphics,
   Elements,
   e_Geometry,
   e_GeometryElement,
   e_QTree,
   g_Elems,
   //i_Choice,
   m_Intera,
   t_FrameSelect,
   t_InterfaceTool,
   t_OrbisDialog,
   t_PickUpGeometry,
   t_Snap,
   t_QTreeTools,
   Tools;


const
  PixelR: integer = 3;

type

TGraphicEditorStatus = (gesPrepare,gesSelect,gesEdit,gesAdd,gesSetup);

TSimpleGraphicEditor = class(TInterfaceTool)
  private
    FY: double;
    FX: double;
    FOnMouseUpScript: string;
    FOnMouseDownScript: string;
    FMB: integer;
    procedure SetOnMouseDownScript(const Value: string);
    procedure SetOnMouseUpScript(const Value: string);
    procedure SetX(const Value: double);
    procedure SetY(const Value: double);
    procedure SetMB(const Value: integer);
public
   procedure ProcessMouseEvent(aMessage: TOrbisMouseEvent);
   procedure On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
published
   property X: double read FX write SetX;
   property Y: double read FY write SetY;
   property MB: integer read FMB write SetMB;
   property OnMouseDownScript: string read FOnMouseDownScript write SetOnMouseDownScript;
   property OnMouseUpScript: string read FOnMouseUpScript write SetOnMouseUpScript;
end;


TGraphicEditor = class(TFrameTool)
private
   FCanCreateNewObject: boolean;
   FSelectTool: TInterfaceTool;
   FStoreTool: TTool;
   FDialog: TOrbisDialog;
   FColor: TColor;
   FModified: boolean;
   FConfirmText: string;
   FEditorStatus: TGraphicEditorStatus;
   FQTFinder: TFindRectEntityInQTreeByXY;
   procedure SetCanCreateNewObject(const Value: boolean);
   procedure SetSelectTool(const Value: TInterfaceTool);
   procedure SetStoreTool(const Value: TTool);
   function GetNewObject: boolean;
   procedure SetDialog(const Value: TOrbisDialog);
    procedure SetModified(const Value: boolean);
    procedure SetConfirmText(const Value: string);
    procedure SetEditorStatus(const Value: TGraphicEditorStatus);
    function GetQTFinder: TFindRectEntityInQTreeByXY;
public
   procedure XORDraw; virtual;
   function  IsPossibleToDeactivate: boolean; override;
   function  FindLevel(aLevel: string): TEntity;
   function  FindLevelInEntity(anEntity: TEntity; aLevel: string): TEntity;
   procedure StartEditing; virtual;
   procedure CreateNewObject; virtual;
   function  CreateObjectCopy: TElement; virtual;
   procedure RemoveElementOwners(const anElement: TElement);
   procedure On_Activate  ; override;
   procedure On_tm_Cursor(var aMessage: TOrbisCursorEvent; var Handled: boolean); override;
   procedure On_tm_ElementSelected(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure StartFlashTool; override;
   procedure EndFlashTool; override;
   procedure BeforeFlashReview; override;
   procedure ExecuteDialog; virtual;
   property  QTFinder: TFindRectEntityInQTreeByXY read GetQTFinder;
   property  EditorStatus: TGraphicEditorStatus read FEditorStatus write SetEditorStatus;
published
   property Color: TColor read FColor write FColor;
   property Modified: boolean read FModified write SetModified stored false;
   property CanCreateNewObject: boolean read FCanCreateNewObject write SetCanCreateNewObject;
   property SelectTool: TInterfaceTool read FSelectTool write SetSelectTool stored false;
   property StoreTool: TTool read FStoreTool write SetStoreTool stored false;
   property NewObject: boolean read GetNewObject stored false;
   property Dialog: TOrbisDialog read FDialog write SetDialog;
   property ConfirmText: string read FConfirmText write SetConfirmText;
end;


TPolyElementEditor = class(TGraphicEditor)
private

   FActX, FActY: double;
    FSnapTool: TPointLineSnapTool;
    FSy: integer;
    FSx: integer;
    FSd: integer;
    FGElement: TGeometryElement;
    FedY: integer;
    FedX: integer;
    FPolyElementBackUp: TPolyElement;
    FFreeToChangeGeometry: boolean;
    FFreeToDelete: boolean;
    FAllowDeleteTool: TBooleanTool;
    FAllowChangeGeometryTool: TBooleanTool;
    FAllowSwapDirection: boolean;
    FNearLines: TQuickEntity;
    FSwap: boolean;
   procedure ChangeCursor; virtual;
   procedure DoOnMeasureClick(Sender: TObject); virtual;

    function GetPoly: TPolyElement;
    procedure SetPoly(const Value: TPolyElement);
    procedure SetSnapTool(const Value: TPointLineSnapTool);
    function GetSnapping: boolean;
    procedure SetGElement(const Value: TGeometryElement);
    procedure SetPolyElementBackUp(const Value: TPolyElement);
    procedure SetAllowDeleteTool(const Value: TBooleanTool);
    procedure SetAllowChangeGeometryTool(const Value: TBooleanTool);
    procedure SetFreeToDelete(const Value: boolean);
    procedure SetFreeToChangeGeometry(const Value: boolean);
    procedure SetAllowSwapDirection(const Value: boolean);
    function GetNearLines: TQuickEntity;
    procedure SetSwap(const Value: boolean);
protected
   FDragIsStarted: boolean;
   FActualCrdIndex: longInt;  // aktuln zpracovvan souadnice
   FMovedCrdIndex: integer;   // souadnice nad kterou se pohybuje my
   FMovedBetween: boolean;
   FHeaderAtTale: boolean;
   FCloseEnds: boolean;
public
   constructor Create; override;
   destructor  Destroy; override;
   function    IsPossibleToDeactivate: boolean; override;
   procedure   StartEditing; override;
   procedure   XORDraw; override;
   procedure   SendInfo; virtual;
   procedure   SnapDraw(aX,aY: integer);
   procedure   DrawRect(X, Y, HalfSize: double);
   procedure   CreateNewObject; override;
   function    CreateObjectCopy: TElement; override;
   procedure   RegisterCursors; override;
   procedure   DoStartDrag; virtual;
   function    NearTo(anIndex: integer): boolean;
   procedure   AddCrdToActualEnd; virtual;
   procedure   CreatePopUp(aX,aY: integer); virtual;
   function    ConfirmAndStore: boolean; virtual;
   procedure   RestoreObject; virtual;
   procedure   StoreObject; virtual;
   procedure   CheckAndStoreObject; virtual;
   procedure   DeleteObject; virtual;
   procedure   DeletePoint; virtual;
   procedure   CloseObject; virtual;
   procedure   SwapDirection; virtual;
   procedure   ExecuteDialog; override;
   procedure   FindNearLines; virtual;
   procedure   NextNearLine; virtual;
   procedure   On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;

   procedure   On_tm_ExternalDeviceDown(var aMessage: TOrbisExternalDeviceEvent; var Handled: boolean); override;


   procedure   On_tm_StartDrag(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;

   procedure   On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   procedure   On_Activate  ; override;
   procedure   On_DeActivate; override;
   procedure   On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   TestForDelete; virtual;
   procedure   TestForChangeGeometry; virtual;
   property    Swap: boolean read FSwap write SetSwap;
   property    PolyElement: TPolyElement read GetPoly write SetPoly;
   property    PolyElementBackUp: TPolyElement read FPolyElementBackUp write SetPolyElementBackUp;
   property    GElement: TGeometryElement read FGElement write SetGElement;
   property    ActX: double read FActX write FActX;
   property    ActY: double read FActY write FActY;
   property    Sx: integer read FSx write FSx;
   property    Sy: integer read FSy write FSy;
   property    Sd: integer read FSd write FSd;
   property    edX: integer read FedX write FedX;
   property    edY: integer read FedY write FedY;
   property    Snapping: boolean read GetSnapping;
   property    FreeToDelete: boolean read FFreeToDelete write SetFreeToDelete;
   property    FreeToChangeGeometry: boolean read FFreeToChangeGeometry write SetFreeToChangeGeometry;
   property    NearLines: TQuickEntity read GetNearLines;
published
   property SnapTool: TPointLineSnapTool read FSnapTool write SetSnapTool;
   property AllowDeleteTool: TBooleanTool read FAllowDeleteTool write SetAllowDeleteTool;
   property AllowChangeGeometryTool: TBooleanTool read FAllowChangeGeometryTool write SetAllowChangeGeometryTool;
   property AllowSwapDirection: boolean read FAllowSwapDirection write SetAllowSwapDirection;
end;


TPolyLineEditor = class(TPolyElementEditor)
private
   function GetLength: double;
protected
//  procedure ChangeCursor; override;
public
   procedure XORDraw; override;
   procedure CreateNewObject; override;
   function  GetCursor: TCursor; override;
   procedure On_tm_DragDrop(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
published
   property Length: double read GetLength stored false;
end;

TPolyLineEditorNoFrame = class(TPolyLineEditor)
private
    FLevelName: string;
    FLevel: TEntity;
    FLevelQT: TRectEntityQTree;
    FPixelTolerance: integer;
    FTopology: boolean;
    FSnapQT: TRectEntityQTree;
    FSnapLevels: string;
    FAuxLevelName: string;
    FAuxProjectsName: string;
    FAuxProjects: TEntity;
    FAuxSnapLevels: string;
    FActualProjectName: string;
    FOnProjectChangeScript: string;
    FActualLevel: TEntity;
    FActualProject: TEntity;
    procedure SetLevel(const Value: TEntity);
    procedure SetLevelName(const Value: string);
    procedure SetLevelQT(const Value: TRectEntityQTree);
    procedure SetPixelTolerance(const Value: integer);
    procedure SetTopology(const Value: boolean);
    procedure SetSnapQT(const Value: TRectEntityQTree);
    procedure SetSnapLevels(const Value: string);
    procedure SetAuxLevelName(const Value: string);
    procedure SetAuxProjectsName(const Value: string);
    procedure SetAuxProjects(const Value: TEntity);
    procedure SetAuxSnapLevels(const Value: string);
    procedure SetActualProjectName(const Value: string);
    procedure SetOnProjectChangeScript(const Value: string);
    procedure SetActualLevel(const Value: TEntity);
    procedure SetActualProject(const Value: TEntity);
public
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_Activate; override;
   procedure SendProjectChange;
   procedure RegisterCursors; override;
   procedure DeleteObject; override;
   procedure StoreObject; override;
   procedure NextNearLine; override;
   procedure ClearTopology(aLevel: TEntity; Rel,R:TRectEntity); virtual;
   procedure PrepareQTree; virtual;
   procedure PrepareSnapQTree; virtual;
   procedure FindLevel_ProjectEntity(aElem: TElement; var aLevel,aProjE: TEntity);
   procedure CHangeSelectCursor;
   property LevelQT: TRectEntityQTree read FLevelQT write SetLevelQT;
   property SnapQT: TRectEntityQTree read FSnapQT write SetSnapQT;
   property Level: TEntity read FLevel write SetLevel;
   property ActualLevel: TEntity read FActualLevel write SetActualLevel;
   property ActualProject: TEntity read FActualProject write SetActualProject;
   property AuxProjects: TEntity read FAuxProjects write SetAuxProjects;
published
   property LevelName: string read FLevelName write SetLevelName;
   property SnapLevels: string read FSnapLevels write SetSnapLevels;
   property PixelTolerance: integer read FPixelTolerance write SetPixelTolerance;
   property Topology: boolean read FTopology write SetTopology;
   property AuxProjectsName: string read FAuxProjectsName write SetAuxProjectsName;
   property AuxLevelName: string read FAuxLevelName write SetAuxLevelName;
   property AuxSnapLevels: string read FAuxSnapLevels write SetAuxSnapLevels;
   property ActualProjectName: string read FActualProjectName write SetActualProjectName;
   property OnProjectChangeScript: string read FOnProjectChangeScript write SetOnProjectChangeScript;
end;

TShapeEditor = class(TPolyElementEditor)
private
   function GetLength: double;
   function GetArea: double;
public
    procedure XORDraw; override;
    procedure CreateNewObject; override;
    procedure SendInfo; override;
published
   function   GetCursor: TCursor; override;
   property Length: double read GetLength stored false;
   property Area: double read GetArea stored false;
end;

TPolyLineAreaEditor = class(TGraphicEditor)
  private
    FY: double;
    FX: double;
    FCopyLinesLevels: string;
    FWorkingLinesLevels: string;
    FAreas: TRectEntity;
    FLines: TRectEntity;
    FArea: TAreaElement;
    FLevelName: string;
    FFrame: TEntity;
    FNeighbourhoodWorkingLinesLevels: string;
    FNeighbourhoodCopyLinesLevels: string;
    FNeighbourhoodTool: TTool;
    FNeighbourhoodFrames: TEntity;
    procedure SetArea(const Value: TAreaElement);
    procedure SetAreas(const Value: TRectEntity);
    procedure SetCopyLinesLevels(const Value: string);
    procedure SetLines(const Value: TRectEntity);
    procedure SetWorkingLinesLevels(const Value: string);
    procedure SetX(const Value: double);
    procedure SetY(const Value: double);
    procedure SetLevelName(const Value: string);
    procedure SetFrame(const Value: TEntity);
    procedure SetNeighbourhoodCopyLinesLevels(const Value: string);
    procedure SetNeighbourhoodTool(const Value: TTool);
    procedure SetNeighbourhoodWorkingLinesLevels(const Value: string);
    procedure SetNeighbourhoodFrames(const Value: TEntity);
protected
   function    FindFrame: boolean;
   function    FindArea: boolean;
   procedure   FindLinesInFrame(aFrame: TEntity; WorkLevels, CopyLevels: string);
   procedure   FindLines;
   procedure   MakeArea;
   procedure   ClearLines;
public
   procedure   XORDraw; override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   RegisterCursors; override;
   property    Area: TAreaElement read FArea write SetArea;
   property    X: double read FX write SetX;
   property    Y: double read FY write SetY;
   property    Lines: TRectEntity read FLines write SetLines;
   property    Areas: TRectEntity read FAreas write SetAreas;
   property    Frame: TEntity read FFrame write SetFrame;
   property    NeighbourhoodFrames: TEntity read FNeighbourhoodFrames write SetNeighbourhoodFrames;
published
   property    LevelName: string read FLevelName write SetLevelName;
   property    WorkingLinesLevels: string read FWorkingLinesLevels write SetWorkingLinesLevels;
   property    CopyLinesLevels: string read FCopyLinesLevels write SetCopyLinesLevels;
   property    NeighbourhoodTool: TTool read FNeighbourhoodTool write SetNeighbourhoodTool;
   property    NeighbourhoodWorkingLinesLevels: string read FNeighbourhoodWorkingLinesLevels write SetNeighbourhoodWorkingLinesLevels;
   property    NeighbourhoodCopyLinesLevels: string read FNeighbourhoodCopyLinesLevels write SetNeighbourhoodCopyLinesLevels;
end;

TPolyLineAreaEditorNoFrame = class(TGraphicEditor)
  private
    FY: double;
    FX: double;
    FNewAreasLevel: string;
    FOldAreasLevel: string;
    FArea: TAreaElement;
    FNewAreas: TEntity;
    FOldAreas: TEntity;
    FGArea: TGeometryElement;
    FLinesLevel: string;
    FLines: TEntity;
    FKillGeometryOnly: boolean;
    FFrameAreasLevel: string;
    FFrameAreas: TEntity;
    procedure SetX(const Value: double);
    procedure SetY(const Value: double);
    procedure SetArea(const Value: TAreaElement);
    procedure SetNewAreasLevel(const Value: string);
    procedure SetOldAreasLevel(const Value: string);
    procedure SetNewAreas(const Value: TEntity);
    procedure SetOldAreas(const Value: TEntity);
    procedure SetGArea(const Value: TGeometryElement);
    procedure SetLinesLevel(const Value: string);
    procedure SetLines(const Value: TEntity);
    procedure SetKillGeometryOnly(const Value: boolean);
    procedure SetFrameAreasLevel(const Value: string);
    procedure SetFrameAreas(const Value: TEntity);
protected
   function    FindArea: boolean;
   procedure   MakeArea;
public
   procedure   XORDraw; override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   RegisterCursors; override;
   procedure   On_Activate  ; override;
   procedure   PrepareAreas;
   property    Area: TAreaElement read FArea write SetArea;
   property    GArea: TGeometryElement read FGArea write SetGArea;
   property    X: double read FX write SetX;
   property    Y: double read FY write SetY;
   property    OldAreas: TEntity read FOldAreas write SetOldAreas;
   property    NewAreas: TEntity read FNewAreas write SetNewAreas;
   property    FrameAreas: TEntity read FFrameAreas write SetFrameAreas;
   property    Lines: TEntity read FLines write SetLines;
published
   property    OldAreasLevel: string read FOldAreasLevel write SetOldAreasLevel;
   property    NewAreasLevel: string read FNewAreasLevel write SetNewAreasLevel;
   property    FrameAreasLevel: string read FFrameAreasLevel write SetFrameAreasLevel;
   property    LinesLevel: string read FLinesLevel write SetLinesLevel;
   property    KillGeometryOnly: boolean read FKillGeometryOnly write SetKillGeometryOnly;
end;



TPointEditor = class(TGraphicEditor)
private
   FIsDraged: boolean;
    FLevelName: string;
    FLevel: TEntity;
    FLevelQT: TRectEntityQTree;
    FPixelTolerance: integer;
    FY: double;
    FX: double;
    FOldY: double;
    FOldX: double;
    FOldAngle: double;
    FActY: double;
    FActX: double;
    procedure SetLevel(const Value: TEntity);
    procedure SetLevelName(const Value: string);
    procedure SetLevelQT(const Value: TRectEntityQTree);
    procedure SetPixelTolerance(const Value: integer);
    function  GetPoint: TGeometryElement;
    procedure SetPoint(const Value: TGeometryElement);
    procedure SetX(const Value: double);
    procedure SetY(const Value: double);
    procedure SetInterpretName(const Value: string);
    function GetInterpretName: string;
    procedure SetOldX(const Value: double);
    procedure SetOldY(const Value: double);
    procedure SetOldAngle(const Value: double);
    procedure SetActX(const Value: double);
    procedure SetActY(const Value: double);
public
   procedure   XORDraw; override;
   procedure   On_Activate; override;
   procedure   StorePoint(DontShowDialog: boolean);
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_StartDrag(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_DragDrop(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_Resize(var Handled: boolean); override;
   procedure   RegisterCursors; override;
   procedure   PrepareQTree;
   procedure   CHangeSelectCursor;
   procedure   ChangeAngle;
   Property    IsDraged: boolean read FIsDraged write FIsDraged;
   property    LevelQT: TRectEntityQTree read FLevelQT write SetLevelQT;
   property    Level: TEntity read FLevel write SetLevel;
   property    X: double read FX write SetX;
   property    Y: double read FY write SetY;
   property    ActX: double read FActX write SetActX;
   property    ActY: double read FActY write SetActY;
   property    OldX: double read FOldX write SetOldX;
   property    OldY: double read FOldY write SetOldY;
   property    OldAngle: double read FOldAngle write SetOldAngle;
   property    Point: TGeometryElement read GetPoint write SetPoint;
published
   property  InterpretName: string read GetInterpretName write SetInterpretName;
   property  LevelName: string read FLevelName write SetLevelName;
   property  PixelTolerance: integer read FPixelTolerance write SetPixelTolerance;
end;


implementation
uses
  WinTypes, Forms, Classes, Menus,
  SysUtils, Dialogs,
// KERNEL
  m_Orbis,
  m_Proj,
  U_String,
  //F_Std, D_Std,
  e_Class,
  e_Device,
  //e_TplPrp,
  //u_Cursor,
  i_Basic,
  Worlds,
  t_Aremak,
  t_LineExtractor,
  t_CmdLn,
// APP
   t_ConfirmDialog,
// GLIB
  //G_Elems,
  //TEST
  t_Paral,
  MathLib
  ;

const
  crSelectLine = 0;
  crMovePoint  = 1;
  crInsertPoint  = 2;



{ TPolyElementEditor }

constructor TPolyElementEditor.Create;
begin
   inherited;
   CreateNewObject;
   FColor:= clSilver;
   EditorStatus := gesEdit;
end;


procedure TPolyElementEditor.SendInfo;
begin
   if PolyElement <> nil then
      PostTextMsgToOwners(tm_LengthChanged,FloatToStr(PolyElement.Length) + ':' + FloatToStr(PolyElement.LastLength))
   else
      PostTextMsgToOwners(tm_LengthChanged,'0:0');
end;

procedure TPolyElementEditor.DrawRect(X, Y, HalfSize: double);
begin
  Device.MoveTo(X+HalfSize, Y+HalfSize);
  Device.LineTo(X-HalfSize, Y+HalfSize);
  Device.LineTo(X-HalfSize, Y-HalfSize);
  Device.LineTo(X+HalfSize, Y-HalfSize);
  Device.LineTo(X+HalfSize, Y+HalfSize);
end;




function  TPolyElementEditor.NearTo(anIndex: integer): boolean;
var
  aR: double;
begin
  aR:= Device.Projection.GetModelLength(PixelR);
  result:= InTolerance(PolyElement.Xi[anIndex], PolyElement.Yi[anIndex], FActX, FActY, 2*aR)
end;



procedure TPolyElementEditor.DoOnMeasureClick(Sender: TObject);
begin
  if PolyElement = nil then
     Exit;
  //if not (input is TRectEntity) then
  //   Exit;
  PolyElement.RecalculateRect;
  PolyElement.CalculateLabelProps;
  if StoreTool <> nil then
  begin
     StoreTool.Input := PolyElement;
     StoreTool.DoIt;
  end;
  //TRectEntity(Input).AddAndUpdateRect(PolyElement);
  //PolyElement:= nil;
  XORDraw;
  PostMsgToOwners(tm_Review, PolyElement, false);
  Application.ProcessMessages;
  PolyElement := nil;
  On_Activate;
  //CreateNewObject;
end;


procedure TPolyElementEditor.CreatePopUp(aX,aY: integer);
var
  aPopUp: TPopUpMenu;
  aMenu1, aMenu2, aMenu3: TMenuItem;
begin
  aPopUp:= TPopUpMenu.Create(Application);
  aPopUp.AutoPopUp:= false;

  aMenu1:= TMenuItem.Create(aPopUp);
  aMenu2:= TMenuItem.Create(aPopUp);
  aMenu1.Caption:= 'X='+FloatToStr(FActX);
  aMenu2.Caption:= 'Y='+FloatToStr(FActY);
  aPopUp.Items.Add(aMenu1);
  aPopUp.Items.Add(aMenu2);

  aMenu3:= TMenuItem.Create(aPopUp);
  aMenu3.Caption:= 'Potvrzen';
  aMenu3.OnClick:= DoOnMeasureClick;
  aPopUp.Items.Add(aMenu3);

  aPopUp.PopUp(aX, aY);
end;




procedure TPolyElementEditor.XORDraw;
var
   i: integer;
begin
   if PolyElement = nil then
      Exit;
   if PolyElement.CoordsCount < 2 then exit;
   Device.Pen.Color:= FColor;
   Device.Pen.Style:= psSolid;
   Device.Pen.Mode:= pmXOR;
   Device.Pen.Width:= 3;
   Device.Brush.Style:= bsClear;
   //Device.Polyline(PolyElement)
   Device.MoveTo(PolyElement.Xi[0], PolyElement.Yi[0]);
   for i := 1 to PolyElement.LastIndex do
       Device.LineTo(PolyElement.Xi[i], PolyElement.Yi[i]);
end;



procedure TPolyElementEditor.On_tm_StartDrag(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
  FDragIsStarted:= true;
end;



procedure TPolyElementEditor.AddCrdToActualEnd;
begin
  if PolyElement = nil then
     Exit;
  try
    if PolyElement.CoordsCount = 0 then
    begin
      PolyElement.AddXY(FActX, FActY);
      FActualCrdIndex:= 0;
      XORDraw;
    end
    else
    begin
      XORDraw;
      if FHeaderAtTale then
      begin
        PolyElement.InsertXY(0, FActX, FActY);
        FActualCrdIndex:= 0;
      end
      else
      begin
        PolyElement.AddXY(FActX, FActY);
        FActualCrdIndex:= PolyElement.CoordsCount-1;
      end;
      XORDraw;
    end
  except
    on E: Exception do
     ShowMessage('Zahjen snmn: ' + E.Message);
  end
end;




procedure TPolyElementEditor.DoStartDrag;
begin
  if FMovedCrdIndex = -1 then exit;

  if not FMovedBetween then
     FActualCrdIndex:= FMovedCrdIndex
  else
  begin
    XORDraw;
    PolyElement.InsertXY(FMovedCrdIndex+1, FActX, FActY);
    FActualCrdIndex:= FMovedCrdIndex + 1;
    XORDraw;
  end;
end;



procedure TPolyElementEditor.On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
  aEndIndex: integer;
begin
  if FDragIsStarted then
     DoStartDrag;
  FDragIsStarted:= false;
  if PolyElement = nil then exit;

  try
   FActX:= Device.Projection.GetModelX(aMessage.x);
   FActY:= Device.Projection.GetModelY(aMessage.y);
   if Snapping then
   begin
      SnapDraw(aMessage.X,aMessage.Y);
      SnapTool.X := ActX;
      SnapTool.Y := ActY;
      SnapTool.DoIt;
      if SnapTool.Snapped then
      begin
         ActX := SnapTool.SnapX;
         ActY := SnapTool.SnapY;
         case SnapTool.SnapStatus of
            1: CursorIndex := 3;
            2: CursorIndex := 4;
         end;
      end
      else
         CursorIndex := 1;
   end;
   XORDraw;

   FCloseEnds:= false;
   if FActualCrdIndex in [0, PolyElement.CoordsCount-1] then
   begin
     aEndIndex:= 0;
     if FHeaderAtTale then
        aEndIndex:= PolyElement.CoordsCount-1;

     if NearTo(aEndIndex) then
     begin
       FCloseEnds:= true;
     end;
     // PPK 14.10.2002 nsledujc dky za podmnkou zpsobovaly exception
     PolyElement.Xi[FActualCrdIndex]:= FActX;
     PolyElement.Yi[FActualCrdIndex]:= FActY;
     Modified := true;
   end;

   // PPK 14.10.2002 nsledujc dky za podmnkou zpsobovaly exception
   if FActualCrdIndex > -1 then
   begin
       PolyElement.Xi[FActualCrdIndex]:= FActX;
       PolyElement.Yi[FActualCrdIndex]:= FActY;
       Modified := true;
   end;
   XORDraw;

   PostMsg(tm_ToolChange, nil, nil, true);

   SendInfo;
  except
    on E: Exception do
     ShowMessage('Men: ' + E.Message);
  end;
  Handled:= true
end;




procedure TPolyLineEditor.On_tm_DragDrop(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
  aFixIndex, aMoveIndex: integer;
begin
  if PolyElement = nil then exit;
  try
    if Snapping then
       SnapDraw(0,0);
    if FCloseEnds then
    begin
      if FHeaderAtTale then
      begin
        aFixIndex:= PolyElement.CoordsCount-1;
        aMoveIndex:= 0;
      end
      else
      begin
        aFixIndex:= 0;
        aMoveIndex:= PolyElement.CoordsCount-1;
      end;
      XOrDraw;
      PolyElement.Xi[aMoveIndex]:= PolyElement.Xi[aFixIndex];
      PolyElement.Yi[aMoveIndex]:= PolyElement.Yi[aFixIndex];
      XOrDraw;
    end;
//    SendInfo;
  except
    on E: Exception do
     ShowMessage('Men: ' + E.Message);
  end;
end;



procedure TPolyElementEditor.On_Activate;
begin
   inherited;
   Active := true;
   //Application.ProcessMessages;
   //XORDraw;
end;


procedure TPolyElementEditor.On_DeActivate;
//var
//  aForm: TStandardForm;
begin
  Active := false;
  if SnapTool <> nil then
     SnapTool.FrameQT := nil;
  Application.ProcessMessages;
  if PolyElement <> nil then XORDraw;

{
  if Device is TCanvasToBmpDevice then
  begin
    aForm:= TStandardForm(TCanvasToBMPDevice(Device).CanvasForm);
    if aForm is TStandardForm then
    begin
      aForm.Label3.Caption:= 'M 1 :';
    end;
  end;
}
end;


procedure TPolyElementEditor.On_tm_ReView(var aMessage: TOrbisMessage; var Handled: boolean);
begin
  Application.ProcessMessages;
  Sx := 0;
  Sy := 0;
  XORDraw;
end;


procedure TPolyElementEditor.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
  if EditorStatus = gesPrepare then exit;
  if (aMessage.Button = mbRight) and (EditorStatus = gesEdit) then
  begin
     ExecuteDialog;
    {
    if (FMovedCrdIndex > -1) then
    begin
      CreatePopUp(aMessage.X,aMessage.Y);
      exit;
    end
    else
    begin
    end
    }
  end;

  if aMessage.Button <> mbLeft then
     exit;

  Handled:= false;
  if Snapping then
     SnapDraw(aMessage.X,aMessage.Y);

  if FMovedCrdIndex = -1 then
  begin
    AddCrdToActualEnd;
    PostMsg(tm_ToolChange, nil, nil, true);
    Modified := true;
    SendInfo;
    exit;
  end;

  if not FMovedBetween and (FActualCrdIndex <> FMovedCrdIndex) then
  begin
    XOrDraw;
    FActualCrdIndex:= FMovedCrdIndex;
    if FActualCrdIndex = 0 then
       FHeaderAtTale:= true
    else
    if PolyElement.CoordsCount-1 = FActualCrdIndex then
       FHeaderAtTale:= false;
    XOrDraw;
    PostMsg(tm_ToolChange, nil, nil, true);
 end
end;



procedure TPolyElementEditor.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
  aX, aY: integer;
begin
  if EditorStatus = gesPrepare then exit;
  aX:= aMessage.x;
  aY:= aMessage.y;
  FActX:= Device.Projection.GetModelX(aX);
  FActY:= Device.Projection.GetModelY(aY);

  if not Active then exit;
  if PolyElement = nil then exit;
  ChangeCursor;
end;


procedure TPolyElementEditor.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   if EditorStatus = gesPrepare then exit;
   if EditorStatus <> gesEdit then exit;
   if Snapping then
      SnapDraw(0,0);
end;




procedure TPolyElementEditor.On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean);
begin
   if EditorStatus = gesPrepare then exit;
   if not (EditorStatus = gesEdit) then
   begin
      case aMessage.Key of
         VK_ESCAPE:
         begin
            XORDraw;
            PolyElement := nil;
            GElement := nil;
            PolyElementBackUp := nil;
            Modified := false;
            Swap := false;
            On_Activate;
            Application.ProcessMessages;
            Application.ProcessMessages;
          end;
          VK_RETURN:
          begin
             if ssCtrl in aMessage.Shift then
             begin
                CheckAndStoreObject;
             end;
          end;
      end;
      exit;
   end;
   case aMessage.Key of
      VK_SHIFT:
      begin
         if SnapTool <> nil then
            SnapTool.SnapToLineBody := not SnapTool.SnapToLineBody;
         exit;
      end;
      VK_SPACE:
      begin
         if SnapTool <> nil then
            SnapTool.SnapToLine := not SnapTool.SnapToLine;
         exit;
      end;
    end;

   if (PolyElement.CoordsCount = 0) then exit;

   case aMessage.Key of
      VK_Back:
      begin
        if (PolyElement.CoordsCount > 1) then
        begin
          XORDraw;
          if FHeaderAtTale then
          begin
            PolyElement.FreeElement(PolyElement.Elements[0]);
            FActualCrdIndex:= 0;
            Modified := true;
          end
          else
          begin
            PolyElement.FreeElement(PolyElement.Elements[PolyElement.CoordsCount-1]);
            FActualCrdIndex:= PolyElement.CoordsCount-1;
            Modified := true;
          end;
          XORDraw;
          SendInfo;
        end;
      end;
      VK_DELETE:
      begin
        if (ssCtrl in aMessage.Shift) then
        // vyprzdnn cel linie
        begin
           DeleteObject;
           SendInfo;
        end
        else
        if (PolyElement.CoordsCount > 1) and (FActualCrdIndex > -1) then
        begin
          XORDraw;
          PolyElement.FreeElement(PolyElement.Elements[FActualCrdIndex]);
          if (FActualCrdIndex >= PolyElement.CoordsCount) then
             FActualCrdIndex:= PolyElement.CoordsCount-1;
          Modified := true;
          XORDraw;
          SendInfo;
        end;
      end;
      VK_Insert:
      begin

      end;
      ord('R'), ord('r'):
      begin
        XORDraw;
      end;
      VK_ESCAPE:
         ConfirmAndStore;
      VK_RETURN:
      begin
         if ssCtrl in aMessage.Shift then
         begin
            if ssShift in aMessage.Shift then
               CloseObject;
            if ssAlt in aMessage.Shift then
               SwapDirection;
            {
            if AllowSwapDirection then
               SwapDirection
            else
               CloseObject;
             }
            CheckAndStoreObject;
         end;
      end;
      VK_TAB:
      begin
         if not (modified or CanCreateNewObject) then
         begin
            StoreObject;
            NextNearLine;
         end;
      end;
   end;
end;


procedure TPolyElementEditor.RegisterCursors;
begin
   NewCursorName('SELECTLINE');
   NewCursorName('MOVE_POINT');
   NewCursorName('INSERT_POINT');
   NewCursorName('SNAP_POINT');
end;





procedure TPolyElementEditor.CreateNewObject;
begin

end;

function TPolyElementEditor.CreateObjectCopy: TElement;
var
   i: integer;
   P: TPolyElement;
begin
   result := CreateElementCopy(PolyElement);
   P := TPolyElement(result);
   for i := 0 to PolyElement.CoordsCount - 1 do
   begin
      P.AddXY(PolyElement.Xi[i],PolyElement.Yi[i]);
   end;
end;

function TPolyElementEditor.GetPoly: TPolyElement;
begin
   result := TPolyElement(Output);
end;

procedure TPolyElementEditor.SetPoly(const Value: TPolyElement);
begin
   ExchangeFieldElement(TElement(FOutput), Value);
end;

procedure TPolyElementEditor.CloseObject;
begin
   if PolyElement.Count < 3 then exit;
   XORDraw;
   PolyElement.AddXY(PolyElement.Xi[0], PolyElement.Yi[0]);
   Modified := true;
   XORDraw;
end;

procedure TPolyElementEditor.DeleteObject;
begin
   CreateNewObject;
   PostMsg(tm_ModelChanged,nil,nil,false);
   Application.ProcessMessages;
   if not NewObject then
      on_Activate;
end;

procedure TPolyElementEditor.DeletePoint;
begin
   if (PolyElement.CoordsCount > 1) and (FActualCrdIndex > -1) then
   begin
      XORDraw;
      PolyElement.FreeElement(PolyElement.Elements[FActualCrdIndex]);
      if (FActualCrdIndex >= PolyElement.CoordsCount) then
         FActualCrdIndex:= PolyElement.CoordsCount-1;
      Modified := true;
      XORDraw;
   end;
end;

procedure TPolyElementEditor.StoreObject;
begin
  if PolyElement <> nil then
  begin
     //if not (input is TRectEntity) then
     //   Exit;
     PolyElement.RecalculateRect;
     PolyElement.CalculateLabelProps;
     if StoreTool <> nil then
     begin
        if (PolyElement <> nil) and (PolyElement.Count > 1) then
        begin
           StoreTool.Input := PolyElement;
           StoreTool.DoIt;
           PostMsg(tm_ModelChanged,nil,nil,false);
        end;
     end;
     //TRectEntity(Input).AddAndUpdateRect(PolyElement);
     //PolyElement:= nil;
     XORDraw;
     PostMsgToOwners(tm_Review, PolyElement, false);
  end;
  PolyElement := nil;
  Modified := false;
  Swap := false;
  On_Activate;
  Application.ProcessMessages;
  Application.ProcessMessages;
  //CreateNewObject;
end;

procedure TPolyElementEditor.StartEditing;
begin
   PostMsgToOwners(tm_ReviewByElement, PolyElement, false);
   Application.ProcessMessages;
   Application.ProcessMessages;
end;

function TPolyElementEditor.IsPossibleToDeactivate: boolean;
var
   D: TOrbisConfirmDialog;
begin
   result := false;
   if EditorStatus = gesPrepare then  exit;
   result := ConfirmAndStore;
end;

procedure TPolyElementEditor.SetSnapTool(const Value: TPointLineSnapTool);
begin
   ExchangeFieldTool(TTool(FSnapTool), Value);
end;

procedure TPolyElementEditor.SnapDraw(aX, aY: integer);
begin
   Device.Pen.Color:= Color;
   Device.Pen.Style:= psDot;
   Device.Pen.Mode:=  pmXOR;
   Device.Pen.Width:= 1;
   Device.Brush.Style:= bsClear;
   if (Sx or Sy) <> 0 then
      TCanvasDevice(Device).Canvas.Ellipse(Sx-Sd,Sy-Sd,Sx+Sd,Sy+Sd)
   else
   begin
      Sd := Device.Projection.GetPageLength(SnapTool.SnapDistance);
   end;
   Sx := aX;
   Sy := aY;
   if (Sx or Sy) <> 0 then
      TCanvasDevice(Device).Canvas.Ellipse(Sx-Sd,Sy-Sd,Sx+Sd,Sy+Sd);
end;

function TPolyElementEditor.GetSnapping: boolean;
begin
   result := false;
   if SnapTool <> nil then
   begin
      result := SnapTool.SnapToLine and (SnapTool.SnapToLinePoint or SnapTool.SnapToLineBody);
   end;
end;

procedure TPolyElementEditor.SetGElement(const Value: TGeometryElement);
begin
   ExchangeFieldElement(TElement(FGElement), Value);
   if Value <> nil then
      PolyElement := TPolyElement(Value.Geometry);
end;

procedure TPolyElementEditor.On_tm_ExternalDeviceDown(var aMessage: TOrbisExternalDeviceEvent; var Handled: boolean);
begin
   {
   if Snapping then
      SnapDraw(aMessage.X,aMessage.Y);
   }
   if EditorStatus = gesPrepare then exit;
   case aMessage.Button of
      1:
      begin
         ActX := aMessage.X;
         ActY := aMessage.Y;
         if Snapping then
         begin
            SnapTool.X := ActX;
            SnapTool.Y := ActY;
            SnapTool.DoIt;
            if SnapTool.Snapped then
            begin

               ActX := SnapTool.SnapX;
               ActY := SnapTool.SnapY;
               case SnapTool.SnapStatus of
                  1: MessageBeep(MB_ICONASTERISK);
                  2: MessageBeep(MB_ICONEXCLAMATION);
               end;
               //CursorIndex := aCurs;
            end;
         end;
         AddCrdToActualEnd;
         PostMsg(tm_ToolChange, nil, nil, true);
         Modified := true;
      end;
      2: DeletePoint;
      3: DeleteObject;
      4: CheckAndStoreObject;
   end;
end;

procedure TPolyElementEditor.SetPolyElementBackUp(const Value: TPolyElement);
begin
   ExchangeFieldElement(TElement(FPolyElementBackUp),Value);
   FActualCRDIndex := -1;
end;

procedure TPolyElementEditor.RestoreObject;
var
   i,iL: integer;
   E: TEntity;
   A: TPolyLineAreaElement;
begin
   if NewObject then
   begin
      PolyElement := nil;
      exit;
   end;
   if PolyElementBackUp = nil then exit;
   PolyElement.FreeAll;
   for i := 0 to PolyElementBackUp.CoordsCount - 1 do
   begin
      PolyElement.AddXY(PolyElementBackUp.Xi[i],PolyElementBackUp.Yi[i]);
   end;
   // otoceni v plochach  - vraceni
   if Swap then
   for i := 0 to PolyElement.OwnersCount - 1 do
   begin
      E := PolyElement.Owners[i];
      if E is TPolyLineAreaElement then
      begin
         A := TPolyLineAreaElement(E);
         iL := A.IndexOf(PolyElement);
         A.LinesOrigines[iL] := not A.LinesOrigines[iL]
      end;
   end;
   Modified := false;
   Swap := false;
end;

function TPolyElementEditor.ConfirmAndStore: boolean;
begin
   result := true;
   if Modified then
   begin
      case Confirm(Name,ConfirmText) of
      1: CheckAndStoreObject;
      3: begin
            XORDraw;
            RestoreObject;
            XORDraw;
            StoreObject;
         end;
      else
         result := false;
      end;
   end
   else
      StoreObject;
end;

procedure TPolyElementEditor.SetAllowDeleteTool(const Value: TBooleanTool);
begin
   ExchangeFieldTool(TTool(FAllowDeleteTool), Value);
end;

procedure TPolyElementEditor.SetAllowChangeGeometryTool(const Value: TBooleanTool);
begin
   ExchangeFieldTool(TTool(FAllowChangeGeometryTool), Value);
end;

procedure TPolyElementEditor.SetFreeToDelete(const Value: boolean);
begin
  FFreeToDelete := Value;
end;

procedure TPolyElementEditor.SetFreeToChangeGeometry(const Value: boolean);
begin
  FFreeToChangeGeometry := Value;
end;

procedure TPolyElementEditor.TestForDelete;
begin
   FreeToDelete := true;
   FreeToCHangeGeometry := true;
   if NewObject then exit;
   if AllowDeleteTool <> nil then
   begin
      AllowDeleteTool.Input := PolyElement;
      AllowDeleteTool.DoIt;
      FreeToDelete := AllowDeleteTool.result;
      FreeToChangeGeometry := FreeToDelete;
   end;
end;

procedure TPolyElementEditor.TestForChangeGeometry;
begin
   if not FreeToDelete then exit;
   FreeToChangeGeometry := true;
   if NewObject then exit;
   if AllowChangeGeometryTool <> nil then
   begin
      AllowChangeGeometryTool.Input := PolyElement;
      AllowChangeGeometryTool.DoIt;
      FreeToChangeGeometry := AllowChangeGeometryTool.result;
   end;
end;

procedure TPolyElementEditor.CheckAndStoreObject;
begin
   if Dialog <> nil then
   begin
      if GElement <> nil then
         Dialog.Input := GElement
      else
         Dialog.Input := PolyElement;
      Dialog.DontShowForm := true;
      Dialog.DoIt;
      Dialog.DontShowForm := false;
      if Dialog.Output is TGeometryElement then
      begin
         if Dialog.Output <> GElement then
            GElement := TGeometryElement(Dialog.output);
      end;
   end;
   StoreObject;
end;

procedure TPolyElementEditor.ExecuteDialog;
begin
     if Dialog <> nil then
     begin
        if NewObject then Dialog.Caption := 'Nov linie'
                     else Dialog.Caption := 'Star linie';
        if GElement <> nil then
           Dialog.Input := GElement
        else
           Dialog.Input := PolyElement;
        Dialog.Left := Mouse.CursorPos.x;
        Dialog.Top := Mouse.CursorPos.y;
        if (Dialog.Left + Dialog.Width) > Screen.DesktopWidth then
           Dialog.Left := Screen.DesktopWidth - Dialog.Width;
        if (Dialog.Top + Dialog.Height) > Screen.Height then
           Dialog.Top := Screen.DesktopHeight - Dialog.Height;
        Dialog.SetPropertyLongIntByName('SnapTool',LongInt(SnapTool));
        Dialog.SetPropertyBooleanByName('FreeToDelete',FreeToDelete);
        Dialog.SetPropertyBooleanByName('FreeToChangeGeometry',FreeToChangeGeometry);
        Dialog.SetPropertyBooleanByName('AllowSwapDirection',AllowSwapDirection);
        Dialog.Modified := Modified;
        Dialog.DoIt;
        Dialog.SendStatus;
        Modified := Dialog.Modified;
        if Dialog.Output is TGeometryElement then
        begin
           if Dialog.Output <> GElement then
              GElement := TGeometryElement(Dialog.output);
        end;
        Case Dialog.Result of
           1: StoreObject;
           3: DeleteObject;
           4: DeletePoint;
           5: CloseObject;
           6: ;
           7: begin
                 PolyElement := TPolyElement(CreateObjectCopy);
                 if GElement <> nil then
                    GElement.Geometry := PolyElement;
                 TestForDelete;
                 TestForChangeGeometry;
              end;
           8: begin
                 if not (modified or CanCreateNewObject) then
                 begin
                    StoreObject;
                    NextNearLine;
                 end;
              end;
           9: SwapDirection;
           else
              ConfirmAndStore;
        end;
     end;
end;

procedure TPolyElementEditor.SetAllowSwapDirection(const Value: boolean);
begin
  FAllowSwapDirection := Value;
end;

procedure TPolyElementEditor.SwapDirection;
var
   QE: TQuickEntity;
   E: TEntity;
   A: TPolyLineAreaElement;
   i: integer;
   iL: integer;
   Orig: boolean;
begin
   if PolyElement = nil then exit;
   XORDraw;
   QE := TQuickEntity.Create;
   QE.AddAllFrom := PolyElement;
   PolyElement.Clear;
   for i := QE.LastIndex downto 0 do
      PolyElement.AddElement(QE.Elements[i]);
   QE.Clear;
   // otoceni v plochach
   for i := 0 to PolyElement.OwnersCount - 1 do
   begin
      E := PolyElement.Owners[i];
      if E is TPolyLineAreaElement then
      begin
         A := TPolyLineAreaElement(E);
         iL := A.IndexOf(PolyElement);
         A.LinesOrigines[iL] := not A.LinesOrigines[iL]
      end;
   end;
   Modified := true;
   Swap := not Swap;
   QE.Free;
   XORDraw;
end;

procedure TPolyElementEditor.FindNearLines;
var
   QTW: TQTreeWalker;
   aX,aY,T: double;
   e: TElement;
begin
   NearLines.Clear;
   if QTFinder.Input = nil then exit;
   if QTFInder.Tolerance <= 0 then exit;
   aX := QTFinder.X;
   aY := QTFinder.Y;
   T := QTFinder.Tolerance;
   QTW := TQTreeWalker.Create;
   e := QTW.GetFirstElementForRectangle(TQtree(QTFinder.Input),aX,aY,aX,aY,T);
   while e <> nil do
   begin
      if NearLines.IndexOf(e) < 0 then
      begin
         if (e is TRectEntity) and TRectEntity(e).IsHitByXY(aX,aY,T) then
            NearLines.AddElement(e);
      end;
      e := QTW.GetNextElement;
   end;
   QTW.Free;
end;

function TPolyElementEditor.GetNearLines: TQuickEntity;
begin
   if FNearLines = nil then
   begin
      FNearLines := TQuickEntity.Create;
      AddElement(FNearLines);
   end;
   result := FNearLines;
end;

procedure TPolyElementEditor.NextNearLine;
begin

end;

destructor TPolyElementEditor.Destroy;
begin
   NearLines.Clear;
   inherited;
end;

procedure TPolyElementEditor.SetSwap(const Value: boolean);
begin
  FSwap := Value;
end;

{ TPolyLineEditor }


function  TPolyLineEditor.GetCursor: TCursor;
begin
   result := inherited GetCursor;
//   result := cr_MLength;
//   result := cr_SelectLine;
end;


procedure TPolyLineEditor.CreateNewObject;
begin
  if PolyElement <> nil then XORDraw;
  GElement := nil;
  PolyElement:= TPolyLineElement.Create;
  FActualCrdIndex:= -1;
  FMovedCrdIndex:= -1;
  if PolyElement = nil then exit;
  //AddElement(PolyElement);
//  PolyElement.AddXY(0, 0);
end;




procedure TPolyLineEditor.XORDraw;
var
  aR: double;
  I: integer;
  K: integer;
begin
   if PolyElement = nil then
      Exit;

  Device.Pen.Color:= FColor;
  Device.Pen.Style:= psSolid;
  Device.Pen.Mode:= pmXOR;
  Device.Pen.Width:= 1;
  Device.Brush.Style:= bsClear;
  if PolyElement.CoordsCount > 1 then
  begin
     Device.MoveTo(PolyElement.Xi[0], PolyElement.Yi[0]);
     for i := 1 to PolyElement.LastIndex do
         Device.LineTo(PolyElement.Xi[i], PolyElement.Yi[i]);
  end;
// vykreslovn znaek
  aR:= Device.Projection.GetModelLength(PixelR);
  K:= PolyElement.CoordsCount-1;
  for I:= 0 to K do
  begin
    DrawRect(PolyElement.Xi[i], PolyElement.Yi[i], aR);
  end;

// Vykreslen vybranho bodu
  I:= FActualCrdIndex;
  aR:= Device.Projection.GetModelLength(PixelR - 1);
  if (I >= 0) and (I <= K) then
  begin
    DrawRect(PolyElement.Xi[i], PolyElement.Yi[i], aR);
  end;

// Vykreslen hlaviky
  aR:= Device.Projection.GetModelLength(PixelR + 2);

  if FHeaderAtTale then
     K:= 0;

  if K >= 0 then
  begin
    DrawRect(PolyElement.Xi[K], PolyElement.Yi[K], aR);

    if FCloseEnds then
    begin
      Device.MoveTo(PolyElement.Xi[K]+2*aR, PolyElement.Yi[K]+2*aR);
      Device.LineTo(PolyElement.Xi[K]-2*aR, PolyElement.Yi[K]-2*aR);
      Device.MoveTo(PolyElement.Xi[K]-2*aR, PolyElement.Yi[K]+2*aR);
      Device.LineTo(PolyElement.Xi[K]+2*aR, PolyElement.Yi[K]-2*aR);
    end
  end;

  PostMsgToOwners(tm_Repaint,nil,DontKill);
end;



procedure TPolyElementEditor.ChangeCursor;
var
  aCurs: TCursor;
  I, K: integer;
  aR: double;
  PataUvnitr: boolean;
  PataX, PataY, Vzdalenost: double;
  Done: boolean;
begin
   Done := false;
   aR:= Device.Projection.GetModelLength(PixelR);
   K:= PolyElement.CoordsCount-1;

   aCurs := crSelectLine;
   //aCurs:= GetCursor;
   FMovedCrdIndex:= -1;
   FMovedBetween:= false;

   for I:= 0 to K do
   begin
     if InTolerance(PolyElement.Xi[i], PolyElement.Yi[i], FActX, FActY, 2*aR) then
     begin
       aCurs:= crMovePoint;
       FMovedCrdIndex:= I;
       Done := true;
       break;
     end;
   end;

   if FMovedCrdIndex = -1 then
   for I:= 0 to K-1 do
   begin
     DejVztahyBod_Usecka(FActX, FActY,
                         PolyElement.Xi[i], PolyElement.Yi[i],
                         PolyElement.Xi[i+1], PolyElement.Yi[i+1],
                         PataUvnitr, PataX, PataY, Vzdalenost);

     if PataUvnitr and (Vzdalenost < aR) then
     begin
       aCurs:= crInsertPoint;
       FMovedCrdIndex:= I;
       FMovedBetween:= true;
       Done := true;
       break;
     end;
   end;
   if not Done and Snapping then
   begin
      //SnapDraw(aMessage.X,aMessage.Y);
      SnapTool.X := ActX;
      SnapTool.Y := ActY;
      SnapTool.DoIt;
      if SnapTool.Snapped then
      begin
         ActX := SnapTool.SnapX;
         ActY := SnapTool.SnapY;
         case SnapTool.SnapStatus of
            1: aCurs := 3;
            2: aCurs := 4;
         end;
      end;
   end;
  CursorIndex := aCurs;
  {
  if Manager.FormCursor <> aCurs then
     Manager.FormCursor:= aCurs;
  }
end;



function TPolyLineEditor.GetLength: double;
begin
   result:= PolyElement.Length;
end;



    { TShapeEditor }


function  TShapeEditor.GetCursor: TCursor;
begin
    result := inherited GetCursor;
   //result := cr_SelectArea;
end;



procedure TShapeEditor.CreateNewObject;
begin
  if PolyElement <> nil then
  begin
     XORDraw;
     FreeElement(PolyElement);
  end;

  PolyElement:= TPolyLineElement.Create;
  FActualCrdIndex:= -1;
  FMovedCrdIndex:= -1;
  if PolyElement = nil then exit;
  AddElement(PolyElement);
//  PolyElement.AddXY(0, 0);

end;



procedure TShapeEditor.XORDraw;
var
  aR: double;
  I: integer;
  K: integer;
begin
  Device.Pen.Color:= FColor;
  Device.Pen.Style:= psSolid;
  Device.Pen.Mode:= pmXOR;
  Device.Pen.Width:= 1;
  Device.Brush.Style:= bsClear;
  if PolyElement.Count < 3 then
     Device.Polyline(PolyElement)
  else
     Device.Polygon(PolyElement);

// vykreslovn znaek
  aR:= Device.Projection.GetModelLength(PixelR);
  K:= PolyElement.CoordsCount-1;
  for I:= 0 to K do
  begin
    DrawRect(PolyElement.Xi[i], PolyElement.Yi[i], aR);
  end;

// Vykreslen vybranho bodu
  I:= FActualCrdIndex;
  aR:= Device.Projection.GetModelLength(PixelR - 1);
  if I >= 0 then
  begin
    DrawRect(PolyElement.Xi[i], PolyElement.Yi[i], aR);
  end;

// Vykreslen hlaviky
  aR:= Device.Projection.GetModelLength(PixelR + 2);

  if FHeaderAtTale then
     K:= 0;

  if K >= 0 then
  begin
    DrawRect(PolyElement.Xi[i], PolyElement.Yi[i], aR);

    if FCloseEnds then
    begin
      Device.MoveTo(PolyElement.Xi[K]+2*aR, PolyElement.Yi[K]+2*aR);
      Device.LineTo(PolyElement.Xi[K]-2*aR, PolyElement.Yi[K]-2*aR);
      Device.MoveTo(PolyElement.Xi[K]-2*aR, PolyElement.Yi[K]+2*aR);
      Device.LineTo(PolyElement.Xi[K]+2*aR, PolyElement.Yi[K]-2*aR);
    end
  end;

  PostMsgToOwners(tm_Repaint,nil,DontKill);
end;



function TShapeEditor.GetLength: double;
begin
  result:= PolyElement.Length;
end;


function TShapeEditor.GetArea: double;
begin
  result:= abs(PolyElement.Area);
  if result < 1E-7 then
     result:= 0;
end;



procedure TShapeEditor.SendInfo;
//var
//  aForm: TStandardForm;
begin
{
  if Device is TCanvasToBmpDevice then
  begin
    aForm:= TStandardForm(TCanvasToBMPDevice(Device).CanvasForm);
    if aForm is TStandardForm then
    begin
      aForm.Label3.Caption:= 'P: ';
      aForm.l_M.Caption:= DoubleToStr(Area, 10, 2);
      Application.ProcessMessages;
    end;
  end;
}
end;




{ TGraphicEditor }

procedure TGraphicEditor.CreateNewObject;
begin

end;

function TGraphicEditor.CreateObjectCopy: TElement;
begin
   result := CreateElementCopy(input);
end;

function TGraphicEditor.GetNewObject: boolean;
begin
   result := Input = nil;
end;

procedure TGraphicEditor.HandleMessage(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   Handled := false;
   if ActiveTool <> nil then
   begin
      case aMessage.Message of
         tm_Cursor:          on_tm_Cursor(TOrbisCursorEvent(aMessage),Handled);
         tm_ElementSelected: on_tm_ElementSelected(aMessage, Handled);
      else
         ActiveTool.HandleMessage(aMessage, Handled);
      end;
      exit;
   end;
   inherited HandleMessage(aMessage, Handled);
end;

procedure TGraphicEditor.On_Activate;
begin
   active := true;
   Input := nil;
   if SelectTool <> nil then
   begin
      ActiveTool := SelectTool;
      SelectTool.Device := Device;
      if SelectTool is TFrameTool then
         TFrameTool(SelectTool).FrameSelector := FrameSelector;
      SelectTool.On_Activate;
   end
   else
   begin
      inherited On_Activate;
      if CanCreateNewObject then
         CreateNewObject;
      ActiveTool := nil;
      XORDraw;
   end;
end;

procedure TGraphicEditor.On_tm_Cursor(var aMessage: TOrbisCursorEvent;
  var Handled: boolean);
var
   i: integer;
begin
   if NoOwners then exit;
   for i := 0 to OwnersCount - 1 do
   begin
      Owners[i].HandleMessage(TOrbisMessage(aMessage), Handled);
   end;
end;

procedure TGraphicEditor.On_tm_ElementSelected(var aMessage: TOrbisMessage;
  var Handled: boolean);
//var
   //i: integer;
   //E: TEntity;
begin
   if not (aMessage.Sender = SelectTool) then exit;
   Input := aMessage.Data;
   //Output := Input; //!!!!!!!!!!!!!!!CreateObjectCopy;
   if Input <> nil then // Odstraneni dosavadnich vlastniku !!!!
   begin
      RemoveElementOwners(Input);
   end;
   ExchangeFieldElement(FOutput,FInput);
   ActiveTool := nil;
   inherited On_Activate;
   StartEditing;
   XORDraw;
end;

procedure TGraphicEditor.RemoveElementOwners(const anElement: TElement);
var
   i: integer;
   E: TEntity;
begin
   if not anElement.NoOwners then
   begin
      for i := anElement.OwnersCount - 1 downto 0 do
      begin
         E := anElement.Owners[i];
         E.RemoveElement(Input);
      end;
   end;
end;

procedure TGraphicEditor.SetCanCreateNewObject(const Value: boolean);
begin
   FCanCreateNewObject := Value;
end;

procedure TGraphicEditor.SetConfirmText(const Value: string);
begin
   FConfirmText := Value;
end;

procedure TGraphicEditor.SetDialog(const Value: TOrbisDialog);
begin
   ExChangeFieldTool(TTool(FDialog),Value);
end;

procedure TGraphicEditor.SetEditorStatus(
  const Value: TGraphicEditorStatus);
begin
   FEditorStatus := Value;
end;

procedure TGraphicEditor.SetModified(const Value: boolean);
begin
  FModified := Value;
end;

procedure TGraphicEditor.SetSelectTool(const Value: TInterfaceTool);
begin
   ExChangeFieldTool(TTool(FSelectTool),TTool(Value));
end;

procedure TGraphicEditor.SetStoreTool(const Value: TTool);
begin

   ExChangeFieldTool(FStoreTool,Value);
end;

procedure TGraphicEditor.StartEditing;
begin

end;

procedure TGraphicEditor.XORDraw;
begin

end;

function TGraphicEditor.FindLevel(aLevel: string): TEntity;
var
   OIL: TCommandLine;
   E: TElement;
   aL: string;
begin
   result := nil;
   OIL := TCommandLine.Create;
   aL := StringReplace(aLevel, '$USR', _USR,[rfReplaceAll, rfIgnoreCase]);
   OIL.Line := aL;
   OIL.DoIt;
   E := TElement(StrToIntDef(OIL.ResultLine, 0));
   OIL.Free;
   if not (E is TEntity) then exit;
   result := TEntity(E);
end;


function TGraphicEditor.FindLevelInEntity(anEntity: TEntity;aLevel: string): TEntity;
var
   OIL: TCommandLine;
   E: TElement;
begin
   result := nil;
   OIL := TCommandLine.Create;
   OIL.Line := aLevel;
   OIL.Input := anEntity;
   OIL.DoIt;
   E := TElement(StrToIntDef(OIL.ResultLine, 0));
   OIL.Free;
   if not (E is TEntity) then exit;
   result := TEntity(E);
end;


function TGraphicEditor.GetQTFinder: TFindRectEntityInQTreeByXY;
begin
   if FQTFinder = nil then
   begin
      FQTFinder := TFindRectEntityInQTreeByXY.Create;
      AddElement(FQTFinder);
   end;
   result := FQTFinder;
end;

procedure TGraphicEditor.EndFlashTool;
begin
   XORDraw;
   FCursor := -1;
   CursorIndex := 0;
end;

procedure TGraphicEditor.StartFlashTool;
begin
   //XORDraw;

end;

procedure TGraphicEditor.BeforeFlashReview;
begin
   XORDraw;
end;

procedure TGraphicEditor.ExecuteDialog;
begin

end;

function TGraphicEditor.IsPossibleToDeactivate: boolean;
begin
   result := not (EditorStatus = gesPrepare);
end;


{ TPolyLineAreaEditor }

procedure TPolyLineAreaEditor.ClearLines;
var
   //L: TPolyLineElement;
   E: TElement;
   OW: TEntity;
   h: integer;

   procedure FreeOtherOwners(anArea: TAreaElement);
   var
      i,j: integer;
   begin
      if anArea <> nil then
      begin
         for i := 0 to anArea.LastIndex do
         begin
            E := anArea.Elements[i];
            for j := E.OwnersCount -1 downto 0 do
            begin
               OW := E.Owners[j];
               if OW = nil then Continue;
               if not(OW is TAreaElement)then
               begin
                  OW.RemoveElement(E);
               end;
            end;
         end;
      end;
   end;

begin
   FreeOtherOwners(Area);
   if Area.Holes <> nil then
   begin
      for H := 0 to Area.Holes.LastIndex do
      begin
         E := Area.Holes.Elements[H];
         if E is TAreaElement then FreeOtherOwners(TAreaElement(E));
      end;
   end;
   Lines := nil;
end;

function TPolyLineAreaEditor.FindArea: boolean;
var
   Pick: TPickUpAreaElement;
   E: TEntity;
begin
   result := false;
    { TODO -cOpravit : Upravit na vbr GeometrzElementu }
   Pick:= TPickUpAreaElement.Create;
   Pick.Tolerance:= 0;
   Pick.X:= X;
   Pick.Y:= Y;
   E := nil;
   if LevelName <> '' then
   begin
      E := TEntity(Frame.FindElementName(LevelName));
      if E = nil then exit;
      Pick.Input := E;
   end;
   //if E = nil then Pick.Input := FrameSelector.Output;
   Pick.DoIt;
   Area:= TAreaElement(Pick.Output);
   Pick.Free;
   if Area <> nil then
   begin
      Output := Area;
      PostMsgToOwners(tm_ElementSelected,Area,false);
      result := true;
   end;
end;

function TPolyLineAreaEditor.FindFrame: boolean;
begin
   result := false;
   Frame := nil;
   if FrameSelector = nil then exit;
   FrameSelector.X := X;
   FrameSelector.Y := Y;
   FrameSelector.DoIt;
   if FrameSelector.Output = nil then exit;
   if FrameSelector.Output is TEntity then
   begin
      Frame := TEntity(FrameSelector.Output);
      result := true;
   end;
end;

procedure TPolyLineAreaEditor.FindLines;
var
   i: integer;
   E: TElement;
begin
   Lines := TRectEntity.Create;
   Lines.Name := 'Founded lines';
   FindLinesInFrame(Frame, WorkingLinesLevels, CopyLinesLevels);
   if NeighbourhoodTool <> nil then
   begin
      NeighbourhoodFrames := TOwnerEntity.Create;
      NeighbourhoodTool.Input := Frame;
      NeighbourhoodTool.Output := NeighbourhoodFrames;
      NeighbourhoodTool.DoIt;
      if not NeighbourhoodFrames.NoElements then
      begin
         for i := 0 to NeighbourhoodFrames.LastIndex do
         begin
            E := NeighbourhoodFrames.Elements[i];
            if E is TEntity then FindLinesInFrame(TEntity(E), NeighbourhoodWorkingLinesLevels, NeighbourhoodCopyLinesLevels);
         end;
      end;
      NeighbourhoodFrames := nil;
   end;
   // prava odstraujc chyby pi nslednm spojovn plynouc ze patnch obdlnk
   for i := 0 to Lines.LastIndex do
   begin
      TRectEntity(Lines.Elements[i]).ResetAndUpdate;
   end;
   Lines.Update;
end;

procedure TPolyLineAreaEditor.FindLinesInFrame(aFrame: TEntity; WorkLevels,
  CopyLevels: string);
var
   LEX: TLineExtractor;
   WorkL,CopyL: TStringList;
   i: integer;
   E: TElement;
begin
   LEX := TLineExtractor.Create;
   WorkL := TStringList.Create;
   CopyL := TStringList.Create;
   WorkL.CommaText := WorkLevels;
   CopyL.CommaText := CopyLevels;
   LEX.CopyLines := true;
   for i := 0 to CopyL.Count - 1 do
   begin
      E := aFrame.FindElementName(CopyL.Strings[i]);
      if E = nil then Continue;
      LEX.Input := E;
      LEX.Output := Lines;
      LEX.DoIt;
   end;
   LEX.CopyLines := false;
   for i := 0 to WorkL.Count - 1 do
   begin
      E := aFrame.FindElementName(WorkL.Strings[i]);
      if E = nil then Continue;
      LEX.Input := E;
      LEX.Output := Lines;
      LEX.DoIt;
   end;
   LEX.Free;
   WorkL.Free;
   CopyL.Free;
end;

procedure TPolyLineAreaEditor.MakeArea;
var
   AM: TCreateAreasFromLinesTool;
   A: TAreaElement;
   E: TElement;
   i: integer;
   C: T2DCoordinate;
begin
   if Lines.NoElements then
   begin
      PostTextMsg(tm_AddToProtocol,Name+': No lines for Area Found! ',nil);
      exit;
   end;
   AM := TCreateAreasFromLinesTool.Create;
   AM.Input := Lines;
   AM.DoIt;
   Areas := TRectEntity(AM.Output);
   //exit;
   C := T2DCoordinate.Create;
   C.X := X;
   C.Y := Y;
   for i := 0 to Areas.LastIndex do
   begin
      E := Areas.Elements[i];
      if not (E is TAreaElement) then Continue;
      A := TAreaElement(E);
      if A.IsHit(C,0) then
      begin
         Area := A;
         Output := Area;
         Break;
      end;
   end;
   Areas := nil;
end;

procedure TPolyLineAreaEditor.On_tm_MouseUp(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
{
var
   Test: boolean;
}
begin
   if ZeroActivity then exit;
   HighActivity := true;
   if EditorStatus = gesPrepare then exit;
   if not (aMessage.Button = mbLeft) then exit;
   X := Device.Projection.GetModelX(aMessage.x);
   Y := Device.Projection.GetModelY(aMessage.y);
   if not FindFrame then exit;
   FindArea;
   Input := Area;
   if NewObject then
   begin
      FindLines;
      {
      Test := false;
      if Test then
      begin
         ShowMessage('Test');
         exit;
      end;
      }
      //****  hokus
      //World.ObjectModel.NextElement := Lines;
      //exit;
      //****
      MakeArea;
      ClearLines;
   end;
   if Area = nil then
   begin
      HighActivity := false;
      exit;
   end;
   if Dialog <> nil then
   begin
      XORDraw;
      if NewObject then Dialog.Caption := 'Nov plocha'
                   else Dialog.Caption := 'Star plocha';
      Dialog.SetPropertyLongIntByName('Frame',LongInt(Frame));
      Dialog.Input := Area;
      Dialog.Left := Mouse.CursorPos.x;
      Dialog.Top := Mouse.CursorPos.y;
      if (Dialog.Left + Dialog.Width) > Screen.DesktopWidth then
         Dialog.Left := Screen.DesktopWidth - Dialog.Width;
      if (Dialog.Top + Dialog.Height) > Screen.Height then
         Dialog.Top := Screen.DesktopHeight - Dialog.Height;
      Dialog.DoIt;
      XORDraw;
      Case Dialog.Result of
         1:
         begin
            if NewObject and (StoreTool <> nil) then
            begin
               StoreTool.SetPropertyLongIntByName('FrameSelector',longInt(FrameSelector));
               StoreTool.Input := Area;
               StoreTool.DoIt;
            end;
            PostMsgToOwners(tm_ReviewByElement,Area,false);
            PostMsg(tm_ModelChanged,nil,nil,false);
            Application.ProcessMessages;
         end;
         2:
         begin
         end;
      end;
   end;
   Input := nil;
   Area := nil;
   HighActivity := false;
end;

procedure TPolyLineAreaEditor.RegisterCursors;
begin
   NewCursorName('SELECTAREA');
end;

procedure TPolyLineAreaEditor.SetArea(const Value: TAreaElement);
begin
   ExchangeFieldElement(TElement(FArea), Value);
end;

procedure TPolyLineAreaEditor.SetAreas(const Value: TRectEntity);
begin
   ExchangeFieldElement(TElement(FAreas),Value);
end;

procedure TPolyLineAreaEditor.SetCopyLinesLevels(const Value: string);
begin
  FCopyLinesLevels := Value;
end;

procedure TPolyLineAreaEditor.SetFrame(const Value: TEntity);
begin
   FFrame := Value;
end;

procedure TPolyLineAreaEditor.SetLevelName(const Value: string);
begin
   FLevelName := Value;
end;

procedure TPolyLineAreaEditor.SetLines(const Value: TRectEntity);
begin
   ExchangeFieldElement(TElement(FLines),Value);
end;

procedure TPolyLineAreaEditor.SetNeighbourhoodCopyLinesLevels(
  const Value: string);
begin
   FNeighbourhoodCopyLinesLevels := Value;
end;

procedure TPolyLineAreaEditor.SetNeighbourhoodFrames(const Value: TEntity);
begin
   ExchangeFieldElement(TElement(FNeighbourhoodFrames),Value);
end;

procedure TPolyLineAreaEditor.SetNeighbourhoodTool(const Value: TTool);
begin
   ExChangeFieldTool(TTool(FNeighbourhoodTool),Value);
end;

procedure TPolyLineAreaEditor.SetNeighbourhoodWorkingLinesLevels(
  const Value: string);
begin
  FNeighbourhoodWorkingLinesLevels := Value;
end;

procedure TPolyLineAreaEditor.SetWorkingLinesLevels(const Value: string);
begin
   FWorkingLinesLevels := Value;
end;

procedure TPolyLineAreaEditor.SetX(const Value: double);
begin
   FX := Value;
end;

procedure TPolyLineAreaEditor.SetY(const Value: double);
begin
   FY := Value;
end;

procedure TPolyLineAreaEditor.XORDraw;
var
   Are: TAreaElement;
begin
   if Area = nil then exit;
   if Color = 0 then Color := $AAAAAA;
   Are := TPolyLineAreaElement(Area).CreateAreaElement;
    Device.Brush.Color:= Color;
    Device.Brush.Style:= bsbDiagonal;
    Device.Pen.Mode:= pmXOR;
    Device.Pen.Width:= 3;
    Device.Pen.Color:= Color;
    Device.Pen.Style:= psSolid;
    Device.PolyPolygon(Are);
    PostMsgToOwners(tm_Repaint,nil,false);
    Application.ProcessMessages;
    Are.Free;
end;

{ TPointEditor }

procedure TPointEditor.ChangeAngle;
var
   P: TGeometryElement;
   v: double;
   A,B: TDoubleCoord;
   anAngle: double;
begin
   //PostTextMsg(tm_AddToProtocol,'ChangeAngle',nil);
   if Busy or IsDraged then exit;
   IsDraged := true;
   Application.ProcessMessages;
   XORDraw;
   P := Point;
   if P = nil then exit;
   X := ActX;
   Y := ActY;
   A.X := P.X;
   A.Y := P.Y;
   B.X := X;
   B.Y := Y;
   XYToPolar(A,B, v,anAngle);
   P.Geometry.Angle := anAngle-PI/2;
   XORDraw;
   IsDraged := false;
end;

procedure TPointEditor.CHangeSelectCursor;
var
   d: double;
begin
   if QTFinder.Busy then exit;
   d := Device.Projection.GetModelLength(PixelTolerance);
   QTFinder.Input := LevelQT;
   QTFinder.X := X;
   QTFInder.Y := Y;
   QTFinder.Tolerance := d;
   QTFinder.DoIt;
   if QTFinder.Output <> nil then
      CursorIndex := 1
   else
      CursorIndex := 0;
end;



function TPointEditor.GetInterpretName: string;
begin
   result := '';
   if Interpret <> nil then
   begin
      result := Interpret.Name;
   end;
end;

function TPointEditor.GetPoint: TGeometryElement;
begin
   result := TGeometryElement(Output);
end;

procedure TPointEditor.On_Activate;
begin
   Level := FindLevel(LevelName);
   if Level = nil then exit;
   HighActivity := true;
   EditorStatus := gesPrepare;
   if not Active then
      PrepareQTree;
   EditorStatus := gesSelect;
   QTFinder.Busy := false;
   inherited;
   HighActivity := false;
end;

procedure TPointEditor.On_tm_DragDrop(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
var
   X,Y: double;
begin
   Handled := true;
   if NewObject then
      CursorIndex := 0
   else
      CursorIndex := 1;
   IsDraged := false;
   //PostTextMsg(tm_AddToProtocol,'M Drag Drop',nil);
   //Application.ProcessMessages;
   X := Device.Projection.GetModelX(aMessage.x);
   Y := Device.Projection.GetModelY(aMessage.y);
   XORDraw;
   Point.X := X;
   Point.Y := Y;
   XORDraw;
   StorePoint(ssShift in aMessage.Shift);
   if HighActivity then HighActivity := false;
end;


procedure TPointEditor.On_tm_DragOver(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
var
   X,Y: double;
begin
   Handled := true;
   //PostTextMsg(tm_AddToProtocol,'M Drag over',nil);
   //Application.ProcessMessages;
   if Busy or IsDraged then
   begin
      //if Busy then
         //PostTextMsg(tm_AddToprotocol,'Busy Drag',nil);
      //PostTextMsg(tm_AddToProtocol,'M Drag over -',nil);
      //Application.ProcessMessages;
      exit;
   end;
   IsDraged := true;
    //PostTextMsg(tm_AddToProtocol,'M Drag over +',nil);
    Application.ProcessMessages;
   XORDraw;
   X := Device.Projection.GetModelX(aMessage.x);
   Y := Device.Projection.GetModelY(aMessage.y);
   Point.X := X;
   Point.Y := Y;
   //Application.ProcessMessages;
   XORDraw;
   IsDraged := false;
   //Application.ProcessMessages;
end;


procedure TPointEditor.On_tm_MouseDown(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
var
   X,Y: double;
   //Prp: TTopolPointProps;
   S: string;
   P: TPointElement;
   C: integer;
begin
   if EditorStatus = gesSetup then exit;
   if EditorStatus = gesPrepare then exit;
   if ZeroActivity then exit;
   HighActivity := true;
   OldX := 0;
   OldY := 0;
   OldAngle := 0;
   Handled := true;
   //PostTextMsg(tm_AddToProtocol,'M Down',nil);
   //Application.ProcessMessages;
   if not (aMessage.Button = mbLeft) then
   begin
      if Dialog <> nil then
      begin
         P := TPointElement.Create;
         Dialog.Caption := 'Vlastnost bodu';
         Dialog.Input := P;
         Dialog.DontShowForm := false;
         Dialog.DoIt;
         P.Free;
         Dialog.Output.Free;
      end;
      exit;
   end;
   Busy := true;
   Input := QTFinder.Output;
   if NewObject then
   begin
      X := Device.Projection.GetModelX(aMessage.x);
      Y := Device.Projection.GetModelY(aMessage.y);
      P := TpointElement.Create;
      P.X := X;
      P.Y := Y;
      if Dialog <> nil then
      begin
         Dialog.DontShowForm := true;
         Dialog.Input := P;
         Dialog.DoIt;
         Dialog.DontShowForm := false;
         Point := TGeometryElement(Dialog.Output);
      end;
   end
   else
   begin
      RemoveElementOwners(Input);
      ExchangeFieldElement(FOutput,FInput);
      OldX := Point.X;
      OldY := Point.Y;
      OldAngle := Point.Geometry.Angle;
      S := 'Barva';
      C := Point.GetPropertyLongIntByName(S);
      Point.SetPropertyLongIntByName(S,0);
      Interpret.Input := Output;
      Interpret.Reset(Device);
      Interpret.DoIt;
      //PostMsgToOwners(tm_Review,Point,false);
      //Application.ProcessMessages;
      Point.SetPropertyLongIntByName(S,C);
   end;
   Interpret.Input := Output;
   Interpret.Reset(Device);
   XORDraw;
   Busy := false;
end;

procedure TPointEditor.On_tm_MouseMove(var aMessage: TOrbisMouseEvent;var Handled: boolean);
begin
   if EditorStatus = gesPrepare then exit;
   if EditorStatus = gesSelect then
   begin
      X:= Device.Projection.GetModelX(aMessage.x);
      Y:= Device.Projection.GetModelY(aMessage.y);
      ChangeSelectCursor;
   end
   else
   if EditorStatus = gesSetup then
   begin
      ActX:= Device.Projection.GetModelX(aMessage.x);
      ActY:= Device.Projection.GetModelY(aMessage.y);
      CHangeAngle;
   end;
end;

procedure TPointEditor.On_tm_MouseUp(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
var
   X,Y: double;
   B: boolean;
begin
   if ZeroActivity and not HighActivity then exit;
   if EditorStatus = gesPrepare then exit;
   Handled := true;
   if Busy then
   begin
      //PostTextMsg(tm_AddToprotocol,'Busy',nil);
   end;
   if not (aMessage.Button = mbLeft) then exit;
   if EditorStatus = gesSetup then
      EditorStatus := gesSelect
   else
   begin
      X := Device.Projection.GetModelX(aMessage.x);
      Y := Device.Projection.GetModelY(aMessage.y);
      if Dialog <> nil then
      begin
         Dialog.DontShowForm := true;
         Dialog.Input := Point;
         Dialog.DoIt;
         Dialog.DontShowForm := false;
         B := Dialog.GetPropertyBooleanByName('AngleChange');
         if B then
         begin
            EditorStatus := gesSetup;
            if HighActivity then HighActivity := false;
            exit;
         end;
      end;
   end;
   //PostTextMsg(tm_AddToProtocol,'M Up',nil);
   //Application.ProcessMessages;
   {
   XORDraw;
   Point.X := X;
   Point.Y := Y;
   XORDraw;
   }
   StorePoint(ssShift in aMessage.Shift);
   if HighActivity then HighActivity := false;
end;

procedure TPointEditor.On_tm_Resize(var Handled: boolean);
begin
   XORDraw;
end;

procedure TPointEditor.On_tm_StartDrag(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
begin
   Handled := true;
   if NewObject then
      CursorIndex := 2
   else
      CursorIndex := 3;
   //PostTextMsg(tm_AddToProtocol,'M Start Drag',nil);
   //Application.ProcessMessages;
end;

procedure TPointEditor.PrepareQTree;
var
   TQ: TAddEntityToQTree;
begin
   if Level = nil then exit;
   StartProgress('Pprava vrstev...',0,0);
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Level;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
   EndProgress;
end;

procedure TPointEditor.RegisterCursors;
begin
   NewCursorName('POINTNEW');
   NewCursorName('POINTOLD');
   NewCursorName('POINTNEWMOVE');
   NewCursorName('POINTOLDMOVE');
end;




procedure TPointEditor.SetActX(const Value: double);
begin
  FActX := Value;
end;

procedure TPointEditor.SetActY(const Value: double);
begin
  FActY := Value;
end;

procedure TPointEditor.SetInterpretName(const Value: string);
var
   EC: TElementClass;
begin
   Interpret := nil;
   EC := GetOrbisClass(Value);
   if EC = nil then exit;
   if not (EC.InheritsFrom(TInterpret)) then exit;
   Interpret := TInterpret(EC.Create);
end;

procedure TPointEditor.SetLevel(const Value: TEntity);
begin
   FLevel := Value;
end;

procedure TPointEditor.SetLevelName(const Value: string);
begin
   FLevelName := Value;
   Level := FindLevel(Value);
end;

procedure TPointEditor.SetLevelQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLevelQT), Value);
end;

procedure TPointEditor.SetOldAngle(const Value: double);
begin
  FOldAngle := Value;
end;

procedure TPointEditor.SetOldX(const Value: double);
begin
  FOldX := Value;
end;

procedure TPointEditor.SetOldY(const Value: double);
begin
  FOldY := Value;
end;

procedure TPointEditor.SetPixelTolerance(const Value: integer);
begin
   FPixelTolerance := Value;
end;

procedure TPointEditor.SetPoint(const Value: TGeometryElement);
var
   P: TElement;
begin
   P := Value;
   ExchangeFieldElement(FOutput, P);
end;

procedure TPointEditor.SetX(const Value: double);
begin
   FX := Value;
end;

procedure TPointEditor.SetY(const Value: double);
begin
   FY := Value;
end;

procedure TPointEditor.StorePoint(DontShowDialog: boolean);
var
   Res: integer;

procedure StoreToLevel;
begin
   if Level <> nil then
   begin
      if Level is TRectEntity then
         TRectEntity(Level).AddAndUpdateRect(Point)
      else
         Level.AddElement(Point);
   end;
   if LevelQT <> nil then
      LevelQT.NextRectEntity := Point;
end;

procedure Redraw;
begin
   Device.Pen.Mode := pmCopy;
   PostMsgToOwners(tm_Review,Point,false);
   PostMsg(tm_ModelChanged,nil,nil,false);
   Application.ProcessMessages;
end;

begin
   //PostTextMsg(tm_AddToProtocol,'StorePoint',nil);
   //Application.ProcessMessages;
   if (Point = nil) then
   begin
      XORDraw;
      exit;
   end;
   if Dialog <> nil then
   begin
      if NewObject then
         Dialog.Caption := 'Nov'
      else
         Dialog.Caption := 'Star';
      Dialog.Input := Point;
      Dialog.DontShowForm := DontShowDialog;
      Dialog.DoIt;
   end;
   XORDraw;
   if DontShowDialog then
      Res := 1
   else
      Res := Dialog.Result;
   Case Res of
      1: //OK
      begin
         StoreToLevel;
         Redraw;
      end;
         //Delete
      3:    PostMsg(tm_ModelChanged,nil,nil,false);
      4: //Angle
      begin
         EditorStatus := gesSetup;
         XORDraw;
         exit;
      end;
      else //Storno
      begin
         if not NewObject then
         begin
            Point.X := OldX;
            Point.Y := OldY;
            Point.Geometry.Angle := OldAngle;
            StoreToLevel;
            Redraw;
         end;
      end;
   end;
   Input := nil;
   Point := nil;
   Interpret.Input := nil;
end;

procedure TPointEditor.XORDraw;
begin
   if interpret = nil then exit;
   if Point = nil then exit;
   {
   Device.Pen.Mode := pmXOR;
   //PostTextMsg(tm_AddToProtocol,'XORDraw',nil);
   if EditorStatus = gesSetup then
   begin
      //PostTextMsg(tm_AddToProtocol,'  gesSetup',nil);
      Device.Pen.Width:= 1;
      Device.Brush.Style:= bsClear;
      Device.Pen.Color := $909090;
      Device.Pen.Style := psDot;
      Device.MoveTo(Point.X, Point.Y);
      Device.LineTo(X, Y);
      Device.Pen.Style:= psSolid;
   end;
   }
   Device.Pen.Mode := pmNotXOR;
   //Device.Pen.Mode := pmXOR;
   //Device.Pen.Color := Color;//$909090;
   {
   Device.MoveTo(Point.x-3,Point.y);
   Device.LineTo(Point.x+3,Point.y);
   Device.MoveTo(Point.x,Point.y-3);
   Device.LineTo(Point.x,Point.y+3);
   }
   //PostTextMsg(tm_AddToProtocol,'XORDraw',nil);
   interpret.DoIt;
   //PostMsgToOwners(tm_Repaint,nil,false);
   //Application.ProcessMessages;
end;

{ TPolyLineAreaEditorNoFrame }

function TPolyLineAreaEditorNoFrame.FindArea: boolean;
var
   //Pick: TPickUpAreaElement;
   i: integer;
   G: TGeometryElement;
 { TODO -cOpravit : Upravit na vbr GeometrzElementu }
begin
   result := false;
   GArea := nil;
   if OldAreas = nil then exit;
   {
   Pick:= TPickUpAreaElement.Create;
   Pick.Tolerance:= 0;
   Pick.X:= X;
   Pick.Y:= Y;
   Pick.Input := OldAreas;
   //if E = nil then Pick.Input := FrameSelector.Output;
   Pick.DoIt;
   Area:= TAreaElement(Pick.Output);

   if Area <> nil then
   begin
      if Pick.Multi then
         GArea := TGeometryElement(Pick.MultiElement.FindOwnerClass(TGeometryElement))
      else
         GArea := TGeometryElement(Area.FindOwnerClass(TGeometryElement));
      Output := GArea;
      PostMsgToOwners(tm_ElementSelected,Area,false);
      result := true;
   end;
   Pick.Free;
   }
   for i := 0 to OldAreas.LastIndex do
   begin
      G := TGeometryElement(OldAreas.Elements[i]);
      if G.PointIsInside(X,Y) then
      begin
         if G.IsHitByXY(X,Y,0) then
         begin
            GArea := G;
            Area := TAreaElement(G.GetSimpleGeometryByXY(X,Y,0));
            result := true;
            Break;
         end;
      end;
   end;
end;

procedure TPolyLineAreaEditorNoFrame.MakeArea;
var
   Pick: TPickUpAreaElement;
begin
   if NewAreas = nil then exit;
   Pick:= TPickUpAreaElement.Create;
   Pick.Tolerance:= 0;
   Pick.X:= X;
   Pick.Y:= Y;
   Pick.Input := NewAreas;
   //if E = nil then Pick.Input := FrameSelector.Output;
   Pick.DoIt;
   Area:= TAreaElement(Pick.Output);
   Pick.Free;
   if Area <> nil then
   begin
      Output := Area;
      PostMsgToOwners(tm_ElementSelected,Area,false);
      //result := true;
   end;
end;

procedure TPolyLineAreaEditorNoFrame.On_Activate;
begin
   EditorStatus := gesPrepare;
   inherited;
   PrepareAreas;
   EditorStatus := gesEdit;
end;

procedure TPolyLineAreaEditorNoFrame.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   R: TRectEntity;
begin
   if ZeroActivity then exit;
   HighActivity := true;
   if EditorStatus = gesPrepare then exit;
   if not (aMessage.Button = mbLeft) then exit;
   X := Device.Projection.GetModelX(aMessage.x);
   Y := Device.Projection.GetModelY(aMessage.y);
   if (OldAreas = nil) or  (NewAreas = nil) then exit;
   FindArea;
   Input := GArea;
   if NewObject then
   begin
      MakeArea;
   end;
   if Area = nil then
   begin
      HighActivity := false;
      exit;
   end;
   if Dialog <> nil then
   begin
      XORDraw;
      Dialog.Input := Area;
      if NewObject then
         Dialog.Output := nil
      else
         Dialog.Output := GArea;
      //Dialog.SetPropertyLongIntByName('Frame',LongInt(Frame));
      Dialog.Left := Mouse.CursorPos.x;
      Dialog.Top := Mouse.CursorPos.y;
      if (Dialog.Left + Dialog.Width) > Screen.DesktopWidth then
         Dialog.Left := Screen.DesktopWidth - Dialog.Width;
      if (Dialog.Top + Dialog.Height) > Screen.Height then
         Dialog.Top := Screen.DesktopHeight - Dialog.Height;
      Dialog.DoIt;
      XORDraw;
      Case Dialog.Result of
         1: //mrOK
         begin
            {
            if NewObject and (StoreTool <> nil) then
            begin
               StoreTool.SetPropertyLongIntByName('FrameSelector',longInt(FrameSelector));
               StoreTool.Input := Area;
               StoreTool.DoIt;
            end;
            }
            GArea := TGeometryElement(Dialog.Output);
            if (OldAreas <> nil) and (GArea <> nil) and not GArea.IsMyOwner(OldAreas) then
            begin
               if OldAreas is TRectEntity then
                  TRectEntity(OldAreas).AddAndUpdateRect(GArea)
               else
                  OldAreas.AddElement(GArea);
            end;
            PostMsgToOwners(tm_ReviewByElement,GArea,false);
            PostMsg(tm_ModelChanged,nil,nil,false);
            Application.ProcessMessages;
         end;
         2: //mrCancel
         begin
         end;
         3: //mrAbort
         begin
            if not NewObject then
            begin
               if GArea <> nil then
               begin
                  R := TRectEntity.Create;
                  if (GArea.Geometry is TMultiAreaElement) and
                     (GArea.Geometry.Count > 1) then
                  begin
                     R.UpdateRect(Area);
                     // Dialog jestli fakt zruit
                     //GArea.Geometry.FreeElement(Area);
                     FArea.Free;
                     FArea := nil;
                  end
                  else
                  if not KillGeometryOnly then
                  begin
                     R.UpdateRect(GArea);
                     // Dialog jestli fakt zruit
                     // Kvuli annotations
                     FArea.Free;
                     FArea := nil;
                     GArea.Free;
                     FGArea := nil;
                  end
                  else
                  begin
                     R.UpdateRect(Area);
                     GArea.Geometry.Free;
                     FArea := nil;
                  end;
                  PostMsgToOwners(tm_ReviewByElement,R,true);
                  PostMsg(tm_ModelChanged,nil,nil,false);
                  Application.ProcessMessages;
               end;
            end;
         end;
      end;
   end;
   Input := nil;
   Area := nil;
   GArea := nil;
   HighActivity := false;
end;

procedure TPolyLineAreaEditorNoFrame.PrepareAreas;
var
   R: TDoubleRect;
   E: TRectEntity;
   i: integer;
   G: TGeometryElement;
   AFL: TAreasFromClearLines;
begin
   R := Device.Projection.ModelView;
   E := TRectEntity.Create;
   E.SetByDoubleRect(R);
   if (FrameAreas <> nil) and not FrameAreas.NoElements then
   begin
      for i := 0 to FrameAreas.LastIndex do
      begin
         G := TGeometryElement(FrameAreas.Elements[i]);
         if not (G is TGeometryElement) then continue;
         if G.IsOverRegion(R.Minx,R.MinY,R.MaxX,R.MaxY) then
            E.UpdateRect(G);
      end;
      R := E.Rect.AsDoubleRect;
   end;
   E.Free;
   if (Lines = nil) or Lines.NoElements then exit;
   E := TRectEntity.Create;
   for i := 0 to Lines.LastIndex do
   begin
      G := TGeometryElement(Lines.Elements[i]);
      if not (G is TGeometryElement) then continue;
      if G.IsOverRegion(R.Minx,R.MinY,R.MaxX,R.MaxY) then
         E.AddAndUpdateRect(G);
   end;
   StartProgress('Pprava plochovn ve vezu',0,0);
   AFL := TAreasFromClearLines.Create;
   AFL.Input := E;
   AFL.DoIt;
   NewAreas := TEntity(AFL.Output);
   AFL.Free;
   E.Free;
   EndProgress;
end;

procedure TPolyLineAreaEditorNoFrame.RegisterCursors;
begin
   NewCursorName('SELECTAREA');
end;

procedure TPolyLineAreaEditorNoFrame.SetArea(const Value: TAreaElement);
begin
   ExchangeFieldElement(TElement(FArea), Value);
end;

procedure TPolyLineAreaEditorNoFrame.SetFrameAreas(const Value: TEntity);
begin
  FFrameAreas := Value;
end;

procedure TPolyLineAreaEditorNoFrame.SetFrameAreasLevel(const Value: string);
begin
   FFrameAreasLevel := Value;
   FrameAreas := FindLevel(Value);
end;

procedure TPolyLineAreaEditorNoFrame.SetGArea(const Value: TGeometryElement);
begin
   ExchangeFieldElement(TElement(FGArea), Value);
end;

procedure TPolyLineAreaEditorNoFrame.SetKillGeometryOnly(
  const Value: boolean);
begin
   FKillGeometryOnly := Value;
end;

procedure TPolyLineAreaEditorNoFrame.SetLines(const Value: TEntity);
begin
   FLines := Value;
end;

procedure TPolyLineAreaEditorNoFrame.SetLinesLevel(const Value: string);
begin
   FLinesLevel := Value;
   Lines := FindLevel(Value);
end;

procedure TPolyLineAreaEditorNoFrame.SetNewAreas(const Value: TEntity);
begin
  ExchangeFieldElement(TElement(FNewAreas),Value);
end;

procedure TPolyLineAreaEditorNoFrame.SetNewAreasLevel(const Value: string);
begin
   FNewAreasLevel := Value;
   NewAreas := FindLevel(Value);
end;

procedure TPolyLineAreaEditorNoFrame.SetOldAreas(const Value: TEntity);
begin
   FOldAreas := Value;
end;

procedure TPolyLineAreaEditorNoFrame.SetOldAreasLevel(const Value: string);
begin
   FOldAreasLevel := Value;
   OldAreas := FindLevel(Value);
end;

procedure TPolyLineAreaEditorNoFrame.SetX(const Value: double);
begin
   FX := Value;
end;

procedure TPolyLineAreaEditorNoFrame.SetY(const Value: double);
begin
   FY := Value;
end;

procedure TPolyLineAreaEditorNoFrame.XORDraw;
var
   i: integer;
   MA: TMultiAreaElement;

procedure PaintArea(anArea: TAreaElement);
var
   Are: TAreaElement;
begin
   if anArea = nil then exit;
   if Color = 0 then Color := $AAAAAA;
   Are := TPolyLineAreaElement(anArea).CreateAreaElement;
    Device.Brush.Color:= Color;
    if anArea = Area then
       Device.Brush.Style:= bsbDiagonal
    else
       Device.Brush.Style:= bsDiagCross;
    Device.Pen.Mode:= pmXOR;
    Device.Pen.Width:= 3;
    Device.Pen.Color:= Color;
    Device.Pen.Style:= psSolid;
    Device.PolyPolygon(Are);
    PostMsgToOwners(tm_Repaint,nil,false);
    Application.ProcessMessages;
    Are.Free;
end;

begin
   if (GArea = nil) or not (GArea.Geometry is TMultiAreaElement) then
      PaintArea(Area)
   else
   begin
      MA := TMultiAreaElement(GArea.Geometry);
      for i := 0 to MA.LastIndex do
      begin
         PaintArea(MA.AreaElement[i]);
      end;
   end;

end;

{ TPolyLineEditorNoFrame }


procedure TPolyLineEditorNoFrame.CHangeSelectCursor;
var
   d: double;
begin
   if QTFinder.Busy then exit;
   d := Device.Projection.GetModelLength(PixelTolerance);
   CursorIndex := 5;
   QTFinder.Input := LevelQT;
   QTFinder.X := ActX;
   QTFInder.Y := ActY;
   QTFinder.Tolerance := d;
   QTFinder.DoIt;
   if QTFinder.Output <> nil then
   begin
      CursorIndex := 0;
   end;
end;

procedure TPolyLineEditorNoFrame.ClearTopology(aLevel: TEntity; Rel,R:TRectEntity);
var
   W: TQTreeWalker;
   CL: TClearLines;
   El: TElement;
   i: integer;
   G: TGeometryElement;
   En: TEntity;
   RE: TRectEntity;
begin
   if LevelQT = nil then exit;
   W := TQTreeWalker.Create;
   El := W.GetFirstElementForRectangle(LevelQT,R.Minx,R.MinY,R.MaxX,R.MaxY,1);
   while El <> nil do
   begin
      if El.IsMyOwner(aLevel) then
         if not El.IsMyOwner(R) then
            R.AddAndUpdateRect(El);
      El := W.GetNextElement;
   end;
   // Uprava kvuli dlouhym liniim pres pul sveta
   if R.Count > 100 then
   begin
      En := TQuickEntity.Create;
      for i := 0 to R.LastIndex do
      begin
         El := R.Elements[i];
         if not (El is TRectEntity) then continue;
         RE := TRectEntity(El);
         if Rel.IsOverRegionEx(RE.MinX-1,RE.MinY-1,RE.MaxX+1,RE.MaxY+1) then
            En.AddElement(RE);
      end;
      R.FreeAll;
      R.AddAllFrom := En;
      R.ResetAndUpdate;
      En.Clear;
      En.Free;
   end;
   // konec upravy
   //RE.Name := 'Test';
   //AddElement(RE);
   CL := TClearLines.Create;
   CL.Input := R;
   CL.DoIt;
   for i := R.LastIndex downto 0 do
   begin
      El := R.Elements[i];
      if El is TGeometryElement then
      begin
         G := TGeometryElement(El);
         if G.NoGeometry then
            G.Free;
      end;
   end;
   CL.Free;
   W.Free;
end;

procedure TPolyLineEditorNoFrame.DeleteObject;
begin
   CreateNewObject;
   PostMsg(tm_ModelChanged,nil,ActualProject,false);
   Application.ProcessMessages;
   if not NewObject then
      on_Activate;
end;

procedure TPolyLineEditorNoFrame.FindLevel_ProjectEntity(aElem: TElement;  var aLevel, aProjE: TEntity);
begin
   aLevel := Level;
   aProjE := nil;
   ActualProjectName := 'Main';
   if aElem = nil then
      exit;
   if aElem.IsMyOwner(Level)then
      exit;
   aLevel := aElem.FindOwnerName(Level.Name);
   if aLevel = nil then exit;
   aProjE := aLevel.FindClassWithOwnerInOwners(TEntity,AuxProjects);
   if aProjE <> nil then
      ActualProjectName := aProjE.Name;
end;

procedure TPolyLineEditorNoFrame.NextNearLine;
var
   aLevel, aProjE: TEntity;
begin
   if NearLines.NoElements then exit;
   HighActivity := True;
   Input := NearLines.Elements[0];
   NearLines.RemoveElement(Input);
   aLevel := Level;
   FindLevel_ProjectEntity(Input,aLevel,aProjE);
   ActualLevel := aLevel;
   ActualProject := aProjE;
   RemoveElementOwners(Input);
   if Input is TGeometryElement then
      GElement := TGeometryElement(Input)
   else
      PolyElement := TPolyElement(Input);
   SendProjectChange;
   SendInfo;
   PolyElementBackUp := TPolyElement(CreateObjectCopy);
   EditorStatus := gesEdit;
   //inherited On_Activate;
   StartEditing;
   XORDraw;
   TestForDelete;
   TestForChangeGeometry;
   if not FreeToDelete or not FreeToChangeGeometry then
      ExecuteDialog;
end;

procedure TPolyLineEditorNoFrame.On_Activate;
begin
   HighActivity := true;
   EditorStatus := gesPrepare;
   ActualProjectName := 'Main';
   ActualProject := nil;
   ActualLevel := nil;
   SendProjectChange;
   if CanCreateNewObject then
   begin
      if not Active then
         PrepareQTree;
      EditorStatus := gesEdit;
      QTFinder.Busy := false;
      FreeToDelete := true;
      FreeToChangeGeometry := true;
      if Dialog <> nil then
         Dialog.SendStatus;
      ActualLevel := Level; // pro opravy se to nastavi pri vyberu linie
   end
   else
   begin
      if not Active then
         PrepareQTree;
      EditorStatus := gesSelect;
      QTFinder.Busy := false;
   end;
  inherited;
  HighActivity := false;
end;

procedure TPolyLineEditorNoFrame.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   if EditorStatus = gesPrepare then exit;
   if EditorStatus = gesSelect then
   begin
      exit;
   end
   else
     inherited;

end;

procedure TPolyLineEditorNoFrame.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
  aX, aY: integer;
begin
   if EditorStatus = gesPrepare then exit;
   aX:= aMessage.x;
   aY:= aMessage.y;
   ActX:= Device.Projection.GetModelX(aX);
   ActY:= Device.Projection.GetModelY(aY);
   if not Active then exit;
   if EditorStatus = gesSelect then
      ChangeSelectCursor
   else
   begin
      if PolyElement = nil then exit;
      ChangeCursor;
   end;
end;



procedure TPolyLineEditorNoFrame.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   aLevel,aProjE: TEntity;
begin
   if ZeroActivity then exit;
   if EditorStatus = gesPrepare then exit;
   if EditorStatus = gesSelect then
   begin
      if QTFinder.Output = nil then exit;
      HighActivity := True;
      FindNearLines;
      Input := QTFinder.Output;
      aLevel := Level;
      FindLevel_ProjectEntity(Input,aLevel,aProjE);
      ActualLevel := aLevel;
      ActualProject := aProjE;
      SendProjectChange;
      NearLines.RemoveElement(Input);
      RemoveElementOwners(Input);
      if Input is TGeometryElement then
         GElement := TGeometryElement(Input)
      else
         PolyElement := TPolyElement(Input);
      SendInfo;
      PolyElementBackUp := TPolyElement(CreateObjectCopy);
      //inherited On_Activate;
      StartEditing;
      XORDraw;
      TestForDelete;
      TestForChangeGeometry;
      EditorStatus := gesEdit;
      if not FreeToDelete or not FreeToChangeGeometry then
         aMessage.Button := mbRight;
      if aMessage.Button = mbRight then
         On_tm_MouseDown(aMessage,Handled);
   end
   else
     inherited;
end;

procedure TPolyLineEditorNoFrame.PrepareQTree;
var
   TQ: TAddEntityToQTree;
   e: TElement;
   En: TEntity;
   En1: TEntity;
   RE: TRectEntity;
   ii: integer;
begin
   LevelName := LevelName;
   if Level = nil then exit;
   StartProgress('Pprava vrstev...',0,0);
   RE := TRectEntity.Create;
   AddElement(RE);
   RE.AddAllFrom := Level;
   AuxProjects := nil;
   if AuxProjectsName <> '' then
   begin
      AuxProjects := FindLevel(AuxProjectsName);
      if AuxProjects <> nil then
      for ii  := 0 to AuxProjects.LastIndex do
      begin
         e := AuxProjects.Elements[ii];
         if not (e is TEntity) then continue;
         En := TEntity(e);
         En1 := FindLevelInEntity(En,AuxLevelName);
         if En1 <> nil then
            RE.AddAllFrom := En1;
      end;
   end;
   RE.Update;

   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := RE;
   TQ.Output := LevelQT;
   TQ.DoIt;
   if SnapTool <> nil then
      SnapTool.FrameQT := LevelQT;
   TQ.Free;
   EndProgress;
   PrepareSnapQTree;
end;

procedure TPolyLineEditorNoFrame.PrepareSnapQTree;
var
   SL: TStringList;
   RE: TRectEntity;
   E: TEntity;
   i: integer;
   ii: integer;
   EE: TElement;
   En: TEntity;
   LEX: TLineExtractor;
   TQ: TAddEntityToQTree;
begin
   if SnapLevels = '' then exit;
   if SnapTool = nil then exit;
   SL := TStringList.Create;
   SL.CommaText := SnapLevels;
   if SL.Count < 1 then
   begin
      SL.Free;
      exit;
   end;
   StartProgress('Pprava chapk...',0,0);
   RE := TRectEntity.Create;
   AddElement(RE);// uloen kvli lexu
   LEX := TLineExtractor.Create;
   for i := 0 to SL.Count - 1 do
   begin
      E := FindLevel(SL.Strings[i]);
      if E <> nil then
      begin
         ProgressText := 'Hledn lini v ' + E.Name;
         Application.ProcessMessages;
         LEX.Input := E;
         LEX.Output := RE;
         LEX.DoIt;
      end;
   end;
   SL.Free;
   LEX.Free;
   // AuxSnapLevels ---------------------------
   if (AuxProjects <> nil) and (AuxSnapLevels <> '') then
   begin
      SL := TStringList.Create;
      SL.CommaText := AuxSnapLevels;
      if SL.Count > 0 then
      begin
         LEX := TLineExtractor.Create;
         for i := 0 to SL.Count - 1 do
         begin
            for ii := 0 to AuxProjects.LastIndex do
            begin
               EE := AuxProjects.Elements[ii];
               if not (E is TEntity) then continue;
               En := TEntity(EE);
               E := FindLevelInEntity(En,SL.Strings[i]);
               if E <> nil then
               begin
                  ProgressText := 'Hledn lini v ' + E.Name;
                  Application.ProcessMessages;
                  LEX.Input := E;
                  LEX.Output := RE;
                  LEX.DoIt;
               end;
            end;
         end;
         SL.Free;
         LEX.Free;
      end;
   end;
   // AuxSnapLevels END --------------------------
   if RE.NoElements then
   begin
      RE.Free;
      EndProgress;
      exit;
   end;
   ProgressText := 'chapci lezou na strom';
   Application.ProcessMessages;
   SnapQT :=TRectEntityQTree.Create;
   SnapQT.Name := 'SNAP QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := RE;
   TQ.Output := SnapQT;
   TQ.DoIt;
   if SnapTool <> nil then
      SnapTool.SnapQT := SnapQT;
   TQ.Free;
   RE.Free;
   EndProgress;
end;

procedure TPolyLineEditorNoFrame.RegisterCursors;
begin
   NewCursorName('LINESELECT');
   NewCursorName('MOVE_POINT');
   NewCursorName('INSERT_POINT');
   NewCursorName('SNAP_LPOINT');
   NewCursorName('SNAP_LBODY');
   NewCursorName('NOSELECT');
end;


procedure TPolyLineEditorNoFrame.SendProjectChange;
begin
   if OnProjectChangeScript <> '' then
   begin
      PostTextMsg(tm_Run,OnProjectChangeScript,nil);
      Application.ProcessMessages;
   end;
end;

procedure TPolyLineEditorNoFrame.SetActualLevel(const Value: TEntity);
begin
  FActualLevel := Value;
end;

procedure TPolyLineEditorNoFrame.SetActualProject(const Value: TEntity);
begin
  FActualProject := Value;
end;

procedure TPolyLineEditorNoFrame.SetActualProjectName(const Value: string);
begin
  FActualProjectName := Value;
end;

procedure TPolyLineEditorNoFrame.SetAuxLevelName(const Value: string);
begin
  FAuxLevelName := Value;
end;

procedure TPolyLineEditorNoFrame.SetAuxProjects(const Value: TEntity);
begin
  FAuxProjects := Value;
end;

procedure TPolyLineEditorNoFrame.SetAuxProjectsName(const Value: string);
begin
  FAuxProjectsName := Value;
end;

procedure TPolyLineEditorNoFrame.SetAuxSnapLevels(const Value: string);
begin
  FAuxSnapLevels := Value;
end;

procedure TPolyLineEditorNoFrame.SetLevel(const Value: TEntity);
begin
  FLevel := Value;
end;

procedure TPolyLineEditorNoFrame.SetLevelName(const Value: string);
begin
   FLevelName := Value;
   Level := FindLevel(Value);
end;

procedure TPolyLineEditorNoFrame.SetLevelQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLevelQT), Value);
end;

procedure TPolyLineEditorNoFrame.SetOnProjectChangeScript(const Value: string);
begin
  FOnProjectChangeScript := Value;
end;

procedure TPolyLineEditorNoFrame.SetPixelTolerance(const Value: integer);
begin
  FPixelTolerance := Value;
end;


procedure TPolyLineEditorNoFrame.SetSnapLevels(const Value: string);
begin
   FSnapLevels := Value;
end;

procedure TPolyLineEditorNoFrame.SetSnapQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FSnapQT), Value);
end;

procedure TPolyLineEditorNoFrame.SetTopology(const Value: boolean);
begin
   FTopology := Value;
end;

procedure TPolyLineEditorNoFrame.StoreObject;
var
   R: TRectEntity;
   GC: TGeometryElementCollection;
   aLevel,aProjE: TEntity;
begin
   if (PolyElement <> nil) and (PolyElement.Count > 1) then
   begin
      //if not (input is TRectEntity) then
      //   Exit;
      PolyElement.RecalculateRect;
      PolyElement.CalculateLabelProps;
      if GElement <> nil then
         R := GElement
      else
         R := PolyElement;
      //if (PolyElement <> nil) and (PolyElement.Count > 1) then
      begin
         aLevel := ActualLevel;
         aProjE := ActualProject;
         if aLevel <> nil then
         begin
            if aLevel is TRectEntity then
               TRectEntity(aLevel).AddAndUpdateRect(R)
            else
               aLevel.AddElement(R);
         end;
         if LevelQT <> nil then
            LevelQT.NextRectEntity := R;
         if Modified then PostMsg(tm_ModelChanged,nil,aProjE,false);
      end;
      //TRectEntity(Input).AddAndUpdateRect(PolyElement);
      //PolyElement:= nil;
      XORDraw;
      if Topology and Modified then
      begin
         GC := TGeometryElementCollection.Create;
         GC.UpdateRect(R);
         PolyElement := nil;
         GElement := nil;
         ClearTopology(aLevel,R,GC);
         //PostMsgToOwners(tm_Review, GC, True);
         PostMsgToOwners(tm_ReviewByElement, GC, True);
      end
      else
         //PostMsgToOwners(tm_Review, R, false);
         PostMsgToOwners(tm_ReviewByElement, R, false);
   end;
   //if HighActivity then HighActivity := false;
   PolyElement := nil;
   GElement := nil;
   PolyElementBackUp := nil;
   Modified := false;
   Swap := false;
   On_Activate;
   Application.ProcessMessages;
   Application.ProcessMessages;
   //CreateNewObject;
end;

{ TSimpleGraphicEditor }

procedure TSimpleGraphicEditor.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   S: string;
begin
   S := OnMouseDownScript;
   if S <> '' then
   begin
      ProcessMouseEvent(aMessage);
      PostTextMsg(tm_Run,S,nil);
      Application.ProcessMessages;
   end;
end;

procedure TSimpleGraphicEditor.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   S: string;
begin
   S := OnMouseUpScript;
   if S <> '' then
   begin
      ProcessMouseEvent(aMessage);
      PostTextMsg(tm_Run,S,nil);
      Application.ProcessMessages;
   end;
end;

procedure TSimpleGraphicEditor.ProcessMouseEvent(aMessage: TOrbisMouseEvent);
begin
   X := Device.Projection.GetModelX(aMessage.x);
   Y := Device.Projection.GetModelY(aMessage.y);
   MB := Ord(aMessage.Button);
end;

procedure TSimpleGraphicEditor.SetMB(const Value: integer);
begin
  FMB := Value;
end;

procedure TSimpleGraphicEditor.SetOnMouseDownScript(const Value: string);
begin
  FOnMouseDownScript := Value;
end;

procedure TSimpleGraphicEditor.SetOnMouseUpScript(const Value: string);
begin
  FOnMouseUpScript := Value;
end;

procedure TSimpleGraphicEditor.SetX(const Value: double);
begin
  FX := Value;
end;

procedure TSimpleGraphicEditor.SetY(const Value: double);
begin
  FY := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSimpleGraphicEditor);
   RegisterOrbisClass(TPolyLineEditor);
   RegisterOrbisClass(TPolyLineEditorNoFrame);
   RegisterOrbisClass(TPolyLineAreaEditor);
   RegisterOrbisClass(TPolyLineAreaEditorNoFrame);
   RegisterOrbisClass(TPointEditor);
   RegisterOrbisClass(TShapeEditor);
end.
