unit t_JoinLines;

interface
uses
   Elements,
   e_GeometryElement,
   Tools;

type

TJoinSimpleLines = class(TTool)
public
   procedure DoIt; override;
end;

TJoinTopolLines = class(TTool)
  private
    FLines: TGeometryElementCollection;
    FRefLines: TEntity;
    FIgnorePresnost: boolean;
    FJoinRefLines: boolean;
    FSameDirection: boolean;
    procedure SetRefLines(const Value: TEntity);
    function GetLines: TGeometryElementCollection;
    procedure SetIgnorePresnost(const Value: boolean);
    procedure SetJoinRefLines(const Value: boolean);
    procedure SetSameDirection(const Value: boolean);
    procedure SetNextSwitch(const Value: string);
public
   procedure DoIt; override;
   property  Lines: TGeometryElementCollection read GetLines;
   function  NumberToPropertyName(aNumber: longInt): string;override;
published
   property  RefLines: TEntity read FRefLines write SetRefLines;
   property  JoinRefLines: boolean read FJoinRefLines write SetJoinRefLines;
   property  IgnorePresnost: boolean read FIgnorePresnost write SetIgnorePresnost;
   property  SameDirection: boolean read FSameDirection write SetSameDirection;
   property  NextSwitch: string write SetNextSwitch stored false;
end;

implementation
uses
   Dialogs,
   Forms,
   SysUtils,
   e_Node,
   e_Class,

   e_Geometry,
   e_TopolElements,
   T_FindBreakLineConnections;

{ TJoinSimpleLines }

procedure TJoinSimpleLines.DoIt;
var
  FB: TFindBreakLineConnections;
  E: TEntity; // entita uzl
  El: TElement;
  N,NN: TNode;
  i,j: integer;
  //TL1,TL2,TL: TTopolLineElement;
  L1,L2,L: TPolyLineElement;
  aX,aY,X1,Y1: double;
  B1,B2: boolean;
  NB1,NB2,NB: TNodeBranch;
  LinesEntity: TEntity;
begin
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   LinesEntity := TEntity(Input);
   if LinesEntity.NoElements then exit;
   StartProgress('Spojen lini...',0,0);
   Application.ProcessMessages;
   FB := TFindBreakLineConnections.Create;
   FB.ConnectionsNumber := 2; //budu hledat uzly dvou lini
   FB.Input := LinesEntity;
   FB.DoIt;
   E := TEntity(FB.output);
   FB.Free;
   MaxProgress := E.LastIndex;
   for i := 0 to E.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      N := TNode(E.Elements[i]);
      NN := N;
      L1 := nil;
      L2 := nil;
      NB1 := nil;
      NB2 := nil;
      for j := 0 to N.LastIndex do
      begin
         El := N.Elements[j];
         if not (El is TNodeBranch) then continue;
         NB := TNodeBranch(El);
         if L1 = nil then
         begin
            NB1 := NB;
            L1 := NB1.Line;
            //TL1 := TTopolLineElement(L1.FindClassWithOwnerInOwners(TTopolLineElement,LinesEntity));
         end
         else
         begin
            NB2 := NB;
            L2 := NB2.Line;
            //TL2 := TTopolLineElement(L2.FindClassWithOwnerInOwners(TTopolLineElement,LinesEntity));
            Break;
         end;
      end;
      if (L1 = nil) or (L2 = nil) then continue;
      if L1 = L2 then continue;
      {
      if (TL1 = nil) or (TL2 = nil) then continue;
      if (TL1.Druh <> TL2.Druh) or (TL1.Presnost <> TL2.Presnost) then continue;
      if (TL1.Symbol > 0) and (TL2.Symbol > 0) then continue;
      if (TL1.Symbol <= 0) and (TL2.Symbol > 0) then
      begin
         TL := TL2;
         L := L2;
         NB := NB2;
         TL2 := TL1;
         L2 := L1;
         NB2 := NB1;
         TL1 := TL;
         L1 := L;
         NB1 := NB;
      end;
      aX := TL1.AbsRefX;
      aY := TL1.AbsRefY;
      }
      for j := 1 to L2.LastIndex do
      begin
         L2.GetOrientedCoord(j,NB2.LineBegin,X1,Y1);
         if NB1.LineBegin then //bude se pidvat na zatek
            L1.InsertXY(0,X1,Y1)
         else                  //nebo na konec
            L1.AddXY(X1,Y1);
      end;
      L1.RecalculateRect;
      L1.CalculateLabelProps;
      //L1.AbsRefX := aX;
      //L1.AbsRefY := aY;
      NN.FreeAll;
      GiveNodeBranchForLine(L2, not(NB2.LineBegin),N,NB);
      if (N <> nil) and (NB <> nil) then
      begin
         N.RemoveElement(L2);
         N.AddElement(L1);
         NB.Line := L1;
         NB.LineBegin := NB1.LineBegin;
         //NB.Free;
      end;
      //if NB2 <> nil then NB2.Free;
      //TL2.Free;
      L2.Free;
   end;
   E.Free;
   //Progress := 0;
end;

{ TJoinTopolLines }

procedure TJoinTopolLines.DoIt;
var
  FB: TFindBreakLineConnections;
  E,E1: TEntity; // entita uzl
  El: TElement;
  N,NN: TNode;
  i,j: integer;
  TL1,TL2,TL: TTopolLineElement;
  L1,L2,L: TPolyLineElement;
  aX,aY,X1,Y1: double;
  B1,B2: boolean;
  NB1,NB2,NB: TNodeBranch;
  LinesEntity: TEntity;
begin
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   LinesEntity := TEntity(Input);
   if LinesEntity.NoElements then exit;
   Lines.AddAllFrom := LinesEntity;
   if (RefLines <> nil) and not RefLines.NoElements then
      Lines.AddAllFrom := RefLines;
   Lines.Update;
   StartProgress('Spojen T lini...',0,0);
   Application.ProcessMessages;
   FB := TFindBreakLineConnections.Create;
   FB.ConnectionsNumber := 2; //budu hledat uzly dvou lini
   FB.Input := Lines;
   FB.DoIt;
   E := TEntity(FB.output);
   FB.Free;
   if E = nil then
   begin
      ShowMessage('Spojen lini se z njakho dvodu nepodailo, asi bude pret');
      exit;
   end;
   MaxProgress := E.LastIndex;
   for i := 0 to E.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      N := TNode(E.Elements[i]);
      NN := N;
      L1 := nil;
      L2 := nil;
      TL1 := nil;
      TL2 := nil;
      NB1 := nil;
      NB2 := nil;
      for j := 0 to N.LastIndex do
      begin
         El := N.Elements[j];
         if not (El is TNodeBranch) then continue;
         NB := TNodeBranch(El);
         E1 := nil;
         if L1 = nil then
         begin
            NB1 := NB;
            L1 := NB1.Line;
            TL1 := TTopolLineElement(L1.FindClassWithOwnerInOwners(TTopolLineElement,LinesEntity));
            if RefLines <> nil then
               E1 := TTopolLineElement(L1.FindClassWithOwnerInOwners(TTopolLineElement,RefLines));
            if (E1 <> nil) and not JoinRefLines  then
            begin
               L1 := nil;
               break;
            end;
         end
         else
         begin
            NB2 := NB;
            L2 := NB2.Line;
            TL2 := TTopolLineElement(L2.FindClassWithOwnerInOwners(TTopolLineElement,LinesEntity));
            if RefLines <> nil then
               E1 := TTopolLineElement(L1.FindClassWithOwnerInOwners(TTopolLineElement,RefLines));
            if (E1 <> nil) and not JoinRefLines  then
               L2 := nil;
            Break;
         end;
      end;
      if (L1 = nil) or (L2 = nil) then continue;
      if L1 = L2 then continue;
      if (TL1 = nil) or (TL2 = nil) then continue;
      if (TL1.Druh <> TL2.Druh)then continue;
      if (TL1.Presnost <> TL2.Presnost) and not IgnorePresnost then continue;
      if (NB1.LineBegin = NB2.LineBegin) and SameDirection then continue;
      if (TL1.Symbol > 0) and (TL2.Symbol > 0) then continue;
      if (TL1.Symbol <= 0) and (TL2.Symbol > 0) then
      begin
         TL := TL2;
         L := L2;
         NB := NB2;
         TL2 := TL1;
         L2 := L1;
         NB2 := NB1;
         TL1 := TL;
         L1 := L;
         NB1 := NB;
      end;
      aX := TL1.AbsRefX;
      aY := TL1.AbsRefY;
      for j := 1 to L2.LastIndex do
      begin
         L2.GetOrientedCoord(j,NB2.LineBegin,X1,Y1);
         if NB1.LineBegin then //bude se pidvat na zatek
            L1.InsertXY(0,X1,Y1)
         else                  //nebo na konec
            L1.AddXY(X1,Y1);
      end;
      L1.RecalculateRect;
      //L1.CalculateLabelProps;
      L1.AbsRefX := aX;
      L1.AbsRefY := aY;
      NN.FreeAll;
      GiveNodeBranchForLine(L2, not(NB2.LineBegin),N,NB);
      if (N <> nil) and (NB <> nil) then
      begin
         N.RemoveElement(L2);
         N.AddElement(L1);
         NB.Line := L1;
         NB.LineBegin := NB1.LineBegin;
         //NB.Free;
      end;
      //if NB2 <> nil then NB2.Free;
      L2.Free;
      TL2.Free;
   end;
   E.Free;
   //Progress := 0;
end;


function TJoinTopolLines.GetLines: TGeometryElementCollection;
begin
   if FLines = nil then
   begin
      FLines := TGeometryElementCollection.Create;
      AddElement(FLines);
   end;
   result := FLines;
end;

function TJoinTopolLines.NumberToPropertyName(aNumber: Integer): string;
begin
   Result := 'NextSwitch';
end;

procedure TJoinTopolLines.SetIgnorePresnost(const Value: boolean);
begin
  FIgnorePresnost := Value;
end;

procedure TJoinTopolLines.SetJoinRefLines(const Value: boolean);
begin
  FJoinRefLines := Value;
end;

procedure TJoinTopolLines.SetNextSwitch(const Value: string);
var
   S: string;
   Ch: char;
begin
   if Value = '' then exit;
   S := UpperCase(Value);
   CH := S[1];
   case CH of
      'P': IgnorePresnost := true;
      'D': SameDirection := true;
   end;
end;

procedure TJoinTopolLines.SetRefLines(const Value: TEntity);
begin
  FRefLines := Value;
end;

procedure TJoinTopolLines.SetSameDirection(const Value: boolean);
begin
  FSameDirection := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TJoinSimpleLines);
   RegisterOrbisClass(TJoinTopolLines);
END.

