unit t_LineDialog;

interface
uses
   f_LineDialog,
   t_OrbisDialog,
   t_Snap;

type

TSimpleLineDialog = class(TOrbisDialog)
private
    FSnapTool: TPointLineSnapTool;
    FFreeToDelete: boolean;
    FFreeToChangeGeometry: boolean;
    FAllowSwapDirection: boolean;
    procedure SetSnapTool(const Value: TPointLineSnapTool);
public
    procedure SetFreeToDelete(const Value: boolean); virtual;
    procedure SetFreeToChangeGeometry(const Value: boolean); virtual;
    procedure SetAllowSwapDirection(const Value: boolean); virtual;
published
   property  SnapTool: TPointLineSnapTool read FSnapTool write SetSnapTool;
   property  FreeToDelete: boolean read FFreeToDelete write SetFreeToDelete;
   property  FreeToChangeGeometry: boolean read FFreeToChangeGeometry write SetFreeToChangeGeometry;
   property  AllowSwapDirection: boolean read FAllowSwapDirection write SetAllowSwapDirection;
end;



TLineDialog = class(TSimpleLineDialog)
private
    function GetForm: TLineDialog_Form;
public
   procedure SetFreeToDelete(const Value: boolean); override;
   procedure SetFreeToChangeGeometry(const Value: boolean); override;
   procedure SetAllowSwapDirection(const Value: boolean); override;
   procedure DoIt; override;
   procedure CreateForm; override;
   property  Form: TLineDialog_Form read GetForm;
end;


implementation
uses
   Forms,
   SysUtils,
   e_Class,
   e_Geometry,
   e_GeometryElement;
   //e_Space;

{ TLineDialog }

procedure TLineDialog.CreateForm;
begin
   aForm := TLineDialog_Form.Create(Application);
   aForm.Tool := Self;
   aForm.IsPossibleToClose := false;
end;

procedure TLineDialog.DoIt;
var
   Lin: TPolyLineElement;
   //S: string;
begin
   if input = nil then exit;
   if (Input is TGeometryElement) then
   begin
      if not (TGeometryElement(Input).Geometry is TPolyLineElement) then
         exit
      else
         Lin :=TPolyLineElement(TGeometryElement(Input).Geometry);
   end
   else
   begin
      if not(Input is TPolyLineElement) then exit;
      Lin := TPolyLineElement(Input);
   end;
   Form.l_length.Caption := FormatFloat('0.000;;Neni co mit',Lin.Length);
   Form.l_Count.Caption := IntToStr(Lin.Count);
   {
   if AllowSwapDirection then
   begin
      Form.b_Close.Caption := 'Otoit';
   end;
   }
   if SnapTool <> nil then
   begin
      Form.cb_SnapToLine.Checked := SnapTool.SnapToLine;
      Form.cb_SnapToLinePoint.CHecked := SnapTool.SnapToLinePoint;
      Form.cb_SnapToLineBody.Checked := SnapTool.SnapToLineBody;
      Form.ed_SnapDistance.Text := SnapTool.GetPropertyStringByName('SnapDistance');
      Form.CHeckSnap;
   end;
   inherited;
   if SnapTool <> nil then
   begin
      SnapTool.SnapToLine := Form.cb_SnapToLine.Checked;
      SnapTool.SnapToLinePoint := Form.cb_SnapToLinePoint.CHecked;
      SnapTool.SnapToLineBody := Form.cb_SnapToLineBody.Checked;
      SnapTool.SetPropertyStringByName('SnapDistance',Form.ed_SnapDistance.Text);
   end;
end;

function TLineDialog.GetForm: TLineDialog_Form;
begin
   result := TLineDialog_Form(aForm);
end;

procedure TLineDialog.SetFreeToDelete(const Value: boolean);
begin
   FFreeToDelete := Value;
   Form.b_DeleteLine.Visible := Value;
end;

procedure TLineDialog.SetFreeToChangeGeometry(const Value: boolean);
begin
   FFreeToChangeGeometry := Value;
   Form.b_DeletePoint.Visible := Value;
   Form.b_Close.Visible := Value;
   Form.b_Swap.Visible := Value;
   Form.b_Continue.Visible := Value;
   Form.b_CreateCopy.Visible := not Value and FreeToDelete;
end;


procedure TLineDialog.SetAllowSwapDirection(const Value: boolean);
begin
  FAllowSwapDirection := Value;
end;

{ TSimpleLineDialog }

procedure TSimpleLineDialog.SetAllowSwapDirection(const Value: boolean);
begin
  FAllowSwapDirection := Value;
end;

procedure TSimpleLineDialog.SetFreeToDelete(const Value: boolean);
begin
   FFreeToDelete := Value;
end;

procedure TSimpleLineDialog.SetFreeToChangeGeometry(const Value: boolean);
begin
   FFreeToChangeGeometry := Value;
end;

procedure TSimpleLineDialog.SetSnapTool(const Value: TPointLineSnapTool);
begin
   FSnapTool := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSimpleLineDialog);
   RegisterOrbisClass(TLineDialog);
END.
