unit t_Snap;

interface
uses
   Elements,
   e_QTree,
   Tools;

type

TSnapTool = class (TTool)
  private
    FLastDistance: double;
    FSnapStatus: integer;
    procedure SetLastDistance(const Value: double);
    procedure SetSnapStatus(const Value: integer);
private
   FSnapped: boolean;
   FX: double;
   FSnapX: double;
   FSnapY: double;
   FSnapDistance: double;
   FY: double;
   procedure SetSnapDistance(const Value: double);
   procedure SetSnapped(const Value: boolean);
   procedure SetSnapX(const Value: double);
   procedure SetSnapY(const Value: double);
   procedure SetX(const Value: double);
   procedure SetY(const Value: double);
   property  LastDistance: double read FLastDistance write SetLastDistance;
published
   property SnapDistance: double read FSnapDistance write SetSnapDistance;
   property Snapped: boolean read FSnapped write SetSnapped;
   property X: double read FX write SetX;
   property Y: double read FY write SetY;
   property SnapX: double read FSnapX write SetSnapX;
   property SnapY: double read FSnapY write SetSnapY;
   property SnapStatus: integer read FSnapStatus write SetSnapStatus;
end;



TPointLineSnapTool = class(TSnapTool)
  private
    FSnapToLinePoint: boolean;
    FSnapToLine: boolean;
    FSnapToLineBody: boolean;
    FSnapToPoint: boolean;
    FSnapLines: TQuickEntity;
    FQTWalker: TQtreeWalker;
    FFrameQT: TRectEntityQTree;
    FTabuElement: TElement;
    FSnapQT: TRectEntityQTree;
    FSnapIndex: integer;
    procedure SetSnapToLineBody(const Value: boolean);
    procedure SetSnapToLinePoint(const Value: boolean);
    procedure SetSnapToPoint(const Value: boolean);
    function GetSnapLines: TQuickEntity;
    function GetQTWalker: TQtreeWalker;
    procedure SetSnapToLine(const Value: boolean);
    procedure SetFrameQT(const Value: TRectEntityQTree);
    procedure SetTabuElement(const Value: TElement);
    procedure SetSnapQT(const Value: TRectEntityQTree);
    procedure SetSnapIndex(const Value: integer);
public
   destructor Destroy; override;
   procedure DoIt; override;
   procedure FindLinesInFrame;
   procedure TrySnapToLinePoint;
   procedure TrySnapToLineBody;
   property  QTWalker: TQtreeWalker read GetQTWalker;
published
   property  FrameQT: TRectEntityQTree read FFrameQT write SetFrameQT;
   property  SnapQT: TRectEntityQTree read FSnapQT write SetSnapQT;
   property  TabuElement: TElement read FTabuElement write SetTabuElement;
   property  SnapLines: TQuickEntity read GetSnapLines;
   property SnapToPoint: boolean read FSnapToPoint write SetSnapToPoint;
   property SnapToLine: boolean read FSnapToLine write SetSnapToLine;
   property SnapToLinePoint: boolean read FSnapToLinePoint write SetSnapToLinePoint;
   property SnapToLineBody: boolean read FSnapToLineBody write SetSnapToLineBody;
   property SnapIndex: integer read FSnapIndex write SetSnapIndex;
end;

implementation
uses
   e_Class,
   e_Geometry,
   e_GeometryElement,
   MathLib;

{ TSnapTool }

procedure TSnapTool.SetLastDistance(const Value: double);
begin
   FLastDistance := Value;
end;

procedure TSnapTool.SetSnapDistance(const Value: double);
begin
   FSnapDistance := Value;
end;

procedure TSnapTool.SetSnapped(const Value: boolean);
begin
   FSnapped := Value;
end;

procedure TSnapTool.SetSnapX(const Value: double);
begin
   FSnapX := Value;
end;

procedure TSnapTool.SetSnapY(const Value: double);
begin
   FSnapY := Value;
end;

procedure TSnapTool.SetSnapStatus(const Value: integer);
begin
  FSnapStatus := Value;
end;

procedure TSnapTool.SetX(const Value: double);
begin
   FX := Value;
end;

procedure TSnapTool.SetY(const Value: double);
begin
   FY := Value;
end;

{ TPointLineSnapTool }

destructor TPointLineSnapTool.Destroy;
begin
   if FSnapLines <> nil then
   begin
      FSnapLines.Clear;
      FSnapLines.Free;
   end;
   if FQTWalker <> nil then
      FQTWalker.Free;
   inherited;
end;

procedure TPointLineSnapTool.DoIt;
begin
   Snapped := false;
   SnapStatus := 0;
   SnapIndex := -1;
   Output := nil;
   if FrameQT = nil then exit;
   LastDistance := SnapDistance * 2;
   if not SnapToLine then exit;
   FindLinesInFrame;
   if SnapToLinePoint then
      TrySnapToLinePoint;
   if Snapped then
   begin
      SnapStatus := 1;
      exit;
   end;
   if SnapToLineBody then
      TrySnapToLineBody;
   if Snapped then
      SnapStatus := 2;
end;

procedure TPointLineSnapTool.FindLinesInFrame;
var
   El: TElement;
   G: TElement;
begin
   SnapLines.Clear;
   if FrameQT = nil then exit;
   El := QTWalker.GetFirstElementForRectangle(FrameQT,X-SnapDistance,Y-SnapDistance,X+SnapDistance,Y+SnapDistance,1);
   while El <> nil do
   begin
      if El is TPolyLineElement then
         SnapLines.AddElement(El)
      else
      if El is TGeometryElement then
      begin
         G := TGeometryElement(El).Geometry;
         if G is TPolyLineElement then
            SnapLines.AddElement(G)
      end;
      El := QTWalker.GetNextElement;
   end;
   //jet dohledn ve snap QTree
   if SnapQT = nil then exit;
   El := QTWalker.GetFirstElementForRectangle(SnapQT,X-SnapDistance,Y-SnapDistance,X+SnapDistance,Y+SnapDistance,1);
   while El <> nil do
   begin
      if El is TPolyLineElement then
         SnapLines.AddElement(El)
      else
      if El is TGeometryElement then
      begin
         G := TGeometryElement(El).Geometry;
         if G is TPolyLineElement then
            SnapLines.AddElement(G)
      end;
      El := QTWalker.GetNextElement;
   end;
end;


function TPointLineSnapTool.GetQTWalker: TQtreeWalker;
begin
   if FQTWalker = nil then
      FQTWalker := TQTreeWalker.Create;
   result := FQTWalker;
end;

function TPointLineSnapTool.GetSnapLines: TQuickEntity;
begin
   if FSnapLines = nil then
      FSnapLines := TQuickEntity.Create;
   result := FSnapLines;
end;

procedure TPointLineSnapTool.SetFrameQT(const Value: TRectEntityQTree);
begin
   FFrameQT := Value;
end;

procedure TPointLineSnapTool.SetSnapIndex(const Value: integer);
begin
  FSnapIndex := Value;
end;

procedure TPointLineSnapTool.SetSnapQT(const Value: TRectEntityQTree);
begin
  FSnapQT := Value;
end;

procedure TPointLineSnapTool.SetSnapToLine(const Value: boolean);
begin
  FSnapToLine := Value;
end;

procedure TPointLineSnapTool.SetSnapToLineBody(const Value: boolean);
begin
  FSnapToLineBody := Value;
end;

procedure TPointLineSnapTool.SetSnapToLinePoint(const Value: boolean);
begin
  FSnapToLinePoint := Value;
end;

procedure TPointLineSnapTool.SetSnapToPoint(const Value: boolean);
begin
  FSnapToPoint := Value;
end;

procedure TPointLineSnapTool.SetTabuElement(const Value: TElement);
begin
  FTabuElement := Value;
end;

procedure TPointLineSnapTool.TrySnapToLineBody;
var
   i,j: integer;
   L: TPolyLineElement;
   X1,Y1,X2,Y2,pX,pY: double;
   aDistance: double;
   IsIn: boolean;
begin
   Snapped := false;
   if SnapLines.NoElements then exit;
   for i := 0 to SnapLines.LastIndex do
   begin
      L := TPolyLineElement(SnapLines.Elements[i]);
      if L = TabuElement then continue;
      if L.CoordsCount < 2 then continue;
      for j := 0 to L.CoordsCount - 2 do
      begin
         X1 := L.Xi[j];
         Y1 := L.Yi[j];
         X2 := L.Xi[j+1];
         Y2 := L.Yi[j+1];
         if not RectanglesOverlay(X1, Y1, X2, Y2, X-SnapDistance, Y-SnapDistance,
                                                  X+SnapDistance, Y+SnapDistance,0) then
            continue;
         DejVztahyBod_Usecka( X, Y, X1, Y1, X2, Y2, IsIn, pX, pY, aDistance);
         if IsIn and (aDistance <= SnapDistance) and (aDistance < LastDistance) then
         begin
            Snapped := True;
            LastDistance := aDistance;
            Output := L;
            SnapX := pX;
            SnapY := pY;
         end;
      end;
   end;
end;

procedure TPointLineSnapTool.TrySnapToLinePoint;
var
   i,j: integer;
   L: TPolyLineElement;
   X1,Y1: double;
   aDistance: double;
begin
   Snapped := false;
   if SnapLines.NoElements then exit;
   for i := 0 to SnapLines.LastIndex do
   begin
      L := TPolyLineElement(SnapLines.Elements[i]);
      if L = TabuElement then continue;
      for j := 0 to L.CoordsCount - 1 do
      begin
         X1 := L.Xi[j];
         Y1 := L.Yi[j];
         if (abs(X1-X) <= SnapDistance) and (abs(Y1-Y) <= SnapDistance) then
         begin
            aDistance := Distance(X,Y,X1,Y1);
            if (aDistance <= SnapDistance) and (aDistance < LastDistance) then
            begin
               Snapped := True;
               LastDistance := aDistance;
               Output := L;
               SnapX := X1;
               SnapY := Y1;
               SnapIndex := j;
            end;
         end;
      end;
   end;
end;

INITIALIZATION
   RegisterOrbisClass(TPointLineSnapTool);
END.
