{$D+}
unit T_layscr;

interface
uses
  Classes,
// KERNEL
  Elements,
  E_Type,
  t_Recurs,
  Tools,
// GISLAYER
  E_LayScr;

type

TModelScriptCreator = class(TTool)
{ Input .. nic nebo StringListElement
           kdy nic, tak lze naplovat postupn pes property
  Output .. ModelScript
}
private
  FModelScript: TModelScript;
  FLayerScript: TLayerScript;
  FDataDirectory: string;
  procedure AddLayer(aName, aValue: string);
  procedure AddLayerProperty(aLayerProperty: string);
  procedure AddLayerItem(aLayer: string);
  procedure AddLayers(aLayers: TStringListElement);
public
  procedure DoIt; override; { napln modelscript a ped na vstup }
published
  property DataDirectory: string read FDataDirectory write FDataDirectory;
  property NextLayer: string write AddLayerItem;
  property NextLayerProperty: string write AddLayerProperty;
  property Layers: TStringListElement write AddLayers;
end;


TStringListFromModelScript = class(TRecursor)
 protected
    function  QProcessBefore(Element: TElement): boolean; override;
    procedure Process(Element: TElement); override;
public
    procedure DoIt; override;
end;

TModelScriptToDynamicModel = class(TTool)
public
  procedure DoIt; override;
end;



implementation
uses
  SysUtils,
// KERNEL
  E_Class,
  U_String,
// GISLAYER
  e_DynM;

{ TStringListFromModelScript }

procedure TStringListFromModelScript.DoIt;
begin
   if Input = nil then exit;
   if not (Input is TModelScript) then exit;
   Output := TStringListElement.Create;
   inherited DoIt;
end;

function  TStringListFromModelScript.QProcessBefore(Element: TElement): boolean;
begin
   result := true;
end;

procedure TStringListFromModelScript.Process(Element: TElement);
var
   C: TElementClass;
begin
   C := TElementClass(Element.ClassType);
   if C = TLayerScript then
   begin
      TStringListElement(Output).AddString(Element.Name+'='+TLayerScript(Element).Description);
   end
   else
   if C = TStringElement then
   begin
      TStringListElement(Output).AddString(TStringElement(Element).NameIsValue);
   end;
end;



{ TModelScriptCreator }

procedure TModelScriptCreator.AddLayer(aName, aValue: string);
begin
  if FModelScript = nil then
  begin
    FModelScript:= TModelScript.Create;
  end;
  if FModelScript = nil then exit;

  FLayerScript:= TLayerScript.Create;
  FLayerScript.Name:= aName;
  FLayerScript.Description:= aValue;

  FModelScript.AddElement(FLayerScript);
end;


procedure TModelScriptCreator.AddLayerProperty(aLayerProperty: string);
var
  aName, aValue: string;
begin
  if not StringAsParameter(aLayerProperty, aName, aValue) then exit;
  if aName = '' then exit;

  if FLayerScript <> nil then
     FLayerScript.AddProperty(TStringElement.CreateAll(aName, aValue))
end;


procedure TModelScriptCreator.AddLayerItem(aLayer: string);
var
  aName, aValue: string;
begin
  if not StringAsParameter(aLayer, aName, aValue) then exit;
  if aName = '' then exit;

  if (CompareText(aName,'LIN')=0) or (CompareText(aName,'ARE')=0) or
     (CompareText(aName,'PNT')=0) or (CompareText(aName,'TXT')=0) or
     (CompareText(aName,'LINSYM')=0) or
     (CompareText(aName,'ELEMENT')=0) or
     (CompareText(aName,'RAS')=0) or
     (CompareText(aName,'FILE')=0) or
     (CompareText(aName,'TAB')=0) then
     AddLayer(aName, aValue)
  else
  if aName[1] in ['@','!','<','>','+','-'] then
     AddLayerProperty(aLayer)
end;


procedure TModelScriptCreator.AddLayers(aLayers: TStringListElement);
var
  I: longInt;
begin
  if aLayers = nil then exit;
  for I:= 0 to aLayers.Count-1 do
  begin
    AddLayerItem(aLayers.Names[I] + '=' + aLayers.Strings[I])
  end
end;



procedure TModelScriptCreator.DoIt;
begin
  if Input is TStringListElement then
  begin
    AddLayers(TStringListElement(Input))
  end;

{  Input.Free; !!!!!!!!!!!!!!!!!!!!!!!!!!!!! pro to tady bylo ?
   kvli Mapperu ? }

  if FModelScript = nil then exit;
  try
    FModelScript.FillWildNames(CheckBackSlash(FDataDirectory));
  except
  end;

  Output:= FModelScript
end;


{ TModelScriptToDynamicModel }

procedure TModelScriptToDynamicModel.DoIt;
var
  aMS, aMSOut: TModelScript;
  DM: TDynamicModel;
  aLayer, aLayerOut: TLayerScript;
  aPL: TPropertyList;
  I, J: longInt;
  aPath: TPathElement;
  aPathS: string;
begin
  if Output = nil then
  begin
    Output:= TDynamicModel.Create;
  end;
  if not (Output is TDynamicModel) then exit;
  if not (Input is TModelScript) then exit;
  DM := TDynamicModel(Output);
  DM.Active := false;

  aMS:= TModelScript(Input);
  for I:= 0 to aMS.Count-1 do
  begin
    aLayer:= TLayerScript(aMS.Elements[I]);
    if not (aLayer is TLayerScript) then continue;
    aPL:= TPropertyList(aLayer.FindElementClass(TPropertyList));

    for J:= 0 to aLayer.Count-1 do
    begin
      aPath:= TPathElement(aLayer.Elements[J]);
      if not (aPath is TPathElement) then continue;
      aPathS := aPath.Value;
      aPathS := ExtractFilePath(aPathS);
      CheckBackSlash(aPathS);
      aMSOut := TModelScript(DM.FindElementName(aPathS));
      if aMSOut = nil then
      begin
         aMSOut := TModelScript.Create;
         aMSOut.Name := aPathS;
         DM.AddElement(aMSOut);
      end;
      aLayerOut:= TLayerScript.Create;
      aLayerOut.Name:= aLayer.Name;
      aLayerOut.Description:= aLayer.Description;
      aLayerOut.AddElement(aPath);
      aLayerOut.AddElement(aPL);
      aMSOut.AddElement(aLayerOut);
    end;
  end;
end;



begin
  RegisterOrbisClass(TModelScriptCreator);
  RegisterOrbisClass(TStringListFromModelScript);
  RegisterOrbisClass(TModelScriptToDynamicModel);
end.
