unit T_lmcre;
{
[Unit_Description]
Description=Nstroj pro import dat a vytven jejich modelu na zklad scriptu
Admin=
TechTopic=Elements
Topic=System
Version=1.0
Library=GISLAYER
Text Input
  Oekv objekt TModelScript
EndText
Text Output
  Pokud je na vstupu objekt TRectEntity, pak do nj ukld vsledek,
  jinak vytvo sm TRectEntity
EndText
Reference=GISLAYER,LAYER,E_LayScr,TModelScript,
}
interface
uses
  // KERNEL
  Elements,
  E_Type,
  Tools,
  // GLIB
  G_Elems,
  // TOPOL
  T_TopImp,
  // GISLAYER
  E_LayScr;

type

{ oekv na vstupu TModelScript,
 vytv Model }

TModelByScriptCreator = class(TTool)
private
  FMinX, FMinY, FMaxX, FMaxY: double;
  FModel: TRectEntity;
  FCutLayer: boolean;
  FOnNewObject: TObjectEvent;
    FCompatibility30: boolean;
  function ReadTable(aPath: string): TElement;
  function ReadFile(aPath: string): TElement;
  function ReadElement(ParStr: string;  Props: TPropertyList): TElement;
  function ReadTopolLayer
           (aType, aPath: string; Props:TPropertyList): TElement;
{$IFDEF SHP}
  function ReadShapeFileLayer
           (aType, aPath: string; Props:TPropertyList): TElement;
{$ENDIF}
{$IFDEF DGN}
  function ReadDGNLayer
          (aType, aPath: string; Props:TPropertyList): TElement;
{$ENDIF}
{$IFDEF ARCASC}
  function ReadARCASCLayer
          (aType, aPath: string; Props:TPropertyList): TElement;
{$ENDIF}
  function ReadLayer
           (aType, aPath: string; Props:TPropertyList): TElement;
  procedure AddToModel(Element: TElement);
  //procedure SetMinX(aValue: double);
  procedure DoDeltaX(aVal: double);
  procedure DoDeltaY(aVal: double);
  procedure SetWidth(aVal: double);
  procedure SetHeight(aVal: double);
  function GetCenterX: double;
  function GetCenterY: double;
    procedure SetCompatibility30(const Value: boolean);
public
  procedure DoIt; override;
published
  property MinX: double read FMinX write FMinX;
  property MinY: double read FminY write FMinY;
  property MaxX: double read FmaxX write FMaxX;
  property MaxY: double read FmaxY write FMaxY;
  property DeltaX: double write DoDeltaX;
  property DeltaY: double write DoDeltaY;
  property Width : double write SetWidth;
  property Height: double write SetHeight;
  property CenterX : double read GetCenterX;
  property CenterY : double read GetCenterY;
  property CutLayer: boolean read FCutLayer write FCutLayer;
  property OnNewObject: TObjectEvent read FOnNewObject write FOnNewObject;
  property Compatibility30: boolean read FCompatibility30 write SetCompatibility30;
end;


implementation
uses
  FileCtrl, SysUtils,
  // KERNEL
  E_Class,
  e_DBTable,
  e_Files,
  U_Props,
  Worlds,
  U_File,
  M_Orbis,
// SPACE
  //e_Space,
// interpret
  e_InterpretContainer,

{$IFDEF BINRAS}
  u_Ras, E_TopRas,
  T_PZ,
{$ENDIF}
// RASTER
{$IFDEF RAS}
  E_Raster,
  //T_PZ1,
{$ENDIF}

{
  // DGN
  T_DGNImp,
}
{$IFDEF SHP}
  // SHPFile
  T_ShpImp,
{$ENDIF}

{$IFDEF ARCASC}
  // ArcAsc
  T_AAImp,
{$ENDIF}

  // GLIB
  e_WCS;


{TModelByScriptCreator}


function TModelByScriptCreator.ReadTable(aPath: string): TElement;
begin
   result := TDBTableElement.Create;
   TDBTableElement(result).FileName:= aPath;
end;

function TModelByScriptCreator.ReadFile(aPath: string): TElement;
begin
   result := TFileElement.Create;
   TFileElement(result).FileName:= aPath;
end;


function TModelByScriptCreator.ReadElement(ParStr: string;  Props: TPropertyList): TElement;
var
  I: integer;
  PropName: string;
begin
  result:= CreateElementByName(ParStr);
  if result = nil then exit;

  if Props <> nil then
  begin
    for I:= 0 to Props.Count-1 do
    begin
      PropName:= Props.Names[I];
      if PropName[1] = '@' then
         PropName:= copy(PropName, 2, 255);
      SetPropStringByName(result, PropName, Props.Strings[I]);
    end
  end
end;




procedure TModelByScriptCreator.AddToModel(Element: TElement);
begin
  if FModel = nil then
     FModel:= TRectEntity.Create;

  if (Element is TRectEntity) then
     FModel.AddAndUpdateRect(Element)
  else
     FModel.AddElement(Element);
end;

(*
procedure TModelByScriptCreator.SetMinX(aValue: double);
begin
  FMinX:= aValue
end;

*)
function TModelByScriptCreator.ReadTopolLayer
          (aType, aPath: string; Props:TPropertyList): TElement;
var
  BlokReader: TTopolBlokReader;
  PropName: string;
  I: longInt;
  RectEnt: TRectEntity;

begin
  result:= nil;
  RectEnt := nil;
  if (FMinX <> 0) and (FMinY <> 0) and (FMaxX <> 0) and (FMaxY <> 0) then
  begin
    RectEnt:= TRectEntity.Create;
    RectEnt.MinX:= FMinX;
    RectEnt.MinY:= FMinY;
    RectEnt.MaxX:= FMaxX;
    RectEnt.MaxY:= FMaxY;

    if not BlokInRect(aPath, RectEnt) then
    begin
       RectEnt.Free;
       exit
    end;
  end;

  result:= nil;

  BlokReader:= TTopolBlokReader.CreateBlock(aPath);
  try
    BlokReader.FiltrDruhu:= CreateFiltrDruhu(aType, Props);
    BlokReader.SetProps(Props);
    BlokReader.OnNewObject:= OnNewObject;

{ nem smysl natahovat objekty, kter jsou mimo }
    if (FMinX <> 0) and (FMinY <> 0) and (FMaxX <> 0) and (FMaxY <> 0)
        and
       CutLayer then
    begin
      BlokReader.RectLimits:= RectEnt
    end;

    if aType = 'LIN' then
    begin
       result:= BlokReader.ReadLines;
    end
    else
    if aType = 'PNT' then
    begin
       result:= BlokReader.ReadPoints;
    end
    else
    if aType = 'ARE' then
    begin
       result:= BlokReader.ReadAreas;
    end
    else
    if aType = 'TXT' then
    begin
       result:= BlokReader.ReadTexts;
    end
    else
    if aType = 'LINSYM' then
    begin
       result:= BlokReader.ReadLineSymbols;
    end;
  finally
    BlokReader.Free
  end;


  if (result <> nil) then
  begin
    if Props <> nil then
    begin
      for I:= 0 to Props.Count-1 do
      begin
        PropName:= Props.Names[I];
        if PropName[1] = '@' then
           PropName:= copy(PropName, 2, 255);
        result.SetPropertyStringByName(PropName, Props.Strings[I]);
      end
    end;

  end
end;



{$IFDEF DGN}
function TModelByScriptCreator.ReadDGNLayer
          (aType, aPath: string; Props:TPropertyList): TElement;
var
  DGNImp: TDGNImport;
  PropName: string;
  I: longInt;
  RectEnt: TRectEntity;
  WCS: TWCS;

begin
  result:= nil;

  WCS:=  TWCSWorld(World).GetIDWCS(cs_JTSK);
  DGNImp:= TDGNImport.Create(aPath, WCS);
  DGNImp.FileName:= aPath;
  DGNImp.DoIt;
{
  if (FMinX <> 0) and (FMinY <> 0) and (FMaxX <> 0) and (FMaxY <> 0) then
  begin
    RectEnt:= TRectEntity.Create;
    RectEnt.MinX:= FMinX;
    RectEnt.MinY:= FMinY;
    RectEnt.MaxX:= FMaxX;
    RectEnt.MaxY:= FMaxY;

    if not BlokInRect(aPath, RectEnt) then
    begin
       RectEnt.Free;
       exit
    end;
  end;
}
{
  try

    BlokReader.FiltrDruhu:= CreateFiltrDruhu(aType, Props);
    BlokReader.SetProps(Props);
}
{ nem smysl natahovat objekty, kter jsou mimo }
{
    if (FMinX <> 0) and (FMinY <> 0) and (FMaxX <> 0) and (FMaxY <> 0)
        and
       CutLayer then
    begin
      BlokReader.RectLimits:= RectEnt
    end;
}
{
    if aType = 'LIN' then
    begin
       result:= BlokReader.ReadLines;
    end
    else
    if aType = 'PNT' then
    begin
       result:= BlokReader.ReadPoints;
    end
    else
    if aType = 'ARE' then
    begin
       result:= BlokReader.ReadAreas;
    end
    else
    if aType = 'TXT' then
    begin
       result:= BlokReader.ReadTexts;
    end
    else
    if aType = 'LINSYM' then
    begin
       result:= BlokReader.ReadLineSymbols;
    end;
  finally
    BlokReader.Free
  end;
}
  if aType = 'LIN' then
  begin
    result:= DGNImp.Output;
  end;

  if (result <> nil) then
  begin
    if Props <> nil then
    begin
      for I:= 0 to Props.Count-1 do
      begin
        PropName:= Props.Names[I];
        if PropName[1] = '@' then
           PropName:= copy(PropName, 2, 255);
        SetPropStringByName(result, PropName, Props.Strings[I]);
      end
    end;

  end;

  DGNImp.Free;
end;
{$ENDIF}




{$IFDEF SHP }
function TModelByScriptCreator.ReadShapeFileLayer
          (aType, aPath: string; Props:TPropertyList): TElement;
var
  SFImp: TShapeFileImport;
  PropName: string;
  I: longInt;
//  RectEnt: TRectEntity;

begin
  result:= nil;
  SFImp:= TShapeFileImport.Create;
  SFImp.FileName:= aPath;

{
  if (FMinX <> 0) and (FMinY <> 0) and (FMaxX <> 0) and (FMaxY <> 0) then
  begin
    RectEnt:= TRectEntity.Create;
    RectEnt.MinX:= FMinX;
    RectEnt.MinY:= FMinY;
    RectEnt.MaxX:= FMaxX;
    RectEnt.MaxY:= FMaxY;

    if not BlokInRect(aPath, RectEnt) then
    begin
       RectEnt.Free;
       exit
    end;
  end;
}
{
  try

    BlokReader.FiltrDruhu:= CreateFiltrDruhu(aType, Props);
    BlokReader.SetProps(Props);
}
{ nem smysl natahovat objekty, kter jsou mimo }
{
    if (FMinX <> 0) and (FMinY <> 0) and (FMaxX <> 0) and (FMaxY <> 0)
        and
       CutLayer then
    begin
      BlokReader.RectLimits:= RectEnt
    end;
}
{
    if aType = 'LIN' then
    begin
       result:= BlokReader.ReadLines;
    end
    else
    if aType = 'PNT' then
    begin
       result:= BlokReader.ReadPoints;
    end
    else
    if aType = 'ARE' then
    begin
       result:= BlokReader.ReadAreas;
    end
    else
    if aType = 'TXT' then
    begin
       result:= BlokReader.ReadTexts;
    end
    else
    if aType = 'LINSYM' then
    begin
       result:= BlokReader.ReadLineSymbols;
    end;
  finally
    BlokReader.Free
  end;
}
  if aType = 'ARE' then
  begin
    result:= SFImp.ReadAreas;
  end
  else
  if aType = 'LIN' then
  begin
    result:= SFImp.ReadLines;
  end
  else
  if aType = 'PNT' then
  begin
    result:= SFImp.ReadPoints;
  end;

  if (result <> nil) then
  begin
    if Props <> nil then
    begin
      for I:= 0 to Props.Count-1 do
      begin
        PropName:= Props.Names[I];
        if PropName[1] = '@' then
           PropName:= copy(PropName, 2, 255);
        SetPropStringByName(result, PropName, Props.Strings[I]);
      end
    end;

  end;

  SFImp.Free
end;
{$ENDIF}




{$IFDEF ARCASC }
function TModelByScriptCreator.ReadARCASCLayer
          (aType, aPath: string; Props:TPropertyList): TElement;
var
  AAImp: TArcASCImport;
  PropName: string;
  I: longInt;
//  RectEnt: TRectEntity;

begin
  result:= nil;
  AAImp:= TArcASCImport.Create;
  AAImp.FileName:= aPath;
  AAImp.SetProps(Props);

{
  if (FMinX <> 0) and (FMinY <> 0) and (FMaxX <> 0) and (FMaxY <> 0) then
  begin
    RectEnt:= TRectEntity.Create;
    RectEnt.MinX:= FMinX;
    RectEnt.MinY:= FMinY;
    RectEnt.MaxX:= FMaxX;
    RectEnt.MaxY:= FMaxY;

    if not BlokInRect(aPath, RectEnt) then
    begin
       RectEnt.Free;
       exit
    end;
  end;
}
{
  try

    BlokReader.FiltrDruhu:= CreateFiltrDruhu(aType, Props);
    BlokReader.SetProps(Props);
}
{ nem smysl natahovat objekty, kter jsou mimo }
{
    if (FMinX <> 0) and (FMinY <> 0) and (FMaxX <> 0) and (FMaxY <> 0)
        and
       CutLayer then
    begin
      BlokReader.RectLimits:= RectEnt
    end;
}
{
    if aType = 'LIN' then
    begin
       result:= BlokReader.ReadLines;
    end
    else
    if aType = 'PNT' then
    begin
       result:= BlokReader.ReadPoints;
    end
    else
    if aType = 'ARE' then
    begin
       result:= BlokReader.ReadAreas;
    end
    else
    if aType = 'TXT' then
    begin
       result:= BlokReader.ReadTexts;
    end
    else
    if aType = 'LINSYM' then
    begin
       result:= BlokReader.ReadLineSymbols;
    end;
  finally
    BlokReader.Free
  end;
}
  if aType = 'ARE' then
  begin
    result:= AAImp.ReadAreas;
  end;

  if (result <> nil) then
  begin
    if Props <> nil then
    begin
      for I:= 0 to Props.Count-1 do
      begin
        PropName:= Props.Names[I];
        if PropName[1] = '@' then
           PropName:= copy(PropName, 2, 255);
        SetPropStringByName(result, PropName, Props.Strings[I]);
      end
    end;

  end;

  AAImp.Free
end;
{$ENDIF}




function TModelByScriptCreator.ReadLayer
          (aType, aPath: string; Props:TPropertyList): TElement;
var
  //BlokReader: TTopolBlokReader;
  //PropName: string;
  //I: longInt;
{$IFDEF BINRAS}
  RasRect: TDoubleRect;
  RectEnt: TRectEntity;
{$ENDIF}
  anExt: string;

{$IFDEF RAS}
  SE: TStringElement;
  RectEnt: TRectEntity;
{$ENDIF}
begin
  result:= nil;
{$IFDEF BINRAS}
  if aType = 'RAS' then
  begin
     RectEnt:= nil;
     if (FMinX <> 0) and (FMinY <> 0) and (FMaxX <> 0) and (FMaxY <> 0) then
     begin
       RectEnt:= TRectEntity.Create;
       RectEnt.MinX:= FMinX;
       RectEnt.MinY:= FMinY;
       RectEnt.MaxX:= FMaxX;
       RectEnt.MaxY:= FMaxY;

       RasRect:= GetRasRect(aPath)
     end;

     if (RectEnt = nil) or
        RectEnt.IsOverRegion
          (RasRect.Min.X, RasRect.Min.Y, RasRect.Max.X, RasRect.Max.Y) then
     begin
       result:= ReadTopolRaster(aPath);
       if result <> nil then
       begin
         TTopRas(result).SetProps(Props);
       end
     end;

     if RectEnt <> nil then
        RectEnt.Free;
     exit
  end
  else
{$ENDIF}
{$IFDEF RAS}
  if (aType = 'RAS') and UseRasters then
  begin
     RectEnt:= nil;
     {!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     if (FMinX <> 0) and (FMinY <> 0) and (FMaxX <> 0) and (FMaxY <> 0) then
     begin
       RectEnt:= TRectEntity.Create;
       RectEnt.MinX:= FMinX;
       RectEnt.MinY:= FMinY;
       RectEnt.MaxX:= FMaxX;
       RectEnt.MaxY:= FMaxY;

       RasRect:= GetRasRect(aPath)
     end;
     }
     {
     if (RectEnt = nil) or
        RectEnt.IsOverRegion
          (RasRect.Min.X, RasRect.Min.Y, RasRect.Max.X, RasRect.Max.Y) then
     }
     begin
       result:= ReadTopolRaster(aPath);

       if result <> nil then
       begin
          if Props <> nil then
          begin
             SE := TStringElement(Props.FindElementName('@COLOR'));
             if SE <> nil then TRaster(result).Color := SE.AsInteger;
          end;
       end

     end;

     if RectEnt <> nil then
        RectEnt.Free;
     exit
  end
  else
{$ENDIF}
  if aType = 'TAB' then
  begin
    result:= ReadTable(aPath);
    exit
  end;
  if aType = 'FILE' then
  begin
    result:= ReadFile(aPath);
    exit
  end;
  if aType = 'ELEMENT' then
  begin
    result:= ReadElement(aPath, Props);
    exit
  end;

  if DirectoryExists(aPath) then
  begin
    anExt:= UpperCase(ExtractFileExt(aPath));
    if anExt = '.BLK' then
       result:= ReadTopolLayer(aType, aPath, Props)
  end
  else
  if FileExists(aPath) then
  begin
    anExt:= UpperCase(ExtractFileExt(aPath));
    if anExt = '.SHP' then
{$IFDEF SHP}
       result:= ReadShapeFileLayer(aType, aPath, Props)
{$ENDIF}
    else
    if anExt = '.ASC' then
{$IFDEF ARCASC}
       result:= ReadArcASCLayer(aType, aPath, Props)
{$ENDIF}
    else
    if anExt = '.DGN' then
{$IFDEF DGN}
       result:= ReadDGNLayer(aType, aPath, Props)
{$ENDIF}
  end

end;



procedure TModelByScriptCreator.DoDeltaX(aVal: double);
begin
  FMinX:= FMinX + aVal;
  FMaxX:= FMaxX + aVal;
end;

procedure TModelByScriptCreator.DoDeltaY(aVal: double);
begin
  FMinY:= FMinY + aVal;
  FMaxY:= FMaxY + aVal;
end;


procedure TModelByScriptCreator.SetWidth(aVal: double);
begin
  FMaxX:= FMinX + aVal
end;

procedure TModelByScriptCreator.SetHeight(aVal: double);
begin
  FMaxY:= FMinY + aVal
end;


function TModelByScriptCreator.GetCenterX: double;
begin
  result:= (FMaxX + FMinX) /2
end;

function TModelByScriptCreator.GetCenterY: double;
begin
  result:= (FMaxY + FMinY) /2
end;



procedure TModelByScriptCreator.DoIt;
var
  I, J: longInt;
  aModelScript: TModelScript;
  aLayerScript: TLayerScript;
  aLayerProps: TPropertyList;
  aPathEl: TPathElement;
  El: TElement;
  aLayCont: TInterpretLayer;
  Transp: integer;
  aColor: integer;

  procedure AddToLayCont(aEl: TElement);
  begin
    if aLayCont = nil then
       aLayCont:= TInterpretLayer.Create;

    if (aEl is TRectEntity) then
       aLayCont.AddAndUpdateRect(aEl)
    else
       aLayCont.AddElement(aEl);
  end;

begin
  if not (input is TModelScript) then exit;
  aModelScript:= TModelScript(input);

  if Output is TRectEntity then
     FModel:= TRectEntity(Output)
  else
     FModel:= TRectEntity.Create;


  for I:= 0 to aModelScript.Count - 1 do
  begin
    aLayerScript:= TLayerScript(aModelScript.Elements[I]);
    if TElement(aLayerScript) is TLayerScript then
    begin
      aLayerProps:= aLayerScript.GetProperties;

      if aLayerScript.Name = 'ELEMENT' then
      begin
        El:= ReadElement(aLayerScript.Description, aLayerProps);
        AddToModel(El);
        continue
      end;

      aLayCont:= nil;
      Transp := 0;
      aColor := 0;
      for J:= 0 to aLayerScript.Count-1 do
      { cyklus pes vechny cesty }
      begin
        aPathEl:= TPathElement(aLayerScript.Elements[J]);
        if aLayerProps <> nil then
        begin
           Transp := aLayerProps.GetIntegerByName('@TRANSPARENCY');
           aColor := aLayerProps.GetIntegerByName('@COLOR');
        end;
        if TElement(aPathEl) is TPathElement then
        begin
          El:= ReadLayer(aLayerScript.Name, aPathEl.Text, aLayerProps);
          if Compatibility30 and (Transp = 0) then
             AddToModel(El)
          else
             AddToLayCont(El);
        end
      end;
      if aLayCont <> nil then
      begin
        aLayCont.Name:= extractFileName(aLayerScript.Description);
        aLayCont.Path:= aLayerScript.Description;
        aLayCont.LayerType:= aLayerScript.Name;
        aLayCont.Transparency := Transp;
        aLayCont.Color := aColor;
        AddToModel(aLayCont);
        aLayCont:= nil
      end;
    end
  end;
  Output:= FModel;
end;




procedure TModelByScriptCreator.SetCompatibility30(const Value: boolean);
begin
  FCompatibility30 := Value;
end;

initialization
  RegisterOrbisClass(TModelByScriptCreator);
end.
