unit T_itable;

interface
uses
   Elements,
   Tools,
   t_oper;

type

TITItem = class(TOwnerElement)
private
   FCondition: TCondition;
   FExpression: TOperator;
   FResult: string;
   FText: string;
   procedure SetText(aValue: string);
public
   destructor Destroy; override;
   property Condition: TCondition read FCondition write FCondition;
   property Expression : TOperator read FExpression write FExpression;
published
   property Text: string read FText write SetText;
   property Result: string read FResult write FResult;
end;

TSimpleInterpretTable = class(TResultTool)
private
   FPropertyName: string;
   FElseValue: string;
   procedure SetNextItem(aValue: string);
public
   procedure DoIt; override;
published
   property  PropertyName: string read FPropertyName write FPropertyName;
   property  NextItem: string write SetNextItem;
   property  ElseValue: string read FElseValue write FElseValue;
end;


TSimpleIntervalInterpretTable = class(TSimpleInterpretTable)
{Pedpokld se uspodan tabulka hodnot.
 Vsledek kad hodnot piazen plat pro cel interval a do
 nsledn i pedchoz hodnoty v zvislosti na LessEqualThen }
private
  FNotInterval: Boolean;
  FLessEqualThen: Boolean;
public
  procedure DoIt; override;
published
  property NotInterval: Boolean read FNotInterval write FNotInterval;
  property LessEqualThen: Boolean read FLessEqualThen write FLessEqualThen;
end;


TInterpretTable = class(TResultTool)
private
   FFileName: string;
   FElseValue: string;
   procedure SetFileName(aValue: string);
   procedure SetNextItem(aValue: string);
public
   procedure DoIt; override;
published
   property  FileName: string read FFileName write SetFileName;
   property  NextItem: string write SetNextItem;
   property  ElseValue: string read FElseValue write FElseValue;
end;

implementation

uses
   Classes,
   SysUtils,
   Creacond,
   e_Class,
   e_Type,
   u_String;


{ TITItem }


destructor TITItem.Destroy;
begin
   if Condition <> nil then Condition.Free;
   inherited Destroy;
end;

procedure TITItem.SetText(aValue: string);
var
   L, R: string;
   CC: TCondCreator;
   EC: TExpressionCreator;
   Op: TOperator;
begin
   if aValue = '' then exit;
   if not StringAsTwoParams(aValue, ',', L, R) then exit;
   if Condition <> nil then
   begin
      Condition.Free;
      Condition := nil;
   end;
   CC := TCondCreator.Create;
   CC.Command:=L;
   CC.DoIt;
   Condition := TCondition(CC.Output);
   CC.Free;
   if not(R[1] in ['0'..'9']) then
   begin
      EC := TExpressionCreator.Create;
      //EC.LocalVars := LocalVars;
      //EC.Vars := Vars;
      EC.Command := R;
      EC.DoIt;
      Op := TOperator(EC.Output);
      Expression := Op;
      EC.Free;
   end
   else
      FResult := R;
   Name := aValue;
end;


{ TSimpleInterpretTable }

procedure TSimpleInterpretTable.SetNextItem(aValue: string);
var
   L, R: string;
   SE: TStringElement;
begin
   if aValue = '' then exit;
   if not StringAsTwoParams(aValue, ',', L, R) then exit;
   SE := TStringElement.Create;
   SE.Name := L;
   SE.Value := R;
   AddElement(SE);
end;


procedure TSimpleInterpretTable.DoIt;
var
   //i: longInt;
   SE: TStringElement;
   S :string;
begin
   if PropertyName = '' then exit;
   Result := ElseValue;
   S := Input.GetPropertyStringByName(PropertyName);
   SE := TStringElement(FindElementName(S));
   if SE <> nil then result := SE.Value;
end;



procedure TSimpleIntervalInterpretTable.DoIt;
var
   //i: longInt;
   SE: TStringElement;
   S :string;
   //SD: Double;
begin
   if PropertyName = '' then exit;
   Result := ElseValue;
   S := Input.GetPropertyStringByName(PropertyName);
   SE := TStringElement(FindElementName(S));
   if SE <> nil then result := SE.Value;
end;




{ TInterpretTable }

procedure TInterpretTable.SetFileName(aValue: string);
var
   S: TStringList;
   i: longInt;
   ITI: TITItem;
begin
   if not FileExists(aValue) then exit;
   S := TStringList.Create;
   S.LoadFromFile(aValue);
   for i := 0 to S.Count - 1 do
   begin
      ITI := TITItem.Create;
      ITI.Text := S.Strings[i];
      AddElement(ITI);
   end;
   S.Free;
end;

procedure TInterpretTable.SetNextItem(aValue: string);
var
   ITI: TITItem;
begin
   ITI := TITItem.Create;
   ITI.Text := aValue;
   AddElement(ITI);
end;

procedure TInterpretTable.DoIt;
var
   i: longInt;
   ITI: TITItem;
   Op: TOperator;
   TE: TTypeElement;
begin
   Result := ElseValue;
   for i := 0 to Count - 1 do
   begin
      ITI := TITItem(Elements[i]);
      ITI.Condition.ParamItem[0]:=Input;
      if ITI.Condition.IsTrue then
      begin
         if ITI.Expression <> nil then
         begin
            Op := ITI.Expression;
            Op.ParamItem[0] := Input;
            Op.DoIt;
            if Op.Output <> nil then
            begin
               TE := TTypeElement(Op.Output);
               Result := TE.AsString;
            end;
            Op.CleanData;  // otazka, jestli se maji resulty operatoru vubec rusit .....
            //Op.Free;
         end
         else
             Result := ITI.Result;
         exit;
      end;
   end;
end;



INITIALIZATION
   RegisterOrbisClass(TITItem);
   RegisterOrbisClass(TSimpleInterpretTable);
   RegisterOrbisClass(TInterpretTable);
END.
