unit E_Class;
{
[Unit_Description]
Description=Elementy pro registraci td ORBISu
Admin=Petr
TechTopic=Elements
Topic=System
Version=1.0
}

interface
uses
   TypInfo,
   Elements;
{$IFDEF MSWINDOWS}
//   ,
//   dialogs;
{$ELSE}
//   ;
{$ENDIF}

type

TOrbisClassProperty = class(TNameElement)
private
   FPropInfo: PPropInfo;
   FIndex: integer;
   FTrueIndex: integer;
    FLostProperty: boolean;
    FLostTypeKind: TTypeKind;
    FTypeMishmash: boolean;
   procedure SetPropInfo(aPropInfo: PPropInfo);
   function  GetTypeKind: TTypeKind;
    procedure SetLostProperty(const Value: boolean);
    procedure SetLostTypeKind(const Value: TTypeKind);
    procedure SetTypeMishmash(const Value: boolean);
public
   property PropInfo: PPropInfo read FPropInfo write SetPropInfo;
   property TrueIndex: integer read FTrueIndex write FTrueIndex;
published
   property PropIndex: integer read FIndex write FIndex;
   property TypeKind: TTypeKind read GetTypeKind;
   Property LostTypeKind: TTypeKind read FLostTypeKind write SetLostTypeKind;
   property LostProperty: boolean read FLostProperty write SetLostProperty;
   property TypeMishmash: boolean read FTypeMishmash write SetTypeMishmash;
end;



TOrbisClass = class(TOwnerEntity)
private
   FElementClass: TElementClass;
   FPropList: PPropList;
   FPropCount: integer;
   function  GetElementClassName: string;
   function  GetAncestorClassName: string;
   procedure SetElementClass(aValue: TElementClass);
   function  GetPropertyName(aIndex: integer): string;
   function  GetPropertyTypeKind (aIndex: integer): TTypeKind;
   function  GetPropertyStored(aIndex: integer): boolean; virtual;
   function  GetPropInfo(aIndex: integer): PPropInfo;
    function GetPropertyIsReadOnly(aIndex: integer): boolean;
    function GetPropertyIsWriteOnly(aIndex: integer): boolean;
    function GetPropertyIsReadOrWriteOnly(aIndex: integer): boolean;
    function GetPropertyClass(aIndex: integer): TClass;
    function GetUnitName: string;
public
   destructor Destroy; override;
   function   IsClassName(const aName: string): boolean;
   procedure  ShowProperties; virtual;
   function   PropertyIndex(aName:string ): integer;virtual;
   procedure  SelfCHeck;
   property   ElementClass: TElementClass read FElementClass write SetElementClass;
   property   PropertyName[aIndex: integer]: string read GetPropertyName;
   property   PropertyTypeKind[aIndex: integer]: TTypeKind read GetPropertyTypeKind;
   property   PropertyStored[aIndex: integer]: boolean read GetPropertyStored;
   property   PropertyIsReadOnly[aIndex: integer]: boolean read GetPropertyIsReadOnly;
   property   PropertyIsWriteOnly[aIndex: integer]: boolean read GetPropertyIsWriteOnly;
   property   PropertyIsReadOrWriteOnly[aIndex: integer]: boolean read GetPropertyIsReadOrWriteOnly;
   property   PropertyClass[aIndex: integer]: TClass read GetPropertyClass;
   property   PropList: PPropList read FPropList;
   property   PropInfo[aIndex: integer]: PPropInfo read GetPropInfo;
published
   property   ElementClassName: string read GetElementClassName;
   property   AncestorClassName: string read GetAncestorClassName;
   property   PropCount: integer read FPropCount;
   property   UnitName: string read GetUnitName stored false;
end;


TOrbisInstance = class(TOrbisClass)
private
   FInstance: TElement;
   procedure SetInstance(aValue: TElement);
   function  GetPropertyStored(aIndex: integer): boolean; override;
public
   procedure SetPropertyValue(aIndex: integer; var aValue);
   procedure GetPropertyValue(aIndex: integer; var aValue);
   function  CreateInstanceCopy: TElement;
   property Instance: TElement read FInstance write SetInstance;
   property PropertyStored[aIndex: integer]: boolean read GetPropertyStored;
end;


TOrbisClasses = class(TOwnerEntity)
private
   FActualClass: TOrbisClass;
   FShowProperties: boolean;
   FSortByName: boolean;
   FIndex: longInt;
   procedure SetSortByName(aValue: boolean);
protected
   function  FindIndex(const aName: string; var NearestRight: longInt): longInt;
public
   procedure AddOrbisClass(aClass: TOrbisClass); virtual;
   procedure AddClass(aClass: TElementClass); virtual;
   function  GetOrbisClassByName(const aName: string): TOrbisClass; virtual;
   function  GetClass (const aClassName: string): TElementClass; virtual;
   function  FindClass(const aClassName: string): TElementClass;  virtual;
   property  ActualClass: TOrbisClass read FActualClass;
published
   property  ShowProperties: boolean read FShowProperties write FShowProperties;
   property  SortByName: boolean read FSortByName write SetSortByname;
end;


var OrbisClasses: TOrbisClasses = nil;


procedure RegisterOrbisClass(aClass: TElementClass);
procedure CHeckOrbisClasses;
function  CreateElementCopy(aElement: TElement): TElement;
function  GetOrbisClass (const aClassName: string): TElementClass;
function  FindOrbisClass(const aClassName: string): TElementClass;


implementation

uses
  {!t_Main,}
   Classes,
   SysUtils
{$IFDEF MSWINDOWS}
   ,
   M_Orbis;
{$ELSE}
   ;
{$ENDIF}


procedure RegisterOrbisClass(aClass: TElementClass);
begin
//   exit;
   if OrbisClasses = nil then
   begin
      OrbisClasses := TOrbisClasses.Create;
      OrbisClasses.ShowProperties := true;
   end;
   OrbisClasses.AddClass(aClass);
end;

procedure CHeckOrbisClasses;
var
   i: integer;
   E: TElement;
   CLS: TOrbisClass;
begin
   if OrbisClasses = nil then exit;
   for i := 0 to OrbisClasses.LastIndex do
   begin
      E := OrbisClasses.Elements[i];
      if not (E is TOrbisClass) then Continue;
      CLS := TOrbisClass(E);
      CLS.SelfCHeck;
   end;
end;


function  CreateElementCopy(aElement: TElement): TElement;
var
   Inst: TOrbisInstance;
begin
   //result := nil;
   Inst := TOrbisInstance.Create;
   Inst.Instance := aElement;
   result := Inst.CreateInstanceCopy;
   Inst.Free;
end;

function  GetOrbisClass (const aClassName: string): TElementClass;
begin
   result := OrbisClasses.GetClass(aClassName);
end;

function  FindOrbisClass(const aClassName: string): TElementClass;
begin
   result := OrbisClasses.FindClass(aClassName);
end;


{ TOrbisClassProperty }


procedure TOrbisClassProperty.SetPropInfo(aPropInfo: PPropInfo);
begin
  FPropInfo:= aPropInfo;
{$IFDEF NEXTGEN}
  Name := FPropInfo.NameFld.ToString;
{$ELSE}
  Name:= FPropInfo^.Name
{$ENDIF}
end;


function TOrbisClassProperty.GetTypeKind: TTypeKind;
begin
  result:= tkUnknown;
  if LostProperty then
  begin
     result := LostTypeKind;
     exit;
  end;
  if FPropInfo = nil then exit;
  result:= FPropInfo^.PropType^.Kind;
end;



procedure TOrbisClassProperty.SetLostProperty(const Value: boolean);
begin
   FLostProperty := Value;
end;

procedure TOrbisClassProperty.SetLostTypeKind(const Value: TTypeKind);
begin
  FLostTypeKind := Value;
end;


procedure TOrbisClassProperty.SetTypeMishmash(const Value: boolean);
begin
  FTypeMishmash := Value;
end;

{ TOrbisClass }




destructor TOrbisClass.Destroy;
begin
  if FPropList <> nil then
     FreeMem(FPropList, FPropCount * SizeOf(Pointer));
  inherited destroy
end;

function TOrbisClass.PropertyIndex(aName:string ): integer;
var
   I:integer;
   Nasel:boolean;
   s: string;
begin
   result:= -1;
   I:= 0;
   Nasel:= false;
   while (I < FPropCount) and not Nasel do
       begin
         {$IFDEF NEXTGEN}
          S := FPropList^[I].NameFld.ToString;
         {$ELSE}
          S := FPropList^[I].Name;
         {$ENDIF}
         if AnsiCompareText(aName,S)= 0 then Nasel:= true;
         if not Nasel then inc(I)
       end;
   if Nasel then result:= I
end;

function  TOrbisClass.GetPropertyName(aIndex: integer): string;
begin
   {$IFDEF NEXTGEN}
    result := FPropList^[aIndex].NameFld.ToString;
   {$ELSE}
    result := FPropList^[aIndex]^.Name;
   {$ENDIF}
//   result := FPropList^[aIndex]^.Name;
end;

function  TOrbisClass.GetPropertyTypeKind (aIndex: integer): TTypeKind;
begin
   result := FPropList^[aIndex]^.PropType^.Kind;
end;

function  TOrbisClass.GetPropInfo(aIndex: integer): PPropInfo;
begin
   result := FPropList^[aIndex];
end;

function  TOrbisClass.GetPropertyStored(aIndex: integer): boolean;
var
//   PropI: PPropInfo;
   L: LongWord;
   res: LongWord;
begin
   result:= true;
   if FPropList = nil then exit;
   L := longWord(PropInfo[aIndex]^.StoredProc);
   res := L and $0FFFFFF00;
   if not longBool(res) then
   { constant value }
   begin
      result := longBool(L and $000000FF);
   end;
end;



function  TOrbisClass.IsClassName(const aName: string): boolean;
begin
   result := AnsiCompareText(aName, Name) = 0;
end;


function  TOrbisClass.GetElementClassName: string;
begin
   result := '';
   if ElementClass = nil then exit;
   result := ElementClass.ClassName;
end;

function  TOrbisClass.GetAncestorClassName: string;
begin
   result := '';
   if ElementClass = nil then exit;
   result := ElementClass.ClassParent.ClassName;
end;

procedure TOrbisClass.SetElementClass(aValue: TElementClass);
begin
   FElementClass := aValue;
   if FPropList <> nil then
      FreeMem(FPropList, FPropCount * SizeOf(Pointer));
   FPropList:=nil;

   if aValue = nil then exit;
   Name:= FElementClass.ClassName;

   FPropCount:= GetTypeData(FElementClass.ClassInfo)^.PropCount;
   if FPropCount > 0 then
   begin
     GetMem(FPropList, FPropCount*sizeOf(Pointer));
     try
       GetPropInfos(FElementClass.ClassInfo, FPropList);
     except
       if FPropList <> nil then
          FreeMem(FPropList, FPropCount * SizeOf(Pointer));
       FPropList:= nil;
       FPropCount:= 0;
     end
   end
end;


procedure  TOrbisClass.ShowProperties;
var
  I: integer;
  OCP: TOrbisClassProperty;
begin
  FreeAll;
  if FPropList = nil then exit;
  for I:= 0 to PropCount-1 do
  begin
     OCP := TOrbisClassProperty.Create;
     OCP.PropInfo := FPropList^[I];
     AddElement(OCP);
  end;
end;

function TOrbisClass.GetPropertyIsReadOnly(aIndex: integer): boolean;
begin
   result:= true;
   if FPropList = nil then exit;
   result := PropInfo[aIndex]^.SetProc = nil;
end;

function TOrbisClass.GetPropertyIsWriteOnly(aIndex: integer): boolean;
begin
   result:= true;
   if FPropList = nil then exit;
   result := PropInfo[aIndex]^.GetProc = nil;
end;

procedure TOrbisClass.SelfCHeck;
var
   i: integer;
   C: TClass;
begin
   for i := 0 to PropCount - 1 do
   begin
      if PropertyStored[i] then
      begin
         (*
         if PropertyIsReadOnly[i] then
         begin
            PostTextMsg(tm_AddToProtocol,ElementClass.ClassName+' property "'+ PropertyName[i]+ '" is stored and Read only',nil);
         end
         else
         if PropertyIsWriteOnly[i] then
         begin
            PostTextMsg(tm_AddToProtocol,ElementClass.ClassName+' property "'+ PropertyName[i]+ '" is stored and Write only',nil);
         end;
         *)
         if PropertyTypeKind[i] = tkClass then
         begin
            C := PropertyClass[i];
            if C <> nil then
            begin
               if not C.InheritsFrom(TElement) then
               begin
               {$IFDEF MSWINDOWS}
                  PostTextMsg(tm_AddToProtocol,ElementClass.ClassName+' property "'+ PropertyName[i]+ '" is not Element',nil);
               {$ENDIF}
               end;
            end;
         end;
      end;
   end;
end;

function TOrbisClass.GetPropertyIsReadOrWriteOnly(
  aIndex: integer): boolean;
begin
   result:= true;
   if FPropList = nil then exit;
   result := (PropInfo[aIndex]^.GetProc = nil) or (PropInfo[aIndex]^.SetProc = nil);
end;

function TOrbisClass.GetPropertyClass(aIndex: integer): TClass;
var
  TypeData: PTypeData;
begin
  // get the prop info
  result := nil;
  if  FPropList^[aIndex]^.PropType^.Kind <> tkClass then exit;

  TypeData := GetTypeData(FPropList^[aIndex]^.PropType^);
  result := TypeData^.ClassType;
end;

//function GetTypeData(TypeInfo: PTypeInfo): PTypeData;
//asm
//        { ->    EAX Pointer to type info        }
//        { <-    EAX Pointer to TypeData         }
//
//        PUSH    EBX
//        PUSH    ESI
//        PUSH    EDI
//
//        XOR     EBX,EBX
//        MOV     BL,[EAX].TTypeInfo.Name.Byte[0]
//        LEA     EAX,[EAX].TTypeInfo.Name[EBX+1]
//
//        POP     EDI
//        POP     ESI
//        POP     EBX
//end;

function TOrbisClass.GetUnitName: string;
var
   TypeData: PTypeData;
begin
   TypeData := GetTypeData(PTypeInfo(ElementClass.ClassInfo));
{$IFDEF NEXTGEN}
      result := TypeData^.UnitNameFld.ToString;
{$ELSE}
   result := TypeData^.UnitName;
{$ENDIF}
end;

{ TOrbisInstance }

procedure TOrbisInstance.SetInstance(aValue: TElement);
begin
   FInstance := aValue;
   ElementClass := TElementCLass(FInstance.ClassType);
end;

function  TOrbisInstance.GetPropertyStored(aIndex: integer): boolean;
begin
  result:= false;
  if FPropList = nil then exit;
  if FInstance = nil then exit;
  result:= IsStoredProp(FInstance, FPropList^[aIndex]);
end;


procedure TOrbisInstance.SetPropertyValue(aIndex: integer; var aValue);
var
   P: PPropInfo;
begin
   P := FPropList^[aIndex];
   (*tkUnknown,
     tkSet,  tkArray, tkRecord, tkInterface,  tkDynArray
     nevim jak ulozit tyto typy
     *)
   case P^.PropType^.Kind of
       tkString
       {$IFDEF Win32}
       ,tkLString,tkWString
       {$ENDIF}
       {$IFDEF UNICODE}
       ,tkUString
       {$ENDIF}
                : SetStrProp(Instance, P, string(aValue));
       tkChar,tkInteger,tkEnumeration,tkSet
       {$IFDEF Win32}
       ,tkWChar
       {$ENDIF}
                         :begin
                         SetOrdProp(Instance, P, longint(aValue));
                         end;
       tkFloat:
                         begin
                         SetFloatProp(Instance, P, Extended(aValue));
                         end;
       tkClass:
                         begin
                         SetOrdProp(Instance, P, longInt(aValue));
                         end;
       tkMethod:
                         begin
                         SetMethodProp(Instance, P, TMethod(aValue));
                         end;
       tkVariant:
                         begin
                         SetVariantProp(Instance, P, Variant(aValue));
                         end;
       {$IFDEF Win32}
       tkInt64:
                         begin
                         SetInt64Prop(Instance, P, Int64(aValue));
                         end;
       {$ENDIF}
       end; { case }

end;

procedure TOrbisInstance.GetPropertyValue(aIndex: integer; var aValue);
var
   P: PPropInfo;
begin
   P := FPropList^[aIndex];
   (*tkUnknown,
     tkSet,  tkArray, tkRecord, tkInterface,  tkDynArray
     nevim jak ulozit tyto typy
     *)
   case P^.PropType^.Kind of
       tkString
       {$IFDEF Win32}
       ,tkLString,tkWString
       {$ENDIF}
       {$IFDEF UNICODE}
       ,tkUString
       {$ENDIF}
                :string(aValue):= GetStrProp(Instance, P);
       tkChar,tkInteger,tkEnumeration,tkSet
       {$IFDEF Win32}
       ,tkWChar
       {$ENDIF}
                         :begin
                         longint(aValue):=GetOrdProp(Instance, P);
                         end;
       tkFloat:
                         begin
                         Extended(aValue):=GetFloatProp(Instance, P);
                         end;
       tkClass:
                         begin
                         longInt(aValue):=GetOrdProp(Instance, P);
                         end;
       tkMethod:
                         begin
                         TMethod(aValue):=GetMethodProp(Instance, P);
                         end;
       tkVariant:
                         begin
                         Variant(aValue):=GetVariantProp(Instance, P);
                         end;
       {$IFDEF Win32}
       tkInt64:
                         begin
                         Int64(aValue):=GetInt64Prop(Instance, P);
                         end;
       {$ENDIF}
       end; { case }

end;

function TOrbisInstance.CreateInstanceCopy: TElement;
var
   i: integer;
   l: longInt;
   s: string;
   e: extended;
   m: TMethod;
   v: Variant;
   i64: Int64;
   InstanceCopy: TOrbisInstance;

   procedure CopyProperty(var aValue);
   begin
      GetPropertyValue(i, aValue);
      InstanceCopy.SetPropertyValue(i, aValue);
   end;
begin
   result := nil;
   if Instance = nil then exit;
   result := ElementClass.Create;
   InstanceCopy := TOrbisInstance.Create;
   InstanceCopy.Instance := result;
   for i := 0 to PropCount - 1 do
   begin
      if PropertyStored[i] then
      begin
         // showmessage('cau name '+PropertyName[i]);
         Case PropertyTypeKind[i] of
            {$IFDEF UNICODE}
            tkUString,
            {$ENDIF}
            tkString,tkLString,tkWString
                     : CopyProperty(s);
            tkChar,tkInteger,tkEnumeration,tkWChar,tkSet
                     : CopyProperty(l);
            tkFloat  : CopyProperty(e);
            tkClass  : CopyProperty(l);
            tkMethod : CopyProperty(m);
            tkVariant: CopyProperty(v);
            tkInt64  : CopyProperty(i64);
           end;
         // showmessage('cau nam hotovo');
      end;
    end;
   InstanceCopy.Free;
end;

{ TOrbisClasses }



procedure TOrbisClasses.SetSortByName(aValue: boolean);
begin
end;



function  TOrbisClasses.FindIndex(const aName: string; var NearestRight: longInt): longInt;
var
   L,R,C: longInt;
   LResult, RResult, CResult: integer;
begin
   result := -1;
   NearestRight := -1;
   if NoElements then exit;
   L := 0;
   R := Count - 1;
   RResult := AnsiCompareText(aName, Elements[R].Name);
   if (RResult > 0) then exit;
   if RResult = 0 then
   begin
      result := R;
      exit;
   end;
   if R > L then
   begin
      LResult := AnsiCompareText(aName, Elements[L].Name);
      if (LResult < 0) then
      begin
         NearestRight := L;
         exit;
      end;
      if LResult = 0 then
      begin
         result := L;
         NearestRight := L + 1;
         exit;
      end;
   end
   else
   begin
      NearestRight := L;
      exit;
   end;
   repeat
      NearestRight := R;
      if R - L <= 1 then exit;
      C := ((R - L) div 2) + L;
      CResult := AnsiCompareText(aName, Elements[C].Name);
      if CResult < 0 then R := C else
      if CResult > 0 then L := C else
      begin
         result := C;
         NearestRight := C + 1;
         exit;
      end;
   until L >= R;
end;



procedure TOrbisClasses.AddOrbisClass(aClass: TOrbisClass);
var
   i: longInt;
   S: string;
begin
   S := aClass.Name;
   FindIndex(S, i);
   if i > -1 then InsertElement(i, aClass)
             else AddElement(aClass);
   FActualClass := aClass;
   aClass.ID := FIndex;
   inc(FIndex);
end;



procedure TOrbisClasses.AddClass(aClass: TElementClass);
var
   OC: TOrbisClass;
   i: longInt;
   S: string;
begin
   OC := TOrbisClass.Create;
   OC.ElementClass := aClass;
   S := aClass.ClassName;
   OC.Name := S;
   OC.ID := FIndex;
   inc(FIndex);
   if ShowProperties then OC.ShowProperties;
   FindIndex(S, i);
   if i > -1 then InsertElement(i, OC)
             else AddElement(OC);
   FActualClass := OC;  //@rad
end;


function  TOrbisClasses.GetOrbisClassByName(const aName: string): TOrbisClass;
var
   i, ri: longInt;
begin
   result := nil;
   i := FindIndex(aName, ri);
   if i > -1 then
   begin
      result := TOrbisClass(Elements[i]);
      FActualClass := result;
   end;
end;


function  TOrbisClasses.GetClass (const aClassName: string): TElementClass;
var
   i, ri: longInt;
   EC: TOrbisClass;
begin
   result := nil;
   i := FindIndex(aClassName, ri);
   if i > -1 then
   begin
      EC := TOrbisClass(Elements[i]);
      result := EC.ElementClass;
      FActualClass := EC; // @rad
   end;
end;

function  TOrbisClasses.FindClass(const aClassName: string): TElementClass;
begin
  result := GetClass(aClassName);
  if result = nil then raise EClassNotFound.Create('Orbis nenael tdu '+ aClassName);
end;





INITIALIZATION
   RegisterOrbisClass(TOrbisClass);
   RegisterOrbisClass(TOrbisClasses);
   RegisterOrbisClass(TOrbisClassProperty);
END.
