unit Tools;
{
[Unit_Description]
Description=Tdy zkladnch nstroj
Admin=Petr
TechTopic=Tools
Topic=System
Version=1.0
}
{
*********************************************************************
*                                                                   *
*  ORBIS ... Tdy nstroj                                         *
*                                                                   *
*********************************************************************
}
interface

uses
   Classes,
//   Controls,
//   WinTypes,
   // KERNEL
   Elements,
   e_ToolModel;

type


TToolStatus = set of (st_FOCUSED, st_BUSY, st_BREAK, st_BREAK_AND_DOIT, st_DIE, st_Progress, st_NoProgress, st_IDLE, st_ZeroActivity, st_HighActivity);

TParameters = class(TOwnerEntity);
TTools      = class(TEnvelopeEntity);


TToolProperties = class(TOwnerElement)
private
published
end;

TTool = class(TOwnerEntity)
 private
   FStatus: TToolStatus;
   FParameters: TParameters;
   FError: longInt;
   FDynamicTool: boolean;
    FClient: boolean;
   function    GetNoParameters: boolean;
   function    GetPropertyNumber(aNumber: longint): string;
   function    GetInProgress: boolean;
   procedure   SetPropertyNumber(aNumber: longint; const aValue: string);
   procedure   SetProgress(const Value: integer);
   procedure   SetProgressText(const Value: string);
   procedure   SetInProgress(const Value: boolean);
   procedure SetSelfModel(const Value: TToolModel);
   function GetSelfModel: TToolModel;
   procedure SetDynamicTool(const Value: boolean);
    function GetIdle: boolean;
    procedure SetIdle(const Value: boolean);
    procedure SetMaxProgress(const Value: integer);
    function GetZeroActivity: boolean;
    procedure SetZeroActivity(const Value: boolean);
    function GetHighActivity: boolean;
    procedure SetHighActivity(const Value: boolean);
    procedure SetNoProgress(const Value: boolean);
    function GetNoProgress: boolean;
    procedure SetClient(const Value: boolean);
 protected
   FInPut:  TElement;
   FOutput: TElement;
   procedure   SetInput(aInput: TElement); virtual;
   procedure   SetOutput(aOutput: TElement); virtual;
   function    GetParameters: TParameters; virtual;
   procedure   SetParameters(aParameters: TParameters); virtual;
   function    GetParameter(i: longInt): TElement;
   procedure   PutParameter(i: longInt; aElement: TElement);
   function    GetNameParameter(aName: string): TElement; virtual;
   procedure   SetNameParameter(aName: string; aElement: TElement); virtual;
   function    GetBusy: boolean;
   procedure   SetBusy(aBusy: boolean);
   function    GetBreak: boolean;
   procedure   SetBreak(aBreak: boolean); virtual;
 public
   destructor  Destroy; override;
   procedure   DoIt; virtual;
   function    IsPossibleToQuit: boolean; virtual;
   function    ReadyForZeroActivity: boolean; virtual;
   procedure   EndZeroActivity; virtual;
   procedure   CreateSelfByModel; virtual;
   procedure   CreateDynamicPart; virtual;
   procedure   DestroyDynamicPart; virtual;
   procedure   UpdateModel; virtual;
   procedure   AddParameter(aParameter: TElement);
   procedure   AddNamedParameter(aParameter: TElement);
   procedure   FreeParameter(aParameter: TElement);
   procedure   FreeParameters;
   procedure   on_tm_StartTask(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   on_tm_QuitTask; virtual;
   procedure   on_tm_EndTask(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   on_tm_Done(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   function    NumberToPropertyName(aNumber: longInt): string; virtual;
   procedure   StartProgress(const ProcessName: string; MinProgress, MaxProgress: integer);
   procedure   EndProgress;

   property    Status: TToolStatus read FStatus write FStatus;
   property    NoParameters: boolean read GetNoParameters;
   property    Parameters: Tparameters read GetParameters write SetParameters;
   property    Parameter[Index: longint]: TElement read GetParameter write PutParameter;
   property    NameParameter[aName: string]: TElement read GetNameParameter write SetNameParameter;
   property    PropertyNumber[aNumber: longInt]: string read GetPropertyNumber write SetPropertyNumber;
   property    Progress: integer write SetProgress;
   property    MaxProgress: integer write SetMaxProgress;
   property    ProgressText: string write SetProgressText;
 published
   property    Input:  TElement read FInput  write SetInput stored false;
   property    Output: TElement read FOutput write SetOutput stored false;

   property    Busy: boolean read GetBusy write SetBusy stored false;
   property    Idle: boolean read GetIdle write SetIdle stored false;
   property    ZeroActivity: boolean read GetZeroActivity write SetZeroActivity;
   property    HighActivity: boolean read GetHighActivity write SetHighActivity;
   property    InProgress: boolean read GetInProgress write SetInProgress stored false;
   property    BreakWork: boolean read GetBreak write SetBreak stored false;
   property    DynamicTool: boolean read FDynamicTool write SetDynamicTool;
   property    SelfModel: TToolModel read GetSelfModel write SetSelfModel stored false;
   property    Error: longInt read FError write FError stored false;
   property    NextParameter: TElement write AddNamedParameter stored false;
   property    NoProgress: boolean read GetNoProgress write SetNoProgress;
   property    Client: boolean read FClient write SetClient;
end;

TProgressMessage = class(TOrbisMessage)
private
    FMaxProgress: integer;
    FProgress: integer;
    FMinProgress: integer;
    FProcessName: string;
    procedure SetMaxProgress(const Value: integer);
    procedure SetMinProgress(const Value: integer);
    procedure SetProcessName(const Value: string);
    procedure SetProgress(const Value: integer);
published
    property ProcessName: string read FProcessName write SetProcessName;
    property MinProgress: integer read FMinProgress write SetMinProgress;
    property MaxProgress: integer read FMaxProgress write SetMaxProgress;
    property Progress: integer read FProgress write SetProgress;
end;

TResultTool = class(TTool)
private
   FResult: string;
published
   property  Result: string read FResult write FResult stored false;
end;


TBooleanTool = class(TTool)
private
   FResult: boolean;
published
   property  Result: boolean read FResult write FResult stored false;
end;



TOrdinalTool = class(TTool)
 protected
   FResult: longInt;
   procedure SetResult(aResult: longInt);
 public
   property  Result: longInt read FResult write SetResult;
end;


{ Obecn technologie }
TTechnology = class(TTool)
 private
   FTools: TTools;
   procedure SetNewToolByModel(const Value: TToolModel);
    procedure SetToolToFree(const Value: TTool);
    function GetNoTools: boolean;
 protected
   FActiveToolIndex: longInt;
   function    GetActiveTool: TTool; virtual;
   function    GetActiveToolIndex: longInt; virtual;
   function    GetTools: TTools;
   function    GetTool(i: longInt): TTool;
   procedure   SetActiveTool(aTool: TTool); virtual;
   procedure   SetActiveToolIndex(aIndex: longInt); virtual;
   procedure   SetTools(aTools: TTools);
   procedure   PutTool(i: longInt; aElement: TTool);
   procedure   AddToolProc(aTool: TTool); virtual;
 public
   function    IsPossibleToQuit: boolean; override;
   function    ReadyForZeroActivity: boolean; override;
   procedure   EndZeroActivity; override;
   procedure   CreateSelfByModel; override;
   procedure   CreateDynamicPart; override;
   procedure   DestroyDynamicPart; override;
   procedure   UpdateModel; override;
   function    AddTool(aTool: TTool): TTool; virtual;
   function    CreateToolByModel(aModel: TToolModel): TTool; virtual;
   function    CreateTool(const aClass, aName: string): TTool; virtual;
   function    InsertTool(aIndex: longInt; aTool: TTool): TTool; virtual;
   procedure   RemoveTool(aTool: TTool); virtual;
   procedure   FreeTool(aTool: TTool); virtual;
   procedure   ExchangeFieldTool(var aField: TTool; const aTool: TTool);
   function    GetToolByName(const aName: string): TTool;
   procedure   on_tm_EndTask(var aMessage: TOrbisMessage; var Handled: boolean); override;

   property    Tools: TTools read GetTools write SetTools;
   property    Tool[Index: longint]: TTool read GetTool write PutTool;
   property    ActiveToolIndex: longInt read GetActiveToolIndex write SetActiveToolIndex;
   property    NoTools: boolean read GetNoTools;
published
   property    ActiveTool: TTool read GetActiveTool write SetActiveTool stored false;
   property    ToolToAdd: TTool write AddToolProc stored false;
   property    NewToolByModel: TToolModel write SetNewToolByModel stored false;
   property    ToolToFree: TTool write SetToolToFree stored false;
   property    ToolToRemove: TTool write RemoveTool stored false;
end;

{ Linern technologie }
TSeqTechnology = class(TTechnology)
 public
   procedure   DoIt; override;
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
end;

{ Iterace }
TIterateTechnology = class(TTechnology)
 protected
   FOrdinalTool: TOrdinalTool;
 public
   procedure TestIterace(var CanIterate: boolean); virtual;
   procedure DoIt; override;
   procedure Iterate; virtual;
   property  OrdinalTool: TOrdinalTool read FOrdinalTool;
end;


{ Selekce }
TSelectTechnology = class(TTechnology)
 private
    FElseTool: TTool;
    procedure SetElseTool(aTool: TTool);
 protected
    FSelIndex: longInt;
    procedure SetSelIndex(aIndex: longInt);
 public
    procedure DoIt; override;

 published
    property SelIndex: longInt read FSelIndex write SetSelIndex;
    property ElseTool: TTool read FElseTool write SetElseTool;
end;


function CreateAndExecuteTool(ToolClassName: string; aInput: TElement): boolean;

implementation

uses
   Fmx.Forms,
//   Messages,
//   WinProcs,
   e_Class,
   e_ElementModel,
   M_Orbis,
   {!T_Main,}
   FMX.Overbyte.MessageHandling,
   Worlds;

{ TTool }


function    TTool.NumberToPropertyName(aNumber: longInt): string;
begin
   result := '';
end;

function    TTool.GetPropertyNumber(aNumber: longint): string;
var
   S: string;
begin
   result := '';
   S := NumberToPropertyName(aNumber);
   if S = '' then exit;
   result := GetPropertyStringByName(S);
end;

procedure   TTool.SetPropertyNumber(aNumber: longint; const aValue: string);
var
   S: string;
begin
   S := NumberToPropertyName(aNumber);
   if S = '' then exit;
   SetPropertyStringByName(S, aValue);
end;


procedure   TTool.DoIt;
begin
   Status := Status + [st_Focused];
end;


function    TTool.GetNoParameters: boolean;
begin
  result:= (FParameters = nil) or
           ((FParameters <> nil) and (FParameters.Count = 0))
end;


procedure   TTool.AddParameter(aParameter: TElement);
begin
   Parameters.AddElement(aParameter);
end;

procedure   TTool.AddNamedParameter(aParameter: TElement);
begin
   SetNameParameter(aParameter.Name, aParameter);
end;



procedure   TTool.FreeParameter(aParameter: TElement);
begin
   Parameters.FreeElement(aParameter);
   { A co odkazy v Aliasech ?????????????????}
end;

procedure   TTool.FreeParameters;
begin
   Parameters.FreeAll;
end;

procedure   TTool.on_tm_StartTask(var aMessage: TOrbisMessage; var Handled: boolean);
begin
end;


procedure   TTool.on_tm_QuitTask;
begin
   if not IsPossibleToQuit then exit;
   Status := Status - [st_Focused];
   PostMsgToOwners(tm_EndTask, nil, false);
end;

procedure   TTool.on_tm_EndTask(var aMessage: TOrbisMessage; var Handled: boolean);
begin
end;


procedure   TTool.on_tm_Done(var aMessage: TOrbisMessage; var Handled: boolean);
begin
end;



procedure   TTool.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_StartTask: on_tm_StartTask(aMessage, Handled);
      tm_QuitTask : on_tm_QuitTask;
      tm_EndTask  : on_tm_EndTask(aMessage, Handled);
      tm_Done     : on_tm_Done(aMessage, Handled);
   end;

end;



procedure   TTool.SetInput(aInput: TElement);
begin
   FInput := aInput;
end;


procedure   TTool.SetOutput(aOutput: TElement);
begin
   FOutput := aOutput;
end;


function    TTool.GetParameters: TParameters;
begin
   if not assigned(FParameters) then
   begin
      FParameters := TParameters.Create;
      AddElement(FParameters);
   end;
   result := FParameters;
end;


procedure   TTool.SetParameters(aParameters: TParameters);
begin
   if assigned(FParameters) then
   begin
      FreeElement(FParameters);
      FParameters := nil;
   end;
   FParameters := aParameters;
   AddElement(FParameters);
end;


function    TTool.GetParameter(i: longInt): TElement;
begin
   result := Parameters.Elements[i];
end;

procedure   TTool.PutParameter(i: longInt; aElement: TElement);
begin
   Parameters.Elements[i] := aElement;
end;


function    TTool.GetNameParameter(aName: string): TElement;
begin
   result := Parameters.Element[aName];
end;


procedure   TTool.SetNameParameter(aName: string; aElement: TElement);
begin
   Parameters.Element[aName] := aElement;
end;


function    TTool.GetBusy: boolean;
begin
   result := st_BUSY in FStatus;
end;

procedure   TTool.SetBusy(aBusy: boolean);
begin
   If not aBusy then FStatus := FStatus - [st_Busy]
                else FStatus := FStatus + [st_Busy];
end;

procedure TTool.SetClient(const Value: boolean);
begin
  FClient := Value;
end;

function    TTool.GetBreak: boolean;
begin
   result := (st_BREAK in FStatus) or (st_BREAK_AND_DOIT in FStatus);
end;

procedure   TTool.SetBreak(aBreak: boolean);
begin
   If not aBreak then FStatus := FStatus - [st_Break, st_BREAK_AND_DOIT]
                 else FStatus := FStatus + [st_Break];
end;





procedure TTool.EndProgress;
begin
   if not InProgress then exit;
   PostMsg(tm_EndProgress, nil, nil, false);
   InProgress := false;
   Application.ProcessMessages;
end;

procedure TTool.SetProgress(const Value: integer);
var
  PM: TProgressMessage;
begin
   if not InProgress then exit;
   PM := TProgressMessage.CreateAll(tm_Progress, Self, nil, nil, false);
   PM.Progress := Value;
   PM.Post;
{$IFDEF AUTOREFCOUNT}
   PM.__ObjAddRef;
{$ENDIF}
end;

procedure TTool.SetProgressText(const Value: string);
begin
   if not inProgress then exit;
   PostTextMsg(tm_ProgressText, Value, nil);
end;

procedure TTool.StartProgress(const ProcessName: string; MinProgress,
  MaxProgress: integer);
var
  PM: TProgressMessage;
begin
   if NoProgress or (GetCurrentThreadID <> MainThreadID) then
      exit;
   PM := TProgressMessage.CreateAll(tm_StartProgress, Self, nil, nil, false);
   PM.ProcessName := ProcessName;
   PM.MinProgress := MinProgress;
   PM.MaxProgress := MaxProgress;
   PM.Post;
{$IFDEF AUTOREFCOUNT}
   PM.__ObjAddRef;
{$ENDIF}
   InProgress := true;
   Application.ProcessMessages;
end;

procedure TTool.SetInProgress(const Value: boolean);
begin
   if Value then FStatus := FStatus + [st_Progress]
            else FStatus := FStatus - [st_Progress];
end;

function TTool.GetInProgress: boolean;
begin
   result := st_Progress in FStatus;
end;

destructor TTool.Destroy;
begin
   if InProgress then
      EndProgress;
   inherited;
end;

procedure TTool.SetSelfModel(const Value: TToolModel);
var
   TM: TToolModel;
begin
   TM := TToolModel(FindElementClass(TToolModel));
   if TM <> nil then FreeElement(TM);
   AddElement(Value);
end;

function TTool.GetSelfModel: TToolModel;
begin
   result := TToolModel(FindElementClass(TToolModel));
end;

procedure TTool.SetDynamicTool(const Value: boolean);
begin
   FDynamicTool := Value;
end;

procedure TTool.CreateSelfByModel;
var
   M: TToolModel;
   PM: TPropertyModel;
   P,L: TOwnerEntity;
   i,j: integer;
   ML: TModelLink;
begin
   M := SelfModel;
   if M = nil then
   begin
      PostTextMsg(tm_AddToProtocol,'Tool: '+ Name + ' class: '+ ClassName + ' Has no model!',nil);
      exit;
   end;
   DynamicTool := M.DynamicTool;
   // Properties
   if not M.NoModelProperties then
   begin
      P := M.ModelProperties;
      for i := 0 to P.LastIndex do
      begin
         PM := TPropertyModel(P.Elements[i]);
         if not PM.LinkOnly then
            SetPropertyStringByName(PM.Name,PM.Value);
         if not PM.NoLinks then
         begin
            L := PM.Links;
            for j := 0 to P.LastIndex do
            begin
               ML := TModelLink(L.Elements[j]);
               ML.Link;
            end;
         end;
      end;
   end;
   // Links
   if not M.NoLinks then
   begin
      P := M.Links;
      for i := 0 to P.LastIndex do
      begin
         ML := TModelLink(P.Elements[i]);
         ML.Link;
      end;
   end;
end;

procedure TTool.CreateDynamicPart;
begin

end;

procedure TTool.UpdateModel;
var
   M: TToolModel;
   PM: TPropertyModel;
   P: TOwnerEntity;
   i: integer;
begin
   M := SelfModel;
   if M = nil then exit;
   //DynamicTool := M.DynamicTool;
   if not M.NoModelProperties then
   begin
      P := M.ModelProperties;
      for i := 0 to P.LastIndex do
      begin
         PM := TPropertyModel(P.Elements[i]);
         PM.Value := GetPropertyStringByName(PM.Name);
      end;
   end;
end;

procedure TTool.DestroyDynamicPart;
begin

end;

function TTool.IsPossibleToQuit: boolean;
begin
   result := true;
end;

function TTool.GetIdle: boolean;
begin
   result := st_IDLE in FStatus;
end;

procedure TTool.SetIdle(const Value: boolean);
begin
   If not Value then FStatus := FStatus - [st_IDLE]
                else FStatus := FStatus + [st_IDLE];
end;

procedure TTool.SetMaxProgress(const Value: integer);
var
   PM: TProgressMessage;
begin
   if not InProgress then exit;
   PM := TProgressMessage.CreateAll(tm_MaxProgress, Self, nil, nil, false);
   PM.MaxProgress := Value;
   PM.Post;
{$IFDEF AUTOREFCOUNT}
   PM.__ObjAddRef;
{$ENDIF}
end;

function TTool.GetZeroActivity: boolean;
begin
   result := st_ZeroActivity in FStatus;
end;


procedure TTool.SetZeroActivity(const Value: boolean);
begin
   If not Value then FStatus := FStatus - [st_ZeroActivity]
                else FStatus := FStatus + [st_ZeroActivity];
end;

procedure TTool.EndZeroActivity;
begin
   ZeroActivity := false;
end;

function TTool.ReadyForZeroActivity: boolean;
begin
   ZeroActivity := true;
   result := not HighActivity;
end;

function TTool.GetHighActivity: boolean;
begin
   result := st_HighActivity in FStatus;
end;

procedure TTool.SetHighActivity(const Value: boolean);
begin
   If not Value then FStatus := FStatus - [st_HighActivity]
                else FStatus := FStatus + [st_HighActivity];
   If not Value and ZeroActivity then
      PostMsg(tm_ZeroActivity,nil,nil,false);
end;

procedure TTool.SetNoProgress(const Value: boolean);
begin
   if Value then FStatus := FStatus + [st_NoProgress]
            else FStatus := FStatus - [st_NoProgress];
end;

function TTool.GetNoProgress: boolean;
begin
   result := st_NoProgress in FStatus;
end;

{ TOrdinalTool }
procedure   TOrdinalTool.SetResult(aResult: longInt);
begin
   FResult := aResult;
end;


{ TTechnology }

function    TTechnology.GetActiveTool: TTool;
begin
  result:= nil;
// Olda 7.7.98 - nemm rd exceptiony
  if Tools = nil then exit;
  if (FActiveToolIndex < 0) or (FActiveToolIndex > Tools.LastIndex) then exit;
  if Tools.Count = 0 then exit;

   try
      result := TTool(Tool[FActiveToolIndex])
   except
      result := nil;
   end;
end;

function    TTechnology.GetActiveToolIndex: longInt;
begin
   result := FActiveToolIndex;
end;


function    TTechnology.GetTools: TTools;
begin
   if not assigned(FTools) then
   begin
      FTools := TTools.Create;
      AddElement(FTools);
   end;
   result := FTools;
end;


function    TTechnology.GetTool(i: longInt): TTool;
begin
   result := TTool(Tools.Elements[i]);
end;


procedure   TTechnology.SetTools(aTools: TTools);
begin
   if assigned(FTools) then
   begin
      FreeElement(FTools);
      FTools := nil;
   end;
   FTools := aTools;
end;

procedure   TTechnology.SetActiveTool(aTool: TTool);
begin
   try
      ActiveToolIndex := Tools.IndexOf(aTool);
   except
      ActiveToolIndex := -1;
   end;
end;

procedure   TTechnology.SetActiveToolIndex(aIndex: longInt);
begin
   if (aIndex > -1) and (aIndex < Tools.Count) then FActiveToolIndex := aIndex
                                              else FActiveToolIndex := -1;
end;

procedure   TTechnology.PutTool(i: longInt; aElement: TTool);
begin
   Tools.Elements[i] := aElement;
end;

function    TTechnology.AddTool(aTool: TTool): TTool;
begin
    Tools.AddElement(aTool);
    Result:= aTool;
end;

procedure   TTechnology.AddToolProc(aTool: TTool);
begin
   AddTool(aTool);
   //Tools.AddElement(aTool);
end;

function    TTechnology.CreateTool(const aClass, aName: string): TTool;
begin
   result := TTool(Tools.CreateElement(aClass, aName));
end;


function    TTechnology.InsertTool(aIndex: longInt; aTool: TTool): TTool;
begin
    Tools.InsertElement(aIndex, aTool);
    Result:= aTool;
end;

procedure   TTechnology.RemoveTool(aTool: TTool);
begin
    Tools.RemoveElement(aTool);
end;

procedure   TTechnology.FreeTool(aTool: TTool);
begin
    Tools.FreeElement(aTool);
end;

function    TTechnology.GetToolByName(const aName: string): TTool;
begin
  result:= TTool(Tools.FindElementName(aName))
end;


procedure   TTechnology.on_tm_EndTask(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   FreeTool(TTool(aMessage.Sender));
   Handled := true;
end;





function TTechnology.CreateToolByModel(aModel: TToolModel): TTool;
var
   EC: TElementClass;
begin
   result := nil;
   if aModel = nil then exit;
   if aModel.ElementClassName = '' then exit;
   EC := GetOrbisClass(aModel.ElementClassName);
   if EC = nil then
   begin
      PostTextMsg(tm_AddToProtocol,'Unknown class: ' + aModel.ElementClassName, nil);
      exit;
   end;
   if not EC.InheritsFrom(TTool) then
   begin
      PostTextMsg(tm_AddToProtocol,'Class: ' + aModel.ElementClassName+' is not a Tool', nil);
      exit;
   end;
   result := TTool(EC.Create);
   result.Name := aModel.Name;
   AddTool(result);
   if aModel.AttachToPropertyName <> '' then
      SetPropertyLongIntByName(aModel.AttachToPropertyName, longint(result));
   result.SelfModel := aModel;
   result.CreateSelfByModel;
end;


procedure TTechnology.CreateSelfByModel;
begin
   inherited;
   if DynamicTool then exit;
   CreateDynamicPart;
end;

procedure TTechnology.CreateDynamicPart;
var
   M: TToolModel;
   MT: TOwnerEntity;
   i: integer;
begin
   inherited;
   M := SelfModel;
   if M = nil then exit;
   if M.NoModelTools then exit;
   MT := M.ModelTools;
   for i := 0 to MT.LastIndex do
   begin
      M := TToolModel(MT.Elements[i]);
      CreateToolByModel(M);
   end;
end;

procedure TTechnology.SetNewToolByModel(const Value: TToolModel);
begin
   CreateToolByModel(Value);
end;

procedure TTechnology.DestroyDynamicPart;
begin
   //PostTextMsg(tm_AddToProtocol,Name+': DestroyDynamic',nil);
   UpdateModel;
   Tools.FreeAll;
end;

procedure TTechnology.ExchangeFieldTool(var aField: TTool;
  const aTool: TTool);
var
   JustExists: boolean;
begin
   JustExists := false;
   if aField <> nil then
   begin
      FreeTool(aField);
      aField := nil;
   end
   else
      JustExists := not NoTools and (Tools.IndexOf(aTool) >= 0);
   aField := aTool;
   if not JustExists then
      AddTool(aTool);
end;

procedure TTechnology.SetToolToFree(const Value: TTool);
begin
   FreeTool(Value);
end;

function TTechnology.IsPossibleToQuit: boolean;
var
   i: integer;
begin
   //PostTextMsg(tm_AddToProtocol,Name+' Quitting...',nil);
   result := true;
   if Tools.Count > 0 then
   for i := 0 to Tools.Count - 1 do
   begin
      result := Tool[i].IsPossibleToQuit;
      if not result then
      begin
         //PostTextMsg(tm_AddToProtocol,Name+' false',nil);
         exit;
      end;
   end;
   //PostTextMsg(tm_AddToProtocol,Name+' true',nil)
end;

procedure TTechnology.UpdateModel;
var
   i: integer;
begin
   inherited;
   for i := 0 to Tools.LastIndex do
   begin
      Tool[i].UpdateModel;
   end;
end;

function TTechnology.GetNoTools: boolean;
begin
   result := true;
   if FTools = nil then exit;
   if Tools.NoElements then exit;
   result := false;
end;

procedure TTechnology.EndZeroActivity;
var
   i: integer;
begin
   inherited;
   if Tools.Count > 0 then
   for i := 0 to Tools.Count - 1 do
   begin
      Tool[i].EndZeroActivity;
   end;
end;

function TTechnology.ReadyForZeroActivity: boolean;
var
   res: boolean;
   i: integer;
begin
   result := inherited ReadyForZeroActivity;
   if Tools.Count > 0 then
   for i := 0 to Tools.Count - 1 do
   begin
      res := Tool[i].ReadyForZeroActivity;
      if result then
         result := res;
   end;
end;

{ TSeqTechnology }


procedure   TSeqTechnology.DoIt;
var
   i: longInt;
   aTool: TTool;
begin
   inherited DoIt;
   {
   if Input = nil then exit;
   }
   if NoElements then exit;
{!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!}
   for i := 0 to (Tools.Count - 1) do
   begin
      FActiveToolIndex := i;
      aTool := Tool[i];
      aTool.Parameters := Parameters;
      aTool.Input := Input;
      aTool.DoIt;
      OutPut := aTool.OutPut;
      Input  := Output;
      {
      if OutPut = nil then break;
      }
   end;
{   PostMsg(tm_EndTask, }
{   SendMessage(Application.Handle, WM_Command, cm_EndOfTask, 0); }
end;


procedure   TSeqTechnology.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   inherited HandleMessage(aMessage, Handled);
end;

{ TIterateTechnology }

procedure TIterateTechnology.TestIterace(var CanIterate: boolean);
begin
end;

procedure TIterateTechnology.DoIt;
var
   CanIterate: boolean;
begin
   Inherited Doit;
   TestIterace(CanIterate)
   {
   if CanIterate then
   }
end;

procedure TIterateTechnology.Iterate;
begin
end;



{ TSelectTechnology }
procedure TSelectTechnology.SetSelIndex(aIndex: longInt);
begin
   FSelIndex := aIndex;
end;

procedure TSelectTechnology.SetElseTool(aTool: TTool);
begin
  if aTool = nil then exit;
  FElseTool:= aTool;
  AddElement(aTool);
end;



procedure TSelectTechnology.DoIt;
var
  aTool: TTool;
begin
  aTool:= nil;
  if (Tools.Count > 0) and (FSelIndex >= 0) and (FSelIndex < Tools.Count) then
     aTool:= TTool(Tools.Elements[FSelIndex]);

  if not (TElement(aTool) is TTool) then
     aTool:= FElseTool;

  if not (TElement(aTool) is TTool) then exit;

  aTool.Input:= Input;
  aTool.DoIt;
  Output:= aTool.Output
end;



function CreateAndExecuteTool(ToolClassName: string; aInput: TElement): boolean;
var
  aTool: TTool;
begin
  result:= false;
  aTool:= TTool(CreateElementByName(ToolClassName));
  if aTool = nil then exit;

  aTool.Input:= aInput;
  aTool.DoIt;
  aTool.Free;
  result:= true
end;



{ TProgressMessage }

procedure TProgressMessage.SetMaxProgress(const Value: integer);
begin
  FMaxProgress := Value;
end;

procedure TProgressMessage.SetMinProgress(const Value: integer);
begin
  FMinProgress := Value;
end;

procedure TProgressMessage.SetProcessName(const Value: string);
begin
  FProcessName := Value;
end;

procedure TProgressMessage.SetProgress(const Value: integer);
begin
  FProgress := Value;
end;

BEGIN
   RegisterOrbisClass(TProgressMessage);
   RegisterOrbisClass(TToolProperties);
   RegisterOrbisClass(TTool);
   RegisterOrbisClass(TResultTool);
   RegisterOrbisClass(TBooleanTool);
   RegisterOrbisClass(TTechnology);
   RegisterOrbisClass(TSeqTechnology);
   RegisterOrbisClass(TSelectTechnology);
END.
