unit e_InhMdl;
{
[Unit_Description]
Description=Elementy pro tvorbu ddin hierarchie
Admin=Petr
TechTopic=Elements
Topic=System
Version=1.0
}

interface

uses
   Elements;

const

   err_NoItem        = 1;
   err_ClassNotFound = 2;
   err_WrongItem     = 3;
   err_WrongElement  = 4;
   err_NoInterpret   = 5;
   err_NoInstance    = 6;
   err_NoClass       = 7;


type

//TInheritanceModel = class;

TModelItems = class(TEnvelopeEntity)
protected
    function  GetName: string; override;
end;

TInheritanceModel = class(TOwnerEntity)
private
   FClass: TElementClass;
   FItems: TModelItems;
protected
   function  GetItems: TModelItems; virtual;
   function  GetNoItems: boolean; virtual;
   procedure SetNextItem(aValue: TInheritanceModel); virtual;
   function  GetItem(Index: longInt): TInheritanceModel;
   procedure SetEClass(aValue: TElementClass); virtual;
   function  GetClassName: string;
   procedure SetClassName(aValue: string); virtual;
public
   function  GetItemForClass(aValue: TElementClass): TInheritanceModel; virtual;
   function  GetItemForClassName(const aValue: string): TInheritanceModel; virtual;
   function  ItsMyClass(aValue: TElementClass): boolean; virtual;
   procedure AddItem(aValue: TInheritanceModel);
   property  Item[Index: longint]: TInheritanceModel read GetItem;
   property  ElementClass: TElementClass read FClass write SetEClass;
published
   property  Items: TModelItems read GetItems stored false;
   property  NoItems: boolean read GetNoItems stored false;
   property  NextItem: TInheritanceModel write SetNextItem stored false;
   property  ElementClassName: string read GetClassName write SetClassName;
end;

TInheritanceModelClass = class of TInheritanceModel;

TInheritanceModelEntity = class(TInheritanceModel)
private
   FError: longInt;
   FActualItem: TInheritanceModel;
   FItemClass: TInheritanceModelClass;
   function  GetActualClassName: string; virtual;
   function  GetIsError: boolean; virtual;
   function  GetItemClassName: string;
   procedure SetItemClassName(const aValue: string);
   procedure SetNewClassName(const aValue: string); virtual;
   procedure SetActualClassName(const aValue: string); virtual;
   procedure SetFindClassName(const aValue: string); virtual;
   procedure SetDeleteAI(aValue: boolean); virtual;

public
   constructor Create; override;
   function  GetItemClass: TInheritanceModelClass; virtual;
published
   property ItemClassName: string read GetItemClassName write SetItemClassName;
   property ActualItem: TInheritanceModel read FActualItem stored false;
   property NewClassName: string write SetNewClassName stored false;
   property FindClassName: string write SetFindClassName stored false;
   property ActualClassName: string read GetActualClassName write SetActualClassName  stored false;
   property DeleteActualItem: boolean write SetDeleteAI stored false;
   property IsError: boolean read GetIsError stored false;
   property Error: longInt read FError write FError stored false;
end;

implementation

uses
   e_Class;

{ TModelItems }

function  TModelItems.GetName: string;
begin
   result := 'ITEMS'
end;


{ TInheritanceModel }

function  TInheritanceModel.GetItemForClassName(const aValue: string): TInheritanceModel;
var
   EC: TElementClass;
begin
   EC := GetOrbisClass(aValue);
   result := GetItemForClass(EC);
end;


function  TInheritanceModel.GetNoItems: boolean;
begin
   result := Items.NoElements;
end;

procedure TInheritanceModel.AddItem(aValue: TInheritanceModel);
begin
   Items.AddElement(aValue);
end;


function  TInheritanceModel.GetItems: TModelItems;
begin
   if not assigned(FItems) then
   begin
      FItems := TModelItems(FindElementClass(TModelItems));
      if FItems = nil then
      begin
         FItems := TModelItems.Create;
         AddElement(FItems);
      end;
   end;
   result := FItems;
end;




procedure TInheritanceModel.SetEClass(aValue: TElementClass);
begin
   FClass := aValue;
   if FClass <> nil then Name := aValue.ClassName;
end;


function  TInheritanceModel.GetClassName: string;
begin
   if ElementClass = nil then result := ''
                         else result := ElementClass.ClassName;
end;


procedure TInheritanceModel.SetClassName(aValue: string);
begin
   ElementClass := GetOrbisClass(aValue);
end;



function  TInheritanceModel.GetItem(Index: longInt): TInheritanceModel;
begin
   result := TInheritanceModel(Items.Elements[Index]);
end;



function  TInheritanceModel.ItsMyClass(aValue: TElementClass): boolean;
begin
   if FClass = nil then result := true
                   else result :=  aValue.InheritsFrom(FClass){ElementClass;}
end;




procedure TInheritanceModel.SetNextItem(aValue: TInheritanceModel);
var
   i: longInt;
   IM: TInheritanceModel;
   IsModel: boolean;
begin
   for i := Items.LastIndex downto 0  do
   begin
      IM := Item[i];
      IsModel := TElement(IM) is TInheritanceModel;
      if ISModel then
      begin
         if IM.ElementClass.InheritsFrom(aValue.ElementClass) then
         begin
            Items.RemoveElement(IM);
            aValue.AddItem(IM);
         end
         else
         if aValue.ElementClass.InheritsFrom(IM.ElementClass) then
         begin
            IM.SetNextItem(aValue);
         end;
      end;
   end;
   if aValue.NoOwners then AddItem(aValue);
end;


function  TInheritanceModel.GetItemForClass(aValue: TElementClass): TInheritanceModel;
var
   i : longInt;
   IM: TInheritanceModel;
begin
   if ElementClass = nil then result := nil
                         else result := Self;
   if aValue = ElementClass then exit;
   if NoItems then exit;
   for i := 0 to Items.LastIndex do
   begin
      IM := Item[i];
      //if not (IM.ClassType is TInheritanceModel) then continue;
      if IM.ItsMyClass(aValue) then
      begin
         result := IM.GetItemForClass(aValue);
         exit;
      end;
   end;
end;


{ TInheritanceModelEntity }


function  TInheritanceModelEntity.GetItemClass: TInheritanceModelClass;
begin
   result := TInheritanceModel;
end;


function  TInheritanceModelEntity.GetItemClassName: string;
begin
   if FItemClass = nil then FItemClass := TInheritanceModel;
   result := FItemClass.ClassName;
end;

procedure TInheritanceModelEntity.SetItemClassName(const aValue: string);
var
   c: TElementClass;
begin
   c := GetOrbisClass(aValue);
   if (c = nil)  or not(c.inheritsFrom(TInheritanceModel))then c := TInheritanceModel;
   FItemClass := TInheritanceModelClass(c);
end;

constructor TInheritanceModelEntity.Create;
begin
   inherited Create;
   FItemClass := GetItemClass;
   ElementClass := nil;//TElement;
   FActualItem := nil;
end;


function TInheritanceModelEntity.GetActualClassName: string;
begin
   if ActualItem = nil then
   begin
      Error := err_WrongItem;
      result := 'NIL';
   end
   else
      result := ActualItem.ElementClass.ClassName;
end;


procedure TInheritanceModelEntity.SetNewClassName(const aValue: string);
var
   c: TElementClass;
begin
   FActualItem := nil;
   Error := 0;
   if aValue = '' then exit;
   c := GetOrbisClass(aValue);
   if c = nil then
   begin
      Error := err_ClassNotFound;
      exit;
   end;
   FActualItem := FItemClass.Create;
   FActualItem.ElementClass := c;
   NextItem := FActualItem;
end;


procedure TInheritanceModelEntity.SetActualClassName(const aValue: string);
begin
   Error := 0;
   if (ActualItem = nil) then
   begin
      Error := err_WrongItem;
      exit;
   end;
   ActualItem.Free;
   NewClassName := aValue;
end;

function  TInheritanceModelEntity.GetIsError: boolean;
begin
   result := Error > 0;
end;

procedure TInheritanceModelEntity.SetFindClassName(const aValue: string);
var
   IM: TInheritanceModel;
   ec: TElementClass;
begin
   Error := 0;
   ec := GetOrbisClass(aValue);
   if ec = nil then
   begin
      Error := err_ClassNotFound;
      FActualItem := nil;
      exit;
   end;
   IM := GetItemForClass(ec);
   if IM = nil then
   begin
      Error := err_NoItem;
      FActualItem := nil;
   end
   else
   begin
      FActualItem := IM;
   end;
end;





procedure TInheritanceModelEntity.SetDeleteAI(aValue: boolean);
begin
   if ActualItem = nil then exit;
   ActualItem.Free;
   FActualItem := nil;
   Error := err_WrongItem;
end;





END.
