unit T_Recurs;
{
[Unit_Description]
Description=Recursor - pedek vech rekurzivnch nstroj (interprety a tak)
Admin=Olda
TechTopic=Tools
Topic=System
Version=1.0
Edit
  TRecursor - pidn published LEVEL
}
{
********************************************************
**                                                    **
**                    ORBIS                           **
**         Skupina Recursor a Interpret               **
**                                                    **
**                  (c) 1996                          **
**                                                    **
********************************************************
}

{ Pedek vech recursor, tj. nstroj, kter rekurzivn spracovvaj
  njak objekt s Elements, t.j. oekv na vstupu TEntity.
  Vlastn process me vyuvat nstroje uleen ve vlastnch
  Elements.

  Po standartnm DoIt recursor prochz vechny vtve
  hierarchie objektu a posuzuje zda
  - dan objekt zpracovat ped (QProcessBefore) nebo
  - jt dovnit (QGoInside) nebo
  - zpracovat objekt po (QProcessAfter).
  Dle se nastavuje co se s kadm objektem provd (Process)

  Rozdl mezi zpracovnm ped a potom:
  Ped: pirozenj zpsob prochzen hierarchi, zpracovv se "zleva"
        do prava od nejvyho k nejnimu. Vy je vzorem pro prce nich.
  Po: Vy se zpracovvaj a po zpracovn svch podzench. M to
      vznam tehdy, kdy zpracovn vyho by mlo zviset na nich - vy
      shrnuje vsledky prce nich.
}

interface
uses
//  SysUtils,
//  Wintypes,

  Elements,
  Tools;

type

TRecursor = class(TTechnology)
 private
   FNotInside: boolean;
   FLevel: integer;
 protected
   procedure GoInside(Element: TElement); virtual;
   function  QGoInside(Element: TElement): boolean; virtual;
   function  QProcessBefore(Element: TElement): boolean; virtual;
   function  QProcessAfter(Element: TElement): boolean; virtual;
   procedure Process(Element: TElement); virtual;
   procedure PrepareOutPut; virtual;
 public
   procedure DoIt; override;
 published
   property NotInside: boolean read FNotInside write FNotInside;
   property Level: integer read FLevel write FLevel;
end;


TRecursorWithTools = class(TRecursor)
private
  FBeforeTool: TTool;
  FAfterTool: TTool;
  FQGoInsideTool: TTool;
protected
  function  QGoInside(Element: TElement): boolean; override;
  function  QProcessBefore(Element: TElement): boolean; override;
  function  QProcessAfter(Element: TElement): boolean; override;
published
  property BeforeTool: TTool read FBeforeTool write FBeforeTool;
  property AfterTool: TTool read FAfterTool write FAfterTool;
  property QGoInsideTool: TTool read FQGoInsideTool write FQGoInsideTool;
end;



implementation


uses
//   WinProcs, Dialogs, Printers, Forms,
//   Worlds, T_Intera,
   e_Class;
//   M_Orbis;

{ TRecursor }


function  TRecursor.QGoInside(Element: TElement): boolean;
begin
   Result:= Element is TEntity
end;

function  TRecursor.QProcessBefore(Element: TElement): boolean;
begin
  Result:= false
end;

function  TRecursor.QProcessAfter(Element: TElement): boolean;
begin
  Result:= false
end;


procedure TRecursor.PrepareOutput;
begin
end;


procedure TRecursor.Process;
begin
end;



procedure TRecursor.GoInside(Element: TElement);
var
  I: longInt;
  aElement: TElement;
begin
  if NotInside then exit;
  if not (Element is TEntity) then exit;

  inc(FLevel);

  for I:= 0 to TEntity(Element).Count-1 do
  begin
    aElement:= TEntity(Element).Elements[I];
    if aElement = nil then continue;

    if QProcessBefore(aElement) then
       Process       (aElement);
    if QGoInside     (aElement) then
       GoInside      (aElement);
    if QProcessAfter (aElement) then
       Process       (aElement)
  end;

  dec(FLevel)
end;



procedure TRecursor.DoIt;
begin
  FLevel:= 1;
  if Input<>nil then
  begin
    PrepareOutPut;
    if QProcessBefore(Input) then
       Process(Input);
    if QGoInside(Input) then
       GoInside(Input);
    if QProcessAfter(Input) then
       Process(Input)
  end
end;




          { TRecursorWithTools }

function  TRecursorWithTools.QGoInside(Element: TElement): boolean;
begin
  result:= true;
  if not (Element is TEntity) then exit;
  if QGoInsideTool = nil then exit;
  if not (TElement(QGoInsideTool) is TTool) then exit;
  QGoInsideTool.Input:= Element;
  QGoInsideTool.DoIt;
  result:= QGoInsideTool.Output <> nil
end;


function  TRecursorWithTools.QProcessBefore(Element: TElement): boolean;
begin
  result:= false;
  if BeforeTool = nil then exit;
  if not (TElement(BeforeTool) is TTool) then exit;
  BeforeTool.Input:= Element;
  BeforeTool.DoIt;
end;


function  TRecursorWithTools.QProcessAfter(Element: TElement): boolean;
begin
  Result:= false;
  if AfterTool = nil then exit;
  if not (TElement(AfterTool) is TTool) then exit;
  AfterTool.Input:= Element;
  AfterTool.DoIt;
end;



INITIALIZATION
   RegisterOrbisClass(TRecursorWithTools);
END.
