unit t_Sort;

interface
uses
   Elements,
   Tools;

type

TSortElements = class(TTool)
  private
    FQE: TQuickEntity;
    FPropertyName: string;
    FPropertyName1: string;
    FDown: boolean;
    procedure SetPropertyName(const Value: string);
    procedure SetPropertyName1(const Value: string);
    function GetQE: TQuickEntity;
    procedure SetDown(const Value: boolean);
protected
   function FindIndex(const aElement: TElement; var NearestRight: integer): integer;
   property QE: TQuickEntity read GetQE;
public
   procedure DoIt; override;
   function NumberToPropertyName(aNumber: longInt): string; override;
   function CompareElements(E1,E2: TElement): integer; virtual;
   function DownPropertyNumber: integer; virtual;
published
   property PropertyName: string read FPropertyName write SetPropertyName;
   property PropertyName1: string read FPropertyName1 write SetPropertyName1;
   property Down: boolean read FDown write SetDown;
end;

TSortElementsByString = class(TSortElements)
public
   function CompareElements(E1,E2: TElement): integer; override;
end;

TSortElementsByInteger = class(TSortElements)
public
   function CompareElements(E1,E2: TElement): integer; override;
end;

TSortElementsBy2Integers = class(TSortElements)
public
   function CompareElements(E1,E2: TElement): integer; override;
   function DownPropertyNumber: integer; override;
end;

TSortElementsByFloat = class(TSortElements)
public
   function CompareElements(E1,E2: TElement): integer; override;
end;

implementation
uses
   SysUtils,
   e_Class,
   u_String;

{ TSortElements }

function TSortElements.CompareElements(E1, E2: TElement): integer;
begin
   result := 1; // default je vetsi
end;

procedure TSortElements.DoIt;
var
   i,ix,li: integer;
   En: TEntity;
   E: TElement;
begin
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   En := TEntity(Input);
   if En.NoElements then exit;
   QE.Clear;
   for i := 0 to En.LastIndex do
   begin
      E := En.Elements[i];
      FindIndex(E,ix);
      if ix > -1 then QE.InsertElement(ix, E)
                else QE.AddElement(E);
   end;
   li := QE.LastIndex;
   for i := 0 to QE.LastIndex do
   begin
      if Down then
         En.Elements[li-i] := QE.Elements[i]
      else
         En.Elements[i] := QE.Elements[i];
   end;
   QE.Clear;
end;

function TSortElements.DownPropertyNumber: integer;
begin
   result := 1;
end;

function TSortElements.FindIndex(const aElement: TElement; var NearestRight: integer): integer;
var
   L,R,C: longInt;
   LResult, RResult, CResult: integer;
begin
   result := -1;
   NearestRight := -1;
   if QE.NoElements then exit;
   L := 0;
   R := QE.Count - 1;
   RResult := CompareElements(aElement, QE.Elements[R]);
   if (RResult > 0) then exit;
   if RResult = 0 then
   begin
      result := R;
      exit;
   end;
   if R > L then
   begin
      LResult := CompareElements(aElement, QE.Elements[L]);
      if (LResult < 0) then
      begin
         NearestRight := L;
         exit;
      end;
      if LResult = 0 then
      begin
         result := L;
         NearestRight := L + 1;
         exit;
      end;
   end
   else
   begin
      NearestRight := L;
      exit;
   end;
   repeat
      NearestRight := R;
      if R - L <= 1 then exit;
      C := ((R - L) div 2) + L;
      CResult := CompareElements(aElement, QE.Elements[C]);
      if CResult < 0 then R := C else
      if CResult > 0 then L := C else
      begin
         result := C;
         NearestRight := C + 1;
         exit;
      end;
   until L >= R;
end;

function TSortElements.GetQE: TQuickEntity;
begin
   if FQE = nil then
   begin
      FQE := TQuickEntity.Create;
      AddElement(FQE);
   end;
   result := FQE;
end;

function TSortElements.NumberToPropertyName(aNumber: Integer): string;
begin
   if aNumber = DownPropertyNumber then
      result := 'Down'
   else
   case aNumber of
      0: result := 'PropertyName';
      1: result := 'PropertyName1';
   end;
end;

procedure TSortElements.SetDown(const Value: boolean);
begin
  FDown := Value;
end;

procedure TSortElements.SetPropertyName(const Value: string);
begin
  FPropertyName := Value;
end;

procedure TSortElements.SetPropertyName1(const Value: string);
begin
  FPropertyName1 := Value;
end;

{ TSortElementsByString }

function TSortElementsByString.CompareElements(E1, E2: TElement): integer;
begin
   result := AnsiCompareText(E1.GetPropertyStringByName(PropertyName), E2.GetPropertyStringByName(PropertyName));
end;

{ TSortElementsByInteger }

function TSortElementsByInteger.CompareElements(E1, E2: TElement): integer;
var
   i1,i2: integer;
begin
   i1 := E1.GetPropertyLongIntByName(PropertyName);
   i2 := E2.GetPropertyLongIntByName(PropertyName);
   result := i1 - i2;
end;

{ TSortElementsBy2Integers }

function TSortElementsBy2Integers.CompareElements(E1,E2: TElement): integer;
var
   i1,i2: integer;
begin
   i1 := E1.GetPropertyLongIntByName(PropertyName);
   i2 := E2.GetPropertyLongIntByName(PropertyName);
   result := i1 - i2;
   if result = 0 then
   begin
      i1 := E1.GetPropertyLongIntByName(PropertyName1);
      i2 := E2.GetPropertyLongIntByName(PropertyName1);
      result := i1 - i2;
   end;
end;

function TSortElementsBy2Integers.DownPropertyNumber: integer;
begin
   result := 2;
end;

{ TSortElementsByFloat }

function TSortElementsByFloat.CompareElements(E1, E2: TElement): integer;
var
   Ex1,Ex2: Extended;
begin
   Ex1 := StrToExtended(E1.GetPropertyStringByName(PropertyName));
   Ex2 := StrToExtended(E2.GetPropertyStringByName(PropertyName));
   result := -1;
   if Ex1 > Ex2 then
      result := 1
   else
   if Ex1 = Ex2 then
      result := 0;
end;

initialization
   RegisterOrbisClass(TSortElementsByString);
   RegisterOrbisClass(TSortElementsByInteger);
   RegisterOrbisClass(TSortElementsBy2Integers);
   RegisterOrbisClass(TSortElementsByFloat);
end.
