unit T_HelLHC;

interface
uses
// KERNEL
  Elements,
  Tools,
// GLIB
  g_elems,
// SPACE
  //e_Space,
  T_HelSel;

type

THeleZaradLHC = class(THeleBasic)
private
  FTemp: TEntity;
  FGroup: integer;
  FLayerName: string;
  FPropertyClassName: string;
  FPropertyClass: TElementClass;
    FHolesGroup: integer;
  function GetPSKByAttribs(anAre: TAreaElement;  var JeToPSK: boolean): TRectEntity;
  function GetPSKBySpace(anAre: TAreaElement;  var JeToPSK: Boolean;
                         var aLHC, aOdd, aDil, aPor, aPSK: string): TRectEntity;
  procedure AddToPSK(JeToPSK: Boolean; aPSK: TRectEntity; aAre: TAreaElement);
  procedure SetGroup(const Value: integer);
  procedure SetPropertyClass(const Value: string);
    procedure SetHolesGroup(const Value: integer);
public
  procedure DoIt; override;
  procedure NactiLayer;
published
  property LayerName: string read FLayerName write FLayerName;
  property Group: integer read FGroup write SetGroup;
  property HolesGroup: integer read FHolesGroup write SetHolesGroup;
  property PropertyClassName: string read  FPropertyClassName write SetPropertyClass;
end;

TRemoveHoles = class(THeleBasic)
private
   FHolesGroup: integer;
   FLayerName: string;
   procedure SetHolesGroup(const Value: integer);
public
   procedure DoIt; override;
published
   property LayerName: string read FLayerName write FLayerName;
   property HolesGroup: integer read FHolesGroup write SetHolesGroup;
end;

THeleClearLHC = class(THeleBasic)
private
   FLayerName: string;
    FCreateHoles: boolean;
    FHolesGroup: integer;
    FSnapDistance: double;
   function  GetLayerEntity(aName: string): TRectEntity;
   function  GetLayerAreas: TRectEntity;
   function  GetNewAreas: TRectEntity;
   function  GetLayerLines: TRectEntity;
   procedure ExtractLHCAreas;
   function  GetGlobalLayer: TREctEntity;
   procedure JoinLines;
   procedure MakeNewAreas;
   procedure CarryProperties;
   procedure Finalize;
    procedure SetCreateHoles(const Value: boolean);
    procedure SetHolesGroup(const Value: integer);
    procedure SetSnapDistance(const Value: double);
public
   procedure DoIt; override;
published
   property LayerName: string read FLayerName write FLayerName;
   property GlobalLayer: TREctEntity read GetGlobalLayer;
   property SnapDistance: double read FSnapDistance write SetSnapDistance;
   property CreateHoles: boolean read FCreateHoles write SetCreateHoles;
   property HolesGroup: integer read FHolesGroup write SetHolesGroup;
end;



TExtractLHC = class(THeleBasic)
private
  FLayerName: string;
public
  procedure DoIt; override;
published
  property LayerName: string read FLayerName write FLayerName;
end;

TDeleteBlockRegister = class(THeleBasic)
private
  FLayerName: string;
public
  procedure DoIt; override;
published
  property LayerName: string read FLayerName write FLayerName;
end;

implementation
uses
  Dialogs, SysUtils,
// KERNEL
  e_Class,
  m_Orbis,
// OIL
  T_SelectByName,
  T_Helhie,
// SPACE
   E_Reg,
   e_SpaPrp,
// SPACETOOL
   T_ToTool,
   T_LinJoi,
// TOPOL
   e_TplPrp,
   T_CopBlk,
// Editor
   T_AreMak,
   t_Lineextractor,
// LHC
   e_LhcProps;

                           { THeleZaradLHC }

procedure THeleZaradLHC.AddToPSK(JeToPSK: Boolean; aPSK: TRectEntity; aAre: TAreaElement);
var
  aLayEl: TRectEntity;
  aTP: TTopolAreaProps;
  aP: TTopolAreaProps;
begin
  if JeToPSK then
     aLayEl:= TRectEntity(aPSK.FindElementName(LayerName))
  else
     aLayEl:= TRectEntity(aPSK.FindElementName(LayerName + ' NEZAAZENO'));

  if aLayEl = nil then
  begin
    aLayEl:= TRectEntity.Create;
    if JeToPSK then
       aLayEl.Name:= LayerName
    else
       aLayEl.Name:= LayerName + ' NEZAAZENO';

    aPSK.AddElement(aLayEl);
  end;

  if not (aAre.Properties is FPropertyClass) then
  begin
    aTP:= TTopolAreaProps(FPropertyClass.Create);
    if aAre.Properties = nil then
    begin
      aAre.Properties:= aTP;
      aTP.Druh:= Group
    end
    else
    begin
      if aAre.Properties is TTopolAreaProps then
      begin
        aP:= TTopolAreaProps(aAre.Properties);
        if aP.Druh <> HolesGroup then // peskakovn dr
        begin
           aTP.Cislo      := aP.Cislo;
           aTP.Druh       := aP.Druh;
           aTP.Barva      := aP.Barva;
           aTP.Znacka     := aP.Znacka;
           aTP.Vyber      := aP.Vyber;
           aTP.Presnost   := aP.Presnost;
           aTP.Symbol     := aP.Symbol;
           aTP.IsDone     := aP.IsDone;

           aTP.LineNo     := aP.LineNo;
           aTP.PaterNo    := aP.PaterNo;
           aTP.HasChildren:= aP.HasChildren;

           aTP.SetPropertyLongIntByName('NALEHAVOST', aP.GetPropertyLongIntByName('NALEHAVOST'));
           aTP.SetPropertyLongIntByName('ZPUSOB', aP.GetPropertyLongIntByName('ZPUSOB'));

           aAre.Properties:= aTP
        end;
      end;
    end
  end;

  aLayEl.AddAndUpdateRect(aAre);
  aPSK.UpdateRect(aAre);
end;


procedure THeleZaradLHC.DoIt;
begin
//   inherited;
   NactiLayer;
end;


function THeleZaradLHC.GetPSKBySpace(anAre: TAreaElement;  var JeToPSK: Boolean;
                                     var aLHC, aOdd, aDil, aPor, aPSK: string): TRectEntity;
var
  HelSku: THeleSkupina;

begin
  Result:= nil;
  JeToPSK:= false;

  HelSku:= THeleSkupina.Create;
  HelSku.Input:= Hierarchy;
  HelSku.X:= anAre.X;
  HelSku.Y:= anAre.Y;
  HelSku.DoIt;
  if not HelSku.Found then
  begin
    HelSku.Free;
    Exit;
  end;

  aLHC := HelSku.LHC;
  aOdd := HelSku.Odd;
  aDIL := HelSku.Dil;
  aPOR := HelSku.Por;
  aPSK := HelSku.PSK;

  result:= HelSku.PSKObject;
  JeToPSK:= Result <> nil;
  HelSku.Free;
end;



function THeleZaradLHC.GetPSKByAttribs(anAre: TAreaElement;  var JeToPSK: boolean): TRectEntity;
var
  THH: THeleHier;
  aLHC, aOdd, aDil, aPor, aPSK: string;

  procedure AddToProtocol(aStr: string);
  begin
     Protocol:= 'Zaazovn plochy ' + LayerName + ' .' + IntToStr(anAre.SpaceId) +
                aStr
  end;

begin
  Result:= nil;
  JeToPSK:= false;

  aLHC := anAre.GetPropertyStringByName('LHC');
  aOdd := anAre.GetPropertyStringByName('ODDELENI');
  aDIL := anAre.GetPropertyStringByName('DILEC');
  aPOR := anAre.GetPropertyStringByName('POROST');
  aPSK := anAre.GetPropertyStringByName('SKUPINA');

  THH := THeleHier.Create;
  THH.Input := Hierarchy;
  THH.LHC := aLHC;

  if THH.LHC = '' then
  begin
    //AddToProtocol(' nelze zaadit kd LHC ' + aLHC);
  end
  else
  begin
    THH.ODD := aODD;
    if THH.ODD = '' then
    begin
      //AddToProtocol(' nelze zaadit kd oddlen ' + aLHC + ' ' + aODD);
    end
    else
    begin
      THH.DIL := aDIL;
      if THH.DIL = '' then
      begin
        //AddToProtocol(' nelze zaadit kd dlce ' + aLHC + ' ' + aODD + ' ' + aDIL);
      end
      else
      begin
        THH.POR := aPOR;
        if THH.POR = '' then
        begin
          //AddToProtocol(' nelze zaadit kd porostu ' + aLHC + ' ' + aODD + ' ' + aDIL + ' ' + aPOR);
        end
        else
        begin
          THH.PSK := aPSK;
          if THH.PSK = '' then
          begin
            //AddToProtocol(' nelze zaadit kd skupiny ' + aLHC + ' ' + aODD + ' ' + aDIL + ' ' + aPOR + ' ' + aPSK);
          end
          else
            JeToPSK:= true
        end
      end
    end
  end;

  THH.DoIt;
  result := TrectEntity(THH.Output);
  if result = nil then
     result:= Hierarchy
end;



procedure THeleZaradLHC.NactiLayer;
var
  I: integer;
  aHier: TEntity;
  aP: TTopolAreaProps;
  aAre: TAreaElement;
  aPSKAttribs: TRectEntity;
  aPSKSpace  : TRectEntity;
  JeToPSKA: Boolean;
  JeToPSKS: Boolean;
  aNeTezba: TRectEntity;
  aLHC, aOdd, aDil, aPor, aPSK: string;
  BlockRegister: TBlockRegister;

begin
//  NajdiVrstvu(FTemp, LayerName, 'ARE');
  FTemp:= FindLayer(LayerName, 'ARE');

  if not(FTEMP is TAreaString) then exit;
  if FTemp.Count = 0 then exit;

  aHier:= Hierarchy;
  if aHier =nil then Exit;

  aNeTezba:= CreateDataRectEntitu(LayerName + ' NEZAAZENO');
  StartProgress(LayerName+': Zaazovn ploch do hierarchie',0,FTemp.Count-1);
  try
  for I:= 0 to FTemp.Count-1 do
  begin
    Progress := i;
    aAre:= TAreaElement(FTemp.elements[I]);
    ProgressText := 'Plocha c. '+IntToStr(aAre.SpaceId);
    //Application.ProcessMessages;
    if I = 0 then
    begin
      aP:= TTopolAreaProps(aAre.Properties);
      if TObject(aP) is TTopolAreaProps then
      begin
        BlockRegister:= aP.BlockRegister;
      end;
    end;

    if not (TObject(aAre) is TAreaElement) then
       continue;
    aPSKAttribs:= GetPSKByAttribs(aAre, JeToPSKA);
    aPSKSpace:= GetPSKBySpace(aAre, JeToPSKS, aLHC, aOdd, aDil, aPor, aPSK);

    if (aPSKAttribs = aPSKSpace) and (aPSKAttribs <> nil) then
    begin
      AddToPSK(JeToPSKS, aPSKAttribs, aAre)
    end
    else
    if (aPSKSpace <> nil) then
    begin
      AddToPSK(JeToPSKS, aPSKSpace, aAre);
      {
      Protocol:= 'Zaazovn plochy ' + LayerName + ' .' + IntToStr(aAre.SpaceId) +
                  '  zaazena dle prostoru do ' + aLHC + ' ' + aOdd + ' ' + aDil + ' ' + aPor + ' ' + aPSK
       }
    end
    else
    if (aPSKAttribs <> nil) then
    begin
      AddToPSK(JeToPSKA, aPSKAttribs, aAre);
      Protocol:= 'Zaazovn plochy ' + LayerName + ' .' + IntToStr(aAre.SpaceId) +
                 ' zaazovn pes atributy - pod touto plochou nele regulrn PSK'
    end
    else
    if aPSKAttribs = nil then
    begin
      Protocol:= 'Zaazovn plochy ' + LayerName + ' .' + IntToStr(aAre.SpaceId) +
                 ' ani pes prostor. vztahy ani pes atributy nelze najt PSK';
      aNeTezba.AddAndUpdateRect(aAre);
      continue
    end
  end;
  //BlockRegister.Free; uvolnn registru se provdi na zvr operac specilnm nstrojem


  except
    ShowMessage('Chyba pi zaazovn ploch vrstvy ' + LayerName);
  end;

// zruseni tezby v registru

end;



procedure THeleZaradLHC.SetGroup(const Value: integer);
begin
  FGroup := Value;
end;


procedure THeleZaradLHC.SetPropertyClass(const Value: string);
begin
  FPropertyClassName := Value;
  FPropertyClass:= FindOrbisClass(Value);
end;





procedure THeleZaradLHC.SetHolesGroup(const Value: integer);
begin
  FHolesGroup := Value;
end;

{ THeleClearLHC }

function THeleClearLHC.GetLayerEntity(aName: string): TRectEntity;
var
  aLayer: TRectEntity;

begin
  Result:= nil;
  aLayer:= GlobalLayer;
  if not (TObject(aLayer) is TRectEntity) then
     Exit;
  Result:= TRectEntity(aLayer.findElementName(aName));
  if Result = nil then
  begin
    Result:= TRectEntity.Create;
    Result.Name:= aName;
    aLayer.AddElement(Result)
  end;
end;



procedure THeleClearLHC.ExtractLHCAreas;
var
  aLayer: TRectEntity;
  aLay: TRectEntity;
  I, J, k: integer;
  aPlochy: TRectEntity;
  aPlocha: TAreaElement;
  PLArea: TPolyLineAreaElement;
  LL: TPolyLineElement;
begin
  aLayer:= GlobalLayer;
  if not (TObject(aLayer) is TRectEntity) then
     Exit;
  aPlochy:= GetLayerAREAS;
  if aPlochy.Count > 0 then
     aPlochy.FreeAll;

  for I:= aLayer.Count-1 downto 0 do
  begin
    aLay:= TRectEntity(aLayer.Elements[I]);
    if not (TObject(aLay) is TRectEntity) then
       continue;
    if aLay.Name <> LayerName then
       continue;
    for J:= 0 to aLay.Count-1 do
    begin
      aPlocha:= TAreaElement(aLay.Elements[J]);
      if not (TObject(aPlocha) is TAreaElement) then
         continue;
      // Oprava opsanch obdlnk ... pro Mejzra .. zpsobuje to chybn itn ploch
      if aPlocha is TPolyLineAreaElement then
      begin
         PLArea := TPolyLineAreaElement(aPlocha);
         for k := 0 to PLArea.LastIndex do
         begin
            LL := TPolyLineElement(PLArea.Elements[k]);
            if not (TObject(LL) is TPolyLineElement) then continue;
            LL.ResetAndUpdate;
         end;
      end;
      aPlochy.AddAndUpdateRect(aPlocha);
    end;
    aLay.Free;
  end;
end;




procedure THeleClearLHC.JoinLines;
var
  aSource: TRectEntity;
  aLines: TRectEntity;
  JL: TJoinLines;
  HL: THledacLinii;
  LEX: TLineExtractor;
  PomLin: TSpaceEntity;
begin
  aSource:= GetLayerAREAS;
  if aSource = nil then
     Exit;
  if aSource.Count = 0 then
     Exit;

{ naten blok pro generovn _20VOD.BLK }

  PomLin:= TSpaceEntity.Create;
  {
  HL:= THledacLinii.Create;
  HL.Input:= aSource;
  HL.Output:= PomLin;
  HL.DoIt;

  HL.Free;
  }

  LEX:= TLineExtractor.Create;
  LEX.Input:= aSource;
  LEX.Output:= PomLin;
  // !!! Test
  //LEX.CopyLines := true;
  // !!! Test
  LEX.DoIt;

  PomLin.ResetAndUpdate;

  JL:= TJoinLines.Create;
  JL.Input:= PomLin;
  if SnapDistance = 0 then SnapDistance := 0.01;
  JL.SnapDistance:= SnapDistance;
  JL.CompareOnlyDifferentSource:= false;
  JL.SelfTest:= true;
  JL.DoIt;

  LEX.Free;
  // LEX Zrusi i pomLin .. aspo myslim
  //PomLin.Free;


  aLines:= GetLayerLINES;
  if aLines = nil then
     Exit;
  if aLines.Count > 0 then
     aLines.FreeAll;

  HL:= THledacLinii.Create;
  HL.Input:= JL.Output;
  HL.Output:= aLines;
  HL.DoIt;
  HL.Free;
  JL.Output.Free;// Uvolnn QTree linii
  JL.Free;
  aLines.Update;
end;



procedure THeleClearLHC.MakeNewAreas;
var
  aLines: TRectEntity;
  aAreas: TRectEntity;
  MNAA: TAreasFromClearLines;

begin
  aLines:= GetLayerLINES;
  aAreas:= GetNewAreas;

  MNAA:= TAreasFromClearLines.Create;
  MNAA.Input:= aLines;
  MNAA.DoIt;
  //!!!Test nodes
  //exit;
  //!!!Test
  aAreas.AddElement(MNAA.Output);
  aAreas.Update;
  MNAA.Free;
end;



procedure THeleClearLHC.DoIt;
var
  aLayer: TRectEntity;
begin
  {test existence Tby }
  aLayer:= GlobalLayer;
  if not (TObject(aLayer) is TRectEntity) then
     Exit;
  StartProgress('Rekonstrukce vrstvy '+ LayerName,0,0);
  { vyjmut tebnch ploch a smazn tby z porostnch skupin }
  ProgressText := 'Hledn ploch ...';
  ExtractLHCAreas;
  { spojen lini }
  ProgressText := 'Nov spojen lini ...';
  JoinLines;

  { nov zaplochovn }
  ProgressText := 'Peplochovn ...';
  MakeNewAreas;
  //!!!Test nodes
  //exit;
  //!!!Test
  // Penos vlastnost ze starch ploch do novch
  ProgressText := 'Penos atribut ...';
  CarryProperties;
  // Pesuny vsledk do layers a ruen vrstev
  ProgressText := 'finalizace ...';
  Finalize;
  PostMsg(tm_ModelChanged,nil,nil,false);
end;



function THeleClearLHC.GetGlobalLayer: TREctEntity;
begin
  result:= TRectEntity(DejDATAEntitu(LayerName));
end;

function THeleClearLHC.GetLayerAreas: TRectEntity;
begin
   result := GetLayerEntity('AREAS');
end;

function THeleClearLHC.GetLayerLines: TRectEntity;
begin
   result := GetLayerEntity('LINES');
end;


function THeleClearLHC.GetNewAreas: TRectEntity;
begin
   result := GetLayerEntity('NEW_AREAS');
end;

procedure THeleClearLHC.CarryProperties;
var
   OAre: TRectEntity; // star plochy
   NAre: TRectEntity; // nov  plochy
   E: TElement;
   Are: TPolyLineAreaElement;
   NeibAre: TPolyLineAreaElement;
   Lin: TPolyLineElement;
   i,j,k: integer;
   SP: TSpaceEntityProps;
   AreaOwners: boolean;
begin
   OAre := GetLayerAreas;
   NAre := GetNewAreas;
   NAre := TRectEntity(NAre.FindElementClass(TAreaString));
   // Cyklus pes vechny nov plochy *********
   for i := NAre.LastIndex downto 0 do
   begin
      E := NAre.Elements[i];
      if not (E is TPolyLineAreaElement) then Continue;
      Are := TPolyLineAreaElement(E);
      // Hledme odpovdajc starou plochu podle lini ************
      NeibAre := nil;
      for j := 0 to Are.LastIndex do
      begin
         E := Are.Elements[j];
         if not (E is TPolyLineElement) then Continue;
         Lin := TPolyLineElement(E);
         AreaOwners := false;
         for k := 0 to Lin.OwnersCount - 1 do
         begin
            E := Lin.Owners[k];
            if not (E is TPolyLineAreaElement) then
            begin
               if not AreaOwners then
                  continue // Jet jsme se nedostali k plonm vlastnkm
               else
                  break; // pvodn plochy doly, zkusme dal linii
            end;
            AreaOwners := true;
            NeibAre := TPolyLineAreaElement(E);
            if Are.XYIsInside(NeibAre.X,NeibAre.Y) then
            begin
               if (NeibAre.Properties <> nil) and (NeibAre.Properties is TSpaceEntityProps) then
               begin
                  SP := TSpaceEntityProps(TSpaceEntityProps(NeibAre.Properties).CreateDuplicate);
                  Are.Properties := SP;
               end
               else
               begin
                  PostTextMsg(tm_AddToProtocol,LayerName+'-Old Area no.'+IntToStr(NeibAre.SpaceID)+' has no properties!',nil);
               end;
               Break;
            end
            else
            begin
               NeibAre := nil;
            end;
         end;
         if NeibAre <> nil then
            break; // Plocha nalezena jinak continue
      end;
      if NeibAre = nil then
      if CreateHoles then
      begin
         Are.Properties := TTopolAreaProps.Create;
         TTopolAreaProps(Are.Properties).Druh := HolesGroup;
      end
      else
         Are.Free; // Plocha nenalezena ... dra
   end;
end;

procedure THeleClearLHC.Finalize;
var
   NAre: TRectEntity; // nov  plochy
   Lay: TRectEntity;
begin
   NAre := GetNewAreas;
   NAre := TRectEntity(NAre.FindElementClass(TAreaString));

   Lay:= TRectEntity(DejDataEntitu('LAYERS'));
   if Lay = nil then
   begin
      PostTextMsg(tm_AddToProtocol,'ClearLHC: LAYERS not found!',nil);
      Exit;
   end;
   Lay:= TRectEntity(Lay.FindElementName('ARE_'+LayerName));
   if not (TObject(Lay) is TRectEntity) then
   begin
      PostTextMsg(tm_AddToProtocol,'ClearLHC: LAYER '+LayerName+' not found!',nil);
      Exit;
   end;
   Lay.FreeAll;
   Lay.NextElement := NAre;
   Lay.ResetAndUpdate;
   Lay := GlobalLayer;
   Lay.FreeAll;
end;

procedure THeleClearLHC.SetCreateHoles(const Value: boolean);
begin
  FCreateHoles := Value;
end;

procedure THeleClearLHC.SetHolesGroup(const Value: integer);
begin
  FHolesGroup := Value;
end;

procedure THeleClearLHC.SetSnapDistance(const Value: double);
begin
  FSnapDistance := Value;
end;

{ TExtractLHC }


procedure TExtractLHC.DoIt;
var
  aLayer: TRectEntity;
  aHier: TRectEntity;
  SBN: TSelectByName;

begin
  aLayer:= CreateDataRectEntitu(LayerName);

  if not(aLayer is TRectEntity) then
     Exit;
  aLayer.FreeAll;
  aHier:= Hierarchy;
  StartProgress(LayerName + 'extrahovn vrstvy z hierarchie',0,0);
  ProgressText := '... provd se';
  SBN:= TSelectByName.Create;
  SBN.CaseSensitive:= False;
  SBN.SelectedName:= LayerName;
  SBN.Input := aHier;
  SBN.Partialy:= false;
  SBN.Output:= aLayer;
  SBN.DoIt;
  SBN.Output.Name := LayerName;
  SBN.Free;

  aLayer.Update;
end;




{ TRemoveHoles }

procedure TRemoveHoles.DoIt;
var
  aTemp: TEntity;
  I: integer;
  aP: TTopolAreaProps;
  aAre: TAreaElement;

begin
//  NajdiVrstvu(FTemp, LayerName, 'ARE');
  aTemp:= FindLayer(LayerName, 'ARE');

  if not(aTEMP is TAreaString) then exit;
  if aTemp.Count = 0 then exit;

  StartProgress(LayerName+': Odstraovn neploch',0,aTemp.LastIndex);
  try
  for i := aTemp.LastIndex downTo 0 do
  begin
    Progress := i;
    aAre:= TAreaElement(aTemp.elements[I]);
    ProgressText := 'Plocha c. '+IntToStr(aAre.SpaceId);
    //Application.ProcessMessages;
      aP:= TTopolAreaProps(aAre.Properties);
      if TObject(aP) is TTopolAreaProps then
      begin
         if aP.Druh = HolesGroup then
         begin
            ProgressText := 'Plocha c. '+IntToStr(aAre.SpaceId)+' odstranna';
            aAre.Free;
         end;
      end;
  end;
  except
    ShowMessage('Chyba pi uvolovn neploch vrstvy ' + LayerName);
  end;

// zruseni tezby v registru

end;

procedure TRemoveHoles.SetHolesGroup(const Value: integer);
begin
  FHolesGroup := Value;
end;

{ TDeleteBlockRegister }

procedure TDeleteBlockRegister.DoIt;
var
   aP: TTopolProps;
   aAre: TAreaElement;
   aPoint: TPointElement;
   BlockRegister: TBlockRegister;
   FTemp: TRectEntity;
begin
   FTemp:= FindLayer(LayerName, 'ARE');
   try
   if not(FTEMP is TAreaString) then exit;
   if FTemp.NoElements then
   begin
      FTemp:= FindLayer(LayerName, 'PNT');
      if not(FTEMP is TPointString) then exit;
      if FTemp.NoElements then exit;
      aPoint:= TPointElement(FTemp.elements[0]);
      aP:= TTopolProps(aPoint.Properties);
   end
   else
   begin
      aAre:= TAreaElement(FTemp.elements[0]);
      aP:= TTopolProps(aAre.Properties);
   end;
   if TObject(aP) is TTopolProps then
   begin
      BlockRegister:= aP.BlockRegister;
      if BlockRegister <> nil then
         BlockRegister.Free;
   end;
   except
   end;
end;

initialization
  RegisterOrbisClass(THeleZaradLHC);
  RegisterOrbisClass(THeleClearLHC);
  RegisterOrbisClass(TExtractLHC);
  RegisterOrbisClass(TRemoveHoles);
  RegisterOrbisClass(TDeleteBlockRegister);
end.

