unit e_TopolDruhy;

interface
uses
   Elements;
type

TTopolDruh = class(TOwnerElement)
private
   FZnacka: integer;
   FKOD: integer;
   FBarva: integer;
    FTyp: Char;
    FZaloha: integer;
    FTrida: string;
   procedure SetBarva(const Value: integer);
   procedure SetKOD(const Value: integer);
   procedure SetZnacka(const Value: integer);
    procedure SetTrida(const Value: string);
    procedure SetTyp(const Value: Char);
    procedure SetZaloha(const Value: integer);
published
   property KOD: integer read FKOD write SetKOD;
   property Barva: integer read FBarva write SetBarva;
   property Znacka: integer read FZnacka write SetZnacka;
   property Typ: Char read FTyp write SetTyp;
   property Trida: string read FTrida write SetTrida;
   property Zaloha: integer read FZaloha write SetZaloha;
end;

TTopolDruhCollection = class(TOwnerEntity)
private
    FActualDruh: TTopolDruh;
    procedure SetActualDruh(const Value: TTopolDruh);
    function GetActualDruhKod: integer;
    function GetActualDruhName: string;
protected
    procedure SetFindDruhKod(const Value: integer); virtual;
    procedure SetFindDruhName(const Value: string); virtual;
published
   property ActualDruh: TTopolDruh read FActualDruh write SetActualDruh stored false;
   property FindDruhKod: integer write SetFindDruhKod stored false;
   property FindDruhName: string write SetFindDruhName stored false;
   property ActualDruhName: string read GetActualDruhName stored false;
   property ActualDruhKod: integer read GetActualDruhKod stored false;
end;

TTopolDruhy = class(TTopolDruhCollection)
private
    FDBFPath: string;
    function GetB: TTopolDruhCollection;
    function GetL: TTopolDruhCollection;
    function GetP: TTopolDruhCollection;
{$IFDEF DBF}
    procedure SetDBFPath(const Value: string);
{$ENDIF}
    procedure SetFindLDruhKod(const Value: integer);
    procedure SetFindLDruhName(const Value: string);
    procedure SetFindBDruhKod(const Value: integer);
    procedure SetFindBDruhName(const Value: string);
protected
    procedure SetFindDruhKod(const Value: integer); override;
    procedure SetFindDruhName(const Value: string); override;
published
{$IFDEF DBF}
   property DBFPath: string read FDBFPath write SetDBFPath stored false;
{$ENDIF}
   property L: TTopolDruhCollection read GetL stored false;
   property P: TTopolDruhCollection read GetP stored false;
   property B: TTopolDruhCollection read GetB stored false;
   property FindLDruhKod: integer write SetFindLDruhKod stored false;
   property FindLDruhName: string write SetFindLDruhName stored false;
   property FindBDruhKod: integer write SetFindBDruhKod stored false;
   property FindBDruhName: string write SetFindBDruhName stored false;
end;

implementation

uses
{$IFDEF DBF}
   DBTables,
   DB,
   DBF,
   Dialogs,
{$ENDIF}
   SysUtils,
   e_Class,
   u_Global,
   u_String;
{ TTopolDruh }

procedure TTopolDruh.SetBarva(const Value: integer);
begin
  FBarva := Value;
end;

procedure TTopolDruh.SetKOD(const Value: integer);
begin
  FKOD := Value;
end;

procedure TTopolDruh.SetTrida(const Value: string);
begin
  FTrida := Value;
end;

procedure TTopolDruh.SetTyp(const Value: Char);
begin
  FTyp := Value;
end;

procedure TTopolDruh.SetZaloha(const Value: integer);
begin
  FZaloha := Value;
end;

procedure TTopolDruh.SetZnacka(const Value: integer);
begin
  FZnacka := Value;
end;

{ TTopolDruhCollection }

function TTopolDruhCollection.GetActualDruhKod: integer;
begin
   result := 0;
   if ActualDruh = nil then exit;
   result := ActualDruh.KOD;
end;

function TTopolDruhCollection.GetActualDruhName: string;
begin
   result := ' ';
   if ActualDruh = nil then exit;
   result := ActualDruh.Name;
end;

procedure TTopolDruhCollection.SetActualDruh(const Value: TTopolDruh);
begin
  FActualDruh := Value;
end;

procedure TTopolDruhCollection.SetFindDruhKod(const Value: integer);
var
   i: integer;
begin
   ActualDruh := nil;
   for i := 0 to LastIndex do
   begin
      if TTopolDruh(Elements[i]).Kod = Value then
      begin
         ActualDruh := TTopolDruh(Elements[i]);
         exit;
      end;
   end;
end;

procedure TTopolDruhCollection.SetFindDruhName(const Value: string);
begin
   ActualDruh := TTopolDruh(FindElementName(Value));
end;

{ TTopolDuhy }

function TTopolDruhy.GetB: TTopolDruhCollection;
begin
   result := TTopolDruhCollection(FindOrCreateElementName(TTopolDruhCollection,'B'));
end;

function TTopolDruhy.GetL: TTopolDruhCollection;
begin
   result := TTopolDruhCollection(FindOrCreateElementName(TTopolDruhCollection,'L'));
end;

function TTopolDruhy.GetP: TTopolDruhCollection;
begin
   result := TTopolDruhCollection(FindOrCreateElementName(TTopolDruhCollection,'P'));
end;

{$IFDEF DBF}
procedure TTopolDruhy.SetDBFPath(const Value: string);
var
   T: TDataSet;
   TD: TTopolDruh;
   S: string;
   i: integer;
   E: TElement;
begin
   FDBFPath := Value;
   S := Value;
   CheckBackSlash(S);
   S := S + 'znacky.dbf';
   if not FileExists(S) then
   begin
      ShowMessage('Soubor '+S+' asi neexistuje... nelze nast druhy z TopoLu');
      exit;
   end;
   FreeAll;
   if NoBDE then
   begin
      T := TDbf.Create(nil);
      TDbf(T).TableName := S;
   end
   else
   begin
      T := TTable.Create(nil);
      TTable(T).TableName := S;
   end;
   T.Active := true;
   T.First;
   While not T.EOF do
   begin
      TD := TTopolDruh.Create;
      TD.Name := T.FieldByName('NAZEV').AsString;
      TD.KOD := T.FieldByName('KOD_OBJEKT').AsInteger;
      TD.Barva := T.FieldByName('DISP_BARVA').AsInteger;
      TD.Znacka := T.FieldByName('DISP_ZNAC').AsInteger;
      TD.Typ := T.FieldByName('TYP_OBJEKT').AsString[1];
      TD.Trida := T.FieldByName('TRIDA').AsString;
      TD.Zaloha := T.FieldByName('REZERVA').AsInteger;
      case TD.Typ of
         'L': L.AddElement(TD);
         'P': P.AddElement(TD);
         'B': B.AddElement(TD);
      else
         AddElement(TD);
      end;
      T.Next;
   end;
   T.Active := false;
   T.Free;
end;
{$ENDIF}
procedure TTopolDruhy.SetFindBDruhKod(const Value: integer);
begin
   B.FindDruhKod := Value;
   ActualDruh := B.ActualDruh;
end;

procedure TTopolDruhy.SetFindBDruhName(const Value: string);
begin
   B.FindDruhName := Value;
   ActualDruh := B.ActualDruh;
end;

procedure TTopolDruhy.SetFindDruhKod(const Value: integer);
begin

end;

procedure TTopolDruhy.SetFindDruhName(const Value: string);
begin

end;

procedure TTopolDruhy.SetFindLDruhKod(const Value: integer);
begin
   L.FindDruhKod := Value;
   ActualDruh := L.ActualDruh;
end;

procedure TTopolDruhy.SetFindLDruhName(const Value: string);
begin
   L.FindDruhName := Value;
   ActualDruh := L.ActualDruh;
end;

INITIALIZATION
   RegisterOrbisClass(TTopolDruh);
   RegisterOrbisClass(TTopolDruhCollection);
   RegisterOrbisClass(TTopolDruhy);
END.
