unit t_Annotation;

interface
uses
   e_Annotation,
   e_BGIF,
   e_Device,
   e_GeometryElement,
   e_pSymb,
   e_QTree,
   e_TopolElements,
   u_Symbol,
   Elements,
   Tools;

type

TBGIFTool = class(TTool)
private
    FComment: string;
    FBGIF: TBGIFont;
    FTextInfo: TBGIText;
    FDevice: TDevice;
    function GetBGIF: TBGIFont;
    function GetTextInfo: TBGIText;
    procedure SetComment(const Value: string);
    procedure SetDevice(const Value: TDevice);
public
   property BGIF: TBGIFont read GetBGIF;
   property TextInfo: TBGIText read GetTextInfo;
   function NumberToPropertyName(aNumber: longInt): string;override;
published
   property Device: TDevice read FDevice write SetDevice stored false;
   property Comment: string read FComment write SetComment;
end;

TCheckTopolTextRectangle=class(TBGIFTool)
  private
    FInfoByDruh: boolean;
    FColorByDruh: boolean;
    procedure SetInfoByDruh(const Value: boolean);
    procedure SetColorByDruh(const Value: boolean);
public
   procedure DoIt; override;
   procedure CHeckRectangle(TE: TTopolTextElement);
   function NumberToPropertyName(aNumber: longInt): string;override;
published
   property InfoByDruh: boolean read FInfoByDruh write SetInfoByDruh;
   property ColorByDruh: boolean read FColorByDruh write SetColorByDruh;
end;

TSeparateNoNumberTexts=class(TCheckTopolTextRectangle)
  private
    FNAltStyle: integer;
    FAltStyle: integer;
    FNStyle: integer;
    FStyle: integer;
    FStyle1: integer;
    FAltStyle1: integer;
    FLinkDistance: double;
    FNModify: integer;
    FLins: TEntity;
    procedure SetAltStyle(const Value: integer);
    procedure SetAltStyle1(const Value: integer);
    procedure SetNAltStyle(const Value: integer);
    procedure SetNStyle(const Value: integer);
    procedure SetStyle(const Value: integer);
    procedure SetStyle1(const Value: integer);
    procedure SetLinkDistance(const Value: double);
    procedure SetNModify(const Value: integer);
    function GetLins: TEntity;
public
   procedure DoIt; override;
   function  NumberToPropertyName(aNumber: longInt): string;override;
   procedure Process(TE: TTopolTextElement);
published
   property Style: integer read FStyle write SetStyle;
   property Style1: integer read FStyle1 write SetStyle1;
   property AltStyle: integer read FAltStyle write SetAltStyle;
   property AltStyle1: integer read FAltStyle1 write SetAltStyle1;
   property NStyle: integer read FNStyle write SetNStyle;
   property NAltStyle: integer read FNAltStyle write SetNAltStyle;
   property LinkDistance: double read FLinkDistance write SetLinkDistance;
   property NModify: integer read FNModify write SetNModify;
   property Lins: TEntity read GetLins stored false;
end;

TTopolTextToLines=class(TCheckTopolTextRectangle)
// Input  ...  Texts
// Output ...  Lines
public
   procedure DoIt; override;
   function  NumberToPropertyName(aNumber: longInt): string;override;
end;

TTopolTextToShapes=class(TTopolTextToLines)
// Input  ...  Texts
// Output ...  Vrstva pro plochy (teda shapesy)
private
   FCreateGeometryElements: boolean;
   procedure SetCreateGeometryElements(const Value: boolean);
public
   procedure CreateShape(TE: TTopolTextElement);
   procedure DoIt; override;
   function  NumberToPropertyName(aNumber: longInt): string; override;
published
   property CreateGeometryElements: boolean read FCreateGeometryElements write SetCreateGeometryElements;
end;


TTopolPointToLines = class(TTool)
 private
   FSymbols: TTopoLSymbols;
    FComment: string;
    FDevice: TDevice;
   procedure SetSymbols(const Value: TTopoLSymbols);
    procedure SetComment(const Value: string);
    procedure SetDevice(const Value: TDevice);
// Input  ...  Points
// Output ...  Lines
protected
   property Symbols: TTopoLSymbols read FSymbols write SetSymbols;
public
   constructor Create; override;
   destructor  Destroy; override;
   procedure DoIt; override;
   function  NumberToPropertyName(aNumber: longInt): string;override;
published
   property Device: TDevice read FDevice write SetDevice stored false;
   property Comment: string read FComment write SetComment;
end;

TTopolPointToShapes = class(TTopolPointToLines)
// Input  ...  Points
// Output ...  Vrstva pro plochy (teda shapesy)
private
   FCreateGeometryElements: boolean;
   procedure SetCreateGeometryElements(const Value: boolean);
public
   procedure CreateShape(TE: TTopolPointElement; aLIN: TGeometryElementCollection);
   procedure DoIt; override;
   function  NumberToPropertyName(aNumber: longInt): string; override;
published
   property CreateGeometryElements: boolean read FCreateGeometryElements write SetCreateGeometryElements;
end;

TCreateTextAnnotation = class(TBGIFTool)
  private
    FElementClassName: string;
    FPropertyName: string;
    FStyle: integer;
    FStyle1: integer;
    FCheckStyle1: boolean;
    FCHeckValue: integer;
    FNoGeometry: boolean;
    FQ: TQuickEntity;
    FAltStyle: integer;
    FAltStyle1: integer;
    FLinkToCollection: TGeometryElementCollection;
    FLink: boolean;
    FNewIsInvisible: boolean;
    FLinkDistance: double;
    FByText: boolean;
    FAltPropertyName: string;
    procedure SetElementClassName(const Value: string);
    procedure SetPropertyName(const Value: string);
    procedure SetStyle(const Value: integer);
    procedure SetStyle1(const Value: integer);
    procedure SetCheckStyle1(const Value: boolean);
    procedure SetCHeckValue(const Value: integer);
    procedure SetNoGeometry(const Value: boolean);
    function GetQ: TQuickEntity;
    procedure SetAltStyle(const Value: integer);
    procedure SetAltStyle1(const Value: integer);
    procedure SetLinkToCollection(const Value: TGeometryElementCollection);
    procedure SetLink(const Value: boolean);
    procedure SetNewIsInvisible(const Value: boolean);
    procedure SetLinkDistance(const Value: double);
    procedure SetByText(const Value: boolean);
    procedure SetAltPropertyName(const Value: string);
public
   procedure DoIt; override;
   procedure CHeckRectANdStyle(GTA: TGeometryTextAnnotation);
   procedure CheckFousy(GTA: TGeometryTextAnnotation);
   property  Q: TQuickEntity read GetQ;
   property  Link: boolean read FLink write SetLink;
   property  NewIsInvisible: boolean read FNewIsInvisible write SetNewIsInvisible;
published
   property ElementClassName: string read FElementClassName write SetElementClassName;
   property PropertyName: string read FPropertyName write SetPropertyName;
   property AltPropertyName: string read FAltPropertyName write SetAltPropertyName;
   property Style: integer read FStyle write SetStyle;
   property AltStyle: integer read FAltStyle write SetAltStyle;
   property Style1: integer read FStyle1 write SetStyle1;
   property AltStyle1: integer read FAltStyle1 write SetAltStyle1;
   property CheckStyle1: boolean read FCheckStyle1 write SetCheckStyle1;
   property CHeckValue: integer read FCHeckValue write SetCHeckValue;
   property NoGeometry: boolean read FNoGeometry write SetNoGeometry;
   property LinkToCollection: TGeometryElementCollection read FLinkToCollection write SetLinkToCollection;
   property LinkDistance: double read FLinkDistance write SetLinkDistance;
   property ByText: boolean read FByText write SetByText;
end;

// input  - plochy - collection geometryElementu
// output - TGeometryAnnotationCollection
TCreateTextAnnotationFromTextsAndLines = class(TCreateTextAnnotation)
  private
    FTexts: TEntity;
    FTXT: TGeometryElementCollection;
    FLIN: TGeometryElementCollection;
    FLevelQT: TRectEntityQTree;
    FLinQT: TRectEntityQTree;
    FLinSnapDistance: double;
    FLinkQT: TRectEntityQTree;
    FNoFous: boolean;
    FLinkTolerance: double;
    FFreeFous: boolean;
    procedure SetTexts(const Value: TEntity);
    procedure SetLIN(const Value: TGeometryElementCollection);
    procedure SetTXT(const Value: TGeometryElementCollection);
    procedure SetLevelQT(const Value: TRectEntityQTree);
    procedure SetLinQT(const Value: TRectEntityQTree);
    procedure SetLinSnapDistance(const Value: double);
    procedure SetLinkQT(const Value: TRectEntityQTree);
    procedure SetNoFous(const Value: boolean);
    procedure SetLinkTolerance(const Value: double);
    procedure SetFreeFous(const Value: boolean);
public
   procedure DoIt; override;
   procedure FindTextAndLines;
   procedure PrepareQTree;
   procedure ProcessTexts;
   function CHeckTextRectangle(TEl: TElement): boolean;
   property TXT: TGeometryElementCollection read FTXT write SetTXT;
   property LIN: TGeometryElementCollection read FLIN write SetLIN;
   property LevelQT: TRectEntityQTree read FLevelQT write SetLevelQT;
   property LinQT: TRectEntityQTree read FLinQT write SetLinQT;
   property LinkQT: TRectEntityQTree read FLinkQT write SetLinkQT;
published
   property Texts: TEntity read FTexts write SetTexts;
   property LinSnapDistance: double read FLinSnapDistance write SetLinSnapDistance;
   property LinkTolerance: double read FLinkTolerance write SetLinkTolerance;
   property NoFous: boolean read FNoFous write SetNoFous;
   property FreeFous: boolean read FFreeFous write SetFreeFous;
end;

TCreateTextsAndLinesFromTextAnnotation = class(TBGIFTool)
private
   FTexts: TGeometryElementCollection;
   FLines: TGeometryElementCollection;
    FLineSymbol: integer;
    FLineGroup: integer;
    FLineColor: integer;
    function GetLines: TGeometryElementCollection;
    function GetTexts: TGeometryElementCollection;
    procedure SetLineColor(const Value: integer);
    procedure SetLineGroup(const Value: integer);
    procedure SetLineSymbol(const Value: integer);
public
   procedure DoIt; override;
   procedure Process(aElement: TElement);
published
   property Texts: TGeometryElementCollection read GetTexts stored false;
   property Lines: TGeometryElementCollection read GetLines stored false;
   property LineGroup: integer read FLineGroup write SetLineGroup;
   property LineColor: integer read FLineColor write SetLineColor;
   property LineSymbol: integer read FLineSymbol write SetLineSymbol;
end;

{ TODO -oPPK -cOpravit : Nsledujc tool asi pijde do jin Unity, je dost speciln }
(*
 SetPorostTextVisibility
 - nastav u anotac pro LHO, kde se texty ad ped skupiny u porost kter maj
   vce ne jednu geom. plochu (z hlediska skupin) prvn text jako viditeln
   a ostatn jako e ne. Zabv se porostem pouze v ppad, e v jeho rmci nenalezne
   dn ji neviditeln text, pak na to kale.
*)
TSetPorostTextVisibility = class(TTool)
  private
    FSolvedText: string;
    FOutputText: string;
    FTouchedText: string;
    FTouched: TGeometryElementCollection;
    FEDone: TEntity;
    FSolved: TGeometryElementCollection;
    FComment: string;
    procedure SetEDone(const Value: TEntity);
    procedure SetGACollection(const Value: TGeometryAnnotationCollection);
    procedure SetOutputText(const Value: string);
    procedure SetSolved(const Value: TGeometryElementCollection);
    procedure SetSolvedText(const Value: string);
    procedure SetTouched(const Value: TGeometryElementCollection);
    procedure SetTouchedText(const Value: string);
    function GetGACollection: TGeometryAnnotationCollection;
    procedure SetComment(const Value: string);
public
   procedure DoIt; override;
   function  Prepare: boolean;
   property  Solved: TGeometryElementCollection read FSolved write SetSolved;
   property  Touched: TGeometryElementCollection read FTouched write SetTouched;
   property  EDone: TEntity read FEDone write SetEDone;
published
   property GACollection: TGeometryAnnotationCollection read GetGACollection write SetGACollection;
   property OutputText: string read FOutputText write SetOutputText;
   property SolvedText: string read FSolvedText write SetSolvedText;
   property TouchedText: string read FTouchedText write SetTouchedText;
   property Comment: string read FComment write SetComment;
end;

implementation
uses
   Dialogs,
   Forms,
   SysUtils,
   cpConvrt,
   d_Geometry,
   e_Class,
   e_Geometry,
   e_LHCModel,
   g_Elems,
   t_QTreeTools,
   t_Snap,
   u_String,
   Mathlib,
   m_Orbis;

{ TCreateTextAnnotation }

procedure TCreateTextAnnotation.CheckFousy(GTA: TGeometryTextAnnotation);
var
   i,ix: integer;
   G: TGeometry;
   e: TElement;
   X1,X2,Y1,Y2,TAX,TAY: double;
   IsFous,OldFous: boolean;
   AA: TAnnotationAbscissa;
   Rovn,U1,U2: boolean;
   LinkGTA: TGeometryTextAnnotation;
   function Prus: boolean;
   var
      Mix,Miy, Max,May: double;
   begin
      //result := false;
      if (LinkGTA <> nil) and LinkGTA.Visible then
         Mix := LinkGTA.MinX -5
      else
         Mix := GTA.MinX - 5;
      Miy := GTA.MinY - 5;
      Max := GTA.MaxX + 2;
      May := GTA.MaxY + 2;
      PrusecikUsecek(X1,Y1, TAX,TAY, Mix,Miy, Max,Miy, X2,Y2,Rovn,U1,U2);
      result := U1 and U2;
      if result then exit;
      PrusecikUsecek(X1,Y1, TAX,TAY, Max,Miy, Max,May, X2,Y2,Rovn,U1,U2);
      result := U1 and U2;
      if result then exit;
      PrusecikUsecek(X1,Y1, TAX,TAY, Max,May, Mix,May, X2,Y2,Rovn,U1,U2);
      result := U1 and U2;
      if result then exit;
      PrusecikUsecek(X1,Y1, TAX,TAY, Mix,May, Mix,Miy, X2,Y2,Rovn,U1,U2);
      result := U1 and U2;
   end;
begin
   for i := 0 to GTA.LastIndex do
   begin
      e := GTA.Elements[i];
      if not (e is TGeometry) then continue;
      Q.AddElement(e);
   end;
   LinkGTA := TGeometryTextAnnotation(GTA.FindElementClass(TGeometryTextAnnotation));
   TAX := GTA.CenterX;
   if (LinkGTA <> nil) and LinkGTA.Visible then
      TAX := TAX - (TAX - LinkGTA.CenterX)/2;
   TAY := GTA.CenterY;
   X2 := TAX;
   Y2 := TAY;
   for i := 0 to Q.LastIndex do
   begin
      G := TGeometry(Q.Elements[i]);
      X1 := G.X;
      Y1 := G.Y;
      OldFous := false;
      ix := GTA.IndexOf(G);
      if (ix > 0) and (GTA.Elements[ix-1].ClassType = TAnnotationAbscissa) then
      begin
         AA := TAnnotationAbscissa(GTA.Elements[ix-1]);
         X1 := AA.X1;
         Y1 := AA.Y1;
         OldFous := true;
      end;
      if GTA.NoFous then
         IsFous := false
      else
         IsFous := not G.IsHitByXY(TAX,TAY,2);
      //IsFous := not G.IsOverRegion(GTA.MinX,GTA.MinY,GTA.MaxX,GTA.MaxY);
      if IsFous then
      begin
         if not Prus then
         begin
            if OldFous then GTA.FreeElement(GTA.Elements[ix-1]);
            continue;
         end;
         if OldFous then
            AA := TAnnotationAbscissa(GTA.Elements[ix-1])
         else
         begin
            AA := TAnnotationAbscissa.Create;
            GTA.InsertElement(ix,AA);
         end;
         AA.X1 := X1;
         AA.Y1 := Y1;
         AA.X2 := X2;
         AA.Y2 := Y2;
         AA.ResetAndUpdate;
      end
      else
      begin
         if OldFous then GTA.FreeElement(GTA.Elements[ix-1]);
      end;
   end;
   Q.FreeAll;
end;

procedure TCreateTextAnnotation.CHeckRectAndStyle(GTA: TGeometryTextAnnotation);
var
   SV,S: string;
   ii: integer;
   Numb: integer;
   AC: TGeometryAnnotationCollection;
   aStyle: integer;
   isAltStyle: boolean;
   LinkGTA: TGeometryTextAnnotation;   // zatim nepouzito...
   RealLink: boolean;
begin
   AC := GTA.GACollection;
   LinkGTA := nil;
   RealLink := false;
   if NoGeometry then
   begin
      //GTA.Style := Style;
   end
   else
   begin
      if Link then
      begin
         LinkGTA := TGeometryTextAnnotation(GTA.FindOwnerClass(TGeometryTextAnnotation));
         RealLink := (LinkGTA <> nil) and (LinkGTA.IsMyOwner(LinkToCollection));
      end;
      aStyle := GTA.Style;
      isAltStyle := (aStyle = AltStyle) or (CheckStyle1 and (aStyle = AltStyle1));
      GTA.Style := 0;
      if isAltStyle then
         GTA.Style := AltStyle;
      if CheckStyle1 then
      begin
         SV := '';
         S := GTA.Text;
         ii := 1;
         if S <> '' then
         begin
            while not (ii > Length(S)) and (S[ii]=' ') do Inc(ii);
            while not (ii > Length(S)) and (S[ii] in ['0'..'9']) do
            begin
               SV := SV + S[ii];
               Inc(ii);
            end;
            Numb := StrToIntDef(SV,0);
            if (Numb >= CheckValue) then
            begin
               if isAltStyle then
                  GTA.Style := AltStyle1
               else
                  GTA.Style := Style1;
            end;
         end;
      end;
   end;
   if RealLink then // u linku nastavime absRFefX na min coord
   begin
      GTA.AbsRefX := LinkGTA.X - LinkDistance;
      GTA.AbsRefY := LinkGTA.Y;
   end;
   if Device <> nil then
   begin
      if GTA.Style > 0 then
         TextInfo.Druh  := GTA.Style
      else
         TextInfo.Druh  := AC.Style;
      { vechno podle stylu .....
      TextInfo.Barva      := TE.Barva;
      TextInfo.Znacka     := TE.Znacka;
      TextInfo.Font       := TE.TopolFont;
      TextInfo.Vyska      := TE.Height;
      TextInfo.KoefSirky  := TE.WidthCoef;
      TextInfo.Sklon      := TE.Slope;
      TextInfo.Rozestupy  := TE.Distance;
      TextInfo.RezimTextu := TE.TextMode;
      }
      TextInfo.VychoziBod.x := GTA.x;
      TextInfo.VychoziBod.y := GTA.y;
      TextInfo.Smer       := GTA.Angle;
      TextInfo.Atributy   := [OkT,T30];
      TextInfo.CodePage   := cpLatin2;
      TextInfo.Veta       := WinTo852(GTA.Text);
      BGIF.TextInfo := TextInfo;
      BGIF.NastavInfoPodleDruhu(TextInfo.Druh);
      BGIF.CalculateOpsanyObdelnik;
      if RealLink then
      begin
         GTA.AbsRefX := GTA.AbsRefX - TextInfo.Rect.Width;
         TextInfo.VychoziBod.x := GTA.x;
         TextInfo.VychoziBod.y := GTA.y;
         BGIF.CalculateOpsanyObdelnik;
      end;
      GTA.MinX := TextInfo.Rect.MinX;
      GTA.MinY := TextInfo.Rect.MinY;
      GTA.MaxX := TextInfo.Rect.MaxX;
      GTA.MaxY := TextInfo.Rect.MaxY;
      GTA.AbsRefx := TextInfo.VychoziBod.x;
      GTA.AbsRefy := TextInfo.VychoziBod.y;
   end;
end;

procedure TCreateTextAnnotation.DoIt;
var
   En: TEntity;
   E: TElement;
   AC: TGeometryAnnotationCollection;
   GTA,LinkGTA: TGeometryTextAnnotation;
   G,GG: TGeometry;
   GE: TGeometryElement;
   i,j: integer;
   LostAnchors: TOwnerEntity;
   EmptyText: boolean;
   LA: TAnnotationLostAnchor;
   AA: TAnnotationAbscissa;
   TQ: TAddEntityToQTree;
   AQT: TRectEntityQTree;
   QTFinder: TFindRectEntityInQTreeByXY;


   procedure CheckG;
   var
      En_: TEntity;
      E_: TElement;
      S,SV: string;
      ii: integer;
      Numb: integer;
      eee: TElement;
      RealLink: boolean;
   begin
      if NoGeometry then
      begin
         if not GE.IsValid then
            exit;
         eee := GE;
      end
      else
         eee := G;
      //GTA :=  TGeometryTextAnnotation(eee.FindClassWithOwnerInOwners(TGeometryTextAnnotation,AC));
      // zjitn, zda Annotation pro vrstvu ji neexistuje
      GTA := nil;
      for ii := 0 to eee.OwnersCount - 1 do
      begin
         En_ := eee.Owners[ii];
         if not (En_ is TGeometryTextAnnotation) then continue;
         if En_.IsMyOwner(AC) then
         begin
            GTA := TGeometryTextAnnotation(En_);
            Break;
         end;
      end;
      // pokus o nalezen ztracench kotev se presunul do hlavniho tela
      if GTA = nil then
      begin
         GTA := TGeometryTextAnnotation.Create;
         if NewIsInvisible then
            GTA.Visible := false
         else
            GTA.Visible := true;
         AC.NextElement := GTA;
         GTA.NextElement := eee;
         GTA.UpdateRect(eee);
         LinkGTA := nil;
         RealLink := false;
         if Link then  // vyeen linku a vech jeho pipojen
         begin
            LinkGTA :=  TGeometryTextAnnotation(eee.FindClassWithOwnerInOwners(TGeometryTextAnnotation,LinkToCollection));
            if LinkGTA <> nil then
            begin
               RealLink := true;
               GTA.NoFous := true;
               if not NewIsInvisible then
                  GTA.Visible := LinkGTA.Visible;
               if not GTA.IsMyOwner(LinkGTA) then
                  LinkGTA.NextElement := GTA;
               for ii := 0 to LinkGTA.LastIndex do
               begin
                  e_ := LinkGTA.Elements[ii];
                  if not (e_ is TGeometry) then continue;
                  if not e_.IsMyOwner(GTA) then
                     GTA.AddAndUpdateRect(e_);
               end;
            end;
         end;
         if NoGeometry then
         begin
            if G <> nil then
            begin
               GTA.AbsRefX := G.X;
               GTA.AbsRefY := G.Y;
            end
            else
            begin
               GTA.AbsRefX := GE.CenterX;
               GTA.AbsRefY := GE.CenterY;
            end;
            GTA.Style := Style;
         end
         else
         begin
            GTA.AbsRefX := G.X;
            GTA.AbsRefY := G.Y;
            if CheckStyle1 then
            begin
               SV := '';
               S := GTA.Text;
               ii := 1;
               if S <> '' then
               begin
                  while not (ii > Length(S)) and (S[ii]=' ') do Inc(ii);
                  while not (ii > Length(S)) and (S[ii] in ['0'..'9']) do
                  begin
                     SV := SV + S[ii];
                     Inc(ii);
                  end;
                  Numb := StrToIntDef(SV,0);
                  if (Numb >= CheckValue) then
                  begin
                     GTA.Style := Style1;
                  end;
               end;
            end;
         end;
         if RealLink then // u linku nastavime absRFefX na min coord
         begin
            GTA.AbsRefX := LinkGTA.X - LinkDistance;
            GTA.AbsRefY := LinkGTA.Y;
         end;
         if Device <> nil then
         begin
            if GTA.Style > 0 then
               TextInfo.Druh  := GTA.Style
            else
               TextInfo.Druh  := AC.Style;
            { vechno podle stylu .....
            TextInfo.Barva      := TE.Barva;
            TextInfo.Znacka     := TE.Znacka;
            TextInfo.Font       := TE.TopolFont;
            TextInfo.Vyska      := TE.Height;
            TextInfo.KoefSirky  := TE.WidthCoef;
            TextInfo.Sklon      := TE.Slope;
            TextInfo.Rozestupy  := TE.Distance;
            TextInfo.RezimTextu := TE.TextMode;
            }
            TextInfo.VychoziBod.x := GTA.x;
            TextInfo.VychoziBod.y := GTA.y;
            TextInfo.Smer       := GTA.Angle;
            TextInfo.Atributy   := [OkT,T30];
            TextInfo.CodePage   := cpLatin2;
            TextInfo.Veta       := WinTo852(GTA.Text);
            BGIF.TextInfo := TextInfo;
            BGIF.NastavInfoPodleDruhu(TextInfo.Druh);
            BGIF.CalculateOpsanyObdelnik;
            if RealLink then
            begin
               GTA.AbsRefX := GTA.AbsRefX - TextInfo.Rect.Width;
               //GTA.AbsRefY := GTA.AbsRefY - TextInfo.Rect.Height/2;  Y zustava...
            end
            else
            begin
               GTA.AbsRefX := GTA.AbsRefX - TextInfo.Rect.Width/2;
               GTA.AbsRefY := GTA.AbsRefY - TextInfo.Rect.Height/2;
            end;
            TextInfo.VychoziBod.x := GTA.x;
            TextInfo.VychoziBod.y := GTA.y;
            BGIF.CalculateOpsanyObdelnik;

            GTA.MinX := TextInfo.Rect.MinX;
            GTA.MinY := TextInfo.Rect.MinY;
            GTA.MaxX := TextInfo.Rect.MaxX;
            GTA.MaxY := TextInfo.Rect.MaxY;
            GTA.AbsRefx := TextInfo.VychoziBod.x;
            GTA.AbsRefy := TextInfo.VychoziBod.y;
         end;
      end;
   end;
begin
   if Input = nil then exit;
   if not(Input is TEntity) then exit;
   if Device = nil then
   begin
      ShowMessage('Create Annotation: No Device!');
      exit;
   end;
   En := TEntity(Input);
   E := FindElementClass(TGeometryAnnotationCollection);
   if E <> nil then FreeElement(E);
   if Output = nil then
   begin
      Output := TGeometryAnnotationCollection.Create;
   end
   else
      if not (Output is TGeometryAnnotationCollection) then exit;
   Link := (LinkToCollection <> nil) and (TElement(LinkToCollection) is TGeometryAnnotationCollection);
   AC := TGeometryAnnotationCollection(Output);
   AddElement(AC);
   AC.ElementClassName := ElementClassName;
   AC.PropertyName := PropertyName;
   AC.AltPropertyName := AltPropertyName;
   AC.Style := Style;
   AC.NoGeometry := NoGeometry;
   AC.ByText := ByText;
   StartProgress(Comment,0,AC.LastIndex);
   ProgressText := 'kotven divnch text ...';
   For i := 0 to AC.LastIndex do
   begin
      E := AC.Elements[i];
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      if not (E is TGeometryTextAnnotation) then continue;
      GTA := TGeometryTextAnnotation(E);
      for j := 0 to GTA.LastIndex do
      begin
         E := GTA.Elements[j];
         if not (E is TGeometry) then continue;
         G := TGeometry(E);                                     // je to plocha
         E := G.FindClassWithOwnerInOwners(AC.ElementClass,En); // hledame element nahore
         if E = nil then                                        // a kdyz chybi...
            Q.AddElement(G);                                    // dame ho do kontejneru
      end;
      if Q.Count > 0 then                                       // pokud je co vyhodit
      begin
         for j := 0 to Q.LastIndex do
         begin
            E := Q.Elements[j];
            GTA.RemoveElement(E);                                // zakotvime text
         end;
         Q.FreeAll;                                              // a pokusime se uvolnit G
      end;
   end;
   ProgressText := 'Hledn ztracench kotev...';
   LostAnchors := TOwnerEntity.Create;
   AddElement(LostAnchors);
   For i := 0 to AC.LastIndex do
   begin
      E := AC.Elements[i];
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      if not (E is TGeometryTextAnnotation) then continue;
      GTA := TGeometryTextAnnotation(E);
      for j := 0 to GTA.LastIndex do
      begin
         E := GTA.Elements[j];
         if not (E is TAnnotationLostAnchor) then continue;
         LostAnchors.AddElement(E);
      end;
   end;
   if LostAnchors.Count > 0 then
   begin
      MaxProgress := LostAnchors.LastIndex;
      Progress := LostAnchors.LastIndex;;
      ProgressText := 'Kotven text...';
      Application.ProcessMessages;
      // QTREE
      AQT :=TRectEntityQTree.Create;
      AQT.Name := 'AQT';
      AddElement(AQT);
      TQ := TAddEntityToQTree.Create;
      TQ.Input := En;
      TQ.Output := AQT;
      TQ.DoIt;
      TQ.Free;
      QTFinder := TFindRectEntityInQTreeByXY.Create;
      AddElement(QTFinder);
      QTFinder.Input := AQT;
      for i := LostAnchors.LastIndex DownTo 0 do
      begin
         if i mod 50 = 0 then
         begin
            Progress := i;
            Application.ProcessMessages;
         end;
         LA := TAnnotationLostAnchor(LostAnchors.Elements[i]);
         GTA := TGeometryTextAnnotation(LA.FindOwnerClass(TGeometryTextAnnotation));
         if GTA = nil then continue; // !?!?!?
         QTFinder.X := LA.X;
         QTFinder.Y := LA.Y;
         QTFinder.Tolerance := 0;
         QTFinder.DoIt;
         G := nil;
         e := QTFinder.Output;
         if e <> nil then
         begin
            if not (e is TGeometryElement) then continue; // !?!?!?
            GE := TGeometryElement(e);
            G := GE.GetSimpleGeometryByXY(LA.X,LA.Y,0);
         end;
         if G = nil then  // nenasli jsme plochu ...
         begin
            // zkusime najit plochu podle linku
            if Link then
            begin
               LinkGTA := TGeometryTextAnnotation(GTA.FindOwnerClass(TGeometryTextAnnotation));
               if (LinkGTA <> nil) and (LinkGTA.IsMyOwner(LinkToCollection)) then
               begin
                  G := TGeometry(LinkGTA.FindElementClass(TGeometry));
                  if G <> nil then
                     GTA.NextElement := G;
               end;
            end;
            LA.Free;
            Continue;  // jinak jdeme na dalsi
         end;
         if LA.Fous then
         begin
            AA := TAnnotationAbscissa.Create;
            AA.X1 := LA.X;
            AA.Y1 := LA.Y;
            GTA.NextElement := AA;
         end;
         GTA.NextElement := G;
         LA.Free;
      end;
      AQT.Free;
      QTFinder.Free;
   end;
   Progress := 0;
   MaxProgress := En.LastIndex;
   ProgressText := 'Kontrola a doplnn text...';
   Application.ProcessMessages;
   for i := 0 to En.LastIndex do
   begin
      E := En.Elements[i];
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      if not (E.ClassType = AC.ElementClass) then continue;
      // pozor !!!! zatim je to jenom pro plochy
      if not (E is TGeometryElement) then continue;
      GE := TGeometryElement(E);
      G := nil;
      if NoGeometry and GE.NoGeometry then
      begin
         CheckG;
         Continue;
      end;
      GG := TGeometryElement(E).Geometry;
      if GG = nil then continue;
      if GG is TMultiAreaElement then
      begin
         for j := 0 to GG.LastIndex do
         begin
            G := TGeometry(GG.Elements[j]);
            CheckG;
         end;
      end
      else
      begin
         G := GG;
         CheckG;
      end;
   end;
   ProgressText := 'Kontrola rekt a fous...';
   For i := AC.LastIndex downto 0 do
   begin
      E := AC.Elements[i];
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      if not (E is TGeometryTextAnnotation) then continue;
      GTA := TGeometryTextAnnotation(E);
      EmptyText := true;
      if not GTA.NoElements then
      for j := 0 to GTA.LastIndex do
      begin
         E := GTA.Elements[j];
         if E is TAnnotationAbscissa then continue;
         if E is TAnnotationLostAnchor then continue;
         if E is TGeometryTextAnnotation then continue;
         EmptyText := false;
         break;
      end;
      if EmptyText then // odstrann przdnch text
      begin
         GTA.Free;
         Continue;
      end;
      CHeckRectAndStyle(GTA);
      if not NoGeometry then
      begin
         CheckFousy(GTA);
         if Link then
         begin
            LinkGTA := TGeometryTextAnnotation(GTA.FindOwnerClass(TGeometryTextAnnotation));
            if LinkGTA <> nil then
               CheckFousy(LinkGTA);
         end;
      end;
   end;
   AC.Update;
end;



function TCreateTextAnnotation.GetQ: TQuickEntity;
begin
   if FQ = nil then
   begin
      FQ := TQuickEntity.Create;
      AddElement(FQ);
   end;
   result := FQ;
end;

procedure TCreateTextAnnotation.SetAltPropertyName(const Value: string);
begin
  FAltPropertyName := Value;
end;

procedure TCreateTextAnnotation.SetAltStyle(const Value: integer);
begin
  FAltStyle := Value;
end;

procedure TCreateTextAnnotation.SetAltStyle1(const Value: integer);
begin
  FAltStyle1 := Value;
end;

procedure TCreateTextAnnotation.SetByText(const Value: boolean);
begin
  FByText := Value;
end;

procedure TCreateTextAnnotation.SetCheckStyle1(const Value: boolean);
begin
  FCheckStyle1 := Value;
end;

procedure TCreateTextAnnotation.SetCHeckValue(const Value: integer);
begin
  FCHeckValue := Value;
end;



procedure TCreateTextAnnotation.SetElementClassName(const Value: string);
begin
  FElementClassName := Value;
end;

procedure TCreateTextAnnotation.SetLink(const Value: boolean);
begin
  FLink := Value;
end;


procedure TCreateTextAnnotation.SetLinkDistance(const Value: double);
begin
  FLinkDistance := Value;
end;

procedure TCreateTextAnnotation.SetLinkToCollection(const Value: TGeometryElementCollection);
begin
  FLinkToCollection := Value;
end;

procedure TCreateTextAnnotation.SetNewIsInvisible(const Value: boolean);
begin
  FNewIsInvisible := Value;
end;

procedure TCreateTextAnnotation.SetNoGeometry(const Value: boolean);
begin
  FNoGeometry := Value;
end;

procedure TCreateTextAnnotation.SetPropertyName(const Value: string);
begin
  FPropertyName := Value;
end;

procedure TCreateTextAnnotation.SetStyle(const Value: integer);
begin
  FStyle := Value;
end;

procedure TCreateTextAnnotation.SetStyle1(const Value: integer);
begin
  FStyle1 := Value;
end;

{ TBGIFTool }

function TBGIFTool.GetBGIF: TBGIFont;
begin
   if FBGIF = nil then
   begin
      FBGIF := TBGIFont.Create;
      AddElement(FBGIF);
      if Device <> nil then
         FBGIF.SetDevice(Device);
   end;
   result := FBGIF;
end;

function TBGIFTool.GetTextInfo: TBGIText;
begin
   if FTextInfo = nil then
   begin
      FTextInfo := TBGIText.Create;
      AddElement(FTextInfo);
   end;
   result := FTextInfo;
end;

function TBGIFTool.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'Device';
      1: result := 'Comment';
   end;
end;

procedure TBGIFTool.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TBGIFTool.SetDevice(const Value: TDevice);
begin
   FDevice := Value;
   BGIF.SetDevice(Value);
end;

{ TCreateTextsAndLinesFromTextAnnotation }

procedure TCreateTextsAndLinesFromTextAnnotation.DoIt;
var
   En: TEntity;
   i: integer;
begin
   if Input = nil then exit;
   if not(Input is TEntity) then exit;
   if Device = nil then
   begin
      ShowMessage('Create Annotation: No Device!');
      exit;
   end;
   En := TEntity(Input);
   if En.NoElements then exit;
   StartProgress(Comment,0,En.LastIndex);
   for i := 0 to En.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      Process(En.Elements[i]);
   end;
end;

function TCreateTextsAndLinesFromTextAnnotation.GetLines: TGeometryElementCollection;
begin
   if FLines = nil then
   begin
      FLines := TPolyLineCollection.Create;
      FLines.Name := 'L';
      AddElement(FLines);
   end;
   result := FLines;
end;

function TCreateTextsAndLinesFromTextAnnotation.GetTexts: TGeometryElementCollection;
begin
   if FTexts = nil then
   begin
      FTexts := TTextCollection.Create;
      FTexts.Name := 'T';
      AddElement(FTexts);
      Output := FTexts;
   end;
   result := FTexts;
end;

procedure TCreateTextsAndLinesFromTextAnnotation.Process(aElement: TElement);
var
  TA: TGeometryTextAnnotation;
  GAC: TGeometryAnnotationCollection;
  AA: TAnnotationAbscissa;
  E: TElement;
  i: integer;
  TT: TTopolTextElement;
  TP: TTextElement;
  TL: TTopolLineElement;
  L: TPolyLineElement;
begin
   if not (aElement is TGeometryTextAnnotation) then exit;
   if BGIF.CHybaFontu then
   begin
      PostTextMsg(tm_AddToProtocol,'Create Annotation Text - FontError !',nil);
      exit;
   end;
   TA := TGeometryTextAnnotation(aElement);
   if not TA.Visible then exit;
   GAC := TA.GACollection;
   if GAC = nil then
   begin
      PostTextMsg(tm_AddToProtocol,'Create Annotation Text - Lost collection !',nil);
      exit;
   end;
   try
    //SetInterpretTopProps(TTopoLText(Element).ObjectOwner);
      if TA.Style > 0 then
         TextInfo.Druh       := TA.Style
      else
         TextInfo.Druh       := GAC.Style;
      { vechno podle stylu .....
      TextInfo.Barva      := TE.Barva;
      TextInfo.Znacka     := TE.Znacka;
      TextInfo.Font       := TE.TopolFont;
      TextInfo.Vyska      := TE.Height;
      TextInfo.KoefSirky  := TE.WidthCoef;
      TextInfo.Sklon      := TE.Slope;
      TextInfo.Rozestupy  := TE.Distance;
      TextInfo.RezimTextu := TE.TextMode;
      }
      TextInfo.VychoziBod.x := TA.x;
      TextInfo.VychoziBod.y := TA.y;
      TextInfo.Smer       := TA.Angle;
      TextInfo.Atributy   := [OkT,T30];
      TextInfo.CodePage   := cpLatin2;
      TextInfo.Veta       := WinTo852(TA.Text);
      BGIF.TextInfo := TextInfo;
      BGIF.NastavInfoPodleDruhu(TextInfo.Druh);
      BGIF.CalculateOpsanyObdelnik;
   except
   end;

   TT := TTopoLTextElement.Create;
   TT.MinX := TextInfo.Rect.MinX;
   TT.MinY := TextInfo.Rect.MinY;
   TT.MaxX := TextInfo.Rect.MaxX;
   TT.MaxY := TextInfo.Rect.MaxY;
   //**************
   TP:= TTextElement.Create;
   //**************
   TT.Geometry := TP;
   //TT.Cislo       := TextInfo.Cislo; na cislo sere pes
   TT.Druh         := TextInfo.Druh;
   TT.TextAtrib    := TextInfo.Atributy;
   TT.Barva        := TextInfo.Barva;
   TT.Znacka       := TextInfo.Znacka;
   TT.TextMode     := TextInfo.RezimTextu;
   TT.Druh         := TextInfo.Druh;
   TT.Direction    := TextInfo.Smer;
   TT.TopolFont    := TextInfo.Font;
   TT.StartPoint.x := TextInfo.VychoziBod.X;
   TT.StartPoint.y := TextInfo.VychoziBod.Y;
   TT.CodePage     := TextInfo.CodePage;
   //**************
   TP.x            := TT.StartPoint.x;
   TP.y            := TT.StartPoint.y;
   //**************
   TT.Height       := TextInfo.Vyska;
   TT.WidthCoef    := TextInfo.KoefSirky;
   TT.Slope        := TextInfo.Sklon;
   TT.Distance     := TextInfo.Rozestupy;
   TP.Text         := TextInfo.Veta;
   Texts.AddAndUpdateRect(TT);
   if GAC.NoGeometry then exit;
   for i := 0 to TA.LastIndex do
   begin
      e := TA.Elements[i];
      if not (e.ClassType = TAnnotationAbscissa) then continue;
      AA := TAnnotationAbscissa(e);
      TL := TTopolLineElement.Create;
      L := TPolyLineElement.Create;
      L.AddXY(AA.X1,AA.Y1);
      L.AddXY(AA.X2,AA.Y2);
      L.RecalculateRect;
      L.CalculateLabelProps;
      TL.Geometry := L;
      TL.Druh := LineGroup;
      TL.Barva := LineColor;
      TL.Znacka := LineSymbol;
      Lines.AddAndUpdateRect(TL);
   end;
end;

procedure TCreateTextsAndLinesFromTextAnnotation.SetLineColor(const Value: integer);
begin
  FLineColor := Value;
end;

procedure TCreateTextsAndLinesFromTextAnnotation.SetLineGroup(const Value: integer);
begin
  FLineGroup := Value;
end;

procedure TCreateTextsAndLinesFromTextAnnotation.SetLineSymbol(const Value: integer);
begin
  FLineSymbol := Value;
end;

{ TSetPorostTextVisibility }

procedure TSetPorostTextVisibility.DoIt;
var
   GAC: TGeometryAnnotationCollection;
   GTA: TGeometryTextAnnotation;
   e: TElement;
   i,j: integer;
   P: Tgis_Porost;
   PSK: Tgis_PorostniSkupina;
   FU: Tgis_ForestUnit;
   G: TGeometry;
   //A: TAreaElement;
   QAre,QAnot: TQuickEntity;

begin
   if not Prepare then exit;
   GAC := GACollection;
   StartProgress(Comment,0,GAC.LastIndex);
   QAre := TQuickEntity.Create;
   QAnot := TQuickEntity.Create;
   for i := 0 to GAC.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      e := GAC.Elements[i];
      if not (e is TGeometryTextAnnotation) then continue;
      GTA := TGeometryTextAnnotation(e);
      // nalezen plochy
      G := TGeometry(GTA.FindElementClass(TGeometry));
      if G = nil then continue;
      // nalezeni porostu pres por skupinu
      PSK := Tgis_PorostniSkupina(G.FindClassInOwners(Tgis_PorostniSkupina));
      if PSK = nil then continue;
      P := PSK._Porost;
      if P = nil then continue;
      if P.IsMyOwner(eDone) then continue; // uz by mel byt vyresen
      eDone.AddElement(P);                 // a nebo ted, hi hi
      // vytahani vsech ploch skupin pro porost a nalezeni anotaci
      QAre.Clear;
      QAnot.Clear;
      // 1) plochy
      for j := 0 to P.Items.LastIndex do
      begin
         FU := P.Items.ForestUnit[j];
         if FU.NoGeometry then continue;
         if FU.Geometry is TMultiAreaElement then
            QAre.AddAllFrom := FU.Geometry
         else
            QAre.AddElement(FU.Geometry);
      end;
      if QAre.Count < 2 then continue; // jedna plocha, kasleme na porost
      // 2) anot
      for j := 0 to QAre.LastIndex do
      begin
         e := QAre.Elements[j];
         GTA := TGeometryTextAnnotation(e.FindClassWithOwnerInOwners(TGeometryTextAnnotation,GAC));
         if GTA = nil then continue;
         if (not GTA.Visible) or (not GTA.Linked) then   // uz na nej nekdo sahal
         begin
            Touched.AddAndUpdateRect(P);
            break;
         end;
         if QAnot.IndexOf(GTA) < 0 then
         begin
            QAnot.AddElement(GTA);
         end;
      end;
      if P.IsMyOwner(Touched) then continue;  // je v dotknutych, jedeme dal
      Solved.AddAndUpdateRect(P); // konecne ... prvni text bude videt, ostatni ne
      for j := 0 to QAnot.LastIndex do
      begin
         GTA := TGeometryTextAnnotation(QAnot.Elements[j]);
         GTA.Visible := j = 0;
      end;
   end;
   QAre.Clear;
   QAre.Free;
   QAnot.Clear;
   QAnot.Free;
end;

function TSetPorostTextVisibility.GetGACollection: TGeometryAnnotationCollection;
begin
   result := TGeometryAnnotationCollection(Input);
end;

function TSetPorostTextVisibility.Prepare: boolean;
begin
   result := false;
   if Output <> nil then
      Output.Free;
   Output := TOwnerEntity.Create;
   if OutputText <> '' then
      output.Name := OutputText
   else
      output.Name := 'Set porost text visibility';
   AddElement(Output);
   if (Input = nil) or not (Input is TGeometryAnnotationCollection) then
   begin
      Output.Name := 'Chyb vrstva autotext';
      exit;
   end;
   if GACollection.NoElements then
   begin
      Output.Name := 'Vrstva autotext je przdn';
      exit;
   end;
   Solved := TGeometryElementCollection.Create;
   if SolvedText <> '' then
      Solved.Name := SolvedText
   else
      Solved.Name := 'Solved';
   TEntity(Output).AddElement(Solved);
   Touched := TGeometryElementCollection.Create;
   if TouchedText <> '' then
      Touched.Name := TouchedText
   else
      Touched.Name := 'Touched';
   TEntity(Output).AddElement(Touched);
   eDone := TOwnerEntity.Create;
   eDone.Name := 'Done';
   result := true;
end;

procedure TSetPorostTextVisibility.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TSetPorostTextVisibility.SetEDone(const Value: TEntity);
begin
  ExchangeFieldElement(TElement(FEDone), Value);
end;

procedure TSetPorostTextVisibility.SetGACollection(const Value: TGeometryAnnotationCollection);
begin
  Input := Value;
end;

procedure TSetPorostTextVisibility.SetOutputText(const Value: string);
begin
  FOutputText := Value;
end;

procedure TSetPorostTextVisibility.SetSolved(const Value: TGeometryElementCollection);
begin
   FSolved := Value;
end;

procedure TSetPorostTextVisibility.SetSolvedText(const Value: string);
begin
  FSolvedText := Value;
end;

procedure TSetPorostTextVisibility.SetTouched(const Value: TGeometryElementCollection);
begin
  FTouched := Value;
end;

procedure TSetPorostTextVisibility.SetTouchedText(const Value: string);
begin
  FTouchedText := Value;
end;

{ TCreateTextAnnotationFromTextsAndLines }

procedure TCreateTextAnnotationFromTextsAndLines.DoIt;
var
   AC: TGeometryAnnotationCollection;
   GAT: TGeometryTextAnnotation;
   //En: TEntity;
   E: TElement;
   i: integer;
begin
   if Input = nil then exit;
   if not(Input is TEntity) then exit;
   if (Texts = nil) or (Texts.NoElements) then
   begin
      ShowMessage('Create Annotation: No Texts!');
      exit;
   end;
   if Device = nil then
   begin
      ShowMessage('Create Annotation: No Device!');
      exit;
   end;
   //En := TEntity(Input);
   E := FindElementClass(TGeometryAnnotationCollection);
   if E <> nil then FreeElement(E);
   if Output = nil then
   begin
      Output := TGeometryAnnotationCollection.Create;
   end
   else
      if not (Output is TGeometryAnnotationCollection) then exit;
   Link := (LinkToCollection <> nil) and (TElement(LinkToCollection) is TGeometryAnnotationCollection) and not LinkToCollection.NoElements;
   AC := TGeometryAnnotationCollection(Output);
   AddElement(AC);
   AC.ElementClassName := ElementClassName;
   AC.PropertyName := PropertyName;
   AC.AltPropertyName := AltPropertyName;
   AC.Style := Style;
   AC.NoGeometry := NoGeometry;
   AC.ByText := ByText;
   StartProgress(Comment,0,0);
   TXT := TGeometryElementCollection.Create;
   TXT.Name := 'TXT';
   LIN := TGeometryElementCollection.Create;
   LIN.Name := 'LIN';
   ProgressText := 'Selekce a kontrola text...';
   Application.ProcessMessages;
   FindTextAndLines;
   ProgressText := 'Pprava hyperprostoru...';
   Application.ProcessMessages;
   PrepareQTree;
   ProgressText := 'Anna a Lza text ...';
   Application.ProcessMessages;
   ProcessTexts;
   EndProgress;
   //TXT := nil;    //Texty se schovaji pro dalsi pouziti
   LIN := nil;
   LevelQT := nil;
   LinQT := nil;
   LinkQT := nil;
   NewIsInvisible := true;  // nove texty nebudou videt
   inherited;
   {
   EndProgress;
   // SwapVisibility
   AC := TGeometryAnnotationCollection(Output);
   if AC = nil then exit;
   for i := 0 to AC.LastIndex do
   begin
      e := AC.Elements[i];
      if not (e is TGeometryTextAnnotation) then continue;
      GAT := TGeometryTextAnnotation(e);
      GAT.Visible := not GAT.Visible;
   end;
   if Link then   // pokud je linkovani, musi se jeste po prehozeni visibility obnovit fousy
      inherited;
   }
end;

function TCreateTextAnnotationFromTextsAndLines.CHeckTextRectangle(TEl: TElement): boolean;
var
   TE: TTopolTextElement;
   TX: TTextElement;
begin
   result := false;
   if not (TEl is TTopolTextElement) then exit;
   TE := TTopolTextElement(TEl);
   TX := TTextElement(TE.Geometry);
   if TX = nil then exit;
   if Device <> nil then
   begin
      result := true;
    //SetInterpretTopProps(TTopoLText(Element).ObjectOwner);

      TextInfo.Druh       := TE.Druh;
      TextInfo.Barva      := TE.Barva;
      TextInfo.Znacka     := TE.Znacka;
      TextInfo.Font       := TE.TopolFont;
      TextInfo.Vyska      := TE.Height;
      TextInfo.KoefSirky  := TE.WidthCoef;
      TextInfo.Smer       := TE.Direction;
      TextInfo.Sklon      := TE.Slope;
      TextInfo.Rozestupy  := TE.Distance;
      TextInfo.Atributy   := TE.TextAtrib;//u_Symbol.TTextAtrib(TE.TextAtrib);
      TextInfo.RezimTextu := TE.TextMode;
      TextInfo.VychoziBod.x := TE.StartPoint.x;
      TextInfo.VychoziBod.y := TE.StartPoint.y;
      if TE.CodePage = cpWindows then
      begin
         TextInfo.CodePage   := cpLatin2;
         TextInfo.Veta       := WinTo852(TX.Text);
      end
      else
      begin
         TextInfo.CodePage   := TE.CodePage;
         TextInfo.Veta       := TX.Text;
      end;
      BGIF.TextInfo := TextInfo;
      BGIF.CalculateOpsanyObdelnik;
      TX.MinX := TextInfo.Rect.MinX;
      TX.MinY := TextInfo.Rect.MinY;
      TX.MaxX := TextInfo.Rect.MaxX;
      TX.MaxY := TextInfo.Rect.MaxY;
      TX.AbsRefX := TE.StartPoint.x;
      TX.AbsRefY := TE.StartPoint.y;
   end;
end;

procedure TCreateTextAnnotationFromTextsAndLines.SetTexts(const Value: TEntity);
begin
  FTexts := Value;
end;

procedure TCreateTextAnnotationFromTextsAndLines.SetLIN(const Value: TGeometryElementCollection);
begin
   ExchangeFieldElement(TElement(FLIN), Value);
end;

procedure TCreateTextAnnotationFromTextsAndLines.SetTXT(const Value: TGeometryElementCollection);
begin
   ExchangeFIeldElement(TElement(FTXT),Value);
end;

procedure TCreateTextAnnotationFromTextsAndLines.FindTextAndLines;
var
   i: integer;
   e: TElement;
   TE: TTopolTextElement;
begin
   Progress := 0;
   for i := 0 to Texts.LastIndex do
   begin
      {
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      }
      e := Texts.Elements[i];
      if (e is TTopolTextElement) then
      begin
         TE := TTopolTextElement(e);
         if not ((TE.Druh = Style) or (TE.Druh = AltStyle) or
                 (TE.Druh = Style1) or (TE.Druh = AltStyle1)) then continue;
         if not CHeckTextRectangle(TE) then continue;
         TXT.AddAndUpdateRect(TE);
      end
      else
      if (e is TTopolLineElement) then
         LIN.AddAndUpdateRect(e);
   end;
end;

procedure TCreateTextAnnotationFromTextsAndLines.SetLevelQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLevelQT), Value);
end;

procedure TCreateTextAnnotationFromTextsAndLines.SetLinQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLinQT), Value);
end;

procedure TCreateTextAnnotationFromTextsAndLines.PrepareQTree;
var
   TQ: TAddEntityToQTree;
begin
   // areas
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Input;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
   // Link
   if Link then
   begin
      LinkQT :=TRectEntityQTree.Create;
      LinkQT.Name := 'LINK QT';
      TQ := TAddEntityToQTree.Create;
      TQ.Input := LinkToCollection;
      TQ.Output := LinkQT;
      TQ.DoIt;
      TQ.Free;
   end;
   // lines
   if not NoFous then
   begin
      LinQT :=TRectEntityQTree.Create;
      LinQT.Name := 'Lin QT';
      TQ := TAddEntityToQTree.Create;
      TQ.Input := LIN;
      TQ.Output := LinQT;
      TQ.DoIt;
      TQ.Free;
   end;
end;

procedure TCreateTextAnnotationFromTextsAndLines.ProcessTexts;
var
   QTFinder: TFindRectEntityInQTreeByXY;
   Snap: TPointLineSnapTool;
   i,ii,idx,iii: integer;
   aX,aY,aX2,aY2,aaX,aaY,aaX2,aaY2: double;
   px,py: double;
   pB: boolean;
   TE: TTopolTextElement;
   e: TElement;
   Et: TElement;
   TL: TTopolLineElement;
   L: TPolyLineElement;
   S_L: TPolyLineElement;
   St,Sg: string;
   //LineBegin: boolean;
   AC: TGeometryAnnotationCollection;
   SolvedLines: TOwnerEntity;
   NSolvedLines: TOwnerEntity;
   PSolvedLines: TOwnerEntity;
   GTA,GTA1,LinkGTA: TGeometryTextAnnotation;
   AA,AA1: TAnnotationAbscissa;
   GE,GE1: TGeometryElement;
   G,G1,G1T : TGeometry;
   W: TQTreeWalker;
   MA: TMultiAreaElement;
   Poly: TPolyElement;
   dd,dd_,dd1,dd2: double;

   procedure SolveAreDistances;
   var
      ip: integer;
      Ow: TEntity;
      IsTx: boolean;
   begin
      IsTx := false;
      for ip := 0 to G.OwnersCount - 1 do
      begin
         Ow := G.Owners[ip];
         if not (Ow is TGeometryTextAnnotation) then continue;
         if Ow.IsMyOwner(AC) then
         begin
            IsTx := true; // uz tam nejaky takovy text je
            break;
         end;
      end;
      Poly := TPolyElement(G);
      for ip := 0 to Poly.CoordsCount - 1 do
      begin
         dd1 := Distance(TE.CenterX,TE.CenterY,Poly.Xi[ip],Poly.Yi[ip]);
         if dd1 < dd then
         begin
            if IsTx then
               G1T := G    // pokud text existuje
            else
               G1 := G;    // a pokud ne
            dd := dd1;
         end;
      end;
   end;

begin
   SolvedLines := TOwnerEntity.Create;
   AddElement(SolvedLines);
   SolvedLines.Name := 'SL';
   AC := TGeometryAnnotationCollection(Output);
   MaxProgress := TXT.LastIndex;
   Progress := 0;
   QTFinder := TFindRectEntityInQTreeByXY.Create;
   if LinSnapDistance < 1 then
      LinSnapDistance := 30;
   if LinkTolerance < 1 then
      LinkTolerance := 2;
   Snap := TPointLineSnapTool.Create;
   if not NoFous then // kasleme na linie
   begin
      Snap.FrameQT := LinQT;
      Snap.SnapToLine := true;
      Snap.SnapToLinePoint := true;
      Snap.SnapDistance := LinSnapDistance;
   end;
   for i := TXT.LastIndex downTo 0 do
   begin
      Progress := i;
      Application.ProcessMessages;
      TE := TTopolTextElement(TXT.Elements[i]);
      St := TE.Text;
      if Link then
      begin
         QTFinder.Input := LinkQT;
         QTFinder.Tolerance := 0;
         aX := TE.MaxX+ LinkTolerance * 2; // pichneme vedle textu
         aY := TE.CenterY;
         QTFinder.X := aX;
         QTFinder.Y := aY;
         QTFinder.DoIt;
         e := QTFinder.Output;
         if (e <> nil) then  // neco tam je
         begin
            LinkGTA := TGeometryTextAnnotation(e);
            // pokud konec jednoho a zacatek druheho lezi v toleranci, asi bude link
            if InTolerance(LinkGTA.MinX,LinkGTA.MinY,TE.MaxX,TE.MinY,LinkTolerance) then
            begin
               G := TGeometry(LinkGTA.FindElementClass(TGeometry)); // najdeme plochu
               if G <> nil then // teda pokud tam je
               begin
                  // hledame geometry element z inputu
                  e := G.FindClassWithOwnerInOwners(TGeometryElement,TEntity(Input));
                  if e <> nil then  // je tam
                  begin
                     Sg := e.GetPropertyStringByName(PropertyName);
                     if SameText(St,Sg) then  // je to hledan GeometryElement
                     begin
                        // vytvoreni Annotation a linkovani
                        //GE := TGeometryElement(e);
                        GTA := TGeometryTextAnnotation.Create;
                        AC.AddElement(GTA);
                        GTA.X := TE.StartPoint.X;
                        GTA.Y := TE.StartPoint.Y;
                        GTA.Width := TE.Width;
                        GTA.Height := TE.Height;
                        GTA.Style := TE.Druh;
                        GTA.Visible := true;
                        GTA.NoFous := true;
                        GTA.Angle := TE.Direction;
                        TXT.FreeElement(TE);
                        LinkGTA.NextElement := GTA;
                        for ii := 0 to LinkGTA.LastIndex do
                        begin
                           e := LinkGTA.Elements[ii];
                           if not (e is TGeometry) then continue;
                           GTA.AddElement(e);
                        end;
                        Continue;
                     end;
                  end;
               end;
            end;
         end;
      end;
      QTFinder.Input := LevelQT;
      QTFinder.Tolerance := 0;
      aX := TE.CenterX;
      aY := TE.CenterY;
      // pichneme si do plochy
      QTFinder.X := aX;
      QTFinder.Y := aY;
      QTFinder.DoIt;
      e := QTFinder.Output;
      if e <> nil then
      begin
         Sg := e.GetPropertyStringByName(PropertyName);
         if SameText(St,Sg) then  // je to hledan GeometryElement
         begin
            // vytvoreni Annotation
            GE := TGeometryElement(e);
            G := GE.GetSimpleGeometryByXY(aX,aY,0);
            if G <> nil then
            begin
               GTA := TGeometryTextAnnotation.Create;
               AC.AddElement(GTA);
               GTA.X := TE.StartPoint.X;
               GTA.Y := TE.StartPoint.Y;
               GTA.Width := TE.Width;
               GTA.Height := TE.Height;
               GTA.Style := TE.Druh;
               GTA.Visible := true;
               GTA.Angle := TE.Direction;
               GTA.AddElement(G);
               TXT.FreeElement(TE);
               Continue;
            end;
         end;
      end;
      if not NoFous then
      begin
         // plocha nic, tak zkusime linii
         Snap.X := aX;
         Snap.Y := aY;
         Snap.DoIt;
         //GTA := nil;
         GE := nil;
         aaX := 0;
         aaY := 0;
         aaX2 := 0;
         aaY2 := 0;
         S_L := nil;
         dd := LinSnapDistance * 10;
         if Snap.Snapped then
         for ii := 0 to Snap.SnapLines.LastIndex do
         begin
            L := TPolyLineElement(Snap.SnapLines.Elements[ii]);
            if L.IsMyOwner(SolvedLines) then continue; // uz ji nekdo ma
            aX := L.Xi[0];
            aY := L.Yi[0];
            aX2 := L.Xi[L.LastIndex];
            aY2 := L.Yi[L.LastIndex];
            dd1 := Distance(aX,aY,TE.CenterX,TE.CenterY);
            dd2 := Distance(aX2,aY2,TE.CenterX,TE.CenterY);
            if dd1 < dd2 then // prvni bod je bliz, prehodime je
            begin
               dd1 := aX2;
               dd2 := aY2;
               aX2 := aX;
               aY2 := aY;
               aX := dd1;
               aY := dd2;
            end;
            QTFinder.X := aX;
            QTFinder.Y := aY;
            QTFinder.DoIt;
            e := QTFinder.Output;
            if e = nil then // zkusime protahnout konec fousu o 1/4
            begin
               QTFinder.X := aX+(aX-aX2)/4;
               QTFinder.Y := aY+(aY-aY2)/4;
               QTFinder.DoIt;
               e := QTFinder.Output;
               if e <> nil then
               begin
                  aX := aX+(aX-aX2)/4;
                  aY := aY+(aY-aY2)/4;
               end;
            end;
            if e = nil then // zkusime protahnout konec fousu o 1/2
            begin
               QTFinder.X := aX+(aX-aX2)/2;
               QTFinder.Y := aY+(aY-aY2)/2;
               QTFinder.DoIt;
               e := QTFinder.Output;
               if e <> nil then
               begin
                  aX := aX+(aX-aX2)/2;
                  aY := aY+(aY-aY2)/2;
               end;
            end;
            if e = nil then
            begin
               dd1 := aX2;
               dd2 := aY2;
               aX2 := aX;
               aY2 := aY;
               aX := dd1;
               aY := dd2;
               QTFinder.X := aX;
               QTFinder.Y := aY;
               QTFinder.DoIt;
               e := QTFinder.Output;
            end;
            if e <> nil then
            begin
               Sg := e.GetPropertyStringByName(PropertyName);
               if SameText(St,Sg) then  // je to hledan GeometryElement
               begin
                  DejVztahyBod_Usecka(Snap.X,Snap.Y,aX,aY,aX2,aY2,pB,pX,pY,dd1);
                  dd2 := Distance(aX2,aY2,Snap.X,Snap.Y);
                  dd1 := dd1 + dd2;
                  if dd1 < dd then // hledame tu nejblizsi a mirici spr.smerem
                  begin
                     GE := TGeometryElement(e);
                     S_L := L;
                     aaX := aX;
                     aaY := aY;
                     aaX2 := aX2;
                     aaY2 := aY2;
                     dd := dd1;
                  end;
               end;
            end;
         end;
         if GE <> nil then  // mame tu adepta
         begin
            // vytvoreni Annotation s fousem podle nalezene linie
            G := GE.GetSimpleGeometryByXY(aaX,aaY,0);
            if G <> nil then
            begin
               GTA := TGeometryTextAnnotation.Create;
               AC.AddElement(GTA);
               GTA.X := TE.StartPoint.X;
               GTA.Y := TE.StartPoint.Y;
               GTA.Width := TE.Width;
               GTA.Height := TE.Height;
               GTA.Style := TE.Druh;
               GTA.Visible := true;
               GTA.Angle := TE.Direction;
               AA := TAnnotationAbscissa.Create;
               AA.X1 := aaX;
               AA.Y1 := aaY;
               AA.X2 := aaX2;
               AA.Y2 := aaY2;
               GTA.AddElement(AA);
               GTA.AddElement(G);
               TXT.FreeElement(TE);
               SolvedLines.AddElement(S_L);
               continue;
            end;
         end;
      end;
   end;
   //******************
   //   zkusime pouzit volne fousy pro existujici annotation
   //******************
   PSolvedLines := TOwnerEntity.Create;
   AddElement(PSolvedLines);
   PSolvedLines.Name := 'PSL';
   if not NoFous and (SolvedLines.Count < LIN.Count) then  // neco zbylo
   for i := 0 to AC.LastIndex do
   begin
      e := AC.Elements[i];
      if not (e is TGeometryTextAnnotation) then continue;
      GTA := TGeometryTextAnnotation(e);
      St := GTA.Text;
      Et := GTA.TextElement; // element for text
      Snap.X := GTA.CenterX;
      Snap.Y := GTA.CenterY;
      Snap.DoIt;
      //GTA := nil;
      GE := nil;
      aaX := 0;
      aaY := 0;
      aaX2 := 0;
      aaY2 := 0;
      dd := LinSnapDistance * 10;
      if Snap.Snapped then
      for ii := 0 to Snap.SnapLines.LastIndex do
      begin
         L := TPolyLineElement(Snap.SnapLines.Elements[ii]);
         if L.IsMyOwner(SolvedLines) then continue;
         aX := L.Xi[0];
         aY := L.Yi[0];
         aX2 := L.Xi[L.LastIndex];
         aY2 := L.Yi[L.LastIndex];
         dd1 := Distance(aX,aY,GTA.CenterX,GTA.CenterY);
         dd2 := Distance(aX2,aY2,GTA.CenterX,GTA.CenterY);
         dd := dd2;
         if dd1 < dd2 then // prvni bod je bliz, prehodime je
         begin
            dd := dd1;
            dd1 := aX2;
            dd2 := aY2;
            aX2 := aX;
            aY2 := aY;
            aX := dd1;
            aY := dd2;
         end;
         QTFinder.X := aX;
         QTFinder.Y := aY;
         QTFinder.DoIt;
         e := QTFinder.Output;
         if (e <> nil) and (FreeFous or (e = Et)) then    // pouze stejny objekt kvuli mishmashum
         begin
            Sg := e.GetPropertyStringByName(PropertyName);
            if SameText(St,Sg) then  // je to hledan GeometryElement
            begin
               GE := TGeometryElement(e);
               AA1 := TAnnotationAbscissa(L.FindOwnerClass(TAnnotationAbscissa));
               if AA1 <> nil then  // nasli jsme nejaky existujici zaznam
               begin
                  GTA1 := TGeometryTextAnnotation(AA1.FindElementClass(TGeometryTextAnnotation));
                  dd_ := distance(AA1.X2, AA1.Y2, GTA1.CenterX, GTA1.CenterY);
                  if dd_ < dd then continue;  // puvodni GTA je bliz
                  AA1.Free; // jinak rusime puvodni zaznam a vlozime novy
               end;
               AA := TAnnotationAbscissa.Create;
               AA.X1 := aX;
               AA.Y1 := aY;
               AA.X2 := aX2;
               AA.Y2 := aY2;
               PSolvedLines.AddElement(AA);
               AA.AddElement(L);
               AA.AddElement(GTA);
               AA.AddElement(GE);
            end;
         end;
      end;
   end;
   // teprve ted vytvarime fousy
   if not PSolvedLines.NoElements then
   for i := 0 to PSolvedLines.LastIndex do
   begin
      e := PSolvedLines.Elements[i];
      if not (e is TAnnotationAbscissa) then continue;
      AA := TAnnotationAbscissa(e);
      GTA := TGeometryTextAnnotation(AA.FindElementClass(TGeometryTextAnnotation));
      GE := TGeometryElement(AA.FindElementClass(TGeometryElement));
      L := TPolyLineElement(AA.FindElementClass(TPolyLineElement));
      G := GE.GetSimpleGeometryByXY(AA.X1,AA.Y1,0);
      if G <> nil then
      begin
         if G.IsMyOwner(GTA) then // tahle plocha uz v annotationu je
         begin
            idx := GTA.IndexOf(G);
            if (idx > 0) and (GTA.Elements[idx - 1] Is TAnnotationAbscissa) then
               continue; //plocha tam je a ma fous takze nic...
            //takze ji tam znova dame s fousem nize...
            GTA.RemoveElement(G);
         end;
         AA1 := TAnnotationAbscissa.Create;
         AA1.X1 := AA.X1;
         AA1.Y1 := AA.Y1;
         AA1.X2 := AA.X2;
         AA1.Y2 := AA.Y2;
         GTA.AddElement(AA1);
         GTA.AddElement(G);
         SolvedLines.AddElement(L);
      end;
   end;
   PSolvedLines.Free;
//******************
   if not TXT.NoElements then // pokud nejake texty jeste zbyly tak...
   begin
      MaxProgress := TXT.LastIndex;
      Progress := 0;
      for i := TXT.LastIndex downTo 0 do
      begin
         Progress := i;
         Application.ProcessMessages;
         TE := TTopolTextElement(TXT.Elements[i]);
         St := TE.Text;
         //A nastupuje Johny Walker se svymi 45%
         W := TQTreeWalker.Create;
         aX := TE.MinX - LinSnapDistance * 2;
         aY := TE.MinY - LinSnapDistance * 2;
         aX2 := TE.MaxX + LinSnapDistance * 2;
         aY2 := TE.MaxY + LinSnapDistance * 2;
         e := W.GetFirstElementForRectangle(LevelQT,aX,aY,aX2,aY2,0);
         G1 := nil;
         G1T := nil;
         DD := 1000;
         while e <> nil do
         begin
            Sg := e.GetPropertyStringByName(PropertyName);
            if SameText(St,Sg) then // to by mohlo byt ono
            begin
               GE := TGeometryElement(e);
               G := GE.Geometry;
               //G1 := nil;
               if G is TMultiAreaElement then
               begin
                  MA := TMultiAreaElement(G);
                  for ii := 0 to MA.LastIndex do
                  begin
                     G := MA.AreaElement[ii];
                     SolveAreDistances;
                  end;
               end
               else
                  SolveAreDistances;
            end;
            e := W.GetNextElement;
         end;
         W.Free;
         if G1 = nil then
            G1 := G1T;  // pokud neni plocha bez textu tak pouzijeme nejakou s textem
         if G1 <> nil then
         begin
            GTA := TGeometryTextAnnotation.Create;
            AC.AddElement(GTA);
            GTA.X := TE.StartPoint.X;
            GTA.Y := TE.StartPoint.Y;
            GTA.Width := TE.Width;
            GTA.Height := TE.Height;
            GTA.Style := TE.Druh;
            GTA.Visible := true;
            GTA.Angle := TE.Direction;
            GTA.AddElement(G1);
            GTA.NoFous := true;
            TXT.FreeElement(TE);
            //Continue;
         end;
      end;
   end;
//******************
   NSolvedLines := TOwnerEntity.Create;
   AddElement(NSolvedLines);
   NSolvedLines.Name := 'NSL';
   for i := 0 to LIN.LastIndex do // vyhozeni neosetrenych fousu
   begin
      e := LIN.Elements[i];
      if not (e is TTopolLineElement) then continue;
      TL := TTopolLineElement(e);
      G := TL.Geometry;
      if G = nil then continue;
      if not G.IsMyOwner(SolvedLines) then
         NSolvedLines.AddElement(TL);
   end;
   SolvedLines.Free;
   QTFinder.Free;
   Snap.Free;
end;

procedure TCreateTextAnnotationFromTextsAndLines.SetLinSnapDistance(
  const Value: double);
begin
  FLinSnapDistance := Value;
end;

procedure TCreateTextAnnotationFromTextsAndLines.SetLinkQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLinkQT), Value);
end;

procedure TCreateTextAnnotationFromTextsAndLines.SetNoFous(
  const Value: boolean);
begin
  FNoFous := Value;
end;

procedure TCreateTextAnnotationFromTextsAndLines.SetLinkTolerance(
  const Value: double);
begin
  FLinkTolerance := Value;
end;

procedure TCreateTextAnnotationFromTextsAndLines.SetFreeFous(
  const Value: boolean);
begin
  FFreeFous := Value;
end;

{ TCheckTopolTextRectangle }

procedure TCheckTopolTextRectangle.DoIt;
var
   TE: TTopolTextElement;
   En: TEntity;
   E: Telement;
   i: integer;
begin
   if Input = nil then exit;
   if (Input is TTopolTextElement) then
   begin
      TE := TTopolTextElement(Input);
      CHeckRectangle(TE);
      exit;
   end;
   if not (Input is TEntity) then exit;
   En := TEntity(input);
   if En.Count > 100 then
      StartProgress(Comment,0,En.LastIndex);
   for i := 0 to En.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      E := En.Elements[i];
      if not (E is TTopolTextElement) then continue;
      TE := TTopolTextElement(E);
      CHeckRectangle(TE);
   end;
end;

procedure TCheckTopolTextRectangle.CHeckRectangle(TE: TTopolTextElement);
var
   TX: TTextElement;
begin
   TX := TTextElement(TE.Geometry);
   if TX = nil then exit;
   if Device <> nil then
   begin
    //SetInterpretTopProps(TTopoLText(Element).ObjectOwner);

      TextInfo.Druh       := TE.Druh;
      TextInfo.Barva      := TE.Barva;
      TextInfo.Znacka     := TE.Znacka;
      TextInfo.Font       := TE.TopolFont;
      TextInfo.Vyska      := TE.Height;
      TextInfo.KoefSirky  := TE.WidthCoef;
      TextInfo.Smer       := TE.Direction;
      TextInfo.Sklon      := TE.Slope;
      TextInfo.Rozestupy  := TE.Distance;
      TextInfo.Atributy   := TE.TextAtrib;//u_Symbol.TTextAtrib(TE.TextAtrib);
      TextInfo.RezimTextu := TE.TextMode;
      TextInfo.VychoziBod.x := TE.StartPoint.x;
      TextInfo.VychoziBod.y := TE.StartPoint.y;
      if TE.CodePage = cpWindows then
      begin
         TextInfo.CodePage   := cpLatin2;
         TextInfo.Veta       := WinTo852(TX.Text);
      end
      else
      begin
         TextInfo.CodePage   := TE.CodePage;
         TextInfo.Veta       := TX.Text;
      end;
      BGIF.TextInfo := TextInfo;
      if InfoByDruh then
      begin
         BGIF.NastavInfoPodleDruhu(TextInfo.Druh);
         TE.TextMode  := TextInfo.RezimTextu;
         TE.TopolFont := TextInfo.Font;
         TE.Height    := TextInfo.Vyska;
         TE.WidthCoef := TextInfo.KoefSirky;
         TE.Slope     := TextInfo.Sklon;
         TE.Distance  := TextInfo.Rozestupy;
         if ColorByDruh then
            TE.Barva := TextInfo.Barva;
      end;
      BGIF.CalculateOpsanyObdelnik;
      TX.MinX := TextInfo.Rect.MinX;
      TX.MinY := TextInfo.Rect.MinY;
      TX.MaxX := TextInfo.Rect.MaxX;
      TX.MaxY := TextInfo.Rect.MaxY;
      TX.AbsRefX := TE.StartPoint.x;
      TX.AbsRefY := TE.StartPoint.y;
   end;
end;

procedure TCheckTopolTextRectangle.SetInfoByDruh(const Value: boolean);
begin
  FInfoByDruh := Value;
end;

function TCheckTopolTextRectangle.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'Device';
      1: result := 'Comment';
      2: result := 'InfoByDruh';
      3: result := 'ColorByDruh';
   end;
end;

procedure TCheckTopolTextRectangle.SetColorByDruh(const Value: boolean);
begin
  FColorByDruh := Value;
end;

{ TTopolTextToLines }

procedure TTopolTextToLines.DoIt;
var
   Eni,Eno: TEntity;
   D: TPolyLine_Device;
   i: integer;
   TE: TTopolTextElement;
   TL: TTopolLineElement;
   L: TPolyLineElement;
   e: TElement;
   Prog: boolean;
   LinE: TEntity;
begin
   Error := 1;
   if Device = nil then exit;
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   Eni := TEntity(Input);
   if Output = nil then
   begin
      Eno := TGeometryElementCollection.Create;
      Output := Eno;
      AddElement(Eno);
   end;
   if not (Output is TEntity) then exit;
   Eno := TEntity(Output);
   Error := 0;
   D := TPolyLine_Device.Create;
   AddElement(D);
   LinE := TGeometryElementCollection.Create;
   AddElement(LinE);
   D.Lines := LinE;
   D.Device := Device;
   BGIF.SetDevice(D);
   Prog := Eni.Count > 50;
   if Prog then
      StartProgress(Comment,0,Eni.LastIndex);
   D.Reset;
   for i := 0 to Eni.LastIndex do
   begin
      if Prog and (i mod 50 = 0) then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Eni.Elements[i];
      if not (e is TTopolTextElement) then continue;
      TE := TTopolTextElement(e);
      CHeckRectangle(TE);
      BGIF.ZobrazText(false);
   end;
   D.Close;
   D.Free;
   if Prog then
   begin
      Progress := 0;
      MaxProgress := LinE.LastIndex;
   end;
   for i := LinE.LastIndex downTo 0 do
   begin
      if Prog and (i mod 50 = 0) then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := LinE.Elements[i];
      if not (e is TPolyLineElement) then continue;
      L := TPolyLineElement(e);
      TL := TTopolLineElement.Create;
      TL.Druh := 1;
      TL.Barva := 13;
      TL.Znacka := 0;
      TL.Geometry := L;
      Eno.AddElement(TL);
   end;
   if Eno is TRectEntity then
      TRectEntity(Eno).Update;
   LinE.Free;
end;

function TTopolTextToLines.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0,1: result := inherited NumberToPropertyName(aNumber);
        2: result := 'Output';
   end;
end;

{ TTopolTextToShapes }

procedure TTopolTextToShapes.CreateShape(TE: TTopolTextElement);
var
   TX: TTextElement;
   LD,PD,LH,PH: TDoubleCoord;
   TL: TTopolLineElement;
   Are: TPolyLineAreaElement;
   L: TPolyLineElement;
   PGE: TPropertyGeometryElement;
begin
   TX := TTextElement(TE.Geometry);
   if TX = nil then exit;
   if Device <> nil then
   begin
    //SetInterpretTopProps(TTopoLText(Element).ObjectOwner);

      TextInfo.Druh       := TE.Druh;
      TextInfo.Barva      := TE.Barva;
      TextInfo.Znacka     := TE.Znacka;
      TextInfo.Font       := TE.TopolFont;
      TextInfo.Vyska      := TE.Height;
      TextInfo.KoefSirky  := TE.WidthCoef;
      TextInfo.Smer       := TE.Direction;
      TextInfo.Sklon      := TE.Slope;
      TextInfo.Rozestupy  := TE.Distance;
      TextInfo.Atributy   := TE.TextAtrib;//u_Symbol.TTextAtrib(TE.TextAtrib);
      TextInfo.RezimTextu := TE.TextMode;
      TextInfo.VychoziBod.x := TE.StartPoint.x;
      TextInfo.VychoziBod.y := TE.StartPoint.y;
      if TE.CodePage = cpWindows then
      begin
         TextInfo.CodePage   := cpLatin2;
         TextInfo.Veta       := WinTo852(TX.Text);
      end
      else
      begin
         TextInfo.CodePage   := TE.CodePage;
         TextInfo.Veta       := TX.Text;
      end;
      BGIF.TextInfo := TextInfo;
      BGIF.OpsanyKosodelnik(LD,PD,LH,PH);
      L := TPolyLineElement.Create;
      L.AddXY(LD.X,LD.Y);
      L.AddXY(PD.X,PD.Y);
      L.AddXY(PH.X,PH.Y);
      L.AddXY(LH.X,LH.Y);
      L.AddXY(LD.X,LD.Y);
      L.RecalculateRect;
      L.CalculateLabelProps;
      Are := TPolyLineAreaElement.Create;
      Are.AddPolyLine(L,true);
      if CreateGeometryElements then
      begin
         PGE := TPropertyGeometryElement.Create;
         PGE.Geometry := Are;
         PGE.Properties := TE;
         PGE.Name := TX.Text;
         TEntity(Output).AddElement(PGE);
      end
      else
      begin
         Are.Name := TX.Text;
         TEntity(Output).AddElement(Are);
      end;
   end;
end;

procedure TTopolTextToShapes.DoIt;
var
   Eni,Eno: TEntity;
   i: integer;
   TE: TTopolTextElement;
   e: TElement;
   Prog: boolean;
begin
   Error := 1;
   if Device = nil then exit;
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   Eni := TEntity(Input);
   if Output = nil then
   begin
      Eno := TGeometryElementCollection.Create;
      Output := Eno;
      AddElement(Eno);
   end;
   if not (Output is TEntity) then exit;
   Eno := TEntity(Output);
   Error := 0;
   Prog := Eni.Count > 50;
   if Prog then
      StartProgress(Comment,0,Eni.LastIndex);
   for i := 0 to Eni.LastIndex do
   begin
      if Prog and (i mod 50 = 0) then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Eni.Elements[i];
      if not (e is TTopolTextElement) then continue;
      TE := TTopolTextElement(e);
      CreateShape(TE);
   end;
   if Eno is TRectEntity then
      TRectEntity(Eno).Update;
end;

function TTopolTextToShapes.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0,1,2: result := inherited NumberToPropertyName(aNumber);
        3: result := 'CreateGeometryElements';
   end;
end;

procedure TTopolTextToShapes.SetCreateGeometryElements(const Value: boolean);
begin
  FCreateGeometryElements := Value;
end;

{ TSeparateNoNumberTexts }

procedure TSeparateNoNumberTexts.DoIt;
var
   TE: TTopolTextElement;
   En: TEntity;
   E: Telement;
   i: integer;
begin
   Error := 1;
   if Input = nil then exit;
   Error := 2;
   if Device = nil then exit;
   Error := 3;
   if not (Input is TEntity) then exit;
   Error := 0;
   En := TEntity(input);
   if En.Count > 100 then
      StartProgress(Comment,0,En.LastIndex);
   NModify := 0;
   InfoByDruh := true;
   for i := 0 to En.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      E := En.Elements[i];
      if not (E is TTopolTextElement) then continue;
      TE := TTopolTextElement(E);
      if (TE.Druh = Style) or (TE.Druh = Style1) or (TE.Druh = AltStyle) or (TE.Druh = AltStyle1) then
         Process(TE);
   end;
   for i := En.LastIndex downto 0 do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      E := En.Elements[i];
      if not (E is TTopolTextElement) then continue;
      TE := TTopolTextElement(E);
      if TE.Text = '' then
         TE.Free;
   end;
end;

function TSeparateNoNumberTexts.GetLins: TEntity;
begin
   if FLins = nil then
   begin
      FLins := TOwnerEntity.Create;
      AddElement(FLins);
      FLins.Name := 'LIN';
   end;
   result := FLins;
end;

function TSeparateNoNumberTexts.NumberToPropertyName(aNumber: Integer): string;
begin
   result := '';
end;

procedure TSeparateNoNumberTexts.Process(TE: TTopolTextElement);
var
   TX: TTextElement;
   TE1: TTopolTextElement;
   TX1: TTextElement;
   S: string;
   S1,S2,S0,S3: string;
   i: integer;
   l: integer;
   NDruh: integer;
   IsFous: boolean;
   LastX: double;
begin
   TX := TTextElement(TE.Geometry);
   if TX = nil then exit;
   S := BezVnejsichMezer(TX.Text);
   if S = '' then exit;
   IsFous := (Pos('-',S) > 0) or (Pos('_',S) > 0);
   if not (S[1] in ['0'..'9']) or IsFous then  //budeme resit ... neni cislo nebo je fous
   begin
      NModify := NModify + 1;
      ProgressText := IntToStr(NModify);
      Application.ProcessMessages;
      S0 := '';      // fousy na zacatku
      S1 := '';      // tady bude zacatek textu az po cislo
      S2 := '';      // a tady zbytek
      S3 := '';      // fousy a tak na konci
      l := Length(S);
      i := 1;
      while (i <= l) and (S[i] in ['-','_']) do  // fous na zacatku
      begin
         S0 := S0 + S[i];
         Inc(i);
      end;
      while (i <= l) and (S[i] = ' ') do Inc(i);  // preskocime mezery
      while (i <= l) and (not(S[i] in ['0'..'9',' ','-','_'])) do  // do cisla
      begin
         S1 := S1 + S[i];
         Inc(i);
      end;
      while (i <= l) and (S[i] = ' ') do Inc(i);  // preskocime mezery
      while (i <= l) and (not(S[i] in ['-','_',' ']))do  // to s cislem
      begin
         S2 := S2 + S[i];
         Inc(i);
      end;
      while (i <= l) and (S[i] = ' ') do Inc(i);  // preskocime mezery
      while (i <= l) and ((S[i] in ['-','_']))do  // to s cislem
      begin
         S3 := S3 + S[i];
         Inc(i);
      end;
      // Novy Druh
      if (TE.Druh = Style) or (TE.Druh = Style1) then
         NDruh := NStyle
      else
         NDruh := NAltStyle;
      if (S0 = '') and (S2 = '') and (S3 = '') then   // jenom text bez cisel ... pouze zmena druhu
      begin
         TE.Druh := NDruh;
         CHeckRectangle(TE);
         exit;
      end;
      LastX := TE.StartPoint.x;
      if S0 <> '' then
      begin
         TE1 := TTopolTextElement.Create;
         TX1 := TTextElement.Create;
         TX1.Text := S0;
         TE1.Geometry := TX1;
         Lins.AddElement(TE1);
         TE1.Druh       := TE.Druh;
         TE1.Barva      := TE.Barva;
         TE1.Direction  := TE.Direction;
         TE1.TextAtrib  := TE.TextAtrib;//u_Symbol.TTextAtrib(TE.TextAtrib);
         TE1.StartPoint.x := Lastx;
         TE1.StartPoint.y := TE.StartPoint.y;
         TE1.CodePage    := TE.CodePage;
         CHeckRectangle(TE1);
         LastX := TE1.MaxX + LinkDistance;
      end;
      if S1 <> '' then
      begin
         TE1 := TTopolTextElement.Create;
         TX1 := TTextElement.Create;
         TX1.Text := S1;
         TE1.Geometry := TX1;
         TEntity(Input).AddElement(TE1);
         TE1.Druh       := NDruh;
         TE1.Barva      := TE.Barva;
         TE1.Direction  := TE.Direction;
         TE1.TextAtrib  := TE.TextAtrib;//u_Symbol.TTextAtrib(TE.TextAtrib);
         TE1.StartPoint.x := Lastx;
         TE1.StartPoint.y := TE.StartPoint.y;
         TE1.CodePage    := TE.CodePage;
         CHeckRectangle(TE1);
         LastX := TE1.MaxX;
      end;
      TX.Text := S2;
      if S2 <> '' then
      begin
         TE.StartPoint.x := LastX;
         CheckRectangle(TE);
         LastX := TE.MaxX + LinkDistance ;
      end;
      if S3 <> '' then
      begin
         TE1 := TTopolTextElement.Create;
         TX1 := TTextElement.Create;
         TX1.Text := S3;
         TE1.Geometry := TX1;
         Lins.AddElement(TE1);
         TE1.Druh       := TE.Druh;
         TE1.Barva      := TE.Barva;
         TE1.Direction  := TE.Direction;
         TE1.TextAtrib  := TE.TextAtrib;//u_Symbol.TTextAtrib(TE.TextAtrib);
         TE1.StartPoint.x := Lastx;
         TE1.StartPoint.y := TE.StartPoint.y;
         TE1.CodePage    := TE.CodePage;
         CHeckRectangle(TE1);
      end;
   end;
end;

procedure TSeparateNoNumberTexts.SetAltStyle(const Value: integer);
begin
  FAltStyle := Value;
end;

procedure TSeparateNoNumberTexts.SetAltStyle1(const Value: integer);
begin
  FAltStyle1 := Value;
end;

procedure TSeparateNoNumberTexts.SetLinkDistance(const Value: double);
begin
  FLinkDistance := Value;
end;

procedure TSeparateNoNumberTexts.SetNAltStyle(const Value: integer);
begin
  FNAltStyle := Value;
end;

procedure TSeparateNoNumberTexts.SetNModify(const Value: integer);
begin
  FNModify := Value;
end;

procedure TSeparateNoNumberTexts.SetNStyle(const Value: integer);
begin
  FNStyle := Value;
end;

procedure TSeparateNoNumberTexts.SetStyle(const Value: integer);
begin
  FStyle := Value;
end;

procedure TSeparateNoNumberTexts.SetStyle1(const Value: integer);
begin
  FStyle1 := Value;
end;

{ TTopolPointToLines }

constructor TTopolPointToLines.Create;
begin
   inherited;
   Symbols := TTopoLSymbols.CreateAll(PathOfLibraries+'\Fonty.txt');
end;

destructor TTopolPointToLines.Destroy;
begin
   if Symbols <> nil then
      Symbols.Free;
   inherited;
end;

procedure TTopolPointToLines.DoIt;
var
   Eni,Eno: TEntity;
   D: TPolyLine_Device;
   i: integer;
   TP: TTopolPointElement;
   P: TPointElement;
   TL: TTopolLineElement;
   L: TPolyLineElement;
   e: TElement;
   Prog: boolean;
   LinE: TEntity;
begin
   Error := 1;
   if Device = nil then exit;
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   Eni := TEntity(Input);
   if Output = nil then
   begin
      Eno := TGeometryElementCollection.Create;
      Output := Eno;
      AddElement(Eno);
   end;
   if not (Output is TEntity) then exit;
   Eno := TEntity(Output);
   Error := 0;
   D := TPolyLine_Device.Create;
   AddElement(D);
   LinE := TGeometryElementCollection.Create;
   AddElement(LinE);
   D.Lines := LinE;
   D.Device := Device;
   Symbols.Device := D;
   Prog := Eni.Count > 50;
   if Prog then
      StartProgress(Comment,0,Eni.LastIndex);
   D.Reset;
   for i := 0 to Eni.LastIndex do
   begin
      if Prog and (i mod 50 = 0) then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Eni.Elements[i];
      if not (e is TTopolPointElement) then continue;
      TP := TTopolPointElement(e);
      P := TPointElement(TP.Geometry);
      Symbols.InterpretSymbol(P,TP.Znacka);
   end;
   D.Close;
   D.Free;
   if Prog then
   begin
      Progress := 0;
      MaxProgress := LinE.LastIndex;
   end;
   for i := LinE.LastIndex downTo 0 do
   begin
      if Prog and (i mod 50 = 0) then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := LinE.Elements[i];
      if not (e is TPolyLineElement) then continue;
      L := TPolyLineElement(e);
      TL := TTopolLineElement.Create;
      TL.Druh := 1;
      TL.Barva := 13;
      TL.Znacka := 0;
      TL.Geometry := L;
      Eno.AddElement(TL);
   end;
   if Eno is TRectEntity then
      TRectEntity(Eno).Update;
   LinE.Free;
end;

function TTopolPointToLines.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'Device';
      1: result := 'Comment';
      2: result := 'Output';
   end;
end;

procedure TTopolPointToLines.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TTopolPointToLines.SetDevice(const Value: TDevice);
begin
   FDevice := Value;
   Symbols.Device := Value;
end;

procedure TTopolPointToLines.SetSymbols(const Value: TTopoLSymbols);
begin
  FSymbols := Value;
end;

{ TTopolPointToShapes }

procedure TTopolPointToShapes.CreateShape(TE: TTopolPointElement; aLIN: TGeometryElementCollection);
var
   TP: TPointElement;
   LD,PD,LH,PH: TDoubleCoord;
   TL: TTopolLineElement;
   Are: TPolyLineAreaElement;
   L: TPolyLineElement;
   PGE: TPropertyGeometryElement;
begin
   TP := TPointElement(TE.Geometry);
   if TP = nil then exit;
   aLIN.FreeAll;
   Symbols.Device.Reset;
   Symbols.InterpretSymbol(TP,TE.Znacka);
   Symbols.Device.Close;
   aLIN.ResetAndUpdate;
   L := TPolyLineElement.Create;
   L.AddXY(aLIN.MinX,aLIN.MinY);
   L.AddXY(aLIN.MaxX,aLIN.MinY);
   L.AddXY(aLIN.MaxX,aLIN.MaxY);
   L.AddXY(aLIN.MinX,aLIN.MaxY);
   L.AddXY(aLIN.MinX,aLIN.MinY);
   aLIN.FreeAll;
   L.RecalculateRect;
   L.CalculateLabelProps;
   Are := TPolyLineAreaElement.Create;
   Are.AddPolyLine(L,true);
   if CreateGeometryElements then
   begin
      PGE := TPropertyGeometryElement.Create;
      PGE.Geometry := Are;
      PGE.Properties := TE;
      PGE.Name := IntToStr(TE.Druh)+':'+IntToStr(TE.Znacka);
      TEntity(Output).AddElement(PGE);
   end
   else
   begin
      Are.Name := IntToStr(TE.Druh)+':'+IntToStr(TE.Znacka);
      TEntity(Output).AddElement(Are);
   end;
end;

procedure TTopolPointToShapes.DoIt;
var
   D: TPolyLine_Device;
   Eni,Eno: TEntity;
   i: integer;
   TP: TTopolPointElement;
   e: TElement;
   Prog: boolean;
   LinE: TGeometryElementCollection;
begin
   Error := 1;
   if Device = nil then exit;
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   Eni := TEntity(Input);
   if Output = nil then
   begin
      Eno := TGeometryElementCollection.Create;
      Output := Eno;
      AddElement(Eno);
   end;
   if not (Output is TEntity) then exit;
   Eno := TEntity(Output);
   Error := 0;
   Prog := Eni.Count > 50;
   if Prog then
      StartProgress(Comment,0,Eni.LastIndex);
   D := TPolyLine_Device.Create;
   AddElement(D);
   LinE := TGeometryElementCollection.Create;
   AddElement(LinE);
   D.Lines := LinE;
   D.Device := Device;
   Symbols.Device := D;
   D.Reset;
   for i := 0 to Eni.LastIndex do
   begin
      if Prog and (i mod 50 = 0) then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Eni.Elements[i];
      if not (e is TTopolPointElement) then continue;
      TP := TTopolPointElement(e);
      CreateShape(TP,LinE);
   end;
   if Eno is TRectEntity then
      TRectEntity(Eno).Update;
   D.Close;
   D.Free;
   LinE.Free;
end;

function TTopolPointToShapes.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0,1,2: result := inherited NumberToPropertyName(aNumber);
        3: result := 'CreateGeometryElements';
   end;
end;

procedure TTopolPointToShapes.SetCreateGeometryElements(const Value: boolean);
begin
  FCreateGeometryElements := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCheckTopolTextRectangle);
   RegisterOrbisClass(TSeparateNoNumberTexts);
   RegisterOrbisClass(TTopolTextToLines);
   RegisterOrbisClass(TTopolTextToShapes);
   RegisterOrbisClass(TTopolPointToLines);
   RegisterOrbisClass(TTopolPointToShapes);
   RegisterOrbisClass(TCreateTextAnnotation);
   RegisterOrbisClass(TCreateTextsAndLinesFromTextAnnotation);
   RegisterOrbisClass(TSetPorostTextVisibility);
   RegisterOrbisClass(TCreateTextAnnotationFromTextsAndLines);
END.
