unit t_AnnotationEditor;

interface
uses
   Elements,
   e_Annotation,
   e_BGIF,
   e_QTree,
   e_Type,
   g_Elems,
   m_Intera,
   t_GraphicEditors,
   u_Symbol;

type

TGeometryTextAnnotationEditor = class(TGraphicEditor)
  private
    FPixelTolerance: integer;
    FLevelName: string;
    FLevel: TEntity;
    FLevelQT: TRectEntityQTree;
    FY: double;
    FX: double;
    FBGIF: TBGIFont;
    FTextInfo: TBGIText;
    FR: TRectEntity;
    FQ: TQuickEntity;
    FQAdd: TQuickEntity;
    FDragBusy: boolean;
    FDragFous: TAnnotationAbscissa;
    FAddingElement: TElement;
    FNoGeometry: boolean;
    FDisconect: boolean;
    FAltStyles: string;
    FStyles: string;
    FStylesList: TIntegerListElement;
    FStylesList1: TIntegerListElement;
    FLinkGTA: TGeometryTextAnnotation;
    FLink: boolean;
    FLinkLevelName: string;
    FLinkToLevel: TEntity;
    FAuxLevelName: string;
    FAuxProjectsName: string;
    FAuxProjects: TEntity;
    FAuxLinkLevelName: string;
    FOldAngle: double;
    FLinkDistance: double;
    FByText: boolean;
    FByTextLevelName: string;
    FByTextLevel: TEntity;
    FByTextLevelQT: TRectEntityQTree;
    FAuxByTextLevelName: string;
    FByTextPropertyName: string;
    procedure SetLevel(const Value: TEntity);
    procedure SetLevelName(const Value: string);
    procedure SetLevelQT(const Value: TRectEntityQTree);
    procedure SetPixelTolerance(const Value: integer);
    procedure SetX(const Value: double);
    procedure SetY(const Value: double);
    function GetBGIF: TBGIFont;
    function GetTextInfo: TBGIText;
    function GetR: TRectEntity;
    procedure SetDragBusy(const Value: boolean);
    function GetQ: TQuickEntity;
    procedure SetDragFous(const Value: TAnnotationAbscissa);
    function GetQAdd: TQuickEntity;
    procedure SetAddingElement(const Value: TElement);
    procedure SetNoGeometry(const Value: boolean);
    procedure SetDisconect(const Value: boolean);
    procedure SetAltStyles(const Value: string);
    procedure SetStyles(const Value: string);
    function GetStylesList: TIntegerListElement;
    function GetStylesList1: TIntegerListElement;
    procedure SetLinkGTA(const Value: TGeometryTextAnnotation);
    procedure SetLink(const Value: boolean);
    procedure SetLinkLevelName(const Value: string);
    procedure SetLinkToLevel(const Value: TEntity);
    procedure SetAuxLevelName(const Value: string);
    procedure SetAuxProjectsName(const Value: string);
    procedure SetAuxProjects(const Value: TEntity);
    procedure SetAuxLinkLevelName(const Value: string);
    procedure SetOldAngle(const Value: double);
    procedure SetLinkDistance(const Value: double);
    procedure SetByText(const Value: boolean);
    procedure SetByTextLevelName(const Value: string);
    procedure SetByTextLevel(const Value: TEntity);
    procedure SetByTextLevelQT(const Value: TRectEntityQTree);
    procedure SetAuxByTextLevelName(const Value: string);
    procedure SetByTextPropertyName(const Value: string);
public
   procedure   XORDraw; override;
   procedure   ReDraw(aColor: integer);
   procedure   DrawText(XORMode: boolean);
   procedure   UpdateR(aTA: TGeometryTextAnnotation);
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_StartDrag(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_DragEnd(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_Activate; override;
   procedure RegisterCursors; override;
   procedure PrepareQTree; virtual;
   procedure PrepareAdding;
   procedure FinishAdding;
   procedure CHangeSelectCursor;
   procedure CHangeSetupCursor;
   procedure ChangeAddCursor;
   procedure ChangeEditCursor; // Angle
   procedure CheckFousy(TA: TGeometryTextAnnotation);
   procedure CHeckRectangle(TA: TGeometryTextAnnotation);
   procedure CreateCopy;
   procedure FindProjectEntity(aElem: TElement; var aProjE: TEntity);
   procedure LinkTo;
   function  IsNextCopy: boolean;
   function  IsLinked: boolean;
   procedure SetAlternateStyle;
   property LevelQT: TRectEntityQTree read FLevelQT write SetLevelQT;
   property Level: TEntity read FLevel write SetLevel;
   property ByTextLevelQT: TRectEntityQTree read FByTextLevelQT write SetByTextLevelQT;
   property ByTextLevel: TEntity read FByTextLevel write SetByTextLevel;
   property ByTextPropertyName: string read FByTextPropertyName write SetByTextPropertyName;
   property AuxProjects: TEntity read FAuxProjects write SetAuxProjects;
   property LinkToLevel: TEntity read FLinkToLevel write SetLinkToLevel;
   property X: double read FX write SetX;
   property Y: double read FY write SetY;
   property BGIF: TBGIFont read GetBGIF;
   property TextInfo: TBGIText read GetTextInfo;
   property R: TRectEntity read GetR;
   property Q: TQuickEntity read GetQ;
   property QAdd: TQuickEntity read GetQAdd;
   property DragBusy: boolean read FDragBusy write SetDragBusy;
   property DragFous: TAnnotationAbscissa read FDragFous write SetDragFous;
   property AddingElement: TElement read FAddingElement write SetAddingElement;
   property Disconect: boolean read FDisconect write SetDisconect;
   property StylesList: TIntegerListElement read GetStylesList;
   property StylesList1: TIntegerListElement read GetStylesList1;
   property LinkGTA: TGeometryTextAnnotation read FLinkGTA write SetLinkGTA;
   property OldAngle: double read FOldAngle write SetOldAngle;
published
   property LevelName: string read FLevelName write SetLevelName;
   property ByText: boolean read FByText write SetByText;
   property ByTextLevelName: string read FByTextLevelName write SetByTextLevelName;
   property AuxProjectsName: string read FAuxProjectsName write SetAuxProjectsName;
   property AuxLevelName: string read FAuxLevelName write SetAuxLevelName;
   property AuxByTextLevelName: string read FAuxByTextLevelName write SetAuxByTextLevelName;
   property PixelTolerance: integer read FPixelTolerance write SetPixelTolerance;
   property NoGeometry: boolean read FNoGeometry write SetNoGeometry;
   property Styles: string read FStyles write SetStyles;
   property AltStyles: string read FAltStyles write SetAltStyles;
   property Link: boolean read FLink write SetLink;
   property LinkLevelName: string read FLinkLevelName write SetLinkLevelName;
   property AuxLinkLevelName: string read FAuxLinkLevelName write SetAuxLinkLevelName;
   property LinkDistance: double read FLinkDistance write SetLinkDistance;
end;

implementation
uses
   Classes,
   Controls,
   Dialogs,
   Forms,
   Graphics,
   Worlds,
   cpConvrt,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   MathLib,
   m_Orbis,
   t_QTreeTools;

{ TGeometryTextAnnotationEditor }

procedure TGeometryTextAnnotationEditor.ChangeAddCursor;
var
   CI: integer;
   G: TGeometry;
   i: integer;
   TA: TGeometryTextAnnotation;
   S: string;
   e: TElement;
   GE: TGeometryElement;
begin
   CI := 1;
   if AddingElement <> nil then
   begin
      if TGeometry(AddingElement).IsHitByXY(X,Y,0) then
      begin
         CursorIndex := 3;
         exit;
      end;
   end;
   AddingElement := nil;
   if not Disconect and ByText then
   begin
      TA := TGeometryTextAnnotation(Input);
      QTFinder.Input := ByTextLevelQT;
      QTFinder.X := X;
      QTFInder.Y := Y;
      QTFinder.Tolerance := 0;
      QTFinder.DoIt;
      e := QTFinder.Output;
      if e <> nil then
      if e is TGeometryElement then
      begin
         GE := TGeometryElement(e);
         G := GE.GetSimpleGeometryByXY(X,Y,0);
         if (G <> nil) and not G.IsMyOwner(TA) then
         begin
            S := GE.GetPropertyStringByName(ByTextPropertyName);
            if S = TA.Text then
            begin
               CI := 3;
               AddingElement := G;
            end;
         end;
      end;
   end
   else
   for i := 0 to QAdd.LastIndex do
   begin
      G := TGeometry(QAdd.Elements[i]);
      if G.IsHitByXY(X,Y,0) then
      begin
         CI := 3;
         AddingElement := G;
         break;
      end;
   end;
   CursorIndex := CI;
end;

procedure TGeometryTextAnnotationEditor.ChangeEditCursor;
var
   i: integer;
   TE: TGeometryTextAnnotation;
   A,B: TDoubleCoord;
   L: double;
   Angle: double;
begin
   TE := TGeometryTextAnnotation(Input);
   if TE = nil then exit;
   XORDraw;
   A.X := TE.X;
   A.Y := TE.Y;
   B.X := X;
   B.Y := Y;
   XYToPolar(A,B,L,Angle);
   TE.Angle := Angle;
   XORDRaw;
end;

procedure TGeometryTextAnnotationEditor.CHangeSelectCursor;
var
   d: double;
   CI: integer;
begin
   if QTFinder.Busy then exit;
   d := Device.Projection.GetModelLength(PixelTolerance);
   CI := 0;
   QTFinder.Input := LevelQT;
   QTFinder.X := X;
   QTFInder.Y := Y;
   QTFinder.Tolerance := d;
   QTFinder.DoIt;
   Input := QTFinder.Output;
   LinkGTA := nil;
   if Input <> nil then
   begin
      LinkGTA := TGeometryTextAnnotation(TGeometryTextAnnotation(Input).FindElementClass(TGeometryTextAnnotation));
      CI := 1;
   end;
   CursorIndex := CI;
end;

procedure TGeometryTextAnnotationEditor.CHangeSetupCursor;
var
   AA: TAnnotationAbscissa;
   i: integer;
   TA: TGeometryTextAnnotation;
   e: TElement;
   CI: integer;
   Tol: double;
begin
   if Input = nil then exit;
   DragFous := nil;
   TA := TGeometryTextAnnotation(Input);
   CI := 1;
   Tol := Device.Projection.GetModelLength(5);
   for i := 0 to TA.LastIndex do
   begin
      e := TA.Elements[i];
      if not (e.ClassType = TAnnotationAbscissa) then continue;
      AA := TAnnotationAbscissa(e);
      if InTolerance(AA.X1,AA.Y1,X,Y,Tol)then
      begin
         CI := 2;
         DragFous := AA;
         break;
      end;
   end;
   CursorIndex := CI;
end;

procedure TGeometryTextAnnotationEditor.CheckFousy(TA: TGeometryTextAnnotation);
var
   i,ix: integer;
   G: TGeometry;
   e: TElement;
   X1,X2,Y1,Y2,TAX,TAY: double;
   IsFous,OldFous: boolean;
   AA: TAnnotationAbscissa;
   Rovn,U1,U2: boolean;
   LnkGTA: TGeometryTextAnnotation;
   function Prus: boolean;
   var
      Mix,Miy, Max,May: double;
   begin
      result := false;
      if (LnkGTA <> nil) and (LnkGTA.Visible) then
         Mix := LnkGTA.MinX -5
      else
         Mix := TA.MinX - 5;
      Miy := TA.MinY - 5;
      Max := TA.MaxX + 2;
      May := TA.MaxY + 2;
      PrusecikUsecek(X1,Y1, TAX,TAY, Mix,Miy, Max,Miy, X2,Y2,Rovn,U1,U2);
      result := U1 and U2;
      if result then exit;
      PrusecikUsecek(X1,Y1, TAX,TAY, Max,Miy, Max,May, X2,Y2,Rovn,U1,U2);
      result := U1 and U2;
      if result then exit;
      PrusecikUsecek(X1,Y1, TAX,TAY, Max,May, Mix,May, X2,Y2,Rovn,U1,U2);
      result := U1 and U2;
      if result then exit;
      PrusecikUsecek(X1,Y1, TAX,TAY, Mix,May, Mix,Miy, X2,Y2,Rovn,U1,U2);
      result := U1 and U2;
   end;
begin
   for i := 0 to TA.LastIndex do
   begin
      e := TA.Elements[i];
      if not (e is TGeometry) then continue;
      Q.AddElement(e);
   end;
   LnkGTA := TGeometryTextAnnotation(TA.FindElementClass(TGeometryTextAnnotation));
   TAX := TA.CenterX;
   if (LnkGTA <> nil) and (LnkGTA.Visible) then
      TAX := TAX - (TAX - LnkGTA.CenterX)/2;
   TAY := TA.CenterY;
   X2 := TAX;
   Y2 := TAY;
   for i := 0 to Q.LastIndex do
   begin
      G := TGeometry(Q.Elements[i]);
      X1 := G.X;
      Y1 := G.Y;
      OldFous := false;
      ix := TA.IndexOf(G);
      if (ix > 0) and (TA.Elements[ix-1].ClassType = TAnnotationAbscissa) then
      begin
         AA := TAnnotationAbscissa(TA.Elements[ix-1]);
         X1 := AA.X1;
         Y1 := AA.Y1;
         OldFous := true;
      end;
      if TA.NoFous then
         IsFous := false
      else
         IsFous := not G.IsHitByXY(TAX,TAY,2);
      //IsFous := not G.IsOverRegion(TA.MinX,TA.MinY,TA.MaxX,TA.MaxY);
      if IsFous then
      begin
         if not Prus then
         begin
            if OldFous then TA.FreeElement(TA.Elements[ix-1]);
            continue;
         end;
         if OldFous then
            AA := TAnnotationAbscissa(TA.Elements[ix-1])
         else
         begin
            AA := TAnnotationAbscissa.Create;
            TA.InsertElement(ix,AA);
         end;
         AA.X1 := X1;
         AA.Y1 := Y1;
         AA.X2 := X2;
         AA.Y2 := Y2;
         AA.ResetAndUpdate;
      end
      else
      begin
         if OldFous then TA.FreeElement(TA.Elements[ix-1]);
      end;
   end;
   Q.FreeAll;
end;

procedure TGeometryTextAnnotationEditor.CHeckRectangle(TA: TGeometryTextAnnotation);
var
   AC: TGeometryAnnotationCollection;
   //aStyle: integer;
begin
   AC := TA.GACollection;
   if Device <> nil then
   begin
      if TA.Style > 0 then
         TextInfo.Druh  := TA.Style
      else
         TextInfo.Druh  := AC.Style;
      TextInfo.VychoziBod.x := TA.x;
      TextInfo.VychoziBod.y := TA.y;
      TextInfo.Smer       := TA.Angle;
      TextInfo.Atributy   := [OkT,T30];
      TextInfo.CodePage   := cpLatin2;
      TextInfo.Veta       := WinTo852(TA.Text);
      BGIF.TextInfo := TextInfo;
      BGIF.NastavInfoPodleDruhu(TextInfo.Druh);
      BGIF.CalculateOpsanyObdelnik;

      TA.MinX := TextInfo.Rect.MinX;
      TA.MinY := TextInfo.Rect.MinY;
      TA.MaxX := TextInfo.Rect.MaxX;
      TA.MaxY := TextInfo.Rect.MaxY;
      TA.AbsRefx := TextInfo.VychoziBod.x;
      TA.AbsRefy := TextInfo.VychoziBod.y;
   end;
end;

procedure TGeometryTextAnnotationEditor.CreateCopy;
var
   TAO,TAN: TGeometryTextAnnotation;
   lTAO,lTAN: TGeometryTextAnnotation;
begin
   if Input = nil then exit;
   TAO := TGeometryTextAnnotation(Input);
   TAN := TGeometryTextAnnotation(CreateElementCopy(TAO));
   Redraw(0);
   if not NoGeometry then
      TAN.NextElement := TAO.FindElementClass(TGeometry)
   else
      if not TAO.NoElements then
         TAN.NextElement := TAO.Elements[0];
   TAO.GACollection.NextElement := TAN;
   TAN.DeltaX := 10;
   TAN.DeltaY := 10;
   LevelQT.NextRectEntity := TAN;
   Input := TAN;
   if LinkGTA <> nil then
   begin
      TAO := TGeometryTextAnnotation(LinkGTA);
      TAN := TGeometryTextAnnotation(CreateElementCopy(TAO));
      if not NoGeometry then
         TAN.NextElement := TAO.FindElementClass(TGeometry)
      else
         if not TAO.NoElements then
            TAN.NextElement := TAO.Elements[0];
      TAO.GACollection.NextElement := TAN;
      TAN.DeltaX := 10;
      TAN.DeltaY := 10;
      TGeometryTextAnnotation(Input).NextElement := TAN;
      LinkGTA := TGeometryTextAnnotation(TAN);
   end;
   Redraw($AA00FF);
end;

procedure TGeometryTextAnnotationEditor.DrawText(XORMode: boolean);
var
   TA: TGeometryTextAnnotation;
   GAC: TGeometryAnnotationCollection;
   AA: TAnnotationAbscissa;
   E: TElement;
   aR: double;
   procedure DrawNow;
   var
      i: integer;
   begin
      GAC := TA.GACollection;
      if GAC = nil then
      begin
         PostTextMsg(tm_AddToProtocol,'Annotation Text interpret - Lost collection !',nil);
         exit;
      end;
      try
       //SetInterpretTopProps(TTopoLText(Element).ObjectOwner);
         if TA.Style > 0 then
            TextInfo.Druh       := TA.Style
         else
            TextInfo.Druh       := GAC.Style;
         { vechno podle stylu .....
         TextInfo.Barva      := TE.Barva;
         TextInfo.Znacka     := TE.Znacka;
         TextInfo.Font       := TE.TopolFont;
         TextInfo.Vyska      := TE.Height;
         TextInfo.KoefSirky  := TE.WidthCoef;
         TextInfo.Sklon      := TE.Slope;
         TextInfo.Rozestupy  := TE.Distance;
         TextInfo.RezimTextu := TE.TextMode;
         }
         TextInfo.VychoziBod.x := TA.x;
         TextInfo.VychoziBod.y := TA.y;
         TextInfo.Smer       := TA.Angle;
         TextInfo.Atributy   := [OkT,T30];
         TextInfo.CodePage   := cpLatin2;
         TextInfo.Veta       := WinTo852(TA.Text);
         BGIF.TextInfo := TextInfo;
         BGIF.NastavInfoPodleDruhu(TextInfo.Druh);
         BGIF.CalculateOpsanyObdelnik;
         TextInfo.Atributy := TextInfo.Atributy - [txtZmena];
         Device.Pen.Style := psSolid;
         if XORMode then
         begin
            Device.Pen.Mode:= pmXOR;
            Device.Pen.Color := $909090;
         end
         else
            Device.Pen.Mode := pmCopy;
         BGIF.ZobrazText(False);

      except
      end;
      if GAC.NoGeometry then exit;
      aR := 0;
      if EditorStatus = gesSetup then
         aR:= Device.Projection.GetModelLength(5);
      for i := 0 to TA.LastIndex do
      begin
         e := TA.Elements[i];
         if not (e.ClassType = TAnnotationAbscissa) then continue;
         AA := TAnnotationAbscissa(e);
         Device.MoveTo(AA.X1,AA.Y1);
         Device.LineTo(AA.X2,AA.Y2);
         if EditorStatus = gesSetup then
         begin
            Device.Pen.Width := 1;
            Device.MoveTo(AA.X1+aR, AA.Y1+aR);
            Device.LineTo(AA.X1-aR, AA.Y1+aR);
            Device.LineTo(AA.X1-aR, AA.Y1-aR);
            Device.LineTo(AA.X1+aR, AA.Y1-aR);
            Device.LineTo(AA.X1+aR, AA.Y1+aR);
         end;
      end;
   end;
begin
   if BGIF.CHybaFontu then
   begin
      PostTextMsg(tm_AddToProtocol,'Annotation Text interpret - FontError !',nil);
      exit;
   end;
   if Input = nil then exit;
   TA := TGeometryTextAnnotation(Input);
   DrawNow;
   if LinkGTA = nil then exit;
   if not XORMode and not LinkGTA.Visible then exit;
   TA := LinkGTA;
   DrawNow;
end;

procedure TGeometryTextAnnotationEditor.FindProjectEntity(aElem: TElement; var aProjE: TEntity);
var
   aLevel: TEntity;
begin
   aLevel := Level;
   aProjE := nil;
   if aElem = nil then
      exit;
   if aElem.IsMyOwner(Level)then
      exit;
   aLevel := aElem.FindOwnerName(Level.Name);
   if aLevel = nil then exit;
   aProjE := aLevel.FindClassWithOwnerInOwners(TEntity,AuxProjects);
end;

procedure TGeometryTextAnnotationEditor.FinishAdding;
var
   TA,TAN: TGeometryTextAnnotation;
begin
   if AddingElement = nil then exit;
   if Input = nil then exit;
   if Disconect then
   begin
      TA := TGeometryTextAnnotation(Input);
      TA.NoAnchor := true; // nekotvit!!!
      TA.RemoveElement(AddingElement);
      TA.NoAnchor := false;
      TAN := TGeometryTextAnnotation(CreateElementCopy(TA));
      Redraw(0);
      TA.GACollection.NextElement := TAN;
      TAN.NextElement := AddingElement;
      TAN.X := TGeometry(AddingElement).X;
      TAN.Y := TGeometry(AddingElement).Y;
      LevelQT.NextRectEntity := TAN;
      Input := TAN;
      Redraw($AA00FF);
   end
   else
   begin
      TA := TGeometryTextAnnotation(AddingElement.FindOwnerClass(TGeometryTextAnnotation));
      if TA <> nil then
      begin
         UpdateR(TA);
         TA.NoAnchor := true;  // nekotvit!!!
         TA.RemoveElement(AddingElement);
         TA.NoAnchor := false;
         //PostMsgToOwners(tm_ReviewByElement, TA, false);
      end;
      TA := TGeometryTextAnnotation(Input);
      TA.NextElement := AddingElement;
      CHeckFousy(TA);
   end;
   AddingElement := nil;
   QAdd.FreeAll;
end;

function TGeometryTextAnnotationEditor.GetBGIF: TBGIFont;
begin
   if FBGIF = nil then
   begin
      FBGIF := TBGIFont.Create;
      AddElement(FBGIF);
      if Device <> nil then
         FBGIF.SetDevice(Device);
   end;
   result := FBGIF;
end;

function TGeometryTextAnnotationEditor.GetQ: TQuickEntity;
begin
   if FQ = nil then
   begin
      FQ := TQuickEntity.Create;
      AddElement(FQ);
   end;
   result := FQ;
end;

function TGeometryTextAnnotationEditor.GetQAdd: TQuickEntity;
begin
   if FQAdd = nil then
   begin
      FQAdd := TQuickEntity.Create;
      AddElement(FQAdd);
   end;
   result := FQAdd;
end;

function TGeometryTextAnnotationEditor.GetR: TRectEntity;
begin
   if FR = nil then
   begin
      FR := TRectEntity.Create;
      AddElement(FR);
   end;
   result := FR;
end;

function TGeometryTextAnnotationEditor.GetStylesList: TIntegerListElement;
begin
   if FStylesList = nil then
   begin
      FStylesList := TIntegerListElement.Create;
      AddElement(FStylesList);
   end;
   result := FStylesList;
end;

function TGeometryTextAnnotationEditor.GetStylesList1: TIntegerListElement;
begin
   if FStylesList1 = nil then
   begin
      FStylesList1 := TIntegerListElement.Create;
      AddElement(FStylesList1);
   end;
   result := FStylesList1;
end;

function TGeometryTextAnnotationEditor.GetTextInfo: TBGIText;
begin
   if FTextInfo = nil then
   begin
      FTextInfo := TBGIText.Create;
      AddElement(FTextInfo);
   end;
   result := FTextInfo;
end;

function TGeometryTextAnnotationEditor.IsLinked: boolean;
var
   GTA: TGeometryTextAnnotation;
begin
   result := false;
   GTA := TGeometryTextAnnotation(Input);
   if GTA = nil then exit;
   result := GTA.Linked;
end;

function TGeometryTextAnnotationEditor.IsNextCopy: boolean;
var
   GAC: TGeometryAnnotationCollection;
   GTA,GTA1: TGeometryTextAnnotation;
   En: TEntity;
   i: integer;
   E: TElement;
begin
   result := false;
   GTA := TGeometryTextAnnotation(Input);
   if GTA = nil then exit;
   GAC := GTA.GACollection;
   if GTA.NoElements then exit;
   if not NoGeometry then
      E := GTA.FindElementClass(TGeometry)
   else
      E := GTA.Elements[0];
   for i := 0 to E.OwnersCount -1 do
   begin
      En := E.Owners[i];
      if En = GTA then continue;
      if not (En is TGeometryTextAnnotation) then continue;
      GTA1 := TGeometryTextAnnotation(En);
      if GTA1.GACollection = GAC then
      begin
         result := true;
         exit;
      end;
   end;
end;

procedure TGeometryTextAnnotationEditor.LinkTo;
var
   GTA,LnkGTA: TGeometryTextAnnotation;
   G: TGeometry;
   i: integer;
   e: TElement;
   aLinkToLevel: TEntity;
   aProjE: TEntity;
begin
   GTA := TGeometryTextAnnotation(Input);
   G := TGeometry(GTA.FindElementClass(TGeometry));
   if G = nil then exit;
   FindProjectEntity(Input,aProjE);
   if aProjE = nil then
      aLinkToLevel := LinkToLevel
   else
      aLinkToLevel := FindLevelInEntity(aProjE,AuxLinkLevelName);
   LnkGTA := TGeometryTextAnnotation(G.FindClassWithOwnerInOwners(TGeometryTextAnnotation,aLinkToLevel));
   if GTA.Linked then
   begin
      GTA.UnLink := true;
      if LnkGTA = nil then exit;
      CheckFousy(LnkGTA);
      PostMsgToOwners(tm_ReviewByElement, R, false);
   end
   else
   begin
      if LnkGTA = nil then exit;
      GTA.X := LnkGTA.X - GTA.Width - LinkDistance;
      GTA.Y := LnkGTA.Y;
      if not GTA.IsMyOwner(LnkGTA) then
         LnkGTA.NextElement := GTA;
      GTA.NoFous := true;
      CHeckFousy(LnkGTA);
      CheckFousy(GTA);
      UpdateR(LinkGTA);
      PostMsgToOwners(tm_ReviewByElement, R, false);
   end;
end;

procedure TGeometryTextAnnotationEditor.On_Activate;
var
   E: TElement;
begin
   EditorStatus := gesPrepare;
   HighActivity := true;
   if not Active then
      PrepareQTree;
   Link := Link and (LinkToLevel <> nil);
   EditorStatus := gesSelect;
   QTFinder.Busy := false;
   inherited;
{ TODO -oPPK -cOpravit : nastaven linkdistance, tohle je nouzovka }   
   // tohle je drsna nouzovka
   LinkDistance := 2;
   e := World.ObjectModel.FindElementName('Setup');
   if e <> nil then
   begin
      e := TEntity(e).FindElementName('Annotation');
      if e <> nil then
      begin
         e := TEntity(e).FindElementName('LinkDistance');
         if (e <> nil) and (e is TStringElement) then
            LinkDistance := TStringElement(e).AsFloat; 
      end;
   end;
   // konec nouzovky
   HighActivity := false;
end;

procedure TGeometryTextAnnotationEditor.On_tm_DragEnd(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   GAC: TGeometryAnnotationCollection;
   TA,LTA: TGeometryTextAnnotation;
   aProjE: TEntity;
begin
   //ShowMessage('DragEnd');
   //XORDraw;
   if Input = nil then exit;
   LTA := nil;
   CursorIndex := 0;
   if EditorStatus = gesSetup then
   begin
      XORDraw;
      EditorStatus := gesSelect;
      //Redraw(0);
      DragBusy := false;
      UpdateR(TGeometryTextAnnotation(Input));
      //exit;
   end
   else
   begin
      TA := TGeometryTextAnnotation(input);
      if TA.Linked then
      begin
         LTA := TGeometryTextAnnotation(TA.FindOwnerClass(TGeometryTextAnnotation));
         TA.UnLink := true;
         CheckFousy(LTA);
         UpdateR(LTA);
      end;
      GAC := TGeometryTextAnnotation(Input).GACollection;
      RemoveElementOwners(Input);
      if GAC <> nil then
         GAC.NextElement := Input;
      if LevelQT <> nil then
         LevelQT.NextRectEntity := TRectEntity(Input);
      UpdateR(TA);
   end;
   XORDraw;
   FindProjectEntity(Input, aProjE);
   PostMsg(tm_ModelChanged,nil,aProjE,false);
   PostMsgToOwners(tm_ReviewByElement, R, false);
   //Application.ProcessMessages;
   //Application.ProcessMessages;

   //Redraw(0);
   DragBusy := false;
end;

procedure TGeometryTextAnnotationEditor.On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   aX,aY: double;
   TA: TGeometryTextAnnotation;
   G: TGeometry;
   ix: integer;
   e: TElement;
begin
   if DragBusy then exit;
   DragBusy := true;
   aX:= Device.Projection.GetModelX(aMessage.X);
   aY:= Device.Projection.GetModelY(aMessage.Y);
   TA := TGeometryTextAnnotation(Input);
   if TA = nil then exit;
   if EditorStatus = gesSelect then
   begin
      XORDraw;
      TA.X := TA.X + aX - X;
      TA.Y := TA.Y + aY - Y;
      if LinkGTA <> nil then
      begin
         LinkGTA.X := LinkGTA.X + aX - X;
         LinkGTA.Y := LinkGTA.Y + aY - Y;
      end;
      if not NoGeometry then
         CheckFousy(TA);
      XORDraw;
   end
   else
   if EditorStatus = gesSetup then
   begin
      if DragFous = nil then exit;
      XORDraw;
      ix  := TA.IndexOf(DragFous);
      e := TA.Elements[ix+1];
      if e is TGeometry then
      begin
         G := TGeometry(e);
         if G.IsHitByXY(aX,aY,0) then
         begin
            DragFous.X1 := aX;
            DragFous.Y1 := aY;
            G.AbsRefX := aX;
            G.AbsRefY := aY;
            CheckFousy(TA);
         end;
      end;
      XORDraw;
   end;
   X := aX;
   Y := aY;
   DragBusy := false;
end;

procedure TGeometryTextAnnotationEditor.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   i: integer;
   e: TElement;
   GTA: TGeometryTextAnnotation;
begin
   if Input = nil then exit;
   if EditorStatus = gesEdit then exit;
   R.Rect.Reset;
   UpdateR(TGeometryTextAnnotation(Input));
   if not NoGeometry then
   begin
      for i := 0 to TEntity(Input).LastIndex do
      begin
         e := TEntity(Input).Elements[i];
         if (e is TGeometryTextAnnotation) then
         begin
            R.UpdateRect(TGeometryTextAnnotation(e));
            LinkGTA := TGeometryTextAnnotation(e);
         end;
      end;
      if TGeometryTextAnnotation(Input).Linked then
      begin
         GTA := TGeometryTextAnnotation(Input.FindOwnerClass(TGeometryTextAnnotation));
         UpdateR(GTA);
      end;
   end;
   Redraw($A0A0A0);
   XORDraw;
end;

procedure TGeometryTextAnnotationEditor.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
  aX, aY: integer;
begin
   aX:= aMessage.x;
   aY:= aMessage.y;
   X:= Device.Projection.GetModelX(aX);
   Y:= Device.Projection.GetModelY(aY);
   if not Active then exit;
   case EditorStatus of
      gesSelect: ChangeSelectCursor;
      gesSetup : CHangeSetupCursor;
      gesAdd   : CHangeAddCursor;
      gesEdit  : ChangeEditCursor;
   end;
   {
   else
   begin
      if PolyElement = nil then exit;
      ChangeCursor;
   end;
   }
end;

procedure TGeometryTextAnnotationEditor.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   FF: boolean;
   FA: boolean;
   aProjE: TEntity;
   GAC: TGeometryAnnotationCollection;
begin
   FF := false;
   FA := false;
   if Input = nil then exit;
   XORDraw;
   if EditorStatus = gesAdd then
   begin
      if (aMessage.Button = mbLeft) then
      begin
         FinishAdding;
         FindProjectEntity(Input, aProjE);
         PostMsg(tm_ModelChanged,nil,aProjE,false);
      end
      else
      begin
         AddingElement := nil;
         QAdd.FreeAll;
      end;
      UpdateR(TGeometryTextAnnotation(Input));
      PostMsgToOwners(tm_ReviewByElement, R, false);
      EditorStatus := gesSelect;
      exit;
   end;
   if EditorStatus = gesEdit then
   begin
      if (aMessage.Button = mbRight) then
      begin
         TGeometryTextAnnotation(Input).Angle := OldAngle;
         aMessage.Button := mbLeft;
         //EditorStatus := gesSelect;
         //exit;
      end
      else
      begin
         CHeckRectangle(TGeometryTextAnnotation(Input));
         CheckFousy(TGeometryTextAnnotation(Input));
         //UpdateR(TGeometryTextAnnotation(Input));
         //PostMsgToOwners(tm_ReviewByElement, R, false);
         GAC := TGeometryTextAnnotation(Input).GACollection;
         RemoveElementOwners(Input);
         if GAC <> nil then
            GAC.NextElement := Input;
         if LevelQT <> nil then
            LevelQT.NextRectEntity := TRectEntity(Input);
         FindProjectEntity(Input, aProjE);
         PostMsg(tm_ModelChanged,nil,aProjE,false);
      end;
      EditorStatus := gesSelect;
      UpdateR(TGeometryTextAnnotation(Input));
      PostMsgToOwners(tm_ReviewByElement, R, false);
      exit;
   end;
   EditorStatus := gesSelect;
   XORDraw;
   if (aMessage.Button = mbRight) and (Dialog <> nil) then
   begin
      FF := not TGeometryTextAnnotation(Input).NoFous;
      FA := TGeometryTextAnnotation(Input).Angle = 0;
      Dialog.SetPropertyBooleanByName('IsVisible',TGeometryTextAnnotation(Input).Visible);
      Dialog.SetPropertyBooleanByName('Fous',FF);
      Dialog.SetPropertyBooleanByName('NoGeometry',NoGeometry);
      Dialog.SetPropertyBooleanByName('CanBeDeleted',IsNextCopy);
      Dialog.SetPropertyBooleanByName('CanBeLinked',Link);
      Dialog.SetPropertyBooleanByName('Link',not IsLinked);
      Dialog.SetPropertyBooleanByName('ZeroAngle',FA);
      Dialog.DoIt;
      case Dialog.Result of
         1: // OK
         begin
            TGeometryTextAnnotation(Input).Visible := Dialog.GetPropertyBooleanByName('IsVisible');
            if LinkGTA <> nil then LinkGTA.Visible := TGeometryTextAnnotation(Input).Visible;
            FF := not Dialog.GetPropertyBooleanByName('Fous');
            if TGeometryTextAnnotation(Input).NoFous <> FF then
            begin
               TGeometryTextAnnotation(Input).NoFous := FF;
               CheckFousy(TGeometryTextAnnotation(Input));
               PostMsgToOwners(tm_ReviewByElement, R, false);
            end;
            FA := Dialog.GetPropertyBooleanByName('ZeroAngle');
            if FA and (TGeometryTextAnnotation(Input).Angle <> 0) then
            begin
               TGeometryTextAnnotation(Input).Angle := 0;
               CHeckRectangle(TGeometryTextAnnotation(Input));
               CheckFousy(TGeometryTextAnnotation(Input));
               PostMsgToOwners(tm_ReviewByElement, R, false);
               GAC := TGeometryTextAnnotation(Input).GACollection;
               RemoveElementOwners(Input);
               if GAC <> nil then
                  GAC.NextElement := Input;
               if LevelQT <> nil then
                  LevelQT.NextRectEntity := TRectEntity(Input);
            end;
            // styly a dalsi ptakoviny
            FindProjectEntity(Input, aProjE);
            PostMsg(tm_ModelChanged,nil,aProjE,false);
         end;
         3: // posun fousu
         begin
            XORDraw;
            EditorStatus := gesSetup;
            XORDraw;
            exit;
         end;
         4: // pripojeni dalsi plochy
         begin
            Disconect := false;
            PrepareAdding;
            if not ByText and QAdd.NoElements then
            begin
               ShowMessage('Nen co pidvat!!!');
            end
            else
            begin
               EditorStatus := gesAdd;
               exit;
            end;
         end;
         5: // kopie
         begin
            CreateCopy;
            FindProjectEntity(Input, aProjE);
            PostMsg(tm_ModelChanged,nil,aProjE,false);
            exit;
         end;
         6: // Odpojeni
         begin
            Disconect := True;
            PrepareAdding;
            if QAdd.NoElements then
            begin
               ShowMessage('Nen co odpojit, zbyla jen jedna plocha!!!');
            end
            else
            begin
               EditorStatus := gesAdd;
               exit;
            end;
         end;
         7: // Zruseni
         begin
            FindProjectEntity(Input, aProjE);
            Input.Free;
            Input := nil;
            PostMsgToOwners(tm_ReviewByElement, R, false);
            PostMsg(tm_ModelChanged,nil,aProjE,false);
            exit;
         end;
         8: // Link
         begin
            LinkTo;
         end;
         9: // Uhel
         begin
            OldAngle := TGeometryTextAnnotation(Input).Angle;
            EditorStatus := gesEdit;
            //XORDraw;
            exit;
         end;
      end;
   end;
   if not NoGeometry and (ssCtrl in aMessage.Shift) then
   begin
      if ssAlt in aMessage.Shift then
      begin
         TGeometryTextAnnotation(Input).AltText := not TGeometryTextAnnotation(Input).AltText;
         CHeckRectangle(TGeometryTextAnnotation(Input));
         CheckFousy(TGeometryTextAnnotation(Input));
         UpdateR(TGeometryTextAnnotation(Input));
      end
      else
         TGeometryTextAnnotation(Input).Visible := not TGeometryTextAnnotation(Input).Visible;
      FindProjectEntity(Input, aProjE);
      PostMsg(tm_ModelChanged,nil,aProjE,false);
      PostMsgToOwners(tm_ReviewByElement, R, false);
   end
   else
   if not NoGeometry and (ssShift in aMessage.Shift) then
   begin
      XORDraw;
      EditorStatus := gesSetup;
      XORDraw;
      exit;
   end
   else
   if not NoGeometry and (ssAlt in aMessage.Shift) then
   begin
      XORDraw;
      SetAlternateStyle;
      //XORDraw;
      FindProjectEntity(Input, aProjE);
      PostMsg(tm_ModelChanged,nil,aProjE,false);
      exit;
   end
   else
      PostMsgToOwners(tm_ReviewByElement, R, false)
   {
   if not TGeometryTextAnnotation(Input).Visible then
      PostMsgToOwners(tm_ReviewByElement, R, false)
   else
      XORDraw; //????
      Redraw(0);
    }
   //ShowMessage('UP');
end;

procedure TGeometryTextAnnotationEditor.On_tm_StartDrag(
  var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
  inherited;

end;

procedure TGeometryTextAnnotationEditor.PrepareAdding;
var
   TA: TGeometryTextAnnotation;
   GAC: TGeometryAnnotationCollection;
   EC: TElementClass;
   i: integer;
   G: TGeometry;
   GE: TGeometryElement;
   MA: TMultiAreaElement;
   e: TElement;
begin
   AddingElement := nil;
   QAdd.FreeAll;
   if Input = nil then exit;
   TA := TGeometryTextAnnotation(input);
   if Disconect then
   begin
      for i := 0 to TA.LastIndex do
      begin
         e := TA.Elements[i];
         if not (e.ClassType = TAnnotationAbscissa) then
            QAdd.AddElement(e);
      end;
      if QAdd.Count < 2 then
         QAdd.FreeAll;
   end
   else
   if ByText then
   begin
      ByTextPropertyName := '';
      GAC := TA.GACollection;
      if GAC = nil then exit;
      ByTextPropertyName := GAC.PropertyName;
   end
   else
   begin
      GAC := TA.GACollection;
      if GAC = nil then exit;
      EC := GAC.ElementClass;
      if EC = nil then exit;
      E := TA.FindElementClass(TGeometry);
      if E = nil then exit;
      E := E.FindClassInOwners(EC);
      if E = nil then exit;
      if not (E is TGeometryElement) then exit;
      GE := TGeometryElement(E);
      G := GE.Geometry;
      if G = nil then exit;
      if not (G is TMultiAreaElement) then exit;
      MA := TMultiAreaElement(G);
      For i := 0 to MA.LastIndex do
      begin
         e := MA.Elements[i];
         if e.IsMyOwner(TA) then continue;
         QAdd.AddElement(e);
      end;
   end;
end;

procedure TGeometryTextAnnotationEditor.PrepareQTree;
var
   TQ: TAddEntityToQTree;
   e: TElement;
   En: TEntity;
   En1: TEntity;
   RE: TRectEntity;
   ii: integer;
begin
   if Level = nil then exit;
   RE := TRectEntity.Create;
   AddElement(RE);
   RE.AddAllFrom := Level;
   if AuxProjectsName <> '' then
   begin
      AuxProjects := FindLevel(AuxProjectsName);
      if AuxProjects <> nil then
      for ii  := 0 to AuxProjects.LastIndex do
      begin
         e := AuxProjects.Elements[ii];
         if not (e is TEntity) then continue;
         En := TEntity(e);
         En1 := FindLevelInEntity(En,AuxLevelName);
         if En1 <> nil then
            RE.AddAllFrom := En1;
      end;
   end;
   RE.Update;
   StartProgress('Pprava vrstev...',0,0);
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := RE;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
   RE.Free;
   EndProgress;
   if ByText then
   begin
      if ByTextLevel = nil then
      begin
         ByText := false;
         exit;
      end;
      RE := TRectEntity.Create;
      AddElement(RE);
      RE.AddAllFrom := ByTextLevel;
      if AuxProjects <> nil then
      for ii  := 0 to AuxProjects.LastIndex do
      begin
         e := AuxProjects.Elements[ii];
         if not (e is TEntity) then continue;
         En := TEntity(e);
         En1 := FindLevelInEntity(En,AuxByTextLevelName);
         if En1 <> nil then
            RE.AddAllFrom := En1;
      end;
      RE.Update;
      StartProgress('Pprava ByText...',0,0);
      ByTextLevelQT :=TRectEntityQTree.Create;
      ByTextLevelQT.Name := 'ByTextLEVEL QT';
      TQ := TAddEntityToQTree.Create;
      TQ.Input := RE;
      TQ.Output := ByTextLevelQT;
      TQ.DoIt;
      TQ.Free;
      RE.Free;
      EndProgress;
   end;
end;

procedure TGeometryTextAnnotationEditor.ReDraw(aColor: integer);
begin
   Device.Pen.Color := aColor;
   DrawText(false);
end;

procedure TGeometryTextAnnotationEditor.RegisterCursors;
begin
   NewCursorName('NOSELECT');
   NewCursorName('EDITTEXT');
   NewCursorName('MOVE_POINT');
   NewCursorName('SELECTAREANEW');
end;

procedure TGeometryTextAnnotationEditor.SetAddingElement(
  const Value: TElement);
begin
  FAddingElement := Value;
end;

procedure TGeometryTextAnnotationEditor.SetAlternateStyle;
var
   TA: TGeometryTextAnnotation;
   aStyle: integer;
   NewStyle: integer;
   isAltStyle: boolean;
   iStyle: integer;
   i: integer;
   exists: boolean;
begin
   if Input = nil then exit;
   if NoGeometry then exit;
   TA := TGeometryTextAnnotation(Input);
   aStyle := TA.Style;
   if aStyle = 0 then
      aStyle := TA.GACollection.Style;
   if StylesList.NoValue then exit;
   if StylesList1.NoValue then exit;
   isAltStyle := false;
   Exists := false;
   iStyle := -1;
   NewStyle := aStyle;
   for i := 0 to StylesList.LastIndex do
   begin
      if aStyle = StylesList.Integers[i] then
      begin
         iStyle := i;
         Exists := true;
         break;
      end;
   end;
   if not Exists then
   for i := 0 to StylesList1.LastIndex do
   begin
      if aStyle = StylesList1.Integers[i] then
      begin
         iStyle := i;
         isAltStyle := true;
         Exists := true;
         break;
      end;
   end;
   if Exists then
   begin
      if isAltStyle then
      begin
         if iStyle <= StylesList.LastIndex then
            NewStyle := StylesList.Integers[iStyle];
      end
      else
      begin
         if iStyle <= StylesList1.LastIndex then
            NewStyle := StylesList1.Integers[iStyle];
      end;
      if NewStyle <> aStyle then
      begin
         TA.Style := NewStyle;
         CHeckRectangle(TA);
         CheckFousy(TA);
         UpdateR(TA);
      end;
      PostMsgToOwners(tm_ReviewByElement, R, false);
   end;
   //Redraw(0);
end;

procedure TGeometryTextAnnotationEditor.SetAltStyles(const Value: string);
begin
  FAltStyles := Value;
  StylesList1.FreeAll;
  StylesList1.ValuesString := Value;
end;

procedure TGeometryTextAnnotationEditor.SetAuxByTextLevelName(const Value: string);
begin
  FAuxByTextLevelName := Value;
end;

procedure TGeometryTextAnnotationEditor.SetAuxLevelName(const Value: string);
begin
  FAuxLevelName := Value;
end;

procedure TGeometryTextAnnotationEditor.SetAuxLinkLevelName(const Value: string);
begin
  FAuxLinkLevelName := Value;
end;

procedure TGeometryTextAnnotationEditor.SetAuxProjects(const Value: TEntity);
begin
  FAuxProjects := Value;
end;

procedure TGeometryTextAnnotationEditor.SetAuxProjectsName(const Value: string);
begin
  FAuxProjectsName := Value;
end;

procedure TGeometryTextAnnotationEditor.SetByText(const Value: boolean);
begin
  FByText := Value;
end;

procedure TGeometryTextAnnotationEditor.SetByTextLevel(
  const Value: TEntity);
begin
  FByTextLevel := Value;
end;

procedure TGeometryTextAnnotationEditor.SetByTextLevelName(const Value: string);
begin
   FByTextLevelName := Value;
   ByTextLevel := FindLevel(Value);
end;

procedure TGeometryTextAnnotationEditor.SetByTextLevelQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FByTextLevelQT), Value);
end;

procedure TGeometryTextAnnotationEditor.SetByTextPropertyName(
  const Value: string);
begin
  FByTextPropertyName := Value;
end;

procedure TGeometryTextAnnotationEditor.SetDisconect(const Value: boolean);
begin
  FDisconect := Value;
end;

procedure TGeometryTextAnnotationEditor.SetDragBusy(const Value: boolean);
begin
  FDragBusy := Value;
end;

procedure TGeometryTextAnnotationEditor.SetDragFous(
  const Value: TAnnotationAbscissa);
begin
  FDragFous := Value;
end;

procedure TGeometryTextAnnotationEditor.SetLevel(const Value: TEntity);
begin
  FLevel := Value;
end;

procedure TGeometryTextAnnotationEditor.SetLevelName(const Value: string);
begin
   FLevelName := Value;
   Level := FindLevel(Value);
end;

procedure TGeometryTextAnnotationEditor.SetLevelQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLevelQT), Value);
end;

procedure TGeometryTextAnnotationEditor.SetLink(const Value: boolean);
begin
  FLink := Value;
end;

procedure TGeometryTextAnnotationEditor.SetLinkDistance(
  const Value: double);
begin
  FLinkDistance := Value;
end;

procedure TGeometryTextAnnotationEditor.SetLinkGTA(const Value: TGeometryTextAnnotation);
begin
  FLinkGTA := Value;
end;

procedure TGeometryTextAnnotationEditor.SetLinkLevelName(const Value: string);
begin
   FLinkLevelName := Value;
   LinkToLevel := FindLevel(Value);
end;

procedure TGeometryTextAnnotationEditor.SetLinkToLevel(const Value: TEntity);
begin
  FLinkToLevel := Value;
end;

procedure TGeometryTextAnnotationEditor.SetNoGeometry(const Value: boolean);
begin
  FNoGeometry := Value;
end;

procedure TGeometryTextAnnotationEditor.SetOldAngle(const Value: double);
begin
  FOldAngle := Value;
end;

procedure TGeometryTextAnnotationEditor.SetPixelTolerance(const Value: integer);
begin
  FPixelTolerance := Value;
end;

procedure TGeometryTextAnnotationEditor.SetStyles(const Value: string);
begin
  FStyles := Value;
  StylesList.FreeAll;
  StylesList.ValuesString := Value;
end;

procedure TGeometryTextAnnotationEditor.SetX(const Value: double);
begin
  FX := Value;
end;

procedure TGeometryTextAnnotationEditor.SetY(const Value: double);
begin
  FY := Value;
end;


procedure TGeometryTextAnnotationEditor.UpdateR(aTA: TGeometryTextAnnotation);
var
   i: integer;
   e: TElement;
begin
   if aTA = nil then exit;
   R.UpdateRect(aTA);
   for i := 0 to aTA.LastIndex do
   begin
      e := aTA.Elements[i];
      if e.ClassType = TAnnotationAbscissa  then
         R.UpdateRect(e);
   end;
end;

procedure TGeometryTextAnnotationEditor.XORDraw;
begin
   DrawText(true);
end;

INITIALIZATION
   RegisterOrbisClass(TGeometryTextAnnotationEditor);
END.
