unit t_CreateSKPGeometry;

interface
uses
   e_KatastrModel,
   Elements,
   Tools;
type

TCreateSKPGeometry = class(TTool)
  private
    FSKPColor: integer;
    FComment: string;
    FLIN_PAR: TEntity;
    FARE_SKP: TEntity;
    FLIN_SKP: TEntity;
    FParcelModel: Tgis_ParcelModel;
    FFreeEmptySKP: boolean;
    FUseParZar: boolean;
    procedure SetARE_SKP(const Value: TEntity);
    procedure SetComment(const Value: string);
    procedure SetLIN_PAR(const Value: TEntity);
    procedure SetLIN_SKP(const Value: TEntity);
    procedure SetParcelModel(const Value: Tgis_ParcelModel);
    procedure SetSKPColor(const Value: integer);
    procedure SetFreeEmptySKP(const Value: boolean);
    procedure SetUseParZar(const Value: boolean);
public
   procedure DoIt; override;
   procedure ResetAll;
published
   property  LIN_PAR: TEntity read FLIN_PAR write SetLIN_PAR stored false;
   property  ARE_SKP: TEntity read FARE_SKP write SetARE_SKP stored false;
   property  LIN_SKP: TEntity read FLIN_SKP write SetLIN_SKP stored false;
   property  ParcelModel: Tgis_ParcelModel read FParcelModel write SetParcelModel stored false;
   property  Comment: string read FComment write SetComment;
   property  SKPColor: integer read FSKPColor write SetSKPColor;
   property  FreeEmptySKP: boolean read FFreeEmptySKP write SetFreeEmptySKP;
   property  UseParZar: boolean read FUseParZar write SetUseParZar;
end;

implementation
uses
   Dialogs,
   Forms,
   SysUtils,
   e_Class,
   e_Geometry,
   e_TopolElements,
   g_Elems,
   m_Orbis,
   t_AreMak,
   t_ConfirmDialog,
   t_CopyLinesFromTo,
   t_ParcelModelSelector,
   t_ParcelyTestTools;

{ TCreateSKPGeometry }

procedure TCreateSKPGeometry.DoIt;
var
   i,j: integer;
   T: THraniceSKPTestTool;
   e: TElement;
   TL: TTopolLineElement;
   L: TPolyLineElement;
   ACL: TAreasFromClearLines;
   En : TEntity;
   PMS: TParcelModelSelector;
   Are: TPolyLineAreaElement;
   K: Tgis_Katastr;
   SKP,SKPN: Tgis_SkupinaParcel;
   P: Tgis_ParcelaCast;
   OE: TOwnerEntity;
   TCL: TCopyLinesFromTo;
   S: string;
begin
   Error := 0;
   if LIN_PAR = nil then Error := 1;
   if ARE_SKP = nil then Error := 2;
   if LIN_SKP = nil then Error := 3;
   if ParcelModel = nil then Error := 4;
   if Error > 0 then
   begin
      case Error of
         1: S := 'LIN_PAR';
         2: S := 'ARE_SKP';
         3: S := 'LIN_SKP';
         4: S := 'Model parcel';
      end;
      ShowMessage(Comment+#13+#10+'Povinn parametr '+S+' chyb');
      exit;
   end;
   if not (ParcelModel is Tgis_ParcelModel) or ParcelModel.NoElements then
   begin
      ShowMessage(Comment+#13+#10+'Model parcel nen modelem parcel nebo neobsahuje dn katastry!');
      exit;
   end;
   if not LIN_SKP.NoElements then
   begin
      if not Confirm('Skupiny parcel u jakousi geometrii maj, ta bude zruena a nahrazena novou. Pokraovat?') then
         exit;
   end;
   StartProgress(Comment,0,0);
   ResetAll;
   if LIN_PAR = nil then exit;
   ProgressText := 'Penos lini z parcel...';
   MaxProgress := LIN_PAR.LastIndex;
   Progress := 0;
   Application.ProcessMessages;
   T := THraniceSKPTestTool.Create;
   T.UseParZar := UseParZar;
   OE := TOwnerEntity.Create;
   for i := 0 to LIN_PAR.LastIndex do
   begin
      if (i mod 50) = 0  then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := LIN_PAR.Elements[i];
      if not(e is TTopolLineElement) then continue;
      TL := TTopolLineElement(e);
      L := TPolyLineElement(TL.Geometry);
      T.Input := L;
      T.DoIt;
      If not T.Result then
         OE.AddElement(TL);
   end;
   Progress := 0;
   MaxProgress := 0;
   ProgressText := 'kopie lini...';
   Application.ProcessMessages;
   TCL := TCopyLinesFromTo.Create;
   TCL.Input := OE;
   TCL.Output := LIN_SKP;
   TCL.DoIt;
   TCL.Free;
   OE.Free;
   {
   if (LIN_SKP is TRectEntity) then
      TRectEntity(LIN_SKP).Update;
   }
   ProgressText := 'Tvorba ploch skupin ...';
   Application.ProcessMessages;
   ACL := TAreasFromClearLines.Create;
   ACL.Input := LIN_SKP;
   ACL.DoIt;
   ParcelModel.UpdateAll := true;
   PMS := TParcelModelSelector.Create;
   PMS.SelectByParcely := true;
   PMS.FindInput;
   En := TEntity(ACL.Output);
   for i := 0 to En.LastIndex do
   begin
      e := En.Elements[i];
      if not (e is TPolyLineAreaElement) then continue;
      Are := TPolyLineAreaElement(e);
      PMS.X := Are.X;
      PMS.Y := Are.Y;
      PMS.DoIt;
      SKP := PMS.ActualSkupinaParcel;
      P   := PMS.ActualParCast;
      if UseParZar And (P <> nil) and not P.ParZar then
         SKP := nil;
      if SKP <> nil then
      begin
         SKP.AddGeometry(Are);
         if P <> nil then
            SKP.KATPAR_KOD := P.KATPAR_KOD;
      end;
   end;
   ParcelModel.UpdateAll := true;
   En.Free;
   ACL.Free;
   PMS.Free;
   ShowMessage('Operace se podaila (patrn)...');
   PostMsg(tm_ModelChanged,nil,nil,false);
end;

procedure TCreateSKPGeometry.ResetAll;
var
   K: Tgis_Katastr;
   SP: Tgis_SkupinaParcel;
   P: Tgis_Parcela;
   PC: Tgis_ParcelaCast;
   i,j,pci: integer;
   S,SN: string;
   E: TElement;
begin
   Progress := 0;
   S := 'Ruen stvajcch skupin';
   ProgressText := S+' Plochy ...';
   Application.ProcessMessages;
   ARE_SKP.FreeAll;
   ProgressText := S+' Linie ...';
   Application.ProcessMessages;
   LIN_SKP.FreeAll;
   //MaxProgress := ParcelModel.LastIndex;
   S := 'Inicializace skupin: ';
   ProgressText := S;
   Progress := 0;
   Application.ProcessMessages;
   for i := 0 to ParcelModel.LastIndex do
   begin
      E := ParcelModel.Elements[i];
      if not (e is Tgis_Katastr) then continue;
      K := Tgis_Katastr(E);
      ParcelModel.ActualKatastr := K;
      ProgressText := S+'katastr '+ IntToStr(i)+'/'+IntToStr(ParcelModel.LastIndex)+' .'+K.Name;
      if UseParZar and not K.NoParcela then
      for j := 0 to K.Parcely.LastIndex do
      begin
         e := K.Parcely.Elements[j];
         if not (e is Tgis_Parcela) then continue;
         P := Tgis_Parcela(e);
         if not P.NoElements then
         for pci := 0 to P.LastIndex do
         begin
            e := P.Elements[pci];
            if not (e is Tgis_ParcelaCast) then continue;
            PC := Tgis_ParcelaCast(e);
            if not PC.ParZar then
            begin
               ParcelModel.CreateSkupinaParcelCislo(0);
               PC.SkupinaParcel := ParcelModel.ActualSkupinaParcel;
            end;
         end;
      end;
      if K.NoSkupinaParcel then continue;
      for j := K.SkupinyParcel.LastIndex downto 0 do
      begin
         E := K.SkupinyParcel.Elements[j];
         if not (E is Tgis_SkupinaParcel) then Continue;
         SP := Tgis_SkupinaParcel(E);
         SP.Geometry := nil;
         if SP.NoParcela and SP.NoParcast and FreeEmptySKP then
         begin
            SP.Free;
         end
         else
         begin
            SP.Barva := SKPColor;
            SP.Znacka := 1;
            ARE_SKP.AddElement(SP);
         end;
      end;
   end;
   if (ARE_SKP is TRectEntity) then
      TRectEntity(ARE_SKP).Update;
end;

procedure TCreateSKPGeometry.SetARE_SKP(const Value: TEntity);
begin
  FARE_SKP := Value;
end;

procedure TCreateSKPGeometry.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCreateSKPGeometry.SetFreeEmptySKP(const Value: boolean);
begin
  FFreeEmptySKP := Value;
end;

procedure TCreateSKPGeometry.SetLIN_PAR(const Value: TEntity);
begin
  FLIN_PAR := Value;
end;

procedure TCreateSKPGeometry.SetLIN_SKP(const Value: TEntity);
begin
  FLIN_SKP := Value;
end;

procedure TCreateSKPGeometry.SetParcelModel(const Value: Tgis_ParcelModel);
begin
  FParcelModel := Value;
end;

procedure TCreateSKPGeometry.SetSKPColor(const Value: integer);
begin
  FSKPColor := Value;
end;

procedure TCreateSKPGeometry.SetUseParZar(const Value: boolean);
begin
  FUseParZar := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCreateSKPGeometry);
END.
