unit t_CreateSKPfromPAR;

interface
uses
   e_KatastrModel,
   Elements,
   e_QTree,
   Tools;

type

TCreateSKPfromPAR = class(TTool)
  private
    FComment: string;
    FLIN_SKP: TEntity;
    FARE_PAR: TEntity;
    FARE_SKP: TEntity;
    FParcelModel: Tgis_ParcelModel;
    FFrameAreas: TEntity;
    FQT: TRectEntityQTree;
    FLIN_PAR: TEntity;
    FSKPColor: integer;
    FSimpleKatparMode: boolean;
    FSolveParZar: boolean;
    FUseKATPAR_KOD: boolean;
    FUseSuperID: boolean;
    FUsePUPFL: boolean;
    FResetParZar: boolean;
    FParzarByKATPAR_KOD13: boolean;
    FUsePUPFLc: boolean;
    FByPropertyList: string;
    FNumberByProperty: string;
    procedure SetARE_PAR(const Value: TEntity);
    procedure SetARE_SKP(const Value: TEntity);
    procedure SetComment(const Value: string);
    procedure SetLIN_SKP(const Value: TEntity);
    procedure SetParcelModel(const Value: Tgis_ParcelModel);
    procedure SetFrameAreas(const Value: TEntity);
    procedure SetQT(const Value: TRectEntityQTree);
    procedure SetLIN_PAR(const Value: TEntity);
    procedure SetSKPColor(const Value: integer);
    procedure SetSimpleKatparMode(const Value: boolean);
    procedure SetSolveParZar(const Value: boolean);
    procedure SetUseKATPAR_KOD(const Value: boolean);
    procedure SetUseSuperID(const Value: boolean);
    procedure SetUsePUPFL(const Value: boolean);
    procedure SetResetParZar(const Value: boolean);
    procedure SetParzarByKATPAR_KOD13(const Value: boolean);
    procedure SetUsePUPFLc(const Value: boolean);
    procedure SetByPropertyList(const Value: string);
    procedure SetNumberByProperty(const Value: string);
public
   procedure DoIt; override;
   function  CheckParcelyHierarchy: boolean;
   function  ResetSkupinyParcel: boolean;
   procedure FindParcelyNoParZar;
   procedure CreateSKPGeometry;
   procedure PrepareQTree;
   procedure DoResetParZar;
   property  QT: TRectEntityQTree read FQT write SetQT;
published
   property  ARE_PAR: TEntity read FARE_PAR write SetARE_PAR stored false;
   property  LIN_PAR: TEntity read FLIN_PAR write SetLIN_PAR stored false;
   property  ARE_SKP: TEntity read FARE_SKP write SetARE_SKP stored false;
   property  LIN_SKP: TEntity read FLIN_SKP write SetLIN_SKP stored false;
   property  ParcelModel: Tgis_ParcelModel read FParcelModel write SetParcelModel stored false;
   property  FrameAreas: TEntity read FFrameAreas write SetFrameAreas;
   property  Comment: string read FComment write SetComment;
   property  SKPColor: integer read FSKPColor write SetSKPColor;
   property  SimpleKatparMode: boolean read FSimpleKatparMode write SetSimpleKatparMode;
   property  ResetParZar: boolean read FResetParZar write SetResetParZar;
   property  SolveParZar: boolean read FSolveParZar write SetSolveParZar;
   property  ParzarByKATPAR_KOD13: boolean read FParzarByKATPAR_KOD13 write SetParzarByKATPAR_KOD13;
   property  UseKATPAR_KOD: boolean read FUseKATPAR_KOD write SetUseKATPAR_KOD;
   property  UseSuperID: boolean read FUseSuperID write SetUseSuperID;
   property  UsePUPFL: boolean read FUsePUPFL write SetUsePUPFL;
   property  UsePUPFLc: boolean read FUsePUPFLc write SetUsePUPFLc;
   property  ByPropertyList: string read FByPropertyList write SetByPropertyList;
   property  NumberByProperty: string read FNumberByProperty write SetNumberByProperty;
end;

implementation
uses
   Classes,
   Forms,
   Dialogs,
   SysUtils,
   e_Class,
   e_Geometry,
   e_TopolElements,
   G_Elems,
   m_Orbis,
   t_AreMak,
   t_CopyLInesFromTo,
   t_ParcelyTestTools,
   t_ParcelyUtils,
   t_ParcelModelSelector,
   t_QTreeTools;

{ TCreateSKPfromPAR }

function TCreateSKPfromPAR.CheckParcelyHierarchy: boolean;
var
   LostH: integer;
   i: integer;
   P: Tgis_ParcelaCast;
   E: TElement;
begin
   result := false;
   if ARE_PAR.NoElements then
   begin
      ShowMessage(Comment+#13+#10+'Nemohu v zadan vrstv nalzt dn parcely!');
      exit;
   end;
   LostH := 0;
   ProgressText := 'Kontrola parcel';
   MaxProgress := ARE_PAR.LastIndex;
   Application.ProcessMessages;
   for i := 0 to ARE_PAR.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      E := ARE_PAR.Elements[i];
      if not (E is Tgis_ParcelaCast)then
      begin
         ShowMessage(Comment+#13+#10+'dajn vrstva parcel neobsahuje parcely(resp.sti parcel), ale objekty typu '+E.ClassName);
         exit;
      end;
      P := Tgis_ParcelaCast(E);
      if P.Katastr = nil then
         Inc(LostH);
   end;
   if LostH > 0 then
   begin
      if LostH >= ARE_PAR.Count then
      begin
         ShowMessage(Comment+#13+#10+'dn z parcel nem piazen katastr - operace kon!');
         exit;
      end;
      ShowMessage(Comment+#13+#10+IntToStr(LostH)+' z '+IntToStr(ARE_PAR.Count)+' parcel nem piazen katastr!');
   end;
   result := true;
end;

procedure TCreateSKPfromPAR.CreateSKPGeometry;
var
   i,ii,iii: integer;
   T: THraniceSKPTestTool;
   e: TElement;
   TL: TTopolLineElement;
   L: TPolyLineElement;
   ACL: TAreasFromClearLines;
   En : TEntity;
   PMS: TParcelModelSelector;
   Are: TPolyLineAreaElement;
   SKP,SKPN: Tgis_SkupinaParcel;
   aP: Tgis_Parcela;
   PC,aPC: Tgis_ParcelaCast;
   OE: TOwnerEntity;
   TCL: TCopyLinesFromTo;
   K: Tgis_Katastr;
begin
   if LIN_PAR = nil then exit;
   ProgressText := 'Tvorba geometrickho obrazu';
   MaxProgress := 0;
   Application.ProcessMessages;
   T := THraniceSKPTestTool.Create;
   T.UseParZar := SolveParZar;
   OE := TOwnerEntity.Create;
   for i := 0 to LIN_PAR.LastIndex do
   begin
      e := LIN_PAR.Elements[i];
      if not(e is TTopolLineElement) then continue;
      TL := TTopolLineElement(e);
      L := TPolyLineElement(TL.Geometry);
      T.Input := L;
      T.DoIt;
      If not T.Result then
         OE.AddElement(TL);
   end;
   TCL := TCopyLinesFromTo.Create;
   TCL.Input := OE;
   TCL.Output := LIN_SKP;
   TCL.DoIt;
   TCL.Free;
   OE.Free;
   {
   if (LIN_SKP is TRectEntity) then
      TRectEntity(LIN_SKP).Update;
   }
   ACL := TAreasFromClearLines.Create;
   ACL.Input := LIN_SKP;
   ACL.DoIt;
   ParcelModel.UpdateAll := true;
   PMS := TParcelModelSelector.Create;
   PMS.SelectByParcely := true;
   PMS.FindInput;
   En := TEntity(ACL.Output);
   for i := 0 to En.LastIndex do
   begin
      e := En.Elements[i];
      if not (e is TPolyLineAreaElement) then continue;
      Are := TPolyLineAreaElement(e);
      PMS.X := Are.X;
      PMS.Y := Are.Y;
      PMS.DoIt;
      SKP := PMS.ActualSkupinaParcel;
      PC := PMS.ActualParCast;
      if SolveParZar And (PC <> nil) and not PC.ParZar then
         SKP := nil;
      if SKP <> nil then
      begin
         if not SimpleKatparMode and not SKP.NoGeometry then
         begin
            K := SKP.Katastr;
            SKPN := Tgis_SkupinaParcel.Create;
            K.NextSkupinaParcel := SKPN;
            SKPN.AddGeometry(Are);
            SKPN.Cislo := K.SkupinyParcel.Count;
            SKPN.KATPAR_KOD := SKP.KATPAR_KOD;
            SKPN.Barva := SKP.Barva;
            SKPN.Znacka := SKP.Znacka;
            SKPN.Comment := SKP.Comment + '+';
            ARE_SKP.AddElement(SKPN);
            if not SKP.NoParcela then
            for ii := SKP.Parcely.LastIndex downto 0 do
            begin
               e := SKP.Parcely.Elements[ii];
               if not (e is Tgis_Parcela) then continue;
               aP := Tgis_Parcela(e);
               for iii := 0 to aP.LastIndex do
               begin
                  e := aP.Elements[iii];
                  if not (e is Tgis_ParcelaCast) then continue;
                  aPC := Tgis_ParcelaCast(e);
                  if aPC.NoGeometry then continue;
                  if Are.IsHitByXY(aPC.X,aPC.Y,0) then
                     aPC.SkupinaParcel := SKPN;
               end;
            end;
            if not SKP.NoParcast then
            for ii := SKP.ParCasti.LastIndex downto 0 do
            begin
               e := SKP.ParCasti.Elements[ii];
               if not (e is Tgis_ParcelaCast) then continue;
               aPC := Tgis_Parcelacast(e);
               if Are.IsHitByXY(aPC.X,aPC.Y,0) then
                  aPC.SkupinaParcel := SKPN;
            end;
            //PC.SkupinaParcel := SKPN;
         end
         else
         begin
            SKP.AddGeometry(Are);
            if PC <> nil then
               SKP.KATPAR_KOD := PC.KATPAR_KOD;
         end;
         {
         SKP.AddGeometry(Are);
         if PC <> nil then
            SKP.KATPAR_KOD := PC.KATPAR_KOD;
         }
      end;
   end;
   ParcelModel.UpdateAll := true;
   En.Free;
   ACL.Free;
   PMS.Free;
end;

procedure TCreateSKPfromPAR.DoIt;
var
   S: string;
begin
   Error := 0;
   if ARE_PAR = nil then Error := 1;
   if ARE_SKP = nil then Error := 2;
   if LIN_SKP = nil then Error := 3;
   if ParcelModel = nil then Error := 4;
   if Error > 0 then
   begin
      case Error of
         1: S := 'ARE_PAR';
         2: S := 'ARE_SKP';
         3: S := 'LIN_SKP';
         4: S := 'Model parcel';
      end;
      ShowMessage(Comment+#13+#10+'Povinn parametr '+S+' chyb');
      exit;
   end;
   StartProgress(Comment,0,0);
   PrepareQTree;
   if ResetParZar then
      DoResetParZar;
   if not CheckParcelyHierarchy then exit;
   if not ResetSkupinyParcel then exit;
   if SolveParZar then
      FindParcelyNoParZar;
   CreateSKPGeometry;
   ParcelModel.UpdateAll := true;
   ShowMessage('Operace se podaila ...');
   PostMsg(tm_ModelChanged,nil,nil,false);
end;

procedure TCreateSKPfromPAR.PrepareQTree;
var
   AQ: TAddEntityToQTree;
begin
   QT := nil;
   if SimpleKatparMode then exit;
   if (FrameAreas = nil) or FrameAreas.NoElements then
   begin
      ShowMessage('Nebyly nalezeny dn referenn plochy (Revry)'+#13+#10+
                  'skupiny budou tvoeny pouze na zkl. KATPAR_KOD a Katastru');
      exit;
   end;
   AQ := TAddEntityToQTree.Create;
   AQ.Input := FrameAreas;
   AQ.DoIt;
   QT := TRectEntityQTree(AQ.Output);
   AQ.Free;
end;

function TCreateSKPfromPAR.ResetSkupinyParcel: boolean;
var
   K: Tgis_Katastr;
   SP: Tgis_SkupinaParcel;
   P: Tgis_Parcela;
   PC: Tgis_ParcelaCast;
   i,j,ic,SPC,iSL: integer;
   S,SN: string;
   E: TElement;
   FQ: TFindRectEntityInQTreeByXY;
   Are,QAre: TAreaElement;
   PupflPom: string;
   SL: TStringList;
   BySL: boolean;
   NByP: boolean;
begin
   result := false;
   Progress := 0;
   S := 'Ruen stvajcch skupin';
   ProgressText := S+' Plochy ...';
   Application.ProcessMessages;
   ARE_SKP.FreeAll;
   ProgressText := S+' Linie ...';
   Application.ProcessMessages;
   LIN_SKP.FreeAll;
   if not (ParcelModel is Tgis_ParcelModel) or ParcelModel.NoElements then
   begin
      ShowMessage(Comment+#13+#10+'Model parcel nen modelem parcel nebo neobsahuje dn katastry!');
      exit;
   end;
   SL := TStringList.Create;
   if ByPropertyList <> '' then
      SL.CommaText := ByPropertyList;
   BySL := SL.Count > 0;
   NByP := NumberByProperty <> '';
   //MaxProgress := ParcelModel.LastIndex;
   S := 'Zmna struktury dle Parametru... ';
   ProgressText := S;
   Progress := 0;
   Application.ProcessMessages;
   FQ := nil;
   if QT <> nil then
   begin
      FQ := TFindRectEntityInQTreeByXY.Create;
      FQ.Input := QT;
   end;
   for i := 0 to ParcelModel.LastIndex do
   begin
      E := ParcelModel.Elements[i];
      if not (e is Tgis_Katastr) then continue;
      K := Tgis_Katastr(E);
      ProgressText := S+'katastr '+ IntToStr(i)+'/'+IntToStr(ParcelModel.LastIndex)+' .'+K.Name;
      if K.SkupinyParcel <> nil then
         K.SkupinyParcel.FreeAll;
      if K.NoParcela then continue;
      ParcelModel.ActualKatastr := K;
      Progress := 0;
      MaxProgress := K.Parcely.LastIndex;
      for j := 0 to K.Parcely.LastIndex do
      begin
         Progress := j;
         Application.ProcessMessages;
         P := Tgis_Parcela(K.Parcely.Elements[j]);
         for ic  := 0 to P.LastIndex do
         begin
            SN := '';
            e := P.Elements[ic];
            if not (e is Tgis_ParcelaCast) then continue;
            PC := Tgis_ParcelaCast(e);
            // nasledujici pasaz je jen pro formu, cast parcely by nidky nemela byt MultiArea
            if PC.Geometry is TMultiAreaElement then
            begin
               Are := TMultiAreaElement(PC.Geometry).AreaElement[0];
            end
            else
               Are := TAreaElement(PC.Geometry);
            if FQ <> nil then
            begin
               FQ.X := Are.X;
               FQ.Y := Are.Y;
               FQ.DoIt;
               QAre := TAreaElement(FQ.Output);
               if QAre <> nil then
                  SN := 'l'+QAre.Name
               else
                  SN := 'l-1';
            end
            else
               SN := 'nl';
            if (SN = '0') then
            begin
               ParcelModel.CreateSkupinaParcelCislo(0);
               ParcelModel.ActualSkupinaParcel.Comment := 'SN0';
               PC.ParZar := false;
            end
            else
            begin
               if not PC.ParZar then
               begin
                  ParcelModel.CreateSkupinaParcelCislo(0);
                  ParcelModel.ActualSkupinaParcel.Comment := 'NPZ';
               end
               else
               begin
                  //PC.ParZar := true;
                  if UseKATPAR_KOD then
                     SN := SN+PC.GetPropertyStringByName('KATPAR_KOD');
                  if UseSuperID then
                     SN := SN+PC.GetPropertyStringByName('SuperID');
                  if UsePUPFLc then
                  begin
                     if PC.PUPFLc then
                        SN := SN + '1'
                     else
                        SN := SN + '2';
                  end
                  else
                  if UsePUPFL then
                  begin
                     if PC.PUPFL then
                        SN := SN + '1'
                     else
                        SN := SN + '2';
                  end;
                  // Nastaveni podle dalsich vlastnosti
                  if BySL then
                  for iSL := 0 to SL.Count - 1 do
                  begin
                     SN := SN + PC.GetPropertyStringByName(SL.Strings[iSL]);
                  end;
                  SN := SN + 'v' + PC.GetPropertyStringByName('VLASTNIK');
                  ParcelModel.CreateSkupinaParcelComment(SN);
                  ParcelModel.ActualSkupinaParcel.Cislo := 1;
               end;
            end;
            PC.SkupinaParcel := ParcelModel.ActualSkupinaParcel;
            ParcelModel.ActualSkupinaParcel.KATPAR_KOD := PC.KATPAR_KOD;
         end;
      end;
      SPC := 1;
      for j := 0 to K.SkupinyParcel.LastIndex do
      begin
         E := K.SkupinyParcel.Elements[j];
         if not (E is Tgis_SkupinaParcel) then Continue;
         SP := Tgis_SkupinaParcel(E);
         //if not SimpleKatparMode then
         if not (SP.Cislo = 0) then
         begin
            if NByP then
            begin
               PC := SP.GetFirstParCast;
               if PC <> nil then
                  SP.Cislo := PC.GetPropertyLongIntByName(NumberByProperty);
            end
            else
            begin
               SP.Cislo := SPC;
               Inc(SPC);
            end;
         end;
         SP.Barva := SKPColor;
         SP.Znacka := 1;
         ARE_SKP.AddElement(SP);
      end;
   end;
   if (ARE_SKP is TRectEntity) then
      TRectEntity(ARE_SKP).Update;
   SL.Free;
   if FQ <> nil then
      FQ.Free;
   result := true;
end;

procedure TCreateSKPfromPAR.SetARE_PAR(const Value: TEntity);
begin
  FARE_PAR := Value;
end;

procedure TCreateSKPfromPAR.SetARE_SKP(const Value: TEntity);
begin
  FARE_SKP := Value;
end;

procedure TCreateSKPfromPAR.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCreateSKPfromPAR.SetFrameAreas(const Value: TEntity);
begin
  FFrameAreas := Value;
end;

procedure TCreateSKPfromPAR.SetLIN_PAR(const Value: TEntity);
begin
  FLIN_PAR := Value;
end;

procedure TCreateSKPfromPAR.SetLIN_SKP(const Value: TEntity);
begin
  FLIN_SKP := Value;
end;

procedure TCreateSKPfromPAR.SetParcelModel(const Value: Tgis_ParcelModel);
begin
  FParcelModel := Value;
end;

procedure TCreateSKPfromPAR.SetQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FQT), Value);
end;

procedure TCreateSKPfromPAR.SetSimpleKatparMode(const Value: boolean);
begin
  FSimpleKatparMode := Value;
end;

procedure TCreateSKPfromPAR.SetSKPColor(const Value: integer);
begin
  FSKPColor := Value;
end;

procedure TCreateSKPfromPAR.SetUseKATPAR_KOD(const Value: boolean);
begin
  FUseKATPAR_KOD := Value;
end;

procedure TCreateSKPfromPAR.SetSolveParZar(const Value: boolean);
begin
  FSolveParZar := Value;
end;

procedure TCreateSKPfromPAR.SetUseSuperID(const Value: boolean);
begin
  FUseSuperID := Value;
end;

procedure TCreateSKPfromPAR.FindParcelyNoParZar;
var
   FP: TFindParcelyNoParZar;
   i,j,pci: integer;
   e: TElement;
   S: string;
   K: Tgis_Katastr;
   P: Tgis_Parcela;
   PC: Tgis_ParcelaCast;
begin
{ TODO -oPPK -cOpravit : PARZAR ne podle KATPAR_kOD13 nehazi par do SKP 0 proste funguje nejak divne }

   Progress := 0;
   S := 'Solve parzar:';
   ProgressText := S+' plochy';
   Application.ProcessMessages;
   FP := TFindParcelyNoParZar.Create;
   // !!!!!! vlozit seznam parcel ParZar = false
   FP.Input := ARE_PAR;
   FP.Test := true; // aby se hodnota parzar neresetovala
   FP.DoIt;
   FP.Free;
   if ParzarByKATPAR_KOD13 then
   begin
      ProgressText := S+' KATPAR_KOD 13';
      Application.ProcessMessages;
      for i := 0 to ARE_PAR.LastIndex do
      begin
         e := ARE_PAR.Elements[i];
         if not (e is Tgis_ParcelaCast) then continue;
         PC := Tgis_ParcelaCast(e);
         if PC.KATPAR_KOD = 13 then
            PC.ParZar := false;
      end;
   end;
   Progress := 0;
   MaxProgress := ParcelModel.LastIndex;
   S := 'PARZAR false do 0:';
   //Nastaveni SKP 0 pro parzar false
   for i := 0 to ParcelModel.LastIndex do
   begin
      E := ParcelModel.Elements[i];
      if not (e is Tgis_Katastr) then continue;
      K := Tgis_Katastr(E);
      ParcelModel.ActualKatastr := K;
      ProgressText := S+'katastr '+ IntToStr(i)+'/'+IntToStr(ParcelModel.LastIndex)+' .'+K.Name;
      Progress := i;
      Application.ProcessMessages;
      if not K.NoParcela then
      for j := 0 to K.Parcely.LastIndex do
      begin
         e := K.Parcely.Elements[j];
         if not (e is Tgis_Parcela) then continue;
         P := Tgis_Parcela(e);
         if not P.NoElements then
         for pci := 0 to P.LastIndex do
         begin
            e := P.Elements[pci];
            if not (e is Tgis_ParcelaCast) then continue;
            PC := Tgis_ParcelaCast(e);
            if not PC.ParZar then
            begin
               ParcelModel.CreateSkupinaParcelCislo(0);
               PC.SkupinaParcel := ParcelModel.ActualSkupinaParcel;
            end;
         end;
      end;
   end;
end;

procedure TCreateSKPfromPAR.SetUsePUPFL(const Value: boolean);
begin
  FUsePUPFL := Value;
end;

procedure TCreateSKPfromPAR.SetResetParZar(const Value: boolean);
begin
  FResetParZar := Value;
end;

procedure TCreateSKPfromPAR.DoResetParZar;
var
   i: integer;
   PC: Tgis_ParcelaCast;
   e: TElement;
begin
   if ARE_PAR = nil then exit;
   if ARE_PAR.NoElements then exit;
   for i := 0 to ARE_PAR.LastIndex do
   begin
      e := ARE_PAR.Elements[i];
      if not (e is Tgis_ParcelaCast) then continue;
      PC := TGis_ParcelaCast(e);
      PC.ParZar := true;
   end;
end;

procedure TCreateSKPfromPAR.SetParzarByKATPAR_KOD13(const Value: boolean);
begin
  FParzarByKATPAR_KOD13 := Value;
end;

procedure TCreateSKPfromPAR.SetUsePUPFLc(const Value: boolean);
begin
  FUsePUPFLc := Value;
end;

procedure TCreateSKPfromPAR.SetByPropertyList(const Value: string);
begin
  FByPropertyList := Value;
end;

procedure TCreateSKPfromPAR.SetNumberByProperty(const Value: string);
begin
  FNumberByProperty := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCreateSKPfromPAR);
END.
