unit t_ExportLCRDialog;

interface
uses
   f_ExportLCRDialog,
   t_FormTool;

type

TExportLCRDialog = class(TFormTool)
  private
    FBlockName: string;
    function GetLCRForm: TExportLCRDialog_Form;
    function GetSourceDataDirectory: string;
    function GetFullBlockName: string;
    procedure SetBlockName(const Value: string);
    function GetCisloReviru: integer;
    function GetCisloLHCLCR: integer;
    procedure SetActiveButton(const Value: string);
    function GetText: string;
    procedure SetText(const Value: string);
    function GetLHCBlockName: string;
    function GetAllInOne: boolean;
    function GetCondition: string;
public
   procedure CreateForm; override;
   property  LCRForm: TExportLCRDialog_Form read GetLCRForm;
published
   property SourceDataDirectory: string read GetSourceDataDirectory stored false;
   property BlockName: string read FBlockName write SetBlockName;
   property FullBlockName: string read GetFullBlockName stored false;
   property LHCBlockName: string read GetLHCBlockName stored false;
   property CisloReviru: integer read GetCisloReviru stored false;
   property CisloLHCLCR: integer read GetCisloLHCLCR stored false;
   property ActiveButton: string write SetActiveButton stored false;
   property Text: string read GetText write SetText stored false;
   property AllInOne: boolean read GetAllInOne;
   property Condition: string read GetCondition;
end;


implementation
uses
   Forms,
   Graphics,
   SysUtils,
   e_Class,
   u_String;

{ TExportLCRDialog }

procedure TExportLCRDialog.CreateForm;
begin
   aForm := TExportLCRDialog_Form.Create(Application);
   aForm.Tool := Self;
end;

function TExportLCRDialog.GetAllInOne: boolean;
begin
   result := false;
   if aForm = nil then exit;
   result := LCRForm.cb_AllInOne.Checked;
end;

function TExportLCRDialog.GetCisloLHCLCR: integer;
begin
   result := 0;
   if aForm = nil then exit;
   result := LCRForm.ed_LHC.Value;
end;

function TExportLCRDialog.GetCisloReviru: integer;
begin
   result := 0;
   if aForm = nil then exit;
   result := LCRForm.ed_Cislo.Value;
end;

function TExportLCRDialog.GetCondition: string;
begin
   result := ('LS_KOD='+IntToStr(CisloReviru));
end;

function TExportLCRDialog.GetFullBlockName: string;
var
   S,S1: string;
begin
   S := LCRForm.l_LCRRevirPath.Caption;
   CheckBackSlash(S);
   S1 := LCRForm.ed_Cislo.Text;
   if length(S1) = 1 then S1 := '0'+S1;
   result := S + S1 + BlockName;
end;

function TExportLCRDialog.GetLCRForm: TExportLCRDialog_Form;
begin
   result := TExportLCRDialog_Form(aForm);
end;

function TExportLCRDialog.GetLHCBlockName: string;
var
   S,S1: string;
begin
   S := LCRForm.ed_LCR.Text;
   //S := ExtractFilePath(S);
   CheckBackSlash(S);
   result := S + BlockName;
end;

function TExportLCRDialog.GetSourceDataDirectory: string;
begin
   if LCRForm = nil then exit;
   result := LCRForm.ed_Revir.Text;
end;

function TExportLCRDialog.GetText: string;
begin
   result := '';
   if LCRForm = nil then exit;
   result := LCRForm.StatusBar.SimpleText;
end;

procedure TExportLCRDialog.SetActiveButton(const Value: string);
begin
   if LCRForm = nil then exit;
   LCRForm.b_CtiRevir.Font.Color := 0;
   LCRForm.b_REV.Font.Color := 0;
   LCRForm.b_ODD.Font.Color := 0;
   LCRForm.b_DIL.Font.Color := 0;
   LCRForm.b_PSK.Font.Color := 0;
   LCRForm.b_BZL.Font.Color := 0;
   LCRForm.b_JP.Font.Color := 0;
   LCRForm.b_OP.Font.Color := 0;
   LCRForm.b_ZRO.Font.Color := 0;
   LCRForm.b_VODKOM.Font.Color := 0;
   LCRForm.b_HOP.Font.Color := 0;
   if Value = 'ZRO' then
      LCRForm.b_ZRO.Font.Color := clRed
   else
   if Value = 'VODKOM' then
      LCRForm.b_VODKOM.Font.Color := clRed
   else
   if Value = 'CTIREVIR' then
      LCRForm.b_CtiRevir.Font.Color := clRed
   else
   if Value = 'REV' then
      LCRForm.b_REV.Font.Color := clRed
   else
   if Value = 'ODD' then
      LCRForm.b_ODD.Font.Color := clRed
   else
   if Value = 'DIL' then
      LCRForm.b_DIL.Font.Color := clRed
   else
   if Value = 'PSK' then
      LCRForm.b_PSK.Font.Color := clRed
   else
   if Value = 'BZL' then
      LCRForm.b_BZL.Font.Color := clRed
   else
   if Value = 'JP' then
      LCRForm.b_JP.Font.Color := clRed
   else
   if Value = 'OP' then
      LCRForm.b_OP.Font.Color := clRed
   else
   if Value = 'HOP' then
      LCRForm.b_HOP.Font.Color := clRed;
end;

procedure TExportLCRDialog.SetBlockName(const Value: string);
begin
  FBlockName := Value;
end;

procedure TExportLCRDialog.SetText(const Value: string);
begin
   if aForm = nil then exit;
   LCRForm.StatusBar.SimpleText := Value;

end;

INITIALIZATION
   RegisterOrbisClass(TExportLCRDialog);
END.
