unit t_ForestModelSelector;

interface
uses
   e_LHCModel,
   t_FrameSelect;

type

TForestModelSelector = class(TFrameSelector)
private
    FForestModel: Tgis_ForestModel;
    FActualDilec: Tgis_Dilec;
    FActualLHC: Tgis_LHC;
    FActualLS: Tgis_LS;
    FActualOddeleni: Tgis_Oddeleni;
    FActualPorost: Tgis_Porost;
    FActualSkupina: Tgis_PorostniSkupina;
    FFrameClassName: string;
    FFrameClass: Tgis_ForestUnitClass;
    FReferenceFrameClassName: string;
    FActualReferenceFrame: Tgis_ForestUnit;
    FReferenceFrameClass: Tgis_ForestUnitClass;
    procedure SetForestModel(const Value: Tgis_ForestModel);
    procedure SetActualDilec(const Value: Tgis_Dilec);
    procedure SetActualLHC(const Value: Tgis_LHC);
    procedure SetActualLS(const Value: Tgis_LS);
    procedure SetActualOddeleni(const Value: Tgis_Oddeleni);
    procedure SetActualPorost(const Value: Tgis_Porost);
    procedure SetActualSkupina(const Value: Tgis_PorostniSkupina);
    procedure SetFrameClass(const Value: Tgis_ForestUnitClass);
    procedure SetFrameClassName(const Value: string);
    function GetActualFrame: Tgis_ForestUnit;
    procedure SetActualFrame(const Value: Tgis_ForestUnit);
    procedure SetActualReferenceFrame(const Value: Tgis_ForestUnit);
    procedure SetReferenceFrameClass(const Value: Tgis_ForestUnitClass);
    procedure SetReferenceFrameClassName(const Value: string);
public
   procedure DoIt; override;
   procedure FindInput; override;
   function  GetFrameByXY(aUnit: Tgis_ItemsForestUnit; X,Y: double): Tgis_ForestUnit;
   property FrameClass: Tgis_ForestUnitClass read FFrameClass write SetFrameClass;
   property ReferenceFrameClass: Tgis_ForestUnitClass read FReferenceFrameClass write SetReferenceFrameClass;
published
   property ForestModel: Tgis_ForestModel read FForestModel write SetForestModel;
   property FrameClassName: string read FFrameClassName write SetFrameClassName;
   property ReferenceFrameClassName: string read FReferenceFrameClassName write SetReferenceFrameClassName;
   property ActualFrame: Tgis_ForestUnit read GetActualFrame write SetActualFrame;
   property ActualReferenceFrame: Tgis_ForestUnit read FActualReferenceFrame write SetActualReferenceFrame;
   property ActualLHC: Tgis_LHC read FActualLHC write SetActualLHC;
   property ActualLS: Tgis_LS read FActualLS write SetActualLS;
   property ActualOddeleni: Tgis_Oddeleni read FActualOddeleni write SetActualOddeleni;
   property ActualDilec: Tgis_Dilec read FActualDilec write SetActualDilec;
   property ActualPorost: Tgis_Porost read FActualPorost write SetActualPorost;
   property ActualSkupina: Tgis_PorostniSkupina read FActualSkupina write SetActualSkupina;
end;

implementation
uses
   Elements,
   e_Class,
   Worlds;

{ TForestModelSelector }

procedure TForestModelSelector.DoIt;
var
   i: integer;
   L: Tgis_LHC;
   FU: Tgis_ForestUnit;
begin
   if Busy then exit;
   if ForestModel = nil then exit;
   if FrameClass = nil then exit;
   if ActualFrame <> nil then
   begin
      if ActualFrame.PointIsInside(X,Y) then
      if ActualFrame.IsHitByXY(X,Y,0) then exit; // je to on
   end;
   Busy := true;
   ActualFrame := nil;
   ActualReferenceFrame := nil;
   for i := 0 to ForestModel.LastIndex do
   begin
      L := Tgis_LHC(ForestModel.Elements[i]);
      if not (L.ClassType = ReferenceFrameClass) then // LHC je reference, nesouhlas geom
         if not L.PointIsInside(X,Y) then continue;
      FU := GetFrameByXY(L,X,Y);
      if FU <> nil then // Frame nalezen
      begin
         ActualFrame := FU;
         Busy := false;
         exit;
      end;
   end;
   ActualReferenceFrame := nil;
   Busy := false;
end;

procedure TForestModelSelector.FindInput;
var
   E: TEntity;
   OM: Tgis_ForestModel;
begin
   OM := ForestModel;
   ForestModel := nil;
   E := TEntity(World.ObjectModel.FindElementName('Data'));
   if E = nil then exit;
   if not (TElement(E) is TEntity) then exit;
   ForestModel := Tgis_ForestModel(E.FindElementClass(Tgis_ForestModel));
   if ForestModel <> OM then
      ActualFrame := nil;
end;



function TForestModelSelector.GetActualFrame: Tgis_ForestUnit;
begin
   result := Tgis_ForestUnit(Output);
end;

function TForestModelSelector.GetFrameByXY(aUnit: Tgis_ItemsForestUnit; X, Y: double): Tgis_ForestUnit;
var
   i: integer;
   TargetLevel: boolean;
   ReferenceLevel: boolean;
   FU: Tgis_ForestUnit;
begin
   result := nil;
   if aUnit = nil then exit;
   if aUnit.NoItems then exit;
   TargetLevel := aUnit.ItemClass = FrameClass; // Konen rove, n se nepoleze
   ReferenceLevel := aUnit.ItemClass = ReferenceFrameClass; // nesm se kontrolovat obdlnky, nesed
   if aUnit.ClassType = ReferenceFrameClass then // Referenn rove, ulo se frame
      ActualReferenceFrame := aUnit;
   for i := 0 to aUnit.Items.LastIndex do
   begin
      FU := aUnit.Items.ForestUnit[i];
      if not ReferenceLevel then // pokud je rozpracovan geometrie, nesed obdlnky
         if not FU.PointIsInside(X,Y) then continue; // ani v obdlnku
      if TargetLevel then
      begin
         if FU.IsHitByXY(X,Y,0) then  // pokud nem Geometry, vrt false
         begin
            result := FU;   // Toto je Frame .. asi
            exit;
         end;
      end
      else   // jet nejsme v clov rovni, budeme se noit
      begin
         if not (FU is Tgis_ItemsForestUnit) then exit; // to je asi blbost
         result := GetFrameByXY(Tgis_ItemsForestUnit(FU),X,Y);
         if result <> nil then exit; // Frame byl nalezen
      end;
   end;
end;


procedure TForestModelSelector.SetActualDilec(const Value: Tgis_Dilec);
begin
   FActualDilec := Value;
   ActualPorost := nil;
end;

procedure TForestModelSelector.SetActualFrame(const Value: Tgis_ForestUnit);
begin
   Output := Value;
end;

procedure TForestModelSelector.SetActualLHC(const Value: Tgis_LHC);
begin
   FActualLHC := Value;
   ActualLS := nil;
end;

procedure TForestModelSelector.SetActualLS(const Value: Tgis_LS);
begin
   FActualLS := Value;
   ActualOddeleni := nil;
end;

procedure TForestModelSelector.SetActualOddeleni(const Value: Tgis_Oddeleni);
begin
   FActualOddeleni := Value;
   ActualDilec := nil;
end;

procedure TForestModelSelector.SetActualPorost(const Value: Tgis_Porost);
begin
   FActualPorost := Value;
   ActualSkupina := nil;
end;

procedure TForestModelSelector.SetActualReferenceFrame(const Value: Tgis_ForestUnit);
begin
  FActualReferenceFrame := Value;
end;

procedure TForestModelSelector.SetActualSkupina(const Value: Tgis_PorostniSkupina);
begin
   FActualSkupina := Value;
end;

procedure TForestModelSelector.SetForestModel(const Value: Tgis_ForestModel);
begin
  FForestModel := Value;
end;

procedure TForestModelSelector.SetFrameClass(const Value: Tgis_ForestUnitClass);
begin
   FFrameClass := Value;
end;

procedure TForestModelSelector.SetFrameClassName(const Value: string);
begin
   FrameClass := Tgis_ForestUnitClass(GetOrbisClass(Value));
end;


procedure TForestModelSelector.SetReferenceFrameClass(const Value: Tgis_ForestUnitClass);
begin
  FReferenceFrameClass := Value;
end;

procedure TForestModelSelector.SetReferenceFrameClassName(const Value: string);
begin
   ReferenceFrameClass := Tgis_ForestUnitClass(GetOrbisClass(Value));
end;

INITIALIZATION
   RegisterOrbisClass(TForestModelSelector);
END.
