unit t_JoinLHCLines;

interface
uses
   Elements,
   Tools;

type

TJoinLHCLines = class(TTool)
  private
    FMasterLines: TEntity;
    FSlaveLines: TEntity;
    FJoinMasterLines: boolean;
    FComment: string;
    FMasterSnapDistance: double;
    FSlaveSnapDistance: double;
    procedure SetMasterLines(const Value: TEntity);
    procedure SetSlaveLines(const Value: TEntity);
    procedure SetJoinMasterLines(const Value: boolean);
    procedure SetComment(const Value: string);
    procedure SetMasterSnapDistance(const Value: double);
    procedure SetSlaveSnapDistance(const Value: double);
public
   procedure DoIt; override;
published
   property MasterLines: TEntity read FMasterLines write SetMasterLines;
   property SlaveLines: TEntity read FSlaveLines write SetSlaveLines;
   property JoinMasterLines: boolean read FJoinMasterLines write SetJoinMasterLines;
   property MasterSnapDistance: double read FMasterSnapDistance write SetMasterSnapDistance;
   property SlaveSnapDistance: double read FSlaveSnapDistance write SetSlaveSnapDistance;
   property Comment: string read FComment write SetComment;
end;


implementation
uses
   Dialogs,
   Forms,
   SysUtils,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   e_QTree,
   t_QTreeTools,
   t_Snap;


{ TJoinLHCLines }

procedure TJoinLHCLines.DoIt;
var
   Snap: TPointLineSnapTool;
   TQ: TAddEntityToQTree;
   LevelQT: TRectEntityQTree;
   i,SnapCount: integer;
   e: TElement;
   L: TPolyLineElement;
begin
   if MasterLines = nil then
   begin
      ShowMessage('JoinLHCLines - MasterLines not found !!!!');
      exit;
   end;
   StartProgress(Comment,0,0);

   ProgressText := 'Pprava vrstev...';
   Application.ProcessMessages;
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'Master QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := MasterLines;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;

   ProgressText := 'Pipojovn Master Lines';
   MaxProgress := MasterLines.LastIndex;
   Progress := 0;
   Application.ProcessMessages;
   Snap := TPointLineSnapTool.Create;
   Snap.FrameQT := LevelQT;
   Snap.SnapToLine := true;
   Snap.SnapToLinePoint := true;
   Snap.SnapToLineBody := true;
   Snap.SnapDistance := MasterSnapDistance;
   SnapCount := 0;
   for i := 0 to MasterLines.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := MasterLines.Elements[i];
      if not (e is TGeometryElement) then continue;
      L := TPolyLineElement(TGeometryElement(e).Geometry);
      if L = nil then continue;
      // zacatek linie
      Snap.X := L.Xi[0];
      Snap.Y := L.Yi[0];
      Snap.TabuElement := L;
      Snap.DoIt;
      if Snap.Snapped {and (Snap.Output <> L)} then
      begin
         if (L.Xi[0] <> Snap.SnapX) or (L.Yi[0] <> Snap.SnapY) then
         begin
            Inc(SnapCount);
            L.InsertXY(0,Snap.SnapX,Snap.SnapY);
            //L.Xi[0] := Snap.SnapX;
            //L.Yi[0] := Snap.SnapY;
         end;
      end;
      // konec linie
      Snap.X := L.Xi[L.LastIndex];
      Snap.Y := L.Yi[L.LastIndex];
      Snap.DoIt;
      if Snap.Snapped {and (Snap.Output <> L)} then
      begin
         if (L.Xi[L.LastIndex] <> Snap.SnapX) or (L.Yi[L.LastIndex] <> Snap.SnapY) then
         begin
            Inc(SnapCount);
            L.AddXY(Snap.SnapX,Snap.SnapY);
            //L.Xi[L.LastIndex] := Snap.SnapX;
            //L.Yi[L.LastIndex] := Snap.SnapY;
         end;
      end;
   end;
   //ShowMessage('Master '+IntToStr(SnapCount));
   if SlaveLines = nil then
   begin
      Snap.Free;
      LevelQT.Free;
      exit;
   end;

   Snap.TabuElement := nil;
   Snap.SnapDistance := SlaveSnapDistance;
   ProgressText := 'Pipojovn Slave Lines';
   MaxProgress := SlaveLines.LastIndex;
   Progress := 0;
   for i := 0 to SlaveLines.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := SlaveLines.Elements[i];
      if not (e is TGeometryElement) then continue;
      L := TPolyLineElement(TGeometryElement(e).Geometry);
      if L = nil then continue;
      // zacatek linie
      Snap.X := L.Xi[0];
      Snap.Y := L.Yi[0];
      Snap.DoIt;
      if Snap.Snapped then
      begin
         if (L.Xi[0] <> Snap.SnapX) or (L.Yi[0] <> Snap.SnapY) then
            L.InsertXY(0,Snap.SnapX,Snap.SnapY);
         //L.Xi[0] := Snap.SnapX;
         //L.Yi[0] := Snap.SnapY;
      end;
      // konec linie
      Snap.X := L.Xi[L.LastIndex];
      Snap.Y := L.Yi[L.LastIndex];
      Snap.DoIt;
      if Snap.Snapped then
      begin
         if (L.Xi[L.LastIndex] <> Snap.SnapX) or (L.Yi[L.LastIndex] <> Snap.SnapY) then
            L.AddXY(Snap.SnapX,Snap.SnapY);
         //L.Xi[L.LastIndex] := Snap.SnapX;
         //L.Yi[L.LastIndex] := Snap.SnapY;
      end;
   end;

   Snap.Free;
   LevelQT.Free;
end;

procedure TJoinLHCLines.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TJoinLHCLines.SetJoinMasterLines(const Value: boolean);
begin
  FJoinMasterLines := Value;
end;

procedure TJoinLHCLines.SetMasterLines(const Value: TEntity);
begin
  FMasterLines := Value;
end;

procedure TJoinLHCLines.SetSlaveLines(const Value: TEntity);
begin
  FSlaveLines := Value;
end;

procedure TJoinLHCLines.SetMasterSnapDistance(const Value: double);
begin
  FMasterSnapDistance := Value;
end;

procedure TJoinLHCLines.SetSlaveSnapDistance(const Value: double);
begin
  FSlaveSnapDistance := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TJoinLHCLines);
END.
