unit t_KatastrPDialog;

interface
uses
   Elements,
   e_KatastrModel,
   f_KatastrPDialog,
   f_KUPDialog,
   t_OrbisDialog,
   t_ParcelModelSelector;

type

TKatastrPDialog = Class(TOrbisDialog)
private
    FFrame: TElement;
    FComment: string;
    FOldObjectText: string;
    FNewObjectText: string;
    FPMSelector: TParcelModelSelector;
    FColor: integer;
    FPAR: boolean;
    FOnlyPSKManipulate: boolean;
    FDynamicProps: string;
    FIniFile: string;
    function GetForm: TKatastrPDialog_Form;
    procedure SetFrame(const Value: TElement);
    function GetNewObject: boolean;
    procedure SetComment(const Value: string);
    procedure SetNewObjectText(const Value: string);
    procedure SetOldObjectText(const Value: string);
    function GetPMSelector: TParcelModelSelector;
    procedure SetColor(const Value: integer);
    procedure SetPAR(const Value: boolean);
    function GetChangeAreaOnly: boolean;
    procedure SetChangeAreaOnly(const Value: boolean);
    procedure SetOnlyPSKManipulate(const Value: boolean);
    procedure SetDynamicProps(const Value: string);
    procedure SetIniFile(const Value: string);
public
   procedure DoIt; override;
   procedure CreateForm; override;
   procedure SetMajitele;
   property  Form: TKatastrPDialog_Form read GetForm;
   property  NewObject: boolean read GetNewObject;
   property  PMSelector: TParcelModelSelector read GetPMSelector;
   property  ChangeAreaOnly: boolean read GetChangeAreaOnly write SetChangeAreaOnly;
   procedure SetupFromIni;
published
   property  Frame: TElement read FFrame write SetFrame stored false;
   property  Comment: string read FComment write SetComment;
   property  NewObjectText: string read FNewObjectText write SetNewObjectText;
   property  OldObjectText: string read FOldObjectText write SetOldObjectText;
   property  Color: integer read FColor write SetColor;
   property  PAR: boolean read FPAR write SetPAR;
   property  OnlyPSKManipulate: boolean read FOnlyPSKManipulate write SetOnlyPSKManipulate;
   property  DynamicProps: string read FDynamicProps write SetDynamicProps;
   property  IniFile: string read FIniFile write SetIniFile;
end;

TKUPDialog = Class(TOrbisDialog)
  private
    FColor: integer;
    FOldObjectText: string;
    FNewObjectText: string;
    FComment: string;
    FFrame: TElement;
    FPModel: TGis_ParcelModel;
    function GetForm: TKUPDialog_Form;
    function GetNewObject: boolean;
    procedure SetColor(const Value: integer);
    procedure SetComment(const Value: string);
    procedure SetFrame(const Value: TElement);
    procedure SetNewObjectText(const Value: string);
    procedure SetOldObjectText(const Value: string);
    procedure SetPModel(const Value: TGis_ParcelModel);
    function GetChangeAreaOnly: boolean;
    procedure SetChangeAreaOnly(const Value: boolean);
public
   procedure DoIt; override;
   procedure CreateForm; override;
   procedure SetMajitele;
   procedure FindPModel;
   property  Form: TKUPDialog_Form read GetForm;
   property  NewObject: boolean read GetNewObject;
   property  PModel: TGis_ParcelModel read FPModel write SetPModel;
   property  ChangeAreaOnly: boolean read GetChangeAreaOnly write SetChangeAreaOnly;
published
   property  Frame: TElement read FFrame write SetFrame stored false;
   property  Comment: string read FComment write SetComment;
   property  NewObjectText: string read FNewObjectText write SetNewObjectText;
   property  OldObjectText: string read FOldObjectText write SetOldObjectText;
   property  Color: integer read FColor write SetColor;
end;


implementation
uses
   Classes,
   Forms,
   SysUtils,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   e_Props,
   u_String,
   IniFiles,
   Worlds;

{ TKatastrPDialog }

procedure TKatastrPDialog.CreateForm;
begin
   aForm := TKatastrPDialog_Form.Create(Application);
   aForm.Tool := Self;
   aForm.IsPossibleToClose := false;
   SetupFromIni;
end;

procedure TKatastrPDialog.DoIt;
var
   Are: TAreaElement;
   G: TGeometryElement;
   KU: Tgis_Katastr;
   Parcela: Tgis_Parcela;
   ParCast: Tgis_ParcelaCast;
   Skupina: Tgis_SkupinaParcel;
   i: integer;
   Mi: string;
   SL: TStringList;
   M: Tgis_Majitel;
   Kat: Tgis_Katastr;
begin
   //Output := nil;
   SetupFromIni;
   Parcela := nil;
   Parcast := nil;
   Skupina := nil;
   PMSelector.FIndInput;
   SL := TStringList.Create;
   SL.CommaText := DynamicProps;
   Form.DynamicProps := SL;
   Form.PModel := PMSelector.ParcelModel;
   SetMajitele;
   Form.NewObject := NewObject;
   Form.Panel_Parcela.Visible := PAR;
   if not PAR then
   begin
      Form.Label9.Caption := 'Poet Geom.ploch';
      Form.Label24.Visible := false;
      Form.l_pcNumber.Visible := false;
      Form.l_Plocha.Visible := false;
      Form.l_pcArea.Visible := false;
      Form.Label21.Visible := false;
      Form.l_Vymera.Visible := false;
      Form.Label23.Visible := false;
      Form.ed_PCVymera.Visible := false;
      Form.ed_Vlastnik.Enabled := false;
      Form.cb_IsKUMajitel.Enabled := false;
   end;
   Form.OnlyPSKManipulate := OnlyPSKManipulate;
   Are := TAreaElement(Input);
   if NewObject then
   begin
      Caption := NewObjectText+Comment;
      //Form.ed_Vymera.Text := ' ';
      //Form.ed_pcVymera.Text := ' ';
      if PAR then
      begin
         PMSelector.X := Are.X;
         PMSelector.Y := Are.Y;
         PMSelector.DoIt;
         Skupina := Tgis_SkupinaParcel(PMSelector.Output);
         if Skupina <> nil then
         begin
            PMSelector.ParcelModel.ActualKatastr := Skupina.Katastr;
            PMSelector.ParcelModel.ActualSkupinaParcel  := Skupina;
         end
         else
         if PMSelector.ActualKatastr <> nil then
         begin
            KU := PMSelector.ActualKatastr;
            PMSelector.ParcelModel.ActualKatastr := KU;
            Form.ed_KU.Text := KU.Name;
            Form.l_KUNazev.Caption := KU.Nazev;
         end
         else
            PMSelector.ParcelModel.ActualKatastr := nil;
      end;
   end
   else
   begin
      Caption := OldObjectText+Comment;
   end;
   {$IFDEF UHUL}
   Form.l_pcArea.Caption := FormatFloat('0.##', Are.Area );
   {$ELSE}
   Form.l_pcArea.Caption := FormatFloat('0.####', Are.Area /10000);
   {$ENDIF}
   Form.l_AreaCount.Caption := '1';
   Form.l_Area.Caption := '?';
   // Nalezen vlastnost dle Frame

   ChangeAreaOnly := PAR;
   if not NewObject then
   begin
      if PAR then
      begin
         ParCast := Tgis_ParcelaCast(Output);
         Parcela := ParCast.Parcela;
         G := ParCast;
         //Form.cb_AreaOnly.Visible := (Parcela.Geometry is TMultiAreaElement) and (TMultiAreaElement(Parcela.Geometry).Count > 1);
         Form.cb_AreaOnly.Visible := false;
         {$IFDEF UHUL}
         Form.Label7.Caption := 'm2';
         Form.Label17.Caption := 'm2';
         Form.Label21.Caption := 'm2';
         Form.Label23.Caption := 'm2';
         Form.l_Area.Caption := FormatFloat('0.##', Parcela.Plocha);
         {$ELSE}
         Form.l_Area.Caption := FormatFloat('0.####', Parcela.Plocha_ha);
         {$ENDIF}
         Form.ed_Vymera.Text := Parcela.GetPropertyStringByName('Vymera');
         Form.ed_pcVymera.Text := ParCast.GetPropertyStringByName('Vymera');
         Form.l_pcNumber.Caption := Parcast.GetPropertyStringByName('ParCast');
         PMSelector.ParcelModel.ActualKatastr := Parcela.Katastr;
         PMSelector.ParcelModel.ActualSkupinaParcel := Parcast.SkupinaParcel;
         PMSelector.ParcelModel.ActualParcela := Parcela;
         PMSelector.ParcelModel.ActualParCast := ParCast;
         Form.ed_KU.Text := Parcast.GetpropertyStringByName('KU_KOD');
         Form.ed_SkParc.Text := Parcast.GetpropertyStringByName('SKUPAR_CIS');
         Form.ed_Kategorie.Text := Parcast.GetpropertyStringByName('KATPAR_KOD');
         Form.ed_Cislo.Text := Parcela.GetpropertyStringByName('PARCIS');
         Form.ed_Podlomeni.Text := Parcela.GetpropertyStringByName('PARPOD');
         Form.ed_KATUZE_KOD_PUV.Text := Parcela.GetpropertyStringByName('KATUZE_KOD_PUV');
         Form.ed_ATR_PUV.Text := ParCast.GetpropertyStringByName('ATR_PUV');
         Form.ed_LV.Text := Parcela.GetpropertyStringByName('LV');
         Form.ed_LVC.Text := ParCast.GetpropertyStringByName('LVC');
         Form.cb_Stavebni.Checked := Parcela.GetpropertyLongIntByName('PARCIS')<1;
         Form.cb_GP.Checked := ParCast.GP;
         Form.cb_NEP.Checked := ParCast.NEP;
         Form.cb_PUPFL.Checked := ParCast.PUPFL;
         Form.cb_PUPFLc.Checked := ParCast.PUPFLc;
         Form.cb_PARZAR.Checked := ParCast.PARZAR;
         Form.ed_Poznamka.Text := ParCast.Poznamka;
         Form.ed_Kultura.Text := Parcela.GetpropertyStringByName('Kultura');
         Form.ed_PKCis.Text := ParCast.GetpropertyStringByName('PKCIS');
         Form.ed_PKPod.Text := ParCast.GetpropertyStringByName('PKPOD');
         Form.ed_PK_Dil.Text := ParCast.GetpropertyStringByName('PK_Dil');
         Form.ed_SuperID.Text := ParCast.GetpropertyStringByName('SuperID');
         Form.ed_Vlastnik.Text := ParCast.GetPropertyStringByName('Vlastnik');
         Form.cb_IsKUMajitel.Checked := Parcast.IsKUMajitel;
         // Vlastnik ?
         Form.l_KUNazev.Caption := Parcast.KU_Nazev;
         Form.l_MajName.Caption := Parcast.MAJ_Name+' '+Parcast.MAJ_Adresa;
         Form.l_AreaCount.Caption := IntToStr(Parcela.Count);
         Form.FillParskukn;
      end
      else
      begin
         Skupina := Tgis_SkupinaParcel(Output);
         G := Skupina;
         Form.cb_AreaOnly.Visible := (Skupina.Geometry is TMultiAreaElement) and (TMultiAreaElement(Skupina.Geometry).Count > 1);
         Form.ed_Vymera.Text := Skupina.GetPropertyStringByName('Vymera');
         PMSelector.ParcelModel.ActualKatastr := Skupina.Katastr;
         PMSelector.ParcelModel.ActualSkupinaParcel := Skupina;
         Form.ed_KU.Text := Skupina.GetpropertyStringByName('KU_KOD');
         Form.ed_SkParc.Text := Skupina.GetpropertyStringByName('SKUPAR_CIS');
         Form.ed_Kategorie.Text := Skupina.GetpropertyStringByName('KATPAR_KOD');
         Form.l_Area.Caption := FormatFloat('0.####', Skupina.Plocha_ha);
         if G.Geometry is TMultiAreaElement then
            Form.l_AreaCount.Caption := IntToStr(G.Geometry.Count);
         Form.ed_Vlastnik.Text := Skupina.GetPropertyStringByName('Vlastnik');
         Form.l_KUNazev.Caption := Skupina.KU_Nazev;
         Form.l_MajName.Caption := Skupina.MAJ_Name+' '+Skupina.MAJ_Adresa;
         Form.cb_IsKUMajitel.Checked := Skupina.IsKUMajitel;
      end;
      //if G.Geometry is TMultiAreaElement then
         //Form.l_AreaCount.Caption := IntToStr(G.Geometry.Count);
      // Dynamicke property
      for i := 0 to SL.Count - 1 do
      begin
         Form.SetDynamicProp(SL.Strings[i],G.GetPropertyStringByName(SL.Strings[i]));
      end;
   end
   else
      Form.cb_AreaOnly.Visible := false;
   if not Form.cb_AreaOnly.Visible then ChangeAreaOnly := false;
   inherited;
   if Result = 1 then
   begin
      if PAR then
      begin
         if (Form.ed_KU.Text <> '') then
            PMSelector.ParcelModel.CreateKatastrCislo(StrToIntDef(Form.ed_KU.Text,0))
         else
            PMSelector.ParcelModel.CreateKatastrCislo(0);
         //
         begin
            if (Form.ed_SKParc.Text <> '') then
               PMSelector.ParcelModel.CreateSkupinaParcelCislo(StrToIntDef(Form.ed_SkParc.Text,0));
            if NewObject then
            begin
               PMSelector.ParcelModel.CreateParcelaCislo
                  (StrToIntDef(Form.ed_Cislo.Text,0),StrToIntDef(Form.ed_Podlomeni.Text,0),StrToIntDef(Form.ed_KATUZE_KOD_PUV.Text,0));
               PMSelector.ParcelModel.CreateParCast;
               Parcela := PMSelector.ParcelModel.ActualParcela;
               Parcast := PMSelector.ParcelModel.ActualParCast;
               ParCast.AddGeometry(TAreaElement(Input));
               Parcela.ResetAndUpdate;
            end
            else
            //if ChangeAreaOnly then   u parcely je vzdycky true
            begin
               PMSelector.ParcelModel.CreateParcelaCislo
                  (StrToIntDef(Form.ed_Cislo.Text,0),StrToIntDef(Form.ed_Podlomeni.Text,0),StrToIntDef(Form.ed_KATUZE_KOD_PUV.Text,0));
               if Parcela <> PMSelector.ParcelModel.ActualParcela then
               begin
                  Parcela.RemoveElement(ParCast);
                  if Parcela.NoElements then Parcela.Free; // uvolneni prazdne parcely???
                  Parcela := PMSelector.ParcelModel.ActualParcela;
                  Parcela.AddElement(ParCast);
                  Parcela.ResetAndUpdate;
                  if PMSelector.ParcelModel.ActualSkupinaParcel <> nil then
                     ParCast.SkupinaParcel := PMSelector.ParcelModel.ActualSkupinaParcel;
               end;
            {   pracuje se pouze s casti parcely
            end
            else
            begin
               PMSelector.ParcelModel.GetParcelaCislo
                  (StrToIntDef(Form.ed_Cislo.Text,0),StrToIntDef(Form.ed_Podlomeni.Text,0));
               if (PMSelector.ParcelModel.ActualParcela <> nil) and
                  (PMSelector.ParcelModel.ActualParcela <> Parcela) then
               begin
                  PMSelector.ParcelModel.ActualParcela.AddGeometry(Parcela.Geometry);
                  Parcela.Geometry := nil;
                  Parcela := PMSelector.ParcelModel.ActualParcela;
               end
               else
               if (Parcela.SkupinaParcel <> PMSelector.ParcelModel.ActualSkupinaParcel) then
               begin
                  if (Parcela.SkupinaParcel <> nil) then
                     Parcela.SkupinaParcel.Parcely.RemoveElement(Parcela);
                  PMSelector.ParcelModel.ActualSkupinaParcel.NextParcela := Parcela;
               end;
            }
            end;
            if PMSelector.ParcelModel.ActualSkupinaParcel <> nil then
               ParCast.SkupinaParcel := PMSelector.ParcelModel.ActualSkupinaParcel;
         end;
         Output := ParCast;
         Parcela.SetPropertyStringByName('Vymera',Form.ed_Vymera.Text);
         ParCast.SetPropertyStringByName('Vymera',Form.ed_pcVymera.Text);
         Parcela.PARCIS := StrToIntdef(Form.ed_Cislo.Text,0);
         Parcela.PARPOD := StrToIntdef(Form.ed_Podlomeni.Text,0);
         Parcela.KATUZE_KOD_PUV := StrToIntdef(Form.ed_KATUZE_KOD_PUV.Text,0);
         ParCast.ATR_PUV := StrToIntdef(Form.ed_ATR_PUV.Text,0);
         ParCast.PKCIS := StrToIntdef(Form.ed_PKCis.Text,0);
         ParCast.PKPOD := StrToIntdef(Form.ed_PKPod.Text,0);
         ParCast.PK_Dil := StrToIntdef(Form.ed_PK_Dil.Text,0);
         Parcast.GP := Form.cb_GP.Checked;
         Parcast.NEP := Form.cb_NEP.Checked;
         Parcast.Poznamka := Form.ed_Poznamka.Text;
         ParCast.KATPAR_KOD := StrToIntdef(Form.ed_Kategorie.Text,0);
         //Parcela.PAR_SKU_KN := StrToIntdef(Form.ed_ParSkuKn.Text,0);
         Parcela.LV := StrToIntdef(Form.ed_LV.Text,0);
         ParCast.LVc := StrToIntdef(Form.ed_LVc.Text,0);
         Parcela.Kultura := StrToIntdef(Form.ed_Kultura.Text,0);
         Parcela.PUPFL := Form.cb_PUPFL.Checked;
         ParCast.PUPFLc := Form.cb_PUPFLc.Checked;
         ParCast.ParZar := Form.cb_PARZAR.Checked;
         ParCast.SuperID := StrToIntDef(Form.ed_SuperID.Text,0);
         ParCast.Znacka := 1;
         ParCast.Barva := Color;
         M := nil;
         if Form.ed_Vlastnik.Text <> '' then
         begin
            Mi := Form.ed_Vlastnik.Text;
            if (Mi <> '') and (Mi <> '0') then
            begin
               if Form.Majitele <> nil then
               begin
                  M := Form.Majitele.CreateMajitelCislo(Mi);
               end;
            end;
         end;
         Kat := Parcast.Katastr;
         if Form.cb_IsKUMajitel.Checked then
         begin
            if Kat <> nil then
               Kat.Majitel := M;
         end
         else
         begin
            if Kat <> nil then
               Kat.Majitel := nil;
            Parcast.Majitel := M;
         end;
      end
      else
      begin
         if OnlyPSKManipulate then
         begin
            PMSelector.ParcelModel.GetSkupinaParcelCislo(StrToIntDef(Form.ed_SkParc.Text,0));
            if (PMSelector.ParcelModel.ActualSkupinaParcel <> nil) and
               (PMSelector.ParcelModel.ActualSkupinaParcel <> Skupina) then
            begin
               PMSelector.ParcelModel.ActualSkupinaParcel.AddGeometry(Skupina.Geometry);
               Skupina.Geometry := nil;
               if not Skupina.NoParcela then
               begin
                  for i := 0 to Skupina.Parcely.LastIndex do
                  begin
                     PMSelector.ParcelModel.ActualSkupinaParcel.NextParcela := Tgis_Parcela(Skupina.Parcely.Elements[i]);
                  end;
                  //PMSelector.ParcelModel.ActualSkupinaParcel.Parcely.AddAllFrom := Skupina.Parcely;
                  SKupina.Parcely := nil;
               end;
               if not Skupina.NoParcast then
               begin
                  for i := 0 to Skupina.ParCasti.LastIndex do
                  begin
                     PMSelector.ParcelModel.ActualSkupinaParcel.NextParCast := Tgis_ParcelaCast(Skupina.ParCasti.Elements[i]);
                  end;
                  //PMSelector.ParcelModel.ActualSkupinaParcel.Parcely.AddAllFrom := Skupina.Parcely;
                  SKupina.ParCasti := nil;
               end;
               //Skupina.Free;
               Skupina := PMSelector.ParcelModel.ActualSkupinaParcel;
            end;
            {
            else
            if (Skupina.Katastr <> PMSelector.ParcelModel.ActualKatastr) then
            begin
               if (skupina.Katastr <> nil) then
                  Skupina.Katastr.SkupinyParcel.RemoveElement(Skupina);
               PMSelector.ParcelModel.ActualKatastr.NextSkupinaParcel := Skupina;
            end;
            }
         end
         else
         if (Form.ed_KU.Text <> '') and (Form.ed_SKParc.Text <> '') then
         begin
            PMSelector.ParcelModel.CreateKatastrCislo(StrToInt(Form.ed_KU.Text));
            //PMSelector.ParcelModel.CreateSkupinaParcelCislo(StrToInt(Form.ed_SkParc.Text));
            if NewObject then
            begin
               PMSelector.ParcelModel.CreateSkupinaParcelCislo(StrToIntDef(Form.ed_SkParc.Text,0));
               Skupina := PMSelector.ParcelModel.ActualSkupinaParcel;
               Skupina.AddGeometry(TAreaElement(Input));
            end
            else
            if ChangeAreaOnly then
            begin
               PMSelector.ParcelModel.CreateSkupinaParcelCislo(StrToIntDef(Form.ed_SkParc.Text,0));
               if Skupina <> PMSelector.ParcelModel.ActualSkupinaParcel then
               begin
                  TMultiAreaElement(Skupina.Geometry).RemoveElement(Are);
                  Skupina := PMSelector.ParcelModel.ActualSkupinaParcel;
                  Skupina.AddGeometry(Are);
               end;
            end
            else
            begin
               PMSelector.ParcelModel.GetSkupinaParcelCislo(StrToIntDef(Form.ed_SkParc.Text,0));
               if (PMSelector.ParcelModel.ActualSkupinaParcel <> nil) and
                  (PMSelector.ParcelModel.ActualSkupinaParcel <> Skupina) then
               begin
                  PMSelector.ParcelModel.ActualSkupinaParcel.AddGeometry(Skupina.Geometry);
                  Skupina.Geometry := nil;
                  Skupina := PMSelector.ParcelModel.ActualSkupinaParcel;
               end
               else
               if (Skupina.Katastr <> PMSelector.ParcelModel.ActualKatastr) then
               begin
                  if (skupina.Katastr <> nil) then
                     Skupina.Katastr.SkupinyParcel.RemoveElement(Skupina);
                  PMSelector.ParcelModel.ActualKatastr.NextSkupinaParcel := Skupina;
               end;
            end;
         end
         else
         if NewObject then
         begin
            Skupina := Tgis_SkupinaParcel.Create;
            Skupina.Geometry := TAreaElement(Input);
         end;
         Output := Skupina;
         Skupina.SetPropertyStringByName('Vymera',Form.ed_Vymera.Text);
         Skupina.KATPAR_KOD := StrToIntdef(Form.ed_Kategorie.Text,0);
         Skupina.Znacka := 1;
         Skupina.Barva := Color;
      end;
      if SL.Count > 0 then
      begin
         if Output is TPropertyGeometryElement then
         begin
            if TPropertyGeometryElement(Output).NoProperties then
               TPropertyGeometryElement(Output).Properties := TDynamicProperties.Create;
         end;
         for i := 0 to SL.Count - 1 do
         begin
            Output.SetPropertyStringByName(SL.Strings[i],Form.GetDynamicProp(SL.Strings[i]));
         end;
      end;
   end;
   SL.Free;
end;

function TKatastrPDialog.GetChangeAreaOnly: boolean;
begin
   result := Form.cb_AreaOnly.Checked;
end;

function TKatastrPDialog.GetForm: TKatastrPDialog_Form;
begin
   result := TKatastrPDialog_Form(aForm);
end;

function TKatastrPDialog.GetNewObject: boolean;
begin
   result := Output = nil;
end;

function TKatastrPDialog.GetPMSelector: TParcelModelSelector;
begin
   if FPMSelector = nil then
   begin
      FPMSelector := TParcelModelSelector.Create;
      AddElement(FPMSelector);
   end;
   result := FPMSelector;
end;

procedure TKatastrPDialog.SetChangeAreaOnly(const Value: boolean);
begin
   Form.cb_AreaOnly.CHecked := Value;
end;

procedure TKatastrPDialog.SetColor(const Value: integer);
begin
  FColor := Value;
end;

procedure TKatastrPDialog.SetComment(const Value: string);
begin
   FComment := Value;
end;

procedure TKatastrPDialog.SetDynamicProps(const Value: string);
begin
  FDynamicProps := Value;
end;

procedure TKatastrPDialog.SetFrame(const Value: TElement);
begin
   FFrame := Value;
end;

procedure TKatastrPDialog.SetIniFile(const Value: string);
begin
  FIniFile := Value;
end;

procedure TKatastrPDialog.SetMajitele;
var
   En: TEntity;
   M: Tgis_Majitele;
begin
   En := TEntity(World.ObjectModel.FindElementName('DATA'));
   if En = nil then exit;
   M := Tgis_Majitele(En.FindElementClass(Tgis_Majitele));
   M.SortByCislo := true;
   Form.Majitele := M;
end;

procedure TKatastrPDialog.SetNewObjectText(const Value: string);
begin
   FNewObjectText := Value;
end;

procedure TKatastrPDialog.SetOldObjectText(const Value: string);
begin
   FOldObjectText := Value;
end;

procedure TKatastrPDialog.SetOnlyPSKManipulate(const Value: boolean);
begin
  FOnlyPSKManipulate := Value;
end;

procedure TKatastrPDialog.SetPAR(const Value: boolean);
begin
   FPAR := Value;
end;

procedure TKatastrPDialog.SetupFromIni;
var
   S: string;
   S1: string;
   Ini: TIniFile;
begin
   if IniFile = '' then exit;
   {$IFDEF UHUL}
   S := GetAbsolutePath(IniFile);
   if not FileExists(S) then exit;
   Ini := TIniFile.Create(S);
   S1 := 'KatastrPDialog';
   Form.l_Vlastnik.Caption := Ini.ReadString(S1,'l_Vlastnik','Vlastnk');
   Form.l_PAR_Plocha.Caption := Ini.ReadString(S1,'l_PAR_Plocha','Plocha:');
   Form.l_PAR_Vymera.Caption := Ini.ReadString(S1,'l_PAR_Vymera','Vmra:');
   Form.l_Plocha.Caption := Ini.ReadString(S1,'l_Plocha','Plocha sti:');
   Form.l_Vymera.Caption := Ini.ReadString(S1,'l_Vymera','Vmra sti:');
   Form.l_KU.Caption := Ini.ReadString(S1,'l_KU','Katastr');
   Form.l_PARCIS.Caption := Ini.ReadString(S1,'l_PARCIS','Parcis');
   Form.l_PARPOD.Caption := Ini.ReadString(S1,'l_PARPOD','Parpod');
   Form.l_LV.Caption := Ini.ReadString(S1,'l_LV','LV');
   Form.l_Kultura.Caption := Ini.ReadString(S1,'l_Kultura','Kultura');
   Form.l_PKCIS.Caption := Ini.ReadString(S1,'l_PKCIS','PKcis');
   Form.l_PKPOD.Caption := Ini.ReadString(S1,'l_PKPOD','- PKpod');
   Form.l_PKDIL.Caption := Ini.ReadString(S1,'l_PKDIL','PKDil');
   Form.l_LVC.Caption := Ini.ReadString(S1,'l_LVC','LVC');
   Form.l_Katpar_KOD.Caption := Ini.ReadString(S1,'l_Katpar_Kod','KATPAR_KOD');
   Form.l_PAR_SKU_KN.Caption := Ini.ReadString(S1,'l_PAR_SKU_KN','PAR_SKU_KN');
   Form.l_SuperID.Caption := Ini.ReadString(S1,'l_SuperID','SuperID');
   Form.l_KATUZE_KOD_PUV.Caption := Ini.ReadString(S1,'l_KATUZE_KOD_PUV','KATUZE_KOD_PUV');
   Ini.Free;
   {$ENDIF}
end;

{ TKUPDialog }

procedure TKUPDialog.CreateForm;
begin
   aForm := TKUPDialog_Form.Create(Application);
   aForm.Tool := Self;
   aForm.IsPossibleToClose := false;
end;

procedure TKUPDialog.DoIt;
var
   Are: TAreaElement;
   MA: TMultiAreaELement;
   KU,NKU: Tgis_Katastr;
   i: integer;
   Mi: string;
   M: Tgis_Majitel;
   aPlocha: double;
begin
   //Output := nil;
   FindPModel;
   SetMajitele;
   Form.NewObject := NewObject;
   Are := TAreaElement(Input);
   if NewObject then
   begin
      Caption := NewObjectText+Comment;
   end
   else
   begin
      Caption := OldObjectText+Comment;
   end;
   Form.l_Area.Caption := FormatFloat('0.####', Are.Area /10000);
   Form.l_SumArea.Caption := FormatFloat('0.####', Are.Area /10000);
   Form.l_AreaCount.Caption := '1';
   MA := nil;
   KU := nil;
   Form.cb_AreaOnly.Visible := false;
   if not NewObject then
   begin
      KU := Tgis_Katastr(Output);
      if KU.Geometry is TMultiAreaElement then
      begin
         MA := TMultiAreaElement(KU.Geometry);
         if MA.Count > 1 then
         begin
            Form.cb_AreaOnly.Visible := true;
            Form.l_AreaCount.Caption := IntToStr(MA.Count);
            aPlocha := 0;
            for i := 0 to MA.LastIndex do
            begin
               aPlocha := aPlocha + MA.AreaElement[i].Area;
            end;
            Form.l_SumArea.Caption := FormatFloat('0.####', aPlocha /10000);
         end;
      end;
      PModel.ActualKatastr := KU;
      Form.ed_KU.Text := KU.GetpropertyStringByName('CISLO');
      Form.ed_Cislo_.Text := KU.GetpropertyStringByName('CISLO_');
      Form.ed_Nazev.Text := KU.Nazev;
      Form.ed_ORP.Text := KU.GetpropertyStringByName('ORP');
      Form.ed_ORP_Nazev.Text := KU.ORP_Nazev;
      Form.ed_Kraj.Text := KU.Kraj;
      Form.ed_Kraj_Nazev.Text := KU.Kraj_Nazev;
      Form.ed_Vlastnik.Text := KU.GetPropertyStringByName('Vlastnik');
      // Vlastnik ?
      {
      Form.l_KUNazev.Caption := Parcast.KU_Nazev;
      Form.l_MajName.Caption := Parcast.MAJ_Name+' '+Parcast.MAJ_Adresa;
      }
   end
   else
      Form.cb_AreaOnly.Visible := false;
   ChangeAreaOnly := Form.cb_AreaOnly.Visible;
   inherited;
   if Result = 1 then
   begin
      if NewObject then
      begin
         if (Form.ed_KU.Text <> '') then
            PModel.CreateKatastrCislo(StrToIntDef(Form.ed_KU.Text,0))
         else
            PModel.CreateKatastrCislo(0);
         NKU := PModel.ActualKatastr;
         NKU.AddGeometry(Are);
      end
      else
      begin
         if CHangeAreaOnly then
         begin
            PModel.CreateKatastrCislo(StrToIntDef(Form.ed_KU.Text,0));
            NKU := PModel.ActualKatastr;
            if KU <> NKU then
            begin
               NKU.AddGeometry(Are);
               if MA <> nil then
               begin
                  MA.RemoveElement(Are);
                  if MA.NoElements then
                     KU.Geometry := nil
                  else
                     MA.ResetAndUpdate;
               end;
            end;
         end
         else
         begin
            NKU := PModel.GetKatastrCislo(StrToIntDef(Form.ed_KU.Text,0));
            if NKU = nil then
               NKU := KU
            else
            if NKU <> KU then
            begin
               NKU.AddGeometry(KU.Geometry);
               KU.Geometry := nil;
            end;
         end;
      end;
      Output := NKU;
      NKU.Cislo  := StrToIntDef(Form.ed_KU.Text,0);
      NKU.Cislo_ := StrToIntDef(Form.ed_Cislo_.Text,0);
      NKU.Nazev := Form.ed_Nazev.Text;
      NKU.ORP := StrToIntDef(Form.ed_ORP.Text,0);
      NKU.ORP_Nazev := Form.ed_ORP_Nazev.Text;
      NKU.Kraj := Form.ed_Kraj.Text;
      NKU.Kraj_Nazev := Form.ed_Kraj_Nazev.Text;
      Mi := Form.ed_Vlastnik.Text;
      if  (Mi <> '') and (Mi <> '0') then
      begin
         if Form.Majitele <> nil then
         begin
            M := Form.Majitele.CreateMajitelCislo(Mi);
            NKU.Majitel := M;
         end;
      end;
      NKU.Barva := Color;
      NKU.Znacka := 1;
   end;
end;

procedure TKUPDialog.FindPModel;
var
   En: TEntity;
   M: Tgis_ParcelModel;
begin
   PModel := nil;
   En := TEntity(World.ObjectModel.FindElementName('DATA'));
   if En = nil then exit;
   M := Tgis_ParcelModel(En.FindElementClass(Tgis_ParcelModel));
   PModel := M;
   Form.PModel := M;
end;

function TKUPDialog.GetChangeAreaOnly: boolean;
begin
   result := Form.cb_AreaOnly.Checked;
end;

function TKUPDialog.GetForm: TKUPDialog_Form;
begin
   result := TKUPDialog_Form(aForm);
end;


function TKUPDialog.GetNewObject: boolean;
begin
   result := Output = nil;
end;

procedure TKUPDialog.SetChangeAreaOnly(const Value: boolean);
begin
   Form.cb_AreaOnly.CHecked := Value;
end;

procedure TKUPDialog.SetColor(const Value: integer);
begin
   FColor := Value;
end;

procedure TKUPDialog.SetComment(const Value: string);
begin
   FComment := Value;
end;

procedure TKUPDialog.SetFrame(const Value: TElement);
begin
   FFrame := Value;
end;

procedure TKUPDialog.SetMajitele;
var
   En: TEntity;
   M: Tgis_Majitele;
begin
   En := TEntity(World.ObjectModel.FindElementName('DATA'));
   if En = nil then exit;
   M := Tgis_Majitele(En.FindElementClass(Tgis_Majitele));
   M.SortByCislo := true;
   Form.Majitele := M;
end;

procedure TKUPDialog.SetNewObjectText(const Value: string);
begin
  FNewObjectText := Value;
end;

procedure TKUPDialog.SetOldObjectText(const Value: string);
begin
  FOldObjectText := Value;
end;

procedure TKUPDialog.SetPModel(const Value: TGis_ParcelModel);
begin
  FPModel := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TKatastrPDialog);
   RegisterOrbisClass(TKUPDialog);
END.

