unit t_LHCTestTools;

interface
uses
   Elements,
   e_Type,
   Tools;

type
{
   input          - TPolyLineElement
   FrameClassName - tda rmce (prostorov jednotky lesa)
      kontroluje, zda linie tvo hranici dvou jednotek dan tdy
      pokud ano, properta result bude true
}
TForestBorderTestTool = class(TBooleanTool)
  private
    FFrameClassName: string;
    FFrameClass: TElementClass;
    FRefFrameClassName: string;
    FRefFrameClass: TElementClass;
    procedure SetFrameClassName(const Value: string);
    procedure SetFrameClass(const Value: TElementClass);
    procedure SetRefFrameClass(const Value: TElementClass);
    procedure SetRefFrameClassName(const Value: string);
public
   procedure DoIt; override;
   property  FrameClass: TElementClass read FFrameClass write SetFrameClass;
   property  RefFrameClass: TElementClass read FRefFrameClass write SetRefFrameClass;
published
   property  FrameClassName: string read FFrameClassName write SetFrameClassName;
   property  RefFrameClassName: string read FRefFrameClassName write SetRefFrameClassName;
end;

TPolyLineISFromZAKTool = class(TBooleanTool)
public
   procedure DoIt; override;
end;

TSRFBorderTestTool = class(TBooleanTool)
  private
    FTEZ: boolean;
    procedure SetTEZ(const Value: boolean);
public
   procedure DoIt; override;
published
   property TEZ: boolean read FTEZ write SetTEZ;
end;

TPresnostTestTool = class(TBooleanTool)
private
   FValues: String;
   FLayerName: string;
   FVals: TIntegerListElement;
   procedure SetValues(const Value: string);
    procedure SetLayerName(const Value: string);
    function GetVals: TIntegerListElement;
public
   procedure DoIt; override;
   property  Vals: TIntegerListElement read GetVals;
published
   property Values: string read FValues write SetValues;
   property LayerName: string read FLayerName write SetLayerName;
end;

implementation
uses
   Dialogs,
   e_Class,
   e_LHCModel,
   e_LHCElements,
   e_Geometry,
   e_GeometryElement,
   e_TopolElements;

{ TForestBorderTestTool }

procedure TForestBorderTestTool.DoIt;
var
   L: TPolyLineElement;
   i: integer;
   E: TEntity;
   P1,P2: TEntity;
   Are: TPolyLineAreaElement;
   TestRef: boolean;
   TestClass: TElementClass;
begin
   Result := True;
   if FrameClass = nil then
   begin
      if FrameClassName <> '' then
      begin
         FrameClass := GetOrbisClass(FrameClassName);
         if FrameClass = nil then
            ShowMessage ('ForestBorderTestTool: UnknownClass - '+FrameClassName);
      end;
      if FrameClass = nil then
         FrameClass := Tgis_Porost;
   end;
   if RefFrameClass = nil then
   begin
      if RefFrameClassName <> '' then
      begin
         RefFrameClass := GetOrbisClass(RefFrameClassName);
         if RefFrameClass = nil then
            ShowMessage ('ForestBorderTestTool: UnknownClass - '+RefFrameClassName);
      end;
      if RefFrameClass = nil then
         RefFrameClass := Tgis_PorostniSkupina;
   end;
   TestRef := false;
   if RefFrameClass <> Tgis_PorostniSkupina then
   begin
      TestRef := true;
      TestClass := Tgis_PorostniSkupina;
   end;
   if not (Input is TPolyLineElement) then exit;
   L := TPolyLineElement(Input);
   P1 := nil;
   P2 := nil;
   for i := 0 to L.OwnersCount - 1 do
   begin
      E := L.Owners[i];
      if not (E is TPolyLineAreaElement) then continue;
      Are := TPolyLineAreaElement(E);
      if Are.IsHole then
         Are := TPolyLineAreaElement(Are.HoleOwnerArea);
      if Are <> nil then
      begin
         if TestRef then
         begin
            P2 := Are.FindClassInOwners(TestClass);
            if P2 = nil then
               P2 := Are.FindClassInOwners(RefFrameClass)
            else
               P2 := nil;   
         end
         else
            P2 := Are.FindClassInOwners(RefFrameClass);
      end;
      if (P2 <> nil) and (RefFrameClass <> FrameClass) then
         P2 := P2.FindClassInOwners(FrameClass);
      if P2 <> nil then
      begin
         if P1 = nil then
         begin
            P1 := P2;
            P2 := nil;
         end
         else
            Break;
      end;
   end;
   if P1 = nil then
   begin
      exit;
   end;
   if P2 = nil then
   begin
      result := false;
      exit;
   end;
   if not (P1 = P2) then
   begin
      result := false;
      exit;
   end;
end;

procedure TForestBorderTestTool.SetFrameClass(const Value: TElementClass);
begin
  FFrameClass := Value;
end;

procedure TForestBorderTestTool.SetFrameClassName(const Value: string);
begin
   FFrameClassName := Value;
end;

procedure TForestBorderTestTool.SetRefFrameClass(const Value: TElementClass);
begin
  FRefFrameClass := Value;
end;

procedure TForestBorderTestTool.SetRefFrameClassName(const Value: string);
begin
  FRefFrameClassName := Value;
end;

{ TPolyLineISFromZAKTool }

procedure TPolyLineISFromZAKTool.DoIt;
var
   L: TPolyLineElement;
   i: integer;
   E: TEntity;
   TL: TTopolLineElement;
begin
   Result := True;
   if not (Input is TPolyLineElement) then exit;
   L := TPolyLineElement(Input);
   for i := 0 to L.OwnersCount - 1 do
   begin
      E := L.Owners[i];
      if not (E is TTopolLineElement) then continue;
      TL := TTopolLineElement(E);
      E := TL.FindOwnerName('LIN_ZAK');
      if E <> nil then
      begin
         Result := false;
         exit;
      end;
   end;
end;

{ TSRFBorderTestTool }

procedure TSRFBorderTestTool.DoIt;
var
   L: TPolyLineElement;
   i: integer;
   E: TEntity;
   P1,P2: TGeometryElement;
   SP1,SP2: TSRFPlocha;
   TP1,TP2: TTezPlocha;
   Are: TPolyLineAreaElement;
   EC: TElementClass;
begin
   Result := True;
   if not (Input is TPolyLineElement) then exit;
   L := TPolyLineElement(Input);
   P1 := nil;
   P2 := nil;
   if TEZ then
      EC := TTEZPlocha
   else
      EC := TSRFPlocha;
   for i := 0 to L.OwnersCount - 1 do
   begin
      E := L.Owners[i];
      if not (E is TPolyLineAreaElement) then continue;
      Are := TPolyLineAreaElement(E);
      if Are.IsHole then
      begin
         Are := TPolyLineAreaElement(Are.HoleOwnerArea);
         if Are <> nil then
            P2 := TGeometryElement(Are.FindClassInOwners(EC));
      end
      else
         P2 := TGeometryElement(E.FindClassInOwners(EC));
      if P2 <> nil then
      begin
         if P1 = nil then
         begin
            P1 := P2;
            P2 := nil;
         end
         else
            Break;
      end;
   end;
   if P1 = nil then
   begin
      exit;
   end;
   if P2 = nil then
   begin
      result := false;
      exit;
   end;
   if TEZ then
   begin
      TP1 := TTezPlocha(P1);
      TP2 := TTezPlocha(P2);
      if not ((TP1.Zpusob = TP2.Zpusob) and (TP1.Nalehavost = TP2.Nalehavost)and
              (TP1.Znacka = TP2.Znacka) and (TP1._Skupina = TP2._Skupina)) then
      begin
         result := false;
         exit;
      end;
   end
   else
   begin
      SP1 := TSRFPlocha(P1);
      SP2 := TSRFPlocha(P2);
      if not (SP1.Znacka = SP2.Znacka) then
      begin
         result := false;
         exit;
      end;
   end;
end;

procedure TSRFBorderTestTool.SetTEZ(const Value: boolean);
begin
  FTEZ := Value;
end;

{ TPresnostTestTool }

procedure TPresnostTestTool.DoIt;
var
   L: TPolyLineElement;
   i: integer;
   E: TEntity;
   TL: TTopolLineElement;
begin
   Result := True;
   if not (Input is TPolyLineElement) then exit;
   L := TPolyLineElement(Input);
   for i := 0 to L.OwnersCount - 1 do
   begin
      E := L.Owners[i];
      if not (E is TTopolLineElement) then continue;
      TL := TTopolLineElement(E);
      E := TL.FindOwnerName(LayerName);
      if E <> nil then
      begin
         if Vals.ValueExists(TL.Presnost) then
         begin
            Result := false;
            exit;
         end;
      end;
   end;
end;

function TPresnostTestTool.GetVals: TIntegerListElement;
begin
   if FVals = nil then
   begin
      FVals := TIntegerListElement.Create;
      FVals.Name := 'Values';
      AddElement(FVals);
   end;
   result := FVals;
end;

procedure TPresnostTestTool.SetLayerName(const Value: string);
begin
  FLayerName := Value;
end;

procedure TPresnostTestTool.SetValues(const Value: string);
begin
   FValues := Value;
   Vals.ValuesString := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TForestBorderTestTool);
   RegisterOrbisClass(TSRFBorderTestTool);
   RegisterOrbisClass(TPolyLineISFromZAKTool);
   RegisterOrbisClass(TPresnostTestTool);
END.
