unit t_SimplePointDialog;

interface
uses
   e_Type,
   Elements,
   f_SimplePointDialog,
   t_OrbisDialog;

type
TSimplePointDialog = class(TOrbisDialog)
private
   FValue_: integer;
    FDruh: integer;
    FSymbol: integer;
    FValueName: string;
    FMaxValue: integer;
    FMinValue: integer;
    FValueCaption: string;
    FElementClass: TElementClass;
    FAllowAngleChange: boolean;
    FAngleCHange: boolean;
   function GetForm: TSimplePointDialog_Form;
   procedure SetColorList(const Value: string);
   procedure SetValue_(const Value: integer);
   function GetColorList: string;
   function GetColors: TIntegerListElement;
   procedure SetColors(const Value: TIntegerListElement);
    procedure SetDruh(const Value: integer);
    procedure SetSymbol(const Value: integer);
    function GetColor: integer;
    procedure SetValueName(const Value: string);
    procedure SetMaxValue(const Value: integer);
    procedure SetMinValue(const Value: integer);
    procedure SetValueCaption(const Value: string);
    function GetElementClassName: string;
    procedure SetElementClass(const Value: TElementClass);
    procedure SetElementClassName(const Value: string);
    procedure SetAllowAngleChange(const Value: boolean);
    procedure SetAngleCHange(const Value: boolean);
public
   procedure DoIt; override;
   procedure CreateForm; override;
   procedure DialogDoIt;
   property  Form: TSimplePointDialog_Form read GetForm;
   property  Colors: TIntegerListElement read GetColors;
   property  ElementClass: TElementClass read FElementClass write SetElementClass;
published
   property  Druh: integer read FDruh write SetDruh;
   property  Symbol: integer read FSymbol write SetSymbol;
   property  Color: integer read GetColor stored false;
   property  Value_: integer read FValue_ write SetValue_;
   property  MinValue: integer read FMinValue write SetMinValue;
   property  MaxValue: integer read FMaxValue write SetMaxValue;
   property  ValueName: string read FValueName write SetValueName;
   property  ValueCaption: string read FValueCaption write SetValueCaption;
   property  ColorList: string read GetColorList write SetColorList stored false;
   property  ElementClassName: string read GetElementClassName write SetElementClassName;
   property  AllowAngleChange: boolean read FAllowAngleChange write SetAllowAngleChange;
   property  AngleCHange: boolean read FAngleCHange write SetAngleCHange;
end;

implementation
uses
   Forms,
   SysUtils,
   e_Class,
   //e_LHCProps,
   e_Geometry,
   e_GeometryElement,
   m_Orbis;

{ TSimplePointDialog }

procedure TSimplePointDialog.CreateForm;
begin
   aForm := TSimplePointDialog_Form.Create(Application);
   aForm.Tool := Self;
   aForm.IsPossibleToClose := false;
end;

procedure TSimplePointDialog.DoIt;
var
   P: TPointElement;
   GP: TGeometryElement;
   S: string;
begin
   if input = nil then exit;
   if not(Input is TPointElement) then
   begin
      if not(Input is TGeometryElement) then exit;
      GP := TGeometryElement(Input);
      P := TPointElement(GP.Geometry);
      Value_ := GP.GetPropertyLongIntByName(ValueName);
   end
   else
   begin
      P := TPointElement(Input);
      if ElementClass = nil then exit;
      GP := TGeometryElement(ElementClass.Create);
      GP.Geometry := P;
   end;
   Output := GP;
   if not DontShowForm then
   begin
      Form.ed_Value.Value := Value_;
      inherited;
      if Result = 1 then
         Value_ := Form.ed_Value.Value;
   end;
   if DontShowForm or (Result = 1) then
   begin
      GP.SetPropertyLongIntByName('DRUH',Druh);
      GP.SetPropertyLongIntByName('Znacka',Symbol);
      GP.SetPropertyLongIntByName('Barva',Color);
      GP.SetPropertyLongIntByName(ValueName,Value_);
   end;
end;

function TSimplePointDialog.GetColor: integer;
begin
   result := 0;
   if Colors.NoElements then exit;
   if Value_ > Colors.LastIndex then exit;
   result := Colors.Integers[Value_];
end;

function TSimplePointDialog.GetColorList: string;
var
   i: integer;
   S: string;
begin
   result := '';
   for i := 0 to Colors.LastIndex do
   begin
      S := IntToStr(Colors.Integers[i]);
      if i = 0 then
         result := S
      else
         result := result + ',' + S;
   end;
end;

function TSimplePointDialog.GetColors: TIntegerListElement;
begin
   result := TIntegerListElement(FindOrCreateElementName(TIntegerListElement,'COLORS'));
end;

function TSimplePointDialog.GetForm: TSimplePointDialog_Form;
begin
   result := TSimplePointDialog_Form(aForm);
end;

procedure TSimplePointDialog.SetColorList(const Value: string);
begin
   Colors.FreeAll;
   Colors.ValuesString := Value;
end;


procedure TSimplePointDialog.SetColors(const Value: TIntegerListElement);
begin

end;


procedure TSimplePointDialog.SetDruh(const Value: integer);
begin
  FDruh := Value;
end;


procedure TSimplePointDialog.SetValue_(const Value: integer);
begin
   FValue_ := Value;
end;

procedure TSimplePointDialog.SetSymbol(const Value: integer);
begin
  FSymbol := Value;
end;

procedure TSimplePointDialog.SetValueName(const Value: string);
begin
   FValueName := Value;
end;

procedure TSimplePointDialog.SetMaxValue(const Value: integer);
begin
   FMaxValue := Value;
   Form.ed_Value.MaxValue := Value;
end;

procedure TSimplePointDialog.SetMinValue(const Value: integer);
begin
   FMinValue := Value;
   Form.ed_Value.MinValue := Value;
end;

procedure TSimplePointDialog.SetValueCaption(const Value: string);
begin
   FValueCaption := Value;
   Form.Label1.Caption := Value;
end;

function TSimplePointDialog.GetElementClassName: string;
begin
   result := '';
   if ElementClass <> nil then
      result := ElementClass.ClassName;
end;

procedure TSimplePointDialog.SetElementClass(const Value: TElementClass);
begin
   FElementClass := Value;
end;

procedure TSimplePointDialog.SetElementClassName(const Value: string);
begin
   ElementClass := GetOrbisClass(Value);
   if ElementClass = nil then
   begin
      PostTextMsg(tm_AddToProtocol,Name + ': ElementClass '+Value+' not found',nil);
   end;
end;

procedure TSimplePointDialog.DialogDoIt;
begin
   inherited DoIt;
end;

procedure TSimplePointDialog.SetAllowAngleChange(const Value: boolean);
begin
   FAllowAngleChange := Value;
   Form.b_Angle.Enabled := Value;
end;

procedure TSimplePointDialog.SetAngleCHange(const Value: boolean);
begin
  FAngleCHange := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSimplePointDialog)
END.
