unit t_SoupisTYP;

interface
uses
   Elements,
   e_Geometry,
   e_QTree,
   Tools;

type

TSoupisTyp = class(TTool)
  private
    FLIN_: TEntity;
    FAreTYP: TEntity;
    FARE_: TEntity;
    FLinTYP: TEntity;
    FLines: TPolyLineString;
    FLevelQT: TRectEntityQTree;
    FAreTypQTree: TRectEntityQTree;
    FLTs: TEntity;
    FSLTs: TEntity;
    FCaption: string;
    procedure SetARE_(const Value: TEntity);
    procedure SetAreTYP(const Value: TEntity);
    procedure SetLIN_(const Value: TEntity);
    procedure SetLinTYP(const Value: TEntity);
    function GetLines: TPolyLineString;
    procedure SetLevelQT(const Value: TRectEntityQTree);
    procedure SetAreTypQTree(const Value: TRectEntityQTree);
    function GetLTs: TEntity;
    function GetSLTs: TEntity;
    procedure SetCaption(const Value: string);
public
   procedure DoIt; override;
   procedure Process;
   procedure ProcessExcel;
   procedure prepareQTree;
   property  Lines: TPolyLineString read GetLines;
   property LevelQT: TRectEntityQTree read FLevelQT write SetLevelQT;
   property AreTypQTree: TRectEntityQTree read FAreTypQTree write SetAreTypQTree;
   property LTs: TEntity read GetLTs;
   property SLTs: TEntity read GetSLTs;
published
   property AreTYP: TEntity read FAreTYP write SetAreTYP;
   property LinTYP: TEntity read FLinTYP write SetLinTYP;
   property ARE_: TEntity read FARE_ write SetARE_;
   property LIN_: TEntity read FLIN_ write SetLIN_;
   property Caption: string read FCaption write SetCaption;
end;


implementation
uses
{$IF RTLVersion >= 18}    // BDS 2006
   Excel2000,
{$ELSE}
   Excel97,
{$IFEND}
   FOrms,
   Dialogs,
   u_Math,
   SysUtils,
   Variants,
   e_Class,
   e_GeometryElement,
   e_Type,
   g_Elems,
   t_AreMak,
   t_LineExtractor,
   t_QTreeTools,
   t_Sort;

{ TSoupisTyp }

procedure TSoupisTyp.DoIt;
begin
   if (ARETyp = nil) or (LinTYP = nil) or (ARE_ = nil) or (LIN_ = nil) then
   begin
      ShowMessage('Chyb vstupn parametry !');
      exit;
   end;
   StartProgress('Soupis typologie',0,0);
   ProgressText := 'pprava ...';
   PrepareQTree;
   Process;
   ProcessExcel;
   ShowMessage('Hotovo');
end;

procedure TSoupisTyp.SetARE_(const Value: TEntity);
begin
  FARE_ := Value;
end;

procedure TSoupisTyp.SetAreTYP(const Value: TEntity);
begin
  FAreTYP := Value;
end;

procedure TSoupisTyp.SetLIN_(const Value: TEntity);
begin
  FLIN_ := Value;
end;

procedure TSoupisTyp.SetLinTYP(const Value: TEntity);
begin
  FLinTYP := Value;
end;

procedure TSoupisTyp.prepareQTree;
var
   TQ: TAddEntityToQTree;
   LEX: TLineExtractor;
   TYPL: TPolyLineString;
   L: TPolyLineString;
   CL: TClearLines;
begin
   if LinTYP = nil then exit;
   if Lin_ = nil then exit;
   ProgressText := 'Koprovn lini';
   Application.ProcessMessages;
   TYPL := TPolyLineString.Create;
   AddElement(TYPL);
   L := TPolyLineString.Create;
   AddElement(L);
   LEX := TLineExtractor.Create;
   LEX.CopyLines := true;
   LEX.Input := LinTYP;
   LEX.Output := TYPL;
   LEX.DoIt;
   LEX.Input := Lin_;
   LEX.Output := L;
   LEX.DoIt;
   ProgressText := '... a spojen lini';
   Application.ProcessMessages;
   CL := TClearLines.Create;
   CL.FixedLines := TYPL;
   CL.Input := L;
   CL.SelfTest := true;
   //CL.SnapDistance := 0.5;//!!!!!!!!!!!!!!
   CL.DoIt;
   CL.Free;
   LEX.CopyLines := false;
   LEX.NoDuplicates := True;
   LEX.Output := Lines;
   LEX.Input := TYPL;
   LEX.DoIt;
   LEX.Input := L;
   LEX.DoIt;
   TYPL.Free;
   L.Free;
   LEX.Free;
   ProgressText := 'QTree lini...';
   Application.ProcessMessages;
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Lines;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
   ProgressText := 'QTree ploch...';
   Application.ProcessMessages;
   AreTypQTree :=TRectEntityQTree.Create;
   AreTypQTree.Name := 'ARETYP QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := ARETYP;
   TQ.Output := AreTypQTree;
   TQ.DoIt;
   TQ.Free;
end;

function TSoupisTyp.GetLines: TPolyLineString;
begin
   if FLines = nil then
   begin
      FLines := TPolyLineString.Create;
      AddElement(FLines);
   end;
   result := FLines;
end;

procedure TSoupisTyp.SetLevelQT(const Value: TRectEntityQTree);
begin
  FLevelQT := Value;
  AddElement(FLevelQT);
end;



procedure TSoupisTyp.SetAreTypQTree(const Value: TRectEntityQTree);
begin
   FAreTypQTree := Value;
   AddElement(FAreTypQTree);
end;

procedure TSoupisTyp.Process;
var
   i,j: integer;
   GE: TGeometryElement;
   GET: TGeometryElement;
   e: TElement;
   W: TQTreeWalker;
   Lin: TRectEntity;
   AM: TAreasFromClearLines;
   Areas: TAreaString;
   Area: TAreaElement;
   QTFinder: TFindRectEntityInQTreeByXY;
   FE: TFloatElement;
begin
   MaxProgress := ARE_.LastIndex;
   ProgressText := 'Hledn pekryv ...';
   QTFinder := TFindRectEntityInQTreeByXY.Create;
   for i := 0 to ARE_.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      e := ARE_.Elements[i];
      if not (e is TGeometryElement) then continue;
      GE := TGeometryElement(e);
      if GE.NoGeometry then continue;
      Lin := TRectEntity.Create;
      Lin.Name := 'L';
      if LevelQT = nil then exit;
      W := TQTreeWalker.Create;
      E := W.GetFirstElementForRectangle(LevelQT,GE.Minx,GE.MinY,GE.MaxX,GE.MaxY,1);
      while E <> nil do
      begin
         if not E.IsMyOwner(Lin) then
            Lin.AddAndUpdateRect(E);
         E := W.GetNextElement;
      end;
      W.Free;
      AM := TAreasFromClearLines.Create;
      AM.Input := Lin;
      AM.DoIt;
      Areas := TAreaString(AM.Output);
      for j := 0 to Areas.LastIndex do
      begin
         e := Areas.Elements[j];
         Area := TAreaElement(e);
         if not GE.IsHitByXY(Area.X,Area.Y,0) then continue;
         QTFinder.Input := AreTypQTree;
         QTFinder.X := Area.X;
         QTFinder.Y := Area.Y;
         QTFinder.Tolerance := 0;
         QTFinder.DoIt;
         if QTFinder.Output = nil then // neni typologie
         begin
            e := LTS.FindOrCreateElementName(TFloatElement,'?');
            FE := TFloatElement(e);
            FE.Value := FE.Value + Area.Area;
            e := SLTS.FindOrCreateElementName(TFloatElement,'?');
            FE := TFloatElement(e);
            FE.Value := FE.Value + Area.Area;
         end
         else
         begin
            GET := TGeometryElement(QTFinder.Output);
            e := LTS.FindOrCreateElementName(TFloatElement,GET.GetPropertyStringByName('LT'));
            FE := TFloatElement(e);
            FE.Value := FE.Value + Area.Area;
            e := SLTS.FindOrCreateElementName(TFloatElement,GET.GetPropertyStringByName('SLT'));
            FE := TFloatElement(e);
            FE.Value := FE.Value + Area.Area;
         end;
      end;
      Areas.Free;
      AM.Free;
      Lin.Free;
   end;
   QTFinder.Free;
end;

function TSoupisTyp.GetLTs: TEntity;
begin
   if FLTs = nil then
   begin
      FLTs := TOwnerEntity.Create;
      FLts.Name := 'LT';
      AddElement(FLTs);
   end;
   result := FLTs;
end;

function TSoupisTyp.GetSLTs: TEntity;
begin
   if FSLTs = nil then
   begin
      FSLTs := TOwnerEntity.Create;
      FSLTs.Name := 'SLT';
      AddElement(FSLTs);
   end;
   result := FSLTs;
end;


procedure TSoupisTyp.ProcessExcel;
var
   ExcelApplication: TExcelApplication;
   ExcelWorksheet: TExcelWorksheet;
   ExcelWorkbook: TExcelWorkbook;
   OV: OleVariant;
   EWB: _WorkBook;
   EWS: _WorkSheet;
   A,B: OleVariant;
   Format: OleVariant;
   SRow: string;
   LastRow: integer;
   FE: TFloatElement;
   i: integer;
   SortS: TSortElementsByString;
   SumLT: Double;
   SumSLT: Double;
begin
   ProgressText := 'Inicializace Excelu ...';
   Application.ProcessMessages;
   ExcelApplication := TExcelApplication.Create(Application);
   ExcelWorksheet   := TExcelWorksheet.Create(Application);
   ExcelWorkbook    := TExcelWorkbook.Create(Application);
   try
      ExcelApplication.Connect;
      ExcelApplication.Visible[0] := true;
   except
      ShowMessage('Nepodailo se navzat spojen s EXCELem !');
      ExcelWorksheet.Free;
      ExcelWorkbook.Free;
      ExcelApplication.Free;
      exit;
   end;
   try
      OV := True;
         //EWB := ExcelApplication.Workbooks.Add(OV,0);
      try
      ExcelApplication.Workbooks.Add(XlWBATemplate(xlWBATWorksheet),0);
      except
         //on E: Exception do ShowMessage(E.Message);
      end;
      EWB := ExcelApplication.Workbooks[1];
      ExcelWorkBook.ConnectTo(EWB);
      ExcelWorkSheet.ConnectTo(ExcelApplication.ActiveSheet as _WorkSheet);
      ExcelWorkSheet.Name := 'Typologie';

      SortS := TSortElementsByString.Create;
      Sorts.PropertyName := 'Name';
      SortS.Input := SLTs;
      SortS.DoIt;
      SortS.Input := LTs;
      SortS.DoIt;
      SortS.Free;
      SumLT := 0;
      SumSLT := 0;
      for i := 0 to LTs.LastIndex do
      begin
         FE := TFloatElement(LTs.Elements[i]);
         FE.Value := Simple_RoundTo(FE.Value/10000,-4);
         SumLT := SumLT + FE.Value;
      end;
      for i := 0 to SLTs.LastIndex do
      begin
         FE := TFloatElement(SLTs.Elements[i]);
         FE.Value := Simple_RoundTo(FE.Value/10000,-4);
         SumSLT := SumSLT + FE.Value;
      end;
      LastRow := 1;
      SRow := IntToStr(LastRow);
      A := 'A'+SRow;
      if Caption = '' then
         Caption := 'Bez nzvu...';
      ExcelWorkSheet.Range[A,A].Value := Caption;
      ExcelWorkSheet.Range[A,A].Font.Size := 12;
      ExcelWorkSheet.Range[A,A].Font.Bold := True;
      LastRow := LastRow + 1;
      SRow := IntToStr(LastRow);
      A := 'A'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'SLT';
      A := 'B'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'PLOCHA';
      A := 'C'+SRow;
      ExcelWorkSheet.Range[A,A].Value := '%';
      A := 'A'+SRow;
      B := 'C'+SRow;
      ExcelWorkSheet.Range[A,B].HorizontalAlignment := xlHAlignCenter;
      LastRow := LastRow + 1;
      SRow := IntToStr(LastRow);
      for i := 0 to SLTs.LastIndex do
      begin
         FE := TFloatElement(SLTs.Elements[i]);
         A := 'A'+SRow;
         ExcelWorkSheet.Range[A,A].Value := FE.Name;
         A := 'B'+SRow;
         ExcelWorkSheet.Range[A,A].Value := FE.Value;
         Format := FormatFloat('0.00',0);
         ExcelWorkSheet.Range[A,A].NumberFormat := Format;
         A := 'C'+SRow;
         ExcelWorkSheet.Range[A,A].Value := FE.Value / SumSLT;
         Format := FormatFloat('0.0',0)+'%';
         ExcelWorkSheet.Range[A,A].NumberFormat := Format;
         LastRow := LastRow + 1;
         SRow := IntToStr(LastRow);
      end;
      A := 'A'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'SUMA';
      A := 'B'+SRow;
      ExcelWorkSheet.Range[A,A].Value := SumSLT;
      Format := FormatFloat('0.00',0);
      ExcelWorkSheet.Range[A,A].NumberFormat := Format;
      A := 'C'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'ha';
      A := 'A'+SRow;
      B := 'C'+SRow;
      ExcelWorkSheet.Range[A,B].Font.Bold := true;

      LastRow := LastRow + 2;
      SRow := IntToStr(LastRow);
      A := 'A'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'LT';
      A := 'B'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'PLOCHA';
      A := 'C'+SRow;
      ExcelWorkSheet.Range[A,A].Value := '%';
      A := 'A'+SRow;
      B := 'C'+SRow;
      ExcelWorkSheet.Range[A,B].HorizontalAlignment := xlHAlignCenter;
      LastRow := LastRow + 1;
      SRow := IntToStr(LastRow);
      for i := 0 to LTs.LastIndex do
      begin
         FE := TFloatElement(LTs.Elements[i]);
         A := 'A'+SRow;
         ExcelWorkSheet.Range[A,A].Value := FE.Name;
         A := 'B'+SRow;
         ExcelWorkSheet.Range[A,A].Value := FE.Value;
         Format := FormatFloat('0.00',0);
         ExcelWorkSheet.Range[A,A].NumberFormat := Format;
         A := 'C'+SRow;
         ExcelWorkSheet.Range[A,A].Value := FE.Value / SumLT;
         Format := FormatFloat('0.0',0)+'%';
         ExcelWorkSheet.Range[A,A].NumberFormat := Format;
         LastRow := LastRow + 1;
         SRow := IntToStr(LastRow);
      end;
      A := 'A'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'SUMA';
      A := 'B'+SRow;
      ExcelWorkSheet.Range[A,A].Value := SumLT;
      Format := FormatFloat('0.00',0);
      ExcelWorkSheet.Range[A,A].NumberFormat := Format;
      A := 'C'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'ha';
      A := 'A'+SRow;
      B := 'C'+SRow;
      ExcelWorkSheet.Range[A,B].Font.Bold := true;

   except
      ShowMessage('Nepodailo se navzat spojen s EXCELem !(1)');
      ExcelWorksheet.Free;
      ExcelWorkbook.Free;
      ExcelApplication.Free;
      exit;
   end;
   ExcelWorksheet.Free;
   ExcelWorkbook.Free;
   ExcelApplication.Free;
end;

procedure TSoupisTyp.SetCaption(const Value: string);
begin
  FCaption := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSoupisTYP);
END.
