unit t_Synchronize_TABLE_Model;
{
   Nstroj pro synchronizaci DB tabulky a Obj. modelu
   ekvivalent spojen DB v TopoLu
}
interface
uses
   e_KatastrModel,
   e_LHCModel,
   f_Synchronize_TABLE_ParcelModel,
   f_Synchronize_TABLE_ForestModel,
   f_Synchronize_TABLE_Majitele,
   Tools;

type

TSynchronize_Table_Model = class(TTool)
  private
    FTableName: string;
    FErrorsToProtocol: boolean;
    procedure SetTableName(const Value: string);
    procedure SetErrorsToProtocol(const Value: boolean);
public
   procedure DoIt; override;
published
   property TableName: string read FTableName write SetTableName;
   property ErrorsToProtocol: boolean read FErrorsToProtocol write SetErrorsToProtocol;
end;

TSynchronize_Table_ParcelModel = class(TSynchronize_TABLE_Model)
  private
    Fdb_KatastrPropertyName: string;
    Fdb_ParpodPropertyName: string;
    Fdb_ParcisPropertyName: string;
    Fdb_DruhParcelyPropertyName: string;
    FParcelModel: Tgis_ParcelModel;
    FForm: TSynchronize_Table_ParcelModel_Form;
    procedure Setdb_DruhParcelyPropertyName(const Value: string);
    procedure Setdb_KatastrPropertyName(const Value: string);
    procedure Setdb_ParcisPropertyName(const Value: string);
    procedure Setdb_ParpodPropertyName(const Value: string);
    procedure SetForm(const Value: TSynchronize_Table_ParcelModel_Form);
    procedure SetParcelModel(const Value: Tgis_ParcelModel);
public
   constructor Create; override;
   destructor Destroy; override;
   procedure DoIt; override;
   procedure CreateForm;
   procedure FindParcelModel;
   property  Form: TSynchronize_Table_ParcelModel_Form read FForm write SetForm;
published
   property ParcelModel: Tgis_ParcelModel read FParcelModel write SetParcelModel;
   property db_KatastrPropertyName: string read Fdb_KatastrPropertyName write Setdb_KatastrPropertyName;
   property db_ParcisPropertyName: string read Fdb_ParcisPropertyName write Setdb_ParcisPropertyName;
   property db_ParpodPropertyName: string read Fdb_ParpodPropertyName write Setdb_ParpodPropertyName;
   property db_DruhParcelyPropertyName: string read Fdb_DruhParcelyPropertyName write Setdb_DruhParcelyPropertyName;
end;

TSynchronize_Table_ForestModel = class(TSynchronize_TABLE_Model)
  private
    FForm: TSynchronize_Table_ForestModel_Form;
    FForestModel: Tgis_ForestModel;
    procedure SetForestModel(const Value: Tgis_ForestModel);
    procedure SetForm(const Value: TSynchronize_Table_ForestModel_Form);
public
   constructor Create; override;
   destructor Destroy; override;
   procedure DoIt; override;
   procedure CreateForm;
   procedure FindForestModel;
   property  Form: TSynchronize_Table_ForestModel_Form read FForm write SetForm;
published
   property ForestModel: Tgis_ForestModel read FForestModel write SetForestModel;
end;

TSynchronize_Table_Majitele = class(TSynchronize_TABLE_Model)
  private
    FMajitele: TGis_Majitele;
    FForm: TSynchronize_Table_Majitele_Form;
    procedure SetForm(const Value: TSynchronize_Table_Majitele_Form);
    procedure SetMajitele(const Value: TGis_Majitele);
public
   constructor Create; override;
   destructor Destroy; override;
   procedure DoIt; override;
   procedure CreateForm;
   procedure FindMajitele;
   property  Form: TSynchronize_Table_Majitele_Form read FForm write SetForm;
published
   property Majitele: TGis_Majitele read FMajitele write SetMajitele;
end;
{
TSynchronize_Table_GeneralModel = class(TSynchronize_TABLE_Model)
  private
    procedure SetForm(const Value: TSynchronize_Table_Majitele_Form);
    procedure SetMajitele(const Value: TGis_Majitele);
public
   constructor Create; override;
   destructor Destroy; override;
   procedure DoIt; override;
   procedure CreateForm;
   procedure FindMajitele;
   property  Form: TSynchronize_Table_Majitele_Form read FForm write SetForm;
published
   property Majitele: TGis_Majitele read FMajitele write SetMajitele;
end;
}
implementation
uses
   Forms,
   Elements,
   e_Class,
   Worlds;

{ TSynchronize_Table_ParcelModel }

constructor TSynchronize_Table_ParcelModel.Create;
begin
   inherited;
   CreateForm;
end;

procedure TSynchronize_Table_ParcelModel.CreateForm;
begin
   Form := TSynchronize_Table_ParcelModel_Form.Create(Application);
   Form.Tool := Self;
end;

destructor TSynchronize_Table_ParcelModel.Destroy;
begin
   if Form <> nil then
   begin
      Form.Free;
      Form := nil;
   end;
   inherited;
end;

procedure TSynchronize_Table_ParcelModel.DoIt;
begin
   FindParcelModel;
   if ParcelModel = nil then exit;
   Form.ParcelModel := ParcelModel;
   Form.cb_Protocol.Checked := ErrorsToProtocol;
   Form.Protocol := ErrorsToProtocol;
   Form.ShowModal;
end;

procedure TSynchronize_Table_ParcelModel.FindParcelModel;
var
   e: TElement;
begin
   e := World.ObjectModel.FindElementName('DATA');
   if e = nil then exit;
   ParcelModel := Tgis_ParcelModel(TEntity(e).FindElementClass(Tgis_ParcelModel));
end;

procedure TSynchronize_Table_ParcelModel.Setdb_DruhParcelyPropertyName(
  const Value: string);
begin
  Fdb_DruhParcelyPropertyName := Value;
end;

procedure TSynchronize_Table_ParcelModel.Setdb_KatastrPropertyName(
  const Value: string);
begin
  Fdb_KatastrPropertyName := Value;
end;

procedure TSynchronize_Table_ParcelModel.Setdb_ParcisPropertyName(
  const Value: string);
begin
  Fdb_ParcisPropertyName := Value;
end;

procedure TSynchronize_Table_ParcelModel.Setdb_ParpodPropertyName(
  const Value: string);
begin
  Fdb_ParpodPropertyName := Value;
end;

procedure TSynchronize_Table_ParcelModel.SetForm(
  const Value: TSynchronize_Table_ParcelModel_Form);
begin
  FForm := Value;
end;

procedure TSynchronize_Table_ParcelModel.SetParcelModel(
  const Value: Tgis_ParcelModel);
begin
  FParcelModel := Value;
end;

{ TSynchronize_Table_Model }

procedure TSynchronize_Table_Model.DoIt;
begin
  inherited;

end;

procedure TSynchronize_Table_Model.SetErrorsToProtocol(
  const Value: boolean);
begin
  FErrorsToProtocol := Value;
end;

procedure TSynchronize_Table_Model.SetTableName(const Value: string);
begin
  FTableName := Value;
end;

{ TSynchronize_Table_ForestModel }

constructor TSynchronize_Table_ForestModel.Create;
begin
   inherited;
   CreateForm;
end;

procedure TSynchronize_Table_ForestModel.CreateForm;
begin
   Form := TSynchronize_Table_ForestModel_Form.Create(Application);
   Form.Tool := Self;
end;

destructor TSynchronize_Table_ForestModel.Destroy;
begin
   if Form <> nil then
   begin
      Form.Free;
      Form := nil;
   end;
   inherited;
end;

procedure TSynchronize_Table_ForestModel.DoIt;
begin
   FindForestModel;
   if ForestModel = nil then exit;
   Form.ForestModel := ForestModel;
   Form.cb_Protocol.Checked := ErrorsToProtocol;
   Form.Protocol := ErrorsToProtocol;
   Form.ShowModal;
end;

procedure TSynchronize_Table_ForestModel.FindForestModel;
var
   e: TElement;
begin
   e := World.ObjectModel.FindElementName('DATA');
   if e = nil then exit;
   ForestModel := Tgis_ForestModel(TEntity(e).FindElementClass(Tgis_ForestModel));
end;

procedure TSynchronize_Table_ForestModel.SetForestModel(const Value: Tgis_ForestModel);
begin
  FForestModel := Value;
end;

procedure TSynchronize_Table_ForestModel.SetForm(const Value: TSynchronize_Table_ForestModel_Form);
begin
  FForm := Value;
end;

{ TSynchronize_Table_Majitele }

constructor TSynchronize_Table_Majitele.Create;
begin
   inherited;
   CreateForm;
end;

procedure TSynchronize_Table_Majitele.CreateForm;
begin
   Form := TSynchronize_Table_Majitele_Form.Create(Application);
   Form.Tool := Self;
end;

destructor TSynchronize_Table_Majitele.Destroy;
begin
   if Form <> nil then
   begin
      Form.Free;
      Form := nil;
   end;
   inherited;
end;

procedure TSynchronize_Table_Majitele.DoIt;
begin
   FindMajitele;
   if Majitele = nil then exit;
   Form.Majitele := Majitele;
   Form.cb_Protocol.Checked := ErrorsToProtocol;
   Form.Protocol := ErrorsToProtocol;
   Form.ShowModal;
end;

procedure TSynchronize_Table_Majitele.FindMajitele;
var
   e: TElement;
begin
   if (Majitele <> nil) and (TElement(Majitele) is Tgis_Majitele) then exit;
   e := World.ObjectModel.FindElementName('DATA');
   if e = nil then exit;
   Majitele := Tgis_Majitele(TEntity(e).FindElementClass(Tgis_Majitele));
end;

procedure TSynchronize_Table_Majitele.SetForm(const Value: TSynchronize_Table_Majitele_Form);
begin
  FForm := Value;
end;

procedure TSynchronize_Table_Majitele.SetMajitele(const Value: TGis_Majitele);
begin
  FMajitele := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSynchronize_Table_ParcelModel);
   RegisterOrbisClass(TSynchronize_Table_ForestModel);
   RegisterOrbisClass(TSynchronize_Table_Majitele);
   //RegisterOrbisClass(TSynchronize_Table_GeneralModel);
END.


