unit t_TextEditor;
interface
uses
   Elements,
   e_TopolElements,
   e_BGIF,
   e_QTree,
   e_Type,
   g_Elems,
   m_Intera,
   t_GraphicEditors,
   u_Symbol;

type

TSimpleTextEditor = class(TGraphicEditor)
  private
    FPixelTolerance: integer;
    FLevelName: string;
    FLevel: TEntity;
    FLevelQT: TRectEntityQTree;
    FY: double;
    FX: double;
    FBGIF: TBGIFont;
    FTextInfo: TBGIText;
    FR: TRectEntity;
    FDragBusy: boolean;
    FAuxProjectsName: string;
    FAuxLevelName: string;
    FAuxProjects: TEntity;
    procedure SetLevel(const Value: TEntity);
    procedure SetLevelName(const Value: string);
    procedure SetLevelQT(const Value: TRectEntityQTree);
    procedure SetPixelTolerance(const Value: integer);
    procedure SetX(const Value: double);
    procedure SetY(const Value: double);
    function GetBGIF: TBGIFont;
    function GetTextInfo: TBGIText;
    function GetR: TRectEntity;
    procedure SetDragBusy(const Value: boolean);
    procedure SetAuxLevelName(const Value: string);
    procedure SetAuxProjects(const Value: TEntity);
    procedure SetAuxProjectsName(const Value: string);
public
   procedure   XORDraw; override;
   procedure   ReDraw(aColor: integer);
   procedure   DrawText(XORMode: boolean);
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_StartDrag(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_DragEnd(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_Activate; override;
   procedure RegisterCursors; override;
   procedure PrepareQTree; virtual;
   procedure CHangeSelectCursor;
   procedure CHangeSetupCursor;
   procedure CHeckRectangle(TE: TTopolTextElement);
   procedure CreateCopy;
   procedure FindLevel_ProjectEntity(aElem: TElement; var aLevel,aProjE: TEntity);
   property LevelQT: TRectEntityQTree read FLevelQT write SetLevelQT;
   property Level: TEntity read FLevel write SetLevel;
   property AuxProjects: TEntity read FAuxProjects write SetAuxProjects;
   property X: double read FX write SetX;
   property Y: double read FY write SetY;
   property BGIF: TBGIFont read GetBGIF;
   property TextInfo: TBGIText read GetTextInfo;
   property R: TRectEntity read GetR;
   property DragBusy: boolean read FDragBusy write SetDragBusy;
published
   property LevelName: string read FLevelName write SetLevelName;
   property AuxProjectsName: string read FAuxProjectsName write SetAuxProjectsName;
   property AuxLevelName: string read FAuxLevelName write SetAuxLevelName;
   property PixelTolerance: integer read FPixelTolerance write SetPixelTolerance;
end;

implementation
uses
   Classes,
   Controls,
   Dialogs,
   Forms,
   Graphics,
   cpConvrt,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   MathLib,
   m_Orbis,
   t_QTreeTools;

{ TSimpleTextEditor }


procedure TSimpleTextEditor.CHangeSelectCursor;
var
   d: double;
   CI: integer;
begin
   if QTFinder.Busy then exit;
   d := Device.Projection.GetModelLength(PixelTolerance);
   CI := 0;
   QTFinder.Input := LevelQT;
   QTFinder.X := X;
   QTFInder.Y := Y;
   QTFinder.Tolerance := d;
   QTFinder.DoIt;
   Input := QTFinder.Output;
   if Input <> nil then
   begin
      CI := 1;
   end;
   CursorIndex := CI;
end;

procedure TSimpleTextEditor.CHangeSetupCursor;
var
   i: integer;
   TE: TTopolTextElement;
   A,B: TDoubleCoord;
   L: double;
   Angle: double;
begin
   TE := TTopolTextElement(Input);
   if TE = nil then exit;
   XORDraw;
   A.X := TE.StartPoint.X;
   A.Y := TE.StartPoint.Y;
   B.X := X;
   B.Y := Y;
   XYToPolar(A,B,L,Angle);
   TE.Direction := Angle;
   XORDRaw;
{
   if Input = nil then exit;
   DragFous := nil;
   TA := TTopolTextElement(Input);
   CI := 1;
   Tol := Device.Projection.GetModelLength(5);
   for i := 0 to TA.LastIndex do
   begin
      e := TA.Elements[i];
      if not (e.ClassType = TAnnotationAbscissa) then continue;
      AA := TAnnotationAbscissa(e);
      if InTolerance(AA.X1,AA.Y1,X,Y,Tol)then
      begin
         CI := 2;
         DragFous := AA;
         break;
      end;
   end;
   CursorIndex := CI;
}
end;


procedure TSimpleTextEditor.CHeckRectangle(TE: TTopolTextElement);
var
   TX: TTextElement;
begin
   TX := TTextElement(TE.Geometry);
   if TX = nil then exit;
   if Device <> nil then
   begin
    //SetInterpretTopProps(TTopoLText(Element).ObjectOwner);

      TextInfo.Druh       := TE.Druh;
      TextInfo.Barva      := TE.Barva;
      TextInfo.Znacka     := TE.Znacka;
      TextInfo.Font       := TE.TopolFont;
      TextInfo.Vyska      := TE.Height;
      TextInfo.KoefSirky  := TE.WidthCoef;
      TextInfo.Smer       := TE.Direction;
      TextInfo.Sklon      := TE.Slope;
      TextInfo.Rozestupy  := TE.Distance;
      TextInfo.Atributy   := TE.TextAtrib;//u_Symbol.TTextAtrib(TE.TextAtrib);
      TextInfo.RezimTextu := TE.TextMode;
      TextInfo.VychoziBod.x := TE.StartPoint.x;
      TextInfo.VychoziBod.y := TE.StartPoint.y;
      if TE.CodePage = cpWindows then
      begin
         TextInfo.CodePage   := cpLatin2;
         TextInfo.Veta       := WinTo852(TX.Text);
      end
      else
      begin
         TextInfo.CodePage   := TE.CodePage;
         TextInfo.Veta       := TX.Text;
      end;
      BGIF.TextInfo := TextInfo;
      BGIF.CalculateOpsanyObdelnik;
      TX.MinX := TextInfo.Rect.MinX;
      TX.MinY := TextInfo.Rect.MinY;
      TX.MaxX := TextInfo.Rect.MaxX;
      TX.MaxY := TextInfo.Rect.MaxY;
      TX.AbsRefX := TE.StartPoint.x;
      TX.AbsRefY := TE.StartPoint.y;
   end;
end;

procedure TSimpleTextEditor.CreateCopy;
var
   TEO,TEN: TTopolTextElement;
   TXO,TXN: TTextElement;
   aX,aY: double;
   aLevel: TEntity;
begin
   if Input = nil then exit;
   TEO := TTopolTextElement(Input);
   TXO := TTextElement(TEO.Geometry);
   aX := TEO.StartPoint.X;
   aY := TEO.StartPoint.Y;
   TEN := TTopolTextElement.Create;
   TEN.UpdateRect(TEO);
   TEN.TextAtrib := TEO.TextAtrib;
   TEN.CodePage := TEO.CodePage;
   TEN.TopolFont := TEO.TopolFont;
   TEN.Height := TEO.Height;
   TEN.WidthCoef := TEO.WidthCoef;
   TEN.Direction := TEO.Direction;
   TEN.Slope := TEO.Slope;
   TEN.Distance := TEO.Distance;
   TEN.TextMode := TEO.TextMode;
   TEN.Cislo := TEO.Cislo;
   TEN.Druh  := TEO.Druh;
   TEN.Barva := TEO.Barva;
   TEN.Znacka:= TEO.Znacka;

   TXN := TTextElement(CreateElementCopy(TXO));
   TEN.Geometry := TXN;
   Redraw(0);
   if TEO.IsMyOwner(Level) then
      aLevel := Level
   else
   begin
      aLevel := TEntity(TEO.FindOwnerName(Level.Name));
   end;
   aLevel.NextElement := TEN;
   TXN.DeltaX := 10;
   TXN.DeltaY := 10;
   TEN.StartPoint.X := aX + 10;
   TEN.StartPoint.Y := aY + 10;
   LevelQT.NextRectEntity := TEN;
   Input := TEN;
   Redraw($AA00FF);
end;

procedure TSimpleTextEditor.DrawText(XORMode: boolean);
var
  TE: TTopolTextElement;
  TX: TTextElement;
  E: TElement;
  i: integer;
  aR: double;
begin
   if BGIF.CHybaFontu then
   begin
      PostTextMsg(tm_AddToProtocol,'Annotation Text interpret - FontError !',nil);
      exit;
   end;
   if Input = nil then exit;
   TE := TTopolTextElement(Input);
   TX := TTextElement(TE.Geometry);
   try
      TextInfo.Druh       := TE.Druh;
      TextInfo.Barva      := TE.Barva;
      TextInfo.Znacka     := TE.Znacka;
      TextInfo.Font       := TE.TopolFont;
      TextInfo.Vyska      := TE.Height;
      TextInfo.KoefSirky  := TE.WidthCoef;
      TextInfo.Smer       := TE.Direction;
      TextInfo.Sklon      := TE.Slope;
      TextInfo.Rozestupy  := TE.Distance;
      TextInfo.Atributy   := u_Symbol.TTextAtrib(TE.TextAtrib);
      TextInfo.RezimTextu := TE.TextMode;
      TextInfo.VychoziBod.x := TE.StartPoint.x;
      TextInfo.VychoziBod.y := TE.StartPoint.y;
      if TE.CodePage = cpWindows then
      begin
         TextInfo.CodePage   := cpLatin2;
         TextInfo.Veta       := WinTo852(TX.Text);
      end
      else
      begin
         TextInfo.CodePage   := TE.CodePage;
         TextInfo.Veta       := TX.Text;
      end;
      BGIF.TextInfo := TextInfo;
      BGIF.NastavInfoPodleDruhu(TextInfo.Druh);
      BGIF.CalculateOpsanyObdelnik;
      Device.Pen.Style := psSolid;
      if XORMode then
      begin
         Device.Pen.Mode:= pmXOR;
         Device.Pen.Color := $909090;
      end
      else
         Device.Pen.Mode := pmCopy;
      BGIF.ZobrazText(False);

   except
   end;

end;


procedure TSimpleTextEditor.FindLevel_ProjectEntity(aElem: TElement; var aLevel, aProjE: TEntity);
begin
   aLevel := Level;
   aProjE := nil;
   if aElem = nil then
      exit;
   if aElem.IsMyOwner(Level)then
      exit;
   aLevel := aElem.FindOwnerName(Level.Name);
   if aLevel = nil then exit;
   aProjE := aLevel.FindClassWithOwnerInOwners(TEntity,AuxProjects);
end;

function TSimpleTextEditor.GetBGIF: TBGIFont;
begin
   if FBGIF = nil then
   begin
      FBGIF := TBGIFont.Create;
      AddElement(FBGIF);
      if Device <> nil then
         FBGIF.SetDevice(Device);
   end;
   result := FBGIF;
end;


function TSimpleTextEditor.GetR: TRectEntity;
begin
   if FR = nil then
   begin
      FR := TRectEntity.Create;
      AddElement(FR);
   end;
   result := FR;
end;


function TSimpleTextEditor.GetTextInfo: TBGIText;
begin
   if FTextInfo = nil then
   begin
      FTextInfo := TBGIText.Create;
      AddElement(FTextInfo);
   end;
   result := FTextInfo;
end;

procedure TSimpleTextEditor.On_Activate;
begin
   EditorStatus := gesPrepare;
   HighActivity := true;
   if not Active then
      PrepareQTree;
   EditorStatus := gesSelect;
   QTFinder.Busy := false;
   inherited;
   HighActivity := false;
end;

procedure TSimpleTextEditor.On_tm_DragEnd(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   TE: TTopolTextElement;
   aLevel,aProjE: TEntity;
begin
   //ShowMessage('DragEnd');
   //XORDraw;
   if Input = nil then exit;
   CursorIndex := 0;
   aLevel := Level;
   aProjE := nil;
   if EditorStatus = gesSetup then
   begin
      XORDraw;
      EditorStatus := gesSelect;
      //Redraw(0);
      DragBusy := false;
      //exit;
   end
   else
   begin
      FindLevel_ProjectEntity(Input, aLevel, aProjE);
      TE := TTopolTextElement(Input);
      RemoveElementOwners(Input);
      RemoveElementOwners(TE);
      CHeckRectangle(TE);
      aLevel.AddElement(TE);
      LevelQT.NextRectEntity := TE;
   end;
   PostMsg(tm_ModelChanged,nil,aProjE,false);
   PostMsgToOwners(tm_ReviewByElement, R, false);
   //Application.ProcessMessages;
   //Application.ProcessMessages;

   Redraw(0);
   DragBusy := false;
end;

procedure TSimpleTextEditor.On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   aX,aY: double;
   TE: TTopolTextElement;
   TX: TTextElement;
   G: TGeometry;
   ix: integer;
   e: TElement;
begin
   if DragBusy then exit;
   DragBusy := true;
   aX:= Device.Projection.GetModelX(aMessage.X);
   aY:= Device.Projection.GetModelY(aMessage.Y);
   TE := TTopolTextElement(Input);
   if TE = nil then exit;
   TX := TTextElement(TE.Geometry);
   if EditorStatus = gesSelect then
   begin
      XORDraw;
      TE.StartPoint.X := TE.StartPoint.X + aX - X;
      TE.StartPoint.Y := TE.StartPoint.Y + aY - Y;
      TX.X := TE.StartPoint.X;
      TX.Y := TE.StartPoint.Y;
      XORDraw;
   end
   else
   if EditorStatus = gesSetup then
   begin
   end;
   X := aX;
   Y := aY;
   DragBusy := false;
end;

procedure TSimpleTextEditor.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   i: integer;
   e: TElement;
begin
   //!!!!!!! Novy text
   if Input = nil then exit;
   R.Rect.Reset;
   R.UpdateRect(Input);
   R.Rect.GrowbyKoef(1.5);
   //Redraw($A0A0A0);
   XORDraw;
end;

procedure TSimpleTextEditor.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
  aX, aY: integer;
begin
   aX:= aMessage.x;
   aY:= aMessage.y;
   X:= Device.Projection.GetModelX(aX);
   Y:= Device.Projection.GetModelY(aY);
   if not Active then exit;
   if EditorStatus = gesSelect then
      ChangeSelectCursor
   else
   if EditorStatus = gesSetup then
      CHangeSetupCursor;
   {
   else
   if EditorStatus = gesAdd then
      CHangeAddCursor;
   }
   {
   else
   begin
      if PolyElement = nil then exit;
      ChangeCursor;
   end;
   }
end;

procedure TSimpleTextEditor.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   TE: TTopolTextElement;
   TX: TTextElement;
   S: string;
   New: boolean;
   aStyl: integer;
   aLevel: TEntity;
   aProjE: TEntity;
begin
   TX := nil;
   New := false;
   New := Input = nil ;
   TE := TTopolTextElement(Input);
   if not New then
      TX := TTextElement(TE.Geometry);
   //aProject := Level.FindElementClass(TOrbisLHCProject);
   //XORDraw;
   if EditorStatus = gesSetup then
   begin
      FindLevel_ProjectEntity(TE,aLevel,aProjE);
      RemoveElementOwners(TE);
      PostMsg(tm_ModelChanged,nil,aProjE,false);
      PostMsgToOwners(tm_ReviewByElement, R, false);
      CHeckRectangle(TE);
      aLevel.AddElement(TE);
      LevelQT.NextRectEntity := TE;
      EditorStatus := gesSelect;
      Redraw(0);
      Exit;
   end;
   EditorStatus := gesSelect;
   //XORDraw;
   if ((aMessage.Button = mbRight)or New) and (Dialog <> nil) then
   begin
      //Dialog.SetPropertyBooleanByName('IsVisible',TTopolTextElement(Input).Visible);
      //Dialog.SetPropertyBooleanByName('NoGeometry',NoGeometry);
      Dialog.SetPropertyLongIntByName('Styly',longInt(BGIF.TextStyles));
      if New Then
      begin
         S := 'Nov text';
      end
      else
      begin
         S := TX.Text;
         if TE.CodePage = cpLatin2 then
            S := cp852ToWin(TX.Text);
         Dialog.SetPropertyLongIntByName('Styl',longInt(TE.Druh));
      end;
      Dialog.SetPropertyBooleanByName('New',New);
      Dialog.SetPropertyStringByName('Text',S);
      Dialog.DoIt;
      case Dialog.Result of
         1: // OK
         begin
            //TTopolTextElement(Input).Visible := Dialog.GetPropertyBooleanByName('IsVisible');
            // styly a dalsi ptakoviny
            aStyl := Dialog.GetPropertyLongIntByName('Styl');
            S := Dialog.GetPropertyStringByName('Text');
            aLevel := Level;
            aProjE := nil;
            if New then
            begin
               TextInfo.Druh       := aStyl;
               { vechno podle stylu .....
               TextInfo.Barva      := TE.Barva;
               TextInfo.Znacka     := TE.Znacka;
               TextInfo.Font       := TE.TopolFont;
               TextInfo.Vyska      := TE.Height;
               TextInfo.KoefSirky  := TE.WidthCoef;
               TextInfo.Sklon      := TE.Slope;
               TextInfo.Rozestupy  := TE.Distance;
               TextInfo.RezimTextu := TE.TextMode;
               }
               TextInfo.VychoziBod.x := X;
               TextInfo.VychoziBod.y := Y;
               TextInfo.Smer       := 0;
               TextInfo.Atributy   := [OkT,T30];
               case DefaultCodePage of
                  cpWindows:
                  begin
                     TextInfo.CodePage   := cpWindows;
                     TextInfo.Veta       := S;
                  end;
               else
                  begin
                    TextInfo.CodePage   := cpLatin2;
                    TextInfo.Veta       := WinTo852(S);
                  end;
               end;
               BGIF.TextInfo := TextInfo;
               BGIF.NastavInfoPodleDruhu(TextInfo.Druh);
               BGIF.CalculateOpsanyObdelnik;

               TE := TTopoLTextElement.Create;
               TE.MinX := TextInfo.Rect.MinX;
               TE.MinY := TextInfo.Rect.MinY;
               TE.MaxX := TextInfo.Rect.MaxX;
               TE.MaxY := TextInfo.Rect.MaxY;
               //**************
               TX:= TTextElement.Create;
               TX.UpdateRect(TE);
               //**************
               TE.Geometry := TX;
               //TT.Cislo       := TextInfo.Cislo; na cislo sere pes
               TE.Druh         := TextInfo.Druh;
               TE.TextAtrib    := TextInfo.Atributy;
               TE.Barva        := TextInfo.Barva;
               TE.Znacka       := TextInfo.Znacka;
               TE.TextMode     := TextInfo.RezimTextu;
               TE.Druh         := TextInfo.Druh;
               TE.Direction    := TextInfo.Smer;
               TE.TopolFont    := TextInfo.Font;
               TE.StartPoint.x := TextInfo.VychoziBod.X;
               TE.StartPoint.y := TextInfo.VychoziBod.Y;
               //**************
               TX.x            := TE.StartPoint.x;
               TX.y            := TE.StartPoint.y;
               //**************
               TE.Height       := TextInfo.Vyska;
               TE.WidthCoef    := TextInfo.KoefSirky;
               TE.Slope        := TextInfo.Sklon;
               TE.Distance     := TextInfo.Rozestupy;
               //TE.CodePage     := cpLatin2;
               //TX.Text         := WinTo852(S);
               TE.CodePage     := TextInfo.CodePage;
               TX.Text         := TextInfo.Veta;
               Input := TE;
            end
            else
            begin
               FindLevel_ProjectEntity(TE, aLevel, aProjE);
               PostMsgToOwners(tm_ReviewByElement, R, false);
               if TE.CodePage = cpLatin2 then
                  TX.Text := WinTo852(S)
               else
                  TX.Text := S;   
               TE.Druh := Dialog.GetPropertyLongIntByName('Styl');
               RemoveElementOwners(TE);
               CHeckRectangle(TE);
            end;
            aLevel.AddElement(TE);
            LevelQT.NextRectEntity := TE;
            PostMsg(tm_ModelChanged,nil,aProjE,false);
            Redraw(0);
         end;
         3: // posun fousu
         begin
            //Application.ProcessMessages;
            Redraw(clWhite);
            XORDraw;
            EditorStatus := gesSetup;
            //XORDraw;
            exit;
         end;
         4: // pripojeni dalsi plochy
         begin
            {
            if QAdd.NoElements then
            begin
               ShowMessage('Nen co pidvat!!!');
            end
            else
            begin
               EditorStatus := gesAdd;
               exit;
            end;
            }
         end;
         5: // kopie
         begin
            CreateCopy;
            FindLevel_ProjectEntity(Input, aLevel, aProjE);
            PostMsg(tm_ModelChanged,nil,aProjE,false);
            exit;
         end;
         6: // Odpojeni
         begin
            FindLevel_ProjectEntity(Input, aLevel, aProjE);
            Input.Free;
            Input := nil;
            PostMsg(tm_ModelChanged,nil,aProjE,false);
            PostMsgToOwners(tm_ReviewByElement, R, false);
            exit;
            {
            Disconect := True;
            PrepareAdding;
            if QAdd.NoElements then
            begin
               ShowMessage('Nen co odpojit, zbyla jen jedna plocha!!!');
            end
            else
            begin
               EditorStatus := gesAdd;
               exit;
            end;
            }
         end;
      end;
   end;
   if (ssCtrl in aMessage.Shift) then
   begin
      {
      TTopolTextElement(Input).Visible := not TTopolTextElement(Input).Visible;
      PostMsg(tm_ModelChanged,nil,nil,false);
      PostMsgToOwners(tm_ReviewByElement, R, false);
      }
   end
   else
   if not New and(ssShift in aMessage.Shift) then
   begin
      XORDraw;
      Redraw(clWhite);
      Application.ProcessMessages;
      EditorStatus := gesSetup;
      XORDraw;
      exit;
   end
   else
   if (ssAlt in aMessage.Shift) then
   begin
      {
      XORDraw;
      SetAlternateStyle;
      //XORDraw;
      PostMsg(tm_ModelChanged,nil,nil,false);
      exit;
      }
   end
   else
      //XORDraw;
      Redraw(0);
   //ShowMessage('UP');
end;

procedure TSimpleTextEditor.On_tm_StartDrag(
  var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
  inherited;

end;


procedure TSimpleTextEditor.PrepareQTree;
var
   TQ: TAddEntityToQTree;
   ii: integer;
   TE: TTopolTextElement;
   e: TElement;
   En: TEntity;
   RE: TRectEntity;

   procedure AddLevel(aL: TEntity);
   var
      i: integer;
   begin
      if aL = nil then exit;
      for i := 0 to aL.LastIndex do
      begin
         e := aL.Elements[i];
         if not(e is TTopolTextElement) then continue;
         TE := TTopolTextElement(e);
         CHeckRectangle(TE);
         RE.AddAndUpdateRect(TE);
      end;
   end;
begin
   Level := FindLevel(LevelName);
   if Level = nil then exit;
   RE := TRectEntity.Create;
   AddElement(RE);
   AddLevel(Level);
   if AuxProjectsName <> '' then
   begin
      AuxProjects := FindLevel(AuxProjectsName);
      if AuxProjects <> nil then
      for ii  := 0 to AuxProjects.LastIndex do
      begin
         e := AuxProjects.Elements[ii];
         if not (e is TEntity) then continue;
         En := TEntity(e);
         AddLevel(FindLevelInEntity(En,AuxLevelName));
      end;
   end;
   StartProgress('Pprava vrstev...',0,0);
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := RE;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
   RE.Free;
   EndProgress;
end;

procedure TSimpleTextEditor.ReDraw(aColor: integer);
begin
   Device.Pen.Color := aColor;
   DrawText(false);
end;

procedure TSimpleTextEditor.RegisterCursors;
begin
   NewCursorName('NEWTEXT');
   NewCursorName('EDITTEXT');
   NewCursorName('MOVE_POINT');
   NewCursorName('SELECTAREANEW');
end;


procedure TSimpleTextEditor.SetAuxLevelName(const Value: string);
begin
  FAuxLevelName := Value;
end;

procedure TSimpleTextEditor.SetAuxProjects(const Value: TEntity);
begin
  FAuxProjects := Value;
end;

procedure TSimpleTextEditor.SetAuxProjectsName(const Value: string);
begin
  FAuxProjectsName := Value;
end;

procedure TSimpleTextEditor.SetDragBusy(const Value: boolean);
begin
  FDragBusy := Value;
end;


procedure TSimpleTextEditor.SetLevel(const Value: TEntity);
begin
  FLevel := Value;
end;

procedure TSimpleTextEditor.SetLevelName(const Value: string);
begin
   FLevelName := Value;
   Level := FindLevel(Value);
end;

procedure TSimpleTextEditor.SetLevelQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLevelQT), Value);
end;


procedure TSimpleTextEditor.SetPixelTolerance(const Value: integer);
begin
  FPixelTolerance := Value;
end;


procedure TSimpleTextEditor.SetX(const Value: double);
begin
  FX := Value;
end;

procedure TSimpleTextEditor.SetY(const Value: double);
begin
  FY := Value;
end;


procedure TSimpleTextEditor.XORDraw;
begin
   DrawText(true);
end;

INITIALIZATION
   RegisterOrbisClass(TSimpleTextEditor);
END.
