unit t_TopolLineDialog;

interface
uses
   e_Type,
   f_TopolLineDialog,
   T_LineDialog;

type

TTopolLineDialog = class(TLineDialog)
private
    FDruhy: TIntegerListElement;
    FActiveDruhIndex: integer;
    FPresnost: integer;
    FDynamicProps: string;
    function GetTLForm: TTopolLineDialog_Form;
    function GetDruhy: TIntegerListElement;
    function GetDruhyList: string;
    procedure SetDruhyList(const Value: string);
    function GetActiveDruhIndex: integer;
    procedure SetActiveDruhIndex(const Value: integer);
    procedure SetPresnost(const Value: integer);
    procedure SetDynamicProps(const Value: string);
    function GetActiveDruh: integer;
    function GetActiveDruhText: string;
public
   procedure DoIt; override;
   procedure CreateForm; override;
   property  TLForm: TTopolLineDialog_Form read GetTLForm;
   property  Druhy: TIntegerListElement read GetDruhy;
published
   property DruhyList: string read GetDruhyList write SetDruhyList;
   property ActiveDruhIndex: integer read GetActiveDruhIndex write SetActiveDruhIndex;
   property Presnost: integer read FPresnost write SetPresnost;
   property DynamicProps: string read FDynamicProps write SetDynamicProps;
   property ActiveDruh: integer read GetActiveDruh;
   property ActiveDruhText: string read GetActiveDruhText;
end;

implementation
uses
   SysUtils,
   Forms,
   e_Color,
   e_Class,
   e_Geometry,
   e_Props,
   e_TopolElements,
   e_TopolDruhy,
   Worlds, Classes, T_OrbisDialog, StdCtrls;


{ TTopolLineDialog }

procedure TTopolLineDialog.CreateForm;
begin
   aForm := TTopolLineDialog_Form.Create(Application);
   aForm.Tool := Self;
   aForm.IsPossibleToClose := false;
end;

procedure TTopolLineDialog.DoIt;
var
   D: TTopolDruhy;
   TD: TTopolDruh;
   i: integer;
   TL: TTopolLineElement;
   S: string;
   _Druh,_Barva,_Znacka,_Presnost: integer;
   SL: TStringList;
begin
   Output := Input;
   D := TTopolDruhy(World.ObjectModel.FindElementClass(TTopolDruhy));
   SL := TStringList.Create;
   SL.CommaText := DynamicProps;
   TLForm.DynamicProps := SL;
   if input is TTopolLineElement then
   begin
      TL := TTopolLineElement(Input);
      _Druh := TL.Druh;
      _Barva := TL.Barva;
      _Znacka := TL.Znacka;
      _Presnost := TL.Presnost;
      if D <> nil then
      begin
         D.FindLDruhKod := TL.Druh;
         TD := D.ActualDruh;
         if TD <> nil then
         begin
            i := TLForm.ComboDruh.Items.IndexOfObject(TD);
            if i < 0 then
               i := TLForm.ComboDruh.Items.AddObject(TD.Name+'['+IntToStr(TD.KOD)+']',TD);
            TLForm.ComboDruh.ItemIndex := i;
            TLForm.DruhIndex := i;
         end
         else
         // neznm druh
         begin
            s := IntToStr(TL.Druh);
            i := TLForm.ComboDruh.Items.IndexOf(s);
            if i < 0 then
               i := TLForm.ComboDruh.Items.Add(s);
            TLForm.ComboDruh.ItemIndex := i;
            TLForm.DruhIndex := i;
         end;
      end;
      TLForm.ColorPanel.Color := DejBarvicku(TL.Barva);
      TLForm.ColorPanel.Caption := TL.GetPropertyStringByName('Barva');
      TLForm.ed_AtrPuv.Value := TL.Presnost;
      TLForm.ed_Znacka.Value := TL.Znacka;
      for i := 0 to SL.Count - 1 do
      begin
         TLForm.SetDynamicProp(SL.Strings[i],TL.GetPropertyStringByName(SL.Strings[i]));
      end;
   end
   else
   begin
      TL := TTopolLineElement.Create;
      TL.Geometry := TGeometry(Input);
      Output := TL;
      TLForm.ed_AtrPuv.Value := Presnost;
   end;
   if TLForm.ComboDruh.ItemIndex < 0 then
   begin
      TLForm.ComboDruh.ItemIndex := 0;
      TLForm.DruhIndex := 0;
      TLForm.ComboDruhChange(nil);
   end;
   inherited;
   if result in [1,3,4,5,6,9] then
   begin
      TL.Barva := StrToInt(TLForm.ColorPanel.Caption);
      Presnost := TLForm.ed_AtrPuv.Value;
      TL.Presnost := TLForm.ed_AtrPuv.Value;
      TL.Znacka := TLForm.ed_Znacka.Value;
      if D <> nil then
      begin
         // upraveno pro vyber neznameho druhu
         TD := TTopolDruh(TLForm.ComboDruh.Items.Objects[TLForm.DruhIndex]);
         if TD <> nil then
            TL.Druh := TD.KOD
         else
            TL.Druh := StrToInt(TLForm.ComboDruh.Items.Strings[TLForm.ComboDruh.ItemIndex]);
      end;
      if (TL.Druh<>_Druh) or (TL.Barva<>_Barva) or (TL.Znacka<>_Znacka) or (TL.Presnost<> _Presnost) then
         Modified := true;
      if SL.Count > 0 then
      begin
         if TL.NoProperties then
            TL.Properties := TDynamicProperties.Create;
         for i := 0 to SL.Count - 1 do
         begin
            TL.Properties.SetPropertyStringByName(SL.Strings[i],TLForm.GetDynamicProp(SL.Strings[i]));
         end;
      end;
   end;
   SL.Free;
end;

function TTopolLineDialog.GetActiveDruh: integer;
var
   TD: TTopolDruh;
begin
   TD := TTopolDruh(TLForm.ComboDruh.Items.Objects[TLForm.DruhIndex]);
   if TD <> nil then
      result := TD.KOD
   else
      result := StrToInt(TLForm.ComboDruh.Items.Strings[TLForm.ComboDruh.ItemIndex]);
end;

function TTopolLineDialog.GetActiveDruhIndex: integer;
begin
   result := -1;
   if TLForm = nil then exit;
   result := TLForm.ComboDruh.ItemIndex;
end;

function TTopolLineDialog.GetActiveDruhText: string;
begin
   result := TLForm.ComboDruh.Text;
end;

function TTopolLineDialog.GetDruhy: TIntegerListElement;
begin
   if FDruhy = nil then
   begin
      FDruhy := TIntegerListElement.Create;
      AddElement(FDruhy);
   end;
   result := FDruhy;
end;

function TTopolLineDialog.GetDruhyList: string;
var
   i: integer;
   S: string;
begin
   result := '';
   for i := 0 to Druhy.LastIndex do
   begin
      S := IntToStr(Druhy.Integers[i]);
      if i = 0 then
         result := S
      else
         result := result + ',' + S;
   end;
end;

function TTopolLineDialog.GetTLForm: TTopolLineDialog_Form;
begin
   result := TTopolLineDialog_Form(aForm);
end;


procedure TTopolLineDialog.SetActiveDruhIndex(const Value: integer);
begin
   FActiveDruhIndex := Value;
   if TLForm = nil then exit;
   TLForm.ComboDruh.ItemIndex := Value;
   TLForm.ComboDruhChange(nil);
end;

procedure TTopolLineDialog.SetDruhyList(const Value: string);
var
   D: TTopolDruhy;
   TD: TTopolDruh;
   i: integer;
   all: boolean;
begin
   all := (UpperCase(Value) = 'ALL');
   if not all then
      Druhy.ValuesString := Value;
   D := TTopolDruhy(World.ObjectModel.FindElementClass(TTopolDruhy));
   if D <> nil then
   begin
      TLForm.ComboDruh.Items.Clear;
      for i := 0 to D.L.LastIndex do
      begin
         TD := TTopolDruh(D.L.Elements[i]);
         if all or Druhy.ValueExists(TD.KOD) then
         begin
            TLForm.ComboDruh.Items.AddObject(TD.Name+'['+IntToStr(TD.KOD)+']',TD);
            if all then
               Druhy.AddInteger(TD.KOD);
         end;
      end;
      TLForm.ComboDruh.ItemIndex := FActiveDruhIndex;
      TLForm.ComboDruhChange(nil);
   end;
end;

procedure TTopolLineDialog.SetDynamicProps(const Value: string);
begin
  FDynamicProps := Value;
end;

procedure TTopolLineDialog.SetPresnost(const Value: integer);
begin
  FPresnost := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TTopolLineDialog);
END.
