unit E_text;

interface
uses
   Classes,
   Graphics,
   e_Files,
   Elements;


type

TTextStyle = class (TOwnerElement)
private
   FTextColor: TColor;
   FTextHeight: double;
   FTextSize: integer;
   FFontName: string;
   FFontStyle: TFontStyle;
published
   property TextColor: TColor read FTextColor write FTextColor;
   property TextHeight: double read FTextHeight write FTextHeight;
   property TextSize: integer read FTextSize write FTextSize;
   property FontName: string read FFontName write FFontName;
   property FontStyle: TFontStyle read FFontStyle write FFontStyle;
end;


TParagraphStyle = class (TTextStyle)
private
   FLineHeight: double;
   FLineSize: integer;
{
   Numbering: Word;
   wReserved: Word;
   dxStartIndent: Longint;
   dxRightIndent: Longint;
   dxOffset: Longint;
   wAlignment: Word;
   cTabCount: Smallint;
   rgxTabs: array [0..MAX_TAB_STOPS - 1] of Longint;
 }
published
   property LineHeight: double read FLineHeight write FLineHeight;
   property LineSize: integer read FLineSize write FLineSize;
end;



TPlainTextBlock = class(TFileBasedEntity)
private
   FLines: TStringList;
   function  GetLines: TStringList;
   procedure SetLines(aValue: TStringList);
   procedure LoadFromFile(aFileName: string); override;
published
   property Lines: TStringList read GetLines write SetLines stored false;
end;



TPlainText = class (TOwnerEntity)
private
   FText: string;
protected
   function GetTextStyle: TTextStyle; virtual;
   procedure SetTextStyle(aValue: TTextStyle); virtual;
   procedure SetNewStyle(aValue: boolean); virtual;
published
   property Text: string read FText write FText;
   property TextStyle: TTextStyle read GetTextStyle write SetTextStyle;
   property NewStyle: boolean write SetNewStyle;
end;



TFormatedText = class (TPlainText)
private
   FTextStyle: TTextStyle;
protected
   function GetTextStyle: TTextStyle; override;
   procedure SetTextStyle(aValue: TTextStyle); override;
   procedure SetNewStyle(aValue: boolean); override;
end;

TVariableText = class(TFormatedText);


TParagraph = class (TFormatedText)
protected
   function GetParagraphStyle: TParagraphStyle; virtual;
   procedure SetParagraphStyle(aValue: TParagraphStyle); virtual;
   procedure SetNewStyle(aValue: boolean); override;
   procedure SetNextText(aValue: TFormatedText);
   procedure SetTextName(aValue: string);
published
   property ParagraphStyle: TParagraphStyle read GetParagraphStyle write SetParagraphStyle;
   property NextText: TFormatedText write SetNextText;
   property NextTextName: string write SetTextName;
end;

TFormatedTextBlock = class(TParagraph)
private
   FActualParagraph: TParagraph;
protected
   procedure SetNextParagraphName(aValue: string);
   procedure SetNextParagraph(aValue: TParagraph);
   procedure SetFindParagraphName(aValue: string);
   function  GetActualParagraph: TParagraph;
published
   property NextParagraphName: string write SetNextparagraphName;
   property NextParagraph: TParagraph write SetNextparagraph;
   property FindParagraphName: string write SetFindparagraphName;
   property ActualParagraph: TParagraph read GetActualParagraph;
end;

TTextPage = class(TParagraph)
private
   FActualParagraph: TParagraph;
published
   property ActualParagraph: TParagraph read FActualParagraph write FActualParagraph;
end;

TCustomTextDocument= class(TFileBasedEntity);

TTextDocument = class(TCustomTextDocument);




implementation

uses
   e_Class;


procedure TPlainTextBlock.LoadFromFile(aFileName: string);
begin
   Lines.Clear;
   Lines.LoadFromFile(aFileName);
end;


function  TPlainTextBlock.GetLines: TStringList;
begin
  if FLines = nil then
     FLines:= TStringList.Create;
  result := FLines;
end;


procedure TPlainTextBlock.SetLines(aValue: TStringList);
begin
   Lines.Assign(aValue);
end;



{ TPlainText }

function  TPlainText.GetTextStyle: TTextStyle;
begin
   result := nil;
end;

procedure TPlainText.SetTextStyle(aValue: TTextStyle);
begin
end;

procedure TPlainText.SetNewStyle(aValue: boolean);
begin
end;


{ TFormatedText }

function  TFormatedText.GetTextStyle: TTextStyle;
begin
   result := FTextStyle;
end;

procedure TFormatedText.SetTextStyle(aValue: TTextStyle);
begin
   if FTextStyle <> nil then
   begin
      FreeElement(FTextStyle);
      FTextStyle := nil;
   end;
   FTextStyle := aValue;
   AddElement(FTextStyle);
end;

procedure TFormatedText.SetNewStyle(aValue: boolean);
begin
   if aValue then
   begin
      if FTextStyle <> nil then
      begin
         FreeElement(FTextStyle);
         FTextStyle := nil;
      end;
      FTextStyle := TTextStyle.Create;
      AddElement(FTextStyle);
   end;
end;


{ TParagraph }

function TParagraph.GetParagraphStyle: TParagraphStyle;
begin
   result := TParagraphStyle(FTextStyle);
end;

procedure TParagraph.SetParagraphStyle(aValue: TParagraphStyle);
begin
   SetTextStyle(aValue);
end;

procedure TParagraph.SetNewStyle(aValue: boolean);
begin
   if aValue then
   begin
      if FTextStyle <> nil then
      begin
         FreeElement(FTextStyle);
         FTextStyle := nil;
      end;
      FTextStyle := TParagraphStyle.Create;
      AddElement(FTextStyle);
   end;
end;


procedure TParagraph.SetNextText(aValue: TFormatedText);
begin
   AddElement(aValue);
end;

procedure TParagraph.SetTextName(aValue: string);
var
   F: TFormatedText;
begin
   F := TFormatedText.Create;
   F.Name := aValue;
   AddElement(F);
end;

{ TFormatedTextBlock }

procedure TFormatedTextBlock.SetNextParagraphName(aValue: string);
var
   P: TParagraph;
begin
   P := TParagraph.Create;
   P.Name := aValue;
   AddElement(P);
   FActualParagraph := P;
end;

procedure TFormatedTextBlock.SetNextParagraph(aValue: TParagraph);
begin
   if aValue = nil then exit;
   if not (aValue is TParagraph) then exit;
   AddElement(aValue);
   FActualParagraph := aValue;
end;

procedure TFormatedTextBlock.SetFindParagraphName(aValue: string);
begin
   FActualParagraph := Tparagraph(FindElementName(aValue));
   if not(FActualParagraph is TParagraph) then FActualParagraph := nil;
end;

function  TFormatedTextBlock.GetActualParagraph: TParagraph;
begin
   result := FActualParagraph;
end;


{===================================================================}

initialization
   RegisterOrbisClass(TPlainTextBlock);
   RegisterOrbisClass(TFormatedText);
   RegisterOrbisClass(TVariableText);
   RegisterOrbisClass(TParagraph);
   RegisterOrbisClass(TFormatedTextBlock);
end.
