unit t_MapSh;
{ uivatelem definovan mapov list }

interface

uses
  _IniFile,
  Elements, Tools, G_elems, e_View, T_Recurs, E_DeskTp, e_LayScr,
  CtiPar;

type

TMSTemplateGenerator = class(TTechnology)
{ vytvo TMapSheet dle textovho souboru *.MSD
  Input  .. neoekv
  Output .. TMapSheet }
 private
   FDeltaX, FDeltaY: double;

   procedure FillProps(Element: TElement);
   procedure FillViewProps(aView: TView);
   function  GetDeskTop(DTFile: TDeskTopFile): TDeskTop;
   procedure ProcessMSI(aEntity: TEntity);

 protected
   FDefInf: TCteniParametru;
   function DejTextElement: TText; virtual;
   function DejBMPElement: TElement; virtual;
   function  DejRectElement: TDrawRectangle; virtual;
 public
   TemplateFileName: string;
   procedure   DoIt; override;
 published
   property DeltaX: double read FDeltaX write FDeltaX;
   property DeltaY: double read FDeltaY write FDeltaY;
end;


TDeskTopSaveToText = class(TRecursor)
private
   FMSDName: string;
{   FIniFile: TRealIniFile; }
   FText: system.Text;
   FSection: string;
   FWasHeader: boolean;
   procedure AddString(aValue: string);
   procedure AddSection(aValue: string);
   procedure AddPar(aParName, aValue: string);
   procedure SaveProperties(Element: TElement);
   procedure SaveViewScript(aView: TView);
   procedure SaveLayer(Layer: TLayerScript);
protected
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   function  QProcessAfter(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
 public
   constructor Create; override;
   procedure DoIt; override;
 published
   property  MSDFileName: string read FMSDName write FMSDName;
end;



procedure CreateMapSheet;



implementation

uses
  Graphics, Dialogs, Classes, Forms, SysUtils,
  e_Class,
// KERNEL
  e_IniFile,
  u_String,
  Worlds,
  u_Props,
  E_type,
  M_Orbis,
// GUI
  T_InterA,
// GLIB
  E_GFile,
// SPACE
  e_Props,
// MAPPER
  E_Map,
  E_MSpace,
  E_TLView,
  I_Desktp;

{
  E_Map, e_View, G_elems, T_Recurs, I_Desktp, T_InterA,
  F_STD}



procedure TMSTemplateGenerator.ProcessMSI(aEntity: TEntity);
var
  aMSIName: string;
  aINIEl: TIniFileElement;
  I: integer;
  anItem: string;
  aValues: TStringListElement;
  NazevParametru, HodnotaParametru: string;
  aView: TView;
  aEl: TElement;


  function SolveName(aName: string): string;
  var
    aTC: string;
  begin
    result:= aName;
    aTC:= copy(aName, 1, 3);
    if (compareText(aTC, 'ARE') = 0)
        or (compareText(aTC, 'LIN') = 0)
        or (compareText(aTC, 'PNT') = 0) then
       result:= aTC;

  end;


begin
  aMSIName:= changeFileExt(TemplateFileName, '.MSI');
  if not FileExists(aMSIName) then
     exit;
  aINIEl:= TIniFileElement.Create;
  aIniEl.FileName:= aMSIName;

  aIniEl.Section:= 'INFOVIEW';
  if aIniEl.Section<>'' then
  begin
    aView:= TView.Create;
    aView.Name:= 'INFOVIEW';
    aEntity.AddElement(aView);
    aValues:= aIniEl.IdentsElement;
    for I:= 0 to aValues.Count-1 do
    begin
      NazevParametru:= aValues.Names[I];
      NazevParametru:= SolveName(NazevParametru);
      HodnotaParametru:= aValues.Strings[I];
      if not SetPropStringByName(aView, NazevParametru, HodnotaParametru) then
         aView.ModelScript.AddLayerOrParameter(NazevParametru, HodnotaParametru)
    end;
  end;

  aIniEl.Section:= 'REMOVE';
  if aIniEl.Section <> '' then
  begin
    I:= 0;
    repeat
      anItem:= 'ELEMENT' + intToStr(I);
      inc(I);
      aIniEl.Ident:= anItem;
      if aIniEl.Value = '' then
         break;
      aEl:= aEntity.FindElementName(aIniEl.Value);
      if aEl <> nil then
         aEl.Free;
    until false;
  end;
  aIniEl.Free;

// Vloen INFO VIEW

end;



procedure TMSTemplateGenerator.FillProps(Element: TElement);
var
   JeToParametr, JeToInfo, PosledniRadek: boolean;
   NazevParametru, HodnotaParametru: string;
begin
   with FDefInf do
   repeat
      DejRadek(JeToParametr, JeToInfo, PosledniRadek, NazevParametru, HodnotaParametru);
      if (NazevParametru <> '') and (NazevParametru <> 'END') then
      begin
         try
            if not SetPropStringByName(Element, NazevParametru, HodnotaParametru) then
               MessageDlg(Element.ClassName + ': Neregistrovan vlastnost '+ NazevParametru, mtError, [mbOK], 0);
         except
            MessageDlg(Element.ClassName + ': Chyba nastaven' + #13 +
            NazevParametru + ' <- ' + HodnotaParametru, mtError, [mbOK], 0)
         end
      end;
   until PosledniRadek or (NazevParametru = 'END');
end;


procedure TMSTemplateGenerator.FillViewProps(aView: TView);
var
  JeToParametr, JeToInfo, PosledniRadek: boolean;
  NazevParametru, HodnotaParametru: string;

begin
  with FDefInf do
  repeat
    DejRadek(JeToParametr, JeToInfo, PosledniRadek, NazevParametru, HodnotaParametru);
    if (NazevParametru <> '') and (NazevParametru <> 'END') then
    begin
      if not SetPropStringByName(aView, NazevParametru, HodnotaParametru) then
         aView.ModelScript.AddLayerOrParameter(NazevParametru, HodnotaParametru)
    end;
  until PosledniRadek or (NazevParametru = 'END');
end;




function TMSTemplateGenerator.DejTextElement: TText;
var
  aText: TText;
//  PosledniRadek: boolean;
//  NazevParametru, HodnotaParametru: string;

begin
  aText:= TText.Create;
  FillProps(aText);
(*
  with FDefInf do
  repeat
    DejRadek(JeToParametr, JeToInfo, PosledniRadek, NazevParametru, HodnotaParametru);
    if NazevParametru <> 'END' then
    begin
      if NazevParametru = 'NAME' then
         aText.Name:= HodnotaParametru
      else
      if NazevParametru = 'X' then
         aText.X:= DejRealParametr
      else
      if NazevParametru = 'Y' then
         aText.Y:= DejRealParametr
      else
      if NazevParametru = 'ORIENTATION' then
         aText.Orientation:= DejRealParametr
      else
      if NazevParametru = 'FONT' then
         aText.Font.Name:= HodnotaParametru
      else
      if NazevParametru = 'HEIGHT' then
         aText.Height:= DejRealParametr
      else
{
      if NazevParametru = 'FONTSTYLE' then
         aText.Font.Style:= DejRealParametr
      else
}
      if NazevParametru = 'TEXT' then
         aText.Text:= HodnotaParametru
      else

    end;
  until PosledniRadek or (NazevParametru = 'END');
*)
  result:= aText
end;


function TMSTemplateGenerator.DejBMPElement: TElement;
var
  aBMP: TGraphicFileElement;
//  JeToParametr, JeToInfo, PosledniRadek: boolean;
//  NazevParametru, HodnotaParametru: string;

begin
  aBMP:= TGraphicFileElement.Create;
  FillProps(aBMP);
(*
  with FDefInf do
  repeat
    DejRadek(JeToParametr, JeToInfo, PosledniRadek, NazevParametru, HodnotaParametru);
    if NazevParametru <> 'END' then
    begin
      if NazevParametru = 'NAME' then
         aBMP.Name:= HodnotaParametru
      else
      if NazevParametru = 'FILENAME' then
         aBMP.FileName:= HodnotaParametru
      else
      if NazevParametru = 'X' then
         aBMP.X:= DejRealParametr
      else
      if NazevParametru = 'Y' then
         aBMP.Y:= DejRealParametr

      else
      if NazevParametru = 'HEIGHT' then
         aBMP.Height:= DejRealParametr
      else
      if NazevParametru = 'WIDTH' then
         aBMP.Width:= DejRealParametr
      else

    end;
  until PosledniRadek or (NazevParametru = 'END');
*)
  result:= aBMP
end;







function TMSTemplateGenerator.DejRectElement: TDrawRectangle;
var
  aRect: TDrawRectangle;
//  JeToParametr, JeToInfo, PosledniRadek: boolean;
//  NazevParametru, HodnotaParametru: string;

begin
  aRect:= TDrawRectangle.Create;
  FillProps(aRect);
(*
  with FDefInf do
  repeat
    DejRadek(JeToParametr, JeToInfo, PosledniRadek, NazevParametru, HodnotaParametru);
    if NazevParametru <> 'END' then
    begin
      if NazevParametru = 'PENSTYLE' then
         aRect.Pen.Style:= TPenStyle(DejLongIntParametr)
      else
      if NazevParametru = 'PENCOLOR' then
         aRect.Pen.Color:= DejLongIntParametr
      else
      if NazevParametru = 'X' then
         aRect.Min.X:= DejRealParametr
      else
      if NazevParametru = 'Y' then
         aRect.Min.Y:= DejRealParametr
      else
      if NazevParametru = 'WIDTH' then
         aRect.Width:= DejRealParametr
      else
      if NazevParametru = 'HEIGHT' then
         aRect.Height:= DejRealParametr
    end;
  until PosledniRadek or (NazevParametru = 'END');
*)
  aRect.BrushStyle:= bsClear;
  result:= aRect
end;



function TMSTemplateGenerator.GetDeskTop(DTFile: TDeskTopFile): TDeskTop;
var
  Jm: string;
  Reader: TMSTemplateGenerator;
  aClass: TElementClass;
begin
  result:= nil;
  JM:= self.ClassName;
  aClass:= TElementClass(GetOrbisClass(Jm));
  if aClass = nil then exit;
  Reader:= TMSTemplateGenerator(aClass.Create);
  Reader.TemplateFileName:= DTFile.FileName;
  Reader.DeltaX:= DTFile.DeltaX;
  Reader.DeltaY:= DTFile.DeltaY;
  Reader.DoIt;
  result:= TDeskTop(Reader.Output);
  Reader.Free
end;


procedure TMSTemplateGenerator.DoIt;
{ vytvo TMapSheet dle textovho souboru *.MSD
  Input .. neoekv
  Outpu .. TMapSheet }
var
   {
   MapSpace: TMapSpace;
   TempLib: TMapSheetTemplatesLibrary;
   }
   MapSheet: TDeskTop;
{
   RE: TRectEntity;
   UpperText: TNamedText;
   BottomText: TNamedText;
   UpperRect, BottomRect: TDrawRectangle;
   R: TDoubleRect;
   Stred: TDoubleCoord;
}
   JeToParametr, JeToInfo, PosledniRadek: boolean;
   NazevParametru, HodnotaParametru: string;
   OKej: boolean;

   aClassName: string;

   Element: tElement;
begin
  Output:= nil;
  MapSheet:= nil;

  if TemplateFileName <> '' then
  begin
    FDefInf:= TCteniParametru.Create(TemplateFileName, OKej);

    if OKej then
    begin
    { cyklus cteni objektu }
      PosledniRadek:= false;
      with FDefInf do
      while not PosledniRadek do
      begin
        DejRadek(JeToParametr, JeToInfo, PosledniRadek, NazevParametru, HodnotaParametru);
        if JeToInfo then
        begin

          aClassName := HodnotaParametru;
          Element:= CreateElementByName(aClassName);
          try
             if Element <> nil then
             begin
               if (MapSheet = nil) and (Element is TDeskTop) then
               begin
                 MapSheet:= TDeskTop(Element);
                 FillProps(MapSheet);
                 MapSheet.Move(DeltaX, DeltaY)
               end
               else
               if MapSheet <> nil then
               begin
                 if Element is TDeskTopFile then
                 begin
                   FillProps(Element);
                   MapSheet.AddElement(GetDeskTop(TDeskTopFile(Element)));
                   Element.Free
                 end
                 else
                 begin
                   if Element is TView then
                      FillViewProps(TView(Element))
                   else
                      FillProps(Element);

                   if Element is TRectEntity then
                      TRectEntity(Element).Move(DeltaX, DeltaY);

                   MapSheet.AddElement(Element)
                 end
               end
               else Element.Free
             end
          except
             MessageDlg('Chyba v natn elementu '+ aClassName, mtError, [mbOK], 0);
          end;

        end;
      end;
    end;

    FDefInf.Destroy;
    FDefInf:= nil;

    ProcessMSI(MapSheet);

    Output := MapSheet;
  end { if FName <> '' }
end;






procedure CreateMapSheet;
var
   MapListy: TMapSheets;
   Generator: TMSTemplateGenerator;
   Mapa: TMapSheet;
   OD: TOpenDialog;
   aName: string;
begin
  {Nalezeni drzaku mapovych listu}
  MapListy := TMapSheets(TEntity(World.ObjectModel).
              FindElementClass(TMapSheets));
  if not assigned(MapListy) then
  begin
     MapListy := TMapSheets.Create;
     TEntity(World.ObjectModel).AddElement(MapListy);
  end;

  OD:= TOpenDialog.Create(Application);
  OD.Filter:= 'Map Sheet Definition (*.MSD)|*.MSD';
  OD.Execute;
  aName:= OD.FileName;
  OD.Free;

  if aName = '' then exit;

  {Generovani mapoveho listu}
  Generator := TMSTemplateGenerator.Create;
  Generator.TemplateFileName:= aName;
  Generator.DoIt;
  Mapa:= TMapSheet(Generator.Output);
  Generator.Free;

  if Mapa <> nil then
  begin
    {Zarazeni mapy do mapovych listu}
    MapListy.AddElement(Mapa);
    World.PostMsg(tm_AddView, World.MainTool, Mapa, DontKill);
  end
end;



procedure TDeskTopSaveToText.AddString(aValue: string);
begin
  writeln(FText, aValue)
end;

procedure TDeskTopSaveToText.AddSection(aValue: string);
begin
  AddString('['+aValue+']')
end;

procedure TDeskTopSaveToText.AddPar(aParName, aValue: string);
begin
  AddString(aParName+'='+aValue)
end;


procedure TDeskTopSaveToText.SaveProperties(Element: TElement);
var
  I, FirstProp: longInt;
  PropEnt: TInstanceProperties;
  PropStr, PropName: string;
begin
  FirstProp:= 4;
  if TObject(Element) is TElement then
     FirstProp:= 4;
  if Element is TEntity then
     FirstProp:= 6;
  if Element is TCoordinate then
     FirstProp:= 8;

  PropEnt:= TInstanceProperties.Create;
  PropEnt.Instance:= Element;

  for I:= FirstProp to PropEnt.PropCount-1 do
  begin
    PropEnt.PropIndex:= I;
    if PropEnt.ActualProperty.IsStored then
    begin
      PropStr:= PropEnt.ActualProperty.PropString;
      PropName:= PropEnt.ActualProperty.Name;
      AddPar(PropName, PropStr)
    end;
  end;

  PropEnt.Free;
end;



procedure TDeskTopSaveToText.SaveLayer(Layer: TLayerScript);
var
  Props: TPropertyList;
  SE: TStringElement;
  I: longInt;
begin
  AddPar(Layer.Name, Layer.Description);
  Props:= TPropertyList(Layer.FindElementClass(TPropertyList));
  if Props = nil then exit;
  for I:= 0 to Props.count-1 do
  begin
    SE:= TStringElement(Props.Elements[I]);
    if not (TElement(SE) is TStringElement) then continue;
    AddPar(SE.Name, SE.Text)
  end;
end;



procedure TDeskTopSaveToText.SaveViewScript(aView: TView);
var
  MS: TModelScript;
  I: longInt;
  Layer: TLayerScript;
begin
  MS:= TModelScript(aView.FindElementClass(TModelScript));
  if MS = nil then exit;
  for I:= 0 to MS.Count-1 do
  begin
    Layer:= TLayerScript(MS.Elements[I]);
    if not (TElement(Layer) is TLayerScript) then continue;
    SaveLayer(Layer);
  end
end;


function  TDeskTopSaveToText.QGoInside(Element: TElement): boolean;
begin
{ leze se dovnit pouze tch Element, kter tvo sekci }
  result:= (Element is TDeskTop)
            or (Element is TDrawRectangle)
            or (Element is TGraphicFileElement)
            or (Element is TText)
            or (Element is TView);
end;


function  TDeskTopSaveToText.QProcessBefore(Element: TElement): boolean;
begin
  result:= true;
  if QGoInside(Element) then
  begin
    if not FWasHeader and not (Element is TDeskTop) then
    { pokud nsleduje po DeskTopu jin objekt tak udlej END }
    begin
      AddString('END');
      AddString('');
      FWasHeader:= true
    end;
    AddSection(Element.ClassName);
    FSection:= Element.ClassName;
    if Element.Name <> Element.ClassName then
       AddPar('NAME',Element.Name);
  end
end;


function  TDeskTopSaveToText.QProcessAfter(Element: TElement): boolean;
begin
  result:= false;
  if QGoInside(Element) then
  begin
    AddString('END');
    AddString('')
  end;
end;


procedure TDeskTopSaveToText.Process(Element: TElement);
begin
  if QGoInside(Element) then
  begin
    SaveProperties(Element);
    if Element is TView then
       SaveViewScript(TView(Element))
  end;
end;

procedure TDeskTopSaveToText.DoIt;
begin
  if not (Input is TDeskTop) then exit;
  if FMSDName = '' then exit;
{  FIniFile:= TRealIniFile.Create(FMSDName); }
  system.assign(FText, FMSDName);
  rewrite(FText);
  inherited DoIt;
  close(FText)
{  FIniFile.Free; }
end;


constructor TDeskTopSaveToText.Create;
begin
  inherited Create;
{  FMSDName:= 'H:\ORBIS\T1.msd' }
end;



begin
  RegisterOrbisClass(TMSTemplateGenerator);
end.
