unit I_desktp;
{****************************************************************************}
{                                                                            }
{          ORBIS ... Interpret Desktopu                                      }
{                    (c) 1996                                                }
{                                                                            }
{****************************************************************************}

interface
uses
   Wintypes,
   e_View,
   Elements,
   g_Elems,
   i_Basic;

type
  TDesktopInterpret = class(TInterpret)
  protected
     function  QGoInside(Element: TElement): boolean; override;
     function  QProcessBefore(Element: TElement): boolean; override;
     function  QProcessAfter(Element: TElement): boolean; override;
     procedure Process(Element: TElement); override;
     function  IsItsClass(AObject: TObject): boolean; override;
     procedure SetSubInterprets; override;
     procedure FreeSubInterprets; override;
  public
     procedure DoIt; override;
  end;

implementation
uses
   Controls, Graphics,
   Forms,
   WinProcs,
   SysUtils,
   M_Orbis, Worlds,
   e_Class,
   e_Device,
   e_Desktp,
   e_GFile,
   e_TxtBlk,
   i_View,
   i_TTText,
   i_TxtBlk,
   i_Rect,
   {$IFDEF RAS}
   e_Raster,
   i_Raster,
   {$ENDIF}
   i_Files,
   i_SelR,
   Tools;

function  TDesktopInterpret.QGoInside(Element: TElement): boolean;
begin
  Result:= (Element is TEntity) and not ((Element is TView)or(Element is TText));
end;

function  TDesktopInterpret.QProcessBefore(Element: TElement): boolean;
begin
  Result:= (Element is TSelectedElement) or
           (Element is TDesktop) or
           (Element is TText) or
           (Element is TTextBlock) or
           (Element is TDrawRectangle) or
           (Element is TGraphicFileElement) or
           (Element is TView);
end;

function  TDesktopInterpret.QProcessAfter(Element: TElement): boolean;
begin
  Result:= false
end;




procedure TDesktopInterpret.Process(Element: TElement);
var
  DT: TDeskTop;
begin
  if (Element is TDesktop) then
  begin
     DT:= TDesktop(Element);
     Device.Brush.Color := DT.BrushColor;
     Device.Brush.Style := DT.BrushStyle;
     Device.Pen.Style   := DT.PenStyle;
     Device.PenWidth    := DT.PenWidth;
     Device.Pen.Color   := DT.PenColor;
     with DT.Rect do Device.Rectangle(Min.x,Min.Y,Max.X,Max.Y);
  end
  else
  begin
    inherited Process(Element);
  end
end;



function TDesktopInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := AObject is TDesktop;
end;


procedure TDesktopInterpret.SetSubInterprets;
begin
   NextInterpret := TSelectedElementInterpret.Create;
   NextInterpret := TDrawRectInterpret.Create;
   NextInterpret := TViewInterpret.Create;
   NextInterpret := TTextInterpret.Create;
   NextInterpret := TTextBlockInterpret.Create;
   {$IFDEF RAS}
   if UseRasters then
   begin
      try
      NextInterpret := TRasterInterpret.Create;
      except
         UseRasters := false;
      end;
   end;
   {$ENDIF}
   NextInterpret := TBMPFileInterpret.Create;
end;


procedure TDeskTopInterpret.FreeSubInterprets;
begin
  inherited FreeSubInterprets
end;


procedure TDesktopInterpret.DoIt;
begin
{$IFNDEF WIN32}
{   PostTextMsg(tm_ShowText, IntToStr(MemAvail), World.MainTool); }
{$ENDIF}
   Application.ProcessMessages;
   inherited DoIt;
end;


BEGIN
   RegisterOrbisClass(TDesktopInterpret);
END.
