unit I_glin;
{
********************************************************
**                                                    **
**                    ORBIS                           **
**     Skupina interpretr Topolovskch lini         **
**                                                    **
********************************************************
}

interface

uses
  WinTypes,
  Elements,
  e_Device,
  e_LinZn,
  E_PSymb,
  g_Elems,
  i_GSC,
  Tools;

const
  LineCountToProcessMessages = 100;
  MaxThickLine = 4;
  ThickL = 0.7;
  ThinL  = 0.25;

type

TGeneralLineInterpret = class(TGeneralSymbolColorInterpret)
protected
   FPropSymbol: string;
   LinioveZnacky: TLinioveZnacky;
   Symbols: TTopoLSymbols;

   //Rect: TDoubleRect;
   counter: integer;
   ThickLine, ThinLine: longInt;
   function  IsItsClass(AObject: TObject): boolean; override;
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
public
   procedure  Reset(aDevice: TDevice); override;
   procedure  Close; override;
   function   CanUniGoInside(Element: TElement): boolean; override;
published
   property   PropSymbol: string read FPropSymbol write FPropSymbol;
 end;



implementation


uses
  Controls,
  Graphics,
  WinProcs,
  Dialogs,
  SysUtils,
  Forms,
  M_Orbis,
  e_Class,
  e_Color,
  e_Geometry,
  e_GeometryElement,
  e_TopolELements,
  i_Top_t,
  i_Top_p,
  u_Symbol,
  Worlds;


{ TGeneralLineInterpret }

function  TGeneralLineInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := (AObject is TTopolLineElement) or (AObject is TPolyLineCollection);
end;


function  TGeneralLineInterpret.QGoInside(Element: TElement): boolean;
begin
   Result:= {(Element is TTopoLLinie)}false;
end;

function  TGeneralLineInterpret.QProcessBefore(Element: TElement): boolean;
begin
  Result:= (Element is TTopolLineElement) ;
end;



procedure TGeneralLineInterpret.Process(Element: TElement);
var
  IX, IY: double;
  I: longInt;
  Linzn: TLiniovaZnacka;
  aStill: boolean;
  Linie: TPolyLineElement;
  TL: TTopolLineELement;
  Barva, Znacka, TPSymbol: longInt;
begin
{ na vstupu oekv objekt typu LineString }
  {
  inc(Counter);
  if Counter > LineCountToProcessMessages then
  begin
     Application.ProcessMessages;
     Counter := 0;
  end;
  }
  if (Element is TTopolLineElement) then
  begin
     TL := TTopolLineELement(Element);
     Linie:= TPolyLineElement(TL.Geometry);
     if Linie = nil then exit;
     if Linie.IsOverRegion(Rect.MinX,Rect.MinY,Rect.MaxX,Rect.MaxY)then
     begin
      try

       Device.Pen.Style := psSolid;

       aStill:= true;

       if ToolBrush <> nil then
       begin
          ToolBrush.Input := Element;
          ToolBrush.DoIt;
          Znacka := StrToInt(ToolBrush.Result);
       end
       else
       if PropBrush = '' then Znacka := longInt(bsSolid)
                         else Znacka := StrToInt(Element.GetPropertyStringByName(PropBrush));

       if Znacka > 999 then
       begin
         if LinioveZnacky <> nil then
         begin
           Linzn := LinioveZnacky.VratZnackuDleKodu(Znacka);
           if assigned(Linzn) then
           begin

              if SymbolColorTool <> nil then
              begin
                SymbolColorTool.Input:= Linzn;
                SymbolColorTool.Palette:= Palette;
                SymbolColorTool.PropColor:=  PropColor;
                SymbolColorTool.ToolColor:= ToolColor;
                SymbolColorTool.Color:= Color;
                SymbolColorTool.UseColor:= UseColor;
                SymbolColorTool.ColorByTopol:= ColorByTopol;
                SymbolColorTool.ColorByPalette:= ColorByPalette;
                SymbolColorTool.Elem:= Element;
                SymbolColorTool.DoIt
              end;

              Linzn.ZobrazLinii (Device, -1, Linie, true, Rect);
              aStill:= false
           end;
         end;
       end;

       if aStill then
       begin
         if UseColor then
            Barva:= Color
         else
         if ToolColor <> nil then
         begin
            ToolColor.Input := Element;
            ToolColor.DoIt;
            Barva := StrToInt(ToolColor.Result);
         end
         else
         if PropColor = '' then Barva := Color
                           else Barva  := StrToInt(Element.GetPropertyStringByName(PropColor));

         if ColorByPalette then Device.Pen.Color := Palette.Color[Barva]
         else
            if ColorByTopol then
            begin
               if Barva = 0 then exit;
               Device.Pen.Color := DejBarvicku(Barva)
            end
            else Device.Pen.Color := Barva;

         if Znacka < 4 then Device.Pen.Width := ThinLine
                       else Device.Pen.Width := ThickLine;
         IX:= T2DCoordinate(Linie.Elements[0]).X;
         IY:= T2DCoordinate(Linie.Elements[0]).Y;
         Device.MoveTo(IX, IY);

         for I:= 1 to Linie.Count-1 do
         begin
           IX:= T2DCoordinate(Linie.Elements[I]).X;
           IY:= T2DCoordinate(Linie.Elements[I]).Y;
           Device.LineTo(IX, IY);
         end
       end;

       begin
         TPSymbol:= TL.Symbol;
       end;

       if (TPSymbol > 999) and (Symbols <> nil) then
       begin
         with Linie do
            Symbols.PaintSymbol(X, Y, Angle+Pi/2, 13, TPSymbol);
       end
      except
         MessageBeep(word(-1));
      end;

     end;
  end
  else inherited process(Element);
end;

procedure   TGeneralLineInterpret.Reset(aDevice: TDevice);
begin
   inherited Reset(aDevice);
   //Rect := Device.Projection.ModelView;
   ThinLine  := 1{Device.Projection.GetDeviceLength(ThinL/1000  * Device.Projection.VirtualScale)};
   ThickLine := Device.Projection.GetPageLengthHoriz1(ThickL/1000 * Device.Projection.VirtualScale);
   if ThickLine > MaxThickLine then ThickLine := MaxThickLine;

   if fileExists(PathOfLibraries+'\linzn.txt') then
   begin
     if LinioveZnacky <> nil then
        LinioveZnacky.Free;
     LinioveZnacky := TLinioveZnacky.CreateAll(PathOfLibraries+'\linzn.txt');
     try
       LinioveZnacky.Rect := Rect;
       LinioveZnacky.Device := Device;
     except
       if LinioveZnacky <> nil then
          LinioveZnacky.Free;
       LinioveZnacky:= nil
     end
   end;

   if SymbolsFile = '' then
      SymbolsFile:= PathOfLibraries+'\Fonty.txt';
   if fileExists(SymbolsFile) then
   begin
     if Symbols <> nil then
        Symbols.Free;
        Symbols := TTopoLSymbols.CreateAll(SymbolsFile);
     try
       Symbols.Device := Device
     except
       if Symbols <> nil then
          Symbols.Free;
       Symbols:= nil
     end;
   end
end;

procedure   TGeneralLineInterpret.Close;
begin
  if Symbols <> nil then
  begin
     Symbols.Free;
     Symbols:= nil
  end;
  if LinioveZnacky <> nil then
  begin
     LinioveZnacky.Free;
     LinioveZnacky:= nil
  end;
  inherited Close
end;

function   TGeneralLineInterpret.CanUniGoInside(Element: TElement): boolean;
begin
   result := (Element is TPolyLineCollection) and
      TPolyLineCollection(Element).IsOverRegion
               (Rect.MinX,Rect.MinY,Rect.MaxX,Rect.MaxY);
end;


BEGIN
   RegisterOrbisClass(TGeneralLineInterpret);
END.
