unit T_editor;

INTERFACE

uses
   Controls,
   Graphics,
   Wintypes,
   Elements,
   e_Desktp,
   g_Elems,
   m_InterA,
   T_InterA,
   t_Recurs;

type

TDragStatus = Set of (dsLeft, dsRight, dsTop, dsBottom);


TDrawLineTool = class(TInteractiveTool)
 private
   FDrawn  : boolean;
   function  GetAngle: double;
   procedure TakePoint;
 protected
   PrevCoord: TDoubleCoord;
   RefPoint: Wintypes.TPoint;
   DragStatus: TDragStatus;
   function  GetName: string; override;
 public
   Pnt1, Pnt2:  Wintypes.TPoint;
   procedure On_Erase(aCoord: TDoubleCoord); virtual;
   procedure On_Draw (aCoord: TDoubleCoord); virtual;
   procedure On_Point(aCoord: TDoubleCoord); virtual;
   procedure On_Line(aCrd1, aCrd2: TDoubleCoord); virtual;
   procedure On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   procedure On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure on_tm_SetDrawnToFalse(var Handled: boolean);

   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
 published
   property Drawn: boolean read FDrawn write FDrawn;
   property Angle: double read GetAngle;
end;




TDrawRectTool = class(TInteractiveTool)
 private
   FDrawn  : boolean;
 protected
   PrevCoord: TDoubleCoord;
   RefPoint: Wintypes.TPoint;
   DragStatus: TDragStatus;
   function  GetName: string; override;
 public
   ShiftRect: TRect;
   procedure On_Erase(aCoord: TDoubleCoord); virtual;
   procedure On_Draw (aCoord: TDoubleCoord); virtual;
   procedure On_Point(aCoord: TDoubleCoord); virtual;
   procedure On_Rect(aRect: TDoubleRect); virtual;
   procedure On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   procedure On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
 published
   property Drawn: boolean read FDrawn write FDrawn;
end;



TSelectRectEntityByPoint = class(TRecursor)
private
   FX: double;
   FY: double;
   FClassFilter: TRectEntityClass;
 protected
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
 public
   procedure DoIt; override;
   property  X: double read FX write FX;
   property  Y: double read FY write FY;
   property  ClassFilter: TRectEntityClass read FClassFilter write FClassFilter;
end;




TRectEntityEditor = class(TDrawRectTool)
private
   FFixedRatio: boolean;
   FRatio: double;
   FSelectedElement: TSelectedElement;
   FSelectTool: TSelectRectEntityByPoint;
   FForm: TFormToolManager;
   FModified: boolean;
   function  GetSelectedElement: TRectEntity;
   function  GetSelCount: longInt;
   function  GetNoSelect: boolean;
   function  GetSelectTool: TSelectRectEntityByPoint;
   {
   function  GetEditorForSelect: TEditorIATool;
   }
   function    GetFormCursor: TCursor; virtual;
   procedure   SetFormCursor(AValue: TCursor); virtual;
   function    GetFormManager: TFormToolManager;
protected
   function  GetCursor: TCursor; override;
   function  GetName: string; override;
   property  FormManager: TFormToolManager read GetFormManager;
   function  GetClassFilter: TRectEntityClass; virtual;
   function  GetNoElementCursor: TCursor; virtual;
   procedure SetSelectedElement(aValue: TRectEntity); virtual;
   procedure CheckRatio; virtual;
public
   OShiftRect: TRect;
   procedure On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   procedure On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_Point(aCoord: TDoubleCoord); override;
   procedure SetupSelect; virtual;
   procedure CheckSelectedElement(ax,ay: longInt; var aManipulators, aRect: boolean; var  aDragStatus: TDragStatus);

   {
   property  EditorForSelect: TEditorIATool read GetEditorForSelect;
   }
   property  SelCount: longInt read GetSelCount;
   property  NoSelect: boolean read GetNoSelect;
   property  SelectedElement: TRectEntity read GetSelectedElement write SetSelectedElement;
   property  SelectTool: TSelectRectEntityByPoint read GetSelectTool;
   property  ClassFilterForSelect: TRectEntityClass read GetClassFilter;
   property  FormCursor: TCursor read GetFormCursor write SetFormCursor;
   property  NoElementCursor: TCursor read GetNoElementCursor;
   property  FixedRatio: boolean read FFixedRatio write FFixedRatio;
   property  Ratio: double read FRatio write FRatio;
published
   property Modified: boolean read FModified write FModified;
end;




IMPLEMENTATION

uses
   WinProcs,
   Classes,
   Dialogs,
   forms,
   MathLib,
   e_Device,
   e_View,
   {$IFNDEF NOMAP}
   f_EdText,
   f_MSEdit,
   t_MSEdit,
   {$ENDIF}
   m_Orbis,
   m_Proj,
   u_Cursor;



{ tdrawLineTool }



function  tDrawLineTool.GetName: string;
begin
   result := 'Kresltko lini';
end;

procedure tDrawLineTool.On_Erase(aCoord: TDoubleCoord);
begin
end;

procedure tDrawLineTool.On_Draw(aCoord: TDoubleCoord);
begin
end;


procedure tDrawLineTool.TakePoint;
var
  C1: TDoubleCoord;
begin
  C1 := Device.Projection.GetModelPoint(Pnt1.X, Pnt1.Y);
  on_Point(C1)
end;


procedure tDrawLineTool.On_Point(aCoord: TDoubleCoord);
begin
end;





procedure tDrawLineTool.On_Line(aCrd1, aCrd2: TDoubleCoord);
begin

end;


procedure tDrawLineTool.On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean);
var
  aPoint: WinTypes.TPoint;
begin
   GetCursorPos(aPoint);
   case aMessage.Key of
   VK_ESCAPE: PostMsgToOwners(tm_BreakInterpret, nil, DontKill);
   VK_Return: begin
                On_Erase(PrevCoord);
                Drawn:= false;
                TakePoint;
              end;
   VK_Down: begin
              SetCursorPos(aPoint.X, aPoint.Y+1)
            end;
   VK_Left: begin
             SetCursorPos(aPoint.X-1, aPoint.Y)
            end;
   VK_Right: begin
               SetCursorPos(aPoint.X+1, aPoint.Y)
             end;
   VK_Up: begin
           SetCursorPos(aPoint.X, aPoint.Y-1)
          end;
   end;
end;




procedure tDrawLineTool.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   if Drawn then
   begin
      On_Erase(PrevCoord);
      Drawn:= false
   end;

   if aMessage.Button = mbLeft then
   begin
      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;
      Device.Brush.Style := bsClear;
      Pnt1.X := aMessage.x;
      Pnt1.Y := aMessage.y;
      Pnt2.X := Pnt1.X;
      Pnt2.Y := Pnt1.Y;
      TCanvasDevice(Device).Canvas.Pen.Width := 1;

      TCanvasDevice(Device).Canvas.MoveTo(Pnt1.X, Pnt1.Y);
      TCanvasDevice(Device).Canvas.LineTo(Pnt2.X, Pnt2.Y);
   end;
   Handled := true;
end;


procedure tDrawLineTool.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   C1, C2: TDoubleCoord;
begin
    Handled := true;
{    if not aMessage.ActiveDevice then exit; }

    if Drawn then
    begin
       On_Erase(PrevCoord);
       Drawn:= false
    end;

    if aMessage.Button = mbLeft then
    begin
      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;

      TCanvasDevice(Device).Canvas.MoveTo(Pnt1.X, Pnt1.Y);
      TCanvasDevice(Device).Canvas.LineTo(Pnt2.X, Pnt2.Y);

       if (abs(Pnt1.X-Pnt2.X) <= 4) and (abs(Pnt1.Y-Pnt2.Y) <= 4) then
       begin
          Pnt1.X := aMessage.x;
          Pnt1.Y := aMessage.y;
          Pnt2:= Pnt1;
          C1 := Device.Projection.GetModelPoint(Pnt1.X, Pnt1.Y);
          on_Point(C1)
       end
       else
       begin
         C1 := Device.Projection.GetModelPoint(Pnt1.X, Pnt1.Y);
         Pnt2.X := aMessage.x;
         Pnt2.Y := aMessage.y;
         C2 := Device.Projection.GetModelPoint(Pnt2.X, Pnt2.Y);
         on_Line(C1, C2);
       end;
       TCanvasDevice(Device).Canvas.Pen.Mode  := pmCopy;
       TCanvasDevice(Device).Canvas.Pen.Color := clBlack;
       TCanvasDevice(Device).Canvas.Pen.Width := 1;
    end;
end;


function TDrawLineTool.GetAngle: double;
begin
  result:= ATan2M(Pnt2.X-Pnt1.X, Pnt2.Y-Pnt1.Y)
end;


procedure tDrawLineTool.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   C: TDoubleCoord;
begin
   Handled := false;
{   if not aMessage.ActiveDevice then exit; }
   C := Device.Projection.GetModelPoint(aMessage.X, aMessage.Y);

   if Drawn then
   begin
      On_Erase(PrevCoord);
      Drawn:= false
   end;

   if not Drawn then
   begin
      On_Draw(C);
      Drawn:= true;
      PrevCoord:= C
   end;

   if  ssLeft in aMessage.Shift then
   begin
      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;
      Device.Brush.Style := bsClear;
      TCanvasDevice(Device).Canvas.Pen.Width := 1;

      TCanvasDevice(Device).Canvas.MoveTo(Pnt1.X, Pnt1.Y);
      TCanvasDevice(Device).Canvas.LineTo(Pnt2.X, Pnt2.Y);

      Pnt2.X  := aMessage.x;
      Pnt2.Y  := aMessage.y;

      TCanvasDevice(Device).Canvas.MoveTo(Pnt1.X, Pnt1.Y);
      TCanvasDevice(Device).Canvas.LineTo(Pnt2.X, Pnt2.Y);
   end
   else
   begin
     Pnt1.X := aMessage.x;
     Pnt1.Y := aMessage.y;
     Pnt2:= Pnt1
   end
end;



procedure tDrawLineTool.on_tm_SetDrawnToFalse(var Handled: boolean);
begin
  Drawn:= false;
end;




procedure TDrawLineTool.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;

   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
{
   case aMessage.Message of
      tm_SetDrawnToFalse:      on_tm_SetDrawnToFalse(Handled);
   end;
}
end;





{ TDrawRectTool }

function  TDrawRectTool.GetName: string;
begin
   result := 'Kresltko obdlnku';
end;

procedure TDrawRectTool.On_Erase(aCoord: TDoubleCoord);
begin
end;

procedure TDrawRectTool.On_Draw(aCoord: TDoubleCoord);
begin
end;


procedure TDrawRectTool.On_Point(aCoord: TDoubleCoord);
begin

end;

procedure TDrawRectTool.On_Rect(aRect: TDoubleRect);
begin

end;


procedure TDrawRectTool.On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean);
begin
   if aMessage.Key = VK_ESCAPE then PostMsgToOwners(tm_BreakInterpret, nil, DontKill);
end;






procedure TDrawRectTool.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   if Drawn then
   begin
      On_Erase(PrevCoord);
      Drawn:= false
   end;

   if aMessage.Button = mbLeft then
   begin
      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;
      Device.Brush.Style := bsClear;
      ShiftRect := Rect(aMessage.x,aMessage.y,aMessage.x,aMessage.y);
      TCanvasDevice(Device).Canvas.Pen.Width := 1;
      with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
   end;
   Handled := true;
end;

procedure TDrawRectTool.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   R: TDoubleRect;
   C: TDoubleCoord;
begin
    Handled := true;
    if not aMessage.ActiveDevice then exit;

    if Drawn then
    begin
       On_Erase(PrevCoord);
       Drawn:= false
    end;

    if aMessage.Button = mbLeft then
    begin
       with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
       TCanvasDevice(Device).Canvas.Pen.Mode  := pmCopy;
       TCanvasDevice(Device).Canvas.Pen.Color := clBlack;
       TCanvasDevice(Device).Canvas.Pen.Width := 1;
       with ShiftRect do
       begin
          if (Right-Left <= 2) and (Bottom - Top <= 2) then
          begin
             C := Device.Projection.GetModelPoint(ShiftRect.Left, ShiftRect.Bottom);
             on_Point(C);
          end
          else
          begin
             with ShiftRect do R := Device.Projection.GetModelRect(Left, Top, Right, Bottom);
             on_Rect(R);
          end;
       end;
    end;
end;


procedure TDrawRectTool.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   C: TDoubleCoord;
begin
   Handled := false;
   if not aMessage.ActiveDevice then exit;
   C := Device.Projection.GetModelPoint(aMessage.X, aMessage.Y);

   if Drawn then
   begin
      On_Erase(PrevCoord);
      Drawn:= false
   end;

   if not Drawn then
   begin
      On_Draw(C);
      Drawn:= true;
      PrevCoord:= C
   end;

   if  ssLeft in aMessage.Shift then
   begin
      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;
      Device.Brush.Style := bsClear;
      TCanvasDevice(Device).Canvas.Pen.Width := 1;
      with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
      ShiftRect.Right  := aMessage.x;
      ShiftRect.Bottom := aMessage.y;
      with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
   end;
end;






{ TRectEntityEditor }

function  TRectEntityEditor.GetName: string;
begin
   result := 'Eitor mapovho listu';
end;


function  TRectEntityEditor.GetCursor: TCursor;
begin
   result := cr_Ruka;
end;


function  TRectEntityEditor.GetClassFilter: TRectEntityClass;
begin
   result := nil;
end;


procedure TRectEntityEditor.CheckSelectedElement(ax,ay: longInt; var aManipulators,
                                                 aRect: boolean; var aDragStatus: TDragStatus);
var
   DR: WinTypes.TRect;
   x2, y2: longInt;
begin
   aManipulators := false;
   aRect := false;
   aDragStatus := [];
   if SelectedElement = nil then exit;
   DR := Device.Projection.GetPageRect(SelectedElement.MinX, SelectedElement.MinY, SelectedElement.MaxX, SelectedElement.MaxY);
   if (ax > DR.Left-6) and (ax < DR.Right+6) and (ay > DR.Top-6) and (ay < DR.Bottom+6) then
   begin
      x2 := DR.Left  + ((DR.Right - DR.Left) div 2);
      y2 := DR.Top   + ((DR.Bottom - DR.Top) div 2);
      if ax < DR.Left+6 then
      begin
         if ay < DR.Top+6 then aDragStatus := [dsLeft, dsTop] else
         if ay > DR.Bottom-6 then aDragStatus := [dsLeft, dsBottom] else
         if (ay > y2-6) and (ay < y2+6)then aDragStatus := [dsLeft];
      end
      else
      if ax > DR.Right-6 then
      begin
         if ay < DR.Top+6 then aDragStatus := [dsRight, dsTop] else
         if ay > DR.Bottom-6 then aDragStatus := [dsRight, dsBottom] else
         if (ay > y2-6) and (ay < y2+6)then aDragStatus := [dsRight];
      end
      else
      if (ax > x2-6) and (ax < x2+6)then
      begin
         if ay < DR.Top+6 then aDragStatus := [dsTop] else
         if ay > DR.Bottom-6 then aDragStatus := [dsBottom];
      end;
      if aDragStatus <> [] then aManipulators := true
      else
      if (ax > DR.Left) and (ax < DR.Right) and (ay > DR.Top) and (ay < DR.Bottom) then
      begin
         aRect := true;
         aDragStatus := [dsLeft, dsTop, dsRight, dsBottom];
      end;
   end
   else exit;
end;


function  TRectEntityEditor.GetSelectTool: TSelectRectEntityByPoint;
begin
   if FSelectTool = nil then
   begin
      FSelectTool := TSelectRectEntityByPoint.Create;
      FSelectTool.ClassFilter := ClassFilterForSelect;
      AddTool(FSelectTool);
   end;
   result := FSelectTool;
end;



procedure TRectEntityEditor.On_Point(aCoord: TDoubleCoord);
//var
//   Sel: TSelectRectEntityByPoint;
begin
   {
   Sel := TSelectRectEntityByPoint.Create;
   Sel.Input := Input;
   Sel.x := aCoord.x;
   Sel.y := aCoord.y;
   Sel.DoIt;
   SelectedElement := TRectEntity(Sel.Output);

   Sel.Free;
   }
end;

procedure TRectEntityEditor.On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean);
var
   R: TRectEntity;
   i: longInt;
begin
   if aMessage.Key = VK_Delete then
   begin
      if (SelectedElement <> nil) and (SelectedElement <> Input) then
      begin
         R := SelectedElement;
         i := TRectEntity(Input).IndexOf(R);
         SelectedElement := nil;
         if i > 0 then SelectedElement :=TRectEntity(TRectEntity(Input).Elements[i-1]) else
         if i < TRectEntity(Input).Count -1 then SelectedElement :=TRectEntity(TRectEntity(Input).Elements[i+1]);
         TRectEntity(Input).FreeElement(R);
         Modified := true;
         PostMsgToOwners(tm_Review, nil, DontKill);
      end;
   end;
end;


procedure TRectEntityEditor.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   R: TRectEntity;
   aCoord: TDoubleCoord;
   aManipulators: boolean;
   aRect: boolean;
   aDragStatus: TDragStatus;
begin
   if aMessage.Button = mbLeft then
   begin
      CheckSelectedElement(aMessage.x, aMessage.y, aManipulators, aRect, aDragStatus);
      if aManipulators or aRect then DragStatus := aDragStatus;

      if not aManipulators then
      begin
         aCoord := Device.Projection.GetModelPoint(aMessage.x, aMessage.y);
         SelectTool.Input := Input;
         SelectTool.x := aCoord.x;
         SelectTool.y := aCoord.y;
         SelectTool.DoIt;
         R := TRectEntity(SelectTool.Output);
         if R = nil then
         begin
            SelectedElement := nil;
            exit;
         end;
         if R <> SelectedElement then
         begin
            SelectedElement := R;
            DragStatus := [];
         end;
      end;

      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;
      Device.Brush.Style := bsClear;
      R := SelectedElement;
      if (R <> nil) then
      begin
         ShiftRect := Device.Projection.GetPageRect(R.Minx, R.Miny, R.Maxx, R.Maxy);
      end
      else
      begin
         ShiftRect := Rect(aMessage.x,aMessage.y,aMessage.x,aMessage.y);
         DragStatus := [dsRight, dsBottom];
      end;
      RefPoint := Point(aMessage.x, aMessage.y);
      TCanvasDevice(Device).Canvas.Pen.Width := 1;
      with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
      Device.Pen.Mode  := pmCopy;
   end;
   Handled := true;
end;


procedure TRectEntityEditor.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   dx, dy: longInt;
   aManipulators, aRect: boolean;
   aDragStatus: TDragStatus;
   aCursor: TCursor;
   R: TRectEntity;
   aCoord: TDoubleCoord;
begin
   Handled := false;
   if not aMessage.ActiveDevice then exit;
   if SelectedElement = nil then exit;
   if SelectedElement is TDesktop then exit;
   if  ssLeft in aMessage.Shift then
   begin
      dx := aMessage.x - RefPoint.x;
      dy := aMessage.y - RefPoint.y;
      RefPoint := Point(aMessage.x, aMessage.y);
      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;
      Device.Brush.Style := bsClear;
      TCanvasDevice(Device).Canvas.Pen.Width := 1;
      OShiftRect := ShiftRect;
      with ShiftRect do
      begin
         TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
         if dsLeft in DragStatus then Left := Left + dx;
         if dsRight in DragStatus then Right := Right + dx;
         if dsTop in DragStatus then Top := Top + dy;
         if dsBottom in DragStatus then Bottom := Bottom + dy;
         if FixedRatio then CheckRatio;
         TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
      end;
      Modified := true;
      Device.Pen.Mode  := pmCopy;
      PostMsgToOwners(tm_SelectSizeChange, nil, DontKill);
   end
   else
   begin
      CheckSelectedElement(aMessage.x, aMessage.y, aManipulators, aRect, aDragStatus);
      if aManipulators then
      begin
         if (aDragStatus = [dsLeft, dsTop]) or (aDragStatus = [dsRight, dsBottom])then
            aCursor := crSizeNWSE else
         if (aDragStatus = [dsLeft, dsBottom]) or (aDragStatus = [dsRight, dsTop])then
            aCursor := crSizeNESW else
         if (aDragStatus = [dsBottom]) or (aDragStatus = [dsTop])then
            aCursor := crSizeNS else
            aCursor := crSizeWE;
      end
      else
      if aRect then
      begin
         aCursor := crHandPoint//crSize;
      end
      else
      begin
         aCursor := Cursor;
         SelectTool.Input := Input;
         aCoord := Device.Projection.GetModelPoint(aMessage.x, aMessage.y);
         SelectTool.x := aCoord.x;
         SelectTool.y := aCoord.y;
         SelectTool.DoIt;
         R := TRectEntity(SelectTool.Output);
         if R = nil then aCursor := NoElementCursor;
      end;
      if aRect then
      begin
         SelectTool.Input := Input;
         aCoord := Device.Projection.GetModelPoint(aMessage.x, aMessage.y);
         SelectTool.x := aCoord.x;
         SelectTool.y := aCoord.y;
         SelectTool.DoIt;
         R := TRectEntity(SelectTool.Output);
         if R <> SelectedElement then aCursor := Cursor;
      end;
      if FormCursor <> aCursor then FormCursor := aCursor;
   end;
end;



procedure TRectEntityEditor.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   R: TDoubleRect;
   RE: TRectEntity;
begin
    Handled := true;
    if SelectedElement = nil then
    begin
       PostMsgToOwners(tm_Review, nil, DontKill);
       exit;
    end;
    if not aMessage.ActiveDevice then exit;
    if aMessage.Button = mbLeft then
    begin
       with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
       TCanvasDevice(Device).Canvas.Pen.Mode  := pmCopy;
       TCanvasDevice(Device).Canvas.Pen.Color := clBlack;
       TCanvasDevice(Device).Canvas.Pen.Width := 1;
       if SelectedElement = nil then exit;
       RE := SelectedElement;
       with ShiftRect do
       begin
          with ShiftRect do R := Device.Projection.GetModelRect(Left, Top, Right, Bottom);
          RE.Minx := R.Minx;
          RE.Miny := R.MinY;
          RE.Maxx := R.MaxX;
          RE.Maxy := R.MaxY;
          {
          RE.Update;
          }
       end;
       SelectedElement := RE;
       PostMsgToOwners(tm_Review, nil, DontKill);
    end;
end;


procedure TRectEntityEditor.SetupSelect;
{$IFNDEF NOMAP}
var
   R: TText;
   DD: TEditTextDialog;
   i: longInt;
begin
   R := TText(SelectedElement);
   if TElement(R) is TText then
   begin
      Application.CreateForm(TEditTextDialog, DD);
      DD.TextElement := TText(R.CreateEnvelope);
      if DD.ShowModal = mrOK then
      begin
         R.Text   := DD.TextElement.Text;
         R.FontName   := DD.TextElement.FontName;
         R.Color   := DD.TextElement.Color;
         R.Height := DD.TextElement.Height;
         R.Orientation := DD.TextElement.Orientation;
         { ... }
      end;
      {
      ActualFont := DD.FontDialog.Font;
      }
      DD.TextElement.Free;
      DD.Free;
      for i := 0 to 30 do Application.ProcessMessages;
      PostMsgToOwners(tm_ReView, nil, DontKill);
   end;
end;
{$ELSE}
begin
end;
{$ENDIF}

function  TRectEntityEditor.GetSelectedElement: TRectEntity;
begin
   result := nil;
   if FSelectedElement = nil then exit;
   result :=  FSelectedElement.Sel;
end;


procedure TRectEntityEditor.SetSelectedElement(aValue: TRectEntity);
begin
   if FSelectedElement <> nil then
   begin
      TEntity(Input).FreeElement(FSelectedElement);
      FSelectedElement := nil;
   end;
   if aValue <> nil then
   begin
      FSelectedElement := TSelectedElement.Create;
      FSelectedElement.Sel := aValue;
      FSelectedElement.Assign(aValue);
      TEntity(Input).AddElement(FSelectedElement);
   end;
   {
   PostMsgToOwners(tm_Review, nil, DontKill);
   }
end;


procedure TRectEntityEditor.CheckRatio;
var
   dx1, dx2, dy1, dy2, dx, dy: longInt;
   ddx, ddy: double;
begin
   if (Ratio < 0.0001) then exit;
   dx1 := OShiftRect.Right - OShiftRect.Left;
   dy1 := OShiftRect.Bottom - OShiftRect.Top;
   dx2 := ShiftRect.Right - ShiftRect.Left;
   dy2 := ShiftRect.Bottom - ShiftRect.Top;
   dx1 := abs(dx2 - dx1);
   dy1 := abs(dy2 - dy1);
   ddx  := dx2;
   ddy  := dy2;
   dx  := dx2;
   dy  := dy2;
   if dx1 > dy1 then dY := abs(round(ddX / Ratio))
                else dX := abs(round(ddY * Ratio));
   with ShiftRect do
   begin
      if (dsLeft in DragStatus) and (dsRight in DragStatus) and
         (dsTop in DragStatus) and (dsBottom in DragStatus) then
      begin
         Right   := Left + dX;
         Bottom  := Top  + dY;
      end
      else
      begin
         if dsLeft in DragStatus then
         begin
            Left := Right - dx;
            if dsTop in DragStatus then Top := Bottom - dy
                                   else Bottom := Top + dy;
            exit;
         end;
         if dsRight in DragStatus then
         begin
            Right := Left + dx;
            if dsTop in DragStatus then Top := Bottom - dy
                                   else Bottom := Top + dy;
            exit;
         end;
         if dsTop in DragStatus then
         begin
            Top := Bottom - dy;
            if dsLeft in DragStatus then Left := Right - dx
                                    else Right:= Left  + dx;
            exit;
         end;
         if dsBottom in DragStatus then
         begin
            Bottom := Top + dy;
            if dsLeft in DragStatus then Left := Right - dx
                                    else Right:= Left  + dx;
            exit;
         end;
      end;
   end;
end;


function  TRectEntityEditor.GetSelCount: longInt;
begin
   {
   result := SelectedElements.Count;
   }
   result := 0;
end;



function  TRectEntityEditor.GetNoSelect: boolean;
begin
   {
   result := (FSelectedElements = nil) or (FSelectedElements.NoElements);
   }
   result := FSelectedElement = nil;
end;

{

function  TRectEntityEditor.GetEditorForSelect: TEditorIATool;
var
   i: longInt;
   E: TEditorIATool;
   SelectClass: TElementClass;
begin
   result := nil;
   if SelectedElement = nil then exit;
   SelectClass := TElementClass(SelectedElement.ClassType);
   for i := 0 to Tools.Count -1 do
   begin
      E := TEditorIATool(Tool[i]);
      if (E is TEditorIATool) and (E.ItsMyClass(SelectClass)) then
      begin
         result := E;
         exit;
      end;
   end;
end;

}

function    TRectEntityEditor.GetFormManager: TFormToolManager;
begin
   if FForm = nil then FForm := TFormToolManager(FindOwnerClass(TFormToolManager));
   result := FForm;
end;


function    TRectEntityEditor.GetFormCursor: TCursor;
begin
   result := FormManager.FormCursor;
end;


procedure   TRectEntityEditor.SetFormCursor(AValue: TCursor);
begin
   FormManager.FormCursor := aValue;
end;


function     TRectEntityEditor.GetNoElementCursor: TCursor;
begin
   result := cr_NoElement;
end;


{ TSelectRectEntityByPoint }


function  TSelectRectEntityByPoint.QGoInside(Element: TElement): boolean;
begin
   result := Element is TRectEntity;
end;


function  TSelectRectEntityByPoint.QProcessBefore(Element: TElement): boolean;
begin
   result := (Element is TRectEntity) and not (Element is TSelectedElement);
   if result then
      if ClassFilter <> nil then result := (Element is ClassFilter);
end;


procedure TSelectRectEntityByPoint.Process(Element: TElement);
begin
   if not (Element is TRectEntity) then exit;
   if TRectEntity(Element).PointIsInside(X, Y) then Output := Element;
end;


procedure TSelectRectEntityByPoint.DoIt;
begin
   Output := nil;
   inherited DoIt;
end;





END.
