unit U_cktm;

interface
uses
   Classes;
const

twOK         = 0;
twNoTimeWall = 1;
twDemo       = 2;
twTimeOut    = 3;
twOther      = 4;
twTimeWall   = 15;

twcTrial     = 1;
twcLicence   = 2;
twcHDControl = 4;


type


TRegRec= packed record
   Key         : longint;
   TimeWall    : TDateTime;
   Licence     : longInt;
   Licence1    : longInt;
   LastTime    : TDateTime;
   User        : string[30];
   Company     : string[30];
   HDControl   : double;
   CheckSum    : double;
   ModuleName  : string[20];
   MajorVersion: word;
   MinorVersion: word;
   IsTimeWall  : boolean;
   IsTrial     : boolean;
   IsLicence   : boolean;
   IsHDControl : boolean;
end;

TAppRec = packed record
   AppName      : string[20];
   MajorVersion : word;
   MinorVersion : word;
end;

TRegistrationInfo = record
   Reg: TRegRec;
   Apps: Array of TAppRec;
end;

TAppInstallInfo = class
private
    FInstaled: boolean;
    FDirs: TStringList;
    FModul: integer;
    function GetDirs: TStringList;
    procedure SetDirs(const Value: TStringList);
published
   property Instaled: boolean read FInstaled write FInstaled;
   property Modul: integer read FModul write FModul;
   property Dirs: TStringList read GetDirs write SetDirs;
end;

function  RBOVersionString(const R: TRegistrationInfo): string;
function  AppVersionString(const A: TAppRec): string;
procedure SetAppVersionString(const ver: string; var A: TAppRec);

function  CheckModulRegistration(const aName: string): boolean;
function  CheckRegistryForSetup(const App: TAppRec;var R: TRegistrationInfo; IsRBO: boolean; AppInfo: TAppInstallInfo): boolean;
procedure SetupRegistry(const App: TAppRec;var R: TRegistrationInfo; IsRBO: boolean; AppInfo: TAppInstallInfo);
function  DecodeRBO(Buf: Pointer; BufSize: integer): TRegistrationInfo;
function  EncodeRBO(const R: TRegistrationInfo; var Buf: Pointer;var BufSize: integer): boolean;
function  FindRBOinRegistry(const R: TRegistrationInfo; var RR: TRegistrationInfo): boolean;
function  LoadRBOFromRegistry(aValue: string): TRegistrationInfo;
procedure StoreRBOToRegistry(var R: TRegistrationInfo);
function  LoadRBOFromFile(aFileName: string): TRegistrationInfo;
procedure SaveRBOToFile(var R: TRegistrationInfo; aFileName: string);

function  CheckModulesInRegistry(var R: TRegistrationInfo): boolean;
procedure JoinModules(var R,R1: TRegistrationInfo);

function  CodeHDC(HDC: longInt;const R: TRegistrationInfo): double;
function  DeCodeHDC(cHDC: double;const R: TRegistrationInfo): longInt;
function  GetProtectionLevel(const R: TRegistrationInfo): integer;

var
   IsDemo: boolean;
   TimeWall: TDateTime;
   Licence: double;

implementation
uses
   Dialogs,
   Forms,
   Controls,
   WinProcs,
   {!T_Main,}
   SysUtils,
   Registry,
   FileVersion,
   f_CkTm,
   f_CkTmRg,
   f_YesNo,
   u_Crypt,
   u_HDTest,
   u_String;

const
  c1899 = 693594;
  rgTopolPro = 'SOFTWARE\Topol Pro';
  rgRBO      = 'SOFTWARE\SCP info';
  rgAPP      =  rgTopolPro + '\APP';
  rgModul    =  rgTopolPro + '\RBO';

function  RBOVersionString(const R: TRegistrationInfo): string;
begin
   Result := IntToStr(R.Reg.MajorVersion)+'.'+IntToStr(R.Reg.MinorVersion);
end;

function  AppVersionString(const A: TAppRec): string;
begin
   Result := IntToStr(A.MajorVersion)+'.'+IntToStr(A.MinorVersion);
end;

procedure SetAppVersionString(const ver: string; var A: TAppRec);
var
   S1,S2: string;
begin
   if not StringAsTwoParams(ver, '.', S1,S2) then S2 := '0';
   A.MajorVersion := StrToInt(S1);
   A.MinorVersion := StrToInt(S2);
end;

function  DecodeRBO(Buf: Pointer; BufSize: integer): TRegistrationInfo;
var
   PB: PByteArray;
   i,j: integer;
   ix: integer;
   AppsCount: integer;
begin
   // decode buffer
   UnCrypt(Buf, BufSize);
   PB := Buf;
   i := SizeOf(TRegRec);
   ix := SizeOf(TAppRec);
   AppsCount := (BufSize - i) div ix;
   SetLength(result.Apps,AppsCount);
   if ((BufSize - i) mod ix) > 0 then ShowMessage('Blbost');
   Move(PB^[0], result.Reg, SizeOf(TRegRec));
   for j := 0 to AppsCount - 1 do
   begin
      move(PB^[i],result.Apps[j],SizeOf(TAppRec));
      inc(i,ix);
   end;
   FreeMem(Buf,BufSize);
end;

function  EncodeRBO(const R: TRegistrationInfo; var Buf: Pointer;var BufSize: integer): boolean;
var
   PB: PByteArray;
   i,Ix,j: integer;
begin
   result := true;
   BufSize := SizeOf(TRegRec) + (High(R.Apps)+1)*SizeOf(TAppRec);
   GetMem(Buf,BufSize);
   PB := Buf;
   i := SizeOf(TRegRec);
   ix := SizeOf(TAppRec);
   Move(R.Reg,PB^[0],i);
   for j := 0 to High(R.Apps) do
   begin
      Move(R.Apps[j],PB^[i],ix);
      inc(i,ix);
   end;
   Crypt(Buf, BufSize);
   // code buffer
end;

function  FindRBOinRegistry(const R: TRegistrationInfo; var RR: TRegistrationInfo): boolean;
var
   Reg: TRegistry;
   SL: TStringList;
   i: integer;
begin
   result := false;
   Reg := TRegistry.Create;
   Reg.RootKey := HKEY_LOCAL_MACHINE;
   Reg.OpenKey(rgRBO, true);
   SL := TStringList.Create;
   Reg.GetValueNames(SL);
   Reg.CloseKey;
   for i := 0 to SL.Count - 1 do
   begin
      RR := LoadRBOFromRegistry(SL.Strings[i]);
      if (R.Reg.ModuleName = RR.Reg.ModuleName) and
         (R.Reg.MajorVersion = RR.Reg.MajorVersion) then
      begin
         Result := true;
         Break;
      end;
   end;
   SL.Free;
   Reg.Free;
end;

function  LoadRBOFromRegistry(aValue: string): TRegistrationInfo;
var
   Reg: Tregistry;
   Buf: Pointer;
   BufSize: integer;
begin
   Reg := TRegistry.Create;
   Reg.RootKey := HKEY_LOCAL_MACHINE;
   if not Reg.OpenKey(rgRBO, true) then ShowMessage('oops');
   BufSize := Reg.GetDataSize(aValue);
   if BufSize < 1 then
   begin
      ShowMessage('Chybn registrace programu');
      exit;
   end;
   GetMem(Buf,BufSize);
   Reg.ReadBinaryData(aValue,Buf^,BufSize);
   result := DecodeRBO(Buf,BufSize);
   Reg.Free;
end;

procedure StoreRBOToRegistry(var R: TRegistrationInfo);
var
   Reg: Tregistry;
   Buf: Pointer;
   BufSize: integer;
   i: integer;
   aValue: integer;
   LK: integer;
   SLK: string;
begin
   Reg := TRegistry.Create;
   Reg.RootKey := HKEY_LOCAL_MACHINE;
   aValue := R.Reg.Key;
   SLK := 'LK';
   if aValue = 0 then
   begin
      Reg.OpenKey(rgRBO+'\LK', true);
      if not Reg.ValueExists(SLK) then
      begin
         LK := 1;
      end
      else
      begin
         LK := Reg.ReadInteger(SLK);
      end;
      aValue := LK;
      inc(LK);
      Reg.WriteInteger(SLK,LK);
      Reg.CloseKey;
      R.Reg.Key := aValue;
   end;
   Reg.OpenKey(rgRBO, true);
   try
   EncodeRBO(R,Buf,BufSize);
   Reg.WriteBinaryData(IntToStr(aValue),Buf^,BufSize);
   except
      ShowMessage('RBO!!!');
   end;
   Reg.CloseKey;
   FreeMem(Buf,BufSize);
   for i := 0 to High(R.Apps) do
   begin
      Reg.OpenKey(rgModul+'\'+R.Apps[i].AppName+'\'+AppVersionString(R.Apps[i]),true);
      (*
      Reg.WriteInteger('Major version',R.Apps[i].MajorVersion);
      Reg.WriteInteger('Minor version',R.Apps[i].MinorVersion);
      Reg.WriteInteger('release',R.Apps[i].reslease);
      Reg.WriteInteger('build',R.Apps[i].reslease);
      *)
      try
      Reg.WriteString('Modul',CryptLongInt(R.Reg.Key));
      except
         ShowMessage('App: '+R.Apps[i].AppName+' i:'+IntToStr(i));
      end;
      Reg.CloseKey;
   end;
   Reg.Free;

   // uloen apps nebo kontrola keys !!!!!!!!!!!!!!!!!!!
end;

function  LoadRBOFromFile(aFileName: string): TRegistrationInfo;
var
   Buf: Pointer;
   BufSize: integer;
   H: integer;
   i: integer;
begin
   H := FileOpen(aFileName,fmOpenRead);
   BufSize := GetFileSize(H,@i);
   GetMem(Buf,BufSize);
   FileRead(H,Buf^,BufSize);
   result := DecodeRBO(Buf,BufSize);
   FIleClose(H);
end;

procedure SaveRBOToFile(var R: TRegistrationInfo; aFileName: string);
var
   Buf: Pointer;
   BufSize: integer;
   H: integer;
begin
   EncodeRBO(R,Buf,BufSize);
   H := FileCreate(aFileName);
   FileWrite(H,Buf^,BufSize);
   FileClose(H);
   FreeMem(Buf,BufSize);
end;

function  CheckModulesInRegistry(var R: TRegistrationInfo): boolean;
var
   Reg: Tregistry;
   i: integer;
   RBO: integer;
   RBOreg: integer;
   S: string;
begin
   result := true;
   RBO := R.Reg.Key;
   Reg := TRegistry.Create;
   Reg.RootKey := HKEY_LOCAL_MACHINE;
   for i := 0 to High(R.Apps) do
   begin
      S := rgModul+'\'+R.Apps[i].AppName+'\'+AppVersionString(R.Apps[i]);
      if Reg.KeyExists(S) then
      begin
         if RBO = 0 then
         begin
            // kontrola verze
            if not YesNoDialog('Program '+R.Apps[i].AppName+' je na tomto potai ji nainstalovn, pokraovat?') then
            begin
               result := false;
               break;
            end;
         end
         else
         begin
            Reg.OpenKey(S,true);
            RBOreg := UnCryptLongInt(Reg.ReadString('Modul'));
            if RBO <> RBOreg then
            if not YesNoDialog('Program '+R.Apps[i].AppName+' je  ji nainstalovn pod jinm modulem, pokraovat?') then
            begin
               result := false;
               break;
            end;
            Reg.CloseKey;
         end;
      end;
   end;
   Reg.Free;
end;

procedure JoinModules(var R,R1: TRegistrationInfo);
var
   i,j: integer;
   Maxi,Maxj: integer;
   RSize: integer;
   Found: boolean;
begin
   Maxi := High(R1.Apps);
   Maxj := High(R.Apps);
   RSize := Maxj + 1;
   for i := 0 to Maxi do
   begin
      Found := false;
      for j := 0 to Maxj do
      begin
         if CompareText(R1.Apps[i].AppName,R.Apps[j].AppName) = 0 then
         begin
            if (R1.Apps[i].MajorVersion = R.Apps[j].MajorVersion) and
               (R1.Apps[i].MinorVersion = R.Apps[j].MinorVersion) then
            begin
               Found := true;
               break;
            end;
         end;
      end;
      if Found then continue;
      inc(RSize);
      SetLength(R.Apps,RSize);
      R.Apps[RSize-1] := R1.Apps[i];
   end;
end;



procedure DecodeDate16(aDate: TDateTime; var aY, aM, aD: word);
begin
(*
{$IFDEF Win32}
  aDate:= aDate - c1899;
{$ENDIF}
*)
  DecodeDate(aDate, aY, aM, aD);
end;

function EncodeDate16(aY, aM, aD: word): TDateTime;
begin
  result:= EncodeDate(aY, aM, aD);
(*
{$IFDEF Win32}
  result:= result + c1899;
{$ENDIF}
*)
end;




function  GetProtectionLevel(const R: TRegistrationInfo): integer;
begin
   {
    trial ............. 1
    licence ........... 2
    HD ................ 4
    licence + trial ... 3
    HD + trial ........ 5
   }
   result := 0;
   if R.Reg.IsTrial then result := result or twcTrial;
   if R.Reg.IsLicence then result := result or twcLicence;
   if R.Reg.IsHDControl then result := result or twcHDControl;
end;

function  GetRandomDate: TDateTime;
begin
   Randomize;
   result := EncodeDate16(1990 + Random(6),1 + Random(11),1+random(27))+EncodeTime(random(23),random(58),random(58),0);
end;


function  CheckRegistryForSetup(const App: TAppRec;var R: TRegistrationInfo; IsRBO: boolean; AppInfo: TAppInstallInfo): boolean;
var
   RII: TRegistrationInfo;
   Reg: TRegistry;
   L1, L2: integer;
   isTW: boolean;
   AppKey: string;
   S: string;
   i: integer;
begin
   // Kontrola aplikace
   result := true;
   AppInfo.Instaled := false;
   Reg := TRegistry.Create;
   AppKey := rgApp + '\'+App.AppName+'\'+AppVersionString(App);
   Reg.RootKey := HKEY_LOCAL_MACHINE;
   if Reg.KeyExists(AppKey) then
   begin
      AppInfo.Instaled := true;
      Reg.OpenKey(AppKey, false);
      AppInfo.Modul := -1;
      if Reg.ValueExists('Modul') then
         AppInfo.Modul := UnCryptLongInt(Reg.ReadString('Modul'));
      Reg.CloseKey;
      Reg.OpenKey(AppKey+'\DIRS',true);
      Reg.GetValueNames(AppInfo.Dirs);
      for i := 0 to AppInfo.Dirs.Count - 1 do
      begin
         S := AppInfo.Dirs.Strings[i];
         S := S + '=' + Reg.ReadString(S);
         AppInfo.Dirs.Strings[i] := S;
      end;
   end;
   Reg.Free;
   if not IsRBO then exit;

   R.Reg.Key := 0;
   S := 'Aplikace '+App.AppName+' je na tomto potai ji nainstalovna pod jinm modulem, pokraovat?';
   if not FindRBOinRegistry(R,RII) then
   begin
      if AppInfo.Instaled then
      begin
         if not YesNoDialog(S) then
         begin
            result := false;
            exit;
         end;
      end;
      if not CheckModulesInRegistry(R) then result := false;
      //else StoreRBOToRegistry(RI);
      exit;
   end
   else
   begin
      if AppInfo.Modul <> RII.Reg.Key then
      begin
         if not YesNoDialog(S) then
         begin
            result := false;
            exit;
         end;
      end;
      R.Reg.Key := RII.Reg.Key;
      L1 :=  GetProtectionLevel(RII);
      L2 :=  GetProtectionLevel(R);
      isTW := false;
      { Pure trial }
      if (L1 and 1 = 1) and (L2 and 1 = 1) then
      begin
         isTW := true;
         if R.Reg.TimeWall = 0 then
         begin
            ShowMessage('Tato trialov verze ji byla na potai instalovna !');
            result := false;
            exit;
         end;
      end;
      // instalovn vy verze
      if ((L2 and 6) > (L1 and 6)) then
      begin
         R.Reg.Licence1 := 0;
         R.Reg.HDControl := 0;
         //R.CheckSum := CKSM;
         if not CheckModulesInRegistry(R) then result := false
         else
         begin
            JoinModules(R,RII);
            //StoreRBOToRegistry(RI);
         end;
         exit;
      end
      else
      { peinstalovn t sam verze }
      if ((L2 and 6) = (L1 and 6)) or IsTW then
      begin
         RII.Reg.TimeWall := R.Reg.TimeWall;
         RII.Reg.IsTrial := R.Reg.IsTrial;
         //CFR.CheckSum := CKSM;
         if not CheckModulesInRegistry(R) then result := false
         else
         begin
            JoinModules(RII,R);
            //StoreRBOToRegistry(RII);
            R := RII;
         end;
         exit;
      end
      else
      begin
         ShowMessage('Na potai ji byla instalovna chrnn verze, kontaktujte vrobce programu !');
         result := false;
         exit;
      end;
   end;
end;

procedure SetupRegistry(const App: TAppRec;var R: TRegistrationInfo; IsRBO: boolean; AppInfo: TAppInstallInfo);
var
   Reg: TRegistry;
   AppKey: string;
   S: string;
   i: integer;
begin
   if IsRBO then StoreRBOToRegistry(R);

   // Kontrola aplikace
   Reg := TRegistry.Create;
   AppKey := rgApp + '\'+App.AppName+'\'+AppVersionString(App);
   Reg.RootKey := HKEY_LOCAL_MACHINE;

   Reg.OpenKey(AppKey, true);
   if IsRBO then Reg.WriteString('Modul',CryptLongInt(R.Reg.Key));
   Reg.CloseKey;
   //Reg.DeleteKey(AppKey+'\DIRS');
   Reg.OpenKey(AppKey+'\DIRS',true);
   for i := 0 to AppInfo.Dirs.Count - 1 do
   begin
      S := AppInfo.Dirs.Names[i];
      Reg.WriteString(S,AppInfo.Dirs.Values[S]);
   end;
   Reg.Free;

end;


function CheckSumOfString(var S: string): double;
var
   i: longInt;
begin
   S := UpperCase(S);
   result := 0;
   for i := 1 to length(S) do
   begin
      if S[i] <> ' ' then result := result + Ord(S[i]);
   end;
end;

function CodeHDC(HDC: longInt;const R: TRegistrationInfo): double;
var
   D: double;
   S: string;
begin
   S := R.Reg.ModuleName;
   D := CheckSumOfString(S)+ R.Reg.MajorVersion+ R.Reg.MinorVersion;
   result := ((D + 3.14) * HDC) + 729.15 + D;
end;

function DeCodeHDC(cHDC: double;const R: TRegistrationInfo): longInt;
var
   D: double;
   S: string;
begin
   S := R.Reg.ModuleName;
   D := CheckSumOfString(S)+ R.Reg.MajorVersion+ R.Reg.MinorVersion;
   result := round((cHDC - 729.15 - D)/(3.14 + D));
end;


function CheckModulRegistration(const aName: string): boolean;
var
   App: TAppRec;
   FVer: TFileVersion;
   Reg: TRegistry;
   RBO: integer;
   RBOString: string;
   N,S: string;
   RInfo: TRegistrationInfo;
   NowTime: TDateTime;
   F: TRegistrationForm;
   LicenceOK, HDCOK: boolean;
   HighLevelControl: boolean;
   Y,M,D: word;
   MustBeStored: boolean;

   procedure ShowInstallError;
   begin
      ShowMessage(ExtractFileName(aName)+': Chybn nebo pokozen instalace programu !');
   end;

   function RegisterOK: boolean;
   var
     RD: TRegisterDialog;
     res: integer;
     HDN: double;
     HDOK: boolean;
   begin
      result := false;
      HDOK := false;
      HDN := 0;
      Application.CreateForm(TRegisterDialog,RD);
      RD.Label1.Caption := 'Trialov verze aplikace '+ ExtractFileName(aName);
      RD.Label2.Caption := 'asov omezen do '+DateTimeToStr(RInfo.Reg.TimeWall
      (*
      {$IFDEF Win32}
      - c1899
      {$ENDIF}
      *)
      );
      RD.ShowModal;
      res := RD.ModalResult;
      RD.Free;
      if not (res = mrOK) then exit;

      Application.CreateForm(TRegistrationForm, F);
      F.lb_Licence.Visible := false;
      F.ed_Licence.Visible := false;
      F.lb_Modul.Caption := RInfo.Reg.ModuleName;
      F.lb_Version.Caption := IntToStr(RInfo.Reg.MajorVersion)+'.'+IntToStr(RInfo.Reg.MinorVersion);
      F.ed_User.Text := RInfo.Reg.User;
      F.ed_Company.Text := RInfo.Reg.Company;
      F.ed_HD.Text := CryptLongInt(GetSystemHDSerialNumer);
      F.ed_HD.Enabled := false;
      F.ShowModal;
      if F.ModalResult = mrOK then
      begin
         HDN := UncryptDouble(F.ed_HDC.Text);
         HDOK := DecodeHDC(HDN,RInfo) =  GetSystemHDSerialNumer;
         if not HDOK then ShowMessage('Nesprvn ovovac kd ... registrace byla nespn');
      end;
      F.Free;
      if HDOK then
      begin
         RInfo.Reg.IsTrial := false;
         RInfo.Reg.IsHDControl := true;
         RInfo.Reg.HDControl := HDN;
         StoreRBOToRegistry(RInfo);
         result := true;
      end;

   end;

begin
   result := true;
   IsDemo := false;
   MustBeStored := false;
   TimeWall := 0;
   Reg := TRegistry.Create;
   Reg.RootKey := HKEY_LOCAL_MACHINE;
   N := ExtractFileName(aName);
   FVer := TFileVersion.Create;
   FVer.FileName := aName;
   App.AppName := N;
   App.MajorVersion := FVer.MajorVersion;
   App.MinorVersion := FVer.MinorVersion;
   FVer.Free;
   S := rgModul+'\'+N+'\'+AppVersionString(App);
   if not Reg.KeyExists(S) then
   begin
      ShowInstallError;
      result := false;
      Reg.Free;
      exit;
   end;
   Reg.OpenKey(S, false);
   if not Reg.ValueExists('Modul')then
   begin
      ShowInstallError;
      result := false;
      Reg.Free;
      exit;
   end;
   RBO := UnCryptLongInt(Reg.ReadString('Modul'));
   RBOString := IntToStr(RBO);
   Reg.CloseKey;
   Reg.OpenKey(rgRBO,true);
   if not Reg.ValueExists(RBOString) then
   begin
      ShowInstallError;
      result := false;
      Reg.Free;
      exit;
   end;
   Reg.Free;
   RInfo := LoadRBOFromRegistry(RBOString);
   // kontrola ptomnosti applikace v RBO !!!!!!!!!

   LicenceOK := true;
   HDCOK := true;
   HighLevelControl := RInfo.Reg.IsHDControl or RInfo.Reg.IsLicence;
   if HighLevelControl then
   begin
      if RInfo.Reg.IsLicence then
      begin
         LicenceOK :=  RInfo.Reg.Licence = RInfo.Reg.Licence1;
      end
      else LicenceOK := true;
      if RInfo.Reg.IsHDControl then
      begin
         HDCOK := DecodeHDC(RInfo.Reg.HDControl, RInfo) = GetSystemHDSerialNumer;
      end
      else HDCOK := true;
      if not (HDCOK and LicenceOK) then
      begin
         Application.CreateForm(TRegistrationForm, F);
         if LicenceOK then
         begin
            F.lb_Licence.Visible := false;
            F.ed_Licence.Visible := false;
         end;
         if HDCOK then F.g_HDControl.Visible := false;
         F.lb_Modul.Caption := RInfo.Reg.ModuleName;
         F.lb_Version.Caption := intToStr(RInfo.Reg.MajorVersion)+'.'+intToStr(RInfo.Reg.MinorVersion);
         if RInfo.Reg.Licence1 = 0 then F.ed_Licence.Text := ''
                                else F.ed_Licence.Text := floatToStr(RInfo.Reg.Licence);
         F.ed_User.Text := RInfo.Reg.User;
         F.ed_Company.Text := RInfo.Reg.Company;
         F.ed_HD.Text := CryptLongInt(GetSystemHDSerialNumer);
         F.ed_HD.Enabled := false;
         F.ShowModal;
         if F.ModalResult = mrOK then
         begin
            if not LicenceOK then
            begin
               MustBeStored := true;
               RInfo.Reg.Licence1 := StrToInt(F.ed_Licence.Text);
               LicenceOK := RInfo.Reg.Licence1 = RInfo.Reg.Licence;
               if not LicenceOK then ShowMessage('slo licence je chybn !');
               RInfo.Reg.User := F.ed_User.Text;
               RInfo.Reg.Company := F.ed_Company.Text;
            end;
            if not HDCOK then
            begin
               MustBeStored := true;
               RInfo.Reg.HDControl := UnCryptDouble(F.ed_HDC.Text);
               HDCOK := DecodeHDC(RInfo.Reg.HDControl,RInfo) =  GetSystemHDSerialNumer;
               if not HDCOK then ShowMessage('Nesprvn ovovac kd ... kontaktujte vrobce programu !');
            end;
            if not LicenceOK then RInfo.Reg.Licence1 := 0;
            if not HDCOK then RInfo.Reg.HDControl := 0;
         end;
         F.Free;
      end;
   end;

   if  HighLevelControl then
   begin
      if (HDCOK and LicenceOK) then
      begin
         if MustBeStored then
            StoreRBOtoRegistry(RInfo);
         Licence := RInfo.Reg.Licence;
      end
      else
      begin
         result := false;
         exit;
      end;
   end;

   if not RInfo.Reg.IsTrial then exit;

   NowTime := Now
   (*
   {$IFDEF Win32}
   + c1899
   {$ENDIF}
   *)
   ; { delam z toho 16 bit tdatetime }

   if RInfo.Reg.TimeWall = 0 then
   begin
      DecodeDate16(NowTime,Y,M,D);
      Inc(M);
      if M > 12 then
      begin
         M := 1;
         Inc(Y);
      end;
      RInfo.Reg.TimeWall := EncodeDate16(Y,M,D);
   end;
   TimeWall := RInfo.Reg.TimeWall;

  if RegisterOK then
  begin
     result := true;
     exit;
  end;

  if NowTime > RInfo.Reg.TimeWall then
   begin
      ShowMessage('Bylo pekroeno asov omezen ('+DateTimeToStr(RInfo.Reg.TimeWall
      (*
      {$IFDEF Win32}
      - c1899
      {$ENDIF}
      *)
      )+'), aplikace se ukon');
      result := false;

      RInfo.Reg.LastTime := NowTime;
      StoreRBOToRegistry(RInfo);
      Application.Terminate;
      exit;
   end;
   if NowTime < RInfo.Reg.LastTime then
   begin
      result := false;
      ShowMessage('Bylo patrn manipulovno s asem potae, aplikace se ukon');
      Application.Terminate;
      exit;
   end;
   RInfo.Reg.LastTime := NowTime;
   StoreRBOToRegistry(RInfo);
end;


{ TAppInstallInfo }

function TAppInstallInfo.GetDirs: TStringList;
begin
   if FDirs = nil then FDirs := TStringList.Create;
   result := FDirs;
end;

procedure TAppInstallInfo.SetDirs(const Value: TStringList);
begin
   if FDirs <> nil then
   begin
      Fdirs.Free;
      FDirs := nil;
   end;
   FDirs := Value;
end;


END.
