unit I_chlstr;

interface

uses
   Classes,
// KERNEL
   Elements,
   Tools,
// SPACE
   e_Geometry,
   e_GeometryElement,
  I_ChkTpl;

type

TCheckLineStringInterpret = class(TTestTplInterpret)
private
  FOutputBlockName: string;
  FErrorTxt: string;
  FErrorCount: integer;
    FBlockList: TStringList;
    FOutputDir: string;
    FDontSendError: boolean;
    FAreasEntity: TEntity;
    FLinesEntity: TEntity;
    FNewAreasString: TAreaString;
    FExportAreas: TAreaCollection;
    FExportLines: TPolyLineCollection;
    FActualTopolBlockName: string;

   procedure SetErrorTxt(const Value: string);
    procedure SetBlockList(const Value: TStringList);
    function GetBlockList: TStringList;
    procedure SetOutputDir(const Value: string);
    procedure SetDontSendError(const Value: boolean);
    procedure SetAreasEntity(const Value: TEntity);
    procedure SetLinesEntity(const Value: TEntity);
    procedure SetNewAreasString(const Value: TAreaString);
    procedure SetExportAreas(const Value: TAreaCollection);
    procedure SetExportLines(const Value: TPolyLineCollection);
    procedure SetActualTopolBlockName(const Value: string);
protected
   procedure Process(Element: TElement); override;
   procedure CloseWriting; override;
public
   procedure FreeRegister;
   procedure FindAreas;
   procedure CreateNewAreas;
   procedure CreateExportAreas;
   procedure CreateExportLines;
   procedure DoExport;
   property  BlockList: TStringList read GetBlockList write SetBlockList;
   property  OutputBlockName: string read FOutputBlockName write FOutputBlockName;
   property  LinesEntity: TEntity read FLinesEntity write SetLinesEntity;
   property  AreasEntity: TEntity read FAreasEntity write SetAreasEntity;
   property  NewAreasString: TAreaString read FNewAreasString write SetNewAreasString;
   property  ExportAreas: TAreaCollection read FExportAreas write SetExportAreas;
   property  ExportLines: TPolyLineCollection read FExportLines write SetExportLines;
   property  ErrorCount: integer read FErrorCount write FErrorCount;
   property  ActualTopolBlockName: string read FActualTopolBlockName write SetActualTopolBlockName;
published
   property OutputDir: string read FOutputDir write SetOutputDir;
   property ErrorTxt: string read FErrorTxt write SetErrorTxt;
   property DontSendError: boolean read FDontSendError write SetDontSendError;
end;

implementation
uses
   Dialogs,
   FileCtrl,
   Forms,
   SysUtils,
// KERNEL
   worlds,
   e_Class,
   e_Type,
// GLIB
   G_Elems,
   e_QTree,
// TOPOL
   //T_areare,
   e_INode,
   e_TopolElements,
   e_Reg,
   T_AreMak,
   T_ExArTL,
   T_ExLnTl,
   T_ExPnTl,
   T_LineExtractor,
   T_QTreeTools,
   T_Topol,
   T_ToTool,
   u_String;


procedure TCheckLineStringInterpret.Process(Element: TElement);
var
   E: Telement;
begin
   if not (Element is TPolyLineCollection) then exit;
   LinesEntity := TEntity(Element);
   if LinesEntity.NoElements then exit;
   //ErrorCount := 0;
   //if not DontSendError then
   //   Device.WritelnText('******************************************');

   //StartProgress(Self.Name,0,0);
   ProgressText := 'Ruen registru';
   Application.ProcessMessages;
   //FreeRegister;
   ProgressText := 'Hledn existujcch ploch';
   Application.ProcessMessages;
   FindAreas;
   if AreasEntity = nil then
   begin
      Device.WritelnText('Pozor! pro blok  '+ActualTopolBlockName);
      Device.WritelnText('  Nebyly nalezeny dn plochy, nen co porovnvat.');
      Device.WritelnText('  Bu opravdu neexistuj nebo je patn zadn datov model!');
      exit;
   end;
   ProgressText := 'Kontroln zaplochovn';
   MaxProgress := 0;
   Application.ProcessMessages;
   CreateNewAreas;
   ProgressText := 'Hledn chybjcch ploch...';
   Application.ProcessMessages;
   CreateExportAreas;
   ProgressText := 'Extrakce lini...';
   Application.ProcessMessages;
   CreateExportLines;
   ProgressText := 'Vlastn export...';
   Application.ProcessMessages;
   DoExport;
   ProgressText := 'Hotovo!';
   Application.ProcessMessages;
  (*
  MakeQTree(TPolyLineString(Element));
  if FQtree = nil then exit;
  MakeAreas;
  if FNewAreas = nil then exit;
  {
  F_GaugeForm:= TF_GaugeForm.Create(Application);
  F_GaugeForm.Caption:= self.Name;
  F_GaugeForm.MaxValue:= FNewAreas.Count;
  F_GaugeForm.Show;
  }
  for I:= FNewAreas.Count-1 downto 0 do
  begin
    Progress:= FNewAreas.Count-I;
    anArea:= TPolyLineAreaElement(FNewAreas.Elements[I]);
    if not (TElement(anArea) is TPolyLineAreaElement) then continue;
    CheckArea(anArea);
  end;

  //F_GaugeForm.Free;

  if (OutputDir <> '') then
  begin
     S := OutputDir;
     CheckBackSlash(S);
     OutputBlockName := S+'Err'+IntToStr(BlockList.Count)+'.blk';
     BlockList.Add(OutputBlockName);
     ExportAreas;
     Device.WritelnText('******************************************');
     Device.WritelnText('Nalezen oblasti uloeny v bloku: '+OutputBlockName);
  end
  else
  begin
     BlockList.Add(IntToStr(BlockList.Count));
  end;
  if not DontSendError then
  begin
     Device.WritelnText('******************************************');
  end;
  *)
  NewAreasString := nil;
  ExportLines := nil;
  ExportAreas := nil;
  //BlockList := nil;
  E := FindElementClass(TRectEntityQTree);
  if E <> nil then
     E.Free;
end;



procedure TCheckLineStringInterpret.SetErrorTxt(const Value: string);
begin
  FErrorTxt := Value;
end;



function TCheckLineStringInterpret.GetBlockList: TStringList;
begin
   if FBlockList = nil then FBlockList := TStringList.Create;
   result := FBlockList;
end;

procedure TCheckLineStringInterpret.SetBlockList(const Value: TStringList);
begin
   if FBlockList <> nil then
   begin
      FBlockList.Free;
      FBlockList := nil;
   end;
   FBlockList := Value;
end;

procedure TCheckLineStringInterpret.SetOutputDir(const Value: string);
begin
  FOutputDir := Value;
end;

procedure TCheckLineStringInterpret.SetDontSendError(const Value: boolean);
begin
  FDontSendError := Value;
end;

procedure TCheckLineStringInterpret.CloseWriting;
var
   S: string;
   i: longInt;
begin
   S := '   ';
   Device.TextColor := 0;
   Device.WritelnText(S);
   Device.WritelnText('******************************************');
   if ErrorCount > 0 then
      Device.TextColor := 255;
   Device.WritelnText(' Nstroj ' + Name + ' nalezl ');
   Device.WritelnText(IntToStr(ErrorCount)+' nezaplochovanch oblast');
   if OutputDir <> '' then
   begin
      Device.WritelnText(' Vsledek je uloen v blocch:');
      for i := 0 to BlockList.Count - 1 do
      begin
         Device.WritelnText(BlockList.Strings[i]);
      end;
   end;
   ErrorCount := 0;
   BlockList := nil;
   Device.WritelnText(' ');
   Device.TextColor := 0;
   Device.WritelnText('******************************************');
end;

procedure TCheckLineStringInterpret.FindAreas;
var
   i: integer;
   E: TElement;
   TL: TTopolLineElement;
begin
   MaxProgress := LinesEntity.LastIndex;
   AreasEntity := nil;
   for i := 0 to LinesEntity.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      E := LinesEntity.Elements[i];
      if not (E is TTopolLineElement) then continue;
      TL := TTopolLineElement(E);
      if i = 0 then
         ActualTopolBlockName := TL.BlockName;
      AreasEntity := TL.Geometry.FindClassInOwners(TAreaCollection);
      if AreasEntity <> nil then exit;
      //AC := TL.
   end;
end;

procedure TCheckLineStringInterpret.SetAreasEntity(const Value: TEntity);
begin
  FAreasEntity := Value;
end;

procedure TCheckLineStringInterpret.SetLinesEntity(const Value: TEntity);
begin
  FLinesEntity := Value;
end;

procedure TCheckLineStringInterpret.CreateNewAreas;
var
   MA: TAreasFromClearLines;
begin
   NewAreasString := nil;
   MA := TAreasFromClearLines.Create;
   MA.Input := LinesEntity;
   MA.DoIt;
   NewAreasString := TAreaString(MA.Output); // nov vytvoen Plochy
   //AddElement(MA.QTree);// uchovn topologie lini - jeste ne
   MA.Free;
end;

procedure TCheckLineStringInterpret.SetNewAreasString(const Value: TAreaString);
begin
   ExchangeFieldElement(TElement(FNewAreasString), Value);
end;

procedure TCheckLineStringInterpret.CreateExportAreas;
var
   Are: TPolyLineAreaElement;
   GAre: TTopolAreaElement;
   TExpAre: TTopolAreaElement;
   i: integer;
   e: TElement;
   OQTree: TRectEntityQTree;
   AddToQTree: TAddEntityToQTree;
   FindE: TFindRectEntityInQTreeByXY;

begin
   if NewAreasString = nil then exit;
   ExportAreas := TAreaCollection.Create;
   AddToQTree := TAddEntityToQTree.Create;
   AddToQTree.Input := AreasEntity;
   AddToQTree.DoIt;
   OQTree := TRectEntityQTree(AddToQTree.Output);
   AddElement(OQTree);
   AddToQTree.Free;
   FindE := TFindRectEntityInQTreeByXY.Create;
   FindE.Input := OQTree;
   for i := 0 to AreasEntity.LastIndex do
   begin
      e := AreasEntity.Elements[i];
      if not (e is TTopolAreaELement) then continue;
      TTopolAreaElement(e).Presnost := 0;
   end;
   if NewAreasString.LastIndex > 50 then
   begin
      MaxProgress := NewAreasString.LastIndex;
      Application.ProcessMessages;
   end;
   for i := 0 to NewAreasString.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      // Vytvoen ExportArea + pipojen star GPlochy
      Are := TPolyLineAreaElement(NewAreasString.Elements[i]);
      //!!!!!!!!!
      //if not Are.IsHitByXY(Are.X,Are.Y,0) then
      //   ShowMessage('popisovej bod v p.....!');
      FindE.X := Are.X;
      FindE.Y := Are.Y;
      FindE.DoIt;
      GAre := TTopolAreaElement(FindE.Output);
      if (GAre = nil) or (GAre.Presnost = 1) then // plocha nenalezena - dra, vyrbme neplochu
      begin
         TExpAre := TTopolExportAreaElement.Create;
         TExpAre.Geometry := Are;
         TExpAre.Druh := 1;
         TExpAre.Barva := 1;
         TExpAre.Znacka := 1;
         ExportAreas.AddAndUpdateRect(TExpAre);

         ErrorCount := ErrorCount + 1;
         if not DontSendError then
         begin
            if BeforeRow <> '' then
               Device.WriteText(BeforeRow);
            WriteInfo(Are.Elements[0].FindClassInOwners(TTopolLineElement));
            if ErrorTxt <> '' then
               Device.WritelnText(ErrorTxt);
            if AfterRow <> '' then
               Device.WriteLnText(AfterRow)
            { sem pijde zpis do protokolu }
         end;
      end
      else
         GAre.Presnost := 1;
   end;
   MaxProgress := 0;
   Application.ProcessMessages;
end;

procedure TCheckLineStringInterpret.CreateExportLines;
var
   E: TEntity;
   LE: TLineExtractor;
   i: integer;
   L: TPolyLineElement;
   TL: TTopolLineElement;
   El: TElement;
   MA: TAreasFromClearLines;
begin
   if ExportAreas = nil then exit;
   if ExportAreas.NoElements then exit;
   ExportLines := TPolyLineCollection.Create;
   LE := TLineExtractor.Create;
   LE.Input := ExportAreas;
   LE.NoDuplicates := True;
   LE.DoIt;
   E := TEntity(LE.Output);
   for i := 0 to E.LastIndex do
   begin
      El := E.Elements[i];
      if not(El is TPolyLineElement) then continue;
      L := TPolyLineElement(El);
      TL := TTopolLineElement.Create;
      TL.Geometry := L;
      TL.Druh := 1;
      TL.Barva := 13;
      TL.Znacka := 0;
      ExportLines.AddAndUpdateRect(TL);
   end;
   //LinesEntity := ExportLines;
   LE.Free;

   // a topologie ...
   MA := TAreasFromClearLines.Create;
   MA.Input := ExportLines;
   MA.DoIt;
   E := TAreaString(MA.Output); // nov vytvoen Plochy
   AddElement(MA.QTree);// uchovn topologie lini - uz
   E.Free;
   MA.Free;
end;

procedure TCheckLineStringInterpret.DoExport;
var
  ArExp: TExportAreasToTopol;
  S: string;
  E: TElement;
  En: TEntity;
  i: integer;
begin
   if ExportAreas = nil then exit;
   if ExportAreas.NoElements then exit;
   if (OutputDir = '') then exit;
   S := GetAbsolutePath(OutputDir);
   CheckBackSlash(S);
   E := World.ObjectModel.FindElementName('Work');
   if E = nil then exit;
   En := TEntity(E);
   E := En.FindElementClass(TInterpretNodeEntity);
   if E = nil then exit;
   S := S + E.Name; // Nzev kontroly
   CheckBackSlash(S);
   E := En.FindElementName('ActualNode');
   if E <> nil then
      S := S+TStringElement(E).Value // nazev uzlu
   else
      S := S+'NODE';
   ForceDirectories(S);
   CheckBackSlash(S);
   i := 0;
   repeat
      OutputBlockName := S+'Err'+IntToStr(i)+'.blk';
      Inc(i);
   until not DirectoryExists(OutputBlockName);
   BlockList.Add('    ' + 'pro blok '+ActualTopolBlockName+#13+#10+'    ' + 'v '+OutputBlockName);

   ArExp:= TExportAreasToTopol.Create;
   try
     ArExp.Input:= ExportAreas;
     ArExp.Lines := ExportLines;
     ArExp.Name:= 'Export ploch a tak do bloku ' + BlockName;
     ArExp.BlockName:= OutputBlockName;
     ArExp.ControlDir:= '';
     ArExp.RegisterObject := true;
     ArExp.ExportDBF:= false;
     ArExp.DoIt;
   finally
     ArExp.Free;
   end;
   if not DontSendError then
   begin
      Device.WritelnText('******************************************');
      Device.WritelnText('Nalezen oblasti uloeny v bloku: '+OutputBlockName);
      Device.WritelnText('******************************************');
   end;
end;

procedure TCheckLineStringInterpret.SetExportAreas(const Value: TAreaCollection);
begin
   ExchangeFieldElement(TElement(FExportAreas),Value);
end;

procedure TCheckLineStringInterpret.SetExportLines(const Value: TPolyLineCollection);
begin
   ExchangeFieldElement(TElement(FExportLines),Value);
end;

procedure TCheckLineStringInterpret.SetActualTopolBlockName(
  const Value: string);
begin
  FActualTopolBlockName := Value;
end;

procedure TCheckLineStringInterpret.FreeRegister;
var
   E: TElement;
begin
   E := World.ObjectModel.FindElementClass(TRegister);
   World.ObjectModel.FreeElement(E);
end;

INITIALIZATION
   RegisterOrbisClass(TCheckLineStringInterpret);
END.
