unit I_inode;

interface

uses
// KERNEL
   Elements,
// INTERPRET
   e_IScrpt,
   i_Uni,
   i_Basic,
   e_INode;

type

TInterpretNodeEntityInterpret = class(TInterpret)
private
   FActualNode: TInterpretNode;
   FWorkingArea: TOwnerEntity;
   FDataDirectory: string;
   FInterpretIModel: boolean;
    FActualDirectory: string;
   function  GetWorkingArea: TOwnerEntity;
   function  GetUI: TUniversalInterpret;
   function  GetDataModel: TElement;
   function  GetInterpretScript: TInterpretScript;
   function  CreateDataModel: boolean;
   function  CreateInterpretScript: boolean;
   procedure FreeLastDataModel;
   procedure FreeLastInterpretModel;
    procedure SetActualNode(const Value: TInterpretNode);
    procedure SetActualDirectory(const Value: string);
protected
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
   function  IsItsClass(AObject: TObject): boolean; override;
   property  DataModel: TElement read GetDataModel;
   property  InterpretScript: TInterpretScript read GetInterpretScript;
   property  ActualNode: TInterpretNode read FActualNode write SetActualNode;
   property  ActualDirectory: string read FActualDirectory write SetActualDirectory;
published
   property WorkingArea: TOwnerEntity read GetWorkingArea write FWorkingArea;
   property DataDirectory: string read FDataDirectory write FDataDirectory;
   property UI: TUniversalInterpret read GetUI stored false;
   property InterpretIModel: boolean read FInterpretIModel write FInterpretIModel;
end;


implementation
uses
   Classes,
   Forms,
   Dialogs,
   SysUtils,
// KERNEL
   e_Class,
   e_Type,
   Worlds,
   e_DynM,
   e_Reg,
   i_IModel,
   t_DirectoryHCheck,
   t_IMtoIS,
   t_Layscr,
   t_LMCre;

{ TInterpretNodeEntityInterpret }

function  TInterpretNodeEntityInterpret.GetDataModel: TElement;
begin
   result := WorkingArea.FindElementName('DATA');
end;

function  TInterpretNodeEntityInterpret.GetInterpretScript: TInterpretScript;
begin
   result := TInterpretScript(WorkingArea.FindElementClass(TInterpretScript));
end;

function  TInterpretNodeEntityInterpret.CreateDataModel: boolean;
var
   MSC: TModelScriptCreator;
   MD: TModelScriptToDynamicModel;
   MBSC: TModelByScriptCreator;
   SLE, DM: TStringListElement;
   i: longInt;
   S: TStringElement;
begin
   MSC  := TModelScriptCreator.Create;
   MBSC := TModelByScriptCreator.Create;
   DM := ActualNode.DataModel;
   SLE := TStringListElement.Create;
   for i := 0 to DM.Lastindex do
   begin
      S := DM.StringElements[i];
      SLE.AddNameString(S.Name,S.Value);
   end;
   MSC.DataDirectory := ActualDirectory;
   MSC.Input := SLE;
   MSC.DoIt;
   if ActualNode.DynamicModel then
   begin
      MD := TModelScriptToDynamicModel.Create;
      MD.Input := MSC.Output;
      MD.DoIt;
      MD.Output.Name := 'DATA';
      WorkingArea.AddElement(MD.Output);
      result := MD.Output <> nil;
      TDynamicModel(MD.Output).Active := true;
      MD.Free;
   end
   else
   begin
      MBSC.Input := MSC.Output;
      MBSC.DoIt;
      MBSC.Output.Name := 'DATA';
      WorkingArea.AddElement(MBSC.Output);
      result := MBSC.Output <> nil;
      MBSC.Free;
   end;
   MSC.Output.Free;
   MSC.Free;
end;

function  TInterpretNodeEntityInterpret.CreateInterpretScript: boolean;
var
   IMIS: TInterpretModelToIntrepretScript;
begin
   IMIS := TInterpretModelToIntrepretScript.Create;
   IMIS.Input := ActualNode.InterpretModel;
   IMIS.DoIt;
   WorkingArea.AddElement(IMIS.Output);
   result := IMIS.Output <> nil;
   IMIS.Free;
end;

procedure TInterpretNodeEntityInterpret.FreeLastDataModel;
var
   E: TElement;
begin
   try
      E := WorkingArea.FindElementName('DATA');
      if E is TDynamicModel then TDynamicModel(E).Active := false;
      WorkingArea.FreeElement(E);

      E := World.ObjectModel.FindElementClass(TRegister);
      World.ObjectModel.FreeElement(E);

   except
      ShowMessage('Chyba uvolnn dat nebo registru');
   end;
end;

procedure TInterpretNodeEntityInterpret.FreeLastInterpretModel;
var
   E: TElement;
begin
   try
   E := WorkingArea.FindElementClass(TInterpretScript);
   WorkingArea.FreeElement(E);
   except
      Device.WriteText(' problmy pi uvolnn InterpretModelu');
   end;
end;




function TInterpretNodeEntityInterpret.GetWorkingArea: TOwnerEntity;
var
   S: string;
begin
   if FWorkingArea = nil then
   begin
      S := 'WORK';
      FWorkingArea := TOwnerEntity(World.ObjectModel.FindElementName(S));
      if FWorkingArea = nil then
      begin
         FWorkingArea := TOwnerEntity.Create;
         FWorkingArea.Name := S;
         World.ObjectModel.AddElement(FWorkingArea);
      end;
   end;
   result := FWorkingArea;
end;


function TInterpretNodeEntityInterpret.GetUI: TUniversalInterpret;
var
   aUI: TUniversalInterpret;
begin
   aUI := TUniversalInterpret(WorkingArea.FindElementClass(TUniversalInterpret));
   if aUI = nil then
   begin
      aUI := TUniversalInterpret.Create;
      aUI.Name := 'UI';
      WorkingArea.AddElement(aUI);
   end;
   result := aUI;
end;


function  TInterpretNodeEntityInterpret.QGoInside(Element: TElement): boolean;
begin
   result := false;
end;

function  TInterpretNodeEntityInterpret.QProcessBefore(Element: TElement): boolean;
begin
   result := Element is TInterpretnodeEntity;
end;

procedure TInterpretNodeEntityInterpret.Process(Element: TElement);
var
   i,j: longInt;
   El: TElement;
   E: TInterpretNodeEntity;
   IIM: TInterpretIModel;
   SS: TstringListElement;
   DCN: TDirectoryHCheckNode;
   DCT: TDirectoryHCheckTool;
   US: string;
   SL: TStringList;

   procedure InterpretNodeName;
   begin
      Device.WritelnText('  ');
      Device.WritelnText('  Kontroln uzel: ' + ActualNode.Name);
      Device.WritelnText('******************************************');
   end;

   procedure InterpretActualDirectory;
   begin
      Device.TextColor := $909090;
      Application.ProcessMessages;
      Device.WritelnText('******************************************');
      Device.WritelnText('  Datov sloka: ' + ActualDirectory);
      Device.WritelnText('******************************************');
      Device.TextColor := 0;
      Application.ProcessMessages;
   end;

   procedure InterpretIM;
   begin
      IIM := TInterpretIModel.Create;
      IIM.Input := ActualNode.InterpretModel;
      //IIM.Header := ActualNode.Name;
      IIM.Device := Device;
      IIM.DoIt;
      IIM.Free;
   end;

   procedure InterpretDM;
   var
      i: longInt;
   begin
      SS := ActualNode.DataModel;
      //Device.WritelnText(' ');
      Device.WritelnText('Datov model:');
      Device.WritelnText('******************************************');
      for i := 0 to SS.LastIndex do
         Device.WritelnText(SS.Names[i]+ '=' + SS.Strings[i]);
      Device.WritelnText('******************************************');

   end;

begin
   SL := TStringList.Create;
   SL.Delimiter := ';';
   SL.DelimitedText := DataDirectory;
   Device.Reset;
   E := TInterpretNodeEntity(Element);
   StartProgress(E.Name+' '{+ActualDirectory},0,E.LastIndex);
   InterpretIModel := not E.DontInterpretModels;
   for i := 0 to E.LastIndex do
   begin
      Progress := i;
      El := E.Elements[i];
      for j := 0 to SL.Count - 1 do
      begin
         ActualDirectory := SL.Strings[j];
         InterpretActualDirectory;
         if not DirectoryExists(ActualDirectory) then
         begin
            Device.WritelnText('      Adres neexistuje !!!');
            continue;
         end;
         if El is TDirectoryHCheckNode then
         begin
            DCN := TDirectoryHCheckNode(El);
            DCT := TDirectoryHCheckTool.Create;
            DCT.Node := DCN;
            DCT.Directory := ActualDirectory;
            DCT.Device := Device;
            DCT.DoIt;
            DCT.Free;
            Continue;
         end;
         if not(El is TInterpretNode) then continue;
         ActualNode := TInterpretNode(El);

         FreeLastDataModel;
         FreeLastInterpretModel;
         US := 'Uzel .'+IntToStr(i+1)+'/'+IntToStr(E.Count)+': '+ActualNode.Name+' '+ActualDirectory;
         ProgressText := US+' -vytven interpret ...';
         Application.ProcessMessages;
         if not CreateInterpretScript then
         begin
            EndProgress;
            Application.ProcessMessages;
            Continue;
         end;
         InterpretNodeName;
         if (j = 0) and InterpretIModel then
         begin
            InterpretIM;
            InterpretDM;
         end;
         Device.TextColor := 255;
         ProgressText := US+' -natn dat ...';
         Application.ProcessMessages;
         if not CreateDataModel then
         begin
            EndProgress;
            Application.ProcessMessages;
            Continue;
         end;
         ProgressText := US+' -interpretace ...';
         Application.ProcessMessages;
         UI.Input := DataModel;
         UI.Script := InterpretScript;
         // test
         //World.ObjectModel.NextElement := InterpretScript;
         // test end
         UI.Device := Device;
         UI.DoIt;
         Device.TextColor := 0;
         ProgressText := US+' -uvolovn pamti...';
         Application.ProcessMessages;
         UI.Free;
         FreeLastDataModel;
         FreeLastInterpretModel;
         if BreakWork then break;
      end;
      if BreakWork then break;
      {
      UI.Device := nil;
      }
   end;
   EndProgress;
   Application.ProcessMessages;
   SL.Free;
   Device.Close;
end;
(*  cela kontrola po adresarich...
begin
   SL := TStringList.Create;
   SL.Delimiter := ';';
   SL.DelimitedText := DataDirectory;
   Device.Reset;
   E := TInterpretNodeEntity(Element);
   for j := 0 to SL.Count - 1 do
   begin
      ActualDirectory := SL.Strings[j];
      StartProgress(E.Name+' '+ActualDirectory,0,E.LastIndex);
      InterpretActualDirectory;
      if not DirectoryExists(ActualDirectory) then
      begin
         Device.WritelnText('      Adres neexistuje !!!');
         continue;
      end;
      InterpretIModel := not E.DontInterpretModels;
      for i := 0 to E.LastIndex do
      begin
         Progress := i;
         El := E.Elements[i];
         if El is TDirectoryHCheckNode then
         begin
            DCN := TDirectoryHCheckNode(El);
            DCT := TDirectoryHCheckTool.Create;
            DCT.Node := DCN;
            DCT.Directory := ActualDirectory;
            DCT.Device := Device;
            DCT.DoIt;
            DCT.Free;
            Continue;
         end;
         if not(El is TInterpretNode) then continue;
         ActualNode := TInterpretNode(El);

         FreeLastDataModel;
         FreeLastInterpretModel;
         US := 'Uzel .'+IntToStr(i+1)+'/'+IntToStr(E.Count)+': '+ActualNode.Name;
         ProgressText := US+' -vytven interpret ...';
         Application.ProcessMessages;
         if not CreateInterpretScript then
         begin
            EndProgress;
            Application.ProcessMessages;
            Continue;
         end;
         InterpretNodeName;
         if (j = 0) and InterpretIModel then
         begin
            InterpretIM;
            InterpretDM;
         end;
         Device.TextColor := 255;
         ProgressText := US+' -natn dat ...';
         Application.ProcessMessages;
         if not CreateDataModel then
         begin
            EndProgress;
            Application.ProcessMessages;
            Continue;
         end;
         ProgressText := US+' -interpretace ...';
         Application.ProcessMessages;
         UI.Input := DataModel;
         UI.Script := InterpretScript;
         // test
         //World.ObjectModel.NextElement := InterpretScript;
         // test end
         UI.Device := Device;
         UI.DoIt;
         Device.TextColor := 0;
         UI.Free;
         FreeLastDataModel;
         FreeLastInterpretModel;
         if BreakWork then break;
         {
         UI.Device := nil;
         }
      end;
      EndProgress;
      Application.ProcessMessages;
   end;
   SL.Free;
   Device.Close;
end;
*)

function  TInterpretNodeEntityInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := aObject is TInterpretNodeEntity;
end;


procedure TInterpretNodeEntityInterpret.SetActualNode(const Value: TInterpretNode);
var
   SE: TStringElement;
begin
   FActualNode := Value;
   SE := TStringElement(WorkingArea.FindOrCreateElementName(TStringElement,'ActualNode'));
   SE.Value := Value.Name;
end;

procedure TInterpretNodeEntityInterpret.SetActualDirectory(
  const Value: string);
begin
  FActualDirectory := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TInterpretNodeEntityInterpret);
END.
