unit t_CHeck_NEXT;

interface
uses
   t_Chk,
   t_ChkSp,
   e_Geometry,
   Elements;

type

TCheck_LIN_Approaching = class(TCheckSpaceEntity)
  private
    FTolerance: double;
    FComment: string;
    FEStr: string;
    FLines: TEntity;
    FOld: boolean;
    FEnds: boolean;
    FENdsNeighborhood: boolean;
    FErrLines: TEntity;
    FRefLines: TEntity;
    FTwoLayers: boolean;
    FSilentMode: boolean;
    procedure SetTolerance(const Value: double);
    procedure SetComment(const Value: string);
    procedure SetLines(const Value: TEntity);
    procedure SetOld(const Value: boolean);
    procedure SetEnds(const Value: boolean);
    procedure SetENdsNeighborhood(const Value: boolean);
    procedure SetErrLines(const Value: TEntity);
    procedure SetRefLines(const Value: TEntity);
    procedure SetTwoLayers(const Value: boolean);
    procedure SetSilentMode(const Value: boolean);
protected
   function GetErrorMsg: string; override;
public
   procedure DoIt; override;
   procedure DoIt_;
   procedure Process(Line1,Line2: TPolyLineElement);
published
   property Lines: TEntity read FLines write SetLines;
   property RefLines: TEntity read FRefLines write SetRefLines;   // ref linie pro TwoLayers
   property ErrLines: TEntity read FErrLines write SetErrLines;
   property Tolerance: double read FTolerance write SetTolerance;
   property Comment: string read FComment write SetComment;
   property TwoLayers: boolean read FTwoLayers write SetTwoLayers;    // 2 vrstvy RefLines povinna
   property SilentMode: boolean read FSilentMode write SetSilentMode; // bez hlasek
   property Old: boolean read FOld write SetOld;                      // puvodni priblizeni
   property Ends: boolean read FEnds write SetEnds;                   // vypisovat shodne konce
   property ENdsNeighborhood: boolean read FENdsNeighborhood write SetENdsNeighborhood; // vypisovat okoli shodnych koncu
end;

TCheck_LIN_NodeAngles = class(TCheckSpaceEntitySingleValue)
private
    FComment: string;
    FLines: TEntity;
    procedure SetComment(const Value: string);
    procedure SetLines(const Value: TEntity);
public
   procedure DoIt; override;
published
   property Lines: TEntity read FLines write SetLines;
   property Comment: string read FComment write SetComment;
end;

//TCheck_PNT_TXT_Overlay = class()

TCheck_ARE_Overlay = class(TCheckTool)
  private
    FAreas: TEntity;
    FDown: boolean;
    FNextErrText: string;
    procedure SetAreas(const Value: TEntity);
    procedure SetDown(const Value: boolean);
    procedure SetNextErrText(const Value: string);
protected
  function GetErrorMsg: string; override;
public
   procedure DoIt; override;
   property  NextErrText: string read FNextErrText write SetNextErrText;
published
   property Areas: TEntity read FAreas write SetAreas;
   property Down: boolean read FDown write SetDown;
end;

TCheck_PULSLU_Pairs = class(TCheckSpaceEntity)
private
    FComment: string;
    FS1,FS2: string;
    FPulsluGroup: integer;
    FPoints: TEntity;
    FCheckForest: boolean;
    FByProperties: boolean;
    FByPropertiesComment: string;
    FByPropertiesList: string;
    FByPropertiesLevel: TEntity;
    FByPropertiesMissingComment: string;
    procedure SetComment(const Value: string);
    procedure SetPulsluGroup(const Value: integer);
    procedure SetPoints(const Value: TEntity);
    procedure SetCheckForest(const Value: boolean);
    procedure SetByProperties(const Value: boolean);
    procedure SetByPropertiesComment(const Value: string);
    procedure SetByPropertiesLevel(const Value: TEntity);
    procedure SetByPropertiesList(const Value: string);
    procedure SetByPropertiesMissingComment(const Value: string);
protected
   function GetErrorMsg: string; override;
public
   procedure DoIt; override;
published
   property Points: TEntity read FPoints write SetPoints;
   property Comment: string read FComment write SetComment;
   property PulsluGroup: integer read FPulsluGroup write SetPulsluGroup;
   property CheckForest: boolean read FCheckForest write SetCheckForest;
   property ByProperties: boolean read FByProperties write SetByProperties;
   property ByPropertiesLevel: TEntity read FByPropertiesLevel write SetByPropertiesLevel;
   property ByPropertiesList: string read FByPropertiesList write SetByPropertiesList;
   property ByPropertiesComment: string read FByPropertiesComment write SetByPropertiesComment;
   property ByPropertiesMissingComment: string read FByPropertiesMissingComment write SetByPropertiesMissingComment;

end;

implementation
uses
   Classes,
   Dialogs,
   Forms,
   SysUtils,
   e_Class,
   e_GeometryElement,
   e_LHCModel,
   e_Node,
   e_QTree,
   e_TopolElements,
   G_Elems,
   m_Orbis,
   MathLib,
   t_FindBreaklineConnections,
   t_ForestModelSelector,
   t_QTreeTools,
   t_Snap, t_FrameSelect;

{ TCheck_LIN_Approaching }

procedure TCheck_LIN_Approaching.DoIt;
var
   TQ: TAddEntityToQTree;
   LevelQT: TRectEntityQTree;
   e: TElement;
   L,L1: TPolyLineElement;
   GE,GE1: TGeometryElement;
   W: TQTreeWalker;
   i: integer;
   QL: TQuickEntity;
begin
   if Old then
   begin
      DoIt_;
      exit;
   end;
   if (Lines = nil) or not (Lines is TEntity) or Lines.NoElements then
   begin
      if not SilentMode then
         ShowMessage('CheckLinApproaching - Lines !!!!');
      exit;
   end;
   if TwoLayers then
   if (RefLines = nil) or not (RefLines is TEntity) or RefLines.NoElements then
   begin
      if not SilentMode then
         ShowMessage('CheckLinApproaching - RefLines !!!!');
      exit;
   end;
   StartProgress(Comment,0,Lines.LastIndex);
   ProgressText := 'Pprava vrstev...';
   Application.ProcessMessages;
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'QT';
   TQ := TAddEntityToQTree.Create;
   TQ.NoProgress := NoProgress;
   if TwoLayers then
      TQ.Input := RefLines
   else
      TQ.Input := Lines;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
   W := TQTreeWalker.Create;
   QL := TQuickEntity.Create;
   ProgressText := 'hop up...';
   //MaxProgress := MasterLines.LastIndex;
   Progress := 0;
   Application.ProcessMessages;
   for i := 0 to Lines.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Lines.Elements[i];
      if not (e is TGeometryElement) then continue;
      GE := TGeometryElement(e);
      Input := GE;
      L := TPolyLineElement(GE.Geometry);
      if L = nil then continue;
      e := W.GetFirstElementForRectangle(LevelQT,L.MinX,L.MinY,L.MaxX,L.MaxY, Tolerance);
      while e <> nil do
      begin
         if e <> GE then
         begin
            if e is TGeometryElement then
            begin
               GE1 := TGeometryElement(e);
               L1 := TPolyLineElement(GE1.Geometry);
               if (L1 <> nil) and (QL.IndexOf(L1)< 0) then
               begin
                  QL.AddElement(L1);
                  Process(L,L1);
               end;
            end;
         end;
         e := W.GetNextElement;
      end;
      QL.Clear;
   end;
   QL.Free;
   W.Free;
   LevelQT.Free;
end;


procedure TCheck_LIN_Approaching.DoIt_;
var
   Snap: TPointLineSnapTool;
   TQ: TAddEntityToQTree;
   LevelQT: TRectEntityQTree;
   i,j: integer;
   //SnapCount: integer;
   e: TElement;
   L: TPolyLineElement;
   GE: TGeometryElement;
   AsiError: boolean;
   aX,aY,X0,Y0,Xl,Yl: double;
begin
   if (Lines = nil) or not (Lines is TEntity) or Lines.NoElements then
   begin
      ShowMessage('CheckLinApproaching - Input si d linie a ty nejsou !!!!');
      exit;
   end;
   StartProgress(Comment,0,Lines.LastIndex);

   ProgressText := 'Pprava vrstev...';
   Application.ProcessMessages;
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Lines;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;

   ProgressText := 'hop up...';
   //MaxProgress := MasterLines.LastIndex;
   Progress := 0;
   Application.ProcessMessages;
   Snap := TPointLineSnapTool.Create;
   Snap.FrameQT := LevelQT;
   Snap.SnapToLine := true;
   Snap.SnapToLinePoint := true;
   Snap.SnapToLineBody := true;
   Snap.SnapDistance := Tolerance;
   //SnapCount := 0;
   for i := 0 to Lines.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Lines.Elements[i];
      if not (e is TGeometryElement) then continue;
      GE := TGeometryElement(e);
      Input := GE;
      L := TPolyLineElement(GE.Geometry);
      if L = nil then continue;
      // zacatek linie
      X0 := L.Xi[0];
      Y0 := L.Yi[0];
      Xl := L.Xi[L.LastIndex];
      Yl := L.Yi[L.LastIndex];
      for j := 0 to L.LastIndex do
      begin
         aX := L.Xi[j];
         aY := L.Yi[j];
         Snap.X := aX;
         Snap.Y := aY;
         Snap.TabuElement := L;
         Snap.DoIt;
         AsiError := false;
         if Snap.Snapped  then  // nkam se to chytlo
         begin
            // souadnice se nepohnula a tak zjistme co se dje
            if (Snap.SnapX = aX) and (Snap.SnapY = aY) then
            begin
               if Snap.SnapStatus = 2 then // na tlo linie ... opravdu piblen
               begin
                  AsiError := true;
               end
               else
               begin
                  if (j = 0) or (j = L.LastIndex) then
                  begin
                     // prvn nebo posledn souadnice, ta by se chytit mla
                     //!!!kontrola zda s inkriminovanou linii nelezime na sobe
                  end
                  else
                     AsiError := true; // ostatni ne
               end;
            end
            else
            {
            if ((Snap.SnapX = X0) and (Snap.SnapY = Y0)) or
               ((Snap.SnapX = Xl) and (Snap.SnapY = Yl)) then
            begin
               //dolo k posunu k prvni nebo posledni souradnici
               if Snap.SnapStatus = 2 then
                  AsiError := true; // jinak body blizko sebe
            end
            else
            }
            if ((Snap.SnapX = X0) and (Snap.SnapY = Y0)) or
               ((Snap.SnapX = Xl) and (Snap.SnapY = Yl)) then
            begin
               //dolo k posunu k prvni nebo posledni souradnici, zkusme jet chytn na tlo
               Snap.SnapToLinePoint := false;
               Snap.DoIt;
               Snap.SnapToLinePoint := true;
               if Snap.Snapped then
               if Snap.SnapStatus = 2 then
                  AsiError := true; // jinak body blizko sebe
            end
            else
               AsiError := true;
         end;
         if AsiError then
         begin
            X:= aX;
            Y:= aY;
            //result:= false;
            Error:= GetError;
            AddErrorElement;
            if not FullCheck then
               break; // na dalsi linii
         end;
      end;
   end;
   Snap.Free;
   LevelQT.Free;
end;


function TCheck_LIN_Approaching.GetErrorMsg: string;
begin
   result := FEStr;
end;

procedure TCheck_LIN_Approaching.Process(Line1, Line2: TPolyLineElement);
var
   i,j,li,lj: integer;
   i11,i12,i21,i22: integer;
   aDistance: double;
   _X,_Y,X1,Y1,X2,Y2,X3,Y3,pX,pY: double;
   End1,End2,IsIn: boolean;
   U1,U2,Rov: boolean;
   SB,SB11,SB12,SB21,SB22: boolean;
   IsError: boolean;
begin
   IsError := false;
   SB := false;
   // 1) Body
   li := Line1.LastIndex;
   lj := Line2.LastIndex;
   i11 := 0;
   i12 := li;
   i21 := 0;
   i22 := lj;
   if not EndsNeighborhood then
   begin
      // nalezeni indexu pro pripad shodnych koncu
      _X := Line1.Xi[0];
      _Y := Line1.Yi[0];
      X1 := Line1.Xi[li];
      Y1 := Line1.Yi[li];    // koncove body
      X2 := Line2.Xi[0];
      Y2 := Line2.Yi[0];
      X3 := Line2.Xi[lj];
      Y3 := Line2.Yi[lj];
      SB11 := (_X = X2) and (_Y = Y2);  // same bod 1 1
      SB12 := (_X = X3) and (_Y = Y3);  // same bod 1 2
      SB21 := (X1 = X2) and (Y1 = Y2);  // same bod 2 1
      SB22 := (X1 = X3) and (Y1 = Y3);  // same bod 2 2
      // Linie1 nastaveni i11
      if SB11 or SB12 then
      begin
         SB := true;
         Inc(i11);  // preskoceni prvniho bodu
         for i := i11 to li do
         begin
            aDistance := Distance(_X,_Y,Line1.Xi[i],Line1.Yi[i]);
            if aDistance <= Tolerance then
               Inc(i11)       // je-li v toleranci, zvysime index
            else
               break;         // jinak koncime
         end;
      end;
      // Linie1 nastaveni i12
      if SB21 or SB22 then
      begin
         SB := true;
         Dec(i12);   // preskoceni posledniho bodu
         for i := i12 Downto 0 do
         begin
            aDistance := Distance(X1,Y1,Line1.Xi[i],Line1.Yi[i]);
            if aDistance <= Tolerance then
               Dec(i12)       // je-li v toleranci, snizime index
            else
               break;         // jinak koncime
         end;
      end;
      // Linie2 nastaveni i21
      if SB21 or SB11 then
      begin
         SB := true;
         Inc(i21);  // preskoceni prvniho bodu
         for i := i21 to lj do
         begin
            aDistance := Distance(X2,Y2,Line2.Xi[i],Line2.Yi[i]);
            if aDistance <= Tolerance then
               Inc(i21)       // je-li v toleranci, zvysime index
            else
               break;         // jinak koncime
         end;
      end;
      // Linie1 nastaveni i12
      if SB12 or SB22 then
      begin
         SB := true;
         Dec(i22);   // preskoceni posledniho bodu
         for i := i22 Downto 0 do
         begin
            aDistance := Distance(X3,Y3,Line2.Xi[i],Line2.Yi[i]);
            if aDistance <= Tolerance then
               Dec(i22)       // je-li v toleranci, snizime index
            else
               break;         // jinak koncime
         end;
      end;
   end;
   X1 := 0;
   Y1 := 0;
   X3 := 0;
   Y3 := 0;
   if i11 <= i12 then           // pokud je co kontrolovat
   for i := i11 to i12 do
   begin
      _X := Line1.Xi[i];
      _Y := Line1.Yi[i];
      End1 := (i = 0) or (i = li);
      lj := Line2.LastIndex;
      if i21 <= i22 then       // pokud je co kontrolovat u 2. linie
      for j := i21 to i22 do
      begin
         End2 := (j = 0) or (j = lj);
         X1 := Line2.Xi[j];
         Y1 := Line2.Yi[j];
         if (abs(X1-_X) <= Tolerance) and (abs(Y1-_Y) <= Tolerance) then
         begin
            aDistance := Distance(_X,_Y,X1,Y1);
            if (aDistance <= Tolerance) then
            begin
               if not Ends then
               if End1 and End2 and (aDistance = 0) then
               begin
                  SB := true;
                  Continue;                // konce zasadne nekontrolujem, proste sou nam fuk
               end;
               X:= _X;
               Y:= _Y;
               Error:= GetError;
               FEStr := 'BB '+FloatToStr(aDistance);
               AddErrorElement;
               IsError := true;
               if not FullCheck then
                  exit; // na dalsi linii
            end;
         end;
      end;
   end;
   // 2) body a usecky
   for i := 0 to li do
   begin
      _X := Line1.Xi[i];
      _Y := Line1.Yi[i];
      End1 := i = li;
      if not End1 then
      begin
         X1 := Line1.Xi[i+1];
         Y1 := Line1.Yi[i+1];
      end;
      lj := Line2.LastIndex;
      for j := 0 to lj do
      begin
         X2 := Line2.Xi[j];
         Y2 := Line2.Yi[j];
         End2 := j = lj;
         SB11 := (_X = X2) and (_Y = Y2);  // same bod 1 1
         SB12 := false;
         SB21 := false;
         if not End2 then      // bod L1 v us L2
         begin
            X3 := Line2.Xi[j+1];
            Y3 := Line2.Yi[j+1];
            SB12 := (_X = X3) and (_Y = Y3);  // same bod 1 2
            if not (SB11 or SB12) then
            if RectanglesOverlay(X2,Y2,X3,Y3,_X-1,_Y-1,_X+1,_Y+1,Tolerance) then
            begin
               DejVztahyBod_Usecka( _X, _Y, X2, Y2, X3, Y3, IsIn, pX, pY, aDistance);
               if IsIn and (aDistance <= Tolerance) then
               begin
                  X:= _X;
                  Y:= _Y;
                  Error:= GetError;
                  FEStr := 'BL '+FloatToStr(aDistance);
                  AddErrorElement;
                  IsError := true;
                  if not FullCheck then
                     exit; // na dalsi linii
               end;
            end;
         end;
         if not End1 then      // bod L2 v us L1
         begin
            SB21 := (X2 = X1) and (Y2 = Y1);
            if not (SB11 or SB21) then
            if RectanglesOverlay(_X,_Y,X1,Y1,X2-1,Y2-1,X2+1,Y2+1,Tolerance) then
            begin
               DejVztahyBod_Usecka( X2, Y2, _X, _Y, X1, Y1, IsIn, pX, pY, aDistance);
               if IsIn and (aDistance <= Tolerance) then
               begin
                  X:= pX;
                  Y:= pY;
                  Error:= GetError;
                  FEStr := 'LB '+FloatToStr(aDistance);
                  AddErrorElement;
                  IsError := true;
                  if not FullCheck then
                     exit; // na dalsi linii
               end;
            end;
         end;
         try   // tady to padalo u jedne rovnobezky po doplneni pX,pY:=0 to prestalo?
         if not (End1 or End2 or SB11 or SB12 or SB21) then  // usecky vzajemne mezi sebou
         begin
            SB22 := (X1 = X3) and (Y1 = Y3);
            if not SB22 then
            begin
               pX := 0;
               pY := 0;
               PrusecikUsecek(_X,_Y,X1,Y1,X2,Y2,X3,Y3,pX,pY,Rov,U1,U2);
               if U1 and U2 then
               begin
                  X:= pX;
                  Y:= pY;
                  Error:= GetError;
                  FEStr := 'LL';
                  AddErrorElement;
                  IsError := true;
                  if not FullCheck then
                     exit; // na dalsi linii
               end;
            end;
         end;
         except
            //X:= pX;
            //Y:= pY;
            X:= _X;
            Y:= _Y;
            Error:= GetError;
            FEStr := 'ERR';
            AddErrorElement;
            if ErrLines <> nil then
            begin
               ErrLines.AddElement(Line1);
               ErrLines.AddElement(Line2);
            end;
         end;
      end;
   end;
{ TODO 1 -oPPK -cOpravit : kontrola OVR je asi pitomost }
{
   if SB and not IsError then  // jsou shodne body ale nebyla chyba - kontrola prekryvu
   begin
      if Line2.IsHitByXY(Line1.X, Line1.Y, Tolerance) then
      begin
         X:= Line1.X;
         Y:= Line1.Y;
         Error:= GetError;
         FEStr := 'OVR';
         AddErrorElement;
      end;
   end;
}
end;

procedure TCheck_LIN_Approaching.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCheck_LIN_Approaching.SetEnds(const Value: boolean);
begin
  FEnds := Value;
end;

procedure TCheck_LIN_Approaching.SetENdsNeighborhood(const Value: boolean);
begin
  FENdsNeighborhood := Value;
end;

procedure TCheck_LIN_Approaching.SetErrLines(const Value: TEntity);
begin
  FErrLines := Value;
end;

procedure TCheck_LIN_Approaching.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;

procedure TCheck_LIN_Approaching.SetOld(const Value: boolean);
begin
  FOld := Value;
end;

procedure TCheck_LIN_Approaching.SetRefLines(const Value: TEntity);
begin
  FRefLines := Value;
end;

procedure TCheck_LIN_Approaching.SetSilentMode(const Value: boolean);
begin
  FSilentMode := Value;
end;

procedure TCheck_LIN_Approaching.SetTolerance(const Value: double);
begin
  FTolerance := Value;
end;

{ TCheck_ARE_Overlay }

{

Pekryvy ploch:
Parametry ... seznam
1) pokud GE nen Multiarea, e se jako dosud, teda trochu
2) Multiarea ...
   a) vyeit pekryv v rmci vlastn geometrie
   b) pekryvy s ostatnm svtem, mou nastat nsledujc situace:
       I) kompletn pekryv s jinou GE, stejn parametry
          (tohle by mla odstranit oprava hierarchi)
      II) kompletn pekryv s jinou GE, rzn parametry
     III) sten pekryv s jinou GE, tam je jedno, jak jsou parametry,
          p se to stejn bude muset eit run.
}


procedure TCheck_ARE_Overlay.DoIt;
var
   En: TEntity;
   E: TElement;
   GE,GE1,GErr: TGeometryElement;
   MA: TMultiAreaElement;
   A,A1: TAreaElement;
   G,G1: TGeometry;
   TQ: TAddEntityToQTree;
   LevelQT: TRectEntityQTree;
   W: TQTreeWalker;
   Err: TOwnerEntity;
   MAErr: TOwnerEntity; // vnitrni prekryvy geometrie
   i,j,k: integer;
   DCount: integer;
   BErr: boolean;
   S: string;
begin
   if (Areas = nil) or not(TElement(Areas) is TEntity) then
   begin
      //if not WithoutPrompt then
      PostTextmsg(tm_AddToProtocol,Name + 'Input nen nastaven nebo nen entitou',nil);
      exit;
   end;
   En := Areas;
   if En.NoElements  then
   begin
      exit;
   end;
   E := En.Elements[0];
   GE := nil;
   G := nil;
   if E is TGeometryElement then
   begin
      GE := TGeometryElement(E);
      G := GE.Geometry;
   end;
   if (GE = nil) or (G = nil) or not (G is TAreaElement)then
   begin
      //if not WithoutPrompt then
      PostTextmsg(tm_AddToProtocol,Name + 'Vrstva neobsahuje plochy',nil);
      exit;
   end;
   StartProgress('Kontrola duplicitnch ploch' ,0,En.LastIndex);
   ProgressText := 'pprava ...';
   Application.ProcessMessages;
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   AddElement(LevelQT);
   TQ := TAddEntityToQTree.Create;
   TQ.Input := En;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
   Err := TOwnerEntity.Create;
   Err.Name := 'ERR';
   AddElement(Err);
   MAErr := TOwnerEntity.Create;
   MAErr.Name := 'Internal Err';
   AddElement(MAErr);
   W := TQTreeWalker.Create;
   DCount := 0;
   GErr := TGeometryElement.Create;
   Input := GErr;
   AddElement(GErr);
   if Down then
      i := En.LastIndex
   else
      i := 0;
   while (i >= 0) AND (i <= EN.LastIndex) do
   begin
      Progress := i;
      ProgressText := 'Duplicity: '+IntToStr(DCount);
      Application.ProcessMessages;
      e := En.Elements[i];
      if not e.IsMyOwner(Err) and (e is TGeometryElement) then
      begin
         GE := TGeometryElement(e);
         G := GE.Geometry;
         if G is TMultiAreaElement then  // prekryvy vlastni geometrie
         begin
            MA := TMultiAreaElement(G);
            MAErr.FreeAll;
            GErr.Geometry := nil;
            if MA.Count > 1 then
            for j := 0 to MA.LastIndex - 1 do // do predposledni plochy
            begin
               A := MA.AreaElement[j];
               BErr := false;
               if A.IsMyOwner(MAErr) then continue; // uz je prekryv
               for k := j + 1 to MA.LastIndex do
               begin
                  A1 := MA.AreaElement[k];
                  if A.IsHitByXY(A1.X,A1.Y,0) then
                  begin
                     MAErr.AddElement(A1);
                     BErr := true;
                     // kontrola velikosti ploch !!!?!?....
                     // tady nekde bude ruseni duplicitnich geometrii
                  end;
               end;
               if BErr then
               begin
                  GErr.Geometry := A;
                  GErr.Name := GE.Name;
                  NextErrText := 'intern pekryv!';
                  AddErrorElement;
                  Inc(DCount);
               end;
            end;
         end;
         // kontrola externich prekryvu
         S := 'extern pekryv!';
         //NextErrText :=
         E := W.GetFirstElementForRectangle(LevelQT,GE.Minx,GE.MinY,GE.MaxX,GE.MaxY,1);
         while E <> nil do
         begin
            GE1 := TGeometryElement(E);
            if not GE1.IsMyOwner(Err) AND (GE <> GE1) then
            begin
               G1 := GE1.Geometry;
               if not (G1 is TMultiAreaElement) then
               begin
                  if G.IsHitByXY(G1.X,G1.Y,0) then
                  begin
                     Err.AddElement(GE1);
                     GErr.Geometry := G1;
                     GErr.Name := GE1.Name;
                     NextErrText := ' pln '+ S;
                     AddErrorElement;
                     Inc(DCount);
                  end;
               end
               else
               begin // multiarea
                  MA := TMultiAreaElement(G1);
                  bErr := true; // zatim kompletni prekryv
                  MAErr.FreeAll;
                  for j := 0 to MA.LastIndex do
                  begin
                     A := MA.AreaElement[j];
                     if G.IsHitByXY(A.X,A.Y,0) then
                        MAErr.AddElement(A)
                     else
                        bErr := false; // neuplny prekryv
                  end;
                  if bErr then
                     NextErrText := ' pln '+ S
                  else
                     NextErrText := ' sten '+ S;
                  GErr.Name := GE1.Name;
                  if not MAErr.NoElements then
                  begin
                     for j := 0 to MAErr.LastIndex do
                     begin
                        GErr.Geometry := TGeometry(MAErr.Elements[j]);
                        AddErrorElement;
                        Inc(DCount);
                     end;
                     Err.AddElement(GE1);
                  end;
               end;
            end;
            E := W.GetNextElement;
         end;
      end;
      if Down then
         Dec(i)
      else
         Inc(i);
   end;
   W.Free;
   Progress := 0;
   MaxProgress := Err.LastIndex;
   ProgressText := 'Hledn duplicit: '+IntToStr(DCount);
   Application.ProcessMessages;
   for i := Err.LastIndex downto 0 do
   begin
      Progress := i;
      Application.ProcessMessages;
      //e := Err.Elements[i];
      //Input := e;
      //AddErrorElement;
      //e.Free;
   end;
   Err.Free;
   LevelQT.Free;
   //if not WithoutPrompt then
   //   ShowMessage(Input.Name+' Bylo odstranno '+IntToStr(DCount)+' Duplicit(asi)');
end;

function TCheck_ARE_Overlay.GetErrorMsg: string;
begin
   result := Input.Name + ' ' + NextErrText;
end;

procedure TCheck_ARE_Overlay.SetAreas(const Value: TEntity);
begin
  FAreas := Value;
end;

procedure TCheck_ARE_Overlay.SetDown(const Value: boolean);
begin
  FDown := Value;
end;

procedure TCheck_ARE_Overlay.SetNextErrText(const Value: string);
begin
  FNextErrText := Value;
end;

procedure TCheck_LIN_Approaching.SetTwoLayers(const Value: boolean);
begin
  FTwoLayers := Value;
end;

{ TCheck_PULSLU_Pairs }

procedure TCheck_PULSLU_Pairs.DoIt;
var
   En: TEntity;
   TP,TP1,TP2: TTopolPointElement;
   P1,P2: TPointElement;
   i,j: integer;
   e,e1,e2: TElement;
   ErrC: integer;
   FMS: TForestModelSelector;
   A,B: TDoubleCoord;
   PSK1,PSK2: Tgis_ForestUnit;
   LevelQT: TRectEntityQTree;
   QTF: TFindRectEntityInQTreeByXY;
   TQ: TAddEntityToQTree;
   SL: TStringList;
   S: string;
   A1,A2: double;
   U1,U2,PI2: double;

   function CheckAngle: boolean;
   var
      D,D1,D2,Len: double;
   begin
      result := false;
      A1 := 0;
      A2 := 0;
      if P1 <> nil then
         A1 := Angle22PI(P1.Angle);
      if P2 <> nil then
         A2 := Angle22PI(P2.Angle);
      D := Abs(A2 - A1);
      if (D > U1) and (D < U2) then
      begin
         D1 := Angle22PI(A1+PI2);
         A.X := P1.X;
         A.Y := P1.Y;
         B.X := P2.X;
         B.Y := P2.Y;
         XYToPolar(A,B,Len,D2);
         D := Abs(D1 - D2);
         if (D > U1) and (D < U2) then
            result := true;
      end;
   end;
begin
   QTF := nil;
   SL := nil;
   LevelQT := nil;
   U1 := 0.75*PI;
   U2 := 1.25*PI;
   PI2:= PI/2;
   if Points = nil then exit;
   if not(TElement(Points) is TEntity) then exit;
   En := Points;
   if En.NoElements then exit;
   ErrC := 0;
   StartProgress(Comment,0,En.LastIndex);
   FMS := nil;
   if CheckForest then
   begin
      FMS := TForestModelSelector.Create;
      FMS.FindInput;
      AddElement(FMS);
      FMS.FrameClass := Tgis_PorostniSkupina;
      FMS.ReferenceFrameClass := Tgis_PorostniSkupina;
   end
   else
   if ByProperties then
   begin
      if (ByPropertiesLevel = nil) or (ByPropertiesLevel.NoElements) then
         ByProperties := false;
      if ByProperties then
      begin
         if ByPropertiesList <> '' then
         begin
            SL := TStringList.Create;
            SL.CommaText := ByPropertiesList;
         end
         else
            ByProperties := false;
      end;
      if ByProperties then
      begin
         LevelQT :=TRectEntityQTree.Create;
         LevelQT.Name := 'BPQT';
         AddElement(LevelQT);
         TQ := TAddEntityToQTree.Create;
         TQ.Input := ByPropertiesLevel;
         TQ.Output := LevelQT;
         TQ.DoIt;
         TQ.Free;
         QTF := TFindRectEntityInQTreeByXY.Create;
         QTF.Input := LevelQT;
      end;
   end;
   //TP := nil;
   TP1 := nil;
   //TP2 := nil;
   for i := 0 to En.LastIndex do
   begin
      Progress := i;
      ProgressText := 'Chyb: '+IntToStr(ErrC);
      Application.ProcessMessages;
      e := En.Elements[i];
      if not (e is TTopolPointElement) then continue;
      TP := TTopolPointElement(e);
      if TP.Druh <> PulsluGroup then // bod jineho druhu
      begin
         continue;
      end;
      // prvni bod paru
      if TP1 = nil then
      begin
         if TP.Presnost <> 10 then // melo by byt 10
         begin
            Input := TP;
            if TP.Presnost = 11 then
               Error := 2
            else
               Error := 1;
            AddErrorElement;
            Inc(ErrC);
            Continue;
         end;
         TP1 := TP;
      end
      else
      begin
         if TP.Presnost = 10 then // mel by byt TP1
         begin
            Input := TP1;
            Error := 2;
            AddErrorElement;
            Inc(ErrC);
            TP1 := TP;
            Continue;
         end;
         if TP.Presnost = 11 then // presnost souhlasi
         begin
            TP2 := TP;
            // kontrola uhlu
            P1 := TPointElement(TP1.Geometry);
            P2 := TPointElement(TP2.Geometry);
            if (P1 = nil) or (P2 = nil) or not CheckAngle then //chyba uhlu
            begin
               Input := TP1;
               Error := 3;
               AddErrorElement;
               Input := TP2;
               AddErrorElement;
               Inc(ErrC);
            end;
            if CheckForest then
            begin
               B.X := P1.X;
               B.Y := P1.Y;
               VratPosunutyBod(B,P1.Angle + PI/2,2,B);
               FMS.X := B.X;
               FMS.Y := B.Y;
               FMS.DoIt;
               PSK1 := FMS.ActualFrame;
               B.X := P2.X;
               B.Y := P2.Y;
               VratPosunutyBod(B,P2.Angle + PI/2,2,B);
               FMS.X := B.X;
               FMS.Y := B.Y;
               FMS.DoIt;
               PSK2 := FMS.ActualFrame;
               if (PSK1 = nil) or (PSK2=nil) or (PSK1 <> PSK2) then
               begin
                  Input := TP1;
                  Error := 4;
                  AddErrorElement;
                  Input := TP2;
                  AddErrorElement;
                  Inc(ErrC);
               end;
            end
            else
            if ByProperties then
            begin
               B.X := P1.X;
               B.Y := P1.Y;
               VratPosunutyBod(B,P1.Angle + PI/2,2,B);
               QTF.X := B.X;
               QTF.Y := B.Y;
               QTF.DoIt;
               e1 := QTF.Output;
               FS1 := '';
               if e1 <> nil then
               begin
                  for j := 0 to SL.Count - 1 do
                  begin
                     S := SL.Strings[j];
                     FS1 := FS1 + e1.GetPropertyStringByName(S) + ' ';
                  end;
               end
               else
                  FS1 := ByPropertiesMissingComment;
               B.X := P2.X;
               B.Y := P2.Y;
               VratPosunutyBod(B,P2.Angle + PI/2,2,B);
               QTF.X := B.X;
               QTF.Y := B.Y;
               QTF.DoIt;
               e2 := QTF.Output;
               FS2 := '';
               if e2 <> nil then
               begin
                  for j := 0 to SL.Count - 1 do
                  begin
                     S := SL.Strings[j];
                     FS2 := FS2 + e2.GetPropertyStringByName(S) + ' ';
                  end;
               end
               else
                  FS2 := ByPropertiesMissingComment;
               if (e1 = nil) or (e2=nil) or (not SameText(FS1,FS2)) then
               begin
                  Input := TP1;
                  Error := 4;
                  AddErrorElement;
                  Input := TP2;
                  S := FS1;
                  FS1 := FS2;
                  FS2 := S;
                  AddErrorElement;
                  Inc(ErrC);
               end;
            end;
            TP1 := nil;
            //TP2 := nil; // ale jinak jedeme od zacatku
         end
         else
         begin   // chybna presnost
            Input := TP;
            Error := 1;
            AddErrorElement;
            Inc(ErrC);
            Continue;
         end;
      end;
   end;
   if TP1 <> nil then
   begin
      Input := TP1;
      Error := 2;
      AddErrorElement;
      //Inc(ErrC);
   end;
   if ByProperties then
   begin
      SL.Free;
      QTF.Free;
      LevelQT.Free;
   end;
end;

function TCheck_PULSLU_Pairs.GetErrorMsg: string;
var
   S: string;
begin
   S := '';
   result := '?';
   if Input <> nil then
      S := Input.GetPropertyStringByName('Presnost');
   case Error of
      1: result := 'Presnost: '+ S;
      2: result := 'Lich: '+S;
      3: result := 'Uhel: '+S;
      4:
      if CheckForest then
      begin
         result := 'Rzn PSK: '+S;
      end
      else    // ByProperties
      begin
         result := ByPropertiesComment +' '+S+':'+FS1+':'+FS2;
      end;
   end;
end;

procedure TCheck_PULSLU_Pairs.SetByProperties(const Value: boolean);
begin
  FByProperties := Value;
end;

procedure TCheck_PULSLU_Pairs.SetByPropertiesComment(const Value: string);
begin
  FByPropertiesComment := Value;
end;

procedure TCheck_PULSLU_Pairs.SetByPropertiesLevel(const Value: TEntity);
begin
  FByPropertiesLevel := Value;
end;

procedure TCheck_PULSLU_Pairs.SetByPropertiesList(const Value: string);
begin
  FByPropertiesList := Value;
end;

procedure TCheck_PULSLU_Pairs.SetByPropertiesMissingComment(
  const Value: string);
begin
  FByPropertiesMissingComment := Value;
end;

procedure TCheck_PULSLU_Pairs.SetCheckForest(const Value: boolean);
begin
  FCheckForest := Value;
end;

procedure TCheck_PULSLU_Pairs.SetComment(const Value: string);
begin
  FComment := Value;
end;


procedure TCheck_PULSLU_Pairs.SetPoints(const Value: TEntity);
begin
  FPoints := Value;
end;

procedure TCheck_PULSLU_Pairs.SetPulsluGroup(const Value: integer);
begin
  FPulsluGroup := Value;
end;

{ TCheck_LIN_NodeAngles }

procedure TCheck_LIN_NodeAngles.DoIt;
var
  FB: TFindBreakLineConnections;
  En: TEntity;
  i,j: integer;
  N: TNode;
  NB,NB1,FirstB: TNodeBranch;
  e: TElement;
  A,B,C: TDoubleCoord;
  _Angle: double;
  GE: TGeometryElement;
begin
   if (Lines = nil) or not (Lines is TEntity) or Lines.NoElements then
   begin
      ShowMessage(Name+'- Input si d linie a ty nejsou !!!!');
      exit;
   end;
   if MinValue = 0 then
      MinValue := 0.1;
   StartProgress(Comment,0,0);
   ProgressText := 'Hledn uzl ...';
   Application.ProcessMessages;
   FB := TFindBreakLineConnections.Create;
   FB.AllNodes := true;
   FB.Input := Lines;
   FB.DoIt;
   En := TEntity(FB.Output);
   AddElement(En);
   FB.Free;
   GE := TGeometryElement.Create;
   AddElement(GE);
   Input := GE;
   MaxProgress := En.LastIndex;
   ProgressText := 'kontrola hl ...';
   Application.ProcessMessages;
   for i := 0 to En.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := En.Elements[i];
      if not (e is TNode) then exit;
      N := TNode(e);
      if N.BranchCount < 2 then continue;
      NB := nil;
      NB1 := nil;
      FirstB := nil;
      for j := 0 to N.Count do
      begin
         if j <= N.LastIndex then
         begin
            e := N.Elements[j];
            if not (e is TNodeBranch) then continue;
            if NB = nil then
            begin
               NB := TNodeBranch(e); // prvni branch
               FirstB := NB;
               continue;
            end;
            NB1 := TNodeBranch(e);
         end
         else
         begin
            NB1 := FirstB;
         end;
         B.X := N.X;
         B.Y := N.Y;
         NB.Line.GetOrientedCoord(1, NB.LineBegin, A.X, A.Y);
         NB1.Line.GetOrientedCoord(1, NB1.LineBegin, C.X, C.Y);
         _Angle := Uhel2Usecek(A,B,C);
         if _Angle < MinValue then
         begin
            X := N.X;
            Y := N.Y;
            Value := _Angle;
            GE.Geometry := NB.Line;
            AddErrorElement;
         end;
         NB := NB1;
      end;
   end;
end;

procedure TCheck_LIN_NodeAngles.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCheck_LIN_NodeAngles.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;


INITIALIZATION
   RegisterOrbisClass(TCHeck_LIN_Approaching);
   RegisterOrbisClass(TCHeck_LIN_NodeAngles);
   RegisterOrbisClass(TCheck_ARE_Overlay);
   RegisterOrbisClass(TCheck_PULSLU_Pairs);
END.

