unit C_ShpDBF;

interface
uses
  DB, DBTables, Classes,
  e_DataSet,
  Elements,
  E_Props,
  T_TlDBF;

type
  TShapeDBFTable = class(TDataSetElement)
  private
    FRecNo: longInt;
    FLastRecNo: longInt;
    FActualRecNo: integer;
    function SetRecordForRecNo(aRecNo: longInt): boolean;
    procedure SetActualRecNo(const Value: integer);
  public
    constructor Create(AOwner: TComponent);
    function FieldExists(aName: string): boolean;
    function SetForName(aName: string): boolean;
    function SetForNameRecNo(aName: string; aRecNo: longInt): boolean;
    procedure SetFieldAsString(aName: string; aValue: string);
    function GetFieldAsString(aName: string): string;
    procedure FieldAsStringAndInteger
              (aName: string;
               var StrResult: string; var IntResult: longInt; var IntOK: boolean);
    function IntegerFieldAsFormatedString(aName: string; FormatString: string): string;
    property ActualRecNo: integer read FActualRecNo write SetActualRecNo;
  end;





implementation
uses
  SysUtils, Dialogs, DBIProcs, Controls,
  CreaDBF, U_String,
  DBF,
  E_Type,
  E_TplReg,
  u_Global,
  U_Props;


constructor TShapeDBFTable.Create(AOwner: TComponent);
begin
  inherited Create (aOwner);
  ActualRecNo := -1;
end;



function TShapeDBFTable.FieldExists(aName: string): boolean;
var
  aField: TField;
  I: integer;
begin
  result:= false;
  aName:= UpperCase(aName);
  for I:= 0 to Table.FieldCount-1 do
  begin
    aField:= Table.Fields[I];
    if aField.FieldName = aName then
    begin
      result:= true;
      break
    end;
  end;
end;




procedure TShapeDBFTable.SetFieldAsString(aName: string; aValue: string);
var
  F: TField;
begin
  if not FieldExists(aName) then exit;
  try
    F:= Table.FieldByName(aName);
    if F<>nil then
       F.AsString:= aValue
  except
  end
end;




function TShapeDBFTable.GetFieldAsString(aName: string): string;
var
  F: TField;
begin
  result:= '';
  if not FieldExists(aName) then exit;
  try
    F:= Table.FieldByName(aName);
    if F<>nil then Result:= F.AsString
    else Result:= ''
  except
  end
end;




procedure TShapeDBFTable.FieldAsStringAndInteger
          (aName: string;
           var StrResult: string; var IntResult: longInt; var IntOK: boolean);
var
  F: TField;
  code: integer;
  Str: string;
begin
  IntOK:= false;
  IntResult:= 0;

  F:= Table.FieldByName(aName);
  if F<>nil then
  begin
    StrResult:= F.AsString;
    Str:= BezVnejsichMezer(StrResult);
    val(Str, IntResult, code);
    if code = 0 then
       IntOK:= true
    else
      IntResult:= 0
  end
  else StrResult:= ''
end;



function TShapeDBFTable.IntegerFieldAsFormatedString
         (aName: string; FormatString: string): string;
var
  F: TField;
begin
  F:= Table.FieldByName(aName);
  if F<>nil then Result:= Format(FormatString, [F.AsInteger])
  else Result:= ''
end;


function TShapeDBFTable.SetRecordForRecNo(aRecNo: longInt): boolean;
var
  Vysledek: longint;
  aDIf: integer;
begin
{  MessageDlg('CISLO ' + IntToStr(aCislo), mtInformation, [mbOK], 0); }

  Result:= false;
  if aRecNo = ActualRecNo then
  begin
     result := true;
     exit;
  end;
  Table.Active:= true;


  if Table.RecordCount < 1 then exit;

  if (aRecNo >= 0) and (aRecNo <= Table.RecordCount) then
  begin
     if ActualRecno < 0 then
     begin
        Table.First;
        ActualRecno := 1;
     end;
    //Active:= true;
    aDif := aRecno - ActualRecNo;
    Vysledek := Table.MoveBy(aDif);
    //Vysledek:= DBISetToRecordNo(Handle, aRecNo);

    if  Vysledek <> aDif then
    begin
{
      MessageDlg('Chyba nastaveni pro objekt c.' + IntToStr(aCislo)+': ' + IntToStr(Vysledek), mtWarning, [mbOK], 0);
}
      ActualRecNo := -1;
      Check(Vysledek);
    end
    else
    begin
      Table.Resync([]);
      ActualRecNo := aRecNo;
      Result:= true
    end;
  end
  else
  begin
    aRecNo:= aRecNo
  end;

  //FLastRecNo:= aRecNo
end;

function TShapeDBFTable.SetForName(aName: string): boolean;
var
   res: boolean;
begin
  Result:= false;
  if No_BDE then
     res := aName <> TDBF(Table).TableName
  else
     res := aName <> TTable(Table).TableName;
  if res then
  begin
    Table.Active:= false;
    ActualRecNo := -1;
    if NoBDE then
    begin
       TDBF(Table).readOnly:= true;
       TDBF(Table).TableName:= aName;
    end
    else
    begin
       TTable(Table).readOnly:= true;
       TTable(Table).TableName:= aName;
    end;
    if FileExists(aName) then
    begin
      result:= true;
      Table.Active:= true;
    end
    else
      Table.FieldDefs.Clear;
  end
  else
  begin
    Result:= true;
    Table.Active:= true;
  end
end;


function TShapeDBFTable.SetForNameRecNo(aName: string; aRecNo: longInt): boolean;
begin
  Result:= false;

  if aName <> TableName then
  begin
    Table.Active:= false;
    ActualRecNo := -1;
    readOnly:= true;
    TableName:= aName;

    if FileExists(TableName) then
    begin
      FRecNo:= aRecNo;
      Table.Active:= true;
      Result:= SetRecordForRecNo(FRecNo);
    end
    else
      Table.FieldDefs.Clear
  end
  else
  begin
    FRecNo:= aRecNo;
    Table.Active:= true;
    Result:= SetRecordForRecNo(FRecNo);
  end
end;

procedure TShapeDBFTable.SetActualRecNo(const Value: integer);
begin
  FActualRecNo := Value;
end;

end.
