unit t_ShpExport;

interface
uses
   Classes,
   C_SHP,
   C_SHPDBF,
   e_SHPFileElement,
   e_Type,
   Elements,
   Tools;

type

TExportSHPFile = class(TTool)
  private
    FFileName: string;
    FLayer: TEntity;
    FComment: string;
    FShapeType: integer;
    FExportLayer: TEntity;
    FDBFDefinition: TEntity;
    FTransferModel: TStringListElement;
    FExportDBF: boolean;
    FCRTFileName: string;
    procedure SetFileName(const Value: string);
    procedure SetLayer(const Value: TEntity);
    procedure SetComment(const Value: string);
    procedure SetExportLayer(const Value: TEntity);
    procedure SetShapeType(const Value: integer);
    function GetExportLayer: TEntity;
    function GetDBDefinition: TEntity;
    function GetTransferModel: TStringListElement;
    procedure SetNextFieldDef(const Value: string);
    procedure SetExportDBF(const Value: boolean);
    procedure SetCRTFileName(const Value: string);
protected
   IndexFileName: string;
   DBFName: string;
   FileHeader: TSHPFileHeader;
   RecHeader: TShpFileRecHeader;
   ArcRec: TShpArcRec;
   PointRec: TShpPointRec;
   IndexRec: TShpIndexRec;
   Stream: TFileStream;
   Streamx: TFileStream;
   DBFTable: TShapeDBFTable;
public
   procedure DoIt; override;
   procedure FindShapeType;
   procedure CreateExportElements;
   procedure CreateDBF;
   procedure CloseDBF;
   procedure ExportToFile;
   procedure ExportLine(aSHPE: TSHPExportElement);
   procedure ExportArea(aSHPE: TSHPExportElement);
   procedure ExportPoint(aSHPE: TSHPExportElement);
   procedure ExportDBFRec(aSHPE: TSHPExportElement);
   property  ExportLayer: TEntity read GetExportLayer write SetExportLayer;
   property  ShapeType: integer read FShapeType write SetShapeType;
   property  DBFDefinition: TEntity read GetDBDefinition;
published
   property FileName: string read FFileName write SetFileName;
   property Layer: TEntity read FLayer write SetLayer;
   property Comment: string read FComment write SetComment;
   property TransferModel: TStringListElement read GetTransferModel;
   property NextFieldDef: string write SetNextFieldDef;
   property ExportDBF: boolean read FExportDBF write SetExportDBF;
   property CRTFileName: string read FCRTFileName write SetCRTFileName;
end;

implementation
uses
   Forms,
   Dialogs,
   SysUtils,
   CreaDBF,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   G_elems,
   u_String;

{ TExportSHPFile }

procedure TExportSHPFile.CloseDBF;
begin
   if DBFTable = nil then exit;
   DBFTable.Table.Close;
   DBFTable.Free;
   DBFTable := nil;
end;

procedure TExportSHPFile.CreateDBF;
var
   S: string;
   OK: boolean;
begin
   DBFTable := nil;
   if not ExportDBF then exit;
   if CRTFileName <> '' then
   begin
      S := GetAbsolutePath(CRTFileName);
      VytvorDBFDle(S, DBFName, OK);
      if not OK then exit;
   end
   else
   begin
      if DBFDefinition.NoElements then
      begin
         ShowMessage('Pro tvorbu DBF Nutno zadat CRT nebo man.definici!');
         exit;
      end;
      CreateTable(DBFName,DBFDefinition);
   end;
   DBFTable := TShapeDBFTable.Create(Application);
   DBFTable.TableName := DBFName;
   DBFTable.Table.Open;
   if not DBFTable.Table.Active then
   begin
      DBFTable.Free;
      DBFTable := nil;
   end;
end;

procedure TExportSHPFile.CreateExportElements;
var
   EL: TEntity;
   i,j: integer;
   e: TElement;
   MA: TMultiAreaElement;
   A: TAreaElement;
   GE: TGeometryElement;
   G: TGeometry;
   SHPE: TSHPExportElement;
   RecNo: Integer;

   procedure AddSHPE(aG: TGeometry);
   begin
      SHPE := TSHPExportElement.Create;
      SHPE.ShapeType := ShapeType;
      SHPE.RecNo := RecNo;
      Inc(RecNo);
      SHPE.Geometry := aG;
      SHPE.ExportProperties := GE;
      SHPE.Barva := 9;
      SHPE.Znacka := 1;
      TGeometryElementCollection(EL).AddAndUpdateRect(SHPE);
   end;
begin
   ProgressText := 'Pprava vrstvy pro export...';
   MaxProgress := Layer.LastIndex;
   Progress := 0;
   Application.ProcessMessages;
   EL := ExportLayer;
   //RecNo := 0;
   // haha nikdo mi nerek, ze to cislujou od 1
   RecNo := 1;
   for i := 0 to Layer.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Layer.Elements[i];
      if not (e is TGeometryElement) then continue;
      GE := TGeometryElement(e);
      if GE.NoGeometry then continue;
      G := GE.Geometry;
      if (G is TMultiAreaElement) then
      begin
         MA := TMultiAreaElement(G);
         for j := 0 to MA.LastIndex do
         begin
            A := MA.AreaElement[j];
            if (A is TPolyLineAreaElement) then
               A := TPolyLineAreaElement(A).CreateAreaElement;
            AddSHPE(A);
         end;
      end
      else
      begin
         if G is TPolyLineAreaElement then
            G := TPolyLineAreaElement(G).CreateAreaElement;
         AddSHPE(G);
      end;
   end;
end;

procedure TExportSHPFile.DoIt;
begin
   if (Layer = nil) or Layer.NoElements then
   begin
      ShowMessage(Comment+' Vrstva pro export nen definovna nebo je przdn!');
      exit;
   end;
   if Comment = '' then
      Comment := 'Export SHP';
   FindShapeType;
   if ShapeType = 0 then
   begin
      ShowMessage(Comment+' Vrstva obsahuje elementy neznmho typu!');
      exit;
   end;
   if FileName = '' then
   begin
      ShowMessage(Comment+' Chyb jmno souboru pro export!');
      exit;
   end;
   StartProgress(Comment,0,0);
   CreateExportElements;
   ExportToFile;
   EndProgress;
end;

procedure TExportSHPFile.ExportArea(aSHPE: TSHPExportElement);
const
  MaxPoints = $7FFF;
type
   TParts = array[0..1023]of longInt;
   TPoints = array[0..MaxPoints]of TShpPoint;
var
   Parts, Points: pointer;
   G: TGeometry;
   A,A1: TAreaElement;
   i,j: integer;
   HasHoles: boolean;
   Lasti: integer;
   aL: longInt;
begin
   G := aSHPE.Geometry;
   if not (G is TAreaElement) then exit;
   A := TAreaElement(G);
   ArcRec.NumParts := 1;
   HasHoles := (A.Holes <> nil) and not A.Holes.NoElements;
   if HasHoles then
      ArcRec.NumParts := A.Holes.Count + 1;
   ArcRec.NumPoints := A.CoordsCount;
   if HasHoles then
   for i := 0 to A.Holes.LastIndex do
   begin
      A1 := TAreaElement(A.Holes.Elements[i]);
      ArcRec.NumPoints := ArcRec.NumPoints + A1.CoordsCount;
   end;
   ArcRec.Xmin := A.MinX;
   ArcRec.Ymin := A.MinY;
   ArcRec.XMax := A.MaxX;
   ArcRec.YMax := A.MaxY;
   RecHeader.RecNo := aSHPE.RecNo;
   RecHeader.ShapeType := ShapeType;
   aL := (44+(ArcRec.NumPoints * 16)+(ArcRec.NumParts * 4)) div 2;
   RecHeader.RecLength := aL;
   IndexRec.Offset := Stream.Position div 2;
   IndexRec.ContentLength := aL;
   Streamx.Write(IndexRec.FOffset,8);
   Stream.Write(RecHeader.FRecNo, 12);
   Stream.Write(ArcRec.FXmin, 40);
   getMem(pointer(Parts), ArcRec.NumParts * 4);
   getMem(Points, ArcRec.NumPoints * 16);
   TParts(Parts^)[0] := 0;
   Lasti := A.CoordsCount;
   if HasHoles then
   for i := 0 to A.Holes.LastIndex do
   begin
      TParts(Parts^)[i+1] := Lasti;
      A1 := TAreaElement(A.Holes.Elements[i]);
      Lasti := Lasti + A1.CoordsCount;
   end;
   for i := 0 to A.CoordsCount - 1 do
   begin
      TPoints(Points^)[i].X := A.Xi[i];
      TPoints(Points^)[i].Y := A.Yi[i];
   end;
   Lasti := A.CoordsCount;
   if HasHoles then
   for i := 0 to A.Holes.LastIndex do
   begin
      A1 := TAreaElement(A.Holes.Elements[i]);
      for j := 0 to A1.CoordsCount - 1 do
      begin
        TPoints(Points^)[Lasti+j].X := A1.Xi[j];
        TPoints(Points^)[Lasti+j].Y := A1.Yi[j];
      end;
      Lasti := Lasti + A1.CoordsCount;
   end;
   Stream.Write(Parts^, ArcRec.NumParts * 4);
   Stream.Write(Points^, ArcRec.NumPoints * 16);
   freeMem(Points, ArcRec.NumPoints * 16);
   freeMem(Parts, ArcRec.NumParts*4);
   ExportDBFRec(aSHPE);
end;

procedure TExportSHPFile.ExportDBFRec(aSHPE: TSHPExportElement);
var
   i: integer;
   SE: TStringElement;
   e: TElement;
   S: string;
begin
   if DBFTable = nil then exit;
   DBFTable.Table.Append;
   if TransferModel.NoElements then exit;
   e := aSHPE.ExportProperties;
   if e = nil then
      e := aSHPE;
   for i := 0 to TransferModel.LastIndex do
   begin
      SE := TransferModel.StringElements[i];
      if SameText(SE.Name,'RecNo') then
         DBFTable.SetFieldAsString(SE.Value,aSHPE.GetPropertyStringByName(SE.Name))
      else
      if pos('G_',SE.Name) = 1 then
      begin
         S := Copy(SE.Name,3,Length(SE.Name)-2);
         DBFTable.SetFieldAsString(SE.Value,aSHPE.Geometry.GetPropertyStringByName(S));
      end
      else
         DBFTable.SetFieldAsString(SE.Value,e.GetPropertyStringByName(SE.Name));
   end;
   DBFTable.Table.Post;
end;

procedure TExportSHPFile.ExportLine(aSHPE: TSHPExportElement);
const
  MaxPoints = $7FFF;
type
   TParts = array[0..1023]of longInt;
   TPoints = array[0..MaxPoints]of TShpPoint;
var
   Parts, Points: pointer;
   G: TGeometry;
   L: TPolyLineElement;
   i: integer;
   aL: longInt;
begin
   G := aSHPE.Geometry;
   if not (G is TPolyLineElement) then exit;
   L := TPolyLineElement(G);
   ArcRec.NumParts := 1;
   ArcRec.NumPoints := L.CoordsCount;
   ArcRec.Xmin := L.MinX;
   ArcRec.Ymin := L.MinY;
   ArcRec.XMax := L.MaxX;
   ArcRec.YMax := L.MaxY;
   RecHeader.RecNo := aSHPE.RecNo;
   RecHeader.ShapeType := ShapeType;
   aL := (44+(ArcRec.NumPoints * 16)+(ArcRec.NumParts * 4)) div 2;
   RecHeader.RecLength := aL;
   IndexRec.Offset := Stream.Position div 2;
   IndexRec.ContentLength := aL;
   Streamx.Write(IndexRec.FOffset,8);
   Stream.Write(RecHeader.FRecNo, 12);
   Stream.Write(ArcRec.FXmin, 40);
   getMem(pointer(Parts), ArcRec.NumParts * 4);
   getMem(Points, ArcRec.NumPoints * 16);
   TParts(Parts^)[0] := 0;
   for i := 0 to L.CoordsCount - 1 do
   begin
      TPoints(Points^)[i].X := L.Xi[i];
      TPoints(Points^)[i].Y := L.Yi[i];
   end;
   Stream.Write(Parts^, ArcRec.NumParts * 4);
   Stream.Write(Points^, ArcRec.NumPoints * 16);
   freeMem(Points, ArcRec.NumPoints * 16);
   freeMem(Parts, ArcRec.NumParts*4);
   ExportDBFRec(aSHPE);
end;

procedure TExportSHPFile.ExportPoint(aSHPE: TSHPExportElement);
var
   G: TGeometry;
   P: TPointElement;
   i: integer;
   aL: longInt;
begin
   G := aSHPE.Geometry;
   if not (G is TPointElement) then exit;
   P := TPointElement(G);
   PointRec.X := P.X;
   PointRec.Y := P.Y;
   RecHeader.RecNo := aSHPE.RecNo;
   RecHeader.ShapeType := ShapeType;
   RecHeader.RecLength := 20;
   IndexRec.Offset := Stream.Position div 2;
   IndexRec.ContentLength := 20;
   Streamx.Write(IndexRec.FOffset,8);
   Stream.Write(RecHeader.FRecNo, 12);
   Stream.Write(PointRec.FX, 16);
   ExportDBFRec(aSHPE);
end;

procedure TExportSHPFile.ExportToFile;
var
   FilePath: string;
   i: integer;
   S: string;
   e: TElement;
   SHPE: TSHPExportElement;
begin
   FilePath := ExtractFilePath(FileName);
   if FilePath <> '' then
   begin
      if not DirectoryExists(FilePath) then
         ForceDirectories(FilePath);
   end;
   FileName := ChangeFileExt(FileName,'.shp');
   IndexFileName := ChangeFileExt(FileName,'.shx');
   DBFName := ChangeFileExt(FileName,'.dbf');
   CreateDBF;
   // * SHP File Header
   FileHeader := TSHPFileHeader.Create;
   FileHeader.FileCode := 9994;
   FileHeader.Version := 1000;
   FileHeader.ShapeType := ShapeType;
   FileHeader.XMin := TGeometryElementCollection(ExportLayer).MinX;
   FileHeader.YMin := TGeometryElementCollection(ExportLayer).MinY;
   FileHeader.XMax := TGeometryElementCollection(ExportLayer).MaxX;
   FileHeader.YMax := TGeometryElementCollection(ExportLayer).MaxY;
   RecHeader := TShpFileRecHeader.Create;
   ArcRec   := TShpArcRec.Create;
   PointRec := TShpPointRec.Create;
   IndexRec := TShpIndexRec.Create;
   // Header ... jeste chybi delka souboru - ta se napise az nakonec
   Stream := TFileStream.Create(FileName,fmCreate);
   Stream.Write(FileHeader.FFileCode, 100);
   Streamx := TFileStream.Create(IndexFileName,fmCreate);
   Streamx.Write(FileHeader.FFileCode, 100);
   Progress := 0;
   MaxProgress := ExportLayer.LastIndex;
   S := 'Export ';
   case ShapeType of
      sht_arc: S := S + 'ARC';
      sht_Polygon: S := S + 'POLYGON';
   else
      S := S + 'ehosi';
   end;
   S := S +' do souboru '+FileName;
   ProgressText := S;
   Application.ProcessMessages;
   for i := 0 to ExportLayer.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      e := ExportLayer.Elements[i];
      SHPE := TSHPExportElement(e);
      case ShapeType of
         sht_arc     : ExportLine(SHPE);
         sht_Polygon : ExportArea(SHPE);
         sht_Point   : ExportPoint(SHPE)
      end;
   end;
   // konec
   CloseDBF;
   FileHeader.FileLength := Stream.Position div 2;
   Stream.Seek(0,0);
   Stream.Write(FileHeader.FFileCode, 100);
   FileHeader.FileLength := Streamx.Position div 2;
   Streamx.Seek(0,0);
   Streamx.Write(FileHeader.FFileCode, 100);
   Streamx.Free;
   Stream.Free;
   IndexRec.Free;
   RecHeader.Free;
   ArcRec.Free;
   PointRec.Free;
   FileHeader.Free;
end;

procedure TExportSHPFile.FindShapeType;
var
   GE: TGeometryElement;
   G: TGeometry;
   e: TElement;
begin
   ShapeType := 0;
   e := Layer.Elements[0];
   if not (e is TGeometryElement) then exit;
   GE := TGeometryElement(e);
   G := GE.Geometry;
   if G = nil then exit;
   if (G is TPolyLineElement) then
      ShapeType := sht_arc
   else if (G is TAreaElement) then
      ShapeType := sht_polygon
   else if (G is TPointElement) then
      ShapeType := sht_point;
end;

function TExportSHPFile.GetDBDefinition: TEntity;
begin
   if FDBFDefinition = nil then
   begin
      FDBFDefinition := TStringListElement.Create;
      FDBFDefinition.Name := 'DBF_Def';
      AddElement(FDBFDefinition);
   end;
   result := FDBFDefinition;
end;

function TExportSHPFile.GetExportLayer: TEntity;
begin
   if FExportLayer = nil then
   begin
      FExportLayer := TGeometryElementCollection.Create;
      FExportLayer.Name := 'Export_Layer';
      AddElement(FExportLayer);
   end;
   result := FExportLayer;
end;

function TExportSHPFile.GetTransferModel: TStringListElement;
begin
   if FTransferModel = nil then
   begin
      FTransferModel := TStringListElement.Create;
      FTransferModel.Name := 'T_Model';
      AddElement(FTransferModel);
   end;
   result := FTransferModel;
end;

procedure TExportSHPFile.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TExportSHPFile.SetCRTFileName(const Value: string);
begin
  FCRTFileName := Value;
end;

procedure TExportSHPFile.SetExportDBF(const Value: boolean);
begin
  FExportDBF := Value;
end;

procedure TExportSHPFile.SetExportLayer(const Value: TEntity);
begin
  FExportLayer := Value;
end;

procedure TExportSHPFile.SetFileName(const Value: string);
begin
  FFileName := Value;
end;

procedure TExportSHPFile.SetLayer(const Value: TEntity);
begin
  FLayer := Value;
end;

procedure TExportSHPFile.SetNextFieldDef(const Value: string);
var
   SL: TStringList;
   S: string;
   i: integer;
   FD: TFieldDefElement;
   aName: string;
   aType: char;
   aSize, aDecimal: word;
begin
   aName := '';
   aType := 'C';
   aSize := 3;
   aDecimal := 0;
   SL := TStringList.Create;
   SL.CommaText := Value;
   if SL.Count < 3 then
   begin
      ShowMessage('Error field definition: '+Value);
      exit;
   end;
   aName := SL.Strings[0];
   S := SL.Strings[1];
   aType := S[1];
   S := SL.Strings[2];
   aSize := StrToIntDef(S,3);
   if SL.Count > 3 then
   begin
      S := SL.Strings[3];
      aDecimal := StrToIntDef(S,0);
   end;
   FD := TFieldDefElement.Create;
   FD.SetField(aName,aType,aSize,aDecimal);
   DBFDefinition.NextElement := FD;
end;

procedure TExportSHPFile.SetShapeType(const Value: integer);
begin
  FShapeType := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TExportSHPFile);
END.
