unit E_psymb;
{
****************************************************************
**                                                            **
**                 ORBIS ... model TopoL                      **
**  Jednotka pro praci s Bodovymi symboly, zaloenmi na      **
**  standardu BGI.                                            **
**                                                            **
**                                                -PPK-96-    **
****************************************************************
}

interface

uses
// KERNEL
   CtiPar,
   Elements,
// SPACE
   e_Geometry,
   //E_Space,
   //u_TopolT,
// DEVICE
   e_Device,
   g_Elems,
// SYMBOL
   u_Symbol,
   //e_TopMDL,
   e_BGIF;

type


TBGISymbol = class(TBGIFont)
 private
   FNumber: longInt;
 public
   function NewFontByNumber(aNumber: longInt; aFile: string): boolean;
   property FontNumber: longInt read FNumber write FNumber default -1;
end;

TBGIFontFile = class(TOwnerElement)
 private
   FNumber: longInt;
   FHeigth: double;
   FMode: byte;
   FPenWidth: double;
 public
   property Number: longInt read FNumber write FNumber default -1;
   property Heigth: double read FHeigth write FHeigth ;
   property Mode: byte read FMode write FMode default Meritkove;
   property PenWidth: double read FPenWidth write FPenWidth;
end;


TTopoLSymbols{Hm...} = class(TOwnerEntity)
 private
   BGIS: TBGISymbol;
   TopText: TBGIText;
   FDevice: TDevice;
   procedure SetDevice(aDevice: TDevice);
 public
   constructor CreateAll(aFile: string);
   procedure   InterpretSymbol(Point: TGeometry; Znacka: longInt);
   procedure   PaintSymbol(X, Y, Uhel: double; Barva, Znacka: integer);
   procedure   OpsanyKosodelnik (Point: TGeometry; Znacka: longInt; var LLD,PPD,LLH,PPH: TDoubleCoord);
   function    GetFileCode(aCode: longInt): TBGIFontFile;
   property    Device: TDevice read FDevice write SetDevice;
end;

{
var
   TopoLSymbols: TTopoLSymbols;
}
implementation


uses
  SysUtils,
// KERNEL
  CtiRadek,
  u_String,
// GLIB
  e_Color;
  //e_TplPrp;

{ TBGISymbol }

function TBGISymbol.NewFontByNumber(aNumber: longInt; aFile: string): boolean;
const
{ Overovaci znacka BGI fontu }
  BGICode = $08084B50;
var
  FontF: file;
  Magic: LongInt;
  //Nalezl: boolean;

begin
  result := true;
  OuKej    := Ano;
  if (aNumber <> FontNumber) and (aFile <> Name) then
  begin
     if PFont <> NIL then begin
        FreeMem(PFont,DelkaFontu);
        FontNumber := -1;
        PFont := NIL;
     end;
     {$I-}
     System.Assign (FontF, aFile);
     fileMode:= moOpenRONN;
     Reset  (FontF, 1);
     fileMode:= moStandard;
     {$I+}
     if IOResult <> 0 then Begin
        Chyba(errCannotOpenVectorFont,aFile);
        Oukej  := false;
        result := false;
        exit;
     end
     else begin
        DelkaFontu := FileSize (FontF)-$80;
        GetMem (PFont,DelkaFontu);
        if PFont = NIL then begin
           Chyba(errOutOfMemory,'');
           OuKej  := Ne;
           result := Ne;
        end
        else begin
           { Je to vubec BGI format? }
           { ----------------------- }
           Seek (FontF,0);
           BlockRead (FontF, Magic, SizeOf(longInt));
           if Magic <> BGICode then
           { Prdlajs! }
           { -------- }
           begin
              OuKej  := false;
              result := false;
           end
           else
           begin
              Seek (FontF,$80);
              BlockRead (FontF, PFont^,DelkaFontu);
              Udaje        := PFont;
              FontNumber   := aNumber;
              Name         := aFile;
           end;
        end;
     end;
     Close (FontF);
  end;
end;


{ TTopoLSymbols }

constructor TTopoLSymbols.CreateAll(aFile: string);
var
  F: System.text;
  Radek: string;
  CtiR: TCtiRadekClass;
  OK: boolean;
  BGIFF : TBGIFontFile;
begin
   inherited Create;
   if (aFile <> '') and fileExists(aFile) then
   begin
      BGIS    := TBGISymbol.Create;
      BGIS.FreeTextInfo := false;
      AddElement(BGIS);
      TopText := TBGIText.Create;
      AddElement(TopText);
      BGIS.TextInfo := TopText;

      System.assign(F, aFile);
      fileMode:= moOpenRONN;
      {$I-} reset(F); {$I+}
      fileMode:= moStandard;
      OK :=  IOResult = 0;
      if OK then
      begin
         while not eof(F) and OK do
         begin
           readLn(F, Radek);
           if Radek <> '' then
           begin
              BGIFF := TBGIFontFile.Create;

              CtiR:= TCtiRadekClass.Create(Radek);
              BGIFF.Number:= CtiR.DejInteger;
              CtiR.PreskocMezery;
              BGIFF.Name:= BezVnejsichMezer(CtiR.DejStringPoZnak(' '));
              CtiR.PreskocMezery;
              case upCase(CtiR.DejChar) of
                 'M': BGIFF.Mode := Meritkove;
                 'D': BGIFF.Mode := Relativne;
                 'S': BGIFF.Mode := Absolutne;
              end;
              CtiR.PreskocMezery;
              BGIFF.Heigth := CtiR.DejReal;
              CtiR.PreskocMezery;
              BGIFF.PenWidth := CtiR.DejReal;
              if (BGIFF.Number < 1 ) or (BGIFF.Heigth < 0.0001) then
                 BGIFF.Free
              else
                 AddElement(BGIFF);

              CtiR.Free
           end;
         end;
         Close(F);
      end;
   end
end;



procedure   TTopoLSymbols.SetDevice(aDevice: TDevice);
begin
   BGIS.SetDevice(aDevice);
   FDevice := aDevice;
end;


procedure TTopoLSymbols.PaintSymbol(X, Y, Uhel: double; Barva, Znacka: integer);
var
   FontCode, SymbolCode: longInt;
   FontFile: TBGIFontFile;
begin
   FontCode := Znacka div 1000;
   SymbolCode := Znacka mod 1000;
   FontFile := GetFileCode(FontCode);
   if not assigned(FontFile) then exit;
   BGIS.NewFontByNumber(FontCode, PathOfLibraries + '\' + FontFile.Name);
   //Device.Pen.Color := DejBarvicku(Barva);
   Device.Pen.Color := Barva;
   Device.Pen.Width := Device.Projection.GetPageLengthHoriz1(FontFile.PenWidth/1000 * Device.Projection.VirtualScale);
   TopText.VychoziBod.x := X;
   TopText.VychoziBod.y := Y;
   TopText.Vyska      := FontFile.Heigth;
   TopText.KoefSirky  := 1;
   TopText.Smer       := Uhel; { Uhel v Radianech }
   TopText.Sklon      := 0; { Uhel v Radianech }
   TopText.Rozestupy  := 0; { Zvetseni default hodnoty v Bodech }
   TopText.Atributy   := [T30];
   TopText.RezimTextu := FontFile.Mode;

   BGIS.PisTextFinal(Char(SymbolCode));
end;



procedure   TTopoLSymbols.InterpretSymbol(Point: TGeometry; Znacka: longInt);
var
   FontCode, SymbolCode: longInt;
   FontFile: TBGIFontFile;
begin
   FontCode := Znacka div 1000;
   SymbolCode := Znacka mod 1000;
   FontFile := GetFileCode(FontCode);
   if not assigned(FontFile) then exit;
   BGIS.NewFontByNumber(FontCode, PathOfLibraries + '\' + FontFile.Name);
   {
   Device.Pen.Color := DejBarvicku(Point.Barva);
   }
   Device.Pen.Width := Device.Projection.GetPageLengthHoriz1(FontFile.PenWidth/1000 * Device.Projection.VirtualScale);
   TopText.VychoziBod.x := Point.x;
   TopText.VychoziBod.y := Point.y;
   TopText.Vyska      := FontFile.Heigth;
   TopText.KoefSirky  := 1;
   TopText.Smer       := Point.Angle; { Uhel v Radianech }
   TopText.Sklon      := 0; { Uhel v Radianech }
   TopText.Rozestupy  := 0; { Zvetseni default hodnoty v Bodech }
   TopText.Atributy   := [T30];
   TopText.RezimTextu := FontFile.Mode;

   try
     BGIS.PisTextFinal(Char(SymbolCode));
   except
   end
end;

function    TTopoLSymbols.GetFileCode(aCode: longInt): TBGIFontFile;
var
   i: longInt;
   BGIFF: TBGIFontFile;
begin
   result := nil;
   for i := 0 to Count -1 do
   begin
      BGIFF := TBGIFontFile(Elements[i]);
      if not (TObject(BGIFF) is TBGIFontFile) then Continue;
      if BGIFF.Number = aCode then
      begin
         result := BGIFF;
         exit;
      end;
   end;
end;


procedure TTopoLSymbols.OpsanyKosodelnik(Point: TGeometry; Znacka: longInt; var LLD, PPD, LLH, PPH: TDoubleCoord);
var
   FontCode, SymbolCode: longInt;
   FontFile: TBGIFontFile;
begin
   FontCode := Znacka div 1000;
   SymbolCode := Znacka mod 1000;
   FontFile := GetFileCode(FontCode);
   if not assigned(FontFile) then exit;
   BGIS.NewFontByNumber(FontCode, PathOfLibraries + '\' + FontFile.Name);
   {
   Device.Pen.Color := DejBarvicku(Point.Barva);
   }
   Device.Pen.Width := Device.Projection.GetPageLengthHoriz1(FontFile.PenWidth/1000 * Device.Projection.VirtualScale);
   TopText.VychoziBod.x := Point.x;
   TopText.VychoziBod.y := Point.y;
   TopText.Vyska      := FontFile.Heigth;
   TopText.KoefSirky  := 1;
   TopText.Smer       := Point.Angle; { Uhel v Radianech }
   TopText.Sklon      := 0; { Uhel v Radianech }
   TopText.Rozestupy  := 0; { Zvetseni default hodnoty v Bodech }
   TopText.Atributy   := [T30];
   TopText.RezimTextu := FontFile.Mode;
   TopText.Veta := Char(SymbolCode);
   //LLD := (0,0);
   //PPD := 0;
   //LLH := 0;
   //PPH := 0;
   try
     BGIS.OpsanyKosodelnik(LLD, PPD, LLH, PPH);
   except
   end
end;

END.
