unit E_QTree;

interface
uses
// KERNEL
  Elements,
// GLIB
  e_Geometry,
  G_Elems;
// SPACE
  //E_Space;

const
  MaxCountInCell: longInt = 256;

type

TQTree = class(TRectEntity)
private
  FLevel: longInt;
  FQWidth: longInt;
  FQMinWidth: integer;
    FMaxLeafCount: integer;

  function GetElements_Count: longInt;
  function GetLeavesCount: longInt;
  function GetIsLeaf: boolean;
  function RectangleIsIn(aXmin, aYmin, aXmax, aYmax: double): boolean;
  procedure MakeNewQTreeAndMove;
  procedure AddRectElement(Element: TElement; aXmin, aYmin, aXmax, aYmax: double);
  procedure CreateSubQTrees(var QTreeBL, QTreeBR, QTreeUL, QTreeUR: TQTree);
  procedure AddDirectly(Element: TElement);
  procedure SetQXMin(aValue: double);
  function  GetQXMin: double;
  procedure SetQYMin(aValue: double);
  function  GetQYMin: double;
  procedure SetQWidth(aValue: longInt);
  procedure SetQMinWidth(const Value: integer);
    procedure SetMaxLeafCount(const Value: integer);

protected
  function MaxCountInCell: longInt; virtual;
  procedure AddDirectlyEx(Element: TElement); virtual;
  procedure GiveElementRectangle(Element: TElement; var aXmin, aYmin, aXmax, aYmax: double); virtual; abstract;
  function IsIn(aQXmin, aQYmin, aWidth: longInt): boolean;
  function IsOver(aQXmin, aQYmin, aWidth: longInt): boolean;
public

  procedure Empty;
  procedure Add   (Element: TElement); {pouze se vlo do existujcho QTree}
  procedure ExtendQuad(aXmin, aYmin, aXmax, aYmax: double);
  procedure Extend(Element: TElement); {roz stvajc QTree }

  procedure SetForRectangle(Xmin, Ymin, Xmax, Ymax: double);
  function  HasElementOverRectangle(Xmin, Ymin, Xmax, Ymax: double): boolean;
  function  HasLeafOverRectangle(Xmin, Ymin, Xmax, Ymax: double): boolean;
  function  IsOverRectangle(Xmin, Ymin, Xmax, Ymax: double): boolean;
  function  XYIsIn(aX, aY: double): boolean;
  function  ElementIsOverRectangle
            (Element: TElement; Xmin, Ymin, Xmax, Ymax: double): boolean; virtual;
  procedure GetPaterAndSelfIndex(var PaterQTree: TQTree; var FIndex: longInt);
  procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
  procedure on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
published
  property Level: longInt read FLevel write FLevel;
  property IsLeaf: boolean read GetIsLeaf;
  property QXMin: double read GetQXmin write SetQXmin;
  property QYMin: double read GetQYmin write SetQYmin;
  property QWidth: longInt read FQWidth write SetQWidth;
  property QMinWidth: integer read FQMinWidth write SetQMinWidth default 1;
  property Elements_Count: longInt read GetElements_Count;
  property LeavesCount: longInt read GetLeavesCount;
  property MaxLeafCount: integer read FMaxLeafCount write SetMaxLeafCount;
end;



TRectEntityQTree = class(TQTree)
private
  procedure SetNextRectEntity(aObject: TRectEntity);
protected
  function MaxCountInCell: longInt; override;
  procedure AddDirectlyEx(Element: TElement); override;
  procedure GiveElementRectangle(Element: TElement; var aXmin, aYmin, aXmax, aYmax: double); override;
published
  property NextRectEntity: TRectEntity write SetNextRectEntity;
end;



TQTreeWalker = class
private
  FXmin, FYmin, FXmax, FYmax: double;
  FActualQTree: TQTree;
  FIndex: longInt;
  FTolerance: double;
  procedure Step;
protected
  function QGoInside(aQTree: TQTree): boolean; virtual;
public
  function GetFirstElementForRectangle(QTree: TQTree; Xmin, Ymin, Xmax, Ymax, aTolerance: double): TElement;
  function GetNextElement: TElement;
end;


TSpaceRect = class(TRectEntity)
private
  HX, HY: double; { polovicni hodnoty }
  FLUQ, FLBQ, FRUQ, FRBQ: TSpaceRect;
  FIsLast: boolean;
  function GetCount: longInt; virtual; abstract;
protected
  procedure PrepareQTree;
public
  constructor Create; override;
published
  property Last: boolean read FIsLast default true;
end;




TSpaceLineBegEndCell = class(TSpaceRect)
private

  function GetCount: longInt; override;
  procedure AddToQTree
           (Line: TPolyLineElement; aX, aY: double; aBegin: boolean);
  procedure AddToQTreeDirectly
           (Line: TPolyLineElement; aX, aY: double; aBegin: boolean);
  procedure AddBegEnd(Line: TPolyLineElement; aBegin: boolean);
  procedure MoveLineToQTree(Lin: TPolyLineElement; aBegin: boolean);
  procedure MoveToQTree;
  procedure AddLineXY(Lin: TPolyLineElement; aX, aY: double; aBegin: boolean);
public
  FLineBegins, FLineEnds: TPolyLineString;

  constructor Create; override;
  procedure AddLine(Lin: TPolyLineElement; aBegin: boolean);
published
  property Count: longInt read GetCount;
end;




{$IFDEF TestProtocol}

const
  ProtocolName: string = 'QTREE.PTC';
  Uroven: longInt = 0;

procedure AddToProtocol(aText: string);
{$ENDIF}


procedure VypisQTree(FName: string; QTree: TQTree);

implementation
uses
  SysUtils, Dialogs,
// KERNEL
  E_Class,
  e_GeometryElement,
  m_Orbis,
  U_String,
// GLIB
  AnalInt;

{$IFDEF TestProtocol}

procedure AddToProtocol(aText: string);
var
  F: TextFile;

  function Mezery(Cislo: longInt): string;
  var
    I: longInt;
  begin
    result:= ' ';
    for I:= 0 to Cislo do
        result:= result + ' '
  end;

begin
   AssignFile(F, ProtocolName);
          { Open file selected in dialog }
   try
     Append(F);	{ Add more text onto end }
   except
     try
       rewrite(F);
     except
       exit
     end;
   end;
   try
     Writeln(F, IntToStrZ(Uroven,3,false)+Mezery(Uroven) + aText);
   finally
     CloseFile(F);	{ Close file, save changes }
   end
end;

{$ENDIF}



procedure VypisVetev(var F: textFile; QTree: TQTree);
var
  I: longInt;
  Lin: TGeometryElement;
begin
  if QTree.IsLeaf then
  begin
    writeln(F, QTree.count:5,' =',QTree.QXmin:10, ' ', QTree.QYmin:10, ' ', QTree.QWidth:10);
    for I:= 0 to QTree.Count-1 do
    begin
      if QTree.Elements[I] is TGeometryElement then
      begin
        Lin:= TGeometryElement(QTree.Elements[I]);
        writeln(F, Lin.SpaceId:10, ' > ', Lin.GetPropertyStringByName('BlockDir'));
      end
    end
  end
  else
  begin
    writeln(F, QTree.Count:5, ' >',QTree.QXmin:10, ' ', QTree.QYmin:10, ' ', QTree.QWidth:10);
    for I:= 0 to QTree.Count-1 do
    begin
      if QTree.Elements[I] is TQtree then
        VypisVetev(F, TQTree(QTree.Elements[I]))
    end
  end
end;



procedure VypisQTree(FName: string; QTree: TQTree);
var
  F: textFile;
begin
  try
  assignFile(F, FName);

  if fileExists(FName) then
     append(F)
  else
     rewrite(F);
  writeln(F, '***************************************************');
  VypisVetev(F, QTree);
  close(F);
  except
    ShowMessage('Chyba pi zpisu do souboru ' + FName)
  end
end;




function TQTree.MaxCountInCell: longInt;
begin
   if MaxLeafCount > 0 then
      result := MaxLeafCount
   else
      result:= MaxCountInCell;
end;


function TQTree.GetElements_Count: longInt;
var
  Sum: longInt;
  I: longInt;
begin
  Sum:= 0;
  if not IsLeaf then
  begin
    for I:= 0 to 3 do
    begin
      Sum:= Sum+TQTree(Elements[I]).GetElements_Count;
    end
  end
  else
  begin
    Sum:= Sum + Count
  end;
  result:= Sum
end;



function TQTree.GetLeavesCount: longInt;
var
  Sum: longInt;
  I: longInt;
begin
  Sum:= 0;
  if not IsLeaf then
  begin
    for I:= 0 to 3 do
    begin
      Sum:= Sum+TQTree(Elements[I]).GetLeavesCount;
    end
  end
  else
  begin
    inc(Sum)
  end;
  result:= Sum
end;



procedure TQTree.SetQXMin(aValue: double);
begin
  MinX:= aValue;
end;


function TQTree.GetQXMin: double;
begin
  result:= MinX;
end;

procedure TQTree.SetQYMin(aValue: double);
begin
  MinY:= aValue;
end;

function TQTree.GetQYMin: double;
begin
  result:= MinY;
end;


procedure TQTree.SetQWidth(aValue: longInt);
begin
  FQWidth:= aValue;
  MaxX:= MinX + aValue;
  MaxY:= MinY + aValue
end;






function TQTree.GetIsLeaf: boolean;
begin
  Result:= NoElements or
           not (Elements[0] is TQTree)
end;

function TQTree.RectangleIsIn(aXmin, aYmin, aXmax, aYmax: double): boolean;
begin
  result:=
     (aXmin >= QXmin) and (aXmax <= QXmin + QWidth) and
     (aYmin >= QYmin) and (aYmax <= QYmin + QWidth)
end;


function TQTree.IsIn(aQXmin, aQYmin, aWidth: longInt): boolean;
begin
  result:= (Width <= FQWidth) and
           (QXMin <= aQXMin) and (QXMin + QWidth >= aQXmin + aWidth) and
           (QYMin <= aQYMin) and (QYMin + QWidth >= aQYmin + aWidth)
end;



procedure TQTree.Empty;
var
  I: longInt;
begin
  if not IsLeaf then
  begin
    for I:= 0 to 3 do
    begin
      TQTree(Elements[I]).Empty;
    end
  end
  else
    FreeAll
end;






function TQTree.IsOver(aQXmin, aQYmin, aWidth: longInt): boolean;
begin
  result:= IsIn(aQXmin, aQYmin, aWidth) or
           (
           not ((QXmin > aQXmin+aWidth) or (QXmin + QWidth < aQXmin))
           and
           not((QYmin > aQYmin+aWidth) or (QYmin + QWidth < aQYmin))
           )
end;


procedure TQTree.CreateSubQTrees(var QTreeBL, QTreeBR, QTreeUL, QTreeUR: TQTree);
var
  HalfWidth: longInt;
begin
  HalfWidth:= FQWidth div 2;

  QTreeBL:= TQTree(TElementClass(self.ClassType).Create);
  QTreeBL.QXMin:= QXMin;
  QTreeBL.QYMin:= QYMin;
  QTreeBL.QWidth:= HalfWidth;
  //QTreeBL.Name:= 'Lev doln tverec';
  QTreeBL.Level:= Level + 1;
  QTreeBL.MaxLeafCount := MaxLeafCount;

  QTreeBR:= TQTree(TElementClass(self.ClassType).Create);
  QTreeBR.QXMin:= QXMin + HalfWidth;
  QTreeBR.QYMin:= QYMin;
  QTreeBR.QWidth:= HalfWidth;
  //QTreeBR.Name:= 'Prav doln tverec';
  QTreeBR.Level:= Level + 1;
  QTreeBR.MaxLeafCount := MaxLeafCount;

  QTreeUL:= TQTree(TElementClass(self.ClassType).Create);
  QTreeUL.QXMin:= QXMin;
  QTreeUL.QYMin:= QYMin + HalfWidth;
  QTreeUL.QWidth:= HalfWidth;
  //QTreeUL.Name:= 'Lev horn tverec';
  QTreeUL.Level:= Level + 1;
  QTreeUL.MaxLeafCount := MaxLeafCount;

  QTreeUR:= TQTree(TElementClass(self.ClassType).Create);
  QTreeUR.QXMin:= QXMin + HalfWidth;
  QTreeUR.QYMin:= QYMin + HalfWidth;
  QTreeUR.QWidth:= HalfWidth;
  //QTreeUR.Name:= 'Prav horn tverec';
  QTreeUR.Level:= Level + 1;
  QTreeUR.MaxLeafCount := MaxLeafCount;
end;



procedure TQTree.AddDirectly(Element: TElement);
var
  aXMin, aYMin, aXmax, aYmax: double;
begin
  GiveElementRectangle(Element, aXmin, aYmin, aXmax, aYmax);

  if IsOverRectangle(aXmin, aYmin, aXmax, aYmax) then
     AddElement(Element)
end;

procedure TQTree.AddDirectlyEx(Element: TElement);
begin
   AddDirectly(Element);
end;


procedure TQTree.MakeNewQTreeAndMove;
var
  QTreeBL, QTreeBR, QTreeUL, QTreeUR: TQTree;
  //HalfWidth: longInt;
  I: longInt;
begin
  CreateSubQTrees(QTreeBL, QTreeBR, QTreeUL, QTreeUR);

  for I:= 0 to Count-1 do
  begin
    QTreeBL.AddDirectlyEx(Elements[I]);
    QTreeBR.AddDirectlyEx(Elements[I]);
    QTreeUL.AddDirectlyEx(Elements[I]);
    QTreeUR.AddDirectlyEx(Elements[I]);
  end;
  {
  if (QTreeBL.Count = Count) or (QTreeBR.Count = Count) or
     (QTreeUL.Count = Count) or (QTreeUR.Count = Count) then
  begin
     QTreeBL.FreeAll;
     QTreeBR.FreeAll;
     QTreeUL.FreeAll;
     QTreeUR.FreeAll;
     for I:= 0 to Count-1 do
     begin
       QTreeBL.AddDirectlyEx(Elements[I]);
       QTreeBR.AddDirectlyEx(Elements[I]);
       QTreeUL.AddDirectlyEx(Elements[I]);
       QTreeUR.AddDirectlyEx(Elements[I]);
     end;
  end;
  }
  FreeAll;

  AddElement(QTreeBL);
  AddElement(QTreeBR);
  AddElement(QTreeUL);
  AddElement(QTreeUR);
end;




procedure TQTree.AddRectElement(Element: TElement; aXmin, aYmin, aXmax, aYmax: double);
var
  I: longInt;
begin
  if IsOverRectangle(aXmin, aYmin, aXmax, aYmax) then
  begin
    if not IsLeaf then
    {Pidvn spodnm vtvm}
    begin
    {$IFDEF TestProtocol}
      inc(Uroven);
    {$ENDIF}
      for I:= 0 to 3 do
      begin
      {$IFDEF TestProtocol}
        AddToProtocol(IntToStrZ(I,2,false)+' >>>>>>>>>>>>>>>');
      {$ENDIF}
        try
          TQTree(Elements[I]).AddRectElement(Element, aXmin, aYmin, aXmax, aYmax)
        except
        end
      end;
    {$IFDEF TestProtocol}
      dec(Uroven)
    {$ENDIF}
    end
    else
    begin
    {Jsme na rovni listu, bude se pidvat nebo prohlubovat}
       AddDirectlyEx(Element);
      //AddElement(Element);
       {$IFDEF TestProtocol}
         AddToProtocol('Add Element ' + IntToStrZ(Count, 4, false) + ': ' +
         RealToStr(aXmin,10,0) + RealToStr(aYmin,10,0) +RealToStr(aXmax,10,0) + RealToStr(aYmax,10,0) + ' > '+
         IntToStrZ(QXmin,10,false) + IntToStrZ(QYmin,10,false) +IntToStrZ(FQWidth,10,false));
       {$ENDIF}
      if (QWidth > 1) and (Count > MaxCountInCell) then
      begin
      {$IFDEF TestProtocol}
        AddToProtocol('New QTrees');
      {$ENDIF}
        MakeNewQTreeAndMove;
      end
    end
  end
end;






procedure TQTree.ExtendQuad(aXmin, aYmin, aXmax, aYmax: double);
var
  QXminPuv, QYminPuv, QWidthPuv: longInt;
  QXMinVys, QYminVys, QWidthVys, QWidthX, QWidthY: longInt;
  QTreeBL, QTreeBR, QTreeUL, QTreeUR: TQTree;
  SQTreeBL, SQTreeBR, SQTreeUL, SQTreeUR: TQTree;
  ActQTree, SubQTree: TQTree;

  procedure  ExcludeSubQTree(aIndex: longInt; var aQTree: TQTree);
  begin
    try
      aQTree:= TQtree(Elements[aIndex]);
      if aQTree <> nil then
         RemoveElement(aQTree);
    except
      aQTree:= nil
    end
  end;

begin
{ Roz stvajc QTree na jeho nsobek mocninou dvou.
  Pvodn PodQTree vyjme a pokou se je vloit do nkterho novho PodQTree }

  if RectangleIsIn(aXmin, aYmin, aXmax, aYmax) then
     exit;

  VratNormRozsireni2D(round(QXmin), round(QWidth), aXmin, aXmax, QXminVys, QWidthX);
  VratNormRozsireni2D(round(QYmin), round(QWidth), aYmin, aYmax, QYminVys, QWidthY);


  QWidthVys:= QWidthX;
  if QWidthY > QWidthVys then QWidthVys:= QWidthY;

  { !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    doplnno 4.4.97,
    upraveno 30.6.97}

  if IsLeaf then
  begin
    if Level = 0 then
    begin
      QXMin:= QXMinVys;
      QYmin:= QYMinVys;
      QWidth:= QWidthVys;
    end;

    exit;
  end;
  { !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! }

  ExcludeSubQTree(3, QTreeUR);
  ExcludeSubQTree(2, QTreeUL);
  ExcludeSubQTree(1, QTreeBR);
  ExcludeSubQTree(0, QTreeBL);
  FreeAll;

  QXminPuv:= round(QXmin);
  QYminPuv:= round(QYmin);
  QWidthPuv:= QWidth;

  QXmin:= QXminVys;
  QYmin:= QYminVys;
  QWidth:= QWidthVys;
  ActQTree:= self;

  while ActQTree.QWidth > QWidthPuv do
  begin
    ActQTree.CreateSubQTrees(SQTreeBL, SQTreeBR, SQTreeUL, SQTreeUR);
    if SQTreeBL.IsIn(QXminPuv, QYminPuv, QWidthPuv) then SubQTree:= SQTreeBL else
    if SQTreeBR.IsIn(QXminPuv, QYminPuv, QWidthPuv) then SubQTree:= SQTreeBR else
    if SQTreeUL.IsIn(QXminPuv, QYminPuv, QWidthPuv) then SubQTree:= SQTreeUL else
    if SQTreeUR.IsIn(QXminPuv, QYminPuv, QWidthPuv) then SubQTree:= SQTreeUR
    else
    begin
      raise Exception.Create('Chyba pi roziovn QTree');
    end;

    ActQTree.AddElement(SQtreeBL);
    ActQTree.AddElement(SQtreeBR);
    ActQTree.AddElement(SQtreeUL);
    ActQTree.AddElement(SQtreeUR);

    ActQTree:= SubQTree
  end;

  ActQtree.AddElement(QTreeBL);
  ActQtree.AddElement(QTreeBR);
  ActQtree.AddElement(QTreeUL);
  ActQtree.AddElement(QTreeUR);

{!!!!!!!!!!!!!!!!!!!!!}
{  VypisQTree(Self) }
end;




procedure TQTree.Extend(Element: TElement);
var
  aXmin, aYmin, aXmax, aYmax: double;
begin
  { umon zvtit QTree aby obsahovalo i nov element }
  GiveElementRectangle(Element,  aXmin, aYmin, aXmax, aYmax);

  if (QWidth = 0) then
    SetForRectangle(aXmin, aYmin, aXmax, aYmax)
  else
    ExtendQuad(aXmin, aYmin, aXmax, aYmax);

  AddRectElement(Element, aXmin, aYmin, aXmax, aYmax)
end;



procedure TQTree.Add(Element: TElement);
var
  XMin, YMin, Xmax, Ymax: double;
begin
{ pouze pid element bez ohledu na to zda pesahuje
  => pokud bude mimo, tak se nezaad }
  GiveElementRectangle(Element, Xmin, Ymin, Xmax, Ymax);
  AddRectElement(Element, Xmin, Ymin, Xmax, Ymax)
end;



procedure TQTree.SetForRectangle(Xmin, Ymin, Xmax, Ymax: double);
var
  aQXMin, aQYmin, aQWidth: longInt;
begin
  VratNormCtverec(Xmin, Ymin, Xmax, Ymax, aQXmin, aQYmin, aQWidth);
  QXMin:= aQXMin;
  QYMin:= aQYMin;
  QWidth:= aQWidth;
end;



function TQTree.ElementIsOverRectangle
         (Element: TElement; Xmin, Ymin, Xmax, Ymax: double): boolean;
begin
  result:= false;
  if not (Element is TRectEntity) then exit;
  result:= TRectEntity(Element).IsOverRegion(Xmin, Ymin, Xmax, Ymax);
end;





function  TQTree.HasLeafOverRectangle(Xmin, Ymin, Xmax, Ymax: double): boolean;
var
  I: longInt;
begin
  result:= false;
  if IsLeaf then
     result:= IsOverRectangle(Xmin, Ymin, Xmax, Ymax)
  else
  begin
    I:= 0;
    while (I<Count) and not result do
    begin
      if Elements[I] is TQTree then
         result:= TQTree(Elements[I]).HasLeafOverRectangle(Xmin, Ymin, Xmax, Ymax);
      if not result then inc(I)
    end
  end
end;



function  TQTree.HasElementOverRectangle(Xmin, Ymin, Xmax, Ymax: double): boolean;
var
  I: longInt;
begin
  result:= false;
  if IsLeaf then
  begin
     if IsOverRectangle(Xmin, Ymin, Xmax, Ymax) then
     begin
       for I:= 0 to Count-1 do
       begin
         result:= ElementIsOverRectangle(Elements[I], Xmin, Ymin, Xmax, Ymax);
         if result then break
       end
     end
  end
  else
  begin
    I:= 0;
    while (I<Count) and not result do
    begin
      if Elements[I] is TQTree then
         result:= TQTree(Elements[I]).HasElementOverRectangle(Xmin, Ymin, Xmax, Ymax);
      if not result then inc(I)
    end
  end
end;







function  TQTree.IsOverRectangle(Xmin, Ymin, Xmax, Ymax: double): boolean;
begin
  result := not ((XMax < QXmin) or (Xmin > QXmin + QWidth))
            and
            not ((YMax < QYmin) or (Ymin > QYmin + QWidth))
end;




function  TQTree.XYIsIn(aX, aY: double): boolean;
begin
  result:= ((aX > QXMin) and (aX <= QXMin + QWidth) and
            (aY > QYMin) and (aY <= QYMin + QWidth)
           )
end;



procedure TQTree.GetPaterAndSelfIndex(var PaterQTree: TQTree; var FIndex: longInt);
begin
  PaterQTree:= TQTree(FindOwnerClass(TQTree));
  if PaterQtree <> nil then
     FIndex:= PaterQTree.IndexOf(Self)
end;





                { TSpaceQTree }

function TRectEntityQTree.MaxCountInCell: longInt;
begin
   if MaxLeafCount > 0 then
      result := MaxLeafCount
   else
      result:= 64;
end;


procedure TRectEntityQTree.GiveElementRectangle(Element: TElement; var aXmin, aYmin, aXmax, aYmax: double);
begin
  if not (Element is TRectEntity) then exit;
  with TRectEntity(Element) do
  begin
    aXmin:= MinX; aYmin:= MinY; aXmax:= MaxX; aYmax:= MaxY
  end;
end;



procedure TRectEntityQTree.SetNextRectEntity(aObject: TRectEntity);
begin
  if not (TElement(aObject) is TRectEntity) then exit;
  Extend(aObject)
end;





                         { TQTreeWalker }




function TQTreeWalker.GetFirstElementForRectangle(QTree: TQtree; Xmin, Ymin, Xmax, Ymax, aTolerance: double): TElement;
begin
  result:= nil;
  if not QTree.IsOverRectangle
     (Xmin-aTolerance, Ymin-aTolerance, Xmax+aTolerance, Ymax+aTolerance) then exit;
  FXmin:= Xmin;
  FYmin:= Ymin;
  FXmax:= Xmax;
  FYmax:= Ymax;
  FTolerance:= aTolerance;
  FIndex:= 0;
  FActualQTree:= QTree;
  result:= GetNextElement
end;



function TQTreeWalker.QGoInside(aQTree: TQTree): boolean;
begin
  result:= aQTree.IsOverRectangle
           (FXmin-FTolerance, FYmin-FTolerance, FXmax+FTolerance, FYmax+FTolerance)
end;


procedure TQTreeWalker.Step;
{ posune se o jeden krok ve stromu - vsledek je nov FActualQtree a FIndex
  Nevstupuje do QTree nevyhovujcc podmnce v QGoInside.
  Kon se kdy FActualQTree = nil }
begin
  if FIndex >= FActualQTree.Count then
  begin { doli jsme na konec seznamu elementu v listu }
    FActualQTree.GetPaterAndSelfIndex(FActualQTree, FIndex);
    inc(FIndex);
    exit;
  end;

  if FActualQTree.IsLeaf then { pohybujeme se v listu, pouze posun }
     inc(FIndex)
  else { FActualQTree dr dal QTree, zkusme vlzt dovnit }
  begin
    if QGoInside(TQTree(FActualQTree.Elements[FIndex])) then
    begin
      FActualQTree:= TQTree(FActualQTree.Elements[FIndex]);
      FIndex:= 0
    end
    else { nelze dovnit, tak se posuneme dle }
      inc(FIndex)
  end;
end;




function TQTreeWalker.GetNextElement: TElement;
var
  ActEl: TElement;
begin
  { me nastat:
    - FActualQTree = nil             ... konec prochzen stromu
    - FIndex >= FActualQTree.Count   ... je poteba zkouet dl
    - FActualQTree obsahuje QTree    ... zkouet dl
    - FActualQTree obsahuje Elementy ... zkusit zda Element [FIndex] vyhovuje }

  result:= nil;

  repeat
    if FActualQTree = nil then break;
    if FIndex >= FActualQTree.Count then
    begin
      Step; continue
    end;

    ActEl:= FActualQTree.Elements[FIndex];
    if ActEl is TQtree then
    begin
      Step; continue
    end
    else
    begin
      if FActualQTree.ElementIsOverRectangle
         (ActEl, FXmin-FTolerance, FYmin-FTolerance, FXmax+FTolerance, FYmax+FTolerance) then
      begin
        result:= ActEl;
        inc(FIndex);
        exit
      end
      else
      begin
        Step; continue
      end
    end
  until false;
end;

{ TSpaceRect }

constructor TSpaceRect.Create;
begin
  inherited Create; { zahada}
  FIsLast:= true
end;


procedure TSpaceRect.PrepareQTree;
begin
  HX:= (maxX+minX)/2;
  HY:= (maxY+minY)/2;
  FLUQ:= TSpaceRect(TElementClass(self.ClassType).Create);
  FLBQ:= TSpaceRect(TElementClass(self.ClassType).Create);
  FRUQ:= TSpaceRect(TElementClass(self.ClassType).Create);
  FRBQ:= TSpaceRect(TElementClass(self.ClassType).Create);
  FLUQ.FIsLast:= true;
  FLBQ.FIsLast:= true;
  FRUQ.FIsLast:= true;
  FRBQ.FIsLast:= true;

  if (FLBQ<>nil) and (FLUQ<>nil) and (FRBQ<>nil) and (FRUQ<>nil) then
  begin
    FLUQ.Rect.minX:= Self.MinX;
    FLUQ.Rect.maxX:= HX;
    FLUQ.Rect.MinY:= HY;
    FLUQ.Rect.maxY:= Self.maxY;
    FLBQ.Rect.minX:= Self.MinX;
    FLBQ.Rect.maxX:= HX;
    FLBQ.Rect.MinY:= Self.MinY;
    FLBQ.Rect.maxY:= HY;
    FRUQ.Rect.minX:= HX;
    FRUQ.Rect.maxX:= Self.MaxX;
    FRUQ.Rect.MinY:= HY;
    FRUQ.Rect.maxY:= Self.maxY;
    FRBQ.Rect.minX:= HX;
    FRBQ.Rect.maxX:= Self.MaxX;
    FRBQ.Rect.MinY:= Self.MinY;
    FRBQ.Rect.maxY:= HY;

    FIsLast:= false;

    FLBQ.Name:= 'LEVY DOLNI';
    FLUQ.Name:= 'LEVY HORNI';
    FRBQ.Name:= 'PRAVY DOLNI';
    FRUQ.Name:= 'PRAVY HORNI';
    AddElement(FLBQ);
    AddElement(FLUQ);
    AddElement(FRBQ);
    AddElement(FRUQ);
  end
end;




                         { TSpaceLineBegEndCell }


constructor TSpaceLineBegEndCell.Create;
begin
  inherited Create;
  FIsLast:= true  { zahada}
end;


procedure TSpaceLineBegEndCell.AddToQTree
          (Line: TPolyLineElement; aX, aY: double; aBegin: boolean);
begin
  if (aX < HX) then
  begin
    if aY < HY then
      TSpaceLineBegEndCell(FLBQ).AddLineXY(Line, aX, aY, aBegin)
    else
      TSpaceLineBegEndCell(FLUQ).AddLineXY(Line, aX, aY, aBegin)
  end
  else
  begin
    if aY < HY then
      TSpaceLineBegEndCell(FRBQ).AddLineXY(Line, aX, aY, aBegin)
    else
      TSpaceLineBegEndCell(FRUQ).AddLineXY(Line, aX, aY, aBegin)
  end
end;


procedure TSpaceLineBegEndCell.AddToQTreeDirectly
          (Line: TPolyLineElement; aX, aY: double; aBegin: boolean);
begin
  if (aX < HX) then
  begin
    if aY < HY then
      TSpaceLineBegEndCell(FLBQ).AddBegEnd(Line, aBegin)
    else
      TSpaceLineBegEndCell(FLUQ).AddBegEnd(Line, aBegin)
  end
  else
  begin
    if aY < HY then
      TSpaceLineBegEndCell(FRBQ).AddLineXY(Line, aX, aY, aBegin)
    else
      TSpaceLineBegEndCell(FRUQ).AddLineXY(Line, aX, aY, aBegin)
  end
end;


procedure TSpaceLineBegEndCell.AddBegEnd(Line: TPolyLineElement; aBegin: boolean);
var
  Lines: TPolyLineString;
begin
  if aBegin then Lines:= FLineBegins
  else Lines:= FLineEnds;

  if Lines = nil then
  begin
    Lines:= TPolyLineString.Create;

    if aBegin then Lines.Name:= 'BEGINS'
    else Lines.Name:= 'ENDS';

    AddElement(Lines);
  end;

  if Lines <> nil then
  begin
    Lines.AddAndUpdateRect(Line)
  end;

  if aBegin then FLineBegins:= Lines
  else FLineEnds:= Lines;
end;


procedure TSpaceLineBegEndCell.MoveLineToQTree(Lin: TPolyLineElement; aBegin: boolean);
var
  aX, aY: double;
begin
  Lin.GetOrientedCoord(0, aBegin, aX, aY);
  AddToQTreeDirectly(Lin, aX, aY, aBegin)
end;


procedure TSpaceLineBegEndCell.MoveToQTree;
var
  I: longInt;
begin
  if FLineBegins <> nil then
  begin
    for I:= 0 to FLineBegins.Count-1 do
        MoveLineToQTree(TPolyLineElement(FLineBegins.Elements[I]), true);
{    FLineBegins.Clear; to by se pak linie nezbavily tohoto vlastnika }
    FreeElement(FLineBegins);
    FLineBegins:= nil
  end;

  if FLineEnds <> nil then
  begin
    for I:= 0 to FLineEnds.Count-1 do
        MoveLineToQTree(TPolyLineElement(FLineEnds.Elements[I]), false);
{    FLineEnds.Clear; }

    FreeElement(FLineEnds);
    FLineEnds:= nil
  end
end;


function TSpaceLineBegEndCell.GetCount: longInt;
var
  Sum: longInt;
begin
  Sum:= 0;
  if not FIsLast then
  begin
    Sum:= Sum+FLBQ.GetCount;
    Sum:= Sum+FLUQ.GetCount;
    Sum:= Sum+FRBQ.GetCount;
    Sum:= Sum+FRUQ.GetCount
  end
  else
  begin
    if FLineBegins <> nil then Sum:= Sum + FLineBegins.Count;
    if FLineEnds <> nil then Sum:= Sum + FLineEnds.Count;
  end;
  result:= Sum
end;


procedure TSpaceLineBegEndCell.AddLineXY
          (Lin: TPolyLineElement; aX, aY: double; aBegin: boolean);
begin
  if not FIsLast then
     AddToQTree(Lin, aX, aY, aBegin)
  else
  begin
    if GetCount < MaxCountInCell then
    begin
      AddBegEnd(Lin, aBegin)
    end
    else
    begin
      PrepareQTree;
      MoveToQTree;
      AddToQTree(Lin, aX, aY, aBegin)
    end
  end
end;





procedure TSpaceLineBegEndCell.AddLine
          (Lin: TPolyLineElement; aBegin: boolean);
var
  aX, aY: double;
begin
try
  Lin.GetOrientedCoord(0, aBegin, aX, aY);
  AddLineXY(Lin, aX, aY, aBegin);
except
  ShowMessage('Error: TSpaceLineBegEndCell.AddLine')
end;
end;

procedure TQTree.HandleMessage(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_DivideGeometry: on_tm_DivideGeometry(aMessage, Handled);
   end;
end;

procedure TQTree.on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean);
var
   R: TRectEntity;
begin
   //Handled := true;
   R := TRectEntity(aMessage.Data);
   if R = nil then exit;
   AddRectElement(R,R.Minx,R.MinY,R.MaxX,R.MaxY);
end;

procedure TQTree.SetQMinWidth(const Value: integer);
begin
  FQMinWidth := Value;
end;

procedure TQTree.SetMaxLeafCount(const Value: integer);
begin
  FMaxLeafCount := Value;
end;


procedure TRectEntityQTree.AddDirectlyEx(Element: TElement);
var
   RE: TRectEntity;
begin
   if not (Element is TRectEntity) then
      exit;
   RE := TRectEntity(Element);
   if RE.IsOverRegionEx(MinX,MinY,MaxX,MaxY) then
      AddElement(RE);
      //Add(RE);
end;

INITIALIZATION
   RegisterOrbisClass(TQTree);
   RegisterOrbisClass(TRectEntityQTree);
   RegisterOrbisClass(TSpaceLineBegEndCell);
END.
