{$D+}
unit E_Reg;

interface
uses
  DBTables, DB,
// KERNEL
  Elements,
  e_DataSet,
  E_GeometryElement,
  E_Type,
  G_Elems,
  Tools;
// SPACE
  //E_Space;

const
  MaxObjectsCount = 500000;

type


TRegisteredGeometryElement = class;

TObjectOwner = class(TOwnerEntity)
public
  function   GetPropertyStringByName(AName: string): string; override;
  procedure  on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean);
  procedure  HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
end;


TObjectRegister = class(TRectEntity)
protected
  FDBF: TDataSetElement;
  function GetBlockName: string;
{ TODO -oO.S. -cZmeny pro ADO : 11.11.99 }
//  function GetDBF: TTable; override;
  function GetDBF: TDataSetElement; virtual;
  function CreateObjectOwner: TObjectOwner; virtual;
public
  procedure RegisterObject(aObject: TGeometryElement);
  function  GetObject(aCislo: longInt): TGeometryElement;
  procedure SetObject(aCislo: longInt; aObject: TGeometryElement);
  procedure RemoveObject(aCislo: longInt);

  property  Objects[Index: longint]: TGeometryElement read GetObject write SetObject;

  destructor Destroy; override;
published
  property BlockName: string read GetBlockName stored false;
{ TODO -oO.S. -cZmeny pro ADO : 11.11.99 }
//  property  DBF: TTable read GetDBF;
  property  DBF: TDataSetElement read GetDBF stored false;
end;


TBlockRegister = class(TRectEntity)
public
  procedure RegisterLogicObject(aObj: TGeometryElement); virtual;
  procedure RegisterObject(aObj: TGeometryElement); virtual;
  function GetObjectRegister(aType: char): TObjectRegister; virtual;
  function GetObject(aType: char; aCislo: longInt): TGeometryElement; virtual;
  procedure RemoveObject(aType: char; aCislo: longInt); virtual;
end;



TRegister = class(TRectEntity)
public
  function GetBlockRegister(aName: string): TBlockRegister; virtual;
  function GetAndCreateBlockRegister(aName, aBlockRegType: string): TBlockRegister; virtual;
end;


TRegisterObjectSelector = class(TTool)
private
  FBlockName: string;
  FObjectNo: longInt;
  FObjectType: string;
public
  procedure DoIt; override;
published
  property BlockName: string  read FBlockName write FBlockName;
  property ObjectNo: longInt  read FObjectNo write FObjectNo;
  property ObjectType: string  read FObjectType write FObjectType;
end;



TRegisteredGeometryElement = class(TPropertyGeometryElement)
private
   FDone: boolean;
protected
   function   GetObjectRegister: TObjectRegister; virtual;
   function   GetBlockRegister: TBlockRegister; virtual;
   function   GetOrigElement: TRegisteredGeometryElement; virtual;
   function   GetOrigNo: longInt; virtual;
   function   GetBlockName: string; virtual;
public
   function   HasDBFItem(aItem: string): boolean; virtual;
   function   GetDBFFieldAsString(aItem: string): string; virtual;
   procedure  SetDBFFieldAsString(aItem, aValue: string); virtual;
   function   SetDBFRecord: boolean; virtual;
   function   HasPropertyByName(aName: string): boolean; override;
   procedure  SetPropertyStringByName(AName: string; aValue: string); override;
   function   GetPropertyStringByName(AName: string): string; override;
   function   CreateDuplicate: TElement; virtual;
published
   property   OrigNo:   longInt read GetOrigNo stored false;
   property   IsDone: boolean   read FDone     write FDone default false;
   property   ObjectRegister: TObjectRegister read GetObjectRegister stored false;
   property   BlockRegister: TBlockRegister read GetBlockRegister stored false;
   property   BlockName: string read GetBlockName stored false;
end;




function GetDBFNameForObject(BlockDir: string; ObjType: char; Druh: smallInt): string;



implementation
uses
  SysUtils, Forms, WinProcs,
// KERNEL
  e_class,
  e_Geometry,
  M_Orbis,
  u_Props,
  Worlds;

function GetDBFNameForObject(BlockDir: string; ObjType: char; Druh: smallInt): string;
begin
  Result:= BlockDir + '\' + ObjType + Format('%4.4d', [Druh]) + '.DBF';
end;




               { TObjectOwner }




function TObjectOwner.GetPropertyStringByName(AName: string): string;
begin
  result:= inherited GetPropertyStringByName(aName);
  if result <> '' then exit;
  if (Count > 0) and (Elements[0] <> nil) then
  try
     result:= Elements[0].GetPropertyStringByName(aName)
  except
    MessageBeep(0)
  end
end;



procedure TObjectOwner.on_tm_DivideGeometry
         (var aMessage: TOrbisMessage; var Handled: boolean);
begin
  AddElement(aMessage.Data)
end;



procedure   TObjectOwner.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_DivideGeometry: on_tm_DivideGeometry(aMessage, Handled);
   end;
end;








                    { TObjectRegister }


destructor TObjectRegister.Destroy;
begin
  if FDBF <> nil then
     FDBF.Free;
  inherited Destroy
end;


procedure TObjectRegister.RegisterObject(aObject: TGeometryElement);
var
  aCislo: longInt;
begin
  if aObject = nil then exit;

  aCislo:= aObject.SpaceId;
  if (aCislo < 1) or (aCislo > MaxObjectsCount) then exit;

  SetObject(aCislo, aObject);
end;



function TObjectRegister.CreateObjectOwner: TObjectOwner;
begin
  result:= TObjectOwner.Create;
end;


procedure TObjectRegister.SetObject(aCislo: longInt; aObject: TGeometryElement);
var
  I: longInt;
  ObjOwn: TObjectOwner;
begin
  if aObject = nil then exit;
  ObjOwn:= nil;

  if aCislo < count then
  begin
    ObjOwn:= TObjectOwner(Elements[aCislo]);
    ObjOwn.AddElement(aObject);
  end
  else
  begin
    for I:= count to aCislo do
    begin
      ObjOwn:= CreateObjectOwner;
      AddElement(ObjOwn)
    end;
    if ObjOwn <> nil then
       ObjOwn.AddElement(aObject)
  end;
  UpDateRect(aObject);
end;




function TObjectRegister.GetObject(aCislo: longInt): TGeometryElement;
var
  ObjOwn: TObjectOwner;
begin
  result:= nil;
  if aCislo < 1 then exit;
  if Count <= aCislo then exit;
  ObjOwn:= TObjectOwner(Elements[aCislo]);
  if ObjOwn = nil then
     exit;
  if ObjOwn.Count = 0 then
     exit;
  result:= TGeometryElement(ObjOwn.Elements[0]);
  if TElement(result) is TGeometryElement then exit;
  result:= nil
end;




procedure TObjectRegister.RemoveObject(aCislo: longInt);
var
  ObjOwn: TObjectOwner;
begin
  ObjOwn:= TObjectOwner(Elements[aCislo]);
  if not (TElement(ObjOwn) is TObjectOwner) then
     exit;
  ObjOwn.freeAll;
end;



function TObjectRegister.GetBlockName: string;
var
  aOwner: TBlockRegister;
begin
  result:= '';
  aOwner:= TBlockRegister(findOwnerClass(TBlockRegister));
  if not (TElement(aOwner) is TBlockRegister) then
     exit;
  result:= aOwner.Name;
end;




function TObjectRegister.GetDBF: TDataSetElement;
begin
  result:= nil;
end;



                      {  TBlockRegister }



procedure TBlockRegister.RegisterLogicObject(aObj: TGeometryElement);
var
  aType: char;
  OE: TObjectRegister;
begin
  aType:= 'X';
  if aObj.Geometry is TPointElement then aType:= 'B';
  if aObj.Geometry is TTextElement then aType:= 'T';
  if aObj.Geometry is TPolyLineElement then aType:= 'L';
  if aObj.Geometry is TAreaElement then aType:= 'P';
  OE:= GetObjectRegister(aType);
  if OE = nil then exit;
  OE.RegisterObject(aObj);
  UpDateRect(aObj);
end;



procedure TBlockRegister.RegisterObject(aObj: TGeometryElement);
var
  aType: char;
  OE: TObjectRegister;
begin
  if not (TElement(aObj) is TGeometryElement) then exit;

  if aObj.Geometry is TPointElement then aType:= 'B'
  else
  if aObj.Geometry is TTextElement then aType:= 'T'
  else
  if aObj.Geometry is TPolyLineElement then aType:= 'L'
  else
  if aObj.Geometry is TAreaElement then aType:= 'P'
  else
     aType:= 'X';
  OE:= GetObjectRegister(aType);
  if OE = nil then exit;
  OE.RegisterObject(aObj);
  UpDateRect(aObj);
end;



function TBlockRegister.GetObjectRegister(aType: char): TObjectRegister;
begin
  result:= TObjectRegister(FindElementName(aType));
  if result <> nil then exit;
  result:= TObjectRegister.Create;
  result.Name:= aType;
  AddElement(result)
end;



function TBlockRegister.GetObject(aType: char; aCislo: longInt): TGeometryElement;
var
  TOR: TObjectRegister;
begin
  result:= nil;
  TOR:= GetObjectRegister(aType);
  if TOR = nil then exit;
  result:= TOR.GetObject(aCislo);
end;



procedure TBlockRegister.RemoveObject(aType: char; aCislo: longInt);
var
  TOR: TObjectRegister;
begin
  TOR:= GetObjectRegister(aType);
  if TOR = nil then exit;
  TOR.RemoveObject(aCislo)
end;

                  { TTopolRegister }


function TRegister.GetBlockRegister(aName: string): TBlockRegister;
begin
  result:= nil;
  if aName = '' then exit;
  aName:= UpperCase(aName);
  result:= TBlockRegister(findElementName(UpperCase(aName)));
end;



function TRegister.GetAndCreateBlockRegister(aName, aBlockRegType: string): TBlockRegister;
begin
  result:= GetBlockRegister(aName);
  if result <> nil then exit;

  result:= TBlockRegister(CreateElementByName(aBlockRegType));
  if not (TElement(result) is TBlockRegister) then
  begin
    result:= nil;
    exit;
  end;
  result.Name:= aName;
  AddElement(result)
end;




                   { TRegisterObjectSelector }

procedure TRegisterObjectSelector.DoIt;
var
  TR: TRegister;
  BR: TBlockRegister;
begin
  TR:= TRegister(World.ObjectModel.FindElementClass(TRegister));
  if TR = nil then
     exit;

  BR:= TR.GetBlockRegister(BlockName);

  if BR = nil then
     exit;

  Output:= BR.GetObject(ObjectType[1], ObjectNo);
end;

{ TRegisteredGeometryElement }
function  TRegisteredGeometryElement.GetObjectRegister: TObjectRegister;
var
  ObjOwn: TObjectOwner;
begin
  result:= nil;
  ObjOwn:= TObjectOwner(FindOwnerClass(TObjectOwner));
  if ObjOwn <> nil then
     result:= TObjectRegister(ObjOwn.FindOwnerClass(TObjectRegister))
end;



function   TRegisteredGeometryElement.GetBlockRegister: TBlockRegister;
var
  ObjReg: TObjectRegister;
begin
  result:= nil;
  ObjReg:= ObjectRegister;
  if ObjReg = nil then exit;
  result:= TBlockRegister(ObjReg.FindOwnerClass(TBlockRegister));
end;

{
function  TRegisteredGeometryElement.GetOrigProps: TSpaceEntityProps;
var
  ObjOwn: TObjectOwner;
  origSE: TSpaceEntity;
begin
  result:= nil;
  ObjOwn:= TObjectOwner(FindOwnerClass(TObjectOwner));
  if ObjOwn = nil then exit;

  if ObjOwn.Count = 1 then
     result:= self
  else
  begin
    origSE:= TSpaceEntity(ObjOwn.Elements[0]);
    if not (TElement(origSE) is TSpaceEntity) then exit;
    result:= TSpaceEntityProps(origSE.Properties)
  end
end;

}

function  TRegisteredGeometryElement.GetOrigNo: longInt;
var
  aReg: TObjectRegister;
  ObjOwn: TObjectOwner;
begin
  result:= -1;

  ObjOwn:= TObjectOwner(FindOwnerClass(TObjectOwner));
  if ObjOwn = nil then exit;
  aReg:= TObjectRegister(ObjOwn.FindOwnerClass(TObjectRegister));
  if aReg = nil then exit;
  result:= aReg.IndexOf(ObjOwn);
end;



function  TRegisteredGeometryElement.GetBlockName: string;
begin
  result:= '';
  if not (TElement(ObjectRegister) is TObjectRegister) then exit;

  result:= TObjectRegister(ObjectRegister).BlockName;
end;



function TRegisteredGeometryElement.SetDBFRecord: boolean;
begin
  result:= false
end;


function TRegisteredGeometryElement.HasDBFItem(aItem: string): boolean;
begin
  result:= false
end;


function  TRegisteredGeometryElement.HasPropertyByName(aName: string): boolean;
begin
  result:= inherited HasPropertyByName(aName);
  if result then exit;

  try
    if ObjectRegister = nil then exit;
    result:= HasDBFItem(aName)
  except
     result := false;
  end;
end;





function TRegisteredGeometryElement.GetDBFFieldAsString(aItem: string): string;
begin
  result:= ''
end;


procedure TRegisteredGeometryElement.SetDBFFieldAsString(aItem, aValue: string);
begin
end;

function   TRegisteredGeometryElement.GetPropertyStringByName(AName: string): string;
begin
  result:= inherited GetPropertyStringByName(aName);
  if result <> '' then exit;

  try
    if ObjectRegister = nil then
       exit;
    if SetDBFRecord then
       result:= GetDBFFieldAsString(aName);
  except
     result := '';
  end;
end;



procedure   TRegisteredGeometryElement.SetPropertyStringByName(AName: string; aValue: string);
begin
  //if GetPropIndex(Self, aName) > -1 then
  if inherited HasPropertyByName(aName) then
  begin
     inherited SetPropertyStringByName(aName, aValue);
     exit;
  end;
  try
    if ObjectRegister = nil then exit;
    if SetDBFRecord then
       SetDBFFieldAsString(aName, aValue);
{    TTopolDBFTable(TTopolObjectRegister(Register).DBF).FieldAsString(aName); }
  except
  end;
end;




function TRegisteredGeometryElement.CreateDuplicate: TElement;
var
  I, aStart: longInt;
begin
  Result:= TElement(TElementClass(self.ClassType).Create);

  aStart:= GetPropIndex(Self,'LastIndex')+1;

  for I:= aStart to GetPropCount(self)-1 do { misto startovani od nuly }
  begin
    SetPropString(Result, I, GetPropString(self, I))
    {!!!!!!!!!!!!!!!!}
  end;
end;





function TRegisteredGeometryElement.GetOrigElement: TRegisteredGeometryElement;
var
  ObjOwn: TObjectOwner;
  origSE: TRegisteredGeometryElement;
begin
  result:= nil;
  ObjOwn:= TObjectOwner(FindOwnerClass(TObjectOwner));
  if ObjOwn = nil then exit;

  if ObjOwn.Count = 1 then
     result:= self
  else
  begin
    origSE:= TRegisteredGeometryElement(ObjOwn.Elements[0]);
    if not (TElement(origSE) is TRegisteredGeometryElement) then exit;
    result:= origSE;
  end
end;

initialization
   RegisterOrbisClass(TBlockRegister);
   RegisterOrbisClass(TRegister);
   RegisterOrbisClass(TRegisterObjectSelector);
   RegisterOrbisClass(TRegisteredGeometryElement);
end.

