unit e_GeometryElement;

interface
uses
   Fmx.Forms,
   Elements,
   e_Geometry,
   G_Elems;

type


TGeometryElement = class(TRectEntity)
private
   FGeometry: TGeometry;
   procedure SetGeometry(const Value: TGeometry);
   function GetGeometry: TGeometry;
   function GetNoGeometry: boolean;
    function GetAngle: double;
    procedure SetAngle(const Value: double);
    function GetGIndex: integer;
    procedure SetGIndex(const Value: integer);
protected
   function GetSpaceID: longInt; virtual;
   function GetGroup: integer; virtual;
   procedure SetGroup(const Value: integer); virtual;
    function GetMultiElement: boolean; virtual;
    function GetMultiElementClass: TElementClass; virtual;
public
   function   FindElementName(aName: string): TElement; override;
   function   GetRect: TRectangle; override;
   procedure  SetRect(aRect: TRectangle); override;
   function   GetRefCoordinate: TCoordinate; override;
   procedure  SetRefCoordinate(aValue: TCoordinate); override;
   procedure  on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  AfterCopy(NewElement: TGeometryElement); virtual;
   procedure  HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure  RemoveElement(aElement: TElement); override;
   procedure  Update; override;
   function   IsOverRegionEx(Minx, MinY, MaxX, MaxY: double): Boolean; override;
   procedure  AddGeometry(Value: TGeometry); virtual;
   function   IsHit(CoordEl: TCoordinate; Tolerance: double): boolean; override;
   function   IsHitByXY(aX, aY: double; Tolerance: double): boolean; override;
   function   GetSimpleGeometryByXY(aX,aY,Tolerance: double): TGeometry; virtual;
   property   MultiElementClass: TElementClass read GetMultiElementClass;
published
   property SpaceID:  longInt read GetSpaceId stored false;
   property Group: integer read GetGroup write SetGroup stored false;
   property Geometry: TGeometry read GetGeometry write SetGeometry stored false;
   property NextGeometry: TGeometry write AddGeometry stored false;
   property NoGeometry: boolean read GetNoGeometry stored false;
   property MultiElement: boolean read GetMultiElement stored false;
   property MinX stored GetNoGeometry;
   property MinY stored GetNoGeometry;
   property Width stored GetNoGeometry;
   property Height stored GetNoGeometry;
   property AbsRefX stored GetNoGeometry;
   property AbsRefY stored GetNoGeometry;
   property Angle: double read GetAngle write SetAngle stored false;
   property GIndex: integer read GetGIndex write SetGIndex stored false;
end;

TPropertyGeometryElement = class(TGeometryElement)
private
    FProperties: TElement;
    procedure SetProperties(const Value: TElement);
    function GetProperties: TElement; // asi by se mly ruit
public
   procedure  AfterCopy(NewElement: TGeometryElement); override;
   function   HasPropertyByName(aName: string): boolean; override;
   function   GetPropertyStringByName(AName: string): string; override;
   procedure  SetPropertyStringByName(AName: string; aValue: string); override;
   function   NoProperties: boolean;
published
   property Properties: TElement read GetProperties write SetProperties stored false;
end;

TGeometryElementCollection = class(TRectEntity)
private
   FUseColor: boolean;
   FUseSymbol: boolean;
   FColor: integer;
   FSymbol: integer;
   procedure SetColor(const Value: integer);
   procedure SetSymbol(const Value: integer);
   procedure SetUseColor(const Value: boolean);
   procedure SetUseSymbol(const Value: boolean);
public
   procedure  on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
published
   property Color: integer read FColor write SetColor;
   property UseColor: boolean read FUseColor write SetUseColor;
   property Symbol: integer read FSymbol write SetSymbol;
   property UseSymbol: boolean read FUseSymbol write SetUseSymbol;
end;

TComplicatedGECollection = class(TRectEntity)
  private
    FProperties: TElement;
    FEnabled: boolean;
    FText: string;
    FObjectType: string;
    FSymbol: integer;
    FUseSymbol: integer;
    FColor: integer;
    FUseColor: boolean;
    FVisible: boolean;
    procedure SetEnabled(const Value: boolean);
    procedure SetText(const Value: string);
    procedure SetObjectType(const Value: string);
    procedure SetColor(const Value: integer);
    procedure SetSymbol(const Value: integer);
    procedure SetUseColor(const Value: boolean);
    procedure SetUseSymbol(const Value: integer);
    procedure SetVisible(const Value: boolean);
    function GetLayer(aName: string): TComplicatedGECollection;
    procedure SetLayer(aName: string; const Value: TComplicatedGECollection);
    function GetLayers(aIndex: integer): TComplicatedGECollection;
    function GetProperties: TElement;
    procedure SetProperties(const Value: TElement);
public
   procedure CheckLayer(aName:string);
   procedure CHeckProperties;
   property  Layer[aName: string]: TComplicatedGECollection read GetLayer write SetLayer;
   property  Layers[aIndex: integer]: TComplicatedGECollection read GetLayers;
   function  NoProperties: boolean;
published
   property Properties: TElement read GetProperties write SetProperties stored false;
   property Color: integer read FColor write SetColor;
   property UseColor: boolean read FUseColor write SetUseColor;
   property Symbol: integer read FSymbol write SetSymbol;
   property UseSymbol: integer read FUseSymbol write SetUseSymbol;
   property Text: string read FText write SetText;
   property Visible: boolean read FVisible write SetVisible;
   property Enabled: boolean read FEnabled write SetEnabled;
   property ObjectType: string read FObjectType write SetObjectType;
end;

TPointCollection = class(TGeometryElementCollection);
TPolyLineCollection = class(TGeometryElementCollection);
TTextCollection = class(TGeometryElementCollection);
TAreaCollection = class(TGeometryElementCollection);

implementation
uses
   SysUtils,
   e_Class,
   e_Props,
   m_Orbis,
   u_Props;

const
   cstr_GEOMETRY: string = 'GEOMETRY';

{ TGeometryElement }

function  TGeometryElement.GetSpaceID: longInt;
begin
  result:= -1
end;


function TGeometryElement.GetGeometry: TGeometry;
begin
   if NoGeometry then;
   {
   begin
      FGeometry := TRectEntity.Create;
      FGeometry.Name := cstr_GEOMETRY;
      AddElement(FGeometry);
   end;
   }
   result := FGeometry;
end;

function TGeometryElement.GetNoGeometry: boolean;
begin
   if FGeometry = nil then
   begin
      FGeometry := TGeometry(FindElementClass(TGeometry));
   end;
   result := FGeometry = nil;
end;

procedure TGeometryElement.SetGeometry(const Value: TGeometry);
begin
   if not NoGeometry then
   begin
      FreeElement(FGeometry);
      FGeometry := nil;
   end;
   if Value = nil then exit;
   Rect := nil;
   RefCoords := nil;
   FGeometry := Value;
   AddElement(FGeometry);
end;

procedure TGeometryElement.RemoveElement(aElement: TElement);
begin
   if aElement = Geometry then
   begin
      FGeometry := nil;
      //Rect.Reset;
   end;
   inherited RemoveElement(aElement);
end;

procedure TGeometryElement.HandleMessage(var aMessage: TOrbisMessage;var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_DivideGeometry: on_tm_DivideGeometry(aMessage, Handled);
   end;
end;

procedure TGeometryElement.on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean);
var
   OrigGeometry, NewGeometry: TGeometry;
   NewElement: TGeometryElement;
begin
   OrigGeometry:= TGeometry(aMessage.Sender);
   NewGeometry:= TGeometry(aMessage.Data);
   if not (OrigGeometry = Geometry) then
      Exit;
   if not (NewGeometry is TGeometry) then
      Exit;
   NewElement := TGeometryElement(CreateElementCopy(Self));
   NewElement.Geometry := NewGeometry;
   AfterCopy(NewElement); // pro budouci kopii propert a tak
   PostMsgToOwners(tm_DivideGeometry,NewElement,false);
   Application.ProcessMessages;
end;

procedure TGeometryElement.Update;
begin
   if NoGeometry then
      inherited;
end;

function TGeometryElement.GetGroup: integer;
begin
   result := -1;
end;

procedure TGeometryElement.SetGroup(const Value: integer);
begin

end;

function TGeometryElement.GetMultiElement: boolean;
begin
   result := false;
end;

function TGeometryElement.GetMultiElementClass: TElementClass;
begin
   result := nil;
end;

procedure TGeometryElement.AddGeometry(Value: TGeometry);
var
   MG: TGeometry;
   i: integer;
begin
   if NoGeometry or not MultiElement then
   begin
      Geometry := Value;
   end
   else
   begin
      if FGeometry.ClassType <> MultiElementClass then
      begin
         MG := TGeometry(MultiElementClass.Create);
         MG.AddAndUpdateRect(FGeometry);
         MG.AbsRefX := FGeometry.AbsRefX;
         MG.AbsRefY := FGeometry.AbsRefY;
         Geometry := MG;
      end;
      if Value.ClassType = MultiElementClass then
      begin
         for i := 0 to Value.LastIndex do
         begin
            FGeometry.AddAndUpdateRect(TGeometry(Value.Elements[i]));
         end;
      end
      else
         FGeometry.AddAndUpdateRect(Value);
      Update;
   end;
end;

function TGeometryElement.IsHit(CoordEl: TCoordinate; Tolerance: double): boolean;
begin
   result := false;
   if NoGeometry then exit;
   result := Geometry.IsHit(CoordEl, Tolerance);
end;

function TGeometryElement.IsHitByXY(aX, aY, Tolerance: double): boolean;
begin
   result := false;
   if NoGeometry then exit;
   result := Geometry.IsHitByXY(aX, aY, Tolerance);
end;

function TGeometryElement.GetSimpleGeometryByXY(aX, aY, Tolerance: double): TGeometry;
begin
   result := nil;
   if NoGeometry then exit;
   result := Geometry.GetSimpleGeometryByXY(aX, aY, Tolerance);
end;

function TGeometryElement.GetRect: TRectangle;
begin
   if NoGeometry then
      result := inherited GetRect
   else
      result := Geometry.GetRect;
end;

function TGeometryElement.GetRefCoordinate: TCoordinate;
begin
   if NoGeometry then
      result := inherited GetRefCoordinate
   else
      result := Geometry.GetRefCoordinate;
end;

procedure TGeometryElement.SetRect(aRect: TRectangle);
begin
   if NoGeometry then
      inherited SetRect(aRect)
   else
      Geometry.SetRect(aRect);
end;

procedure TGeometryElement.SetRefCoordinate(aValue: TCoordinate);
begin
   if NoGeometry then
      inherited SetRefCoordinate(aValue)
   else
      Geometry.SetRefCoordinate(aValue);
end;

function TGeometryElement.FindElementName(aName: string): TElement;
begin
   if AnsiSameText(aName,'GEOMETRY') then
      result := Geometry
   else
      result := inherited FindElementName(aName);
end;

procedure TGeometryElement.AfterCopy(NewElement: TGeometryElement);
begin

end;

function TGeometryElement.GetAngle: double;
begin
   result := 0;
   if NoGeometry then exit;
   result := Geometry.Angle;
end;

procedure TGeometryElement.SetAngle(const Value: double);
begin
   if NoGeometry then exit;
   Geometry.Angle := Value;
end;

function TGeometryElement.IsOverRegionEx(Minx, MinY, MaxX, MaxY: double): Boolean;
begin
   if NoGeometry then
   begin
      result := inherited IsOverRegionEx(Minx, MinY, MaxX, MaxY);
      exit;
   end;
   result := Geometry.IsOverRegionEx(Minx, MinY, MaxX, MaxY);
end;

function TGeometryElement.GetGIndex: integer;
begin
   result := 0;
   if NoGeometry then exit;
   result := Geometry.GIndex;
end;

procedure TGeometryElement.SetGIndex(const Value: integer);
begin
   if NoGeometry then exit;
   Geometry.GIndex := Value;
end;

{ TGeometryElementCollection }


procedure TGeometryElementCollection.HandleMessage(
  var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_DivideGeometry: on_tm_DivideGeometry(aMessage, Handled);
   end;
end;

procedure TGeometryElementCollection.on_tm_DivideGeometry(
  var aMessage: TOrbisMessage; var Handled: boolean);
begin
   AddElement(aMessage.Data)
end;

procedure TGeometryElementCollection.SetColor(const Value: integer);
begin
  FColor := Value;
end;

procedure TGeometryElementCollection.SetSymbol(const Value: integer);
begin
  FSymbol := Value;
end;

procedure TGeometryElementCollection.SetUseColor(const Value: boolean);
begin
  FUseColor := Value;
end;

procedure TGeometryElementCollection.SetUseSymbol(const Value: boolean);
begin
  FUseSymbol := Value;
end;


{ TPropertyGeometryElement }

procedure TPropertyGeometryElement.AfterCopy(NewElement: TGeometryElement);
var
   e: TElement;
   P: TDynamicProperties;
   P1: TDynamicProperties;
   PGE: TPropertyGeometryElement;
begin
   if not (NewElement is TPropertyGeometryElement) then exit;
   PGE := TPropertyGeometryElement(NewElement);
   if NoProperties then exit;
   e := Properties;
   if not (e is TDynamicProperties) then exit;
   P := TDynamicProperties(e);
   P1 := P.MakeSelfCopy;
   PGE.Properties := P1;
end;

function TPropertyGeometryElement.GetProperties: TElement;
begin
   if NoProperties then;
   result := FProperties;
end;

function TPropertyGeometryElement.GetPropertyStringByName(
  AName: string): string;
begin
   result:= inherited GetPropertyStringByName(aName);
   if result <> '' then exit;
   if Properties = nil then exit;
   result:= Properties.GetPropertyStringByName(AName)
end;

function TPropertyGeometryElement.HasPropertyByName(
  aName: string): boolean;
begin
   result:= inherited HasPropertyByName(aName);
   if result then exit;
   if NoProperties then exit;
   result:= Properties.HasPropertyByName(aName);
end;

function TPropertyGeometryElement.NoProperties: boolean;
begin
   if FProperties = nil then
   begin
      FProperties := TDynamicProperties(FindElementClass(TDynamicProperties));
   end;
   result := FProperties = nil;
end;

procedure TPropertyGeometryElement.SetProperties(const Value: TElement);
begin
   NoProperties;
   if FProperties = Value then exit;
   ExchangeFieldElement(FProperties,Value);
end;

procedure TPropertyGeometryElement.SetPropertyStringByName(AName,
  aValue: string);
begin
   if GetPropIndex(Self, aName) > -1 then
      inherited SetPropertyStringByName(aName, aValue)
   else
   begin
     if Properties = nil then exit;
     Properties.SetPropertyStringByName(AName, aValue)
   end
end;

{ TComplicatedGECollection }

procedure TComplicatedGECollection.CheckLayer(aName: string);
begin
   GetLayer(aName);
end;

procedure TComplicatedGECollection.CHeckProperties;
begin
   if Properties = nil then
      Properties := TDynamicProperties.Create;
end;

function TComplicatedGECollection.GetLayer(aName: string): TComplicatedGECollection;
var
   e: TElement;

   function CreateIt: TComplicatedGECollection;
   begin
      result := TComplicatedGECollection.Create;
      AddElement(result);
      result.Name := aName;
      result.ObjectType := UpperCase(aName[1]);
   end;
begin
   e := FindElementName(aName);
   if e = nil then
      result := CreateIt
   else
   begin
      if not (e is TComplicatedGECollection) then
      begin
         FreeElement(e);
         result := CreateIt;
      end
      else
         result := TComplicatedGECollection(e);
   end;
end;

function TComplicatedGECollection.GetLayers(aIndex: integer): TComplicatedGECollection;
var
   e: TElement;
begin
   e := Elements[aIndex];
   result := TComplicatedGECollection(e);
end;

function TComplicatedGECollection.GetProperties: TElement;
begin
   if NoProperties then;
   result := FProperties;
end;

function TComplicatedGECollection.NoProperties: boolean;
begin
   if FProperties = nil then
   begin
      FProperties := TDynamicProperties(FindElementClass(TDynamicProperties));
   end;
   result := FProperties = nil;
end;

procedure TComplicatedGECollection.SetColor(const Value: integer);
begin
  FColor := Value;
end;

procedure TComplicatedGECollection.SetEnabled(const Value: boolean);
begin
  FEnabled := Value;
end;

procedure TComplicatedGECollection.SetLayer(aName: string;
  const Value: TComplicatedGECollection);
begin

end;

procedure TComplicatedGECollection.SetObjectType(const Value: string);
begin
  FObjectType := Value;
end;

procedure TComplicatedGECollection.SetProperties(const Value: TElement);
begin
   NoProperties;
   if FProperties = Value then exit;
   ExchangeFieldElement(FProperties,Value);
end;

procedure TComplicatedGECollection.SetSymbol(const Value: integer);
begin
  FSymbol := Value;
end;

procedure TComplicatedGECollection.SetText(const Value: string);
begin
  FText := Value;
end;

procedure TComplicatedGECollection.SetUseColor(const Value: boolean);
begin
  FUseColor := Value;
end;

procedure TComplicatedGECollection.SetUseSymbol(const Value: integer);
begin
  FUseSymbol := Value;
end;

procedure TComplicatedGECollection.SetVisible(const Value: boolean);
begin
  FVisible := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TGeometryElement);
   RegisterOrbisClass(TPropertyGeometryElement);
   RegisterOrbisClass(TGeometryElementCollection);
   RegisterOrbisClass(TPointCollection);
   RegisterOrbisClass(TPolyLineCollection);
   RegisterOrbisClass(TAreaCollection);
   RegisterOrbisClass(TTextCollection);
   RegisterOrbisClass(TComplicatedGECollection);
END.
