unit t_QTreeTools;

interface
uses
   e_Qtree,
   g_Elems,
   Tools;

type

TFindRectEntityInQTreeByXY = class(TTool)
private
  FX, FY: double;
    FTolerance: double;
  function GetQTreeForXY(aQT: TRectEntityQTree): TRectEntityQTree;
  function GetRectEntityForXY(aQT: TRectEntityQTree): TRectEntity;
    procedure SetTolerance(const Value: double);
public
  procedure DoIt; override;
published
  property X: double read FX write FX;
  property Y: double read FY write FY;
  property Tolerance: double read FTolerance write SetTolerance;
end;



TAddEntityToQTree = class(TTool)
public
   Procedure DoIt; override;
end;

implementation
uses
   Forms,
   SysUtils,
   Elements,
   e_Class;

{ TFindRectEntityInQTreeByXY }

function TFindRectEntityInQTreeByXY.GetQTreeForXY(aQT: TRectEntityQTree): TRectEntityQTree;
var
  aaQT: TRectEntityQTree;
  I: longInt;

begin
  result:= nil;

  if aQT.IsLeaf then
  begin
    result:= aQT;
    exit;
  end;

  for I:= 0 to aQT.Count-1 do
  begin
    aaQT:= TRectEntityQTree(aQT.Elements[I]);
    if not (aaQT is TRectEntityQTree) then continue;

    if aaQT.XYIsIn(X, Y) then
    begin
      result:= GetQTreeForXY(aaQT);
      break
    end;
  end;
end;




function TFindRectEntityInQTreeByXY.GetRectEntityForXY(aQT: TRectEntityQTree): TRectEntity;
var
  I: longInt;
  //Found: boolean;
  SE: TRectEntity;
begin
  result:= nil;
  //I:= 0;
  for I:= 0 to aQT.Count-1 do
  begin
    SE:= TRectEntity(aQT.Elements[I]);
    if not (SE is TRectEntity) then continue;
    if SE.IsHitByXY(X,Y, Tolerance) then
    begin
      result:= SE;
      break
    end;
  end;
end;




procedure TFindRectEntityInQTreeByXY.DoIt;
var
  aQT: TRectEntityQTree;

begin
  if Busy then exit;
  Busy := True;
  if not (Input is TRectEntityQTree) then
  begin
     Busy := false;
     exit;
  end;
  Output:= nil;
  if not TRectEntityQTree(Input).XYIsIn(X, Y) then
  begin
     Busy := false;
     exit;
  end;
  aQT:= GetQTreeForXY(TRectEntityQTree(Input));
  if aQT = nil then
  begin
     Busy := false;
     exit;
  end;
  Output:= GetRectEntityForXY(aQT);
  Busy := false;
end;


procedure TFindRectEntityInQTreeByXY.SetTolerance(const Value: double);
begin
   FTolerance := Value;
end;

{ TAddEntityToQTree }

procedure TAddEntityToQTree.DoIt;
var
   E: TEntity;
   El: TElement;
   QT: TRectEntityQTree;
   i: integer;
   m: integer;
begin
   if not (Input is TEntity) then
      exit;
   E := TEntity(Input);
   if Output = nil then
   begin
      Output := TRectEntityQTree.Create;
      Output.Name := 'REQT';
   end;
   if not (Output is TRectEntityQTree) then
      exit;
   if E.NoElements then
      exit;
   QT := TRectEntityQTree(Output);
   StartProgress('Tvorba QTree',0,E.LastIndex);
   ProgressText := 'Bude se ukldat '+ E.GetPropertyStringByName('Count')+ ' element';
   m := E.LastIndex div 30;
   if m < 10 then m := 10;
   for i := 0 to E.LastIndex do
   begin
      El := E.Elements[i];
      if not (El is TRectEntity) or not TRectEntity(El).IsValid then   // to valid tam dost chybelo...
         continue;
      QT.NextRectEntity := TRectEntity(El);
      if i mod m = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
   end;
   ProgressText := 'OK.';
   Application.ProcessMessages;
end;

INITIALIZATION
   RegisterOrbisClass(TAddEntityToQTree);
   RegisterOrbisClass(TFindRectEntityInQTreeByXY);
end.
