unit T_ExArTL;
{ Soustava nstroj pro export ploch do topolsk struktury }

interface
uses
// KERNEL
  Elements,
  Tools,
// SPACE
  //E_Space,
  E_QTree,
  E_Props,
// TOPOL
  e_TopolElements,
  T_AreLin,
  T_DBFexp,
  T_TlDBF;

type


TPrepareExportAreasToTopol = class(TTool)
{ Pprava entity ploch pro export
  input - TSpaceEntity exportovanch ploch
  output = input
  DoIt:
    Exportovan linie oekv v entit Lines, jinak si je vyjme sama volnm
    TExtractLinesForAreas.
    nejdve pro vechny plochy nastav sla ploch, paternitu, sla lini.
}
private
  FLines: TEntity;
  FTopolNo: longInt;
  FDruh: integer;
  procedure SetLines(aLines: TEntity);
  function  GetLines: TEntity;
  procedure DoArea(anArea: TTopoLAreaElement);
{  procedure SetAreaNoToLines(aNo: longInt; anArea: TPolyLineAreaElement); }
  //procedure SetTopolProps(anArea: TAreaElement);
public
  procedure DoIt; override;
published
  property Lines: TEntity read GetLines write SetLines;
  property Druh: integer read FDruh write FDruh;
end;


TExportAreasToTopol = class(TExportTopolElements)
{ input - TSpaceEntity exportovanch ploch
  output = input
  DoIt
    provede vlastn export do soubor TN.ARE a TN.LIN tak jak jsou nastaveny.
}
private
  FLines: TEntity;
  FDruhLinii: smallInt;
  FDBFLinii: boolean;
  FDruhPloch: smallInt;
protected
  function  GetAreaNoForLine
            (aLine: TTopoLLineElement; aLeft: boolean): longInt;
  procedure ExportLines; virtual;
  procedure ExportAreas; virtual;
  procedure TransferFieldsFromDynamicProperties
            (anElement: TElement; aDynProps: TDynamicProperties);
public
  procedure DoIt; override;
published
  property DruhLinii: smallInt read FDruhLinii write FDruhLinii;
  property DBFLinii: boolean read FDBFLinii write FDBFLinii;
  property DruhPloch: smallInt read FDruhPloch write FDruhPloch;
end;

{
TExportAreasToTopol2 = class(TExportAreasToTopol)
protected
  procedure ExportLines; override;
  procedure ExportAreas; override;
end;
}

TExportAreasToTopolPropsFromAreaQTree = class(TExportAreasToTopol)
private
  FAreaQTree: TRectEntityQTree;
  FTransferScript: TMovedItemContainer;

  function  GetTransferScript(aDruh: integer): TMovedItemContainer;
  procedure AddDynamicProps
            (aDruh: integer; anArea: TTopoLAreaElement);
public
  procedure ExportObject(aObject: TTopoLElement); override;
published
  property AreaQTree: TRectEntityQTree read FAreaQTree write FAreaQTree;
end;


{
TExportDBFForTopolAreas = class(TExportDBF)
private
  FBlockName: string;
  procedure ExportAreas; override;
end;
}


implementation
uses
  SysUtils, Forms, Controls, Dialogs, Graphics, FileCtrl,
// KERNEL
  e_Class,
  E_Type,
  u_String,
// GLIB
  e_WCS,
// DIALOG
  //F_Gauge,
// SPACE
  e_Geometry,
  e_GeometryElement,
// TOPOL
  //E_TplPrp,
  T_TopExp,
  T_ExLnTl;

procedure TPrepareExportAreasToTopol.SetLines(aLines: TEntity);
begin
  FLines:= aLines
end;


function  TPrepareExportAreasToTopol.GetLines: TEntity;
var
  ELFA: TExtractLinesForAreas;
begin
  result:= FLines;
  if FLines <> nil then exit;
  ELFA:= TExtractLinesForAreas.Create;
  ELFA.Input:= Input;
  ELFA.DoIt;
  FLines:= TEntity(ELFA.Output);
  result:= FLines;
  ELFA.Free;
end;


(*
procedure TPrepareExportAreasToTopol.SetTopolProps(anArea: TAreaElement);
begin
  { otestovn a ppadn vytvoen topolskch propert }
  if not (anArea.Properties is TTopolAreaProps) then
  begin
    if anArea.Properties <> nil then
    begin
      anArea.Properties.Free;
      anArea.Properties:= nil;
    end;
  end;

  if anArea.Properties = nil then
  begin
    anArea.Properties:= TTopolAreaProps.Create;
    with TTopolAreaProps(anArea.Properties) do
    begin
      Barva:= 2;
      Znacka:= 1;
      PaterNo:= -1;
    end
  end;
end;

*)


procedure TPrepareExportAreasToTopol.DoArea(anArea: TTopoLAreaElement);
var
  //aProps: TTopolAreaProps;
  I: longInt;
  ESAFA: TExtractSubAreasForArea;
  Areas: TEntity;
  aSubArea: TTopoLAreaElement;
  Are: TAreaElement;
begin
  if anArea.ID > -1 then exit;
  if TAreaElement(anArea.Geometry).IsHole then exit;


  { nyn jsou nastaveny topolsk vlastnosti }
  //SetTopolProps(anArea);

  { vloen CISLO }
  //aProps:= TTopolAreaProps(anArea.Properties);
  //if aProps = nil then exit;
  inc(FTopolNo);
  anArea.Cislo:= FTopolNo;
  anArea.ID:= FTopolNo; { < -- toto je smerodatne }

  if Druh > 0 then
     anArea.Druh:= Druh;

{ vloen linim odkaz na tuto plochu
  if anArea is TPolyLineAreaElement then
     SetAreaNoToLines(FTopolNo, TPolyLineAreaElement(anArea));
}
  Are := TAreaElement(anArea.Geometry);
  { obsluha dr }
  if Are.Holes = nil then exit;
  if Are.Holes.Count < 1 then exit;

  { vloen vnjm linim dr odkazy na tuto plochu
  for I:= 0 to anArea.Holes.Count-1 do
  begin
    SetAreaNoToLines(FTopolNo, TPolyLineAreaElement(anArea.Holes.Elements[I]))
  end;
}

  { vloen sla otce podzenm plochm }
  ESAFA:= TExtractSubAreasForArea.Create;
  ESAFA.Input:= anArea;
  ESAFA.AreaCollection:= TAreaCollection(Input); { aby vyhledval pouze plochy z te entity }
  ESAFA.DoIt;
  Areas:= TEntity(ESAFA.Output);
  ESAFA.Free;

  if Areas = nil then exit;

  anArea.HasChildren:= true;

  for I:= 0 to Areas.Count-1 do
  begin
    aSubArea:= TTopoLAreaElement(Areas.Elements[I]);
    if not (TElement(aSubArea) is TTopoLAreaElement) then continue;

    //SetTopolProps(aSubArea);
    aSubArea.SetPropertyLongIntByName('PATERNO', FTopolNo)
  end;

  Areas.Free;
end;



procedure TPrepareExportAreasToTopol.DoIt;
var
  I: longInt;
  anAre: TTopoLAreaElement;
  //aProps: TTopolAreaProps;

begin
   if not (Input is TEntity) then exit;
   if not (Lines is TEntity) then exit;
   { resetovn vech ploch }
   for I:= 0 to TEntity(Input).Count-1 do
   begin
      anAre:= TTopoLAreaElement(TEntity(Input).Elements[I]);
      if not (TElement(anAre) is TTopoLAreaElement) then continue;
      anAre.Id:= -1;

      anAre.PaterNo:= -1;
      anAre.HasChildren:= false;
   end;

   { zpracovn vech ploch }
   FTopolNo:= 0;
   for I:= 0 to TEntity(Input).Count-1 do
   begin
      anAre:= TTopoLAreaElement(TEntity(Input).Elements[I]);
      if not (TElement(anAre) is TTopoLAreaElement) then continue;
      if TAreaElement(anAre.Geometry).IsHole then continue;
      DoArea(anAre);
   end;
   Output:= Input
end;




procedure TExportAreasToTopol.TransferFieldsFromDynamicProperties
           (anElement: TElement; aDynProps: TDynamicProperties);

var
  I: longInt;
  TypEl: TTypeElement;
  //SName, TName, SString: string;

begin
  { oekv v objektu aDynProps poloky o sprvnch nzvech pro tabulku DBF }
  if aDynProps = nil then exit;

  for I:= 0 to aDynProps.Count-1 do
  begin
    TypEl:= TTypeElement(aDynProps.Elements[I]);
    if (TypEl.UpperName <> '') then
    begin
      anElement.SetPropertyStringByName(TypEl.UpperName, TypEl.AsString)
    end { if smysluplny nazev polozky }
  end;
end;



function TExportAreasToTopol.GetAreaNoForLine
         (aLine: TTopoLLineElement; aLeft: boolean): longInt;
var
  I: longInt;
  E: TElement;
  Lin: TPolyLineElement;
  anArea: TTopoLAreaElement;
  Are: TPolyLineAreaElement;
begin
  result:= -1;
  if aLine = nil then exit;
  Lin := TPolyLineElement(aline.Geometry);
  if Lin = nil then exit;
  for I:= 0 to Lin.OwnersCount-1 do
  begin
    E:= Lin.Owners[I];
    if not (E is TPolyLineAreaElement) then continue;
    Are := TPolyLineAreaElement(E);
    { hledm pouze ty plochy drc tuto linii, kter exportuji }
    anArea:= TTopoLAreaElement(Are.FindOwnerClass(TTopoLAreaElement));
    if anArea = nil then continue;
    if not (AnArea.IsMyOwner(TEntity(Input))) then continue;

    if Are.HasLine(Lin, aLeft) then
    begin
      result:= anArea.ID;
      break
    end;
  end;
end;




procedure TExportAreasToTopol.ExportLines;
var
  aLine: TTopoLLineElement;
  I: longInt;
  TLW: TTopolLinWriter;
  //F_GaugeForm: TF_GaugeForm;
  aLeftArea, aRightArea: longInt;
  PomBool: boolean;
  PomDruh: integer;
begin
  TLW:= TTopolLinWriter.CreateAll(BlockName, TJTSK.Create, true);

  if FDruhLinii > 0 then
     TLW.Druh:= FDruhLinii;

  if TLW = nil then exit;
  {
  Application.CreateForm(TF_GaugeForm, F_GaugeForm);
  F_GaugeForm.MaxValue:= FLines.Count;
  F_GaugeForm.Caption:= 'Export lines';
  F_GaugeForm.Show;
  }
  StartProgress('ExportLines',0,FLines.Count);
  for I:= 0 to FLines.Count-1 do
  begin
    {
    if F_gaugeForm.DoCloseQuery and
       F_gaugeForm.RealyClose('Peruit export lini ?') then
       break;
    }
    Progress:= I;
    Application.ProcessMessages;
    aLine:= TTopoLLineElement(FLines.Elements[I]);
    if not (TElement(aLine) is TTopoLLineElement) then continue;

    aLeftArea:= GetAreaNoForLine(aLine, true);
    aRightArea:= GetAreaNoForLine(aLine, false);

    DoBeforeExport(aLine);

    TLW.WriteLinCisla(aLine, aLeftArea, aRightArea);

    { zpis do databze }
    PomBool:= ExportDBF;
    PomDruh:= JedenDruh;

    JedenDruh:= DruhLinii;
    ExportDBF:= DBFLinii;
    if DBFLinii then
       ExportObject(aLine);

    ExportDBF:= PomBool;
    JedenDruh:= PomDruh;
  end;

  //F_GaugeForm.Free;
  ProgressText := 'OK.';

  TLW.Free;
end;



procedure TExportAreasToTopol.ExportAreas;
var
  I: longInt;
  TAW: TTopolAreaWriter;
  anArea: TTopoLAreaElement;
  //F_GaugeForm: TF_GaugeForm;
begin
  TAW:= TTopolAreaWriter.CreateAll(BlockName, TJTSK.Create, true);
  if TAW = nil then exit;
  {
  Application.CreateForm(TF_GaugeForm, F_GaugeForm);
  F_GaugeForm.MaxValue:= TEntity(Input).Count;
  F_GaugeForm.Caption:= 'Export ploch do ' + BlockName;
  F_GaugeForm.Show;
  }
  StartProgress('Export ploch do ' + BlockName,0,TEntity(Input).Count);
  for I:= 0 to TEntity(Input).LastIndex do
  begin
    {
    if F_gaugeForm.DoCloseQuery and
       F_gaugeForm.RealyClose('Peruit export ploch ?') then
       break;
    }
    Progress:= I;
    Application.ProcessMessages;
    anArea:= TTopoLAreaElement(TEntity(Input).Elements[I]);
    if not (TElement(anArea) is TTopoLAreaElement) then continue;
    if TAreaElement(anArea.Geometry).IsHole then continue;

    DoBeforeExport(anArea);

    { zpis jen geometrie}
    TAW.WriteArea(anArea);

    { zpis do databze }
    ExportObject(anArea);
  end;

  //F_GaugeForm.Free;
  ProgressText := 'OK';

  TAW.Free;
end;



procedure TExportAreasToTopol.DoIt;
var
  PEAT: TPrepareExportAreasToTopol;
  PELT: TPrepareExportLinesToTopol;
  TheCursor: TCursor;

begin
  if input = nil then exit;
  if not (Input is TEntity) then exit;
  if BlockName = '' then exit;

  TheCursor:= Screen.Cursor;
  Screen.Cursor:= crHourGlass;
try
  { pprava zatm pouze vyjme linie z ploch }
  PEAT:= TPrepareExportAreasToTopol.Create;
  PEAT.Input:= Input;
  PEAT.Druh:= DruhPloch;
  PEAT.DoIt;
  FLines:= PEAT.Lines;
  PEAT.Free;

  if FLines <> nil then
  begin
  { pprava lini - oslovn a zauzlovn }

    PELT:= TPrepareExportLinesToTopol.Create;
    PELT.Input:= FLines;
    PELT.DoIt;
    PELT.Free;

    ExportLines;

    ExportAreas;

    FLines.Free
  end;
finally
  Screen.Cursor:= TheCursor;
end
end;



(*
                   {  TExportAreasToTopol2 }

procedure TExportAreasToTopol2.ExportLines;
{ bez gauge a bez monosti mnit druh lini }
var
  aLine: TPolyLineElement;
  I: longInt;
  TLW: TTopolLinWriter;
  aCislo: longInt;
begin
  TLW:= TTopolLinWriter.Create(BlockName, TJTSK.Create, true);

  if TLW = nil then exit;

  for I:= 0 to FLines.Count-1 do
  begin
    aLine:= TPolyLineElement(FLines.Elements[I]);
    if not (aLine is TPolyLineElement) then continue;
    aCislo:= aLine.SpaceId;
    TLW.WriteLin2(aLine);
  end;

  TLW.Free;
end;



procedure TExportAreasToTopol2.ExportAreas;
{ bez gauge }
var
  I: longInt;
  TAW: TTopolAreaWriter;
  anArea: TAreaElement;
begin
  TAW:= TTopolAreaWriter.Create(BlockName, TJTSK.Create, true);
  if TAW = nil then exit;

  for I:= 0 to TEntity(Input).Count-1 do
  begin
    anArea:= TAreaElement(TEntity(Input).Elements[I]);
    if not (anArea is TAreaElement) then continue;
    if anArea.IsHole then continue;
    TAW.WriteArea(TPolyLineAreaElement(anArea));
    ExportObject(anArea);
  end;

  TAW.Free;
end;

*)

                { TExportAreasToTopolPropsFromAreaQTree }
function TExportAreasToTopolPropsFromAreaQTree.
         GetTransferScript(aDruh: integer): TMovedItemContainer;
var
  FName: string;
begin
  result:= FTransferScript;

  FName:= upperCase(ControlDir+ '\P'+ IntToStrZ(aDruh, 4, true) +'.TDS');
  if not FileExists(FName) then exit;

  if (result <> nil) and (result.FileName = FName) then exit;
  { Transfer Script pro tento druh jiz existuje }

  if result = nil then
  begin
    result:= TMovedItemContainer.Create;
    FTransferScript:= result;
    AddElement(FTransferScript)
  end;

  result.FileName:= FName; { nate dky }
end;



procedure TExportAreasToTopolPropsFromAreaQTree.AddDynamicProps
          (aDruh: integer; anArea: TTopoLAreaElement);
var
  TS: TMovedItemContainer;
  Zdroj, Cil: string;
  anItem: TStringElement;
  I: longInt;
begin

  if DynamicProps = nil then exit;

  TS:= GetTransferScript(aDruh);
  if TS = nil then exit;

  for I:= 0 to TS.Count-1 do
  begin
    Zdroj:= TS.GetSourceItem(I);
    if Zdroj[1] = '*' then continue;
    Cil:= TS.GetTargetItem(I);

    if anArea.HasPropertyByName(Zdroj) then
    begin
      anItem:= TStringElement.CreateAll(Cil, anArea.GetPropertyStringByName(Zdroj));
      DynamicProps.AddElement(anItem)
    end
  end;
end;




procedure TExportAreasToTopolPropsFromAreaQTree.ExportObject(aObject: TTopoLElement);
var
  QTW: TQTreeWalker;
  anArea: TTopoLAreaElement;
  aDruh: integer;
  //aCislo: longInt;
begin
  if AreaQTree = nil then exit;
  if not (TElement(aObject) is TTopoLElement) then exit;
  //if aObject.Properties = nil then exit;
  //if not (aObject.Properties is TTopolProps) then exit;
  aDruh:= aObject.Druh;
  //aCislo:= aObject.SpaceId;

  if (DynamicProps <> nil) and (DynamicProps.Count > 0) then
     DynamicProps.FreeAll;

{ Vyhledn zdrojovch blok a podle penosovho souboru naplnn DynamicProps}

  QTW:= TQTreeWalker.Create;
  anArea:= TTopoLAreaElement(QTW.GetFirstElementForRectangle
                        (AreaQTree, aObject.Geometry.X, aObject.Geometry.Y, aObject.Geometry.X, aObject.Geometry.Y, 0));
  while TElement(anArea) is TTopoLAreaElement do
  begin
    if TAreaElement(anArea.Geometry).XYIsInside(aObject.Geometry.X, aObject.Geometry.Y) then
       AddDynamicProps(aDruh, anArea);
    anArea:= TTopoLAreaElement(QTW.GetNextElement)
  end;

  if ExportDBF then
  begin
    UseExportDBFClass(aObject);
{
    if FExpDBFTool = nil then
       FExpDBFTool:= TExportDBFObject.Create;

    if FExpDBFTool = nil then exit;

    FExpDBFTool.JedenDruh:= JedenDruh;
    FExpDBFTool.ControlDir:= ControlDir;
    FExpDBFTool.ExpBlockName:= BlockName;
    FExpDBFTool.Input:= aObject;
    FExpDBFTool.DynamicProps:= FDynProps;
    FExpDBFTool.CreateFromSource:= CreateFromSource;
    FExpDBFTool.DoIt;
}
    if RegisterObject then
       Register.RegisterObject(aObject);

  end
  else
  begin
    TransferFieldsFromDynamicProperties
           (aObject, DynamicProps)
  end;

  QTW.Free
end;





INITIALIZATION
  RegisterOrbisClass(TPrepareExportAreasToTopol);
  RegisterOrbisClass(TExportAreasToTopol);
  RegisterOrbisClass(TExportAreasToTopolPropsFromAreaQTree);
END.
