unit WLink;
{$C FIXED PRELOAD PERMANENT}
interface
uses WinDos,DPMI,WinProcs;
const
  _Buffer_Max    = 1024;
  _Delay:longint = 36;


var
  _OriginalVector : pointer;
  _Buffer       : Array[0.._Buffer_Max] of byte;

  _Open_Flag    : Boolean;   {true kdyz je port otevren}
  _Port         : Integer;   {otevreny port (1 nebo 2) }
  _Base         : Integer;   {bazova adresa aktualniho portu}
  _Irq          : Integer;   {irq aktualniho otevreneho portu}

  _Buffer_Overflow : Boolean;  {true pri preteceni bufferu}
  _Buffer_Used     : Integer;
  _MaxBufferUsed   : Integer;

    {_Buffer je prazdny kdyz Head = Tail }
  _Buffer_Head  : Integer;   {Lokace v _Buffer pro vlozeni dalsiho znaku}
  _Buffer_Tail  : Integer;   {Lokace v _Buffer pro cteni dalsiho znaku}
  _Buffer_NewTail : Integer;



Procedure _Init;
{inicializace promennych}

Procedure _Close;
{reset pouziteho hradwaroveho preruseni}

Function _Open(ComPort       : Integer;
                    BaudRate      : Integer;
                    Parity        : Char;
                    WordSize      : Integer;
                    StopBits      : Integer) : Boolean;

Function _Buffer_Check(var C : Byte) : Boolean;
{znak z bufferu, jestlize _Buffer_Check je true}

Function _Send(C : Byte):boolean;
{vyslani byte, vraci True v pripade, ze se vysilani povedlo}

implementation

const
  UART_THR = $00;    {offset UART registru v IBM PC }
  UART_RBR = $00;
  UART_IER = $01;
  UART_IIR = $02;
  UART_LCR = $03;
  UART_MCR = $04;
  UART_LSR = $05;
  UART_MSR = $06;

  I8088_IMR = $21;   {adresa portu Interrupt Mask Register }



{  _BIOS_Port_Table : Array[1..2] of Integer absolute $0040:$0000;}
               {Tabulka registru RS232 v pameti BIOSu}
{  _AbsTime:longint absolute $0000:$046C;}

const
  _Num_Bauds = 8;
  _Baud_Table : array [1.._Num_Bauds] of record
                                                     Baud, Bits : integer
                                                   end
                   = ((Baud:110;  Bits:$00),
                      (Baud:150;  Bits:$20),
                      (Baud:300;  Bits:$40),
                      (Baud:600;  Bits:$60),
                      (Baud:1200; Bits:$80),
                      (Baud:2400; Bits:$A0),
                      (Baud:4800; Bits:$C0),
                      (Baud:9600; Bits:$E0));


Procedure DisableInterrupts; inline($FA {cli} );
Procedure EnableInterrupts;  inline($FB {sti} );


Procedure BIOS_RS232_Init(ComPort, ComParm:Integer);
{volani sluzby BIOSu, ktera inicializuje UART}
var
  Regs : Tregisters;
begin
  with Regs do
    begin
      ax := ComParm and $00FF;  { AH=0; AL=ComParm }
      dx := ComPort;
      DPMI_Intr($14, Regs)
    end
end;

Procedure _Isr;  Interrupt;
{interrupt - vola ji UART pri prijmu jednoho znaku}
begin
Inline(
    $FB/                           {STI}

      {_Buffer[_Buffer_Head] := Chr(Port[UART_RBR + _Base]);}
    $8B/$16/_Base/                 {MOV DX,_Base }
    $EC/                           {IN AL,DX }
    $8B/$1E/_Buffer_Head/          {MOV BX,_Buffer_Head }
    $88/$87/_Buffer/               {MOV _Buffer[BX],AL }

      {_Buffer_NewHead := _Buffer_Head + 1;}
    $43/                           {INC BX}

      {if _Buffer_NewHead>_Buffer_Max then _Buffer_NewHead:=0;}
    $81/$FB/_Buffer_Max/           {CMP BX,_Buffer_Max}
    $7E/$02/                       {JLE @001}
    $33/$DB/                       {XOR BX,BX}

      {if _Buffer_NewHead = _Buffer_Tail then _Buffer_Overflow:=TRUE else}
{@001:}
    $3B/$1E/_Buffer_Tail/          {CMP BX,_Buffer_Tail }
    $75/$08/                       {JNE @002 }
    $C6/$06/_Buffer_Overflow/$01/  {MOV _Buffer_Overflow,1}
    $90/                           {NOP}
    $EB/$16/                       {JMP SHORT L003}

      {_Buffer_Head := _Buffer_NewHead;
       _Buffer_Used := _Buffer_Used + 1;
       if _Buffer_Used > _MaxBufferUsed then _MaxBufferUsed := _Buffer_Used}
{@002:}
    $89/$1E/_Buffer_Head/         {MOV _Buffer_Head,BX }
    $FF/$06/_Buffer_Used/         {INC _Buffer_Used }
    $8B/$1E/_Buffer_Used/         {MOV BX,_Buffer_Used }
    $3B/$1E/_MaxBufferUsed/       {CMP BX,_MaxBufferUsed }
    $7E/$04/                      {JLE @003 }
    $89/$1E/_MaxBufferUsed/       {MOV _MaxBufferUsed,BX }

{@003:}

    $FA/                          {CLI}

      {Port[$20] := $20;} {ukoceni preruseni pro UART}
    $B0/$20/                      {MOV AL,20h}
    $E6/$20                       {OUT 20h,AL}
       )
end;

Procedure _Init;
{inicializace promennych}
begin
  _Open_Flag := FALSE;
  _Buffer_Overflow := FALSE;
  _Buffer_Used := 0;
  _MaxBufferUsed := 0;
end;


Procedure _Close;
{reset prerusovaciho sytemu UART}
var
  i, m : Integer;
begin
  if _Open_Flag then
    begin

      { disable the IRQ on the 8259 }
      DisableInterrupts;
      i := Port[I8088_IMR];        {vzit masku preruseni}
      m := 1 shl _Irq;             {nastavit puvodni masku}
      Port[I8088_IMR] := i or m;

      {zakaz 8250 preruseni pri cteni dat}
      Port[UART_IER + _Base] := 0;

      {nulovat OUT2 na 8250}
      Port[UART_MCR + _Base] := 0;
      EnableInterrupts;

      SetIntVec(_Irq+8,_OriginalVector);

      {reinicializace dat}
      _Open_Flag := FALSE
    end
end;

Function _Open(ComPort       : Integer;
                    BaudRate      : Integer;
                    Parity        : Char;
                    WordSize      : Integer;
                    StopBits      : Integer) : Boolean;
{pokus o otevreni komunikacniho portu}

var
  ComParm : Integer;
  i, m : Integer;
  s:string;
begin
  if _Open_Flag then _Close;
  if ComPort=2  then
    _Port:=2 else _Port:=1;  {standardne COM1}

  str(_port,s); s:='COM'+s+#0;
  i:=OpenComm(@s[1],1000,1000);
  If i<=0 then
  begin
       _Open:=False;
       Exit;
  end;
  CloseComm(i);
  If _Port=2 then _Base:=$2F8 else _Base:=$3F8;
  _Irq := Hi(_Base) + 1;

  if (Port[UART_IIR+_Base] and $00F8)<>0 then _Open := FALSE
  else
  begin
      _Buffer_Head := 0;
      _Buffer_Tail := 0;
      _Buffer_Overflow := FALSE;

      ComParm := $0000;

      {nastaveni prenosove rychlosti}

      i:=0;
      repeat
            Inc(i);
      until (_Baud_Table[i].Baud=BaudRate) or (i=_Num_Bauds);

      ComParm:=ComParm or _Baud_Table[i].Bits;

      if Parity in ['E','e'] then ComParm:=ComParm or $0018
      else if Parity in ['O','o'] then ComParm:=ComParm or $0008
      else ComParm:=ComParm or $0000; {standardne bez parity}

      if WordSize=7 then ComParm:=ComParm or $0002
      else ComParm:=ComParm or $0003; {standardne 8 bitu}

      if StopBits = 2 then ComParm:=ComParm or $0004
      else ComParm:=ComParm or $0000; {standardne 1 stop bit}

      {funkce BIOSu pro nastaveni COMu}
      BIOS_RS232_Init(_Port-1,ComParm);

      {uschovani puvodniho vektoru a nastaveni vlastniho}
      GetIntVec(_Irq+8, _OriginalVector);
      SetIntVec(_Irq+8, @_Isr);


      DisableInterrupts;

      {Nulovat DLAB}
      Port[UART_LCR+_Base]:=Port[UART_LCR+_Base] and $7F;


      {vyprazdneni vstupniho bufferu}
      i:=Port[UART_LSR+_Base];
      i:=Port[UART_RBR+_Base];


      i:=Port[I8088_IMR];
      m:=(1 shl _Irq) xor $00FF;
      Port[I8088_IMR] := i and m;


      {povoleni preruseni od prijateho znaku}
      Port[UART_IER+_Base]:=$01;


      {aktivace OUT2}
      i:=Port[UART_MCR+_Base];
      Port[UART_MCR+_Base]:= i or $08;

      EnableInterrupts;
      _Open_Flag:=TRUE;
      _Open:=TRUE;

      {aktivace RTS,OUT2 a DTR}
      Port[UART_MCR+_Base]:=$0B;
    end
end;


Function _Buffer_Check(var C:Byte):Boolean;
{je-li neco v bufferu vraci True a prislusny byte}
begin
  if _Buffer_Head = _Buffer_Tail then
    _Buffer_Check := FALSE
  else
    begin
      C := Ord(_Buffer[_Buffer_Tail]);
      _Buffer_Tail := _Buffer_Tail + 1;
      if _Buffer_Tail > _Buffer_Max then
        _Buffer_Tail := 0;
      _Buffer_Used := _Buffer_Used - 1;
      _Buffer_Check := TRUE
    end
end;

Function _Send(C : Byte):boolean;
{vyslani jednoho byte}
var
  i, m: Integer;
  counter:longint;
begin
  Port[UART_MCR+_Base]:=$0B; {nastavit OUT2,DTR a RTS}

  {cekani na nastaveni CTS}
  counter:=GetTickCount+_Delay*55; {*55  kvuli shodnosti s DOSem}
  While (counter >=GetTickCount) and ((Port[UART_MSR + _Base] and $10) = 0) do;
  If counter <GetTickCount then begin _Send:=False; Exit; end;
  {CTS nebylo nastaveno v pozadovanem case}

  {cekani na vyprazdneni vysilaciho bufferu}

  counter:=GetTickCount+_Delay*55; {*55  kvuli shodnosti s DOSem}
  While (Counter>=GetTickCount) and ((Port[UART_LSR + _Base] and $20) = 0) do;


  If counter >=GetTickCount then
  begin
      {cekani dopadlo dobre, je vysilani}
      DisableInterrupts;
      Port[UART_THR +_Base]:=C;
      EnableInterrupts;
      _Send:=True;
  end else _Send:=False;
end;

end.