unit e_XML;
(*
  -------------------------------------------------------------------------
  Zkladn elementy pro vytvoen modelu DTD pro XML
  pozor ! konstrukce je zatm vytvoena tak, e neslou pro uloen do ORB
  (bhv jak by to dopadlo...)
  PPK 5.5.2003
  ppk 11.2011 rozsireno pro XSD
{ TODO -oPPK -cDodlat : naten ENTIT !!! }
  -------------------------------------------------------------------------
*)
interface
uses
   Classes,
   Elements,
   e_Sort,
   e_Type;

const
   dtde_FileNotExists = 1;
type

TDTDElements = class(TOwnerEntity);
Txs_SimpleTypes = class(TOwnerEntity);

Txs_Restriction = class(TOwnerEntity)
  private
    FminLength: integer;
    FmaxLength: integer;
    FPattern: string;
    FminInclusive: string;
    Fbase: string;
    FmaxInclusive: string;
    FEnumeration: TSortedEntity;
    procedure Setbase(const Value: string);
    procedure SetEnumeration(const Value: TSortedEntity);
    procedure SetmaxInclusive(const Value: string);
    procedure SetmaxLength(const Value: integer);
    procedure SetminInclusive(const Value: string);
    procedure SetminLength(const Value: integer);
    procedure SetPattern(const Value: string);
public
    function AddEnumeration(aValue: string): TStringElement;
published
   property base: string read Fbase write Setbase;
   property Pattern: string read FPattern write SetPattern;
   property minInclusive: string read FminInclusive write SetminInclusive;
   property maxInclusive: string read FmaxInclusive write SetmaxInclusive;
   property minLength: integer read FminLength write SetminLength;
   property maxLength: integer read FmaxLength write SetmaxLength;
   property Enumeration: TSortedEntity read FEnumeration write SetEnumeration;
end;

Txs_AttributeType = class;

Txs_Attribute = class(TOwnerEntity)
  private
    FValue: string;
    FRestriction: Txs_Restriction;
    Fxs_AType: Txs_AttributeType;
    FComment: string;
    procedure SetValue(const Value: string);
    procedure SetRestriction(const Value: Txs_Restriction);
    procedure Setxs_AType(const Value: Txs_AttributeType);
    procedure SetComment(const Value: string);
    function GetRestriction: Txs_Restriction;
published
   property Value: string read FValue write SetValue;
   property Restriction: Txs_Restriction read GetRestriction write SetRestriction;
   property xs_AType: Txs_AttributeType read Fxs_AType write Setxs_AType;
   property Comment: string read FComment write SetComment;
end;

Txs_AttributeType = class(Txs_Attribute);

Txs_Attributes = class(TOwnerEntity)
  private
    function GetAttribute(aindex: integer): Txs_Attribute;
public
    function AddAttribute(aName, aValue: string): Txs_Attribute;
    function  GetAttributeByName(aValue: string): Txs_Attribute;
    procedure SetAttributeByName(aName, aValue: string);
    property Attribute[aindex: integer]: Txs_Attribute read GetAttribute;
end;

Txs_Type = class;

TDTDModelElement = class(TOwnerEntity)
  private
    Fxs_Type: Txs_Type;
    function GetNoAttributes: boolean;
    procedure Setxs_Type(const Value: Txs_Type);
  private
    FEmpty: boolean;
    FDTDElements: TDTDElements;
    FAttributes: Txs_Attributes;
    procedure SetAttributes(const Value: Txs_Attributes);
    procedure SetDTDElements(const Value: TDTDElements);
    procedure SetEmpty(const Value: boolean);
    function GetAttributes: Txs_Attributes;
    function GetDTDElements: TDTDElements;
public
   procedure ClearAttributes;
   function  DTDElementExists(const aName: string): boolean;
   function  FindDTDElement(const aName: string): TDTDModelElement;
published
   property Empty: boolean read FEmpty write SetEmpty;
   property NoAttributes: boolean read GetNoAttributes;
   property DTDElements: TDTDElements read GetDTDElements write SetDTDElements;
   property Attributes: Txs_Attributes read GetAttributes write SetAttributes;
   property xs_Type: Txs_Type read Fxs_Type write Setxs_Type;
end;

Txs_Type = class(TDTDModelElement);


TDTDModel = class(TOwnerEntity)
private
   FTemplateDTDFileName: string;
   FDTDFileName: string;
    FDTD: TStringList;
    FError: integer;
    FDTDElements: TDTDElements;
    FRootElement: TDTDModelElement;
    FActualElement: TDTDModelElement;
    Fxs_SimpleTypes: Txs_SimpleTypes;
   procedure SetDTDFileName(const Value: string);
   procedure SetTemplateDTDFileName(const Value: string);
    procedure SetDTD(const Value: TStringList);
    procedure SetError(const Value: integer);
    procedure SetDTDElements(const Value: TDTDElements);
    procedure SetRootElement(const Value: TDTDModelElement);
    function GetDTDElements: TDTDElements;
    procedure SetActualElement(const Value: TDTDModelElement);
    procedure Setxs_SimpleTypes(const Value: Txs_SimpleTypes);
    function Getxs_SimpleTypes: Txs_SimpleTypes;
public
   Constructor Create; override;
   destructor  Destroy; override;
   procedure   LoadFromFile(const aFileName: string);
   procedure   SaveToFile(const aFileName: string);
   procedure   ProcessDTD;
   function    GetDTDElement(const aName: string): TDTDModelElement;
   function    FindDTDElement(const aName: string): TDTDModelElement;
   function    AddSimleType(aName: string): Txs_AttributeType;
   property    DTD: TStringList read FDTD write SetDTD;
   property    ActualElement: TDTDModelElement read FActualElement write SetActualElement;
published
   property RootElement: TDTDModelElement read FRootElement write SetRootElement;
   property DTDElements: TDTDElements read GetDTDElements write SetDTDElements;
   property TemplateDTDFileName: string read FTemplateDTDFileName write SetTemplateDTDFileName;
   property DTDFileName: string read FDTDFileName write SetDTDFileName;
   property Error: integer read FError write SetError;
   property xs_SimpleTypes: Txs_SimpleTypes read Getxs_SimpleTypes write Setxs_SimpleTypes;
end;

TXML_NodeElement = class(TOwnerEntity)
  private
    FAttributes: TStringListElement;
    FText: string;
    procedure SetAttributes(const Value: TStringListElement);
    function GetAttributes: TStringListElement;
    function GetNoAttributes: boolean;
    procedure SetNoAttributes(const Value: boolean);
    procedure SetText(const Value: string);
    procedure SetNextAttribute(const Value: string);
public
   destructor Destroy; override;
published
   property Attributes: TStringListElement read GetAttributes write SetAttributes;
   property NoAttributes: boolean read GetNoAttributes write SetNoAttributes stored false;
   property NextAttribute: string write SetNextAttribute stored false;
   property Text: string read FText write SetText;
end;


TXML_FileElement = class(TOwnerEntity)
  private
    FHeader: string;
    procedure SetHeader(const Value: string);
published
   property Header: string read FHeader write SetHeader;
end;

implementation
uses
   SysUtils,
   e_Class,
   u_String;

{ TDTDModel }

function TDTDModel.AddSimleType(aName: string): Txs_AttributeType;
begin
   result := Txs_AttributeType(xs_SimpleTypes.FindOrCreateElementName(Txs_AttributeType,aName));
end;

constructor TDTDModel.Create;
begin
   inherited;
   DTD := TStringList.Create;
end;

destructor TDTDModel.Destroy;
begin
   if DTD <> nil then
   begin
      DTD.Free;
      DTD := nil;
   end;
   inherited;
end;

function TDTDModel.FindDTDElement(const aName: string): TDTDModelElement;
begin
   result := nil;
   if (ActualElement <> nil) and (ActualElement.Name = aName) then
   begin
      result := ActualElement;
      exit;
   end;
   result := TDTDModelElement(DTDElements.FindElementName(aName));
   ActualElement := result;
end;

function TDTDModel.GetDTDElement(const aName: string): TDTDModelElement;
begin
   result := nil;
   if (ActualElement <> nil) and (ActualElement.Name = aName) then
   begin
      result := ActualElement;
      exit;
   end;
   result := TDTDModelElement(DTDElements.FindOrCreateElementName(TDTDModelElement,aName));
   ActualElement := result;
   if RootElement = nil then
      RootElement := result;
end;

function TDTDModel.GetDTDElements: TDTDElements;
begin
   if FDTDElements = nil then
   begin
      FDTDElements := TDTDElements.Create;
      AddElement(FDTDElements);
   end;
   result := FDTDElements;
end;

function TDTDModel.Getxs_SimpleTypes: Txs_SimpleTypes;
begin
   if Fxs_SimpleTypes = nil then
   begin
      Fxs_SimpleTypes := Txs_SimpleTypes.Create;
      Fxs_SimpleTypes.Name := 'SimpleTypes';
      AddElement(Fxs_SimpleTypes);
   end;
   result := Fxs_SimpleTypes;
end;

procedure TDTDModel.LoadFromFile(const aFileName: string);
begin
   TemplateDTDFileName := aFileName;
end;

procedure TDTDModel.ProcessDTD;
var
   i,ii: integer;
   S,SS,EName,Key: string;
   CH: char;
   Position: integer;
   aElement: TDTDModelElement;
begin
   RootElement := nil;
   DTDElements := nil;
   for i := 0 to DTD.Count - 1 do
   begin
      S := DTD.Strings[i];
      if S = '' then continue;
      // preskoceni mezer a hledani Key, tedy <!-- nebo <!ELEMENT nebo <!ATTLIST
      SS := ReadToDelimiterWithoutWS(S,['!'],1,CH,Position);
      if not ((SS = '<') and (Ch = '!')) then
      begin
         // Error
         continue;
      end;
      SS := ReadToDelimiter(S,[' '],Position,CH,Position);
      if SS = '--' then
         continue
      else
         Key := SS;
      // preskoceni mezer
      while (Position < Length(S)) and (S[Position] = ' ') do Inc(Position);
      if Position = Length(S) then
      begin
         //Error;
         Continue;
      end;
      // jmeno elementu
      EName := ReadToDelimiter(S,[' '],Position,CH,Position);
      aElement := GetDTDElement(EName);
      // preskoceni mezer
      while (Position < Length(S)) and (S[Position] = ' ') do Inc(Position);
      if Position = Length(S) then
      begin
         //Error;
         Continue;
      end;
      if Key = 'ELEMENT' then
      begin
         if S[Position] = '(' then
         begin
            Inc(Position);
            repeat
               SS := ReadToDelimiterWithoutWS(S,[',','+','*','?',')'],Position,Ch,Position);
               aElement.DTDElements.AddElement(GetDTDElement(SS));
               if not (Ch in [',',')']) then
                  ReadToDelimiterWithoutWS(S,[',',')'],Position,Ch,Position);
            until Ch = ')';
         end
         else
         begin
            aElement.Empty := true;
         end;
      end
      else
      if Key = 'ATTLIST' then
      begin
         SS := ReadToDelimiter(S,[' '],Position,Ch,Position);
         ActualElement.Attributes.AddAttribute(SS,'');
      end;
   end;
end;

procedure TDTDModel.SaveToFile(const aFileName: string);
begin
   DTDFileName := aFileName;
end;

procedure TDTDModel.SetActualElement(const Value: TDTDModelElement);
begin
  FActualElement := Value;
end;

procedure TDTDModel.SetDTD(const Value: TStringList);
begin
  FDTD := Value;
end;

procedure TDTDModel.SetDTDElements(const Value: TDTDElements);
begin
  FDTDElements := Value;
end;

procedure TDTDModel.SetDTDFileName(const Value: string);
begin
   FDTDFileName := Value;
   DTD.SaveToFile(Value);
end;

procedure TDTDModel.SetError(const Value: integer);
begin
  FError := Value;
end;

procedure TDTDModel.SetRootElement(const Value: TDTDModelElement);
begin
  FRootElement := Value;
end;

procedure TDTDModel.SetTemplateDTDFileName(const Value: string);
var
   S: string;
begin
  S := GetAbsolutePath(Value);
  FTemplateDTDFileName := S;
  DTD.Clear;
  if not FileExists(S) then
  begin
     Error := dtde_FileNotExists;
     exit;
  end;
  DTD.LoadFromFile(S);
  ProcessDTD;
end;

procedure TDTDModel.Setxs_SimpleTypes(const Value: Txs_SimpleTypes);
begin
   ExchangeFieldElement(TElement(Fxs_SimpleTypes), Value);
end;

{ TDTDModelElement }

procedure TDTDModelElement.ClearAttributes;
var
   i: integer;
   AT: Txs_Attributes;
begin
   if NoAttributes then exit;
   AT := Attributes;
   for i := 0 to AT.LastIndex do
   begin
      AT.Attribute[i].Value := '';
   end;
end;

function TDTDModelElement.DTDElementExists(const aName: string): boolean;
begin
   result := false;
   result := FindDTDElement(aName) <> nil;
end;

function TDTDModelElement.FindDTDElement(const aName: string): TDTDModelElement;
begin
   result := nil;
   if xs_Type <> nil then
   begin
      result := xs_Type.FindDTDElement(aName);
      exit;
   end;
   if Empty then exit;
   result := TDTDModelElement(DTDElements.FindElementName(aName));
end;

function TDTDModelElement.GetAttributes: Txs_Attributes;
begin
   if xs_Type <> nil then
   begin
      result := xs_Type.Attributes;
      exit;
   end;
   if FAttributes = nil then
   begin
      FAttributes := Txs_Attributes.Create;
      AddElement(FAttributes);
      FAttributes.Name := 'ATTLIST';
   end;
   result := FAttributes;
end;

function TDTDModelElement.GetDTDElements: TDTDElements;
begin
   if FDTDElements = nil then
   begin
      FDTDElements := TDTDElements.Create;
      if Empty then
         FDTDElements.Name := 'EMPTY'
      else
         FDTDElements.Name := 'ELEMENTS';
      AddElement(FDTDElements);
   end;
   result := FDTDElements;
end;

function TDTDModelElement.GetNoAttributes: boolean;
begin
   if xs_Type <> nil then
   begin
      result := xs_Type.NoAttributes;
      exit;
   end;
   result := (FAttributes = nil) or (FAttributes.Count = 0);
end;

procedure TDTDModelElement.SetAttributes(const Value: Txs_Attributes);
begin
   ExchangeFieldElement(TElement(FAttributes), Value);
end;

procedure TDTDModelElement.SetDTDElements(const Value: TDTDElements);
begin
   ExchangeFieldElement(TElement(FDTDElements), Value);
end;

procedure TDTDModelElement.SetEmpty(const Value: boolean);
begin
  FEmpty := Value;
end;

procedure TDTDModelElement.Setxs_Type(const Value: Txs_Type);
begin
  Fxs_Type := Value;
  AddElement(Value);
end;

{ TXML_NodeElement }

destructor TXML_NodeElement.Destroy;
begin
   if FAttributes <> nil then
   begin
      FAttributes.Free;
      FAttributes := nil;
   end;
   inherited;
end;

function TXML_NodeElement.GetAttributes: TStringListElement;
begin
   if FAttributes = nil then
      FAttributes := TStringListElement.Create;
   result := FAttributes;
end;

function TXML_NodeElement.GetNoAttributes: boolean;
begin
   result := (FAttributes = nil) or (FAttributes.NoElements);
end;

procedure TXML_NodeElement.SetAttributes(const Value: TStringListElement);
begin
   if FAttributes <> nil then
   begin
      FAttributes.Free;
      FAttributes := nil;
   end;
   FAttributes := Value;
end;

procedure TXML_NodeElement.SetNextAttribute(const Value: string);
var
   S1,S2: string;
   Ch: char;
begin
   if StringAsParameter(Value,S1,S2) then
   begin
      ch := S2[1];
      if ch in ['''','"'] then
         S2 := AnsiDequotedStr(S2,Ch);
      Attributes.AddNameString(S1,S2);
   end
   else
      Attributes.AddNameString(S1,'');
end;

procedure TXML_NodeElement.SetNoAttributes(const Value: boolean);
begin
   if Value then
      Attributes := nil;
end;

procedure TXML_NodeElement.SetText(const Value: string);
begin
  FText := Value;
end;

{ TXML_FileElement }

procedure TXML_FileElement.SetHeader(const Value: string);
begin
  FHeader := Value;
end;

{ Txs_Attribute }

function Txs_Attribute.GetRestriction: Txs_Restriction;
begin
   if FRestriction = nil then
   begin
      FRestriction := Txs_Restriction.Create;
      AddElement(FRestriction);
   end;
   Result := FRestriction;
end;

procedure Txs_Attribute.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure Txs_Attribute.SetRestriction(const Value: Txs_Restriction);
begin
   ExchangeFieldElement(TElement(FRestriction), Value);
end;

procedure Txs_Attribute.SetValue(const Value: string);
begin
   FValue := Value;
end;

{ Txs_AttributeTypes }

function Txs_Attributes.AddAttribute(aName, aValue: string): Txs_Attribute;
begin
   Result := Txs_Attribute.Create;
   AddElement(Result);
   Result.Name := aName;
   Result.Value := aValue;
end;

function Txs_Attributes.GetAttribute(aindex: integer): Txs_Attribute;
var
   e: TElement;
begin
   result := nil;
   e := Elements[aindex];
   if not (e is Txs_Attribute) then exit;
   result := Txs_Attribute(e);
end;

function Txs_Attributes.GetAttributeByName(aValue: string): Txs_Attribute;
begin
   result := Txs_Attribute(FindElementName(aValue));
end;

procedure Txs_Attributes.SetAttributeByName(aName, aValue: string);
var
   e: TElement;
   At: Txs_Attribute;
begin
   e := FindElementName(aName);
   if e = nil then exit;
   if not (e is Txs_Attribute) then exit;
   At := Txs_Attribute(e);
   At.Value := aValue;
end;

procedure Txs_Attribute.Setxs_AType(const Value: Txs_AttributeType);
begin
   ExchangeFieldElement(TElement(Fxs_AType), Value);
end;

{ Txs_Restriction }

function Txs_Restriction.AddEnumeration(aValue: string): TStringElement;
begin
   if Enumeration = nil then
   begin
      Enumeration := TSortedEntity.Create;
   end;
   result := TStringElement.CreateAll(aValue,'');
   Enumeration.AddElement(result);
end;

procedure Txs_Restriction.Setbase(const Value: string);
begin
  Fbase := Value;
end;

procedure Txs_Restriction.SetEnumeration(const Value: TSortedEntity);
begin
   ExchangeFieldElement(TElement(FEnumeration), Value);
end;

procedure Txs_Restriction.SetmaxInclusive(const Value: string);
begin
  FmaxInclusive := Value;
end;

procedure Txs_Restriction.SetmaxLength(const Value: integer);
begin
  FmaxLength := Value;
end;

procedure Txs_Restriction.SetminInclusive(const Value: string);
begin
  FminInclusive := Value;
end;

procedure Txs_Restriction.SetminLength(const Value: integer);
begin
  FminLength := Value;
end;

procedure Txs_Restriction.SetPattern(const Value: string);
begin
  FPattern := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TXML_NodeElement);
   RegisterOrbisClass(TXML_FileElement);
END.

