unit t_XML;
(*
   -----------------------------------------------------------
   Zkladn nstroje pro ukldn a natn model do a z XML
   TXMLLoader
   TXMLSaver
   PPK  28.4.2003
   -----------------------------------------------------------
*)

interface
uses
   classes,
   Elements,
   e_XML,
   Tools,
   u_xeString;
const
// Errors

erxml_NoDTD        = 1;
erxml_NoDTDElement = 2;

type
TCharSet = set of _xchar;


TXMLTool = class(TTool)
private
   FFileName: string;
    FComment: string;
   procedure SetFileName(const Value: string);
    procedure SetComment(const Value: string);
public
   function NumberToPropertyName(aNumber: longInt): string; override;
   function StrToXML(const Value: string): string; virtual; // prevod nepovolenych znaku do XML
   function XMLToStr(const Value: string): string; virtual; // a zpet
   procedure Start; virtual;
   procedure Finish; virtual;
published
   property FileName: string read FFileName write SetFileName; //cesta k souboru XML
   property Comment: string read FComment write SetComment;
end;

TXMLLoader = class(TXMLTool)
private
   FReader: TReader;
   FStream: TStream;
   FXMLSize: integer;
   FFWChar: _xchar;
   FLastPosition: integer;        // dopedn naten znak
   FLineNo: integer;
   function EndOfFile: Boolean;
   function IsSpace(aChar: _xchar): Boolean;
   function  GetSingleChar: _xchar;
   function  GetChar: _xchar;
    procedure SetLastPosition(const Value: integer);
   function  GetXMLName(aCharSet: TCharSet): _xstring; // naten etzec po space
   procedure LoadElemAttribute(aSlovo: string; aElement: TElement);
   procedure LoadElement(aName: string; aElement: TElement; aProp: string);{ <Name      /> }
   function  LoadComment(aName: string; aElement: TElement; aProp: string): _xstring;  { <!--           --> }
   procedure LoadProcessingInstruction(aName: string; aElement: TElement; aProp: string);{ <?Name      /> }
   procedure LoadDeclaration(aName: string; aElement: TElement); { <!Name      /> }
   function  LoadTag(aElement: TElement): boolean;   { < > }
protected
   property  Reader: TReader read FReader write FReader;
   property  Stream: TStream read FStream write FStream;
   property  LastPosition: integer read FLastPosition write SetLastPosition;
public
   procedure   DoIt; override;
   procedure   ProcessElement(aElement: TElement); virtual;  // pokud neni vytvaren subelement
   function    CreateSubElement(aName: string; aElement: TElement): TElement; virtual; // tvorba SubELementu
   procedure   SetElementText(aElement: TElement; aText: string); virtual; // Nastaveni testu SubElementu
   procedure   SetXMLHeader(aHeader: string); virtual;
   procedure   AddParam(aElement: TElement; aName, aParam: string); virtual;
   procedure   CreateOutput; virtual;
published
   property XMLSize: integer read FXMLSize;
end;

TXMLLoad = class(TXMLLoader)
public
   procedure   DoIt; override;
   function    CreateSubElement(aName: string; aElement: TElement): TElement; override;
   procedure   SetElementText(aElement: TElement; aText: string); override;
   procedure   SetXMLHeader(aHeader: string); override;
   procedure   AddParam(aElement: TElement; aName, aParam: string); override;
   procedure   CreateOutput; override;
end;

// XSDLoad .... Nacte xsd, vyledkem je DTDModel ....

TXSDLoad = class(TXMLTool)
public
   function    LoadXML: TXML_FileElement;
   procedure   ProcessXML(aXL: TXML_FileElement);
   procedure   DoIt; override;
end;

TXMLSaver = class(TXMLTool)
private
   FLevel: integer;
    FRootElement: string;
    FDTD: TDTDModel;
    FLevelSpaceString: string;
    FXSD: boolean;
    procedure SetLevel(const Value: integer);
    procedure SetRootElement(const Value: string);
    procedure SetDTD(const Value: TDTDModel);
    function GetDTD: TDTDModel;
    procedure SetLevelSpaceString(const Value: string);
    procedure SetXSD(const Value: boolean);
protected
   FText: System.Text;
public
   procedure   DoIt; override;
   procedure   SaveHeader; virtual;
   procedure   Start; override;
   procedure   SaveRootElement; virtual;
   procedure   SaveDTDElementSTART(const Value: TDTDModelElement); virtual;
   procedure   SaveDTDElementEND(const Value: TDTDModelElement); virtual;
   procedure   SaveActualDTDElementSTART; virtual;
   procedure   SaveActualDTDElementEND; virtual;
   procedure   Finish; override;
   procedure   WriteToXML(const aText: string);
   procedure   WriteLnToXML(const aText: string);
   function    GetLevelSpace: string;
   procedure   IncLevel;
   procedure   DecLevel;
   property    Level: integer read FLevel write SetLevel; // uroven vnoreni elementu
published
   property    RootElement: string read FRootElement write SetRootElement;
   property    DTD: TDTDModel read GetDTD write SetDTD;
   property    LevelSpaceString: string read FLevelSpaceString write SetLevelSpaceString;
   property    XSD: boolean read FXSD write SetXSD;
end;

TXMLSave = class(TXMLSaver)
public
   procedure   DoIt; override;
   procedure   SaveHeader; override;
   procedure   SaveRootElement; override;
   procedure   SaveXMLELement(aELement: TXML_NodeElement); virtual;
end;


implementation
uses
   Dialogs,
   Forms,
   StrUtils,
   SysUtils,
   e_Class,
   e_Type,
   m_Orbis,
   u_String;

{ TXMLTool }

procedure TXMLTool.Finish;
begin

end;

function TXMLTool.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'FileName';
   else
      result := 'Comment';
   end;   
end;

procedure TXMLTool.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TXMLTool.SetFileName(const Value: string);
begin
  FFileName := Value;
end;

{ TODO 1 -oPPK -cDodlat : pevod znak na entity v souladu s pouitou znakovou sadou. }
procedure TXMLTool.Start;
begin

end;

function TXMLTool.StrToXML(const Value: string): string;
const
   InValidCharTable = (['<','>','&','''','"']);
var
  c : char;
  S: string;
  i : integer;
begin
   result := Value;
   for i := Length(result) downto 1 do
   begin
      c := result[i];
      if (c in InValidCharTable)  then
      begin
         result[i] := '&';
         case c of
            '<': S := 'lt;';
            '>': S := 'gt;';
            '&': S := 'amp;';
            '''':S := 'apos;';
            '"': S := 'quot;';
         end;
        //Insert('#x' + IntToHex(Ord(c),2) + ';', result,i+1);
        Insert(S, result,i+1);
      end;
   end;
end;

function TXMLTool.XMLToStr(const Value: string): string;
var
   i: integer;
   c : char;
   LastEnd: integer;
   S: string;
begin
   result := Value;
   LastEnd := -1;
   for i := Length(result) downto 1 do
   begin
      c := result[i];
      if c = ';' then
         LastEnd := i;
      if (c = '&') and (LastEnd > -1) then
      begin
         S := LowerCase(Copy(result,i,LastEnd - i));
         if S = '&lt' then
            c := '<'
         else if S = '&gt' then
            c := '>'
         else if S = '&amp' then
            c := '&'
         else if S = '&apos' then
            c := ''''
         else if S = '&quot' then
            c := '"';
         Delete(result,i+1,LastEnd - i);
         result[i] := c;
         LastEnd := -1;
      end;
   end;
end;

{ TXMLSaver }


procedure TXMLSaver.DecLevel;
begin
   Dec(FLevel);
end;

procedure TXMLSaver.DoIt;
begin
   Start;
   SaveHeader;
   SaveRootElement;
   Finish;
end;

procedure TXMLSaver.Finish;
begin
   system.close(FText);
end;

function TXMLSaver.GetDTD: TDTDModel;
begin
   if FDTD = nil then
   begin
      FDTD := TDTDModel.Create;
      AddElement(FDTD);
   end;
   result := FDTD;
end;

function TXMLSaver.GetLevelSpace: string;
begin
   if Level <= 0 then
      result := ''
   else
      result := DupeString(LevelSpaceString,Level);
end;

procedure TXMLSaver.IncLevel;
begin
   Inc(FLevel);
end;

procedure TXMLSaver.SaveActualDTDElementEND;
begin
   if DTD = nil then exit;
   if DTD.ActualElement = nil then exit;
   SaveDTDElementEND(DTD.ActualElement);
end;

procedure TXMLSaver.SaveActualDTDElementStart;
begin
   if DTD = nil then // chybi DTD soubor (template DTD)
   begin
      Error := erxml_NoDTD;
      exit;
   end;
   if DTD.ActualElement = nil then // neni nastaven ActualDTDElement
   begin
      Error := erxml_NoDTDElement;
      exit;
   end;
   SaveDTDElementStart(DTD.ActualElement);
end;

procedure TXMLSaver.SaveDTDElementEND(const Value: TDTDModelElement);
begin
   if Value = nil then exit;
   if Value.Empty then exit;
   DecLevel;
   WriteLnToXML(GetLevelSpace + '</'+Value.Name + '>');
end;

procedure TXMLSaver.SaveDTDElementSTART(const Value: TDTDModelElement);
var
   S: string;
   At: Txs_Attribute;
   i: integer;
begin
   if Value = nil then exit;
   S := GetLevelSpace + '<'+ Value.Name;
   if not Value.NoAttributes then
   for i := 0 to Value.Attributes.LastIndex do
   begin
      At := Value.Attributes.Attribute[i];
      S := S + ' ' + At.Name+ '="' + At.Value + '"';
   end;
   if Value.Empty then
      S := S + '/'
   else
      IncLevel;
   S := S + '>';
   WriteLnToXML(S);
end;

procedure TXMLSaver.SaveHeader;
begin
   writeLnToXML('<?xml version="1.0" encoding="Windows-1250" ?>');
end;

procedure TXMLSaver.SaveRootElement;
begin
   WriteLnToXML('<'+RootElement+'/>');
end;

procedure TXMLSaver.SetDTD(const Value: TDTDModel);
begin
   ExchangeFieldElement(TElement(FDTD), Value);
end;

procedure TXMLSaver.SetLevel(const Value: integer);
begin
  FLevel := Value;
end;

procedure TXMLSaver.SetLevelSpaceString(const Value: string);
var
   S,S1,S2,Sch: string;
   B: boolean;
   i: integer;
   Bch: byte;
begin
   FLevelSpaceString := '';
   if Value = '' then exit;
   S := Value;
   repeat
      B := StringAsTwoParams(S,',',S1,S2);
      S := S2;
      if S1 <> '' then
      begin
         if S1[1] = '#' then
         begin
            Sch := '';
            for i := 2 to Length(S1) do
            begin
               Sch := Sch + S1[i];
            end;
            Bch := StrToIntDef(Sch,20);
            FLevelSpaceString := FLevelSpaceString + CHar(Bch);
         end
         else
         begin
            for i := 1 to Length(S1) do
            begin
               S1[i] := ' ';
            end;
            FLevelSpaceString := FLevelSpaceString + S1;
         end;
      end;
   until not B;
end;

procedure TXMLSaver.SetRootElement(const Value: string);
begin
  FRootElement := Value;
end;

procedure TXMLSaver.SetXSD(const Value: boolean);
begin
  FXSD := Value;
end;

procedure TXMLSaver.Start;
begin
   system.assign(FText, FileName);
   system.rewrite(FText);
   Level := 0;
end;

procedure TXMLSaver.WriteLnToXML(const aText: string);
begin
   system.writeln(FText, aText);
end;

procedure TXMLSaver.WriteToXML(const aText: string);
begin
   system.write(FText, aText);
end;

{ TXMLLoader }

procedure TXMLLoader.AddParam(aElement: TElement; aName, aParam: string);
begin
  if aElement is TStringListElement then
     TStringListElement(aElement).AddNameString(aName, aParam)
end;

procedure TXMLLoader.CreateOutput;
begin
   Output := TOwnerEntity.Create;
   Output.Name := 'XML';
end;

function TXMLLoader.CreateSubElement(aName: string; aElement: TElement): TElement;
begin
   result := nil;
end;

procedure TXMLLoader.DoIt;
begin
   Stream := TMemoryStream.Create;
   try
      TMemoryStream(Stream).LoadFromFile(FileName);
      FXMLSize:= Stream.Size;
   except
      Stream.Free;
      Stream := TFileStream.Create(FileName, fmOpenRead{Write !!!!!!!} );
   end;
   Stream := TFileStream.Create(FileName, fmOpenRead{Write !!!!!!!} );
   Reader := TReader.Create(Stream, 4096{32768});
   FXMLSize := Stream.Size;
   if XMLSize > 256 then
   begin
      StartProgress('xml',0,XMLSize);
      ProgressText := FileName;
      Application.ProcessMessages;
   end;
   FLineNo:= 1;
   CreateOutput;
   FFWChar:= GetSingleChar;
   while not EndOfFile do
   begin
     LoadTag(Output);
   end;
   //Output := Container.elements[0];
   Reader.Free;
   Stream.Free;
end;

function TXMLLoader.EndOfFile: Boolean;
begin
  Result:= Reader.Position >= XMLSize
end;

function TXMLLoader.GetChar: _xchar;
var
  aChar: _xchar;
begin
{ dopedn ten znaku, aby se vynechvaly space znaky }
  Result:= FFWChar;
  aChar:= FFWChar;

  if IsSpace(FFWChar) then
  begin
    repeat
      aChar:= GetSingleChar
    until not IsSpace(aChar) or EndOfFile;
  end
  else
    aChar:= GetSingleChar;

  FFWChar:= aChar;
end;

function TXMLLoader.GetSingleChar: _xchar;
var
   aChar: _xchar;
begin
   Reader.read(aChar, 1);
   if aChar = #13 then
      Inc(FLineNo);
   result := aChar;
   if (Stream.Position > LastPosition) and (Stream.Position mod 256{4096} = 0) then
   begin
      LastPosition := Stream.Position;
      Progress := LastPosition;
      Application.ProcessMessages
   end;
end;

function TXMLLoader.GetXMLName(aCharSet: TCharSet): _xstring;
var
  aChar: _xchar;
begin
  Result:= '';
  if EndOfFile then
     Exit;
  aChar:= GetChar;
  while not isSpace(aChar) and not EndOfFile do
  begin
     Result:= Result + aChar;
     if aChar in aCharSet then Exit;
     aChar:= GetChar;
  end;
end;

function TXMLLoader.IsSpace(aChar: _xchar): Boolean;
begin
  Result:= aChar in [#9, #10, #13, #32]
end;

function TXMLLoader.LoadComment(aName: string; aElement: TElement; aProp: string): _xstring;
var
  aChar: _xchar;
  aStat: integer; {0: bezne, 1: 1.pom, 2: 2.pom za 1.}
  P1,P2: integer;
begin
{ proztmn natn, polyk mezery, atd. }
  aStat:= 0;
  P1 := Pos('!--', aName);
  P2 := Pos('-->', aName);
  if P2 > 0  then
  begin
     Result:= Copy(aName, P1 + 3, P2 - 4);
     exit;
  end;
  Result:= '' + FFWChar;
  aName:= Copy(aName, P1 + 3, Length(aName));
  //FFWChar:= GetSingleChar;
  while not EndOfFile do
  begin
    aChar:= FFWChar;
    FFWChar:= GetSingleChar;

    case aStat of
    0: if aChar = '-' then
          aStat:= 1
       else
          Result:= Result + aChar;
    1: if aChar = '-' then
          aStat:= 2
       else
       begin
          Result:= Result + '-' + aChar;
          aStat:= 0
       end;
    2: if aChar = '>' then
       begin
         Result:= aName + Result;
//         ShowMessage('<!-- ' + result + ' -->');
         Exit
       end
       else
       begin
          Result:= Result + '--' + aChar;
          aStat:= 0
       end;
    end;
  end;
end;

procedure TXMLLoader.LoadDeclaration(aName: string; aElement: TElement);
begin
  while (Pos('>', aName) <= 0) and not EndOfFile do
  begin
    aName:= GetXMLName(['>']);
  end;
end;

procedure TXMLLoader.LoadElemAttribute(aSlovo: string; aElement: TElement);
var
  aName, aParam: _xstring;
  PosEq: integer;
  aChar: _xchar;
  FirstDel: _xchar;
  aChyba: Boolean;
begin
  PosEq:= Pos('=', aSlovo);
  if PosEq <= 0 then
  begin
     aName:= aSlovo;
     aParam:= GetXMLName(['=']);
  end
  else
  begin
    aName:= Copy(aSlovo, 1, PosEq-1);
  end;

  repeat // Peskoen vodn vozovky nebo apostrofu nebo <
    FirstDel:= GetChar;
  until (FirstDel in ['''', '"', '<']) or EndOfFile;


  if not (FirstDel in ['''','"', '<']) then
  begin
    ShowMessage('CHYBA pi natn atributu "' + aName + '", nenalezen vodn apostrof nebo uvozovka'
                + #13 + #10 + 'dek .' + IntToStr(FLineNo));
    Exit;
  end;

  {
  if FirstDel = '<' then
  begin
    LoadElemElemProperty(aElement, aName);
    Exit;
  end;
  }
  aParam:= '';
  aChyba:= False;
  repeat
    aChar:= FFWChar;
    FFWChar:= GetSingleChar;

    if aChar=FirstDel then
    begin
      break;
    end;
    aParam:= aParam + aChar;
  until EndOfFile;

  if EndOfFile then
  begin
    ShowMessage('CHYBA pi natn atributu "' + aName + '"'
                + #13 + #10 + 'dek .' + IntToStr(FLineNo));
    Exit;
  end;

  AddParam(aElement, aName, aParam);
end;


procedure TXMLLoader.LoadElement(aName: string; aElement: TElement; aProp: string);
var
  aSlovo: string;
  EmptyElement: Boolean;
  ReadContent: Boolean;
  aSubElement: TElement;
  aSubDestroy: boolean;
begin
  { oven sprvnosti jmna elementu }

  { naten hlaviky, zatm nevm jestli je to przdn element }
  EmptyElement:= False;
  ReadContent:= False;

  if Pos('/>', aName) > 0 then
  begin
    EmptyElement:= True;
    aName:= Copy(aName, 1, Pos('/>', aName)-2);
  end
  else if Pos('>', aName) > 0 then
  begin
    ReadContent:= True;
    aName:= Copy(aName, 1, Pos('>', aName)-1);
  end;

  { Zaloen aSubElementu a pidn do aElementu }
  aSubDestroy := false;
  aSubElement := CreateSubElement(aName, aElement);
  if aSubElement <> nil then
  begin
     if (aElement <> nil) and (aElement is TEntity) then
        TEntity(aElement).AddElement(aSubElement);
  end
  else
  begin
     aSubElement:= TStringListElement.Create;
     aSubElement.Name:= aName;
     aSubDestroy := true;
  end;

  if not EmptyElement and not ReadContent then
  repeat
    aSlovo:= getXMLName(['>', '=']);
    if Pos('!--', aSlovo) > 0 then
    begin
      LoadComment(aSlovo, aSubElement, aProp);
      aSlovo:= '';
    end
    else
    if Pos('/>', aSlovo) > 0 then
    begin
      EmptyElement:= True;
      aSlovo:= copy(aSlovo, 1, Pos('/>', aSlovo)-1);
    end
    else if Pos('>', aSlovo) > 0 then
    begin
      ReadContent:= True;
      aSlovo:= copy(aSlovo, 1, Pos('>', aSlovo)-1);
    end
    else if Pos('=', aSlovo) > 0 then
    begin
       LoadElemAttribute(aSlovo, aSubElement);
       aSlovo:= ''
    end;
    if aSlovo <> '' then
       LoadElemAttribute(aSlovo, aSubElement);
  until EmptyElement or ReadContent;
  if aSubDestroy then
  begin
     ProcessElement(aSubElement);
     aSubElement.Free;
     aSubElement := nil;
  end;
  if ReadContent then
  begin
    repeat
    until not LoadTag(aSubElement);
  end;
end;

procedure TXMLLoader.LoadProcessingInstruction(aName: string; aElement: TElement; aProp: string);
var
  anID, aRes: integer;
  e: TElement;
  IsHeader: boolean;
  aHeader: string;
begin
  IsHeader := false;
  if copy(aName,2,3) = 'xml' then
  begin
     IsHeader := true;
     aHeader := '<?xml'
  end;
  while (Pos('?>', aName) <= 0) and not EndOfFile do
  begin
    aName:= GetXMLName(['>']);
    if IsHeader then
       aHeader := aHeader + ' ' +aName;
  end;
  if IsHeader then
     SetXMLHeader(aHeader);
end;

function TXMLLoader.LoadTag(aElement: TElement): boolean;
var
  Znak: _xchar;
  aStr: _xstring;
  aName: string;
  OnlySpace: Boolean;
begin
  Result:= True;

  aStr:= '';
  OnlySpace:= True;
  repeat
    Znak:= FFWChar;
    FFWChar:= GetSingleChar;

    if (Znak <> '<') then
    begin
      if not IsSpace(Znak) then
         OnlySpace:= False;
      aStr:= aStr + Znak;
    end
  until (Znak = '<') or EndOfFile;

  if EndOfFile then
  begin
    Result:= False;
    Exit;
  end;
  // !!!!!!!!! Na text neprdme....
  if (aElement <> nil) and not OnlySpace then
  begin
     SetElementText(aElement, aStr);
     //TEntity(aElement).AddElement(TStringElement.CreateAll('TEXT', aStr));
  end;

  aName:= GetXMLName(['>']);
  if Copy(aName, 1, 3) = '!--' then
     LoadComment(aName, aElement, '')
  else
  if Copy(aName, 1, 1) = '!' then
     LoadDeclaration(aName, aElement)
  else
  if Copy(aName, 1, 1) = '?' then
     LoadProcessingInstruction(aName, aElement, '')
  else
  if Copy(aName, 1, 1) = '/' then
     Result:= false
  else
     LoadElement(aName, aElement, '')
end;

procedure TXMLLoader.ProcessElement(aElement: TElement);
begin

end;

procedure TXMLLoader.SetElementText(aElement: TElement; aText: string);
begin

end;

procedure TXMLLoader.SetLastPosition(const Value: integer);
begin
  FLastPosition := Value;
end;

procedure TXMLLoader.SetXMLHeader(aHeader: string);
begin

end;

{ TXMLLoad }

procedure TXMLLoad.AddParam(aElement: TElement; aName, aParam: string);
var
   S: string;
begin
   S := UpperCase(aName);
   if (S = 'ID') or (S = 'VALUE') or (S = 'NAME') then
      aName := '@'+aName;
   if aElement is TXML_NodeElement then
      TXML_NodeElement(aElement).Attributes.SetPropertyStringByName(aName, aParam);
end;

procedure TXMLLoad.CreateOutput;
begin
   Output := TXML_FileElement.Create;
   Output.Name := FileName;
end;

function TXMLLoad.CreateSubElement(aName: string; aElement: TElement): TElement;
begin
   result := TXML_NodeElement.Create;
   result.Name := aName;
end;

procedure TXMLLoad.DoIt;
begin
{$IFDEF FILEEVENT}
    PostTextMsg(tm_FileBeforeLoad,FileName,nil);
    Application.ProcessMessages;
{$ENDIF}
  inherited;
end;

procedure TXMLLoad.SetElementText(aElement: TElement; aText: string);
begin
   if aELement is TXML_NodeElement then
      TXML_NodeElement(aElement).Text := aText;
end;

procedure TXMLLoad.SetXMLHeader(aHeader: string);
begin
   if Output is TXML_FileElement then
      TXML_FileElement(Output).Header := aHeader;
end;

{ TXMLSave }

procedure TXMLSave.DoIt;
begin
   if (Input = nil) or not (Input is TXML_FileElement) then
   begin
      ShowMessage('xml input is not valid XML element!');
      exit;
   end;
   if FileName = '' then
   begin
      ShowMessage('xml FileName err!: "'+FileName+'"');
      exit;
   end;
   inherited;
{$IFDEF FILEEVENT}
    PostTextMsg(tm_FileAfterSave,FileName,nil);
    Application.ProcessMessages;
{$ENDIF}
end;

procedure TXMLSave.SaveHeader;
begin
   writeLnToXML(TXML_FileElement(Input).Header);
end;

procedure TXMLSave.SaveRootElement;
begin
   Level := -1;
   LevelSpaceString := '**';
   SaveXMLElement(TXML_NodeElement(TEntity(Input).FindElementClass(TXML_NodeElement)));
end;

procedure TXMLSave.SaveXMLELement(aELement: TXML_NodeElement);
var
   S: string;
   SE: TStringElement;
   AttName: string;
   E: TElement;
   i: integer;
begin
   S := GetLevelSpace + '<'+aElement.Name;
   if not aELement.NoAttributes then
   for i := 0 to aElement.Attributes.LastIndex do
   begin
      SE := aElement.Attributes.StringElements[i];
      if SE.Name[1]='@' then
         AttName := copy(SE.Name,2,Length(SE.Name))
      else
         AttName := SE.Name;
      S := S + ' ' + AttName + '="' +SE.Value +'"';
   end;
   if aElement.NoElements then
   begin
      if aElement.Text <> '' then
         S := S + '>' + aElement.Text + '</' + aElement.Name + '>'
      else
         S := S + '/>';
      WriteLnToXML(S);
   end
   else
   begin
      S := S + '>';
      if aELement.Text <> '' then
         S := S + aElement.Text;
      WritelnToXML(S);
      S := '';
      IncLevel;
      for i := 0 to aElement.LastIndex do
      begin
         e := aElement.Elements[i];
         if e is TXML_NodeElement then
            SaveXMLElement(TXML_NodeElement(e));
      end;
      DecLevel;
      S := GetLevelSpace + '</' + aElement.Name + '>';
      WriteLnToXML(S);
   end;
end;

{ TXSDLoad }

procedure TXSDLoad.DoIt;
var
  XL: TXML_FileElement;
begin
   StartProgress(Comment,0,0);
   ProgressText := 'xsd loading...';
   Application.ProcessMessages;
   XL := LoadXML;
   AddElement(XL);
   ProgressText := 'xsd processing...';
   Application.ProcessMessages;
   ProcessXML(XL);
end;

function TXSDLoad.LoadXML: TXML_FileElement;
var
   XL: TXMLLoad;
begin
   result := nil;
   XL := TXMLLoad.Create;
   XL.FileName := FileName;
   XL.Comment := Comment;
   XL.DoIt;
   result := TXML_FileElement(XL.Output);
   XL.Free;
end;

procedure TXSDLoad.ProcessXML(aXL: TXML_FileElement);
var
   E: TElement;
   DTD: TDTDModel;
   ActualDTDElement: TDTDModelElement;

   procedure ProcessElement(aEl: TElement; ActualDTDE: TDTDModelElement; ActualType: Txs_Attribute; LastForComment: TElement);
   var
      XN: TXML_NodeElement;
      En: TEntity;
      S: string;
      SE: TStringElement;
      i: integer;
      DTE: TDTDModelElement;
      refonly: boolean;
      At: Txs_Attribute;
      aNS,aName: string;
   begin
      En := nil;
      if aEl is TEntity then En := TEntity(aEl);
      if aEl is TXML_NodeElement then
      begin
         XN := TXML_NodeElement(aEl);
         StringAsTwoParams(XN.Name,':',aNS,aName);
         if aName = 'element' then
         begin
            // name or reference
            refOnly := true;
            SE := XN.Attributes.GetStringElementByName('ref');
            if SE = nil then
            begin
               SE := XN.Attributes.GetStringElementByName('@name');
               refonly := false;
            end;
            if SE <> nil then
            begin
               DTE := TDTDModelElement(DTD.DTDElements.FindOrCreateElementName(TDTDModelElement,SE.Value));
               if ActualDTDE = nil then
               begin
                  ActualDTDE := DTE;
                  //DTD.RootElement := DTE
               end
               else
               begin
                  ActualDTDE.DTDElements.AddElement(DTE);
                  if not RefOnly then
                     ActualDTDE := DTE;
               end;
               SE := XN.Attributes.GetStringElementByName('type');
               // type
               if SE <> nil then
               begin
                  DTE.xs_Type := Txs_Type(DTD.DTDElements.FindOrCreateElementName(Txs_Type,SE.Value));
               end;
            end;
         end
         else
         if aName = 'complexType' then
         begin
            // name
            SE := XN.Attributes.GetStringElementByName('@name');
            if SE <> nil then
            begin
               ActualDTDE := Txs_Type(DTD.DTDElements.FindOrCreateElementName(Txs_Type,SE.Value));
            end;
         end
         else
         if aName = 'attribute' then
         begin
            // name
            SE := XN.Attributes.GetStringElementByName('@name');
            if SE <> nil then
            begin
               At := ActualDTDE.Attributes.AddAttribute(SE.Value,'');
               ActualType := At;
            end;
            SE := XN.Attributes.GetStringElementByName('type');
            if SE <> nil then
            begin
               At.xs_AType := DTD.AddSimleType(SE.Value);
            end;
         end
         else
         if aName = 'simpleType' then
         begin
            // name
            SE := XN.Attributes.GetStringElementByName('@name');
            if SE <> nil then
            begin
               ActualType := DTD.AddSimleType(SE.Value);
            end;
         end
         else
         if aName = 'restriction' then
         begin
            // name
            SE := XN.Attributes.GetStringElementByName('base');
            if SE <> nil then
            if ActualType <> nil then
            begin
               ActualType.Restriction.base := SE.Value;
            end;
         end
         else
         if aName = 'enumeration' then
         begin
            // name
            SE := XN.Attributes.GetStringElementByName('@value');
            if SE <> nil then
            if ActualType <> nil then
            begin
               ActualType.Restriction.AddEnumeration(SE.Value);
            end;
         end;
      end;
      if (En <> nil) and not En.NoElements then
      for i := 0 to En.LastIndex do
      begin
         ProcessElement(En.Elements[i],ActualDTDE,ActualType,LastForComment);
      end;
   end;
begin
   // Nejaka kontrola, ze je to fakt XML schema, by to chtelo :-)
   DTD := TDTDModel.Create;
   Output := DTD;
   ProcessElement(aXL,nil,nil,nil);
   DTD.RootElement := TDTDModelElement(DTD.DTDElements.FindElementClass(TDTDModelElement));
end;

INITIALIZATION
   RegisterOrbisClass(TXMLLoader);
   RegisterOrbisClass(TXMLLoad);
   RegisterOrbisClass(TXSDLoad);
   RegisterOrbisClass(TXMLSaver);
   RegisterOrbisClass(TXMLSave);
END.
