unit F_stdtxt;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls,
  TabNotBk, Menus, Spin, Gauges,
// DEVICE
  e_Device, ComCtrls, ToolWin, ImgList, ActnList;

type
  TStdtxtForm = class(TForm)
    SaveDialog: TSaveDialog;
    StatusBar: TStatusBar;
    CoolBar1: TCoolBar;
    RichEdit: TRichEdit;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolBar2: TToolBar;
    cb_WrapLines: TCheckBox;
    ToolButton3: TToolButton;
    ImageList1: TImageList;
    ToolButton4: TToolButton;
    FindDialog: TFindDialog;
    OpenDialog: TOpenDialog;
    ActionList1: TActionList;
    Open: TAction;
    Save: TAction;
    FindText: TAction;
    SetFont: TAction;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    Abort: TAction;
    CloseForm: TAction;
    ToolButton9: TToolButton;
    ToolButton5: TToolButton;
    FontDialog: TFontDialog;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure OpenExecute(Sender: TObject);
    procedure SaveExecute(Sender: TObject);
    procedure SetFontExecute(Sender: TObject);
    procedure AbortExecute(Sender: TObject);
    procedure CloseFormExecute(Sender: TObject);
    procedure cb_WrapLinesClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FindTextExecute(Sender: TObject);
    procedure FindDialogFind(Sender: TObject);
    procedure SaveDialogTypeChange(Sender: TObject);
    procedure RichEditChange(Sender: TObject);
    procedure RichEditSelectionChange(Sender: TObject);
  private
    FClosed: boolean;
    FModified: boolean;
    procedure SetClosed(const Value: boolean);
    procedure SetModified(const Value: boolean);
  public
     S: string;
     Busy: boolean;
     Device: TDevice;
     procedure WriteText(const aText: string);
     procedure WriteLnText(const aText: string);
     procedure StartWriting;
     procedure EndOfWriting;
     property  Closed: boolean read FClosed write SetClosed;
     property  Modified: boolean read FModified write SetModified;
  end;


implementation
uses
   {!PubUnits,}
// INTERPRET
   i_Uni,
// APP
   f_YesNo,
// KERNEL
   u_String;


{$R *.DFM}

procedure TStdtxtForm.WriteText(const aText: string);
begin
   S := S + aText;
   RichEdit.Lines.Strings[RichEdit.Lines.Count-1] := S;
   Modified := true;
end;

procedure TStdtxtForm.WriteLnText(const aText: string);
begin
   WriteText(aText);
   S := '';
   RichEdit.Lines.Add(' ');
   SendMessage(RichEdit.Handle, EM_SCROLLCARET, 0,0);
end;

procedure TStdtxtForm.StartWriting;
begin
   Busy := true;
   Application.ProcessMessages;
end;

procedure TStdtxtForm.EndOfWriting;
begin
   Busy := false;
end;



procedure TStdtxtForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
   CanClose := false;
   if not Busy then
   begin
      if Modified then
      begin
         if YesNoDialog('Obsah souboru byl zmnn ... uloit?') then
            SaveExecute(nil);
      end;
      Closed := true;
      CanClose := true;
      Application.ProcessMessages;
      if Device <> nil then
      try
         Device.Free;
      except
      end;
   end;
end;

procedure TStdtxtForm.OpenExecute(Sender: TObject);
begin
   if Modified then
   begin
      if YesNoDialog('Obsah souboru byl zmnn ... uloit?') then
         SaveExecute(nil);
   end;
   if OpenDialog.Execute then
   begin
      SaveDialog.InitialDir := ExtractFilePath(OpenDialog.FileName);
      OpenDialog.InitialDir := SaveDialog.InitialDir;
      RichEdit.Lines.LoadFromFile(OpenDialog.FileName);
      Modified := false;
   end;
end;

procedure TStdtxtForm.SaveExecute(Sender: TObject);
begin
   if SaveDialog.Execute then
   begin
      S := ExtractFileExt(SaveDialog.FileName);
      if UpperCase(S) = '.TXT' then
      begin
         RichEdit.PlainText := true;
      end;
      SaveDialog.InitialDir := ExtractFilePath(SaveDialog.FileName);
      OpenDialog.InitialDir := SaveDialog.InitialDir;
      RichEdit.Lines.SaveToFile(SaveDialog.FileName);
      RichEdit.PlainText := false;
      Modified := false;
   end;
end;

procedure TStdtxtForm.SetFontExecute(Sender: TObject);
begin
   FontDialog.Font := RichEdit.Font;
   if FontDialog.Execute then
   begin
      RichEdit.Font  := FontDialog.Font;
   end;
end;

procedure TStdtxtForm.AbortExecute(Sender: TObject);
var
   ii: TUniversalInterpret;
begin
   if Device <> nil then
   try
      ii := TUniversalInterpret(Device.FindOwnerClass(TUniversalInterpret));
      if ii <> nil then
      begin
         ii.BreakWork := true;
         RichEdit.Lines.Add('***** Perueno uivatelem !!!!!! *****');
         RichEdit.Lines.Add('  ');
      end;
   except
   end;
end;

procedure TStdtxtForm.CloseFormExecute(Sender: TObject);
begin
   Close;
end;

procedure TStdtxtForm.cb_WrapLinesClick(Sender: TObject);
begin
   RichEdit.WordWrap := cb_WrapLines.Checked;
end;

procedure TStdtxtForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   Action := caFree;
end;

procedure TStdtxtForm.FindTextExecute(Sender: TObject);
begin
   FindDialog.Execute;
end;

procedure TStdtxtForm.FindDialogFind(Sender: TObject);
var
   Pos: integer;
begin
   Pos := RichEdit.FindText(FindDialog.FindText,RichEdit.SelStart+1,Length(RichEdit.Text),[]);
   if Pos < 0 then ShowMessage('Text '+FindDialog.FindText+' Nenalezen !')
   else
   begin
      RichEdit.SelStart := Pos;
      RichEdit.SelLength := Length(FindDialog.FindText);
      SendMessage(RichEdit.Handle, EM_SCROLLCARET, 0,0);
      RichEdit.SetFocus;
   end;

end;

procedure TStdtxtForm.SaveDialogTypeChange(Sender: TObject);
begin
   if SaveDialog.FilterIndex = 1 then SaveDialog.DefaultExt := 'TXT'
                                 else SaveDialog.DefaultExt := 'RTF'
end;

procedure TStdtxtForm.SetClosed(const Value: boolean);
begin
  FClosed := Value;
end;

procedure TStdtxtForm.SetModified(const Value: boolean);
var
   S: string;
begin
  if FModified = Value then exit;
  //S := ' Poet dk: '+ IntToStr(RichEdit.Lines.Count);
  FModified := Value;
  {
  if FModified then StatusBar.SimpleText := 'Zmna !' + S
               else StatusBar.SimpleText := ' - '+ S;
   }            
end;

procedure TStdtxtForm.RichEditChange(Sender: TObject);
begin
   Modified := true;
end;

procedure TStdtxtForm.RichEditSelectionChange(Sender: TObject);
var
   ln,li: integer;
begin
   Ln :=   RichEdit.Perform(EM_LINEFROMCHAR, -1, 0);
   Li :=   RichEdit.Perform(EM_LINEINDEX, -1, 0);
   Li :=   RichEdit.SelStart - Li;
   StatusBar.SimpleText := IntToStr(Ln)+'/'+IntToStr(Li);
end;

end.
