unit t_xFileDialog;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.Layouts,
  FMX.ListBox, FMX.ListView.Types, //FMX.ListView.Appearances,
  FMX.Edit, FMX.ListView, FMX.StdCtrls,
  FMX.Controls.Presentation, FMX.Objects, FMX.ListView.Appearances,
  FMX.ListView.Adapters.Base;

type


  TxFileDialog = class(TForm)
    ListView: TListView;
    ed_Directory: TEdit;
    ImageControl1: TImageControl;
    ImageControl2: TImageControl;
    ImageControl3: TImageControl;
    b_OK: TButton;
    b_CLose: TButton;
    ed_FileName: TEdit;
    b_Up: TButton;
    ed_Caption: TEdit;
    ImageControl4: TImageControl;
    procedure ListViewMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure b_UpClick(Sender: TObject);
    procedure b_OKClick(Sender: TObject);
    procedure b_CLoseClick(Sender: TObject);
    procedure ListViewItemClickEx(const Sender: TObject; ItemIndex: Integer;
      const LocalClickPos: TPointF; const ItemObject: TListItemDrawable{TListItemObject});
  private
    FResult: boolean;
    FFilter: string;
    FOnSelect: TGetStrProc;
    FDirsOnly: boolean;
    LastDirItem: TListViewItem;
    function getDirectory: string;
    function GetFileName: string;
    procedure SetDirectory(const Value: string);
    procedure SetFileName(const Value: string);
    procedure SetResult(const Value: boolean);
    procedure SetFilter(const Value: string);
    procedure SetOnSelect(const Value: TGetStrProc);
    procedure SetDirsOnly(const Value: boolean);
    { Private declarations }
  public
    { Public declarations }
    Dirs: TStringDynArray;
    Files: TStringDynArray;
    property FileName: string read GetFileName write SetFileName;
    property Directory: string read getDirectory write SetDirectory;
    property Result: boolean read FResult write SetResult;
    property Filter: string read FFilter write SetFilter;
    property OnSelect: TGetStrProc read FOnSelect write SetOnSelect;
    property DirsOnly: boolean read FDirsOnly write SetDirsOnly;
  end;

var
  xFileDialog: TxFileDialog;

//function xSelectFileDlg(Caption, InitDir, Filter: string): string;

implementation
uses
   System.IOUtils;

{$R *.fmx}

//function xSelectFileDlg(Caption, InitDir, Filter: string): string;
//var
//   XD: TXFileDialog;
//   S: string;
//begin
//   result := '';
//   XD := TXFileDialog.Create(Application);
//   XD.Directory := InitDir;
//   XD.ed_Caption.Text := Caption;
//   XD.Filter := Filter;
//   XD.ShowModal(procedure(ModalResult: TModalResult)
//   begin
//     if ModalResult = mrOK then
//     // if OK was pressed and an item is selected, pick it
//        S := XD.FileName
//     else
//        S := '';
//     XD.DisposeOf;
//   end);
//   result := S;
//end;


{ TxFileDialog }

procedure TxFileDialog.b_CLoseClick(Sender: TObject);
begin
   Close;
end;

procedure TxFileDialog.b_OKClick(Sender: TObject);
begin
   Close;
   if Assigned(OnSelect) then
      OnSelect(FileName);
end;

procedure TxFileDialog.b_UpClick(Sender: TObject);
begin
   Directory := TPath.GetDirectoryName(Directory);
end;

function TxFileDialog.getDirectory: string;
begin
   result := ed_Directory.Text;
end;

function TxFileDialog.GetFileName: string;
begin
   result := ed_FileName.Text;
end;

procedure TxFileDialog.ListViewItemClickEx(const Sender: TObject;
  ItemIndex: Integer; const LocalClickPos: TPointF;
  const ItemObject: TListItemDrawable{TListItemObject});
var
   aItem: TListViewItem;
begin
//   if ListView.ItemIndex > 0 then
//      ed_Directory.Text := 'Click '+ ListView.Items[ListView.ItemIndex].Text
//   else
//      ed_Directory.Text := 'Click';
   ed_Caption.Text := 'Click';
   if ItemIndex > -1 then
      aItem := ListView.Items[ItemIndex]
   else
      exit;
   ed_Caption.Text := 'Click '+ aItem.Text;
   if DirsOnly then
   begin
      if aItem = LastDirItem then  // into the dir
         Directory := aItem.Detail
      else
      if aItem.Tag = 0 then
         Directory := TPath.GetDirectoryName(Directory)
      else
      begin
         if LastDirItem <> nil then
            LastDirItem.BitmapRef := ImageControl1.Bitmap;
         AItem.BitmapRef := ImageControl4.Bitmap;
         LastDirItem := AItem;
         ed_FileName.Text := aItem.Detail;
         b_OK.Enabled := true;
      end;
      exit;
   end;
   // Select Files
   case aItem.Tag of
      0: Directory := TPath.GetDirectoryName(Directory);
      1: Directory := aItem.Detail;
      2:
      begin
         fileName := aItem.Detail;
      end
   else
   end;
   if aItem.Tag = 2 then
   begin
      b_OK.Enabled := true;
   end
   else
   begin
      b_OK.Enabled := false;
      FileName := '';
   end;
end;

procedure TxFileDialog.ListViewMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
var
  Li: TListViewItem;
begin
   ed_Directory.Text := 'mouse';
end;

procedure TxFileDialog.SetDirectory(const Value: string);
var
   S: string;
   Lvi: TListViewItem;
   refext: string;
   IsFilter: boolean;
   ext: string;
   ok: boolean;
begin
   b_OK.Enabled := false;
   ListView.BeginUpdate;
   ListView.Items.Clear;
   LastDirItem := nil;
   ed_Directory.Text := Value;
   try
   Dirs := TDirectory.GetDirectories(value);
   if not DirsOnly then
      Files := TDirectory.GetFiles(Value);
   Lvi := ListView.Items.Add;
   Lvi.Text := '..';
   Lvi.Detail := 'UP';
   Lvi.Tag := 0;
   Lvi.BitmapRef := ImageControl3.Bitmap;
   for S in Dirs do
   begin
      Lvi := ListView.Items.Add;
      Lvi.Text := TPath.GetFileName(S);
      Lvi.Tag := 1;
      Lvi.Detail := S;
      Lvi.BitmapRef := ImageControl1.Bitmap;
   end;
   if not DirsOnly then
   for S in Files do
   begin
      IsFilter := Filter <> '';
      refext := UpperCase(Filter);
      if IsFilter then
      begin
         ext := UpperCase(TPath.GetExtension(S));
         OK := ext = refext;
      end
      else
        OK := true;
      if OK  then
      begin
         Lvi := ListView.Items.Add;
         Lvi.Text := TPath.GetFileName(S);
         Lvi.Tag := 2;
         Lvi.Detail := S;
         Lvi.BitmapRef := ImageControl2.Bitmap;
      end;
   end;
   ListView.ItemIndex := -1;
   ed_FileName.Text := '???';
   finally
   ListView.EndUpdate;
   end;
end;

procedure TxFileDialog.SetDirsOnly(const Value: boolean);
begin
  FDirsOnly := Value;
end;

procedure TxFileDialog.SetFileName(const Value: string);
begin
   ed_FileName.Text := Value;
end;

procedure TxFileDialog.SetFilter(const Value: string);
begin
  FFilter := Value;
end;

procedure TxFileDialog.SetOnSelect(const Value: TGetStrProc);
begin
  FOnSelect := Value;
end;

procedure TxFileDialog.SetResult(const Value: boolean);
begin
   FResult := Value;
end;

end.
